/***************************************************************************
 *     (c)2007-2011 Broadcom Corporation
 *
 *  This program is the proprietary software of Broadcom Corporation and/or its licensors,
 *  and may only be used, duplicated, modified or distributed pursuant to the terms and
 *  conditions of a separate, written license agreement executed between you and Broadcom
 *  (an "Authorized License").  Except as set forth in an Authorized License, Broadcom grants
 *  no license (express or implied), right to use, or waiver of any kind with respect to the
 *  Software, and Broadcom expressly reserves all rights in and to the Software and all
 *  intellectual property rights therein.  IF YOU HAVE NO AUTHORIZED LICENSE, THEN YOU
 *  HAVE NO RIGHT TO USE THIS SOFTWARE IN ANY WAY, AND SHOULD IMMEDIATELY
 *  NOTIFY BROADCOM AND DISCONTINUE ALL USE OF THE SOFTWARE.
 *
 *  Except as expressly set forth in the Authorized License,
 *
 *  1.     This program, including its structure, sequence and organization, constitutes the valuable trade
 *  secrets of Broadcom, and you shall use all reasonable efforts to protect the confidentiality thereof,
 *  and to use this information only in connection with your use of Broadcom integrated circuit products.
 *
 *  2.     TO THE MAXIMUM EXTENT PERMITTED BY LAW, THE SOFTWARE IS PROVIDED "AS IS"
 *  AND WITH ALL FAULTS AND BROADCOM MAKES NO PROMISES, REPRESENTATIONS OR
 *  WARRANTIES, EITHER EXPRESS, IMPLIED, STATUTORY, OR OTHERWISE, WITH RESPECT TO
 *  THE SOFTWARE.  BROADCOM SPECIFICALLY DISCLAIMS ANY AND ALL IMPLIED WARRANTIES
 *  OF TITLE, MERCHANTABILITY, NONINFRINGEMENT, FITNESS FOR A PARTICULAR PURPOSE,
 *  LACK OF VIRUSES, ACCURACY OR COMPLETENESS, QUIET ENJOYMENT, QUIET POSSESSION
 *  OR CORRESPONDENCE TO DESCRIPTION. YOU ASSUME THE ENTIRE RISK ARISING OUT OF
 *  USE OR PERFORMANCE OF THE SOFTWARE.
 *
 *  3.     TO THE MAXIMUM EXTENT PERMITTED BY LAW, IN NO EVENT SHALL BROADCOM OR ITS
 *  LICENSORS BE LIABLE FOR (i) CONSEQUENTIAL, INCIDENTAL, SPECIAL, INDIRECT, OR
 *  EXEMPLARY DAMAGES WHATSOEVER ARISING OUT OF OR IN ANY WAY RELATING TO YOUR
 *  USE OF OR INABILITY TO USE THE SOFTWARE EVEN IF BROADCOM HAS BEEN ADVISED OF
 *  THE POSSIBILITY OF SUCH DAMAGES; OR (ii) ANY AMOUNT IN EXCESS OF THE AMOUNT
 *  ACTUALLY PAID FOR THE SOFTWARE ITSELF OR U.S. $1, WHICHEVER IS GREATER. THESE
 *  LIMITATIONS SHALL APPLY NOTWITHSTANDING ANY FAILURE OF ESSENTIAL PURPOSE OF
 *  ANY LIMITED REMEDY.
 *
 * $brcm_Workfile: bcmnexus_gfx.h $
 * $brcm_Revision: DirectFB_1_4_Port/37 $
 *
 * $brcm_Date: 3/21/11 4:58p $
 *
 * Module Description:
 *
 * Revision History:
 *
 * $brcm_Log: /AppLibs/opensource/directfb/src/broadcom_files/private/DirectFB/1.4/gfxdrivers/bcmnexus/bcmnexus_gfx.h $
 * 
 * DirectFB_1_4_Port/37   3/21/11 4:58p robertwm
 * SW7422-393: DirectFB: Hang in graphics driver due to serialisation of
 *  graphics packet-buffer.
 * 
 * DirectFB_1_4_Port/36   3/17/11 12:43p robertwm
 * SW7422-384: DirectFB: Multi-application packet-buffer graphics fails
 *  with assertion.
 * 
 * DirectFB_1_4_Port/35   3/8/11 6:33p apandav
 * SW35125-124: DirectFB Build fails on BCM35125
 * 
 * DirectFB_1_4_Port/34   3/5/11 1:32a robertwm
 * SW7405-5179: DirectFB: NEXUS_GRAPHICS2D_QUEUE_FULL messages when
 *  running df_dok with Phase 7.5 refsw.
 * 
 * DirectFB_1_4_Port/33   3/3/11 3:09p christ
 * SW7231-44: DirectFB: Add support for BCM7231A0 chip type.
 *
 * DirectFB_1_4_Port/32   2/24/11 8:16p raywu
 * SW7422-278: DirectFB: ImageProvider needs to be re-architected to use
 *  internal graphics APIs
 *
 * DirectFB_1_4_Port/31   2/20/11 7:29p robertwm
 * SW7420-1525: DirectFB: Cannot build DirectFB with packet-buffer support
 *  on Phase 5.0.
 *
 * DirectFB_1_4_Port/30   2/18/11 9:47p robertwm
 * SW7422-279: DirectFB: Cannot see apps when pixelformat is YUY2 or UYVY.
 *
 * DirectFB_1_4_Port/29   1/25/11 11:43a christ
 * SW7420-1411: DirectFB: Merge of features required for DirectFB 1.4.5
 *  phase 1.5 release - Merge in packet blit driver.
 *
 * DirectFB_1_4_Port/DirectFB_JIRA_SW7420_1134/20   1/4/11 4:25p raywu
 * SW7420-1346: DirectFB: Added dual source batch blit supported
 *
 * DirectFB_1_4_Port/DirectFB_JIRA_SW7420_1134/19   1/3/11 4:42p scchang
 * SW7420-1338: Add A8 and other pixel formats support for FillTrapezoid()
 *  for M2MC core
 *
 * DirectFB_1_4_Port/DirectFB_JIRA_SW7420_1134/18   12/23/10 11:42a christ
 * SW7425-61: DirectFB: Add support for BCM7425A0 chip type.
 *
 * DirectFB_1_4_Port/DirectFB_JIRA_SW7420_1134/17   12/16/10 7:08p robertwm
 * SW7344-17: DirectFB: Require support for BCM7344A0 chip.
 * SW7346-6: DirectFB: Require support for the BCM7346A0 chip.
 *
 * DirectFB_1_4_Port/DirectFB_JIRA_SW7420_1134/16   12/16/10 4:17p robertwm
 * SW7208-145: DirectFB: Need to support BCM7208 B0.
 * SW7340-240: DirectFB: Require support for BCM7340 variants.
 *
 * DirectFB_1_4_Port/DirectFB_JIRA_SW7420_1134/15   12/10/10 10:57a robertwm
 * SW7420-1309: DirectFB: Fix S/T co-ordinates for TextureTriangles using
 *  PX3D
 *
 * DirectFB_1_4_Port/DirectFB_JIRA_SW7420_1134/14   12/8/10 6:17p raywu
 * SW7550-629: DirectFB: Integrated ZSP code to DFB packet buffer branch
 *
 * DirectFB_1_4_Port/DirectFB_JIRA_SW7420_1134/13   12/5/10 4:31p robertwm
 * SW35230-2493: DirectFB does not compile for C0 chip version.
 *
 * DirectFB_1_4_Port/DirectFB_JIRA_SW7420_1134/12   12/3/10 8:37p raywu
 * SW7550-629: DirectFB: Integrated ZSP code to DFB packet buffer branch
 *
 * DirectFB_1_4_Port/DirectFB_JIRA_SW7420_1134/11   11/25/10 8:36p raywu
 * SW7420-1134: Add packet buffer (blit) support for gfxdriver
 *
 * DirectFB_1_4_Port/DirectFB_JIRA_SW7420_1134/10   11/19/10 5:49p raywu
 * SW7420-1251: Add Directfb packet buffer, multi-app, Kernel mode
 *  supported
 *
 * DirectFB_1_4_Port/DirectFB_JIRA_SW7420_1134/9   11/17/10 9:40p raywu
 * SW7420-1134: DirectFB: Add packet buffer (blit) support for gfxdriver;
 *  Multi-App Kernel mode Supported
 *
 * DirectFB_1_4_Port/DirectFB_JIRA_SW7420_1134/8   11/15/10 3:14p robertwm
 * SW7420-1241: DirectFB: Add support for FillTrapezoid() to the driver
 *  and core.
 * SW7420-1242: DirectFB: Add support for SetMatrix() function for both
 *  M2MC and PX3D cores.
 *
 * DirectFB_1_4_Port/DirectFB_JIRA_SW7420_1134/7   11/3/10 7:43p raywu
 * SW7420-1134 : DirectFB: Add packet buffer (blit) support for gfxdriver
 *
 * DirectFB_1_4_Port/DirectFB_JIRA_SW7420_1134/6   11/2/10 7:30p raywu
 * SW7420-1134: DirectFB: Add packet buffer (blit) support for gfxdriver
 *
 * DirectFB_1_4_Port/DirectFB_JIRA_SW7420_1134/5   10/29/10 6:50p raywu
 * SW7420-1134: Add packet buffer (blit) support for gfxdriver
 *
 * DirectFB_1_4_Port/DirectFB_JIRA_SW7420_1134/4   10/28/10 8:13p scchang
 * SW7420-1134: Fix the drawing icon issue on web browser
 *
 * DirectFB_1_4_Port/DirectFB_JIRA_SW7420_1134/3   10/27/10 5:44p robertwm
 * SW7420-1214: DirectFB: Add support for "EmitCommands" to our packet-
 *  blit version of the gfxdriver.
 *
 * DirectFB_1_4_Port/DirectFB_JIRA_SW7420_1134/2   10/8/10 12:44p raywu
 * SW7420-1134: Add packet buffer (blit) support for gfxdriver
 *
 * DirectFB_1_4_Port/DirectFB_JIRA_SW7420_1134/1   10/4/10 8:20p raywu
 * SW7420-1134 : DirectFB: Add packet buffer (blit) support for gfxdriver
 *
 * DirectFB_1_4_Port/25   8/20/10 3:35p robertwm
 * SW7405-4791: DirectFB: DirectFB-1.4.1 Phase 2.5 fails to build with
 *  Nexus Phase 5.0 reference software.
 *
 * DirectFB_1_4_Port/24   8/11/10 4:58p robertwm
 * SW35230-957: DirectFB: platform_init fails to build on 35230 with
 *  recent 3D changes.
 *
 * DirectFB_1_4_Port/23   8/9/10 8:42p robertwm
 * SW7405-4183: DirectFB: Add support for Texture Triangles in the
 *  graphics driver.
 * SW7405-4734: DirectFB: Add hardware support for FillTriangle().
 *
 * DirectFB_1_4_Port/22   7/22/10 4:13p robertwm
 * SW7405-4649: DirectFB: Optimise surface locking and wait idle syncing.
 *
 * DirectFB_1_4_Port/21   7/15/10 11:05a robertwm
 * SW7342-230: DirectFB: Support DirectFB on BCM7342.
 *
 * DirectFB_1_4_Port/20   6/8/10 1:05p robertwm
 * SW7405-4468: DirectFB: Graphics driver fails to build if Nexus doesn't
 *  support source alpha premultiplication.
 * SW7468-252: DirectFB: Need to support 7468B0 chip revision.
 * SW7422-6: DirectFB: Need to support 7422A0 chip revision.
 * SW7420-816: DirectFB: Need to support 7420C1 chip revision.
 * SW35230-276: DirectFB: Support 35230A0 and B0 chip revision checking.
 *
 * DirectFB_1_4_Port/19   4/9/10 4:21p robertwm
 * SW7405-4183: DirectFB: Add support for Texture Triangles in the
 *  graphics driver.
 *
 * DirectFB_1_4_Port/18   3/2/10 10:37a robertwm
 * SW3556-1069: DirectFB: Run-time dynamic linking between gfxdriver and
 *  system drivers not working.
 *
 * DirectFB_1_4_Port/17   12/23/09 4:34p robertwm
 * SW7468-57: DirectFB: Support DirectFB on BCM7208 and BCM7468.
 * SW7405-3668: DirectFB: Support DirectFB on BCM7019.
 *
 * DirectFB_1_4_Port/16   12/23/09 2:19p robertwm
 * SW7550-51: Support DirectFB on 7550.
 *
 * DirectFB_1_4_Port/SW7550-51/1   12/22/09 3:22p mphillip
 * SW7550-51: Add 7550 build support (on branch)
 *
 * DirectFB_1_4_Port/15   12/4/09 9:19p robertwm
 * SW7405-3544: DirectFB: Maximum Vertical Downscale check is incorrect.
 * SW7405-3543: DirectFB: Use NEXUS_Graphics2D_FastBlit() for performing
 *  Copy Blits.
 *
 * DirectFB_1_4_Port/14   12/1/09 1:00p gmohile
 * SW7408-30 : Add 7125 and 7408 support
 *
 * DirectFB_1_4_Port/13   11/30/09 3:52p robertwm
 * SW7405-3471: Nexus needs to support additional pixel format for
 *  DirectFB.
 *
 * DirectFB_1_4_Port/12   11/27/09 2:03p robertwm
 * SW7405-3504: DirectFB: Add accelerated graphics support for
 *  DrawRectangle().
 * SW7405-3503: DirectFB: Destination color keying does not work properly
 *  when alpha premult block is unavailable.
 * SW7405-3502: Poor graphics performance of 1.3.x and 1.4.x compared with
 *  1.0.0.
 *
 * DirectFB_1_4_Port/11   11/25/09 10:52a robertwm
 * SW7405-3471: Nexus needs to support additional pixel formats for
 *  DirectFB.
 * SW7405-3475: Nexus does not perform source color keying correctly if
 *  any form of blending is required.
 * SW7405-3438: DirectFB: Palette based blits do not work correctly.
 * SW7405-3336: DirectFB: A8 blits are NOT displayed correctly.
 * SW7405-3481: DSRO_SMOOTH_UPSCALE not implemented.
 * SW7405-3122: DirectFB: Support different blending combinations,
 *  premultiply alpha ,XOR and mirorr/rotation.
 *
 * DirectFB_1_4_Port/10   10/30/09 2:49p robertwm
 * SW7400-2588: DFB: DirectFB FillRect renders differently with hardware
 *  and software surfaces.
 *
 * DirectFB_1_4_Port/9   10/12/09 6:34p robertwm
 * SW7405-3189: DirectFB: Disable alpha premult by default.
 *
 * DirectFB_1_4_Port/8   10/2/09 10:49a robertwm
 * SW7405-3122: DirectFB: Support different blending combinations,
 *  premultiply alpha ,XOR and mirorr/rotation.
 *
 * DirectFB_1_4_Port/7   8/27/09 6:35p ismailk
 * SW3556-800 : DeviceIds for 93556 added to bcmnexus_gfx.h
 *
 * DirectFB_1_4_Port/6   8/25/09 9:23p robertwm
 * SW7405-2935: DirectFB 1.4.1 introduced a bug when displaying bitmaps
 *  with an alpha channel.
 *
 * DirectFB_1_4_Port/5   8/7/09 3:56p robertwm
 * PR57500: Add DirectFB support for 7413 platform.
 *
 * DirectFB_1_4_Port/4   7/17/09 6:24p robertwm
 * PR56900: Possible font caching problem.
 *
 * DirectFB_1_4_Port/3   7/10/09 1:39p gmohile
 * PR 56731 : Add 7401 chip versions
 *
 * DirectFB_1_4_Port/2   7/9/09 1:31p mphillip
 * PR56658: Add 7403 defines
 *
 * DirectFB_1_4_Port/1   7/8/09 3:18p mphillip
 * PR56658: Add DirectFB 1.4.1 broadcom_files
 *
 * 9   3/16/09 4:43p mphillip
 * PR52806: Add support for DSBLIT_BLEND_COLORALPHA combined with
 *  DSBLIT_BLEND_ALPHACHANNEL
 *
 * 8   2/20/09 11:09a gmohile
 * PR 51629 : Add 7335 support
 *
 * 7   2/4/09 10:43a jrubio
 * PR51629: add 7336 support
 *
 * DirectFB_1_4_Port/4   6/28/09 9:05p robertwm
 * Add support for 7400 E0.
 *
 * DirectFB_1_4_Port/3   6/28/09 8:57p robertwm
 * Add support for 7420 A0,A1 and B0.
 *
 * DirectFB_1_4_Port/2   6/18/09 12:29p robertwm
 * Add in support for 7413A0,B0,B1 devices.
 *
 * 6   1/23/09 4:55p mphillip
 * PR50924: Enable premultiplying alpha blending options
 *
 * 5   12/10/08 1:47p gmohile
 * PR 50167 : Add 3458 B1/B2 support
 *
 * 4   11/6/08 8:23a gmohile
 * PR 45243 : Add 3548 B0 support
 *
 *
 ***************************************************************************/

#ifndef __bcmnexus_gfx_H__
#define __bcmnexus_gfx_H__

#include "bcmnexus.h"
#include "bcmnexus_utils.h"
#ifdef BCMNEXUS_GFX_HAS_P3D
#include <bp3d.h>
#include <nexus_graphics3d_init.h>
#include <nexus_graphics3d_priv.h>
#define NEXUS_THUNK_LAYER
#include <nexus_graphics2d_module.h>
#include <nexus_surface_priv.h>
#endif

#ifdef BCMNEXUS_GFX_PACKET_BUFFER
#include "bm2mc_packet.h"
#endif

typedef enum {
     BCMNEXUS_GFX_IDLE,
     BCMNEXUS_GFX_2D,
     BCMNEXUS_GFX_3D
} bcmnexus_gfxMode;

typedef enum {
     m_None             = 0x0000,
     m_Source           = 0x0001,
     m_Destination      = 0x0002,
     m_Output           = 0x0004,
     m_SrcColorKey      = 0x0008,
     m_DstColorKey      = 0x0010,
     m_BlendEq          = 0x0020,
     m_Filter           = 0x0040,
     m_Matrix           = 0x0080,
     m_ColorMatrix      = 0x0100,
     m_AlphaPremult     = 0x0200,
     m_Mirror           = 0x0400,
     m_Rop              = 0x0800,
     m_All              = 0x0FFF

} BcmnexusStateBits;

#ifndef BCMNEXUS_GFX_PACKET_BUFFER

#define BCMNEXUS_VALIDATE(b)
#define BCMNEXUS_INVALIDATE(b)
#define BCMNEXUS_IS_VALID(b)      (false)
#define BCMNEXUS_ARE_VALID(b)     (false)

#else

#define BCMNEXUS_VALIDATE(b)      (pBdev->stateValidFlags |= (b))
#define BCMNEXUS_INVALIDATE(b)    (pBdev->stateValidFlags &= ~(b))
#define BCMNEXUS_IS_VALID(b)      (pBdev->stateValidFlags & (b))
#define BCMNEXUS_ARE_VALID(b)     (!((pBdev->stateValidFlags & (b)) ^ (b)))

typedef struct bcmnexus_gfxPacketHWStates_tag
{
    // Source Feeder
    BM2MC_PACKET_Plane                      packetSrcPlane;
    uint32_t                                packetSrcColor;
    bool                                    packetSrcNone;

    // Destination Feeder
    BM2MC_PACKET_Plane                      packetDstPlane;
    uint32_t                                packetDstColor;
    bool                                    packetDstNone;

    // Output Feeder
    BM2MC_PACKET_Plane                      packetOutPlane;

    // Blend Equation
    BM2MC_PACKET_Blend                      packetColorBlendOp;
    BM2MC_PACKET_Blend                      packetAlphaBlendOp;
    uint32_t                                packetBlendColor;

    // ROP
    uint32_t                                packetRop;

    // Source Color Key
    uint32_t                                packetSrcColorkey;
    int                                     packetSrcColorkeyEnable;

    // Destination Color Key
    uint32_t                                packetDstColorkey;
    int                                     packetDstColorkeyEnable;

    // Filter
    NEXUS_Graphics2DFilterCoeffs            packetHorzFilter;
    NEXUS_Graphics2DFilterCoeffs            packetVertFilter;
    BM2MC_PACKET_Rectangle                  packetFilterSrcRect;
    BM2MC_PACKET_Rectangle                  packetFilterOutRect;
    BM2MC_PACKET_FilterCoeffs               packetFilterHorzCoeffs;
    BM2MC_PACKET_FilterCoeffs               packetFilterVertCoeffs;
    int                                     packetFilterEnable;

    // Color Matrix
    NEXUS_Graphics2DColorMatrix             packetColorMatrix;
    int                                     packetColorMatrixEnable;

    // Alpha Premultiply
    int                                     packetAlphaPremultiply;

    // Mirror
    int                                     packetMirrorSrcHorz;
    int                                     packetMirrorSrcVert;
    int                                     packetMirrorDstHorz;
    int                                     packetMirrorDstVert;

} bcmnexus_gfxPacketHWStates;

typedef struct bcmnexus_gfxPacketSettings_tag
{
    void *                                  packetBufStartPtr;
    void *                                  packetBufNextPtr;
    size_t                                  packetBufSize;

    NEXUS_Graphics2DHandle                  hGfx2D;

    uint32_t                                packetConstantColor;
    NEXUS_Graphics2DFilterCoeffs            packetHorzFilter;
    NEXUS_Graphics2DFilterCoeffs            packetVertFilter;
    BM2MC_PACKET_Rectangle                  packetSrcRect;
    BM2MC_PACKET_Rectangle                  packetOutRect;
    BM2MC_PACKET_Rectangle                  packetDstRect;

    BM2MC_PACKET_Plane                      packetSrcPlane;
    BM2MC_PACKET_Plane                      packetDstPlane;
    BM2MC_PACKET_Plane                      packetOutPlane;
    bool                                    packetSetRGBYUVColorMatrix;
}bcmnexus_gfxPacketSettings;
#endif

typedef struct
{
    FusionRef                               ref;
    FusionSkirmish                          syncSkirmish;
#ifdef BCMNEXUS_GFX_PACKET_BUFFER
    FusionRef                               packetSpaceAvailRef;
    FusionSkirmish                          packetSpaceAvailSkirmish;
    bcmnexus_gfxPacketHWStates              packetHWStates;
    DFBSurfaceDrawingFlags                  drawingflags;
    DFBSurfaceBlittingFlags                 blittingflags;
    DFBAccelerationMask                     accel;
    DFBSurfaceRenderOptions                 renderOptions;
    BcmnexusStateBits                       stateValidFlags;
#endif
    bcmnexus_gfxMode                        mode;
    unsigned int                            count;
#ifndef BCMNEXUS_CHECKPOINT_IN_SETTINGS_SUPPORTED
    void (*callback) (void *context, int param);
#endif
} bcmnexus_gfxDeviceData;

typedef struct
{
    NEXUS_Graphics2DHandle                  hGfx2D;
    DFBBCMNEXUS                            *pBrcmDfb;               /* pointer to brcm system data structure - needed for surf cache */
#ifndef BCMNEXUS_ALPHA_PREMULT_SUPPORTED
    bool                                    blitSingle;             /* set to false when 2 passes are required in the M2MC */
#endif
    bool                                    blitDest;               /* set to true when "destination" block in M2MC is enabled */
    bool                                    blitFast;               /* set to true when we need to do a fast blit */
    bool                                    odd_shift_enable;       /* set to true if checking for odd alignment in x or w is required */
    NEXUS_Pixel                             sourcePixelFormat;      /* Source surface Nexus pixel format */
    NEXUS_Graphics2DBlitSettings            blitSettings;
    NEXUS_Graphics2DFillSettings            fillSettings;
#ifdef BCMNEXUS_PORTERDUFF_FILL_SUPPORTED
    NEXUS_Graphics2DPorterDuffFillSettings  PorterDuffFillSettings;
#endif
#ifdef BCMNEXUS_GFX_PACKET_BUFFER
    bcmnexus_gfxPacketSettings              packetSettings;
#endif
    NEXUS_SurfaceCreateSettings             intermediateSurfaceSettings;
    NEXUS_SurfaceHandle                     hIntermediateSurface;   /* required when alpha blending AND colour blending */
#ifdef BCMNEXUS_GFX_HAS_P3D
    BP3D_Handle                             hP3d;
    BP3D_Context                            hContext;
    uint32_t                                diffuseColor;           /* diffuse color in ARGB8888 */
    uint32_t                                specularColor;          /* specular color in FRGB888, F=Fog */
    float                                   fSrcWidth;
    float                                   fSrcHeight;
    float                                   fSrcS;
    float                                   fSrcT;
#ifdef BCMNEXUS_SOFT3D_MATRIX_SUPPORTED
    bool                                    matrix_3d;
#endif
    bool                                    gfx_3d_support;
#endif
    s32                                     matrix[9];
    bool                                    matrix_affine;
    bool                                    matrix_enabled;
    bool                                    matrix_scales;
    bool                                    matrix_flips_x;
    bool                                    matrix_flips_y;
    DFBRegion                               dst_clip;

} bcmnexus_gfxDriverData;

#ifdef BCMNEXUS_PIXELFORMAT_R8_G8_B8_SUPPORTED
#define NEXUS_PIXEL_FORMAT_ALPHA_NONE(f)    \
(                                           \
    ((f) == NEXUS_PixelFormat_eR5_G6_B5) || \
    ((f) == NEXUS_PixelFormat_eR8_G8_B8)    \
)
#else
#define NEXUS_PIXEL_FORMAT_ALPHA_NONE(f)    \
(                                           \
    ((f) == NEXUS_PixelFormat_eR5_G6_B5)    \
)
#endif

#ifdef BCMNEXUS_GFX_PACKET_BUFFER
#define bcmnexus_gfx_SUPPORTED_DRAWINGFLAGS \
               (DSDRAW_BLEND | DSDRAW_DST_COLORKEY | DSDRAW_XOR |DSDRAW_SRC_PREMULTIPLY)
#else
#define bcmnexus_gfx_SUPPORTED_DRAWINGFLAGS \
               (DSDRAW_BLEND | DSDRAW_SRC_PREMULTIPLY)
#endif

#ifdef BCMNEXUS_TRAPEZOID_SUPPORTED
#define bcmnexus_gfx_SUPPORTED_DRAWINGFUNCTIONS \
               (DFXL_FILLRECTANGLE | DFXL_DRAWRECTANGLE | DFXL_FILLTRAPEZOID)
#else
#define bcmnexus_gfx_SUPPORTED_DRAWINGFUNCTIONS \
               (DFXL_FILLRECTANGLE | DFXL_DRAWRECTANGLE)
#endif

/* ZSP graphics support */
#if NEXUS_HAS_ZSP_GRAPHICS
#define bcmnexus_gfx_SUPPORTED_BLITTINGFLAGS  \
              ( DSBLIT_BLEND_ALPHACHANNEL   | \
                DSBLIT_BLEND_COLORALPHA     | \
                DSBLIT_COLORIZE             | \
                DSBLIT_SRC_PREMULTCOLOR     | \
                DSBLIT_XOR )

#define bcmnexus_gfx_SUPPORTED_BLITTINGFUNCTIONS \
              ( DFXL_BLIT )
#else
/* M2MC graphics support */
#define bcmnexus_gfx_SUPPORTED_BLITTINGFLAGS  \
              ( DSBLIT_BLEND_ALPHACHANNEL   | \
                DSBLIT_BLEND_COLORALPHA     | \
                DSBLIT_COLORIZE             | \
                DSBLIT_SRC_PREMULTIPLY      | \
                DSBLIT_SRC_COLORKEY         | \
                DSBLIT_DST_COLORKEY         | \
                DSBLIT_SRC_PREMULTCOLOR     | \
                DSBLIT_XOR                  | \
                DSBLIT_ROTATE180            | \
                DSBLIT_FLIP_HORIZONTAL      | \
                DSBLIT_FLIP_VERTICAL )

#define bcmnexus_gfx_SUPPORTED_BLITTINGFUNCTIONS \
              ( DFXL_BLIT                   |    \
                DFXL_BLIT2                  |    \
                DFXL_STRETCHBLIT )
#endif

/* PX3D graphics support */
#define bcmnexus_gfx_p3d_SUPPORTED_DRAWINGFLAGS \
              ( DSDRAW_BLEND                |   \
                DSDRAW_SRC_PREMULTIPLY )

#define bcmnexus_gfx_p3d_SUPPORTED_DRAWINGFUNCTIONS \
              ( DFXL_FILLRECTANGLE          |       \
                DFXL_DRAWRECTANGLE          |       \
                DFXL_FILLTRIANGLE           |       \
                DFXL_FILLTRIANGLES          |       \
                DFXL_FILLTRAPEZOID )

#define bcmnexus_gfx_p3d_SUPPORTED_BLITTINGFLAGS \
              ( DSBLIT_SRC_COLORKEY         |    \
                DSBLIT_BLEND_COLORALPHA     |    \
                DSBLIT_BLEND_ALPHACHANNEL   |    \
                DSBLIT_ROTATE180            |    \
                DSBLIT_FLIP_HORIZONTAL      |    \
                DSBLIT_FLIP_VERTICAL )

#define bcmnexus_gfx_p3d_SUPPORTED_BLITTINGFUNCTIONS \
              ( DFXL_BLIT                   |        \
                DFXL_STRETCHBLIT            |        \
                DFXL_TEXTRIANGLES )

/* Event timeout in milliseconds to wait for graphics to complete */
#define bcmnexus_gfx_EVENT_TIMEOUT 5000

/* Define Graphics 2D flush counts and increments */
#if NEXUS_HAS_ZSP_GRAPHICS

#define BCMNEXUS_GRAPHICS2D_FLUSH_COUNT     20
#define BCMNEXUS_GRAPHICS2D_DRAW_RECT_INC   4

#elif defined(BCMNEXUS_GFX_PACKET_BUFFER)

#define BCMNEXUS_GRAPHICS2D_FLUSH_COUNT     4000
#define BCMNEXUS_GRAPHICS2D_DRAW_RECT_INC   1

#else

#define BCMNEXUS_GRAPHICS2D_FLUSH_COUNT     400
#define BCMNEXUS_GRAPHICS2D_DRAW_RECT_INC   4

#endif
#define BCMNEXUS_GRAPHICS2D_FILL_RECT_INC   1
#define BCMNEXUS_GRAPHICS2D_FILL_TRAP_INC   1
#define BCMNEXUS_GRAPHICS2D_BLIT_INC        1

/* Define Graphics 3D flush counts and increments */
#define BCMNEXUS_GRAPHICS3D_FLUSH_COUNT     400
#define BCMNEXUS_GRAPHICS3D_DRAW_RECT_INC   4
#define BCMNEXUS_GRAPHICS3D_DRAW_LINE_INC   1
#define BCMNEXUS_GRAPHICS3D_FILL_RECT_INC   4
#define BCMNEXUS_GRAPHICS3D_FILL_TRAP_INC   6
#define BCMNEXUS_GRAPHICS3D_FILL_TRI_INC    3
#define BCMNEXUS_GRAPHICS3D_TEXT_TRI_INC    3
#define BCMNEXUS_GRAPHICS3D_BLIT_INC        6

#define ARGB_MASK   0xFFFFFFFF
#define RGB_MASK    0x00FFFFFF
#define ROP_XOR     0x66
#define ROP_SOURCE  0xCC


/* device ID setup.  Try to match build for build */
#define BCM7401A0_DEVICE_ID 0x74010000
#define BCM7401C0_DEVICE_ID 0x74010020
#define BCM7401C1_DEVICE_ID 0x74010021
#define BCM7401C2_DEVICE_ID 0x74010022
#define BCM7401C3_DEVICE_ID 0x74010023

#define BCM7400B0_DEVICE_ID 0x74000010
#define BCM7400C0_DEVICE_ID 0x74000020
#define BCM7400D0_DEVICE_ID 0x74000030
#define BCM7400E0_DEVICE_ID 0x74000040

#define BCM7403A0_DEVICE_ID 0x74030000
#define BCM7403A1_DEVICE_ID 0x74030001

#define BCM7405A0_DEVICE_ID 0x74050000
#define BCM7405A1_DEVICE_ID 0x74050001
#define BCM7405B0_DEVICE_ID 0x74050010
#define BCM7405B1_DEVICE_ID 0x74050011

#define BCM7413A0_DEVICE_ID 0x74130000
#define BCM7413B0_DEVICE_ID 0x74130010
#define BCM7413B1_DEVICE_ID 0x74130011

#define BCM7420A0_DEVICE_ID 0x74200000
#define BCM7420A1_DEVICE_ID 0x74200001
#define BCM7420B0_DEVICE_ID 0x74200010
#define BCM7420C0_DEVICE_ID 0x74200020
#define BCM7420C1_DEVICE_ID 0x74200021

#define BCM7422A0_DEVICE_ID 0x74220000
#define BCM7422A1_DEVICE_ID 0x74220001

#define BCM7425A0_DEVICE_ID 0x74250000
#define BCM7425A1_DEVICE_ID 0x74250001

#define BCM3548A0_DEVICE_ID 0x35480000
#define BCM3548B0_DEVICE_ID 0x35480010
#define BCM3548B1_DEVICE_ID 0x35480011
#define BCM3548B2_DEVICE_ID 0x35480012
#define BCM3548B3_DEVICE_ID 0x35480013
#define BCM3548B4_DEVICE_ID 0x35480014

#define BCM3556A0_DEVICE_ID 0x35560000
#define BCM3556B0_DEVICE_ID 0x35560010
#define BCM3556B1_DEVICE_ID 0x35560011
#define BCM3556B2_DEVICE_ID 0x35560012
#define BCM3556C0_DEVICE_ID 0x35560020

#define BCM7325A0_DEVICE_ID 0x73250000
#define BCM7325A1_DEVICE_ID 0x73250001
#define BCM7325A2_DEVICE_ID 0x73250002
#define BCM7325B0_DEVICE_ID 0x73250010
#define BCM7325B1_DEVICE_ID 0x73250011
#define BCM7325C0_DEVICE_ID 0x73250020

#define BCM7335A0_DEVICE_ID 0x73350000
#define BCM7335A1_DEVICE_ID 0x73350001
#define BCM7335B0_DEVICE_ID 0x73350010
#define BCM7336A0_DEVICE_ID 0x73360000

#define BCM7340A0_DEVICE_ID 0x73400000
#define BCM7340B0_DEVICE_ID 0x73400010

#define BCM7342A0_DEVICE_ID 0x73420000
#define BCM7342B0_DEVICE_ID 0x73420010

#define BCM7344A0_DEVICE_ID 0x73440000

#define BCM7346A0_DEVICE_ID 0x73460000

#define BCM7125A0_DEVICE_ID 0x71250000
#define BCM7125B0_DEVICE_ID 0x71250010
#define BCM7125B1_DEVICE_ID 0x71250011
#define BCM7125C0_DEVICE_ID 0x71250020

#define BCM7408A0_DEVICE_ID 0x74080000

#define BCM7019A0_DEVICE_ID 0x70190000
#define BCM7019B0_DEVICE_ID 0x70190010
#define BCM7019C0_DEVICE_ID 0x70190020

#define BCM7468A0_DEVICE_ID 0x74680000
#define BCM7468B0_DEVICE_ID 0x74680010

#define BCM7208A0_DEVICE_ID 0x74680000
#define BCM7208B0_DEVICE_ID 0x74680010

#define BCM7231A0_DEVICE_ID 0x72310000

#define BCM35230A0_DEVICE_ID 0x03523000
#define BCM35230B0_DEVICE_ID 0x03523010
#define BCM35230C0_DEVICE_ID 0x03523020

#define BCM35125A0_DEVICE_ID 0x03512500
#define BCM35125B0_DEVICE_ID 0x03512510
#define BCM35125C0_DEVICE_ID 0x03512520
#define BCM35125C1_DEVICE_ID 0x03512521

#define BCM7550A0_DEVICE_ID  0x75500000
#define BCM7550A1_DEVICE_ID  0x75500001
#define BCM7550B0_DEVICE_ID  0x75500010

#if (BCHP_CHIP == 7401)
/* List supported 7401 rev's here */
#if (BCHP_VER==BCHP_VER_A0)
#define BCM740X_DEVICE_ID BCM7401A0_DEVICE_ID
#elif (BCHP_VER==BCHP_VER_C0)
#define BCM740X_DEVICE_ID BCM7401C0_DEVICE_ID
#elif (BCHP_VER==BCHP_VER_C1)
#define BCM740X_DEVICE_ID BCM7401C1_DEVICE_ID
#elif (BCHP_VER==BCHP_VER_C2)
#define BCM740X_DEVICE_ID BCM7401C2_DEVICE_ID
#elif (BCHP_VER==BCHP_VER_C3)
#define BCM740X_DEVICE_ID BCM7401C3_DEVICE_ID
#endif
#endif

#if (BCHP_CHIP == 7400)
#if (BCHP_VER==BCHP_VER_B0)
#define BCM740X_DEVICE_ID BCM7400B0_DEVICE_ID
#endif
#if (BCHP_VER==BCHP_VER_C0)
#define BCM740X_DEVICE_ID BCM7400C0_DEVICE_ID
#endif
#if (BCHP_VER==BCHP_VER_D0)
#define BCM740X_DEVICE_ID BCM7400D0_DEVICE_ID
#endif
#if (BCHP_VER==BCHP_VER_E0)
#define BCM740X_DEVICE_ID BCM7400E0_DEVICE_ID
#endif
#endif

#if (BCHP_CHIP == 7403)
#if (BCHP_VER==BCHP_VER_A0)
#define BCM740X_DEVICE_ID BCM7403A0_DEVICE_ID
#elif (BCHP_VER==BCHP_VER_A1)
#define BCM740X_DEVICE_ID BCM7403A1_DEVICE_ID
#endif
#endif

#if (BCHP_CHIP == 7405)
#if (BCHP_VER==BCHP_VER_A0)
#define BCM740X_DEVICE_ID BCM7405A0_DEVICE_ID
#endif
#if (BCHP_VER==BCHP_VER_A1)
#define BCM740X_DEVICE_ID BCM7405A1_DEVICE_ID
#endif
#if (BCHP_VER==BCHP_VER_B0)
#define BCM740X_DEVICE_ID BCM7405B0_DEVICE_ID
#endif
#if (BCHP_VER==BCHP_VER_B1)
#define BCM740X_DEVICE_ID BCM7405B1_DEVICE_ID
#endif

/* 7413A0 is compiled with BCHP_CHIP=7405 and BCHP_VER=C0 */
#if (BCHP_VER==BCHP_VER_C0)
#define BCM740X_DEVICE_ID BCM7413A0_DEVICE_ID
#endif
/* 7413B0 is compiled with BCHP_CHIP=7405 and BCHP_VER=D0 */
#if (BCHP_VER==BCHP_VER_D0)
#define BCM740X_DEVICE_ID BCM7413B0_DEVICE_ID
#endif
/* 7413B1 is compiled with BCHP_CHIP=7405 and BCHP_VER=D1 */
#if (BCHP_VER==BCHP_VER_D1)
#define BCM740X_DEVICE_ID BCM7413B1_DEVICE_ID
#endif
#endif

#if (BCHP_CHIP == 7325)
#if (BCHP_VER==BCHP_VER_A0)
#define BCM740X_DEVICE_ID BCM7325A0_DEVICE_ID
#endif
#if (BCHP_VER==BCHP_VER_A1)
#define BCM740X_DEVICE_ID BCM7325A1_DEVICE_ID
#endif
#if (BCHP_VER==BCHP_VER_A2)
#define BCM740X_DEVICE_ID BCM7325A2_DEVICE_ID
#endif
#if (BCHP_VER==BCHP_VER_B0)
#define BCM740X_DEVICE_ID BCM7325B0_DEVICE_ID
#endif
#if (BCHP_VER==BCHP_VER_B1)
#define BCM740X_DEVICE_ID BCM7325B1_DEVICE_ID
#endif
#if (BCHP_VER==BCHP_VER_C0)
#define BCM740X_DEVICE_ID BCM7325C0_DEVICE_ID
#endif
#endif

#if (BCHP_CHIP == 7336)
#if (BCHP_VER==BCHP_VER_A0)
#define BCM740X_DEVICE_ID BCM7336A0_DEVICE_ID
#endif
#endif

#if (BCHP_CHIP == 7335)
#if (BCHP_VER==BCHP_VER_A0)
#define BCM740X_DEVICE_ID BCM7335A0_DEVICE_ID
#endif
#if (BCHP_VER==BCHP_VER_A1)
#define BCM740X_DEVICE_ID BCM7335A1_DEVICE_ID
#endif
#if (BCHP_VER==BCHP_VER_B0)
#define BCM740X_DEVICE_ID BCM7335B0_DEVICE_ID
#endif
#endif

#if (BCHP_CHIP == 7340)
#if (BCHP_VER==BCHP_VER_A0)
#define BCM740X_DEVICE_ID BCM7340A0_DEVICE_ID
#endif
#if (BCHP_VER==BCHP_VER_B0)
#define BCM740X_DEVICE_ID BCM7340B0_DEVICE_ID
#endif
#endif

#if (BCHP_CHIP == 7342)
#if (BCHP_VER==BCHP_VER_A0)
#define BCM740X_DEVICE_ID BCM7342A0_DEVICE_ID
#endif
#if (BCHP_VER==BCHP_VER_B0)
#define BCM740X_DEVICE_ID BCM7342B0_DEVICE_ID
#endif
#endif

#if (BCHP_CHIP == 7344)
#if (BCHP_VER==BCHP_VER_A0)
#define BCM740X_DEVICE_ID BCM7344A0_DEVICE_ID
#endif
#endif

#if (BCHP_CHIP == 7346)
#if (BCHP_VER==BCHP_VER_A0)
#define BCM740X_DEVICE_ID BCM7346A0_DEVICE_ID
#endif
#endif

#if (BCHP_CHIP == 7420)
#if (BCHP_VER==BCHP_VER_A0)
#define BCM740X_DEVICE_ID BCM7420A0_DEVICE_ID
#endif
#if (BCHP_VER==BCHP_VER_A1)
#define BCM740X_DEVICE_ID BCM7420A1_DEVICE_ID
#endif
#if (BCHP_VER==BCHP_VER_B0)
#define BCM740X_DEVICE_ID BCM7420B0_DEVICE_ID
#endif
#if (BCHP_VER==BCHP_VER_C0)
#define BCM740X_DEVICE_ID BCM7420C0_DEVICE_ID
#endif
#if (BCHP_VER==BCHP_VER_C1)
#define BCM740X_DEVICE_ID BCM7420C1_DEVICE_ID
#endif
#endif

#if (BCHP_CHIP == 7422)
#if (BCHP_VER==BCHP_VER_A0)
#define BCM740X_DEVICE_ID BCM7422A0_DEVICE_ID
#endif
#if (BCHP_VER==BCHP_VER_A1)
#define BCM740X_DEVICE_ID BCM7422A1_DEVICE_ID
#endif
#endif

#if (BCHP_CHIP == 7425)
#if (BCHP_VER==BCHP_VER_A0)
#define BCM740X_DEVICE_ID BCM7425A0_DEVICE_ID
#endif
#if (BCHP_VER==BCHP_VER_A1)
#define BCM740X_DEVICE_ID BCM7425A1_DEVICE_ID
#endif
#endif

#if (BCHP_CHIP == 3548)
#if (BCHP_VER==BCHP_VER_A0)
#define BCM740X_DEVICE_ID BCM3548A0_DEVICE_ID
#endif
#if (BCHP_VER==BCHP_VER_B0)
#define BCM740X_DEVICE_ID BCM3548B0_DEVICE_ID
#endif
#if (BCHP_VER==BCHP_VER_B1)
#define BCM740X_DEVICE_ID BCM3548B1_DEVICE_ID
#endif
#if (BCHP_VER==BCHP_VER_B2)
#define BCM740X_DEVICE_ID BCM3548B2_DEVICE_ID
#endif
#if (BCHP_VER==BCHP_VER_B3)
#define BCM740X_DEVICE_ID BCM3548B3_DEVICE_ID
#endif
#if (BCHP_VER==BCHP_VER_B4)
#define BCM740X_DEVICE_ID BCM3548B4_DEVICE_ID
#endif
#endif

#if (BCHP_CHIP == 3556)
#if (BCHP_VER==BCHP_VER_A0)
#define BCM740X_DEVICE_ID BCM3556A0_DEVICE_ID
#endif
#if (BCHP_VER==BCHP_VER_B0)
#define BCM740X_DEVICE_ID BCM3556B0_DEVICE_ID
#endif
#if (BCHP_VER==BCHP_VER_B1)
#define BCM740X_DEVICE_ID BCM3556B1_DEVICE_ID
#endif
#if (BCHP_VER==BCHP_VER_B2)
#define BCM740X_DEVICE_ID BCM3556B2_DEVICE_ID
#endif
#if (BCHP_VER==BCHP_VER_C0)
#define BCM740X_DEVICE_ID BCM3556C0_DEVICE_ID
#endif
#endif

#if (BCHP_CHIP == 7125)
#if (BCHP_VER==BCHP_VER_A0)
#define BCM740X_DEVICE_ID BCM7125A0_DEVICE_ID
#endif
#if (BCHP_VER==BCHP_VER_B0)
#define BCM740X_DEVICE_ID BCM7125B0_DEVICE_ID
#endif
#if (BCHP_VER==BCHP_VER_B1)
#define BCM740X_DEVICE_ID BCM7125B1_DEVICE_ID
#endif
#if (BCHP_VER==BCHP_VER_C0)
#define BCM740X_DEVICE_ID BCM7125C0_DEVICE_ID
#endif
#endif

#if (BCHP_CHIP == 7408)
#if (BCHP_VER==BCHP_VER_A0)
#define BCM740X_DEVICE_ID BCM7408A0_DEVICE_ID
#endif
#endif

#if (BCHP_CHIP == 7019)
#if (BCHP_VER==BCHP_VER_A0)
#define BCM740X_DEVICE_ID BCM7019A0_DEVICE_ID
#endif
#if (BCHP_VER==BCHP_VER_B0)
#define BCM740X_DEVICE_ID BCM7019B0_DEVICE_ID
#endif
#if (BCHP_VER==BCHP_VER_C0)
#define BCM740X_DEVICE_ID BCM7019C0_DEVICE_ID
#endif
#endif

#if (BCHP_CHIP == 7208)
#if (BCHP_VER==BCHP_VER_A0)
#define BCM740X_DEVICE_ID BCM7208A0_DEVICE_ID
#endif
#if (BCHP_VER==BCHP_VER_B0)
#define BCM740X_DEVICE_ID BCM7208B0_DEVICE_ID
#endif
#endif

#if (BCHP_CHIP == 7231)
#if (BCHP_VER==BCHP_VER_A0)
#define BCM740X_DEVICE_ID BCM7231A0_DEVICE_ID
#endif
#endif

#if (BCHP_CHIP == 7468)
#if (BCHP_VER==BCHP_VER_A0)
#define BCM740X_DEVICE_ID BCM7468A0_DEVICE_ID
#endif
#if (BCHP_VER==BCHP_VER_B0)
#define BCM740X_DEVICE_ID BCM7468B0_DEVICE_ID
#endif
#endif

#if (BCHP_CHIP == 35230)
#if (BCHP_VER==BCHP_VER_A0)
#define BCM740X_DEVICE_ID BCM35230A0_DEVICE_ID
#endif
#if (BCHP_VER==BCHP_VER_B0)
#define BCM740X_DEVICE_ID BCM35230B0_DEVICE_ID
#endif
#if (BCHP_VER==BCHP_VER_C0)
#define BCM740X_DEVICE_ID BCM35230C0_DEVICE_ID
#endif
#endif

#if (BCHP_CHIP == 35125)
#if (BCHP_VER==BCHP_VER_A0)
#define BCM740X_DEVICE_ID BCM35125A0_DEVICE_ID
#endif
#if (BCHP_VER==BCHP_VER_B0)
#define BCM740X_DEVICE_ID BCM35125B0_DEVICE_ID
#endif
#if (BCHP_VER==BCHP_VER_C0)
#define BCM740X_DEVICE_ID BCM35125C0_DEVICE_ID
#endif
#if (BCHP_VER==BCHP_VER_C1)
#define BCM740X_DEVICE_ID BCM35125C1_DEVICE_ID
#endif
#endif
#if (BCHP_CHIP == 7550)
#if (BCHP_VER==BCHP_VER_A0)
#define BCM740X_DEVICE_ID BCM7550A0_DEVICE_ID
#endif
#if (BCHP_VER==BCHP_VER_A1)
#define BCM740X_DEVICE_ID BCM7550A1_DEVICE_ID
#endif
#if (BCHP_VER==BCHP_VER_B0)
#define BCM740X_DEVICE_ID BCM7550B0_DEVICE_ID
#endif
#endif

#ifndef BCM740X_DEVICE_ID
#error "Unknown/untested chip rev"
#endif

#endif
