/***************************************************************************
 *     (c)2007-2010 Broadcom Corporation
 *
 *  This program is the proprietary software of Broadcom Corporation and/or its licensors,
 *  and may only be used, duplicated, modified or distributed pursuant to the terms and
 *  conditions of a separate, written license agreement executed between you and Broadcom
 *  (an "Authorized License").  Except as set forth in an Authorized License, Broadcom grants
 *  no license (express or implied), right to use, or waiver of any kind with respect to the
 *  Software, and Broadcom expressly reserves all rights in and to the Software and all
 *  intellectual property rights therein.  IF YOU HAVE NO AUTHORIZED LICENSE, THEN YOU
 *  HAVE NO RIGHT TO USE THIS SOFTWARE IN ANY WAY, AND SHOULD IMMEDIATELY
 *  NOTIFY BROADCOM AND DISCONTINUE ALL USE OF THE SOFTWARE.
 *
 *  Except as expressly set forth in the Authorized License,
 *
 *  1.     This program, including its structure, sequence and organization, constitutes the valuable trade
 *  secrets of Broadcom, and you shall use all reasonable efforts to protect the confidentiality thereof,
 *  and to use this information only in connection with your use of Broadcom integrated circuit products.
 *
 *  2.     TO THE MAXIMUM EXTENT PERMITTED BY LAW, THE SOFTWARE IS PROVIDED "AS IS"
 *  AND WITH ALL FAULTS AND BROADCOM MAKES NO PROMISES, REPRESENTATIONS OR
 *  WARRANTIES, EITHER EXPRESS, IMPLIED, STATUTORY, OR OTHERWISE, WITH RESPECT TO
 *  THE SOFTWARE.  BROADCOM SPECIFICALLY DISCLAIMS ANY AND ALL IMPLIED WARRANTIES
 *  OF TITLE, MERCHANTABILITY, NONINFRINGEMENT, FITNESS FOR A PARTICULAR PURPOSE,
 *  LACK OF VIRUSES, ACCURACY OR COMPLETENESS, QUIET ENJOYMENT, QUIET POSSESSION
 *  OR CORRESPONDENCE TO DESCRIPTION. YOU ASSUME THE ENTIRE RISK ARISING OUT OF
 *  USE OR PERFORMANCE OF THE SOFTWARE.
 *
 *  3.     TO THE MAXIMUM EXTENT PERMITTED BY LAW, IN NO EVENT SHALL BROADCOM OR ITS
 *  LICENSORS BE LIABLE FOR (i) CONSEQUENTIAL, INCIDENTAL, SPECIAL, INDIRECT, OR
 *  EXEMPLARY DAMAGES WHATSOEVER ARISING OUT OF OR IN ANY WAY RELATING TO YOUR
 *  USE OF OR INABILITY TO USE THE SOFTWARE EVEN IF BROADCOM HAS BEEN ADVISED OF
 *  THE POSSIBILITY OF SUCH DAMAGES; OR (ii) ANY AMOUNT IN EXCESS OF THE AMOUNT
 *  ACTUALLY PAID FOR THE SOFTWARE ITSELF OR U.S. $1, WHICHEVER IS GREATER. THESE
 *  LIMITATIONS SHALL APPLY NOTWITHSTANDING ANY FAILURE OF ESSENTIAL PURPOSE OF
 *  ANY LIMITED REMEDY.
 *
 * $brcm_Workfile: bcmnexus_bt.c $
 * $brcm_Revision: DirectFB_1_4_Port/DirectFB_1_4_1/1 $
 *
 * [File Description:]
 *
 * Revision History:
 *
 * $brcm_Log: /AppLibs/opensource/directfb/src/broadcom_files/private/DirectFB/1.4.1/inputdrivers/bcmnexus/bcmnexus_bt.c $
 * 
 * DirectFB_1_4_Port/DirectFB_1_4_1/1   12/13/10 7:56p scchang
 * SW7420-1313: Add bluetooth handset support
 * 

 * This code implements a DirectFB input driver for the Bluetooth remote, keyboard, and mouse.
 * It was created to covert HID keycodes to DirectFB keyevents.
 * It assumed that the input HID keycodes from Bluetooth driver are derived from
 * 		- Consumer Page (0x0C) for remote keys
 * 		- Keyboard/Keypad Page (0x07) for keyboard keys
 * 		- Generic Desktop Page (0x01) for mouse movement
 * 		- Button Page (0x09) for mouse button events
 ***************************************************************************/

#include "shim_nexus.h" // Nexus function redirection

#include <config.h>

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <errno.h>
#include <limits.h>

#include <sys/types.h>
#include <sys/socket.h>
#include <sys/un.h>

#include <directfb.h>
#include <directfb_keynames.h>

#include <core/coredefs.h>
#include <core/coretypes.h>

#include <core/input.h>

#include <direct/debug.h>
#include <direct/mem.h>
#include <direct/messages.h>
#include <direct/thread.h>

#include <core/core.h>
#include <core/input_driver.h>

#include "bt_bsa.h"

/*
** Broadcom specific includes.
*/
#include "bstd.h"

D_DEBUG_DOMAIN( bcmnexusBtInput, "bcmNexus/BtInput", "Broadcom Bluetooth Input Event driver" );
DFB_INPUT_DRIVER( bcmNexusBtInputDevice )

/*
** type definitions.
*/

/*
** For mapping hardware codes to DirectFB key codes.
*/
typedef struct {

    DFBInputDeviceKeySymbol     dfbKeySymbol;
    uint32_t                    hwEventCode;

} KeyMapElement;

/* Include the correct BT Keycodes file.  This is defined on the make command line
   with the option DIRECTFB_BT_KEYCODES_FILE=xxxx. */
#include DIRECTFB_BT_KEYCODES_FILE

/*
** Constant definitions.
*/


/*
** Global data.
*/

static CoreInputDevice *my_pDfbDevice;
static int icsmrNumBasicCodes;   /* these variables store the size of the preceding arrays */
static int icsmrNumAdvanCodes;
static int iKbdNumBasicCodes;
static int iKbdNumAdvanCodes;

/*
** Broadcom specific routines.
*/

/*
** Routine to map the Consumer Page codes to DirectFB keycodes.
*/
static void BDFB_lookup_bt_csmr_keycodes(
    uint32_t             uiEventCode,
    DFBInputEvent       *pDfbEvent
    )
{
    int i;

    /*
    **  Hardware keycode, i.e. no mapping applied.
    */
    pDfbEvent->key_code = uiEventCode;

    /*
    ** Basic mapping, modifier independent.
    ** If we don't get a "hit" on the hardware key code, we'll return "DIKI_UNKNOWN"
    */

    pDfbEvent->key_id = DIKI_UNKNOWN;

    for( i=0; i < icsmrNumBasicCodes; i++ )
    {
        if ( uiEventCode == csmrBasicCodeMap[ i ].hwEventCode )
        {
            pDfbEvent->key_id = csmrBasicCodeMap[ i ].dfbKeySymbol;
            break;
        }
    }

    /*
    ** Advanced mapping, unicode compatible, modified dependent.
    ** If we don't get a "hit" on the hardware key code, we'll return "DIKS_NULL".
    ** Is this the right solution for the real code?
    */

    pDfbEvent->key_symbol = DIKS_NULL;


    for( i=0; i < icsmrNumAdvanCodes; i++ )
    {
        if ( uiEventCode == csmrAdvanCodeMap[ i ].hwEventCode )
        {
            pDfbEvent->key_symbol = csmrAdvanCodeMap[ i ].dfbKeySymbol;
            break;
        }
    }

    return;

}       /* end of BDFB_lookup_bt_csmr_keycodes */


/*
** Routine to map the Keyboard Page codes to DirectFB keycodes.
*/

static void BDFB_lookup_bt_kbd_keycodes(
    uint32_t             uiEventCode,
    DFBInputEvent       *pDfbEvent, 
    uint8_t              attr
    )
{
    /*
    **  Hardware keycode, i.e. no mapping applied.
    */
    pDfbEvent->key_code = uiEventCode;

    /*
    ** Basic mapping, modifier independent.
    ** If we don't get a "hit" on the hardware key code, we'll return "DIKI_UNKNOWN"
    */

    pDfbEvent->key_id = DIKI_UNKNOWN;

    if (uiEventCode < iKbdNumBasicCodes)
    {
        if (attr & 0x80)
            pDfbEvent->key_id = kbdBasicCodeAltMap[uiEventCode];
        else
            pDfbEvent->key_id = kbdBasicCodeMap[uiEventCode];
    }

    /*
    ** Advanced mapping, unicode compatible, modified dependent.
    ** If we don't get a "hit" on the hardware key code, we'll return "DIKS_NULL".
    ** Is this the right solution for the real code?
    */

    pDfbEvent->key_symbol = DIKS_NULL;

    if (uiEventCode < iKbdNumAdvanCodes)
    {
        if (attr & 0x80)
            pDfbEvent->key_symbol = kbdAdvanCodeAltMap[uiEventCode];
        else
            pDfbEvent->key_symbol = kbdAdvanCodeMap[uiEventCode];
    }
}       /* end of BDFB_lookup_bt_kbd_keycodes() */


/*
**  Data ready callback for the BT receiver.
**  key_attr: b0 - 0:key-up, 1:key-down
**            b1 - 1:repeating
**            b7 - 1:shift/alt on
*/
void BDFB_bt_input_dataReady(unsigned short eventCode, unsigned char usagepage, unsigned char key_attr)
{
    DFBInputEvent        dfbEvent;

    memset(&dfbEvent, 0, sizeof( dfbEvent ));

    D_DEBUG_AT(bcmnexusBtInput, "%s: eventCode=0x%x\n", __FUNCTION__, eventCode);

    if (my_pDfbDevice)
    {
        if (usagepage == 0x01)
        {   /* Mouse events */
            dfbEvent.type  = DIET_AXISMOTION;
            dfbEvent.flags = DIEF_AXISREL;

            if (key_attr & 0x01)
            {
                dfbEvent.axis = DIAI_X;
            }
            else if (key_attr & 0x02)
            {
                dfbEvent.axis = DIAI_Y;
            }
            else
            {
                dfbEvent.axis = DIAI_Z;
            }
				
            dfbEvent.axisrel = ((signed short) eventCode);

            dfb_input_dispatch(my_pDfbDevice, &dfbEvent);			
        }
        else if (usagepage == 0x09)
        {   /* Mouse Button events */
            dfbEvent.flags = DIEF_NONE;

            if (eventCode == 0)
            {
                dfbEvent.button = DIBI_LEFT;
            }
            else if (eventCode == 1)
            {
                dfbEvent.button = DIBI_RIGHT;
            }
            else
            {
                dfbEvent.button = DIBI_MIDDLE;
            }

            dfbEvent.type = (key_attr) ? DIET_BUTTONPRESS : DIET_BUTTONRELEASE;				
            dfb_input_dispatch(my_pDfbDevice, &dfbEvent);	
        }
        else
        {   /* Keyboard or Remote events */
            dfbEvent.flags = ( DIEF_KEYID | DIEF_KEYSYMBOL | DIEF_KEYCODE );

            if (key_attr & 0x02)
            {
                dfbEvent.flags |= DIEF_REPEAT;
            }

            if (usagepage == 0x0C)
            {
                BDFB_lookup_bt_csmr_keycodes(eventCode, &dfbEvent);
            }
            else if (usagepage == 0x07)
            {
                BDFB_lookup_bt_kbd_keycodes(eventCode, &dfbEvent, key_attr);
            }

            dfbEvent.type = (key_attr&0x01) ? DIET_KEYPRESS : DIET_KEYRELEASE;
            dfb_input_dispatch(my_pDfbDevice, &dfbEvent);
        }
    }
}       /* end of BDFB_bt_input_dataReady() */


/*
** DirectFB "standard' input driver interface routines.
*/

static int driver_get_available( void )
{
    /*
    ** Are there any registers or services we should check?
    ** Or should we simply assume that since the driver has been built, we are good to go?
    */
     return 1;
}

static void driver_get_info( InputDriverInfo *pDeviceInfo )
{
    /*
    ** These values are simply a guess.
    */

    snprintf( pDeviceInfo->name,
               DFB_INPUT_DRIVER_INFO_NAME_LENGTH,
               "BDFB Bluetooth HID driver" );

    snprintf( pDeviceInfo->vendor,
               DFB_INPUT_DRIVER_INFO_VENDOR_LENGTH,
               "Broadcom BSE" );

    pDeviceInfo->version.major = 1;
    pDeviceInfo->version.minor = 0;
}       /* end of driver_get_info() */

static DFBResult driver_open_device(
    CoreInputDevice *   pDfbDevice,
    unsigned int        uiDeviceId,
    InputDeviceInfo *   pDfbDeviceInfo,
    void **             pDriverData
    )
{
    int rc;

    BSTD_UNUSED(uiDeviceId);
    /*
    ** Allocate and initialize the device and "global" data.
    */
    icsmrNumBasicCodes = ( sizeof( csmrBasicCodeMap ) / sizeof ( KeyMapElement ) );
    icsmrNumAdvanCodes = ( sizeof( csmrAdvanCodeMap ) / sizeof ( KeyMapElement ) );
    iKbdNumBasicCodes = ( sizeof( kbdBasicCodeMap ) / sizeof ( DFBInputDeviceKeyIdentifier ) );
    iKbdNumAdvanCodes = ( sizeof( kbdAdvanCodeMap ) / sizeof ( DFBInputDeviceKeySymbol ) );

    my_pDfbDevice = pDfbDevice;

    /*
    ** Fill in the driver info with the supported features.
    */
    pDfbDeviceInfo->prefered_id = DIDID_KEYBOARD;
    pDfbDeviceInfo->desc.type   = DIDTF_KEYBOARD | DIDTF_REMOTE | DIDTF_MOUSE;
    pDfbDeviceInfo->desc.caps   = DICAPS_ALL;

    *pDriverData = NULL;

    /*
    ** Init Bluetooth HID host and register a callback for getting keycode.
    */
    if ((rc = BT_BSA_Init()) == 0)
    {
        BT_BSA_HhSettings hhSettings; 

        D_DEBUG_AT(bcmnexusBtInput, "%s: Bluetooth BSA initial...\n", __FUNCTION__);

        BT_BSA_HhGetSettings(&hhSettings);
        hhSettings.callback = BDFB_bt_input_dataReady;
        hhSettings.callbackContext = NULL; 
        BT_BSA_HhSetSettings(&hhSettings);
    }

    return DFB_OK;
}       /* end of driver_open_device() */

static DFBResult driver_get_keymap_entry(
    CoreInputDevice *         	pDfbDevice,
    void *						pDriverData,
    DFBInputDeviceKeymapEntry * pKeymapEntry
    )
{
    BSTD_UNUSED(pDfbDevice);
    BSTD_UNUSED(pDriverData);
    BSTD_UNUSED(pKeymapEntry);
    return DFB_UNSUPPORTED;
}       /* end of driver_get_keymap_entry() */

static void driver_close_device( void *pDriverData )
{
    BSTD_UNUSED(pDriverData);
    return;
}       /* end of driver_close_device() */

