/***************************************************************************
 *     (c)2007-2011 Broadcom Corporation
 *  
 *  This program is the proprietary software of Broadcom Corporation and/or its licensors,
 *  and may only be used, duplicated, modified or distributed pursuant to the terms and
 *  conditions of a separate, written license agreement executed between you and Broadcom
 *  (an "Authorized License").  Except as set forth in an Authorized License, Broadcom grants
 *  no license (express or implied), right to use, or waiver of any kind with respect to the
 *  Software, and Broadcom expressly reserves all rights in and to the Software and all
 *  intellectual property rights therein.  IF YOU HAVE NO AUTHORIZED LICENSE, THEN YOU
 *  HAVE NO RIGHT TO USE THIS SOFTWARE IN ANY WAY, AND SHOULD IMMEDIATELY
 *  NOTIFY BROADCOM AND DISCONTINUE ALL USE OF THE SOFTWARE.  
 *   
 *  Except as expressly set forth in the Authorized License,
 *   
 *  1.     This program, including its structure, sequence and organization, constitutes the valuable trade
 *  secrets of Broadcom, and you shall use all reasonable efforts to protect the confidentiality thereof,
 *  and to use this information only in connection with your use of Broadcom integrated circuit products.
 *   
 *  2.     TO THE MAXIMUM EXTENT PERMITTED BY LAW, THE SOFTWARE IS PROVIDED "AS IS" 
 *  AND WITH ALL FAULTS AND BROADCOM MAKES NO PROMISES, REPRESENTATIONS OR 
 *  WARRANTIES, EITHER EXPRESS, IMPLIED, STATUTORY, OR OTHERWISE, WITH RESPECT TO 
 *  THE SOFTWARE.  BROADCOM SPECIFICALLY DISCLAIMS ANY AND ALL IMPLIED WARRANTIES 
 *  OF TITLE, MERCHANTABILITY, NONINFRINGEMENT, FITNESS FOR A PARTICULAR PURPOSE, 
 *  LACK OF VIRUSES, ACCURACY OR COMPLETENESS, QUIET ENJOYMENT, QUIET POSSESSION 
 *  OR CORRESPONDENCE TO DESCRIPTION. YOU ASSUME THE ENTIRE RISK ARISING OUT OF 
 *  USE OR PERFORMANCE OF THE SOFTWARE.
 *  
 *  3.     TO THE MAXIMUM EXTENT PERMITTED BY LAW, IN NO EVENT SHALL BROADCOM OR ITS 
 *  LICENSORS BE LIABLE FOR (i) CONSEQUENTIAL, INCIDENTAL, SPECIAL, INDIRECT, OR 
 *  EXEMPLARY DAMAGES WHATSOEVER ARISING OUT OF OR IN ANY WAY RELATING TO YOUR 
 *  USE OF OR INABILITY TO USE THE SOFTWARE EVEN IF BROADCOM HAS BEEN ADVISED OF 
 *  THE POSSIBILITY OF SUCH DAMAGES; OR (ii) ANY AMOUNT IN EXCESS OF THE AMOUNT 
 *  ACTUALLY PAID FOR THE SOFTWARE ITSELF OR U.S. $1, WHICHEVER IS GREATER. THESE 
 *  LIMITATIONS SHALL APPLY NOTWITHSTANDING ANY FAILURE OF ESSENTIAL PURPOSE OF 
 *  ANY LIMITED REMEDY.
 * 
 * $brcm_Workfile: idirectfbimageprovider_nexus.c $
 * $brcm_Revision: DirectFB_1_4_Port/9 $
 * $brcm_Date: 3/17/11 12:43p $
 *
 * Module Description:
 *
 * Revision History:
 *
 * $brcm_Log: /AppLibs/opensource/directfb/src/broadcom_files/private/DirectFB/1.4/interfaces/IDirectFBImageProvider/idirectfbimageprovider_nexus.c $
 * 
 * DirectFB_1_4_Port/9   3/17/11 12:43p robertwm
 * SW7422-383: DirectFB: Need to serialise access to SID to prevent
 *  assertions.
 * SW7422-359: DirectFB: Add support for A8 Palette8 (DSPF_ALUT8) support
 *  into DirectFB.
 * SW7422-278: DirectFB: ImageProvider needs to be re-architected to use
 *  internal graphics APIs.
 * 
 * DirectFB_1_4_Port/8   2/18/11 2:57p robertwm
 * SW7422-278: DirectFB: ImageProvider needs to be re-architected to use
 *  internal graphics APIs
 * 
 * DirectFB_1_4_Port/7   7/26/10 5:32p jackf
 * SW35230-771: Added include for shim_nexus.h.
 * 
 * DirectFB_1_4_Port/6   7/22/10 4:27p robertwm
 * SW7405-4649: DirectFB: Optimise surface locking and wait idle syncing.
 * 
 * DirectFB_1_4_Port/5   7/13/10 12:06p jackf
 * SW35230-618: Changes needed to support JPEG (JFIF) format files
 *  embedded in Flash content.
 * 
 * DirectFB_1_4_Port/5   7/13/10 12:04p jackf
 * SW35230-618: Changes needed to support JPEG (JFIF) format files
 *  embedded in Flash content.
 * 
 * DirectFB_1_4_Port/4   5/20/10 12:49p robertwm
 * SW3556-1119: DirectFB: Nexus ImageProvider fails to build with
 *  DirectFB-1.4.1 Phase 2.0.
 * 
 * DirectFB_1_4_Port/3   4/19/10 2:11p robertwm
 * SW3556-1098: DirectFB: Nexus ImageProvider fails to build with
 *  DirectFB-1.4.1 Phase 2.0 PreRel.
 * SW3556-1099: DirectFB: SID should not be enabled if there is
 *  insufficient driver memory to initialise it.
 * 
 * DirectFB_1_4_Port/2   1/14/10 6:55p robertwm
 * SW3556-996: DirectFB: Implement ImageProvider for DirectFB-1.4.1 using
 *  SID.
 * 
 * DirectFB_1_4_Port/1   7/8/09 3:19p mphillip
 * PR56658: Add DirectFB 1.4.1 broadcom_files
 * 
 * 5   1/16/09 1:45p gmohile
 * PR 46490 : Free allocated surface after rendering
 * 
 * 4   11/21/08 8:06p vsilyaev
 * PR 49524: Added graphics2d_sync routine that could be used by different
 *  users of Nexus_Graphics2D
 * 
 * 3   10/24/08 12:42p gmohile
 * PR 46490 : Add hw picture decode support
 * 
 * 2   10/24/08 11:05a gmohile
 * PR 46490 : Add hw picture decode support
 *
 ***************************************************************************/

#include "shim_nexus.h" // Nexus function redirection

#include <config.h>

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <errno.h>
#include <stdarg.h>
#include <fcntl.h>

#include <directfb.h>

#include <display/idirectfbsurface.h>

#include <media/idirectfbimageprovider.h>

#include <core/coredefs.h>
#include <core/coretypes.h>

#include <core/core.h>
#include <core/layers.h>
#include <core/surface.h>
#include <core/system.h>
#include <core/palette.h>

#include <misc/conf.h>
#include <misc/util.h>
#include <misc/gfx_util.h>
#include <gfx/convert.h>
#include <gfx/util.h>

#include <direct/interface.h>
#include <direct/mem.h>
#include <direct/memcpy.h>

#include "bcmnexus.h"
#include "bcmnexus_utils.h"

D_DEBUG_DOMAIN( bcmnexusImageProvider,        "bcmNexus/ImageProvider",        "Broadcom NEXUS Image Provider module" );
D_DEBUG_DOMAIN( bcmnexusImageProviderPalette, "bcmNexus/ImageProviderPalette", "Broadcom NEXUS Image Provider Palette debug" );

static DFBResult
Probe( IDirectFBImageProvider_ProbeContext *ctx );

static DFBResult
Construct( IDirectFBImageProvider *thiz,
           ... );

#include <direct/interface_implementation.h>

DIRECT_INTERFACE_IMPLEMENTATION( IDirectFBImageProvider, NEXUS )

/*
 * private data struct of IDirectFBImageProvider_NEXUS
 */
typedef struct {
     int                         ref;      /* reference counter */
     FusionSkirmish              skirmish;
     IDirectFBDataBuffer        *buffer;
     DIRenderCallback            render_callback;
     void                       *render_callback_context;
     int                         width;
     int                         height;
     DFBSurfacePixelFormat       pixelformat;
     CoreDFB                    *core;     
     NEXUS_PictureDecoderHandle  pictureDecoder;
     NEXUS_PictureFormat         pictureFormat;
     NEXUS_PixelFormat           nexusPixelFormat;
     NEXUS_PictureDecoderPalette nexusPalette;

} IDirectFBImageProvider_NEXUS_data;

static DirectResult
IDirectFBImageProvider_NEXUS_AddRef  ( IDirectFBImageProvider *thiz );

static DirectResult
IDirectFBImageProvider_NEXUS_Release ( IDirectFBImageProvider *thiz );

static DFBResult
IDirectFBImageProvider_NEXUS_RenderTo( IDirectFBImageProvider *thiz,
                                       IDirectFBSurface       *destination,
                                       const DFBRectangle     *destination_rect );

static DFBResult
IDirectFBImageProvider_NEXUS_SetRenderCallback( IDirectFBImageProvider *thiz,
                                                DIRenderCallback        callback,
                                                void                   *context );

static DFBResult
IDirectFBImageProvider_NEXUS_WriteBack( IDirectFBImageProvider *thiz,
                                        IDirectFBSurface       *surface,
                                        const DFBRectangle     *src_rect,
                                        const char             *filename );
static DFBResult
IDirectFBImageProvider_NEXUS_GetSurfaceDescription( IDirectFBImageProvider *thiz,
                                                    DFBSurfaceDescription  *dsc );

static DFBResult
IDirectFBImageProvider_NEXUS_GetImageDescription( IDirectFBImageProvider *thiz,
                                                  DFBImageDescription    *dsc );


static void PictureDecoderStep(void *context, int unused)
{
    FusionSkirmish *mutex = context;
    unused = unused;

    D_DEBUG_AT(bcmnexusImageProvider, "%s()\n", __FUNCTION__);

    fusion_skirmish_prevail( mutex );
    fusion_skirmish_notify( mutex );
    fusion_skirmish_dismiss( mutex );
}

static DFBResult
Probe( IDirectFBImageProvider_ProbeContext *ctx )
{
    D_DEBUG_AT(bcmnexusImageProvider, "%s() sw_picture_decode=%d\n", __FUNCTION__, dfb_config->sw_picture_decode);

    if (dfb_config->sw_picture_decode == false) {
        if (ctx->filename)
            D_DEBUG_AT(bcmnexusImageProvider, "%s(): Filename=%s\n", __FUNCTION__, ctx->filename);

        /* Look of the Jpeg SOI marker */
        if (ctx->header[0] == 0xff && ctx->header[1] == 0xd8) {
            /* Look for JFIF or Exif strings, also could look at header[3:2] for APP0(0xFFE0),
            * APP1(0xFFE1) or even other APPx markers.
            */
            if (strncmp ((char*) ctx->header + 6, "JFIF", 4) == 0 ||
                strncmp ((char*) ctx->header + 6, "Exif", 4) == 0 ||
                strncmp ((char*) ctx->header + 6, "VVL", 3) == 0 ||
                strncmp ((char*) ctx->header + 6, "WANG", 4) == 0)
            {
                D_DEBUG_AT(bcmnexusImageProvider, "%s: JFIF/Exif format detected.\n", __FUNCTION__);
                return DFB_OK;
            }

            /* Else look for Quantization table marker or Define Huffman table marker,
            * useful for EXIF thumbnails that have no APPx markers.
            */
            if (ctx->header[2] == 0xff && (ctx->header[3] == 0xdb || ctx->header[3] == 0xc4))
            {
                D_DEBUG_AT(bcmnexusImageProvider, "%s: JFIF/Exif format detected.\n", __FUNCTION__);
                return DFB_OK;
            }

            if (ctx->filename && strchr (ctx->filename, '.' ) &&
                (strcasecmp ( strrchr (ctx->filename, '.' ), ".jpg" ) == 0 ||
                 strcasecmp ( strrchr (ctx->filename, '.' ), ".jpeg") == 0))
            {
                D_DEBUG_AT(bcmnexusImageProvider, "%s: JPEG format detected.\n", __FUNCTION__);
                return DFB_OK;
            }
        }     
        
        if(ctx->header[1] == 'P' && ctx->header[2] == 'N' && ctx->header[3] == 'G'){         
            if (ctx->filename && strrchr (ctx->filename, '.' ) &&
                (strcasecmp ( strrchr (ctx->filename, '.' ), ".png") == 0))
            {
                D_DEBUG_AT(bcmnexusImageProvider, "%s: PNG format detected.\n", __FUNCTION__);
                return DFB_OK;
            }
        }

        if (strncmp ((char*) ctx->header, "GIF8", 4) == 0) {
            D_DEBUG_AT(bcmnexusImageProvider, "%s: GIF format detected.\n", __FUNCTION__);
            return DFB_OK;
        }
    }

    return DFB_UNSUPPORTED;     
}

static DFBResult
Construct( IDirectFBImageProvider *thiz,
           ... )
{
     NEXUS_PictureDecoderStartSettings pictureSettings;
     NEXUS_PictureDecoderStatus        pictureStatus;
     DFBBCMNEXUS                      *pBrcmDfb = (DFBBCMNEXUS *) dfb_system_data();
     void                             *decode_buffer;
     size_t                            size;
     unsigned int                      nbytes = 0;    
     unsigned char                     header[32];
     IDirectFBDataBuffer              *buffer;
     CoreDFB                          *core;
     va_list                           tag;

     /* Get Picture Decoder global access lock */
     fusion_skirmish_prevail( &GET_PICTURE_DECODER_LOCK(pBrcmDfb) );

     DIRECT_ALLOCATE_INTERFACE_DATA(thiz, IDirectFBImageProvider_NEXUS)

     va_start( tag, thiz );
     buffer = va_arg( tag, IDirectFBDataBuffer * );
     core = va_arg( tag, CoreDFB * );
     va_end( tag );

     data->ref    = 1;
     data->buffer = buffer;
     data->core   = core;
     data->pictureDecoder = GET_PICTURE_DECODER_HND(pBrcmDfb);
     D_ASSERT(data->pictureDecoder != NULL);

     fusion_skirmish_init( &data->skirmish, "Broadcom Image Provider Mutex", dfb_core_world(NULL) );

     buffer->PeekData( buffer, 32, 0, header, &nbytes );
     if(header[1] == 'P' && header[2] == 'N' && header[3] == 'G'){
         data->pictureFormat = NEXUS_PictureFormat_ePng;
     }
     else if(header[0] == 0xff && header[1] == 0xd8){
         data->pictureFormat = NEXUS_PictureFormat_eJpeg;
     }
     else if (strncmp ((char *)header, "GIF8", 4) == 0) {
         data->pictureFormat = NEXUS_PictureFormat_eGif;
     }
     else {
        D_DEBUG_AT(bcmnexusImageProvider, "%s: Unsupported format.\n", __FUNCTION__);
        goto err_format;
     }
       
     
     buffer->AddRef( buffer );
         
     NEXUS_PictureDecoder_GetBuffer(data->pictureDecoder, &decode_buffer, &size);
     buffer->PeekData( buffer, size, 0, decode_buffer, &nbytes );
     NEXUS_PictureDecoder_ReadComplete(data->pictureDecoder, 0, nbytes);
     NEXUS_PictureDecoder_GetDefaultStartSettings(&pictureSettings);
     pictureSettings.format = data->pictureFormat;
     pictureSettings.pictureParsed.callback = PictureDecoderStep;
     pictureSettings.pictureParsed.context = &data->skirmish;

     fusion_skirmish_prevail( &data->skirmish);

     if (NEXUS_PictureDecoder_Start(data->pictureDecoder, &pictureSettings) != NEXUS_SUCCESS)
     {
          D_WARN("SID: Start() call failed!\n");
          goto err_start;
     }

     /* Wait up to 1s for image header */
     fusion_skirmish_wait( &data->skirmish, 1000 );
     fusion_skirmish_dismiss( &data->skirmish );

     NEXUS_PictureDecoder_GetStatus(data->pictureDecoder, &pictureStatus);
     if (pictureStatus.state == NEXUS_PictureDecoderState_eError) {
         D_WARN("SID: Decoding Failed\n");
         goto err_decode;
     } else if (!pictureStatus.headerValid) {
         D_WARN("SID: Could not decode image header\n");
         goto err_decode;
     }

     data->width  = pictureStatus.header.surface.width;
     data->height = pictureStatus.header.surface.height;
     data->nexusPixelFormat = pictureStatus.header.format;
     data->pixelformat = bcmnexus_getDFBSurfacePixelFormat(pictureStatus.header.format);

     /* If indexed colour, then load palette */
     if (pictureStatus.header.palette)
     {
        D_DEBUG_AT(bcmnexusImageProvider, "%s: Palette detected.\n", __FUNCTION__);
        if (NEXUS_PictureDecoder_GetPalette(data->pictureDecoder, &data->nexusPalette) != NEXUS_SUCCESS)
        {
             D_WARN("SID: Failed to get Palette\n");
             goto err_decode;
        }
     }

     NEXUS_PictureDecoder_Stop(data->pictureDecoder);

     D_DEBUG_AT(bcmnexusImageProvider, "%s: Nexus Picture format=%d, Nexus pixel format=%d, pixelformat=%s, size=%dx%d multiscan=%d\n", __FUNCTION__,
                data->pictureFormat, data->nexusPixelFormat, dfb_pixelformat_name(data->pixelformat),
                data->width, data->height, pictureStatus.header.multiscan);

     thiz->AddRef                = IDirectFBImageProvider_NEXUS_AddRef;
     thiz->Release               = IDirectFBImageProvider_NEXUS_Release;
     thiz->RenderTo              = IDirectFBImageProvider_NEXUS_RenderTo;
     thiz->SetRenderCallback     = IDirectFBImageProvider_NEXUS_SetRenderCallback;
     thiz->GetImageDescription   = IDirectFBImageProvider_NEXUS_GetImageDescription;
     thiz->GetSurfaceDescription = IDirectFBImageProvider_NEXUS_GetSurfaceDescription;
     thiz->WriteBack             = IDirectFBImageProvider_NEXUS_WriteBack;

     return DFB_OK;

err_decode:
     NEXUS_PictureDecoder_Stop(data->pictureDecoder);
err_start:
     buffer->Release( buffer );
err_format:
     fusion_skirmish_destroy( &data->skirmish );
     DIRECT_DEALLOCATE_INTERFACE( thiz );
     return DFB_FAILURE;
}

static void
IDirectFBImageProvider_NEXUS_Destruct( IDirectFBImageProvider *thiz )
{
     DFBBCMNEXUS                       *pBrcmDfb = (DFBBCMNEXUS *) dfb_system_data();
     IDirectFBImageProvider_NEXUS_data *data     =
                              (IDirectFBImageProvider_NEXUS_data*)thiz->priv;

     D_DEBUG_AT(bcmnexusImageProvider, "%s()\n", __FUNCTION__);

     data->buffer->Release( data->buffer );

     fusion_skirmish_destroy( &data->skirmish );
     DIRECT_DEALLOCATE_INTERFACE( thiz );

     /* Release Picture Decoder global access lock */
     fusion_skirmish_dismiss( &GET_PICTURE_DECODER_LOCK(pBrcmDfb) );
}

static DirectResult
IDirectFBImageProvider_NEXUS_AddRef( IDirectFBImageProvider *thiz )
{
     DIRECT_INTERFACE_GET_DATA(IDirectFBImageProvider_NEXUS)

     D_DEBUG_AT(bcmnexusImageProvider, "%s()\n", __FUNCTION__);

     data->ref++;

     return DFB_OK;
}

static DirectResult
IDirectFBImageProvider_NEXUS_Release( IDirectFBImageProvider *thiz )
{
     DIRECT_INTERFACE_GET_DATA(IDirectFBImageProvider_NEXUS)

     D_DEBUG_AT(bcmnexusImageProvider, "%s()\n", __FUNCTION__);

     if (--data->ref == 0) {
          IDirectFBImageProvider_NEXUS_Destruct( thiz );
     }

     return DFB_OK;
}


static DFBResult
IDirectFBImageProvider_NEXUS_RenderTo( IDirectFBImageProvider *thiz,
                                       IDirectFBSurface       *destination,
                                       const DFBRectangle     *dest_rect )
{
    DFBResult                         ret     = DFB_OK;
    IDirectFBSurface_data            *dst_data;
    CoreSurface                      *src_surface;
    CoreSurface                      *dst_surface;
    CorePalette                      *palette = NULL;
    CoreSurfaceBufferLock             src_surface_lock;
    DFBRegion                         clip;
    DFBRectangle                      rect;
    NEXUS_Rect                        src_rect;
    NEXUS_PictureDecoderStartSettings pictureSettings;
    NEXUS_PictureDecoderStatus        pictureStatus;
    void                             *decode_buffer;
    size_t                            size;
    unsigned int                      nbytes = 0; 
    int                               numEntries;
    int                               index;

    DIRECT_INTERFACE_GET_DATA(IDirectFBImageProvider_NEXUS)

    D_DEBUG_AT(bcmnexusImageProvider, "%s()\n", __FUNCTION__);

    dst_data = (IDirectFBSurface_data*) destination->priv;
    if (!dst_data)
          return DFB_DEAD;

    dst_surface = dst_data->surface;
    if (!dst_surface)
          return DFB_DESTROYED;

    dfb_region_from_rectangle( &clip, &dst_data->area.current );

    if (dest_rect) {
          if (dest_rect->w < 1 || dest_rect->h < 1)
               return DFB_INVARG;
          rect = *dest_rect; 
          rect.x += dst_data->area.wanted.x;
          rect.y += dst_data->area.wanted.y;
    }
    else {
          rect = dst_data->area.wanted;
    }

    /* actual loading and rendering */
    if (dfb_rectangle_region_intersects( &rect, &clip )) {
          if (DFB_PIXELFORMAT_IS_INDEXED(data->pixelformat))
          {
              numEntries = data->nexusPalette.numPaletteEntries;
              D_DEBUG_AT(bcmnexusImageProvider, "%s(): Num Palette Entries=%d\n", __FUNCTION__, numEntries);

              /* Sanity check! */
              D_ASSERT( numEntries <= D_ARRAY_SIZE(data->nexusPalette.palette));

              ret = dfb_palette_create( data->core, numEntries, &palette );
              if (ret)
              {
                   D_ERROR("bcmNexus/ImageProvider: Cannot create palette of %d entries!\n", numEntries);
                   goto err_palette;
              }

              /* Convert from ARGB8888 NEXUS pixel format to DFB format */
              for (index = 0; index < numEntries; index++)
              {
                   dfb_pixel_to_color(DSPF_ARGB, data->nexusPalette.palette[index], &palette->entries[index]);
                   D_DEBUG_AT(bcmnexusImageProviderPalette, "%s(): Palette [%d] = 0x%08x\n", __FUNCTION__, index, data->nexusPalette.palette[index]);
              }

              dfb_palette_update( palette, 0, numEntries - 1 );
          }

          D_DEBUG_AT(bcmnexusImageProvider, "%s(): creating surface %dx%d format=%s...\n", __FUNCTION__,
                     data->width, data->height, dfb_pixelformat_name(data->pixelformat));

          /* Create a surface for the picture decoder to decode into (call it src surface) */
          ret = dfb_surface_create_simple( data->core,
                                           data->width,
                                           data->height,
                                           data->pixelformat,
                                           DSCAPS_VIDEOONLY,
                                           CSTF_SHARED, 0, palette, &src_surface );

          if (ret)
          {
              D_ERROR("bcmNexus/ImageProvider: Cannot create surface %dx%d format=%s!\n",
                      data->width, data->height, dfb_pixelformat_name(data->pixelformat));
              goto err_buffer;
          }

          if (NEXUS_PictureDecoder_GetBuffer(data->pictureDecoder, &decode_buffer, &size) != NEXUS_SUCCESS)
          {
              D_ERROR("bcmNexus/ImageProvider: NEXUS_PictureDecoder_GetBuffer() call failed!\n");
              ret = DFB_FAILURE;
              goto err_surface;
          }

          data->buffer->PeekData( data->buffer, size, 0, decode_buffer, &nbytes );
          
          if (NEXUS_PictureDecoder_ReadComplete(data->pictureDecoder, 0, nbytes) != NEXUS_SUCCESS)
          {
              D_ERROR("bcmNexus/ImageProvider: NEXUS_PictureDecoder_ReadComplete() call failed!\n");
              ret = DFB_FAILURE;
              goto err_surface;
          }

          NEXUS_PictureDecoder_GetDefaultStartSettings(&pictureSettings);
          pictureSettings.format = data->pictureFormat;
          pictureSettings.segmentDecoded.callback = PictureDecoderStep;
          pictureSettings.segmentDecoded.context = &data->skirmish;

          if (NEXUS_PictureDecoder_Start(data->pictureDecoder, &pictureSettings) != NEXUS_SUCCESS)
          {
              D_ERROR("bcmNexus/ImageProvider: NEXUS_PictureDecoder_Start() call failed!\n");
              ret = DFB_FAILURE;
              goto err_surface;
          }
          
          /* Actually instantiate the buffer */
          ret = dfb_surface_lock_buffer( src_surface, CSBR_BACK, CSAID_CPU, CSAF_READ|CSAF_WRITE, &src_surface_lock );
          if (ret)
          {
              D_ERROR( "bcmNexus/ImageProvider: Could NOT lock source surface! (res=%d)\n", ret);
              goto err_surface;
          }

          fusion_skirmish_prevail( &data->skirmish);

          src_rect.x      = 0;
          src_rect.y      = 0;
          src_rect.width  = data->width;
          src_rect.height = data->height;

          if (NEXUS_PictureDecoder_DecodeSegment(data->pictureDecoder, src_surface_lock.handle, &src_rect) != NEXUS_SUCCESS)
          {
              D_ERROR("bcmNexus/ImageProvider: NEXUS_PictureDecoder_DecodeSegment() call failed!\n");
              fusion_skirmish_dismiss( &data->skirmish );
              dfb_surface_unlock_buffer( src_surface, &src_surface_lock );
              ret = DFB_FAILURE;
              goto err_segment;
          }

          /* Wait up to 2s for image segment decoded */
          fusion_skirmish_wait( &data->skirmish, 2000 );
          fusion_skirmish_dismiss( &data->skirmish );
        
          dfb_surface_unlock_buffer( src_surface, &src_surface_lock );

          NEXUS_PictureDecoder_GetStatus(data->pictureDecoder, &pictureStatus);
          if (pictureStatus.state != NEXUS_PictureDecoderState_eSegmentDone) {
             D_ERROR("bcmNexus/ImageProvider: Picture Decoding Failed!\n");
             ret = DFB_FAILURE;
             goto err_segment;
          }

          dfb_gfx_stretch_to( src_surface, dst_surface, &rect, false);
          dfb_gfxcard_sync();
        
          if (data->render_callback) {
               DIRenderCallbackResult r;

               rect.x = 0;
               rect.y = 0;
               rect.w = pictureStatus.header.width;
               rect.h = pictureStatus.header.height;

               r = data->render_callback( &rect, data->render_callback_context );

               if (r != DIRCR_OK)
                   ret = DFB_INTERRUPTED;
          }

err_segment:
          NEXUS_PictureDecoder_Stop(data->pictureDecoder);
err_surface:
          dfb_surface_unref(src_surface);
err_buffer:
          if (palette) {
              dfb_palette_unref( palette );
          }
    }
err_palette:
    D_DEBUG_AT(bcmnexusImageProvider, "%s() Exit [%d]\n", __FUNCTION__, ret);
    return ret;
}


static DFBResult
IDirectFBImageProvider_NEXUS_WriteBack( IDirectFBImageProvider *thiz,
                                        IDirectFBSurface       *surface,
                                        const DFBRectangle     *src_rect,
                                        const char             *filename )
{
     DFBResult                   ret = DFB_OK;
     int                         fd;
     ssize_t                     cnt;
     IDirectFBSurface_data      *sur_data;
     CoreSurface                *core_surface;
     CoreSurfaceBufferLock       lock;
     NEXUS_SurfaceHandle         handle;
     NEXUS_SurfaceMemory         mem;
     NEXUS_SurfaceCreateSettings createSettings;

     BSTD_UNUSED(src_rect);

     DIRECT_INTERFACE_GET_DATA (IDirectFBImageProvider_NEXUS)

     if (filename) {
          sur_data = (IDirectFBSurface_data*) surface->priv;
          if (!sur_data)
              return DFB_DEAD;

          core_surface = sur_data->surface;
          if (!core_surface)
              return DFB_DESTROYED;

          /* First check to see if the file already exists.  If it does
             then just overwrite it. */
          fd = open( filename, O_RDWR);
          if (fd < 0) {
              fd = open( filename,  O_EXCL | O_WRONLY | O_CREAT, 0644 );

              if (fd < 0) {
                   D_PERROR("%s: could not open %s for writing!\n", __FUNCTION__, filename);
                   return DFB_IO;
              }
          }

          ret = dfb_surface_lock_buffer( core_surface, CSBR_BACK, CSAID_CPU, CSAF_READ, &lock );
          if (ret)
          {
               D_WARN("Could not lock surface buffer %p!\n", (void *)core_surface);
               close(fd);
               return ret;
          }

          handle = lock.handle;
          NEXUS_Surface_GetMemory(handle, &mem);
          NEXUS_Surface_GetCreateSettings(handle, &createSettings);
          cnt = write(fd, mem.buffer, mem.pitch*createSettings.height);
          if (cnt < 0) {
              D_WARN("Could not write to %s (err=%s)!\n", filename, strerror(errno));
              ret = DFB_IO;
          } else {
            D_INFO("Wrote %d bytes of surface %dx%d (Nexus pixelformat=%d) to \"%s\".\n", cnt, createSettings.width,
                    createSettings.height, createSettings.pixelFormat, filename);
          }

          dfb_surface_unlock_buffer( core_surface, &lock );
          close(fd);
     }
     return ret;
}

static DFBResult
IDirectFBImageProvider_NEXUS_SetRenderCallback( IDirectFBImageProvider *thiz,
                                                DIRenderCallback        callback,
                                                void                   *context )
{
     DIRECT_INTERFACE_GET_DATA (IDirectFBImageProvider_NEXUS)

     D_DEBUG_AT(bcmnexusImageProvider, "%s()\n", __FUNCTION__);
     
     data->render_callback         = callback;
     data->render_callback_context = context;

     return DFB_OK;
}

static DFBResult
IDirectFBImageProvider_NEXUS_GetSurfaceDescription( IDirectFBImageProvider *thiz,
                                                    DFBSurfaceDescription  *dsc )
{
     DIRECT_INTERFACE_GET_DATA(IDirectFBImageProvider_NEXUS)
    
     dsc->flags       = DSDESC_WIDTH |  DSDESC_HEIGHT | DSDESC_PIXELFORMAT | DSDESC_CAPS;
     dsc->caps        = DSCAPS_VIDEOONLY;  
     dsc->height      = data->height;
     dsc->width       = data->width;
     dsc->pixelformat = dfb_primary_layer_pixelformat();

     D_DEBUG_AT(bcmnexusImageProvider, "%s() %dx%d format=%s\n", __FUNCTION__, dsc->width, dsc->height, dfb_pixelformat_name(dsc->pixelformat));

     return DFB_OK;
}

static DFBResult
IDirectFBImageProvider_NEXUS_GetImageDescription( IDirectFBImageProvider *thiz,
                                                  DFBImageDescription    *dsc )
{
     DIRECT_INTERFACE_GET_DATA(IDirectFBImageProvider_NEXUS)

     D_DEBUG_AT(bcmnexusImageProvider, "%s()\n", __FUNCTION__);

     if (!dsc)
          return DFB_INVARG;

     dsc->caps = DICAPS_NONE;

     return DFB_OK;
}

