/******************************************************************************
 *    (c)2011 Broadcom Corporation
 * 
 * This program is the proprietary software of Broadcom Corporation and/or its licensors,
 * and may only be used, duplicated, modified or distributed pursuant to the terms and
 * conditions of a separate, written license agreement executed between you and Broadcom
 * (an "Authorized License").  Except as set forth in an Authorized License, Broadcom grants
 * no license (express or implied), right to use, or waiver of any kind with respect to the
 * Software, and Broadcom expressly reserves all rights in and to the Software and all
 * intellectual property rights therein.  IF YOU HAVE NO AUTHORIZED LICENSE, THEN YOU
 * HAVE NO RIGHT TO USE THIS SOFTWARE IN ANY WAY, AND SHOULD IMMEDIATELY
 * NOTIFY BROADCOM AND DISCONTINUE ALL USE OF THE SOFTWARE.  
 *  
 * Except as expressly set forth in the Authorized License,
 *  
 * 1.     This program, including its structure, sequence and organization, constitutes the valuable trade
 * secrets of Broadcom, and you shall use all reasonable efforts to protect the confidentiality thereof,
 * and to use this information only in connection with your use of Broadcom integrated circuit products.
 *  
 * 2.     TO THE MAXIMUM EXTENT PERMITTED BY LAW, THE SOFTWARE IS PROVIDED "AS IS" 
 * AND WITH ALL FAULTS AND BROADCOM MAKES NO PROMISES, REPRESENTATIONS OR 
 * WARRANTIES, EITHER EXPRESS, IMPLIED, STATUTORY, OR OTHERWISE, WITH RESPECT TO 
 * THE SOFTWARE.  BROADCOM SPECIFICALLY DISCLAIMS ANY AND ALL IMPLIED WARRANTIES 
 * OF TITLE, MERCHANTABILITY, NONINFRINGEMENT, FITNESS FOR A PARTICULAR PURPOSE, 
 * LACK OF VIRUSES, ACCURACY OR COMPLETENESS, QUIET ENJOYMENT, QUIET POSSESSION 
 * OR CORRESPONDENCE TO DESCRIPTION. YOU ASSUME THE ENTIRE RISK ARISING OUT OF 
 * USE OR PERFORMANCE OF THE SOFTWARE.
 * 
 * 3.     TO THE MAXIMUM EXTENT PERMITTED BY LAW, IN NO EVENT SHALL BROADCOM OR ITS 
 * LICENSORS BE LIABLE FOR (i) CONSEQUENTIAL, INCIDENTAL, SPECIAL, INDIRECT, OR 
 * EXEMPLARY DAMAGES WHATSOEVER ARISING OUT OF OR IN ANY WAY RELATING TO YOUR 
 * USE OF OR INABILITY TO USE THE SOFTWARE EVEN IF BROADCOM HAS BEEN ADVISED OF 
 * THE POSSIBILITY OF SUCH DAMAGES; OR (ii) ANY AMOUNT IN EXCESS OF THE AMOUNT 
 * ACTUALLY PAID FOR THE SOFTWARE ITSELF OR U.S. $1, WHICHEVER IS GREATER. THESE 
 * LIMITATIONS SHALL APPLY NOTWITHSTANDING ANY FAILURE OF ESSENTIAL PURPOSE OF 
 * ANY LIMITED REMEDY.
 *
 * $brcm_Workfile: dtv_init.c $
 * $brcm_Revision: DirectFB_1_4_Port/6 $
 * $brcm_Date: 3/15/11 2:14p $
 * 
 * Module Description:
 * 
 * Revision History:
 * 
 * $brcm_Log: /AppLibs/opensource/directfb/src/broadcom_files/private/DirectFB/1.4/libinit/dtv_init.c $
 * 
 * DirectFB_1_4_Port/6   3/15/11 2:14p robertwm
 * SW3556-1225: DirectFB: DirectFB-1.4.1 Phase 3.0 PreRel fails to build.
 * 
 * DirectFB_1_4_Port/5   3/8/11 6:37p apandav
 * SW35125-124: DirectFB Build fails on BCM35125
 * 
 * DirectFB_1_4_Port/4   2/4/11 8:40p robertwm
 * SW7400-3000: DirectFB: Support non-DFB master application in
 *  platform_init code.
 * SW7400-3002: DirectFB: Support non-DFB master application in
 *  platform_init code.
 * 
 * DirectFB_1_4_Port/3   2/4/11 4:18p robertwm
 * SW35230-2943: DirectFB: Cannot build platform_init code for 35230
 *  platform.
 * 
 * DirectFB_1_4_Port/2   2/3/11 6:01p christ
 * SW7420-1458: DirectFB: Rework internal handling of nexus heaps.
 * 
 * DirectFB_1_4_Port/1   1/25/11 8:07p robertwm
 * SW7420-1424: DirectFB: Refactor platform_init code to separate DTV and
 *  STB parts and to support NEXUS heap m'ment.
 * 
 * DirectFB_1_4_Port/DirectFB_STEREO_3D_JIRA_SW7422_138/2   1/18/11 1:26p christ
 * Fix DTV compile problems.
 * 
 * DirectFB_1_4_Port/DirectFB_STEREO_3D_JIRA_SW7422_138/1   1/17/11 12:34p christ
 * DTV specific platform init code.
 * 
 *****************************************************************************/

#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/mman.h>
#include <fcntl.h>
#include <stdio.h>

#include "platform_init.h"

#ifdef NEXUS_DTV_PLATFORM

#ifdef BCMNEXUS_PANEL_OUTPUT_INIT_SETTINGS_HAS_LVDS

#define PANEL_OUTPUT_SETTINGS(s)                platformSettings->displayModuleSettings.panel.lvds.s
#define PANEL_OUTPUT_SETTINGS_LVDS_COLOR_MODE() platformSettings->displayModuleSettings.panel.lvds.lvdsColorMode
#define PANEL_OUTPUT_SETTINGS_LINK(i, s)        platformSettings->displayModuleSettings.panel.lvds.linkChannel[i].s

#else

#define PANEL_OUTPUT_SETTINGS(s)                platformSettings->displayModuleSettings.panel.s
#define PANEL_OUTPUT_SETTINGS_LVDS_COLOR_MODE() platformSettings->displayModuleSettings.panel.customLinkSettings.lvdsColorMode
#define PANEL_OUTPUT_SETTINGS_LINK(i, s)        platformSettings->displayModuleSettings.panel.linkSettings[i].s

#endif

NEXUS_HeapHandle  nexus_display_heap;
NEXUS_HeapHandle  nexus_offscreen_heap;
NEXUS_HeapHandle  nexus_palette_heap;
NEXUS_HeapHandle  nexus_video_heap;

NEXUS_VideoFormat nexus_video_format_preferred;
#ifdef NEXUS_HAS_PICTURE_DECODER
NEXUS_PictureDecoderHandle nexus_picturedecoder_handle;
#endif
NEXUS_DisplayHandle nexus_display_handle0;
#if (NUM_DISPLAYS > 1)
NEXUS_DisplayHandle nexus_display_handle1;
#endif
#if NEXUS_HAS_GRAPHICS2D || NEXUS_HAS_ZSP_GRAPHICS
NEXUS_Graphics2DHandle nexus_graphics2d_handle;
#endif

#if NEXUS_HAS_GRAPHICS3D && defined(BCMNEXUS_GFX_HAS_P3D)
NEXUS_Graphics3DHandle nexus_graphics3d_handle;
#endif


int configurePlatformDisplay(NEXUS_DisplayHandle        *display0,
                             NEXUS_DisplayHandle        *display1,
                             NEXUS_DisplaySettings       displayCfg,
                             NEXUS_Shared_Handles       *sharedHandles,
                             NEXUS_PlatformConfiguration platformConfig)
{
    NEXUS_Error rc;

    (void)display1; /* Not used presently (mostly due to RTS issues with DTV platforms) */

    displayCfg.format       = NEXUS_VideoFormat_e1080p;    
    displayCfg.xvYccEnabled = false;    
    displayCfg.background   = 0xFF0000FF;
    
    /* Should we open up display0 module? */
    if (*display0 == NULL)
    {
        *display0 = NEXUS_Display_Open(0, &displayCfg);
        if(!display0) {rc=BERR_TRACE(NEXUS_NOT_SUPPORTED); return -1;}
        BDBG_MSG(("Using panel output"));
        rc = NEXUS_Display_AddOutput(*display0, NEXUS_PanelOutput_GetConnector(platformConfig.outputs.panel[0]));
        if(rc!=NEXUS_SUCCESS) {rc=BERR_TRACE(rc);return -1;}
        rc = NEXUS_BoardCfg_ConfigurePanel(true, true, true);       
        if(rc!=NEXUS_SUCCESS) {rc=BERR_TRACE(rc); return -1;}
    }

    SET_NEXUS_DISPLAY_HND(0, *display0); 
    sharedHandles->disp_handle0 = *display0;

    return 0;
}


void setupDTVPanel(NEXUS_PlatformSettings *platformSettings)
{
    /* Any panel type specified? */
    if (getenv("panel_type") != NULL)
    {
        if (strcmp("JEIDA", getenv("panel_type")) == 0)
        {
            PANEL_OUTPUT_SETTINGS_LVDS_COLOR_MODE()                = NEXUS_LvdsColorMode_e10Bit;
            PANEL_OUTPUT_SETTINGS(lvdsMappingRule)                 = NEXUS_PanelOutputMappingRule_eJeidaRule;
            PANEL_OUTPUT_SETTINGS(customLinkSettings.lvdsLinkMap1) = NEXUS_LvdsLinkMap_eInternalLink1;
            PANEL_OUTPUT_SETTINGS(customLinkSettings.lvdsLinkMap2) = NEXUS_LvdsLinkMap_eInternalLink2;
            PANEL_OUTPUT_SETTINGS(dvoLinkMode)                     = NEXUS_PanelOutputLinkMode_eCustom;
        }
        else if (strcmp("CMO", getenv("panel_type")) == 0)
        {
            /*TBD*/
        }
        else if (strcmp("AUO", getenv("panel_type")) == 0)
        {
            PANEL_OUTPUT_SETTINGS(dvoLinkMode)                     = NEXUS_PanelOutputLinkMode_eSingleChannel1;
        }
        else if (strcmp("B552", getenv("panel_type")) == 0)
        {
#if (BCHP_CHIP != 35230)
            PANEL_OUTPUT_SETTINGS(lvdsMappingRule)                 = NEXUS_PanelOutputMappingRule_eOpenLdiRule;
            PANEL_OUTPUT_SETTINGS(customLinkSettings.lvdsLinkMap1) = NEXUS_LvdsLinkMap_eInternalLink1;
            PANEL_OUTPUT_SETTINGS(customLinkSettings.lvdsLinkMap2) = NEXUS_LvdsLinkMap_eInternalLink2;

            PANEL_OUTPUT_SETTINGS_LINK(0, maskLinkHSync)           = 0;
            PANEL_OUTPUT_SETTINGS_LINK(0, maskLinkVSync)           = 0;
            PANEL_OUTPUT_SETTINGS_LINK(0, maskLinkDE)              = 0;

            PANEL_OUTPUT_SETTINGS_LINK(1, maskLinkHSync)           = 0;
            PANEL_OUTPUT_SETTINGS_LINK(1, maskLinkVSync)           = 0;
            PANEL_OUTPUT_SETTINGS_LINK(1, maskLinkDE)              = 0;
            
#else
            PANEL_OUTPUT_SETTINGS(dvoLinkMode)                     = NEXUS_PanelOutputLinkMode_eDualChannel1;                    
            PANEL_OUTPUT_SETTINGS(lvdsColorMode)                   = NEXUS_LvdsColorMode_e8Bit ;
            PANEL_OUTPUT_SETTINGS(useNegativeEdge)                 = false ;
#endif /* (BCHP_CHIP != 35230) */

            if (getenv("pixel_swap") != NULL && strcasecmp("yes", getenv("pixel_swap")) == 0)
            {
                PANEL_OUTPUT_SETTINGS(dvoLinkMode)                 = NEXUS_PanelOutputLinkMode_eCustom;
            }
        }
        else if (strcmp("OPENLDI", getenv("panel_type")) == 0)
        {
#ifdef BCMNEXUS_PANEL_OUTPUT_HAS_LINK_MODE_QUAD
            PANEL_OUTPUT_SETTINGS(dvoLinkMode)                     = NEXUS_PanelOutputLinkMode_eQuad;
#endif
        }
    }
}

void setupNexusHeaps( NEXUS_PlatformSettings *platformSettings, 
                      unsigned char          *nexusHeapIndex)
{
    /* The DTV heap API differs from settop. */
    platformSettings->memc[0].heapSize[0] = -1; /* remainder */
    platformSettings->memc[0].heapSize[1] = get_gfx_heap_size();

    nexusHeapIndex[0] = nexusHeapIndex[1] = 1;
}

#else /* NEXUS_DTV_PLATFORM */

// stupid but stops a compiler warning
void nothing(void);

#endif /* NEXUS_DTV_PLATFORM */
