/******************************************************************************
 *    (c)2010-2011 Broadcom Corporation
 *
 * This program is the proprietary software of Broadcom Corporation and/or its licensors,
 * and may only be used, duplicated, modified or distributed pursuant to the terms and
 * conditions of a separate, written license agreement executed between you and Broadcom
 * (an "Authorized License").  Except as set forth in an Authorized License, Broadcom grants
 * no license (express or implied), right to use, or waiver of any kind with respect to the
 * Software, and Broadcom expressly reserves all rights in and to the Software and all
 * intellectual property rights therein.  IF YOU HAVE NO AUTHORIZED LICENSE, THEN YOU
 * HAVE NO RIGHT TO USE THIS SOFTWARE IN ANY WAY, AND SHOULD IMMEDIATELY
 * NOTIFY BROADCOM AND DISCONTINUE ALL USE OF THE SOFTWARE.
 *
 * Except as expressly set forth in the Authorized License,
 *
 * 1.     This program, including its structure, sequence and organization, constitutes the valuable trade
 * secrets of Broadcom, and you shall use all reasonable efforts to protect the confidentiality thereof,
 * and to use this information only in connection with your use of Broadcom integrated circuit products.
 *
 * 2.     TO THE MAXIMUM EXTENT PERMITTED BY LAW, THE SOFTWARE IS PROVIDED "AS IS"
 * AND WITH ALL FAULTS AND BROADCOM MAKES NO PROMISES, REPRESENTATIONS OR
 * WARRANTIES, EITHER EXPRESS, IMPLIED, STATUTORY, OR OTHERWISE, WITH RESPECT TO
 * THE SOFTWARE.  BROADCOM SPECIFICALLY DISCLAIMS ANY AND ALL IMPLIED WARRANTIES
 * OF TITLE, MERCHANTABILITY, NONINFRINGEMENT, FITNESS FOR A PARTICULAR PURPOSE,
 * LACK OF VIRUSES, ACCURACY OR COMPLETENESS, QUIET ENJOYMENT, QUIET POSSESSION
 * OR CORRESPONDENCE TO DESCRIPTION. YOU ASSUME THE ENTIRE RISK ARISING OUT OF
 * USE OR PERFORMANCE OF THE SOFTWARE.
 *
 * 3.     TO THE MAXIMUM EXTENT PERMITTED BY LAW, IN NO EVENT SHALL BROADCOM OR ITS
 * LICENSORS BE LIABLE FOR (i) CONSEQUENTIAL, INCIDENTAL, SPECIAL, INDIRECT, OR
 * EXEMPLARY DAMAGES WHATSOEVER ARISING OUT OF OR IN ANY WAY RELATING TO YOUR
 * USE OF OR INABILITY TO USE THE SOFTWARE EVEN IF BROADCOM HAS BEEN ADVISED OF
 * THE POSSIBILITY OF SUCH DAMAGES; OR (ii) ANY AMOUNT IN EXCESS OF THE AMOUNT
 * ACTUALLY PAID FOR THE SOFTWARE ITSELF OR U.S. $1, WHICHEVER IS GREATER. THESE
 * LIMITATIONS SHALL APPLY NOTWITHSTANDING ANY FAILURE OF ESSENTIAL PURPOSE OF
 * ANY LIMITED REMEDY.
 *
 * $brcm_Workfile: stb_init.c $
 * $brcm_Revision: DirectFB_1_4_Port/9 $
 * $brcm_Date: 3/11/11 5:15p $
 *
 * Module Description:
 *
 * Revision History:
 *
 * $brcm_Log: /AppLibs/opensource/directfb/src/broadcom_files/private/DirectFB/1.4.1/libinit/stb_init.c $
 * 
 * DirectFB_1_4_Port/9   3/11/11 5:15p christ
 * SW7468-370: DirectFB: 1.4.1 release fails to compile for platforms
 *  without component output.
 * 
 * DirectFB_1_4_Port/7   3/7/11 12:02p robertwm
 * SW7405-5178: DirectFB: Cannot run DFB with 7405 Phase 7.5 release
 *  because of NEXUS_P_AddMap failing.
 * 
 * DirectFB_1_4_Port/6   3/5/11 12:46a robertwm
 * SW7405-5178: DirectFB: Cannot run DFB with 7405 Phase 7.5 release
 *  because of NEXUS_P_AddMap failing.
 * 
 * DirectFB_1_4_Port/5   3/4/11 12:24p christ
 * SW7405-5175: DirectFB: Platform init code for dual display STB looks
 *  broken.
 *
 * DirectFB_1_4_Port/4   2/18/11 2:37p robertwm
 * SW7468-366: DirectFB: Platforms without component output should not
 *  call component functions.
 *
 * DirectFB_1_4_Port/3   2/4/11 8:39p robertwm
 * SW7400-3000: DirectFB: Support non-DFB master application in
 *  platform_init code.
 * SW7400-3002: DirectFB: Support non-DFB master application in
 *  platform_init code.
 *
 * DirectFB_1_4_Port/2   2/3/11 6:01p christ
 * SW7420-1458: DirectFB: Rework internal handling of nexus heaps.
 *
 * DirectFB_1_4_Port/1   1/25/11 8:05p robertwm
 * SW7420-1424: DirectFB: Refactor platform_init code to separate DTV and
 *  STB parts and to support NEXUS heap m'ment.
 *
 * DirectFB_1_4_Port/DirectFB_STEREO_3D_JIRA_SW7422_138/2   1/18/11 1:27p christ
 * Fix DTV compile problems.
 *
 * DirectFB_1_4_Port/DirectFB_STEREO_3D_JIRA_SW7422_138/1   1/17/11 12:33p christ
 * STB specific platform init code.
 */

#include <stdlib.h>
#include <string.h>

#include "platform_init.h"

#ifndef NEXUS_DTV_PLATFORM

NEXUS_HeapHandle  nexus_display_heap;
NEXUS_HeapHandle  nexus_offscreen_heap;
NEXUS_HeapHandle  nexus_palette_heap;
NEXUS_HeapHandle  nexus_video_heap;

NEXUS_VideoFormat nexus_video_format_preferred;
#ifdef NEXUS_HAS_PICTURE_DECODER
NEXUS_PictureDecoderHandle nexus_picturedecoder_handle;
#endif
NEXUS_DisplayHandle nexus_display_handle0;
#if (NUM_DISPLAYS > 1)
NEXUS_DisplayHandle nexus_display_handle1;
#endif
#if NEXUS_HAS_GRAPHICS2D || NEXUS_HAS_ZSP_GRAPHICS
NEXUS_Graphics2DHandle nexus_graphics2d_handle;
#endif

#if NEXUS_HAS_GRAPHICS3D && defined(BCMNEXUS_GFX_HAS_P3D)
NEXUS_Graphics3DHandle nexus_graphics3d_handle;
#endif

#if NEXUS_NUM_HDMI_OUTPUTS
static void hotplug_callback(void *pParam, int iParam)
{
    NEXUS_HdmiOutputStatus status;
    NEXUS_HdmiOutputHandle hdmi = pParam;
    NEXUS_DisplayHandle display = (NEXUS_DisplayHandle)iParam;
    NEXUS_DisplaySettings displaySettings;

    NEXUS_Display_GetSettings(display, &displaySettings);
    NEXUS_HdmiOutput_GetStatus(hdmi, &status);
    BDBG_WRN(("HDMI hotplug event: %s\n", status.connected?"connected":"not connected"));

    /* the app can choose to switch to the preferred format, but it's not required. */
    if ( status.connected )
    {
        if ( !status.videoFormatSupported[displaySettings.format] )
        {
            BDBG_WRN(("\nCurrent format not supported by attached monitor. Switching to preferred format %d\n", status.preferredVideoFormat));
#if NEXUS_NUM_COMPONENT_OUTPUTS
            if (NEXUS_IsComponentOutputAttached(0) && (status.preferredVideoFormat >= NEXUS_VideoFormat_eVesa640x480p60hz)) {
                BDBG_WRN(("Removing Component output as it cannot support this video format.\n"));
                NEXUS_DetachComponentOutput(display, 0);
            }
#endif
            displaySettings.format = status.preferredVideoFormat;
            SET_NEXUS_VIDEO_FORMAT_PREFERRED(displaySettings.format);
        }
        NEXUS_Display_SetSettings(display, &displaySettings);
    }
#if NEXUS_NUM_COMPONENT_OUTPUTS
    else if (NEXUS_IsComponentOutputAttached(0) == false) {
        if (displaySettings.format < NEXUS_VideoFormat_eVesa640x480p60hz) {
            NEXUS_AttachComponentOutput(display, 0);
        }
    }
#endif
}
#endif


NEXUS_Error NEXUS_DetachComponentOutput(NEXUS_DisplayHandle display, int output)
{
    NEXUS_Error rc = NEXUS_SUCCESS;
#if NEXUS_NUM_COMPONENT_OUTPUTS
    NEXUS_PlatformConfiguration platformConfig;
    NEXUS_Platform_GetConfiguration(&platformConfig);
    rc = NEXUS_Display_RemoveOutput(display, NEXUS_ComponentOutput_GetConnector(platformConfig.outputs.component[output]));
    if (rc == NEXUS_SUCCESS) {
        nexus_is_component_connected = false;
    }
    if (component_output_callback)
        component_output_callback(display, output, nexus_is_component_connected);

#else
    (void)display;
    (void)output;
#endif
    return rc;
}

NEXUS_Error NEXUS_AttachComponentOutput(NEXUS_DisplayHandle display, int output)
{
    NEXUS_Error rc = NEXUS_SUCCESS;
#if NEXUS_NUM_COMPONENT_OUTPUTS
    NEXUS_PlatformConfiguration platformConfig;
    NEXUS_Platform_GetConfiguration(&platformConfig);
    rc = NEXUS_Display_AddOutput(display, NEXUS_ComponentOutput_GetConnector(platformConfig.outputs.component[output]));
    if (rc == NEXUS_SUCCESS) {
        nexus_is_component_connected = true;
    }
    if (component_output_callback)
        component_output_callback(display, output, nexus_is_component_connected);

#else
    (void)display;
    (void)output;
#endif
    return rc;
}

#if NEXUS_NUM_COMPONENT_OUTPUTS
bool NEXUS_IsComponentOutputAttached(int output)
{
    (void)output;

    return nexus_is_component_connected;
}

void NEXUS_Register_ComponentOutputCallback(output_callback_t callback)
{
    component_output_callback = callback;
}

void NEXUS_Unregister_ComponentOutputCallback(void)
{
    component_output_callback = NULL;
}
#endif

int configurePlatformDisplay( NEXUS_DisplayHandle        *display0,
                              NEXUS_DisplayHandle        *display1,
                              NEXUS_DisplaySettings       displayCfg,
                              NEXUS_Shared_Handles       *sharedHandles,
                              NEXUS_PlatformConfiguration platformConfig)
{
    bool openDisplay0 = false;
    bool openDisplay1 = false;
#if NEXUS_NUM_HDMI_OUTPUTS
    NEXUS_HdmiOutputSettings hdmiSettings;
#endif

    NEXUS_Error rc;

    /* Should we open up the display0 module? */
    if (*display0 == NULL) {
        openDisplay0 = true;
        *display0 = NEXUS_Display_Open(0, &displayCfg);
    }

    SET_NEXUS_DISPLAY_HND(0, *display0);
    sharedHandles->disp_handle0 = *display0;

#if NEXUS_NUM_COMPONENT_OUTPUTS

    nexus_is_component_connected = false;

    if (openDisplay0 == true)
    {
        rc = NEXUS_Display_AddOutput(*display0, NEXUS_ComponentOutput_GetConnector(platformConfig.outputs.component[0]));
        if (rc!=NEXUS_SUCCESS) {rc=BERR_TRACE(rc); return -1;}
    }
    nexus_is_component_connected = true;

    sharedHandles->is_component_connected = nexus_is_component_connected;
#endif

#if NEXUS_NUM_HDMI_OUTPUTS
    if (openDisplay0 == true)
    {
        rc = NEXUS_Display_AddOutput(*display0, NEXUS_HdmiOutput_GetVideoConnector(platformConfig.outputs.hdmi[0]));
        if(rc!=NEXUS_SUCCESS) {rc=BERR_TRACE(rc); return -1;}
    }
    SET_NEXUS_HDMI_HND(platformConfig.outputs.hdmi[0]);
#endif

#if NEXUS_NUM_SCART_INPUTS
    if (openDisplay0 == true)
    {
        rc = NEXUS_Display_AddOutput(*display0, NEXUS_ScartInput_GetVideoOutputConnector(platformConfig.inputs.scart[0]));
        if(rc!=NEXUS_SUCCESS) {rc=BERR_TRACE(rc); return -1;}

#if NEXUS_NUM_SCART_INPUTS > 1
        rc = NEXUS_Display_AddOutput(*display0, NEXUS_ScartInput_GetVideoOutputConnector(platformConfig.inputs.scart[1]));
        if(rc!=NEXUS_SUCCESS) {rc=BERR_TRACE(rc); return -1;}
#endif
    }

#endif // NEXUS_NUM_SCART_INPUTS

    if ((getenv("hdsd_mode") != NULL) && (strcmp(getenv("hdsd_mode"), "0") == 0))
    {
#if NEXUS_NUM_COMPOSITE_OUTPUTS
        if (openDisplay0 == true)
        {
            rc = NEXUS_Display_AddOutput(*display0, NEXUS_CompositeOutput_GetConnector(platformConfig.outputs.composite[0]));
            if(rc!=NEXUS_SUCCESS) {rc=BERR_TRACE(rc);return -1;}
        }
#endif

#if NEXUS_NUM_RFM_OUTPUTS
        if ((getenv("enable_rfm") != NULL) && (strcmp(getenv("enable_rfm"), "y") == 0))
        {
            if (openDisplay0 == true)
            {
                rc=NEXUS_Display_AddOutput(*display0, NEXUS_Rfm_GetVideoConnector(platformConfig.outputs.rfm[0]));
                if(rc!=NEXUS_SUCCESS) {rc=BERR_TRACE(rc);return -1;}
            }
        }
#endif
    }

#if (NUM_DISPLAYS > 1)
    /* Should we open up the display1 module? */
    if (*display1 == NULL) {
        openDisplay1 = true;
        *display1 = NEXUS_Display_Open(1, &displayCfg);
    }

    if(!display1) {rc=BERR_TRACE(NEXUS_NOT_SUPPORTED); return -2;}
    SET_NEXUS_DISPLAY_HND(1, *display1);
    sharedHandles->disp_handle1 = *display1;

    if ((getenv("hdsd_mode") == NULL) || (strcmp(getenv("hdsd_mode"), "0") != 0))
    {
        if (openDisplay1 == true)
        {
#if NEXUS_NUM_COMPOSITE_OUTPUTS
            rc = NEXUS_Display_AddOutput(*display1, NEXUS_CompositeOutput_GetConnector(platformConfig.outputs.composite[0]));
            if(rc!=NEXUS_SUCCESS) {rc=BERR_TRACE(rc);return -2;}
#endif
#if NEXUS_NUM_RFM_OUTPUTS && !defined(DIRECTFB_GRAPHICS_MIRROR)
            if ((getenv("enable_rfm") != NULL) && (strcmp(getenv("enable_rfm"), "y") == 0))
            {
                rc=NEXUS_Display_AddOutput(*display1, NEXUS_Rfm_GetVideoConnector(platformConfig.outputs.rfm[0]));
                if(rc!=NEXUS_SUCCESS) {rc=BERR_TRACE(rc);return -2;}
            }
#endif
        }
    }
#endif  // (NUM_DISPLAYS > 1)

#if NEXUS_NUM_HDMI_OUTPUTS
    if (openDisplay0 == true)
    {
        /* Install hotplug callback -- video only for now */
        NEXUS_HdmiOutput_GetSettings(platformConfig.outputs.hdmi[0], &hdmiSettings);
        hdmiSettings.hotplugCallback.callback = hotplug_callback;
        hdmiSettings.hotplugCallback.context = platformConfig.outputs.hdmi[0];
        hdmiSettings.hotplugCallback.param = (int)*display0;
        NEXUS_HdmiOutput_SetSettings(platformConfig.outputs.hdmi[0], &hdmiSettings);
        /* Force a hotplug to switch to preferred format */
        hotplug_callback(platformConfig.outputs.hdmi[0], (int)*display0);
    }
#endif

    return 0;
}

/* This is no way to discover if a system is UMA/non-UMA without running NEXUS_Platform_Init
   for systems with 2 memory controllers.
   For customer applications, they will know beforehand and can hardcode it.
   Also for cases where we want to have multiple heaps spread across memory controllers
   we need to initialise the system to find out how much memory the rest of the system needs.
   For generic test apps, we have to do an init/uninit to discover this information. */

void getSystemHeapUsage(unsigned int *memc0Free, unsigned int *memc1Free, bool *isNonUma)
{

    NEXUS_PlatformConfiguration platformConfiguration;
    NEXUS_Error rc;
    unsigned i;
    NEXUS_PlatformSettings platformSettings;

    *memc0Free = 0;
    *memc1Free = 0;
    *isNonUma  = false;

    NEXUS_Platform_GetDefaultSettings(&platformSettings);

    platformSettings.openFrontend = false;
    platformSettings.openI2c = false;
    platformSettings.openFpga = false;
    platformSettings.openOutputs = false;

    NEXUS_Platform_Init(&platformSettings);
    NEXUS_Platform_GetConfiguration(&platformConfiguration);

    for (i=0;i<NEXUS_MAX_HEAPS;i++)
    {
        NEXUS_MemoryStatus status;
        NEXUS_HeapHandle heap = platformConfiguration.heap[i];

        if (!heap)
            break;

        rc = NEXUS_Heap_GetStatus(heap, &status);

        BDBG_ASSERT(!rc);
        BDBG_MSG(("platform_init: Heap[%d]: memcIndex=%d, size=%d, offset=0x%08x, type=%d\n",
          i, status.memcIndex, status.size, status.offset, status.memoryType));

        if (status.memcIndex == 0)
            *memc0Free += status.largestFreeBlock;

        if (status.memcIndex == 1)
        {
            *memc1Free += status.largestFreeBlock;
            *isNonUma = true;
        }
    }

    NEXUS_Platform_Uninit();
}

void setupNexusHeaps( NEXUS_PlatformSettings *platformSettings,
                      unsigned char          *nexusHeapIndex)
{

    unsigned long heapSize;

#if (NEXUS_NUM_MEMC > 1)
    unsigned int memc0Free;
    unsigned int memc1Free;
    bool         isNonUma = true;
#endif

    /*
      Possible Nexus heap configurations:

      Systems with 1 MEMC
      ===================
      heap[0] will be the default MEMC0 heap.
      heap[1] will be the dedicated surface heap also on MEMC0.

      Systems with 2 MEMC
      ===================
      heap[0] will be the default MEMC0 heap (and shared gfx heap on 7400).
      heap[1] will be the default MEMC1 heap.
      heap[2] will be the dedicated surface heap on MEMC0 execpt for 7420 which is MEMC1.
      heap[3] will be the dedicated secondary surface heap on MEMC1 except for 7420 which is MEMC0.

    */

    heapSize = get_gfx_heap_size();

    platformSettings->heap[0].memcIndex = 0;
    platformSettings->heap[0].size = -1;
    platformSettings->heap[0].alignment = BCMNEXUS_MAX_CPU_CACHE_LINE_SIZE;
    platformSettings->heap[0].memoryType = NEXUS_MemoryType_eDevice;

#if (NEXUS_NUM_MEMC == 1)

    platformSettings->heap[1].memcIndex = 0;
    platformSettings->heap[1].size = heapSize;
    platformSettings->heap[1].alignment = BCMNEXUS_MAX_CPU_CACHE_LINE_SIZE;
    platformSettings->heap[1].memoryType = NEXUS_MemoryType_eDevice;

    nexusHeapIndex[0] = nexusHeapIndex[1] = 1;

#else

    /* IMPORTANT NOTE:
       --------------
       Even though NEXUS indicates that there is more than one memory controller,
       at run-time the memory controllers can be unified causing memory heap
       mapping problems in Nexus.  The user can set the envar "mem_non_uma=n" for
       platforms that have more than one memory controller but are configured
       as a single controller (unified memory).

       The user can determine whether the platform has unified (UMA) or
       non-unified memory by inspecting CFE at boot-time and looking for the
       line "Memory Config:". For example, if this line says "64-bit UMA" then
       the user will need to set the envar "mem_non_uma=n".
    */

    if (getenv("mem_non_uma") && !strcmp(getenv("mem_non_uma"), "n"))
        isNonUma = false;

    /* If the user forgot to setup "mem_non_uma" envar to speed up platform
       initialisation time, then we can check whether memory is UMA here... */
    if (isNonUma == true)
        getSystemHeapUsage(&memc0Free,&memc1Free,&isNonUma);

    if (isNonUma == false)
    {
        platformSettings->heap[1].memcIndex = 0;
        platformSettings->heap[1].size = heapSize;
        platformSettings->heap[1].alignment = BCMNEXUS_MAX_CPU_CACHE_LINE_SIZE;
        platformSettings->heap[1].memoryType = NEXUS_MemoryType_eDevice;

        nexusHeapIndex[0] = nexusHeapIndex[1] = 1;
    }
    else
    {
        // On systems with 2 MEMCs Nexus presumes that the 2nd heap will be on the other memory controller
        // So we need to make sure that our heap is the last one in the system.

        platformSettings->heap[1].memcIndex = 1;
        platformSettings->heap[1].size = -1;
        platformSettings->heap[1].alignment = BCMNEXUS_MAX_CPU_CACHE_LINE_SIZE;
        platformSettings->heap[1].memoryType = NEXUS_MemoryType_eDevice;

        platformSettings->heap[2].memcIndex = GFX_FEEDER_MEMC_INDEX;
        platformSettings->heap[2].size = heapSize;
        platformSettings->heap[2].alignment = BCMNEXUS_MAX_CPU_CACHE_LINE_SIZE;
        platformSettings->heap[2].memoryType = NEXUS_MemoryType_eDevice;

        nexusHeapIndex[0] = 2;

        if (heapSize > (GFX_FEEDER_MEMC_INDEX ? memc1Free : memc0Free) )
        {
            // Not enough room for all the requested gfx memory, allocate some in the other memc

            unsigned int secondaryHeapSize = heapSize - (GFX_FEEDER_MEMC_INDEX ? memc1Free : memc0Free) ;
            heapSize = (GFX_FEEDER_MEMC_INDEX ? memc1Free : memc0Free);

            platformSettings->heap[2].size = heapSize;

            platformSettings->heap[3].memcIndex = GFX_FEEDER_MEMC_INDEX ? 0 : 1;
            platformSettings->heap[3].size = secondaryHeapSize ;
            platformSettings->heap[3].alignment = BCMNEXUS_MAX_CPU_CACHE_LINE_SIZE;
            platformSettings->heap[3].memoryType = NEXUS_MemoryType_eDevice;

            nexusHeapIndex[1] = 3;

        }
        else
            nexusHeapIndex[1] = 2;
    }

    BDBG_MSG(("Heap 0 - memc %d size %d\nHeap 1 - memc %d size %d\nHeap 2 - memc %d size %d\nHeap 3 - memc %d size %d\n",
          platformSettings->heap[0].memcIndex,platformSettings->heap[0].size,
          platformSettings->heap[1].memcIndex,platformSettings->heap[1].size,
          platformSettings->heap[2].memcIndex,platformSettings->heap[2].size,
          platformSettings->heap[3].memcIndex,platformSettings->heap[3].size));

#endif

}

void setupDTVPanel(NEXUS_PlatformSettings *platformSettings)
{
    // DTV only - no need to implement
    BDBG_ASSERT(platformSettings);
}

#else /* ! NEXUS_DTV_PLATFORM */

// stupid but stops warning
void nothing(void);

#endif /* ! NEXUS_DTV_PLATFORM */
