/******************************************************************************
 *    (c)2010 Broadcom Corporation
 * 
 * This program is the proprietary software of Broadcom Corporation and/or its licensors,
 * and may only be used, duplicated, modified or distributed pursuant to the terms and
 * conditions of a separate, written license agreement executed between you and Broadcom
 * (an "Authorized License").  Except as set forth in an Authorized License, Broadcom grants
 * no license (express or implied), right to use, or waiver of any kind with respect to the
 * Software, and Broadcom expressly reserves all rights in and to the Software and all
 * intellectual property rights therein.  IF YOU HAVE NO AUTHORIZED LICENSE, THEN YOU
 * HAVE NO RIGHT TO USE THIS SOFTWARE IN ANY WAY, AND SHOULD IMMEDIATELY
 * NOTIFY BROADCOM AND DISCONTINUE ALL USE OF THE SOFTWARE.  
 *  
 * Except as expressly set forth in the Authorized License,
 *  
 * 1.     This program, including its structure, sequence and organization, constitutes the valuable trade
 * secrets of Broadcom, and you shall use all reasonable efforts to protect the confidentiality thereof,
 * and to use this information only in connection with your use of Broadcom integrated circuit products.
 *  
 * 2.     TO THE MAXIMUM EXTENT PERMITTED BY LAW, THE SOFTWARE IS PROVIDED "AS IS" 
 * AND WITH ALL FAULTS AND BROADCOM MAKES NO PROMISES, REPRESENTATIONS OR 
 * WARRANTIES, EITHER EXPRESS, IMPLIED, STATUTORY, OR OTHERWISE, WITH RESPECT TO 
 * THE SOFTWARE.  BROADCOM SPECIFICALLY DISCLAIMS ANY AND ALL IMPLIED WARRANTIES 
 * OF TITLE, MERCHANTABILITY, NONINFRINGEMENT, FITNESS FOR A PARTICULAR PURPOSE, 
 * LACK OF VIRUSES, ACCURACY OR COMPLETENESS, QUIET ENJOYMENT, QUIET POSSESSION 
 * OR CORRESPONDENCE TO DESCRIPTION. YOU ASSUME THE ENTIRE RISK ARISING OUT OF 
 * USE OR PERFORMANCE OF THE SOFTWARE.
 * 
 * 3.     TO THE MAXIMUM EXTENT PERMITTED BY LAW, IN NO EVENT SHALL BROADCOM OR ITS 
 * LICENSORS BE LIABLE FOR (i) CONSEQUENTIAL, INCIDENTAL, SPECIAL, INDIRECT, OR 
 * EXEMPLARY DAMAGES WHATSOEVER ARISING OUT OF OR IN ANY WAY RELATING TO YOUR 
 * USE OF OR INABILITY TO USE THE SOFTWARE EVEN IF BROADCOM HAS BEEN ADVISED OF 
 * THE POSSIBILITY OF SUCH DAMAGES; OR (ii) ANY AMOUNT IN EXCESS OF THE AMOUNT 
 * ACTUALLY PAID FOR THE SOFTWARE ITSELF OR U.S. $1, WHICHEVER IS GREATER. THESE 
 * LIMITATIONS SHALL APPLY NOTWITHSTANDING ANY FAILURE OF ESSENTIAL PURPOSE OF 
 * ANY LIMITED REMEDY.
 *
 * $brcm_Workfile: shim_nexus.c $
 * $brcm_Revision: DirectFB_1_4_Port/1 $
 * $brcm_Date: 7/26/10 5:06p $
 * 
 * Module Description:
 * 
 * Revision History:
 * 
 * $brcm_Log: /AppLibs/opensource/directfb/src/broadcom_files/public/DirectFB/1.4/shim_nexus.c $
 * 
 * DirectFB_1_4_Port/1   7/26/10 5:06p jackf
 * SW35230-771: Added passthrough shim_nexus.c file.
 * 
 * 
 *****************************************************************************/

// These should be all that are needed.

#include "nexus_platform.h"
#include "nexus_surface.h"
#if NEXUS_HAS_GRAPHICS2D
#include "nexus_graphics2d.h"
#endif
#include "nexus_display.h"
#if NEXUS_HAS_PICTURE_DECODER
#include "nexus_picture_decoder.h"
#endif
#include "nexus_core_utils.h"
#if NEXUS_HAS_IR_INPUT
#include "nexus_ir_input.h"
#endif
#if NEXUS_HAS_KEYPAD
#include "nexus_keypad.h"
#endif
#include "nexus_memory.h"

//================================================================================
// Display group
//================================================================================

void                        shim_nexus_Display_GetSettings(             NEXUS_DisplayHandle                         display, 
                                                                        NEXUS_DisplaySettings                      *pSettings)
{
    NEXUS_Display_GetSettings(display, pSettings);
}

NEXUS_Error                 shim_nexus_Display_SetSettings(             NEXUS_DisplayHandle                         display, 
                                                                        const NEXUS_DisplaySettings                *pSettings)
{
    return NEXUS_Display_SetSettings(display, pSettings);
}

void                        shim_nexus_Display_GetGraphicsSettings(     NEXUS_DisplayHandle                         display, 
                                                                        NEXUS_GraphicsSettings                     *pSettings)
{
    NEXUS_Display_GetGraphicsSettings(display, pSettings);
}

NEXUS_Error                 shim_nexus_Display_SetGraphicsSettings(     NEXUS_DisplayHandle                         display, 
                                                                        const NEXUS_GraphicsSettings               *pSettings)
{
    return NEXUS_Display_SetGraphicsSettings(display, pSettings);
}

NEXUS_Error                 shim_nexus_Display_SetGraphicsFramebuffer(  NEXUS_DisplayHandle                         display, 
                                                                        NEXUS_SurfaceHandle                         frameBuffer)
{
    return NEXUS_Display_SetGraphicsFramebuffer(display, frameBuffer);
}

//================================================================================
// Graphics 2D group
//================================================================================

#if NEXUS_HAS_GRAPHICS2D
void                        shim_nexus_Graphics2D_GetSettings(          NEXUS_Graphics2DHandle                      handle,
                                                                        NEXUS_Graphics2DSettings                   *pSettings)
{
    NEXUS_Graphics2D_GetSettings(handle, pSettings);
}

NEXUS_Error                 shim_nexus_Graphics2D_SetSettings(          NEXUS_Graphics2DHandle                      handle, 
                                                                        const NEXUS_Graphics2DSettings             *pSettings)
{
    return NEXUS_Graphics2D_SetSettings(handle, pSettings);
}

void                        shim_nexus_Graphics2D_GetDefaultBlitSettings(
                                                                        NEXUS_Graphics2DBlitSettings               *pSettings)
{
    NEXUS_Graphics2D_GetDefaultBlitSettings(pSettings);
}

void                        shim_nexus_Graphics2D_GetDefaultFillSettings(
                                                                        NEXUS_Graphics2DFillSettings               *pSettings)
{
    NEXUS_Graphics2D_GetDefaultFillSettings(pSettings);
}

void                        shim_nexus_Graphics2D_GetDefaultPorterDuffFillSettings(
                                                                        NEXUS_Graphics2DPorterDuffFillSettings     *pSettings)
{
    NEXUS_Graphics2D_GetDefaultPorterDuffFillSettings(pSettings);
}

NEXUS_Error                 shim_nexus_Graphics2D_Blit(                 NEXUS_Graphics2DHandle                      handle,
                                                                        const NEXUS_Graphics2DBlitSettings         *pSettings)
{
    return NEXUS_Graphics2D_Blit(handle, pSettings);
}

NEXUS_Error                 shim_nexus_Graphics2D_FastBlit(             NEXUS_Graphics2DHandle                      handle,
                                                                        NEXUS_SurfaceHandle                         outputSurface,
                                                                        const NEXUS_Rect                           *pOutputRect,
                                                                        NEXUS_SurfaceHandle                         sourceSurface,
                                                                        const NEXUS_Rect                           *pSourceRect,
                                                                        NEXUS_BlitColorOp                           colorOp,
                                                                        NEXUS_BlitAlphaOp                           alphaOp,
                                                                        NEXUS_Pixel                                 constantColor)
{
    return NEXUS_Graphics2D_FastBlit(handle, outputSurface, pOutputRect, sourceSurface, pSourceRect, colorOp, alphaOp, constantColor);
}

NEXUS_Error                 shim_nexus_Graphics2D_Fill(                 NEXUS_Graphics2DHandle                      handle,
                                                                        const NEXUS_Graphics2DFillSettings         *pSettings)
{
    return NEXUS_Graphics2D_Fill(handle, pSettings);
}

NEXUS_Error                 shim_nexus_Graphics2D_PorterDuffFill(       NEXUS_Graphics2DHandle                      handle,
                                                                        const NEXUS_Graphics2DPorterDuffFillSettings *pSettings)
{
    return NEXUS_Graphics2D_PorterDuffFill(handle, pSettings);
}

NEXUS_Error                 shim_nexus_Graphics2D_Checkpoint(           NEXUS_Graphics2DHandle                      handle,
                                                                        const NEXUS_CallbackDesc                   *pLegacyCallback)
{
    return NEXUS_Graphics2D_Checkpoint(handle, pLegacyCallback);
}
#endif

//================================================================================
// Surface group
//================================================================================

void                        shim_nexus_Surface_GetDefaultCreateSettings(NEXUS_SurfaceCreateSettings                *pCreateSettings)
{
    NEXUS_Surface_GetDefaultCreateSettings(pCreateSettings);
}

void                        shim_nexus_Surface_GetCreateSettings(       NEXUS_SurfaceHandle                         surface,
                                                                        NEXUS_SurfaceCreateSettings                *pCreateSettings)
{
    NEXUS_Surface_GetCreateSettings(surface, pCreateSettings);
}

NEXUS_SurfaceHandle         shim_nexus_Surface_Create(                  const NEXUS_SurfaceCreateSettings          *pCreateSettings)
{
    return NEXUS_Surface_Create(pCreateSettings);
}

void                        shim_nexus_Surface_Destroy(                 NEXUS_SurfaceHandle                         surface)
{
    NEXUS_Surface_Destroy(surface);
}

void                        shim_nexus_Surface_Flush(                   NEXUS_SurfaceHandle                         surface)
{
    NEXUS_Surface_Flush(surface);
}

void                        shim_nexus_Surface_GetMemory(               NEXUS_SurfaceHandle                         surface, 
                                                                        NEXUS_SurfaceMemory                        *pMemory)
{
    NEXUS_Surface_GetMemory(surface, pMemory);
}

//================================================================================
// IR group
//================================================================================

#if NEXUS_HAS_IR_INPUT
NEXUS_IrInputHandle         shim_nexus_IrInput_Open(                    unsigned                                    index,
                                                                        const NEXUS_IrInputSettings                *pSettings)
{
    return NEXUS_IrInput_Open(index, pSettings);
}

void                        shim_nexus_IrInput_Close(                   NEXUS_IrInputHandle                         handle)
{
    NEXUS_IrInput_Close(handle);
}

void                        shim_nexus_IrInput_GetDefaultSettings(      NEXUS_IrInputSettings                      *pSettings)
{
    NEXUS_IrInput_GetDefaultSettings(pSettings);
}

NEXUS_Error                 shim_nexus_IrInput_GetEvents(               NEXUS_IrInputHandle                         handle,
                                                                        NEXUS_IrInputEvent                         *pEvents,
                                                                        size_t                                      numEvents,
                                                                        size_t                                     *pNumEventsRead,
                                                                        bool                                       *pOverflow)
{
    return NEXUS_IrInput_GetEvents(handle, pEvents, numEvents, pNumEventsRead, pOverflow);
}
#endif

//================================================================================
// PictureDecoder group
//================================================================================

#if NEXUS_HAS_PICTURE_DECODER
NEXUS_PictureDecoderHandle  shim_nexus_PictureDecoder_Open(             unsigned                                    index,
                                                                        const NEXUS_PictureDecoderOpenSettings     *pSettings)
{
    return NEXUS_PictureDecoder_Open(index, pSettings);
}

void                        shim_nexus_PictureDecoder_Close(            NEXUS_PictureDecoderHandle                  handle)
{
    NEXUS_PictureDecoder_Close(handle);
}

void                        shim_nexus_PictureDecoder_GetDefaultStartSettings(
                                                                        NEXUS_PictureDecoderStartSettings          *pSettings)
{
    NEXUS_PictureDecoder_GetDefaultStartSettings(pSettings);
}

NEXUS_Error                 shim_nexus_PictureDecoder_GetBuffer(        NEXUS_PictureDecoderHandle                  decoder,
                                                                        void                                      **pBuffer,
                                                                        size_t                                     *pSize)
{
    return NEXUS_PictureDecoder_GetBuffer(decoder, pBuffer, pSize);
}

NEXUS_Error                 shim_nexus_PictureDecoder_GetPalette(       NEXUS_PictureDecoderHandle                  decoder,
                                                                        NEXUS_PictureDecoderPalette                *pPalette)
{
    return NEXUS_PictureDecoder_GetPalette(decoder, pPalette);
}

NEXUS_Error                 shim_nexus_PictureDecoder_GetStatus(        NEXUS_PictureDecoderHandle                  decoder,
                                                                        NEXUS_PictureDecoderStatus                 *pStatus)
{
    return NEXUS_PictureDecoder_GetStatus(decoder, pStatus);
}

NEXUS_Error                 shim_nexus_PictureDecoder_DecodeSegment(    NEXUS_PictureDecoderHandle                  handle,
                                                                        NEXUS_SurfaceHandle                         destination,
                                                                        const NEXUS_Rect                           *pSegment)
{
    return NEXUS_PictureDecoder_DecodeSegment(handle, destination, pSegment);
}

NEXUS_Error                 shim_nexus_PictureDecoder_ReadComplete(     NEXUS_PictureDecoderHandle                  decoder,
                                                                        size_t                                      skip,
                                                                        size_t                                      amountUsed)
{
    return NEXUS_PictureDecoder_ReadComplete(decoder, skip, amountUsed);
}

NEXUS_Error                 shim_nexus_PictureDecoder_Start(            NEXUS_PictureDecoderHandle                  handle,
                                                                        const NEXUS_PictureDecoderStartSettings    *pSettings)
{
    return NEXUS_PictureDecoder_Start(handle, pSettings);
}

void                        shim_nexus_PictureDecoder_Stop(             NEXUS_PictureDecoderHandle                  handle)
{
    NEXUS_PictureDecoder_Stop(handle);
}
#endif

//================================================================================
// Keypad group
//================================================================================

#if NEXUS_HAS_KEYPAD
NEXUS_KeypadHandle          shim_nexus_Keypad_Open(                     unsigned                                    index,
                                                                        const NEXUS_KeypadSettings                 *pSettings)
{
    return NEXUS_Keypad_Open(index, pSettings);
}

void                        shim_nexus_Keypad_Close(                    NEXUS_KeypadHandle                          handle)
{
    NEXUS_Keypad_Close(handle);
}

void                        shim_nexus_Keypad_GetDefaultSettings(       NEXUS_KeypadSettings                       *pSettings)
{
    NEXUS_Keypad_GetDefaultSettings(pSettings);
}

NEXUS_Error                 shim_nexus_Keypad_GetEvents(                NEXUS_KeypadHandle                          handle,
                                                                        NEXUS_KeypadEvent                          *pEvents,
                                                                        size_t                                      numEvents,
                                                                        size_t                                     *pNumEventsRead,
                                                                        bool                                       *pOverflow)
{
    return NEXUS_Keypad_GetEvents(handle, pEvents, numEvents, pNumEventsRead, pOverflow);
}
#endif

//================================================================================
// Memory functions
//================================================================================

void                        shim_nexus_Memory_GetDefaultAllocationSettings(
                                                                        NEXUS_MemoryAllocationSettings             *pSettings)
{
    NEXUS_Memory_GetDefaultAllocationSettings(pSettings);
}

NEXUS_Error                 shim_nexus_Memory_Allocate(                 size_t                                      numBytes,
                                                                        const NEXUS_MemoryAllocationSettings       *pSettings,
                                                                        void                                      **ppMemory)
{
    return NEXUS_Memory_Allocate(numBytes, pSettings, ppMemory);
}

void                        shim_nexus_Memory_Free(                     void                                       *pMemory)
{
    NEXUS_Memory_Free(pMemory);
}

//================================================================================
// Misc functions
//================================================================================

void                        shim_nexus_Platform_GetConfiguration(       NEXUS_PlatformConfiguration                *pConfiguration)
{
    NEXUS_Platform_GetConfiguration(pConfiguration);
}

NEXUS_Error                 shim_nexus_Platform_Join(                   void)
{
    return NEXUS_Platform_Join();
}

NEXUS_Error                 shim_nexus_Heap_GetStatus(                  NEXUS_HeapHandle                            heap,
                                                                        NEXUS_MemoryStatus                         *pStatus)
{
    return NEXUS_Heap_GetStatus(heap, pStatus);
}

void                        shim_nexus_StopCallbacks(                   void                                       *interfaceHandle)
{
    NEXUS_StopCallbacks(interfaceHandle);
}

void                        shim_nexus_VideoFormat_GetInfo(             NEXUS_VideoFormat                           videoFmt,
                                                                        NEXUS_VideoFormatInfo                      *pVideoFmtInfo)
{
    NEXUS_VideoFormat_GetInfo(videoFmt, pVideoFmtInfo);
}

