/******************************************************************************
 *    (c)2011 Broadcom Corporation
 *
 * This program is the proprietary software of Broadcom Corporation and/or its licensors,
 * and may only be used, duplicated, modified or distributed pursuant to the terms and
 * conditions of a separate, written license agreement executed between you and Broadcom
 * (an "Authorized License").  Except as set forth in an Authorized License, Broadcom grants
 * no license (express or implied), right to use, or waiver of any kind with respect to the
 * Software, and Broadcom expressly reserves all rights in and to the Software and all
 * intellectual property rights therein.  IF YOU HAVE NO AUTHORIZED LICENSE, THEN YOU
 * HAVE NO RIGHT TO USE THIS SOFTWARE IN ANY WAY, AND SHOULD IMMEDIATELY
 * NOTIFY BROADCOM AND DISCONTINUE ALL USE OF THE SOFTWARE.
 *
 * Except as expressly set forth in the Authorized License,
 *
 * 1.     This program, including its structure, sequence and organization, constitutes the valuable trade
 * secrets of Broadcom, and you shall use all reasonable efforts to protect the confidentiality thereof,
 * and to use this information only in connection with your use of Broadcom integrated circuit products.
 *
 * 2.     TO THE MAXIMUM EXTENT PERMITTED BY LAW, THE SOFTWARE IS PROVIDED "AS IS"
 * AND WITH ALL FAULTS AND BROADCOM MAKES NO PROMISES, REPRESENTATIONS OR
 * WARRANTIES, EITHER EXPRESS, IMPLIED, STATUTORY, OR OTHERWISE, WITH RESPECT TO
 * THE SOFTWARE.  BROADCOM SPECIFICALLY DISCLAIMS ANY AND ALL IMPLIED WARRANTIES
 * OF TITLE, MERCHANTABILITY, NONINFRINGEMENT, FITNESS FOR A PARTICULAR PURPOSE,
 * LACK OF VIRUSES, ACCURACY OR COMPLETENESS, QUIET ENJOYMENT, QUIET POSSESSION
 * OR CORRESPONDENCE TO DESCRIPTION. YOU ASSUME THE ENTIRE RISK ARISING OUT OF
 * USE OR PERFORMANCE OF THE SOFTWARE.
 *
 * 3.     TO THE MAXIMUM EXTENT PERMITTED BY LAW, IN NO EVENT SHALL BROADCOM OR ITS
 * LICENSORS BE LIABLE FOR (i) CONSEQUENTIAL, INCIDENTAL, SPECIAL, INDIRECT, OR
 * EXEMPLARY DAMAGES WHATSOEVER ARISING OUT OF OR IN ANY WAY RELATING TO YOUR
 * USE OF OR INABILITY TO USE THE SOFTWARE EVEN IF BROADCOM HAS BEEN ADVISED OF
 * THE POSSIBILITY OF SUCH DAMAGES; OR (ii) ANY AMOUNT IN EXCESS OF THE AMOUNT
 * ACTUALLY PAID FOR THE SOFTWARE ITSELF OR U.S. $1, WHICHEVER IS GREATER. THESE
 * LIMITATIONS SHALL APPLY NOTWITHSTANDING ANY FAILURE OF ESSENTIAL PURPOSE OF
 * ANY LIMITED REMEDY.
 *
 * $brcm_Workfile: stb_init.h $
 * $brcm_Revision: DirectFB_1_4_Port/3 $
 * $brcm_Date: 3/11/11 5:05p $
 *
 * Module Description:
 *
 * Revision History:
 *
 * $brcm_Log: /AppLibs/opensource/directfb/src/broadcom_files/private/DirectFB/1.4.1/stb_init.h $
 * 
 * DirectFB_1_4_Port/3   3/11/11 5:05p christ
 * SW7468-370: DirectFB: 1.4.1 release fails to compile for platforms
 *  without component output.
 *
 * DirectFB_1_4_Port/2   2/3/11 6:01p christ
 * SW7420-1458: DirectFB: Rework internal handling of nexus heaps.
 *
 * DirectFB_1_4_Port/1   1/25/11 8:08p robertwm
 * SW7420-1424: DirectFB: Refactor platform_init code to separate DTV and
 *  STB parts and to support NEXUS heap m'ment.
 *
 * DirectFB_1_4_Port/DirectFB_STEREO_3D_JIRA_SW7422_138/1   1/17/11 2:45p christ
 * libinit changes.
 *
 *****************************************************************************/
#ifndef __STB_INIT_H__
#define __STB_INIT_H__

#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/mman.h>
#include <fcntl.h>
#include <stdio.h>

#include "bstd.h"
#include "bkni.h"
#include "bkni_multi.h"

#include "platform_init.h"

/* Specify default Graphics Heap Size here.
   NOTE: This can be overriden by specifying
         "gfx_heap_size = abc" in the environment.
*/

#define DEFAULT_GFX_HEAP_SIZE (16 * 1024 * 1024)

#define HEAP platformConfig.heap

#if defined(NEXUS_MAX_HEAPS)

#if (BCHP_CHIP == 7420) && ((NEXUS_HAS_3D != 1) || (BCHP_VER >= BCHP_VER_B0))
#define GFX_FEEDER_MEMC_INDEX 1
#else
#define GFX_FEEDER_MEMC_INDEX 0
#endif

#endif /* NEXUS_MAX_HEAPS */

#if NEXUS_NUM_COMPONENT_OUTPUTS
bool NEXUS_IsComponentOutputAttached(int output);
void NEXUS_Register_ComponentOutputCallback(output_callback_t callback);
void NEXUS_Unregister_ComponentOutputCallback(void);

NEXUS_Error NEXUS_DetachComponentOutput(NEXUS_DisplayHandle display, int output);
NEXUS_Error NEXUS_AttachComponentOutput(NEXUS_DisplayHandle display, int output);

bool nexus_is_component_connected;
output_callback_t component_output_callback;
#endif

#if NEXUS_NUM_HDMI_OUTPUTS
NEXUS_HdmiOutputHandle nexus_hdmi_handle;
#endif

#endif /* __STB_INIT_H__ */
