/***************************************************************************
 *     (c)2007-2011 Broadcom Corporation
 *  
 *  This program is the proprietary software of Broadcom Corporation and/or its licensors,
 *  and may only be used, duplicated, modified or distributed pursuant to the terms and
 *  conditions of a separate, written license agreement executed between you and Broadcom
 *  (an "Authorized License").  Except as set forth in an Authorized License, Broadcom grants
 *  no license (express or implied), right to use, or waiver of any kind with respect to the
 *  Software, and Broadcom expressly reserves all rights in and to the Software and all
 *  intellectual property rights therein.  IF YOU HAVE NO AUTHORIZED LICENSE, THEN YOU
 *  HAVE NO RIGHT TO USE THIS SOFTWARE IN ANY WAY, AND SHOULD IMMEDIATELY
 *  NOTIFY BROADCOM AND DISCONTINUE ALL USE OF THE SOFTWARE.  
 *   
 *  Except as expressly set forth in the Authorized License,
 *   
 *  1.     This program, including its structure, sequence and organization, constitutes the valuable trade
 *  secrets of Broadcom, and you shall use all reasonable efforts to protect the confidentiality thereof,
 *  and to use this information only in connection with your use of Broadcom integrated circuit products.
 *   
 *  2.     TO THE MAXIMUM EXTENT PERMITTED BY LAW, THE SOFTWARE IS PROVIDED "AS IS" 
 *  AND WITH ALL FAULTS AND BROADCOM MAKES NO PROMISES, REPRESENTATIONS OR 
 *  WARRANTIES, EITHER EXPRESS, IMPLIED, STATUTORY, OR OTHERWISE, WITH RESPECT TO 
 *  THE SOFTWARE.  BROADCOM SPECIFICALLY DISCLAIMS ANY AND ALL IMPLIED WARRANTIES 
 *  OF TITLE, MERCHANTABILITY, NONINFRINGEMENT, FITNESS FOR A PARTICULAR PURPOSE, 
 *  LACK OF VIRUSES, ACCURACY OR COMPLETENESS, QUIET ENJOYMENT, QUIET POSSESSION 
 *  OR CORRESPONDENCE TO DESCRIPTION. YOU ASSUME THE ENTIRE RISK ARISING OUT OF 
 *  USE OR PERFORMANCE OF THE SOFTWARE.
 *  
 *  3.     TO THE MAXIMUM EXTENT PERMITTED BY LAW, IN NO EVENT SHALL BROADCOM OR ITS 
 *  LICENSORS BE LIABLE FOR (i) CONSEQUENTIAL, INCIDENTAL, SPECIAL, INDIRECT, OR 
 *  EXEMPLARY DAMAGES WHATSOEVER ARISING OUT OF OR IN ANY WAY RELATING TO YOUR 
 *  USE OF OR INABILITY TO USE THE SOFTWARE EVEN IF BROADCOM HAS BEEN ADVISED OF 
 *  THE POSSIBILITY OF SUCH DAMAGES; OR (ii) ANY AMOUNT IN EXCESS OF THE AMOUNT 
 *  ACTUALLY PAID FOR THE SOFTWARE ITSELF OR U.S. $1, WHICHEVER IS GREATER. THESE 
 *  LIMITATIONS SHALL APPLY NOTWITHSTANDING ANY FAILURE OF ESSENTIAL PURPOSE OF 
 *  ANY LIMITED REMEDY.
 * 
 * $brcm_Workfile: bcmnexus_layer.c $
 * $brcm_Revision: DirectFB_1_4_Port/49 $
 * $brcm_Date: 3/17/11 1:15p $
 *
 * [File Description:]
 *
 * Revision History:
 *
 * $brcm_Log: /AppLibs/opensource/directfb/src/broadcom_files/private/DirectFB/1.4/systems/bcmnexus/bcmnexus_layer.c $
 * 
 * DirectFB_1_4_Port/49   3/17/11 1:15p robertwm
 * SW7420-1683: No display output on sawman multi-app with the latest
 *  direct1.4.5-1.5 in glib.
 * 
 * DirectFB_1_4_Port/48   3/17/11 12:40p robertwm
 * SW7422-359: DirectFB: Add support for A8 Palette8 (DSPF_ALUT8) support
 *  into DirectFB.
 * 
 * DirectFB_1_4_Port/47   3/4/11 5:03p robertwm
 * SW7420-1593: DirectFB-1.4.1: Add ShutdownLayer() API.
 * 
 * DirectFB_1_4_Port/46   3/4/11 3:30p robertwm
 * SW7420-1590: DirectFB-1.4.5: Deadlock when closing down multiple
 *  applications.
 * 
 * DirectFB_1_4_Port/45   3/1/11 12:01p christ
 * SW7420-1545: DirectFB: Application hang when using multi app DirectFB
 *  with SawMan window manager.
 * 
 * DirectFB_1_4_Port/44   2/18/11 11:28a robertwm
 * SW7422-136: Green tint noticed on running any DFB app with pixel format
 *  not containing an alpha component.
 * 
 * DirectFB_1_4_Port/43   2/9/11 4:25p robertwm
 * SW7422-252: DirectFB: Add ABGR pixelformat support to DirectFB.
 * 
 * DirectFB_1_4_Port/42   2/8/11 1:31p robertwm
 * SW7420-1471: DirectFB: Replace fusion skirmishes with pthread
 *  semantics.
 * 
 * DirectFB_1_4_Port/41   2/8/11 1:12p robertwm
 * SW7420-1467: DirectFB: Multiple apps should NOT share NEXUS Display
 *  handle.
 * 
 * DirectFB_1_4_Port/40   1/26/11 1:48p robertwm
 * SW7420-1430: DirectFB: Cannot compile DFB with 7420 Phase 4.1 refsw.
 * 
 * DirectFB_1_4_Port/39   1/24/11 8:54p christ
 * SW7420-1411: DirectFB: Merge of features required for DirectFB 1.4.5
 *  phase 1.5 release - Add some encoder support
 * 
 * DirectFB_1_4_Port/38   1/24/11 4:32p christ
 * 
 * SW7420-1411: DirectFB: Merge of features required for DirectFB 1.4.5
 *  phase 1.5 release - Add some encoder support
 * 
 * DirectFB_1_4_Port/37   1/24/11 3:28p robertwm
 * SW7420-1410: DirectFB: Random hangs using SaWMan-1.4.1 after tearing
 *  patches applied.
 * 
 * DirectFB_1_4_Port/36   1/6/11 5:25p raywu
 * SW7420-1360: DirectFB: Tearing issue for SaWMan flipping
 * 
 * DirectFB_1_4_Port/35   1/3/11 4:39p scchang
 * SW7420-1338: Add A8 and other pixel formats support for FillTrapezoid()
 *  for M2MC core
 * 
 * DirectFB_1_4_Port/34   12/27/10 2:43p scchang
 * SW7550-527: Add dual output support on 7550
 * 
 * DirectFB_1_4_Port/33   12/16/10 6:45p robertwm
 * SW35230-2598: DirectFB: Remove warning messages from bcmnexus_layer.c
 *  due to secondary layer not being present.
 * 
 * DirectFB_1_4_Port/32   12/16/10 6:35p robertwm
 * SW35230-2595: DirectFB: Change default GFD filter co-efficient from
 *  Anisotropic to Bilinear.
 * 
 * DirectFB_1_4_Port/31   12/14/10 1:18p robertwm
 * SW7422-140: DirectFB: Support OpenGL ES 2.0 applications on the VC-4
 *  core.
 * 
 * DirectFB_1_4_Port/30   10/21/10 5:02a robertwm
 * SW7405-4958: DirectFB-1.4.5: Running df_window results in bcmTestRegion
 *  failures.
 * 
 * DirectFB_1_4_Port/29   10/15/10 5:24p robertwm
 * SW7420-1180: DirectFB-1.4.5: Add Layer Shutdown code.
 * 
 * DirectFB_1_4_Port/28   10/14/10 1:01p lig
 * SW7405-4939: sprintf corrupt stack and cause app crashes. change to use
 *  snprintf
 * 
 * DirectFB_1_4_Port/27   10/12/10 2:54p robertwm
 * SW7550-589: DirectFB: Add LUT4 pixel format support to DirectFB.
 * 
 * DirectFB_1_4_Port/26   9/23/10 4:11p robertwm
 * SW7405-4896: DirectFB: Kernel oops after running SaWMan "Penguins"
 *  twice in a row on DirectFB 1.4.0 release.
 * SW7405-4877: DirectFB: Optimise layer handling code when setting the
 *  graphics framebuffer.
 * SW7405-4897: DirectFB: Relaunching testman causes kernel Oops.
 * SW7405-4898: DirectFB: Poor performance when running multiple instances
 *  of df_andi in mirror mode.
 * 
 * DirectFB_1_4_Port/25   9/2/10 2:46p robertwm
 * SW7405-4841: DirectFB: SetPowerMode() causes DFB to hang.
 * 
 * DirectFB_1_4_Port/24   8/17/10 1:21a robertwm
 * SW7405-4764: DirectFB: Color keying on the primary layer doesn't work.
 * 
 * DirectFB_1_4_Port/23   8/16/10 11:19p robertwm
 * SW7405-4758: DirectFB: Seg fault when running dual output apps with the
 *  "force-windowed,autoflip-window" options.
 * 
 * DirectFB_1_4_Port/22   8/11/10 2:00a robertwm
 * SW7405-4250: DirectFB: Screen destination rectangle and source
 *  rectangle not saved across contexts.
 * SW7405-4743: DirectFB: Multiple instances of df_andi.c do not get
 *  displayed.
 * 
 * DirectFB_1_4_Port/21   7/26/10 5:32p jackf
 * SW35230-771: Added include for shim_nexus.h.
 * 
 * DirectFB_1_4_Port/20   6/7/10 4:45p robertwm
 * SW7405-4464: DirectFB: Display blending equations are wrong!
 * 
 * DirectFB_1_4_Port/19   5/27/10 9:20p robertwm
 * SW7420-784: DirectFB: SetPalette on a non-layer surface doesn't work.
 * 
 * DirectFB_1_4_Port/18   4/19/10 4:46p robertwm
 * SW3556-1100: DirectFB: Remove warnings generated when NUM_DISPLAYS is
 *  1.
 * 
 * DirectFB_1_4_Port/17   4/15/10 10:39p robertwm
 * SW7405-4184: Allow independent mode and resolution to be specified.
 * 
 * DirectFB_1_4_Port/16   4/15/10 7:51p robertwm
 * SW7405-4184: Allow independent mode and resolution to be specified.
 *
 ***************************************************************************/

#include "shim_nexus.h" // Nexus function redirection

#include "bcmnexus.h"
#include "bcmnexus_screen.h"
#include "bcmnexus_layer.h"
#include "bcmnexus_utils.h"
#include "bcmnexus_pool.h"
#include "nexus_core_utils.h"
#include "bdbg.h"

#include <core/core.h>
#include <core/system.h>
#include <core/palette.h>
#include <core/layers_internal.h>
#include <core/layer_control.h>
#include <core/screens.h>
#include <fusion/shmalloc.h>
#include <misc/conf.h>
#include <gfx/util.h>

#ifdef __GNUC__
#define UNUSED_ __attribute__ ((unused))
#else
#define UNUSED_
#endif 

D_DEBUG_DOMAIN( bcmnexusLayer,  "bcmNexus/Layer",  "Broadcom NEXUS Layer driver" );
D_DEBUG_DOMAIN( bcmnexusLayerX, "bcmNexus/LayerX", "Broadcom NEXUS Layer driver (eXtra debug)" );

/**********************************************************************************************************************/


/**********************************************************************************************************************/

bcmLayerData *
bcmGetLayerData(DFBDisplayLayerID layerId)
{
    CoreLayer *layer;

    layer = dfb_layer_at(layerId);

    if (layer)
        return layer->layer_data;
    else
        return NULL;
}

static void
frameBufferCallback( void *context, int param )
{
     bcmLayerData *layerData = context;
     pthread_mutex_t *pLock;
     pthread_cond_t  *pSem;

     D_DEBUG_AT( bcmnexusLayerX, "  -> screen %d gfx layer %d framebuffer callback handled\n", layerData->displayId, param);

     pLock = &layerData->frameBufferSemLock;
     pSem  = &layerData->frameBufferSem;
     pthread_mutex_lock( pLock );
     pthread_cond_broadcast( pSem );
     pthread_mutex_unlock( pLock );
}

static void *
frameBufferThread( UNUSED_ DirectThread *thread, void *arg )
{
    DFBBCMNEXUS         *dfb_bcmnexus;
    bcmLayerData        *layerData;
    NEXUS_DisplayHandle  hDisplay;
    NEXUS_SurfaceHandle  hSurface;

    layerData    = arg;
    dfb_bcmnexus = dfb_system_data();

    D_DEBUG_AT( bcmnexusLayer, "%s: screen %d gfx layer %d: Starting...\n", __FUNCTION__, layerData->displayId, layerData->layerId );

    while (!layerData->frameBufferThreadQuit) {
        /* Wait for someone to signal that they want the framebuffer set... */
        pthread_mutex_lock( &layerData->frameBufferThreadLockIn );
        pthread_cond_wait( &layerData->frameBufferThreadSemIn, &layerData->frameBufferThreadLockIn );

        if (layerData->layerEnabled)
        {
            hDisplay = GET_DISPLAY_HND(dfb_bcmnexus, layerData->displayId);
            hSurface = layerData->regionData->bufferLock.handle;
            D_DEBUG_AT( bcmnexusLayer, "%s() screen %d gfx layer %d: Settings Gfx Framebuffer handle=%p...\n",
                        __FUNCTION__, layerData->displayId, layerData->layerId, (void *)hSurface );

            pthread_mutex_lock( &layerData->frameBufferThreadLockOut );
            layerData->frameBufferThreadResult = (NEXUS_Display_SetGraphicsFramebuffer(hDisplay, hSurface) == NEXUS_SUCCESS) ? DFB_OK : DFB_FAILURE;

            /* Signal that the result of setting the framebuffer is not ready to be read. */
            pthread_cond_broadcast( &layerData->frameBufferThreadSemOut );
            pthread_mutex_unlock( &layerData->frameBufferThreadLockOut );
        }

        pthread_mutex_unlock( &layerData->frameBufferThreadLockIn );
    }

    D_DEBUG_AT( bcmnexusLayer, "%s: screen %d gfx layer %d: Exiting...\n", __FUNCTION__, layerData->displayId, layerData->layerId );

    return NULL;
}

static DFBResult
bcmGetLayerGraphicsSettings(DFBBCMNEXUS *dfb_bcmnexus, DFBDisplayLayerID layerId, NEXUS_GraphicsSettings *pGraphicsSettings)
{
    int                    source_height, source_width;
    int                    display_width, display_height;
    int                    canvas_width,  canvas_height;
    bcmLayerData          *layerData;
    CoreLayerRegionConfig *regionConfig;
    DFBScreenID            displayId;
    
    layerData      = bcmGetLayerData(layerId);
    if (layerData == NULL)
        return DFB_FAILURE;

    regionConfig   = (CoreLayerRegionConfig *)&(layerData->regionData->config);
    displayId      = layerData->displayId;

    /* Get Canvas size */
    canvas_width   = GET_DISPLAY_WIDTH(dfb_bcmnexus, displayId);
    canvas_height  = GET_DISPLAY_HEIGHT(dfb_bcmnexus, displayId);

    /* Get Display Rectangle size */
    display_width  = MIN(layerData->destRectangle.w, canvas_width);
    display_height = MIN(layerData->destRectangle.h, canvas_height);

    /* Get Source Rectangle size */
    source_width   = layerData->srcRectangle.w;
    if (layerData->hasVscaler)
        source_height = layerData->srcRectangle.h;
    else
        source_height = display_height;

    NEXUS_Display_GetGraphicsSettings(GET_DISPLAY_HND(dfb_bcmnexus, displayId), pGraphicsSettings);

    pGraphicsSettings->enabled           = layerData->layerEnabled;
    pGraphicsSettings->zorder            = layerData->level + BCMNEXUS_Z_ORDER_MAX/2;
    pGraphicsSettings->position.width    = display_width;
    pGraphicsSettings->position.height   = display_height;
    pGraphicsSettings->position.x        = (layerData->destRectangle.x + display_width  > canvas_width)  ? 0 : layerData->destRectangle.x;
    pGraphicsSettings->position.y        = (layerData->destRectangle.y + display_height > canvas_height) ? 0 : layerData->destRectangle.y;
    pGraphicsSettings->clip.x            = layerData->srcRectangle.x;
    pGraphicsSettings->clip.y            = layerData->srcRectangle.y;
    pGraphicsSettings->clip.width        = source_width;
    pGraphicsSettings->clip.height       = source_height;

    pGraphicsSettings->lowerChromakey    = PIXEL_ARGB( 0x00, regionConfig->src_key.r, regionConfig->src_key.g, regionConfig->src_key.b );
    pGraphicsSettings->upperChromakey    = PIXEL_ARGB( 0xff, regionConfig->src_key.r, regionConfig->src_key.g, regionConfig->src_key.b );
    pGraphicsSettings->chromakeyEnabled  = layerData->srcColorKeyOn;
    pGraphicsSettings->sourceBlendFactor = layerData->srcBlendFunc;
    pGraphicsSettings->destBlendFactor   = layerData->destBlendFunc;
    pGraphicsSettings->alpha             = layerData->alpha;
    pGraphicsSettings->constantAlpha     = layerData->constantAlpha;

    pGraphicsSettings->horizontalFilter  = BCMNEXUS_GFD_SCALE_FILTER_COEFF;

#if BCMNEXUS_DISPLAY_HAS_VERTICAL_FILTER
    pGraphicsSettings->verticalFilter    = BCMNEXUS_GFD_SCALE_FILTER_COEFF;
#endif

    if (layerData->layerEnabled) {
        pGraphicsSettings->frameBufferCallback.callback = layerData->layerCallback;
        pGraphicsSettings->frameBufferCallback.context  = layerData;
        pGraphicsSettings->frameBufferCallback.param    = layerId;
    } else {
        pGraphicsSettings->frameBufferCallback.callback = NULL;
    }

    return DFB_OK;
}

static DFBResult
bcmSetLayerFrameBuffer(UNUSED_ DFBBCMNEXUS *dfb_bcmnexus, DFBDisplayLayerID layerId)
{
    DFBResult            result = DFB_OK;
    bcmLayerData        *layerData;

    layerData = bcmGetLayerData(layerId);
    if (layerData == NULL) {
        result = DFB_FAILURE;
    } else {
        D_DEBUG_AT( bcmnexusLayer, "%s() screen %d gfx layer %d (enabled=%d, master=%c)\n",
                    __FUNCTION__, layerData->displayId, layerId, layerData->layerEnabled, dfb_core_is_master( dfb_bcmnexus_core ) ? 'y' : 'n');

        if (layerData->layerEnabled)
        {
#if 1
            pthread_mutex_lock( &layerData->frameBufferThreadLockOut );

            /* Indicate to our framebuffer thread that the framebuffer needs to be set... */
            pthread_mutex_lock( &layerData->frameBufferThreadLockIn );
            pthread_cond_broadcast( &layerData->frameBufferThreadSemIn );
            pthread_mutex_unlock( &layerData->frameBufferThreadLockIn );

            /* Wait for the result... */
            pthread_cond_wait( &layerData->frameBufferThreadSemOut, &layerData->frameBufferThreadLockOut );
            result = layerData->frameBufferThreadResult;
            pthread_mutex_unlock( &layerData->frameBufferThreadLockOut );
#else
            NEXUS_DisplayHandle  hDisplay;
            NEXUS_SurfaceHandle  hSurface;

            hDisplay = GET_DISPLAY_HND(dfb_bcmnexus, layerData->displayId);
            hSurface = layerData->regionData->bufferLock.handle;

            if (dfb_core_is_master( dfb_bcmnexus_core ))
                result = (NEXUS_Display_SetGraphicsFramebuffer(hDisplay, hSurface) == NEXUS_SUCCESS) ? DFB_OK : DFB_FAILURE;
            else
            {
                bcmLayerCallSetGfxFrameBuffer *pCallArgs;

                pCallArgs = SHMALLOC( dfb_core_shmpool(dfb_bcmnexus_core), sizeof(*pCallArgs) );

                if (!pCallArgs)
                    return D_OOSHM();

                pCallArgs->layerId  = layerId;
                pCallArgs->hSurface = hSurface;

                fusion_call_execute( &GET_CALL_HANDLER(dfb_bcmnexus), FCEF_NONE, BCM_LAYER_SET_GFX_FRAMEBUFFER, pCallArgs, (int *)&result );

                SHFREE( dfb_core_shmpool(dfb_bcmnexus_core), pCallArgs );
            }
#endif
        }
    }
    return result;
}

static DFBResult
bcmSetLayerGraphicsSettings(DFBBCMNEXUS *dfb_bcmnexus, DFBDisplayLayerID layerId)
{
    NEXUS_GraphicsSettings graphicsSettings;
    NEXUS_DisplayHandle    hDisplay;
    bcmLayerData          *layerData;
    DFBResult              result = DFB_OK;

    layerData = bcmGetLayerData(layerId);
    if (layerData == NULL) {
        result = DFB_FAILURE;
    } else {
        hDisplay = GET_DISPLAY_HND(dfb_bcmnexus, layerData->displayId);

        D_DEBUG_AT( bcmnexusLayer, "%s() screen %d gfx layer %d (master=%c)\n",
                __FUNCTION__, layerData->displayId, layerId, dfb_core_is_master( dfb_bcmnexus_core ) ? 'y' : 'n');

        bcmGetLayerGraphicsSettings(dfb_bcmnexus, layerId, &graphicsSettings);

        if (dfb_core_is_master( dfb_bcmnexus_core ))
            result = (NEXUS_Display_SetGraphicsSettings(hDisplay, &graphicsSettings) == NEXUS_SUCCESS) ? DFB_OK : DFB_FAILURE;
        else
        {
            bcmLayerCallSetGfxSettings *pCallArgs;

            pCallArgs = SHMALLOC( dfb_core_shmpool(dfb_bcmnexus_core), sizeof(*pCallArgs) );

            if (!pCallArgs)
                return D_OOSHM();

            pCallArgs->layerId = layerId;
            direct_memcpy( &pCallArgs->gfxSettings, &graphicsSettings, sizeof(graphicsSettings) );

            fusion_call_execute( &GET_CALL_HANDLER(dfb_bcmnexus), FCEF_NONE, BCM_LAYER_SET_GFX_SETTINGS, pCallArgs, (int *)&result );

            SHFREE( dfb_core_shmpool(dfb_bcmnexus_core), pCallArgs );
        }
    }
    return result;
}

DFBResult
bcmSetLayerState(DFBBCMNEXUS *dfb_bcmnexus, DFBDisplayLayerID layerId, bool enable)
{
    DFBResult               result;
    bcmLayerData           *layerData;
    DFBScreenID             displayId = 0xff;

    layerData = bcmGetLayerData(layerId);

    D_DEBUG_AT( bcmnexusLayer, "%s() screen %d gfx layer %d: enable=%d\n", __FUNCTION__, displayId, layerId, enable);

    if (layerData == NULL) {
        result = DFB_FAILURE;
    } else {
        /* Lock access to the layerData */
        pthread_mutex_lock( &layerData->lock );

        displayId = layerData->displayId;

        /* Get whether the screen/display is on */
        layerData->layerEnabled = (bcmGetPowerMode(dfb_screens_at(displayId)) == DSPM_ON) && enable;

        result = bcmSetLayerGraphicsSettings(dfb_bcmnexus, layerId);

        if (result == DFB_OK)
            result = bcmSetLayerFrameBuffer(dfb_bcmnexus, layerId);

        /* Unlock access to the layerData */
        pthread_mutex_unlock( &layerData->lock );
    }
    D_DEBUG_AT( bcmnexusLayer, "%s() screen %d gfx layer %d: Exit [%d].\n", __FUNCTION__, displayId, layerId, result );
    return result;
}

DFBResult
bcmSetLayerDestRectangle(DFBBCMNEXUS *dfb_bcmnexus, DFBDisplayLayerID layerId, DFBRectangle *destRectangle)
{
    DFBResult               result;
    DFBScreenID             displayId = 0xff;
    bcmLayerData           *layerData;
    bcmRegionData          *regionData;
    NEXUS_GraphicsSettings  graphicsSettings;
    
    D_DEBUG_AT( bcmnexusLayer, "%s() screen %d gfx layer %d: rect=%dx%d@%d,%d\n", __FUNCTION__,
                    displayId, layerId, destRectangle->w, destRectangle->h, destRectangle->x, destRectangle->y);

    layerData = bcmGetLayerData(layerId);

    if (layerData == NULL) {
        result = DFB_FAILURE;
    } else {
        /* Lock access to the layerData */
        pthread_mutex_lock( &layerData->lock );

        displayId = layerData->layerId;

        /* Save new destination rectangle settings */
        layerData->destRectangle = *destRectangle;

        bcmGetLayerGraphicsSettings(dfb_bcmnexus, layerId, &graphicsSettings);

        regionData = layerData->regionData;
        regionData->vscale = false;

        if (graphicsSettings.clip.height != layerData->srcRectangle.h)
        {
            regionData->vscale = true;
        }

        result = bcmSetLayerGraphicsSettings(dfb_bcmnexus, layerId);

        if (result == DFB_OK)
            result = bcmSetLayerFrameBuffer(dfb_bcmnexus, layerId);

        /* Unlock access to the layerData */
        pthread_mutex_unlock( &layerData->lock );
    }
    D_DEBUG_AT( bcmnexusLayer, "%s() screen %d gfx layer %d: Exit [%d].\n", __FUNCTION__, displayId, layerId, result );
    return result;
}

static void
bcmGetLayerDefaultSize(DFBBCMNEXUS *dfb_bcmnexus, DFBDisplayLayerID layerId, int *pWidth, int *pHeight)
{
    int width, height;
    DFBScreenID displayId;

    displayId = dfb_screen_id_translated(dfb_layer_screen(dfb_layer_at_translated(layerId)));

    width  = GET_DISPLAY_WIDTH(dfb_bcmnexus,  displayId);
    height = GET_DISPLAY_HEIGHT(dfb_bcmnexus, displayId);

    if (dfb_config->layers[layerId].config.flags & DLCONF_WIDTH)
        width = dfb_config->layers[layerId].config.width;

    if (dfb_config->layers[layerId].config.flags & DLCONF_HEIGHT)
        height = dfb_config->layers[layerId].config.height;

    *pWidth = width;
    *pHeight = height;
}

static void
bcmGetLayerDefaultFormat(DFBDisplayLayerID layerId, DFBSurfacePixelFormat *pFormat)
{
    DFBSurfacePixelFormat pixelFormat = DSPF_ARGB;

    if (dfb_config->layers[layerId].config.flags & DLCONF_PIXELFORMAT)
        pixelFormat = dfb_config->layers[layerId].config.pixelformat;

    *pFormat = pixelFormat;
}

static void
bcmGetLayerDefaultBufferMode(DFBDisplayLayerID layerId, DFBDisplayLayerBufferMode *pMode)
{
    DFBDisplayLayerBufferMode bufferMode = DLBM_FRONTONLY;

    if (dfb_config->layers[layerId].config.flags & DLCONF_BUFFERMODE)
        bufferMode = dfb_config->layers[layerId].config.buffermode;

    *pMode = bufferMode;
}

static void
bcmGetLayerDefaultOptions(DFBDisplayLayerID layerId, DFBDisplayLayerOptions *pOptions)
{
    DFBDisplayLayerOptions options = DLOP_OPACITY | DLOP_ALPHACHANNEL;

    if (dfb_config->layers[layerId].config.flags & DLCONF_OPTIONS)
        options = dfb_config->layers[layerId].config.options;

    *pOptions = options;
}

static void
bcmGetLayerDefaultSurfaceCaps(UNUSED_ DFBDisplayLayerID layerId, DFBSurfaceCapabilities *pCaps)
{
    DFBSurfaceCapabilities caps = DSCAPS_PRIMARY | DSCAPS_VIDEOONLY | DSCAPS_PREMULTIPLIED;

    *pCaps = caps;
}

static void
bcmGetLayerDefaultLevel(UNUSED_ DFBDisplayLayerID layerId, int *pLevel)
{
    *pLevel = 0;
}

static int
bcmGetLayerDefaultUpdateSkipCnt(DFBDisplayLayerID layerId)
{
    return dfb_config->layers[layerId].update_skip_cnt;
}

static int bcmLayerDataSize( void )
{
    return sizeof(bcmLayerData);
}

static int bcmRegionDataSize( void )
{
    return sizeof(bcmRegionData);
}

/********************************************
 * Common functions to setup layers/regions *
 ********************************************/

/* Common function to help initialize the layer data. */
static void bcmInitLayer( DFBBCMNEXUS                *dfb_bcmnexus,
                          bcmLayerData               *layerData,
                          DFBDisplayLayerDescription *description,
                          DFBDisplayLayerConfig      *config)
{
    DFBScreenID         displayId;
    DFBDisplayLayerID   layerId;
    pthread_mutexattr_t mutexAttr;
    pthread_condattr_t  condAttr;
    char                thread_name[24];

    layerId   = layerData->layerId;
    displayId = layerData->displayId;

    D_DEBUG_AT( bcmnexusLayer, "%s() %s display graphics layer %d\n", __FUNCTION__,
                (displayId == PRIMARY_DISPLAY_ID) ? "primary" : "secondary", layerId) ;

    /* Save the graphics layer ID for the display */
    SET_DISPLAY_GFX_LAYER_ID(dfb_bcmnexus, layerId, 0, displayId);

    /* semaphore init */
    pthread_mutexattr_init(&mutexAttr);
    pthread_mutexattr_setpshared(&mutexAttr, PTHREAD_PROCESS_SHARED);
    pthread_condattr_init(&condAttr);
    pthread_condattr_setpshared(&condAttr, PTHREAD_PROCESS_SHARED);

    pthread_mutex_init(&layerData->lock, &mutexAttr);
    pthread_mutex_init(&layerData->frameBufferSemLock, &mutexAttr);
    pthread_cond_init(&layerData->frameBufferSem, &condAttr);

    /* Thread init */
    pthread_mutex_init( &layerData->frameBufferThreadLockIn, &mutexAttr );
    pthread_mutex_init( &layerData->frameBufferThreadLockOut, &mutexAttr );
    pthread_cond_init( &layerData->frameBufferThreadSemIn, &condAttr );
    pthread_cond_init( &layerData->frameBufferThreadSemOut, &condAttr );

    layerData->frameBufferThreadQuit = false;
    snprintf(thread_name, D_ARRAY_SIZE(thread_name), "BRCM FrameBuffer %d", layerId);
    layerData->frameBufferThread = direct_thread_create( DTT_OUTPUT, frameBufferThread, (void *)layerData, thread_name );

    /* Set type and capabilities */
    description->type = DLTF_GRAPHICS;
    description->caps = DLCAPS_SURFACE         | DLCAPS_OPACITY      | DLCAPS_ALPHACHANNEL |
                        DLCAPS_PREMULTIPLIED   | DLCAPS_SRC_COLORKEY | DLCAPS_LEVELS |
                        DLCAPS_SCREEN_POSITION | DLCAPS_SCREEN_SIZE  | DLCAPS_SOURCES;

    /* Set the screen name. */
    snprintf( description->name, DFB_DISPLAY_LAYER_DESC_NAME_LENGTH,
              "BRCM %s disp gfx layer %d", (displayId == PRIMARY_DISPLAY_ID) ? "primary" : "secondary", layerId );

    /* fill out the default configuration */
    config->flags = DLCONF_WIDTH | DLCONF_HEIGHT | DLCONF_PIXELFORMAT | DLCONF_BUFFERMODE | DLCONF_OPTIONS | DLCONF_SURFACE_CAPS | DLCONF_SOURCE;

    /* Set default level, number of regions, sources and clipping regions */
    description->level        = 0;  /* Default level is 0. Negative means below and positive means above */
    description->regions      = 1;  /* We only support a single region per layer context */
    description->sources      = 2;  /* HD and SD graphics sources */
    description->clip_regions = 0;
    
    /* Set the default level for this layer */
    bcmGetLayerDefaultLevel(layerId, &layerData->level);

    /* Set the default size of this layer */
    bcmGetLayerDefaultSize(dfb_bcmnexus, layerId, &config->width, &config->height);

    /* Set the default pixel format for this layer */
    bcmGetLayerDefaultFormat(layerId, &config->pixelformat);
    
    /* Set the default buffer mode for this layer */
    bcmGetLayerDefaultBufferMode(layerId, &config->buffermode);

    /* Set the default options for this layer */
    bcmGetLayerDefaultOptions(layerId, &config->options);

    /* Set the default surface capabilities for this layer */
    bcmGetLayerDefaultSurfaceCaps(layerId, &config->surface_caps);

    /* Set layer callback... */
    layerData->layerCallback = frameBufferCallback;

    D_DEBUG_AT( bcmnexusLayer, "%s() screen %d gfx layer %d: flags=0x%08x, %dx%d, format=%s, bm=%d, opt=0x%08x, surf.caps=0x%08x\n", __FUNCTION__, 
            displayId, layerId, config->flags,
            config->width, config->height, dfb_pixelformat_name(config->pixelformat),
            config->buffermode, config->options, config->surface_caps );

    return;
}

static DFBResult
bcmShutdownLayer( UNUSED_ CoreLayer *layer,
                  UNUSED_ void      *driver_data,
                          void      *layer_data )
{
    bcmLayerData *layerData = layer_data;

    D_DEBUG_AT( bcmnexusLayer, "%s() screen %d gfx layer %d\n", __FUNCTION__, layerData->displayId, layerData->layerId );

    layerData->frameBufferThreadQuit = true;
    pthread_cond_signal( &layerData->frameBufferThreadSemOut );
    pthread_cond_signal( &layerData->frameBufferThreadSemIn );
    direct_thread_join( layerData->frameBufferThread );
    direct_thread_destroy( layerData->frameBufferThread );

    pthread_mutex_destroy( &layerData->frameBufferThreadLockOut );
    pthread_mutex_destroy( &layerData->frameBufferThreadLockIn );
    pthread_cond_destroy( &layerData->frameBufferThreadSemOut );
    pthread_cond_destroy( &layerData->frameBufferThreadSemIn );

    pthread_mutex_destroy( &layerData->lock );
    pthread_mutex_destroy( &layerData->frameBufferSemLock );
    pthread_cond_destroy( &layerData->frameBufferSem );

    return DFB_OK;
}

/* Common function to test whether the region can be modified to these settings. */
static DFBResult
bcmTestRegion( UNUSED_ CoreLayer                  *layer,
                       void                       *driver_data,
                       void                       *layer_data,
                       CoreLayerRegionConfig      *config,
                       CoreLayerRegionConfigFlags *failed )
{
    DFBBCMNEXUS                *dfb_bcmnexus = driver_data;
    bcmLayerData               *layerData    = layer_data;
    CoreLayerRegionConfigFlags  fail         = CLRCF_NONE;
    int                         source_height, source_width;
    int                         display_width, display_height;
    DFBScreenID                 displayId;
    DFBDisplayLayerID           layerId;

    D_ASSERT( dfb_bcmnexus != NULL );

    displayId  = layerData->displayId;
    layerId    = layerData->layerId;

    D_DEBUG_AT( bcmnexusLayer, "%s() screen %d gfx layer %d (%dx%d) format=%s, caps=0x%08x, bm=%d,\n"
                               "options=0x%08x, source_id=%d, opacity=%d [%dx%d@%d,%d]=>[%dx%d@%d,%d]\n",
                               __FUNCTION__, displayId, layerId, config->width, config->height, dfb_pixelformat_name(config->format),
                               config->surface_caps, config->buffermode, config->options, config->source_id, config->opacity,
                               config->source.w, config->source.h, config->source.x, config->source.y,
                               config->dest.w, config->dest.h, config->dest.x, config->dest.y);

    /* Get Display Rectangle size */
    display_width  = MIN(config->dest.w, GET_DISPLAY_WIDTH(dfb_bcmnexus, displayId));
    display_height = MIN(config->dest.h, GET_DISPLAY_HEIGHT(dfb_bcmnexus, displayId));

    /* Get Source Rectangle size */
    source_width = config->source.w;

    /* Lock access to the layerData */
    pthread_mutex_lock( &layerData->lock );

    if (layerData->hasVscaler)
        source_height = config->source.h;
    else
        source_height = display_height;

    /* Unlock access to the layerData */
    pthread_mutex_unlock( &layerData->lock );

#if (DIRECTFB_MAJOR_VERSION >= 1) && (DIRECTFB_MINOR_VERSION >= 4) && (DIRECTFB_MICRO_VERSION >= 3)
#else
    /* The horizontal/vertical GFD scaler can upscale to a max factor of 4 and can be used to downscale to max factor of 6.  However,
       downscaling requires additional memory bandwidth, so RTS settings may need to be tweaked. */
    if ((display_width > source_width * 4) || (display_width * 6 < source_width))
         fail |= CLRCF_WIDTH;

    /* Check height */
    if ((display_height > source_height * 4) || (display_height * 6 < source_height))
         fail |= CLRCF_HEIGHT;
#endif

    /* Check format */
    switch (config->format) {
          case DSPF_A8:
          case DSPF_LUT2:
          case DSPF_LUT4:
          case DSPF_LUT8:
          case DSPF_ALUT8:
#ifdef BCMNEXUS_PIXELFORMAT_X4_R4_G4_B4_SUPPORTED        
          case DSPF_RGB444:
#endif
#ifdef BCMNEXUS_PIXELFORMAT_X1_R5_G5_B5_SUPPORTED
          case DSPF_RGB555:
#endif
#ifdef BCMNEXUS_PIXELFORMAT_X1_B5_G5_R5_SUPPORTED
          case DSPF_BGR555:
#endif
          case DSPF_RGBA4444:
          case DSPF_ARGB4444:
          case DSPF_ARGB1555:
          case DSPF_RGB16:
#ifdef BCMNEXUS_PIXELFORMAT_R8_G8_B8_SUPPORTED
          case DSPF_RGB24:
#endif
          case DSPF_RGB32:
          case DSPF_ARGB:
#if defined(BCMNEXUS_PIXELFORMAT_A8_B8_G8_R8_SUPPORTED) && defined(PIXEL_ABGR)
          case DSPF_ABGR:
#endif
          case DSPF_YUY2:
          case DSPF_UYVY:
          case DSPF_AYUV:
               break;

          default:
               fail |= CLRCF_FORMAT;
    }

    /* Check options */
    if (!D_FLAGS_ARE_IN( config->options, DLOP_OPACITY | DLOP_ALPHACHANNEL | DLOP_SRC_COLORKEY ))
         fail |= CLRCF_OPTIONS;

    if (failed)
         *failed = fail;

    if (fail)
    {
         D_ERROR("bcmNexus/Layer: %s() screen %d gfx layer %d: Failed (flags = 0x%08x)!\n", __FUNCTION__, displayId, layerId, fail);
         return DFB_UNSUPPORTED;
    }

    D_DEBUG_AT( bcmnexusLayer, "%s() screen %d gfx layer %d: Exit.\n", __FUNCTION__, displayId, layerId );
    return DFB_OK; 
}

/* Common function to add a region to a layer */
static DFBResult
bcmAddRegion( CoreLayer             *layer,
              bcmLayerData          *layerData,
              bcmRegionData         *regionData,
              CoreLayerRegionConfig *config,
              int                    width,
              int                    height)
{
    DFBResult         res = DFB_OK;
    CoreSurface      *tmp_surface;
    DFBScreenID       displayId;
    DFBDisplayLayerID layerId;

    displayId = layerData->displayId;
    layerId   = layerData->layerId;

    D_DEBUG_AT( bcmnexusLayer, "%s() screen %d gfx layer %d: %dx%d, format=%s, region=%dx%d, caps=0x%08x,\n"
                               "bm=%d, options=0x%08x, source_id=%d, opacity=%d [%dx%d@%d,%d] -> [%dx%d@%d,%d]\n",
                               __FUNCTION__, displayId, layerId, width, height, dfb_pixelformat_name(config->format),
                               config->width, config->height, config->surface_caps, config->buffermode,
                               config->options, config->source_id, config->opacity,
                               config->source.w, config->source.h, config->source.x, config->source.y,
                               config->dest.w, config->dest.h, config->dest.x, config->dest.y);

    /* Lock access to the layerData */
    pthread_mutex_lock( &layerData->lock );

    /* Save current region pointer in layer data */
    layerData->regionData = regionData;

    /* Ensure the update skip count is 0 initially */
    layerData->updateSkipCnt = 0;

    /* Store new region configuration */
    regionData->config = *config;

    /* Save Layer's source screen/display rectangle if not already set */
    if (layerData->srcRectangle.w == 0)
        layerData->srcRectangle = config->source;

    /* Save Layer's destination screen/display rectangle if not already set */
    if (layerData->destRectangle.w == 0)
        layerData->destRectangle = config->dest;

    /* If graphics feeder doesn't have a vertical scaler, then we need to create
       a temporary framebuffer to allow vertical scaling to occur */
    if (!layerData->hasVscaler)
    {
        res = dfb_surface_create_simple( layer->core,
                                         width,
                                         height,
                                         config->format, DSCAPS_VIDEOONLY, CSTF_SHARED, layerId, NULL, &tmp_surface );
        if (res == DFB_OK)
        {
            D_DEBUG_AT( bcmnexusLayer, "%s() screen %d gfx layer %d: Created framebuffer %p: %dx%d (format=%s).\n", __FUNCTION__,
                        displayId, layerId, (void *)tmp_surface, width, height, dfb_pixelformat_name(config->format) );

            layerData->frameBuffer = tmp_surface;

            /* Make the surface global for all processes to access the reference count */
            res = dfb_surface_globalize( layerData->frameBuffer );
        }
    }
    /* Unlock access to the layerData */
    pthread_mutex_unlock( &layerData->lock );

    D_ASSERT( res == DFB_OK );

    D_DEBUG_AT( bcmnexusLayer, "%s() screen %d gfx layer %d: Exit [%d].\n", __FUNCTION__, displayId, layerId, res );
    return res;
}

/* Common function to set the primary region to the new settings. */
static DFBResult
bcmSetRegion( UNUSED_ CoreLayer                  *layer,
                      void                       *driver_data,
                      void                       *layer_data,
                      void                       *region_data,
                      CoreLayerRegionConfig      *config,
                      CoreLayerRegionConfigFlags  updated,
                      CoreSurface                *surface,
                      CorePalette                *palette,
                      CoreSurfaceBufferLock      *lock )
{
    DFBResult              res;
    DFBBCMNEXUS           *dfb_bcmnexus = driver_data;
    bcmLayerData          *layerData    = layer_data;
    bcmRegionData         *regionData   = region_data;
    CoreSurfaceBufferLock  frameBufferLock;
    CoreLayerRegionConfig *regionConfig;
    NEXUS_SurfaceHandle    hFrameBuffer;
    NEXUS_GraphicsSettings graphicsSettings;
    DFBScreenID            displayId;
    DFBDisplayLayerID      layerId;
    int                    source_width, source_height;
    bool                   reenable_gfx;
    bool                   displayOn;

    D_ASSERT( dfb_bcmnexus != NULL );

    /* Lock access to the layerData */
    pthread_mutex_lock( &layerData->lock );

    displayId    = layerData->displayId;
    layerId      = layerData->layerId;
    regionConfig = &regionData->config;

    D_DEBUG_AT( bcmnexusLayer, "%s() screen %d gfx layer %d: updated flags=0x%08x, surface=%p,\n"
                               "handle=%p, format=%s, region=%dx%d, caps=0x%08x, bm=%d,\n"
                               "options=0x%08x, source_id=%d, opacity=%d [%dx%d@%d,%d] -> [%dx%d@%d,%d]\n",
                               __FUNCTION__, displayId, layerId, updated, (void *)surface, lock->handle, dfb_pixelformat_name(config->format),
                               config->width, config->height, config->surface_caps, config->buffermode,
                               config->options, config->source_id, config->opacity,
                               config->source.w, config->source.h, config->source.x, config->source.y,
                               config->dest.w, config->dest.h, config->dest.x, config->dest.y);

    regionData->bufferLock = *lock;
    hFrameBuffer = lock->handle;

    if (D_FLAGS_IS_SET(updated, CLRCF_PALETTE) && palette) {
        /* iterate through all connected buffers */
        int i;

        /* Lock the surface prior to modifying the palette */
        if (dfb_surface_lock( surface )) {
            /* Unlock access to the layerData */
            pthread_mutex_unlock( &layerData->lock );
            return DFB_FUSION;
        }

        for( i=0; i<surface->num_buffers; i++ ) {
            CoreSurfaceBuffer     *buffer = surface->buffers[i];
            CoreSurfaceBufferLock  lock;
            res = dfb_surface_buffer_lock( buffer, CSAID_CPU, CSAF_WRITE | CSAF_READ, &lock );
            if ( res == DFB_OK ) {
                copy_palette( surface, lock.handle );
                dfb_surface_buffer_unlock( &lock );
            }
            else
                fprintf(stderr,"FIX THIS!!!\n");
        }
        dfb_surface_unlock( surface );
    }

    if (!layerData->layerCallback) {
        layerData->layerCallback = frameBufferCallback;
    }

    if (!layerData->frameBuffer) {
        layerData->frameBuffer = surface;
    }

    /* If all flags are set, then don't update anything as it will
       have already been updated in AddRegion */
#if (DIRECTFB_MAJOR_VERSION >= 1) && (DIRECTFB_MINOR_VERSION >= 4) && (DIRECTFB_MICRO_VERSION >= 3)
    if (true)
#else
    if (!D_FLAGS_ARE_SET(updated, CLRCF_ALL))
#endif
    {
        /* Save Region Configuration */
        if (D_FLAGS_IS_SET(updated, CLRCF_WIDTH))
            regionConfig->width = config->width;

        if (D_FLAGS_IS_SET(updated, CLRCF_HEIGHT))
            regionConfig->height = config->height;

        if (D_FLAGS_IS_SET(updated, CLRCF_FORMAT))
            regionConfig->format = config->format;

        if (D_FLAGS_IS_SET(updated, CLRCF_SURFACE_CAPS))
            regionConfig->surface_caps = config->surface_caps;

        if (D_FLAGS_IS_SET(updated, CLRCF_BUFFERMODE))
            regionConfig->buffermode = config->buffermode;

        if (D_FLAGS_IS_SET(updated, CLRCF_OPTIONS))
            regionConfig->options = config->options;

        if (D_FLAGS_IS_SET(updated, CLRCF_SOURCE_ID))
            regionConfig->source_id = config->source_id;

        if (D_FLAGS_IS_SET(updated, CLRCF_SOURCE)) {
            regionConfig->source = config->source;
            layerData->srcRectangle = config->source;
        }

        if (D_FLAGS_IS_SET(updated, CLRCF_DEST)) {
            regionConfig->dest = config->dest;
            layerData->destRectangle = config->dest;
        }

        if (D_FLAGS_IS_SET(updated, CLRCF_CLIPS)) {
            regionConfig->clips     = config->clips;
            regionConfig->num_clips = config->num_clips;
            regionConfig->positive  = config->positive;
        }

        if (D_FLAGS_IS_SET(updated, CLRCF_OPACITY))
            regionConfig->opacity = config->opacity;

        if (D_FLAGS_IS_SET(updated, CLRCF_ALPHA_RAMP)) {
            regionConfig->alpha_ramp[0] = config->alpha_ramp[0];
            regionConfig->alpha_ramp[1] = config->alpha_ramp[1];
            regionConfig->alpha_ramp[2] = config->alpha_ramp[2];
            regionConfig->alpha_ramp[3] = config->alpha_ramp[3];
        }

        if (D_FLAGS_IS_SET(updated, CLRCF_SRCKEY))
            regionConfig->src_key = config->src_key;

        if (D_FLAGS_IS_SET(updated, CLRCF_DSTKEY))
            regionConfig->dst_key = config->dst_key;

        if (D_FLAGS_IS_SET(updated, CLRCF_PARITY))
            regionConfig->parity = config->parity;
    }

    /* Setup Source Configuration */
    layerData->sourceId = config->source_id;

    /* Get whether the screen/display is on */
    displayOn = (bcmGetPowerMode(dfb_screens_at(displayId)) == DSPM_ON);

    /* Check is we need to re-enable the graphics layer */
    reenable_gfx = ((layerData->layerEnabled == false) && displayOn);

    layerData->layerEnabled  = displayOn;
    layerData->srcColorKeyOn = D_FLAGS_IS_SET(regionConfig->options, DLOP_SRC_COLORKEY);

    /* Case 1: Per-pixel alpha blending enabled on a non-premultiplied source surface.
               In this case, we need to use the following blending equation:
               Oc = (Sc * Sa) + (1 - Sa) * Dc

       Case 2: Per-pixel alpha blending enabled on a pre-multiplied source surface.
               In this case, we don't need to multiply the Sc by the Sa as this has
               already been completed.  The Sa needs to be set to 1 by using the constant alpha
               value of 0xff:
               Oc = (Sc * Ca) + (1 - Sa) * Dc
               Ca = 0xff
       Case 3: Global alpha blending enabled on a source surface.
               In this case, we need to use the constant alpha as the "global" alpha value:
               Oc = (Sc * Ca) + (1 - Ca) * Dc
       Case 4: Both per-pixel alpha blending and global alpha value are set on a non-premultiplied source surface.
               In this case, we can use the standard blending equation as well as the independent alpha value:
               Oc = (Sc * Sa) + (1 - Sa) * Dc
       Case 5: Both per-pixel alpha blending and global alpha value are set on a premultiplied source surface.
               In this case, we can use the independent alpha value along with constant source alpha of 0xff:
               Oc = (Sc * Ca) + (1 - Sa) * Dc
               Ca = 0xff
    */

    layerData->srcBlendFunc             = NEXUS_CompositorBlendFactor_eOne;
    layerData->destBlendFunc            = NEXUS_CompositorBlendFactor_eZero;
    layerData->alpha                    = 0xFF;
    layerData->constantAlpha            = 0xFF;

    if (regionConfig->options & DLOP_ALPHACHANNEL) {
        layerData->srcBlendFunc         = NEXUS_CompositorBlendFactor_eSourceAlpha;
        layerData->destBlendFunc        = NEXUS_CompositorBlendFactor_eInverseSourceAlpha;

        if (surface->config.caps & DSCAPS_PREMULTIPLIED)
        {
            layerData->srcBlendFunc     = NEXUS_CompositorBlendFactor_eConstantAlpha;
        }

        if (!DFB_PIXELFORMAT_HAS_ALPHA(surface->config.format))
        {
            layerData->srcBlendFunc     = NEXUS_CompositorBlendFactor_eConstantAlpha;

            /* There is a bug in the VDC on some chips where setting the dest blend func to (1-Ca) causes
               a green tint.  The temporary solution is to set this to (1-Sa) if the constant alpha
               value is 0xFF. */
            layerData->destBlendFunc    = NEXUS_CompositorBlendFactor_eInverseSourceAlpha;
        }
        layerData->alpha = (regionConfig->options & DLOP_OPACITY) ? regionConfig->opacity : 0xff;
    }
    else if (regionConfig->options & DLOP_OPACITY) {
        layerData->constantAlpha        = regionConfig->opacity;
        layerData->srcBlendFunc         = NEXUS_CompositorBlendFactor_eConstantAlpha;
        /* There is a bug in the VDC on some chips where setting the dest blend func to (1-Ca) causes
           a green tint.  The temporary solution is to set this to (1-Sa) if the constant alpha
           value is 0xFF. */
        if (layerData->constantAlpha == 0xff)
            layerData->destBlendFunc    = NEXUS_CompositorBlendFactor_eInverseSourceAlpha;
        else
            layerData->destBlendFunc    = NEXUS_CompositorBlendFactor_eInverseConstantAlpha;
    }

    /* Color keying doesn't work if the blending equations use constants
       e.g. Sa=Ca, Da=(1-Ca)  or Sa=1, Da=0 */
    if (layerData->srcColorKeyOn) {
        layerData->srcBlendFunc         = NEXUS_CompositorBlendFactor_eSourceAlpha;
        layerData->destBlendFunc        = NEXUS_CompositorBlendFactor_eInverseSourceAlpha;

        /* Ensure the opacity can still be changed, as this uses the alpha value */
        if (layerData->constantAlpha != 0xFF)
            layerData->alpha = layerData->constantAlpha;
    }

    /* Now get the updated graphics settings */
    bcmGetLayerGraphicsSettings(dfb_bcmnexus, layerId, &graphicsSettings);

    source_width  = graphicsSettings.clip.width;
    source_height = graphicsSettings.clip.height;

    D_DEBUG_AT( bcmnexusLayer, "%s() screen %d gfx layer %d: Requested: %dx%d: src:[%dx%d@%d,%d] => dst:[%dx%d@%d,%d], zorder=%d\n", __FUNCTION__,
                displayId, layerId,
                regionConfig->width, regionConfig->height,
                regionConfig->source.w, regionConfig->source.h, regionConfig->source.x, regionConfig->source.y,
                regionConfig->dest.w, regionConfig->dest.h, regionConfig->dest.x, regionConfig->dest.y,
                graphicsSettings.zorder );

    D_DEBUG_AT( bcmnexusLayer, "%s() screen %d gfx layer %d: Actual:    %dx%d: src:[%dx%d@%d,%d] => dst:[%dx%d@%d,%d], zorder=%d\n", __FUNCTION__,
                displayId, layerId,
                GET_DISPLAY_WIDTH(dfb_bcmnexus, displayId), GET_DISPLAY_HEIGHT(dfb_bcmnexus, displayId),
                source_width, source_height, graphicsSettings.clip.x, graphicsSettings.clip.y,
                graphicsSettings.position.width, graphicsSettings.position.height, graphicsSettings.position.x, graphicsSettings.position.y,
                graphicsSettings.zorder );

    if (layerData->srcColorKeyOn)
        D_DEBUG_AT( bcmnexusLayer, "%s() screen %d gfx layer %d: ColorKey[RGB]=0x%02x,0x%02x,0x%02x\n", __FUNCTION__,
                    displayId, layerId, regionConfig->src_key.r, regionConfig->src_key.g, regionConfig->src_key.b);

    D_DEBUG_AT( bcmnexusLayer, "%s() screen %d gfx layer %d: sourceBlendFactor=%d,destBlendFactor=%d,alpha=%d,constantAlpha=%d\n", __FUNCTION__,
                displayId, layerId, graphicsSettings.sourceBlendFactor, graphicsSettings.destBlendFactor,
                graphicsSettings.alpha, graphicsSettings.constantAlpha);

    regionData->vscale = false;
    if (graphicsSettings.clip.height != layerData->srcRectangle.h)
    {
        DFBRectangle dest_rect;

        D_DEBUG_AT( bcmnexusLayer, "%s() screen %d gfx layer %d: using blitter to vscale layer\n", __FUNCTION__, displayId, layerId);
        regionData->vscale = true;

        /* Copy the source surface to the layer's framebuffer */
        D_DEBUG_AT( bcmnexusLayer, "%s() screen %d gfx layer %d: Going to flush %p...\n", __FUNCTION__, displayId, layerId, (void *)hFrameBuffer );
        NEXUS_Surface_Flush( hFrameBuffer );

        if (dfb_surface_lock( layerData->frameBuffer )) {
            /* Unlock access to the layerData */
            pthread_mutex_unlock( &layerData->lock );
            return DFB_FUSION;
        }

        /* We use the M2MC to scale only the vertical and the GFD for the horizontal */
        dest_rect.x = 0;
        dest_rect.y = 0;
        dest_rect.w = source_width;
        dest_rect.h = source_height;

        dfb_gfx_stretch_to( surface, layerData->frameBuffer, &dest_rect, false);
        dfb_gfxcard_sync();

        /* Need to instantiate a real surface buffer for the framebuffer */
        res = dfb_surface_lock_buffer( layerData->frameBuffer, CSBR_FRONT, CSAID_GPU, CSAF_READ, &frameBufferLock );
        if (res)
        {
            D_ERROR( "%s() screen %d gfx layer %d: Could NOT lock framebuffer surface! (res=%d)\n", __FUNCTION__, displayId, layerId, res);
            dfb_surface_unlock( layerData->frameBuffer );
            /* Unlock access to the layerData */
            pthread_mutex_unlock( &layerData->lock );
            return res;
        }
        regionData->bufferLock = frameBufferLock;
        hFrameBuffer = frameBufferLock.handle;
    }

    /* Ensure the surface is flushed prior to the graphics feeder reading it. */
    D_DEBUG_AT( bcmnexusLayer, "%s() screen %d gfx layer %d: Going to flush %p...\n", __FUNCTION__, displayId, layerId, (void *)hFrameBuffer );
    NEXUS_Surface_Flush( hFrameBuffer );

    /* If the graphics was disabled, then we need to call NEXUS_Display_SetGraphicsSettings first before calling
       NEXUS_Display_SetGraphicsFramebuffer.  If not, then we need to make sure we do this in the opposite order
       to prevent a stale surface buffer being used in the call to NEXUS_Display_SetGraphicsSettings! */
    if (reenable_gfx == true)
    {
        bcmSetLayerGraphicsSettings(dfb_bcmnexus, layerId);
        bcmSetLayerFrameBuffer(dfb_bcmnexus, layerId);
    } else {
        bcmSetLayerFrameBuffer(dfb_bcmnexus, layerId);
        bcmSetLayerGraphicsSettings(dfb_bcmnexus, layerId);
    }

    if (regionData->vscale == true)
    {
        dfb_surface_unlock_buffer(layerData->frameBuffer, &frameBufferLock);

        if (dfb_surface_unlock( layerData->frameBuffer )) {
            /* Unlock access to the layerData */
            pthread_mutex_unlock( &layerData->lock );
            return DFB_FUSION;
        }
    }

    /* Unlock access to the layerData */
    pthread_mutex_unlock( &layerData->lock );
    
    D_DEBUG_AT( bcmnexusLayer, "%s() screen %d gfx layer %d: Exit.\n", __FUNCTION__, displayId, layerId );
    return DFB_OK;
}

/* Common function to remove a region from a layer */
static DFBResult
bcmRemoveRegion( UNUSED_ CoreLayer *layer,
                         void      *driver_data,
                         void      *layer_data,
                 UNUSED_ void      *region_data )
{
    DFBBCMNEXUS            *dfb_bcmnexus = driver_data;
    bcmLayerData           *layerData    = layer_data;
    DFBDisplayLayerID       layerId;
    DFBScreenID             displayId;

    D_ASSERT( dfb_bcmnexus != NULL );

    layerId      = layerData->layerId;
    displayId    = layerData->displayId;

    D_DEBUG_AT( bcmnexusLayer, "%s() screen %d gfx layer %d\n", __FUNCTION__, displayId, layerId );

    /* Lock access to the layerData */
    pthread_mutex_lock( &layerData->lock );

    layerData->layerEnabled = false;
    bcmSetLayerGraphicsSettings(dfb_bcmnexus, layerId);

    if (!layerData->hasVscaler)
    {
        D_DEBUG_AT( bcmnexusLayer, "%s() screen %d gfx layer %d: Removing framebuffer %p.\n", __FUNCTION__,
                    displayId, layerId, (void *)layerData->frameBuffer);
        
        /* Decrement the internal reference count and if 0, then really destroy the surface */
        dfb_surface_unlink( &layerData->frameBuffer );
    }
    layerData->frameBuffer = NULL;
    layerData->regionData  = NULL;

    /* Unlock access to the layerData */
    pthread_mutex_unlock( &layerData->lock );

    return DFB_OK;
}

/* Common function to invalidate a region */
static DFBResult
bcmInvalidateRegion( DFBBCMNEXUS           *dfb_bcmnexus,
                     bcmLayerData          *layerData,
                     bcmRegionData         *regionData,
                     CoreSurface           *surface,
                     DFBSurfaceFlipFlags    flags,
                     CoreSurfaceBufferLock *lock)
{
    DFBResult              res = DFB_OK;
    DFBDisplayLayerID      layerId;
    DFBScreenID            displayId;
    CoreSurfaceBufferLock  frameBufferLock;
    CoreSurfaceBufferLock *pFrameBufferLock;
    DFBRectangle           dest_rect;
    bool                   vscale;

    D_ASSERT( dfb_bcmnexus != NULL );

    layerId   = layerData->layerId;
    displayId = layerData->displayId;

    pFrameBufferLock = lock;

    vscale = regionData->vscale;

    if ((vscale == true) && layerData->frameBuffer)
    {
        D_DEBUG_AT( bcmnexusLayerX, "%s() screen %d gfx layer %d: using blitter to vscale layer\n", __FUNCTION__, displayId, layerId);
        if (dfb_surface_lock( layerData->frameBuffer )) {
            return DFB_FUSION;
        }

        /* We use the M2MC to scale only the vertical and the GFD for the horizontal */
        dest_rect.x = 0;
        dest_rect.y = 0;
        dest_rect.w = layerData->srcRectangle.w;
        dest_rect.h = MIN(layerData->destRectangle.h, GET_DISPLAY_HEIGHT(dfb_bcmnexus, displayId));

        dfb_gfx_stretch_to( surface, layerData->frameBuffer, &dest_rect, false);
        dfb_gfxcard_sync();

        pFrameBufferLock = &frameBufferLock;
        res = dfb_surface_lock_buffer( layerData->frameBuffer, CSBR_FRONT, CSAID_GPU, CSAF_READ, pFrameBufferLock );
        if (res)
        {
            D_ERROR( "%s() screen %d gfx layer %d: Could NOT lock framebuffer surface! (res=%d)\n", __FUNCTION__, displayId, layerId, res);
            dfb_surface_unlock( layerData->frameBuffer );
            return res;
        }
    }

    /* Only Set the Graphics Framebuffer if the layer is enabled and the framebuffer has changed. */
    if (layerData->layerEnabled && (pFrameBufferLock->handle != regionData->bufferLock.handle))
    {
        regionData->bufferLock = *pFrameBufferLock;
        if (flags & DSFLIP_WAIT) {
            D_DEBUG_AT( bcmnexusLayerX, "%s() screen %d gfx layer %d: waiting for framebuffer %p callback...\n", __FUNCTION__,
                    displayId, layerId, (void *)layerData->frameBuffer);
            pthread_mutex_lock( &layerData->frameBufferSemLock );
        }

        bcmSetLayerFrameBuffer(dfb_bcmnexus, layerId);

        if (flags & DSFLIP_WAIT) {
            pthread_cond_wait( &layerData->frameBufferSem, &layerData->frameBufferSemLock );
            pthread_mutex_unlock( &layerData->frameBufferSemLock );

            D_DEBUG_AT( bcmnexusLayerX, "%s() screen %d gfx layer %d: received callback for framebuffer %p.\n", __FUNCTION__,
                    displayId, layerId, (void *)layerData->frameBuffer);
        }
        /* Inform DirectFB that the buffer has been displayed */
        dfb_surface_notify_display( layerData->frameBuffer, pFrameBufferLock->buffer );
    }

    if ((vscale == true) && layerData->frameBuffer)
    {
        D_DEBUG_AT( bcmnexusLayerX, "%s() screen %d gfx layer %d: unlocking framebuffer %p\n", __FUNCTION__,
                    displayId, layerId, (void *)layerData->frameBuffer);

        dfb_surface_unlock_buffer(layerData->frameBuffer, pFrameBufferLock);

        if (dfb_surface_unlock( layerData->frameBuffer )) {
            return DFB_FUSION;
        }
    }
    return res;
}

/* Common function to flip a region */
static DFBResult
bcmFlipRegion( DFBBCMNEXUS           *dfb_bcmnexus,
               bcmLayerData          *layerData,
               bcmRegionData         *regionData,
               CoreSurface           *surface,
               DFBSurfaceFlipFlags    flags,
               CoreSurfaceBufferLock *lock)
{
    DFBResult res;

    res = bcmInvalidateRegion(dfb_bcmnexus, layerData, regionData, surface, flags, lock);

    /* Inform DirectFB that the buffers have changed function */
    dfb_surface_flip( surface, false );

    return res;
}

/* Common function to initialise the source for each layer */
static DFBResult
bcmInitSource( UNUSED_ CoreLayer                         *layer,
               UNUSED_ void                              *driver_data,
               UNUSED_ void                              *layer_data,
                       int                                source,
                       DFBDisplayLayerSourceDescription  *description )
{
    description->source_id = DLSID_SURFACE + source;
    sprintf(description->name, "BCMNEXUS Gfx Source %d", description->source_id);
    description->caps = DDLSCAPS_SURFACE;
    return DFB_OK;
}

static DFBResult
bcmGetLevel( UNUSED_ CoreLayer *layer,
             UNUSED_ void      *driver_data,
                     void      *layer_data,
                     int       *level )
{
    bcmLayerData *layerData = layer_data;

    /* Lock access to the layerData */
    pthread_mutex_lock( &layerData->lock );

    *level = layerData->level;

    /* Unlock access to the layerData */
    pthread_mutex_unlock( &layerData->lock );

    D_DEBUG_AT(bcmnexusLayer, "%s() screen %d gfx layer %d: level=%d\n", __FUNCTION__, layerData->displayId, layerData->layerId, *level);
    return DFB_OK;
}

/* Set the z-order of this layer.
   0 is the default layer 
   +ve are higher than this layer
   -ve are lower than this layer

   Nexus levels go from 0 (lowest) to BCMNEXUS_Z_ORDER_MAX (highest)
   Default level is 2 for graphics (which is 0 level for DFB)
*/
static DFBResult
bcmSetLevel( UNUSED_ CoreLayer *layer,
                     void      *driver_data,
                     void      *layer_data,
                     int        level )
{
    DFBResult              res          = DFB_INVARG;
    DFBBCMNEXUS           *dfb_bcmnexus = driver_data;
    bcmLayerData          *layerData    = layer_data;
    DFBDisplayLayerID      layerId;
    DFBScreenID            displayId;

    layerId   = layerData->layerId;
    displayId = layerData->displayId;
    
    D_DEBUG_AT(bcmnexusLayer, "%s() screen %d gfx layer %d: level=%d\n", __FUNCTION__, displayId, layerId, level);

    if ((level <= BCMNEXUS_Z_ORDER_MAX/2) && (level >= -BCMNEXUS_Z_ORDER_MAX/2))
    {
        /* Lock access to the layerData */
        pthread_mutex_lock( &layerData->lock );

        layerData->level = level;
        res = bcmSetLayerGraphicsSettings(dfb_bcmnexus, layerId);
        if (res != DFB_OK)
            D_ERROR("%s() screen %d gfx layer %d: Cannot set layer level to %d!\n", __FUNCTION__,
                    displayId, layerId, level);

        /* Unlock access to the layerData */
        pthread_mutex_unlock( &layerData->lock );
    } else
        D_ERROR("%s() screen %d gfx layer %d: Invalid level %d! It needs to be between %d and -%d.\n", __FUNCTION__, 
                displayId, layerId, level, BCMNEXUS_Z_ORDER_MAX/2, BCMNEXUS_Z_ORDER_MAX/2);

    D_DEBUG_AT( bcmnexusLayer, "%s() screen %d gfx layer %d: Exit [%d].\n", __FUNCTION__, displayId, layerId, res );
    return res;
}

/**************************************************
 * Setup the Primary Display Primary Layer/Region *
 **************************************************/

/* initialize the primary graphics layer data
 * this function is called for the master fusionee only
 */
static DFBResult
bcmInitPrimaryDisplayPrimaryLayer( CoreLayer                  *layer,
                                   void                       *driver_data,
                                   void                       *layer_data,
                                   DFBDisplayLayerDescription *description,
                                   DFBDisplayLayerConfig      *config,
                           UNUSED_ DFBColorAdjustment         *adjustment )
{
    DFBBCMNEXUS  *dfb_bcmnexus = driver_data;
    bcmLayerData *layerData    = layer_data;

    D_ASSERT( dfb_bcmnexus != NULL );

    layerData->layerId   = DLID_PRIMARY;
    layerData->displayId = dfb_screen_id_translated(dfb_layer_screen(layer));
    layerData->sourceId  = DLSID_SURFACE;
#ifdef BCMNEXUS_GFD0_HAS_VERTICAL_SCALER
    layerData->hasVscaler = true;
#endif

    bcmInitLayer(dfb_bcmnexus, layerData, description, config);

    return DFB_OK;
}

/* Add a primary region to the primary display's primary layer */
static DFBResult
bcmAddPrimaryDisplayPrimaryRegion(         CoreLayer             *layer,
                                   UNUSED_ void                  *driver_data,
                                           void                  *layer_data,
                                           void                  *region_data,
                                           CoreLayerRegionConfig *config )
{
    return bcmAddRegion(layer, layer_data, region_data, config, BCMNEXUS_MAX_HD_DISPLAY_WIDTH, BCMNEXUS_MAX_HD_DISPLAY_HEIGHT);
}

/* Flip the primary region */
static DFBResult
bcmFlipPrimaryDisplayPrimaryRegion( UNUSED_ CoreLayer             *layer,
                                            void                  *driver_data,
                                            void                  *layer_data,
                                            void                  *region_data,
                                            CoreSurface           *surface,
                                            DFBSurfaceFlipFlags    flags,
                                            CoreSurfaceBufferLock *lock )
{
    DFBResult             res;
    DFBBCMNEXUS          *dfb_bcmnexus = driver_data;
    bcmLayerData         *layerData    = layer_data;
    DFBDisplayLayerID     layerId;
    DFBScreenID           displayId;
#if (NUM_DISPLAYS > 1)
    bcmLayerData         *layerDataSec;
    bcmRegionData        *regionData;
    CoreSurfaceBufferLock frameBufferLock;
    DFBRectangle          dest_rect;
#endif

    layerId   = layerData->layerId;
    displayId = layerData->displayId;
    
    D_DEBUG_AT( bcmnexusLayerX, "%s() screen %d gfx layer %d: surface=%p, flags=0x%08x, handle=%p, pitch=%d\n", __FUNCTION__,
                displayId, layerId, (void *)surface, flags, (void *)lock->handle, lock->pitch);

    /* Lock access to the layerData */
    pthread_mutex_lock( &layerData->lock );

    res = bcmFlipRegion(dfb_bcmnexus, layer_data, region_data, surface, flags, lock);

    if (res != DFB_OK)
    {
        D_ERROR("%s() screen %d gfx layer %d: Could NOT flip primary region (res=%d)!\n", __FUNCTION__, displayId, layerId, res);
        /* Unlock access to the layerData */
        pthread_mutex_unlock( &layerData->lock );
        return res;
    }

#if (NUM_DISPLAYS > 1)
    /* Check to see whether we need to use the M2MC to blit from the primary graphics source path
       to the secondary path when we are in mirroring mode. */
    layerDataSec = bcmGetLayerData(GET_DISPLAY_GFX_LAYER_ID(dfb_bcmnexus, PRIMARY_GFX_LAYER_INDEX, SECONDARY_DISPLAY_ID));
    if (layerDataSec != NULL)
    {
        regionData = layerDataSec->regionData;

        if (regionData && (layerDataSec->sourceId == DLSID_SURFACE) && layerDataSec->frameBuffer)
        {
            if (dfb_surface_lock( layerDataSec->frameBuffer )) {
                /* Unlock access to the layerData */
                pthread_mutex_unlock( &layerData->lock );
                return DFB_FUSION;
            }

            /* Use the M2MC to scale the primary gfx layer H size to the H size of the secondary gfx layer.
               Also use the M2MC to scale the primary gfx layer V size to the V size of the secondary gfx layer
               if we have a vertical scaler in the graphics feeder (GFD), otherwise we need to use the M2MC to
               scale it vertically to the display height (which isn't necessarily the canvas height) */
            dest_rect.x = 0;
            dest_rect.y = 0;
            dest_rect.w = regionData->config.width;
            if (layerDataSec->hasVscaler)
                dest_rect.h = regionData->config.height;
            else
                dest_rect.h = MIN(layerDataSec->destRectangle.h, GET_DISPLAY_HEIGHT(dfb_bcmnexus, SECONDARY_DISPLAY_ID));

            D_DEBUG_AT( bcmnexusLayerX, "%s() screen %d gfx layer %d: About to stretch blit surface %p to framebuffer %p [%dx%d@%d,%d]\n", __FUNCTION__,
                    displayId, layerId, (void *)surface, (void *)layerDataSec->frameBuffer, dest_rect.w, dest_rect.h, dest_rect.x, dest_rect.y);

            dfb_gfx_stretch_to( surface, layerDataSec->frameBuffer, &dest_rect, false);
            dfb_gfxcard_sync();

            res = dfb_surface_lock_buffer( layerDataSec->frameBuffer, CSBR_FRONT, CSAID_GPU, CSAF_WRITE, &frameBufferLock );
            if (res)
            {
                D_ERROR( "%s() screen %d gfx layer %d: Could NOT lock framebuffer surface! (res=%d)\n", __FUNCTION__, displayId, layerId, res);
                dfb_surface_unlock( layerDataSec->frameBuffer );
                /* Unlock access to the layerData */
                pthread_mutex_unlock( &layerData->lock );
                return res;
            }

            if (layerDataSec->layerEnabled && (frameBufferLock.handle != regionData->bufferLock.handle)) {
                regionData->bufferLock = frameBufferLock;
                bcmSetLayerFrameBuffer(dfb_bcmnexus, SECONDARY_GFX_LAYER_INDEX);
            }
            
            dfb_surface_unlock_buffer(layerDataSec->frameBuffer, &frameBufferLock);

            if (dfb_surface_unlock( layerDataSec->frameBuffer )) {
                /* Unlock access to the layerData */
                pthread_mutex_unlock( &layerData->lock );
                return DFB_FUSION;
            }
        }
    }
#endif
    /* Unlock access to the layerData */
    pthread_mutex_unlock( &layerData->lock );

    D_DEBUG_AT( bcmnexusLayerX, "%s() screen %d gfx layer %d: Exit.\n", __FUNCTION__, displayId, layerId );
    return DFB_OK;
}

/* Update the primary region */
static DFBResult
bcmUpdatePrimaryDisplayPrimaryRegion( UNUSED_ CoreLayer             *layer,
                                              void                  *driver_data,
                                              void                  *layer_data,
                                              void                  *region_data,
                                              CoreSurface           *surface,
                                              const DFBRegion       *update,
                                              CoreSurfaceBufferLock *lock )
{
    DFBResult             res;
    DFBBCMNEXUS          *dfb_bcmnexus = driver_data;
    bcmLayerData         *layerData    = layer_data;
    DFBDisplayLayerID     layerId;
    DFBScreenID           displayId;
    bool                  primaryUpdated;
#if (NUM_DISPLAYS > 1)
    bcmLayerData         *layerDataSec;
    bcmRegionData        *regionData;
    CoreSurfaceBufferLock frameBufferLock;
    DFBRectangle          dest_rect;
#endif

    layerId   = layerData->layerId;
    displayId = layerData->displayId;
    
    D_DEBUG_AT( bcmnexusLayerX, "%s() screen %d gfx layer %d: surf=%p, handle=%p [%d,%d->%d,%d]\n", __FUNCTION__,
                                displayId, layerId, (void *)surface, (void *)lock->handle,
                                update->x1, update->y1, update->x2, update->y2);

    /* Lock access to the layerData */
    pthread_mutex_lock( &layerData->lock );

    /* If update skip count is set to a value greater than 0, it means we won't
       update this layer region as often.  This is useful for reducing memory
       bandwidth and improving graphics performance.
    */
    if (layerData->updateSkipCnt) {
        layerData->updateSkipCnt--;
        primaryUpdated = false;
    } else {
        res = bcmInvalidateRegion(dfb_bcmnexus, layer_data, region_data, surface, DSFLIP_NONE, lock);

        if (res != DFB_OK)
        {
            D_ERROR("%s() screen %d gfx layer %d: Could NOT update primary region (res=%d)!\n", __FUNCTION__, displayId, layerId, res);
            /* Unlock access to the layerData */
            pthread_mutex_unlock( &layerData->lock );
            return res;
        }
        layerData->updateSkipCnt = bcmGetLayerDefaultUpdateSkipCnt(layerData->layerId);
        primaryUpdated = true;
    }

#if (NUM_DISPLAYS > 1)
    /* Check to see whether we need to use the M2MC to blit from the primary graphics source path
       to the secondary path when we are in mirroring mode. */
    layerDataSec = bcmGetLayerData(GET_DISPLAY_GFX_LAYER_ID(dfb_bcmnexus, PRIMARY_GFX_LAYER_INDEX, SECONDARY_DISPLAY_ID));
    if (layerDataSec && primaryUpdated)
    {
        regionData = layerDataSec->regionData;

        if (regionData && (layerDataSec->sourceId == DLSID_SURFACE) && layerDataSec->frameBuffer)
        {
            /* If update skip count is set to a value greater than 0, it means we won't
               update the secondary display with the contents of the primary as often.
               This is useful for reducing memory bandwidth and improving graphics
               performance.
            */
            if (layerDataSec->updateSkipCnt) {
                layerDataSec->updateSkipCnt--;
            } else {
                if (dfb_surface_lock( layerDataSec->frameBuffer )) {
                    /* Unlock access to the layerData */
                    pthread_mutex_unlock( &layerData->lock );
                    return DFB_FUSION;
                }

                /* Use the M2MC to scale the primary gfx layer H size to the H size of the secondary gfx layer.
                   Also use the M2MC to scale the primary gfx layer V size to the V size of the secondary gfx layer
                   if we have a vertical scaler in the graphics feeder (GFD), otherwise we need to use the M2MC to
                   scale it vertically to the display height (which isn't necessarily the canvas height) */
                dest_rect.x = 0;
                dest_rect.y = 0;
                dest_rect.w = regionData->config.width;
                if (layerDataSec->hasVscaler)
                    dest_rect.h = regionData->config.height;
                else
                    dest_rect.h = MIN(layerDataSec->destRectangle.h, GET_DISPLAY_HEIGHT(dfb_bcmnexus, SECONDARY_DISPLAY_ID));

                D_DEBUG_AT( bcmnexusLayerX, "%s() screen %d gfx layer %d: About to stretch blit surface %p to framebuffer %p [%dx%d@%d,%d]\n",
                            __FUNCTION__, displayId, layerId, (void *)surface, (void *)layerDataSec->frameBuffer,
                            dest_rect.w, dest_rect.h, dest_rect.x, dest_rect.y);

                dfb_gfx_stretch_to( surface, layerDataSec->frameBuffer, &dest_rect, false);
                dfb_gfxcard_sync();

                res = dfb_surface_lock_buffer( layerDataSec->frameBuffer, CSBR_FRONT, CSAID_GPU, CSAF_READ, &frameBufferLock );
                if (res) {
                    D_ERROR( "%s() screen %d gfx layer %d: Could NOT lock framebuffer surface! (res=%d)\n", __FUNCTION__, displayId, layerId, res);
                    dfb_surface_unlock( layerDataSec->frameBuffer );
                    /* Unlock access to the layerData */
                    pthread_mutex_unlock( &layerData->lock );
                    return res;
                }

                if (layerDataSec->layerEnabled && (frameBufferLock.handle != regionData->bufferLock.handle)) {
                    D_DEBUG_AT( bcmnexusLayerX, "%s() screen %d gfx layer %d: Setting framebuffer %p.\n",
                            __FUNCTION__, displayId, layerId, (void *)layerDataSec->frameBuffer);

                    regionData->bufferLock = frameBufferLock;
                    bcmSetLayerFrameBuffer(dfb_bcmnexus, SECONDARY_GFX_LAYER_INDEX);
                }
                
                dfb_surface_unlock_buffer(layerDataSec->frameBuffer, &frameBufferLock);

                if (dfb_surface_unlock( layerDataSec->frameBuffer )) {
                    /* Unlock access to the layerData */
                    pthread_mutex_unlock( &layerData->lock );
                    return DFB_FUSION;
                }
                layerDataSec->updateSkipCnt = bcmGetLayerDefaultUpdateSkipCnt(layerDataSec->layerId);
            }
        }
    }
#endif
    /* Unlock access to the layerData */
    pthread_mutex_unlock( &layerData->lock );

    D_DEBUG_AT( bcmnexusLayerX, "%s() screen %d gfx layer %d: Exit.\n", __FUNCTION__, displayId, layerId );
    return DFB_OK;
}


DisplayLayerFuncs bcmnexusPrimaryGfxLayerFuncs = {
    .LayerDataSize         = bcmLayerDataSize,
    .RegionDataSize        = bcmRegionDataSize,
    .InitLayer             = bcmInitPrimaryDisplayPrimaryLayer,
    .ShutdownLayer         = bcmShutdownLayer,
    .TestRegion            = bcmTestRegion,
    .AddRegion             = bcmAddPrimaryDisplayPrimaryRegion,
    .SetRegion             = bcmSetRegion,
    .RemoveRegion          = bcmRemoveRegion,
    .FlipRegion            = bcmFlipPrimaryDisplayPrimaryRegion,
    .UpdateRegion          = bcmUpdatePrimaryDisplayPrimaryRegion,
    .InitSource            = bcmInitSource,
    .SetLevel              = bcmSetLevel,
    .GetLevel              = bcmGetLevel,
};


/******************************************************
 * Setup the Secondary Display's Primary Layer/Region *
 ******************************************************/

#if (NUM_DISPLAYS > 1)
/* initialize the secondary graphics layer data
 * this function is called for the master fusionee only
 */
static DFBResult
bcmInitSecondaryDisplayPrimaryLayer(         CoreLayer                  *layer,
                                             void                       *driver_data,
                                             void                       *layer_data,
                                             DFBDisplayLayerDescription *description,
                                             DFBDisplayLayerConfig      *config,
                                     UNUSED_ DFBColorAdjustment         *adjustment )
{
    DFBBCMNEXUS  *dfb_bcmnexus = driver_data;
    bcmLayerData *layerData    = layer_data;

    D_ASSERT( dfb_bcmnexus != NULL );

    layerData->layerId   = DLID_PRIMARY+1;
    layerData->displayId = dfb_screen_id_translated(dfb_layer_screen(layer));
    layerData->sourceId  = DLSID_SURFACE+1;
#ifdef BCMNEXUS_GFD1_HAS_VERTICAL_SCALER
    layerData->hasVscaler = true;
#endif

    bcmInitLayer(dfb_bcmnexus, layerData, description, config);

    return DFB_OK;
}

/* Add a region to the secondary display's primary layer */
static DFBResult
bcmAddSecondaryDisplayPrimaryRegion(         CoreLayer             *layer,
                                     UNUSED_ void                  *driver_data,
                                             void                  *layer_data,
                                             void                  *region_data,
                                             CoreLayerRegionConfig *config )
{
    return bcmAddRegion(layer, layer_data, region_data, config, BCMNEXUS_MAX_SD_DISPLAY_WIDTH, BCMNEXUS_MAX_SD_DISPLAY_HEIGHT);
}

/* Flip the secondary region */
static DFBResult
bcmFlipSecondaryDisplayPrimaryRegion( UNUSED_ CoreLayer             *layer,
                                              void                  *driver_data,
                                              void                  *layer_data,
                                              void                  *region_data,
                                              CoreSurface           *surface,
                                              DFBSurfaceFlipFlags    flags,
                                              CoreSurfaceBufferLock *lock )
{
    DFBResult          res;
    bcmLayerData      *layerData = layer_data;
    DFBDisplayLayerID  layerId;
    DFBScreenID        displayId;

    layerId   = layerData->layerId;
    displayId = layerData->displayId;

    D_DEBUG_AT( bcmnexusLayerX, "%s() screen %d gfx layer %d: surface=%p, flags=0x%08x, handle=%p\n", __FUNCTION__,
                displayId, layerId, (void *)surface, flags, (void *)lock->handle);

    /* Lock access to the layerData */
    pthread_mutex_lock( &layerData->lock );

    res = bcmFlipRegion(driver_data, layer_data, region_data, surface, flags, lock);

    /* Unlock access to the layerData */
    pthread_mutex_unlock( &layerData->lock );

    if (res != DFB_OK) {
        D_ERROR("%s() screen %d gfx layer %d: Could NOT flip secondary region (res=%d)!\n", __FUNCTION__,  displayId, layerId, res);
        return res;
    }

    D_DEBUG_AT( bcmnexusLayerX, "%s() screen %d gfx layer %d: Exit.\n", __FUNCTION__, displayId, layerId );
    return DFB_OK;
}

/* Update the secondary region */
static DFBResult
bcmUpdateSecondaryDisplayPrimaryRegion( UNUSED_ CoreLayer             *layer,
                                                void                  *driver_data,
                                                void                  *layer_data,
                                                void                  *region_data,
                                                CoreSurface           *surface,
                                                const DFBRegion       *update,
                                                CoreSurfaceBufferLock *lock )
{
    DFBResult          res       = DFB_OK;
    bcmLayerData      *layerData = layer_data;
    DFBDisplayLayerID  layerId;
    DFBScreenID        displayId;

    layerId   = layerData->layerId;
    displayId = layerData->displayId;

    D_DEBUG_AT( bcmnexusLayerX, "%s() screen %d gfx layer %d: surf=%p, handle=%p [%d,%d->%d,%d]\n", __FUNCTION__,
                                    displayId, layerId, (void *)surface, (void *)lock->handle,
                                    update->x1, update->y1, update->x2, update->y2);

    /* Lock access to the layerData */
    pthread_mutex_lock( &layerData->lock );

    /* If update skip count is set to a value greater than 0, it means we won't
       update this layer region as often.  This is useful for reducing memory
       bandwidth and improving graphics performance.
    */
    if (layerData->updateSkipCnt) {
        layerData->updateSkipCnt--;
    } else {

        res = bcmInvalidateRegion(driver_data, layer_data, region_data, surface, DSFLIP_NONE, lock);

        if (res != DFB_OK)
            D_ERROR("%s() screen %d gfx layer %d: Could NOT update secondary region (res=%d)!\n", __FUNCTION__, displayId, layerId, res);
        
        layerData->updateSkipCnt = bcmGetLayerDefaultUpdateSkipCnt(layerData->layerId);
    }
    /* Unlock access to the layerData */
    pthread_mutex_unlock( &layerData->lock );

    D_DEBUG_AT( bcmnexusLayerX, "%s() screen %d gfx layer %d: Exit [%d].\n", __FUNCTION__, displayId, layerId, res );
    return res;
}

DisplayLayerFuncs bcmnexusSecondaryGfxLayerFuncs = {
    .LayerDataSize         = bcmLayerDataSize,
    .RegionDataSize        = bcmRegionDataSize,
    .InitLayer             = bcmInitSecondaryDisplayPrimaryLayer,
    .ShutdownLayer         = bcmShutdownLayer,
    .TestRegion            = bcmTestRegion,
    .AddRegion             = bcmAddSecondaryDisplayPrimaryRegion,
    .SetRegion             = bcmSetRegion,
    .RemoveRegion          = bcmRemoveRegion,
    .FlipRegion            = bcmFlipSecondaryDisplayPrimaryRegion,
    .UpdateRegion          = bcmUpdateSecondaryDisplayPrimaryRegion,
    .InitSource            = bcmInitSource,
    .SetLevel              = bcmSetLevel,
    .GetLevel              = bcmGetLevel,
};
#endif
