/***************************************************************************
 *     (c)2007-2011 Broadcom Corporation
 *  
 *  This program is the proprietary software of Broadcom Corporation and/or its licensors,
 *  and may only be used, duplicated, modified or distributed pursuant to the terms and
 *  conditions of a separate, written license agreement executed between you and Broadcom
 *  (an "Authorized License").  Except as set forth in an Authorized License, Broadcom grants
 *  no license (express or implied), right to use, or waiver of any kind with respect to the
 *  Software, and Broadcom expressly reserves all rights in and to the Software and all
 *  intellectual property rights therein.  IF YOU HAVE NO AUTHORIZED LICENSE, THEN YOU
 *  HAVE NO RIGHT TO USE THIS SOFTWARE IN ANY WAY, AND SHOULD IMMEDIATELY
 *  NOTIFY BROADCOM AND DISCONTINUE ALL USE OF THE SOFTWARE.  
 *   
 *  Except as expressly set forth in the Authorized License,
 *   
 *  1.     This program, including its structure, sequence and organization, constitutes the valuable trade
 *  secrets of Broadcom, and you shall use all reasonable efforts to protect the confidentiality thereof,
 *  and to use this information only in connection with your use of Broadcom integrated circuit products.
 *   
 *  2.     TO THE MAXIMUM EXTENT PERMITTED BY LAW, THE SOFTWARE IS PROVIDED "AS IS" 
 *  AND WITH ALL FAULTS AND BROADCOM MAKES NO PROMISES, REPRESENTATIONS OR 
 *  WARRANTIES, EITHER EXPRESS, IMPLIED, STATUTORY, OR OTHERWISE, WITH RESPECT TO 
 *  THE SOFTWARE.  BROADCOM SPECIFICALLY DISCLAIMS ANY AND ALL IMPLIED WARRANTIES 
 *  OF TITLE, MERCHANTABILITY, NONINFRINGEMENT, FITNESS FOR A PARTICULAR PURPOSE, 
 *  LACK OF VIRUSES, ACCURACY OR COMPLETENESS, QUIET ENJOYMENT, QUIET POSSESSION 
 *  OR CORRESPONDENCE TO DESCRIPTION. YOU ASSUME THE ENTIRE RISK ARISING OUT OF 
 *  USE OR PERFORMANCE OF THE SOFTWARE.
 *  
 *  3.     TO THE MAXIMUM EXTENT PERMITTED BY LAW, IN NO EVENT SHALL BROADCOM OR ITS 
 *  LICENSORS BE LIABLE FOR (i) CONSEQUENTIAL, INCIDENTAL, SPECIAL, INDIRECT, OR 
 *  EXEMPLARY DAMAGES WHATSOEVER ARISING OUT OF OR IN ANY WAY RELATING TO YOUR 
 *  USE OF OR INABILITY TO USE THE SOFTWARE EVEN IF BROADCOM HAS BEEN ADVISED OF 
 *  THE POSSIBILITY OF SUCH DAMAGES; OR (ii) ANY AMOUNT IN EXCESS OF THE AMOUNT 
 *  ACTUALLY PAID FOR THE SOFTWARE ITSELF OR U.S. $1, WHICHEVER IS GREATER. THESE 
 *  LIMITATIONS SHALL APPLY NOTWITHSTANDING ANY FAILURE OF ESSENTIAL PURPOSE OF 
 *  ANY LIMITED REMEDY.
 * 
 * $brcm_Workfile: bcmnexus_layer.h $
 * $brcm_Revision: DirectFB_1_4_Port/10 $
 * $brcm_Date: 3/4/11 5:04p $
 *
 * [File Description:]
 *
 * Revision History:
 *
 * $brcm_Log: /AppLibs/opensource/directfb/src/broadcom_files/private/DirectFB/1.4/systems/bcmnexus/bcmnexus_layer.h $
 * 
 * DirectFB_1_4_Port/10   3/4/11 5:04p robertwm
 * SW7420-1593: DirectFB-1.4.1: Add ShutdownLayer() API.
 * 
 * DirectFB_1_4_Port/9   3/4/11 3:30p robertwm
 * SW7420-1590: DirectFB-1.4.5: Deadlock when closing down multiple
 *  applications.
 * 
 * DirectFB_1_4_Port/8   3/1/11 12:01p christ
 * SW7420-1545: DirectFB: Application hang when using multi app DirectFB
 *  with SawMan window manager.
 * 
 * DirectFB_1_4_Port/7   2/8/11 1:31p robertwm
 * SW7420-1471: DirectFB: Replace fusion skirmishes with pthread
 *  semantics.
 * 
 * DirectFB_1_4_Port/6   2/8/11 1:13p robertwm
 * SW7420-1467: DirectFB: Multiple apps should NOT share NEXUS Display
 *  handle.
 * 
 * DirectFB_1_4_Port/5   1/24/11 4:32p christ
 * SW7420-1411: DirectFB: Merge of features required for DirectFB 1.4.5
 *  phase 1.5 release - Add some encoder support
 * 
 * DirectFB_1_4_Port/4   8/11/10 2:00a robertwm
 * SW7405-4250: DirectFB: Screen destination rectangle and source
 *  rectangle not saved across contexts.
 * SW7405-4743: DirectFB: Multiple instances of df_andi.c do not get
 *  displayed.
 * 
 * DirectFB_1_4_Port/3   4/9/10 4:31p robertwm
 * SW7405-4179: DirectFB: Support setting the z-order (level) of a layer.
 * SW7405-4180: DirectFB: Require independent setting of graphics
 *  resolution on multiple output paths.
 * 
 * DirectFB_1_4_Port/2   12/16/09 9:49p robertwm
 * SW7400-2643: DirectFB: Add support for setting/getting video modes.
 *
 ***************************************************************************/

#ifndef __CORE__BCMNEXUS__LAYER_H__
#define __CORE__BCMNEXUS__LAYER_H__

#include <directfb_version.h>
#include <core/layers.h>
#include <direct/thread.h>

/******************************************************************************/

typedef struct {
     CoreLayerRegionConfig          config;             /* Current Region Configuration */
     bool                           vscale;             /* Set to true when a vertical scale using blitter is required */
     CoreSurfaceBufferLock          bufferLock;         /* Remember the last surface buffer lock */
} bcmRegionData;

typedef struct {
     bcmRegionData                 *regionData;         /* Pointer to region data */
     DFBDisplayLayerID              layerId;            /* Graphics layer ID */
     DFBScreenID                    displayId;          /* Primary or secondary display/screen */
     DFBDisplayLayerSourceID        sourceId;           /* Graphics source Id */
     bool                           layerEnabled;       /* true if this layer is enabled */
     int                            level;              /* z-order of this layer (default is 0 for primary, +ve means above and -ve means below) */
     CoreSurface                   *frameBuffer;        /* Used when there is no vertical scaler in the GFD */
     pthread_cond_t                 frameBufferSem;     /* Framebuffer callback semaphore */
     pthread_mutex_t                frameBufferSemLock; /* Framebuffer callback protective mutex */
     DFBRectangle                   srcRectangle;       /* Source rectangle for the display/screen */
     DFBRectangle                   destRectangle;      /* Destination rectangle for the display/screen */
     NEXUS_CompositorBlendFactor    srcBlendFunc;       /* Source blend function */
     NEXUS_CompositorBlendFactor    destBlendFunc;      /* Destination blend function */
     int                            alpha;              /* FF = fully opaque, 00 = fully transparent */
     int                            constantAlpha;      /* FF = fully opaque, 00 = fully transparent */
     bool                           srcColorKeyOn;      /* true if source color keying is enabled */
     bool                           hasVscaler;         /* Set to true if graphics feeder has vertical scaler */
     int                            updateSkipCnt;      /* Set > 0, mean how many times we can skip updating this layer */
     pthread_mutex_t                lock;               /* Lock access to this structure */
     void                         (*layerCallback)(void *context, int param);
     DirectThread                  *frameBufferThread;  /* Framebuffer thread */
     pthread_mutex_t                frameBufferThreadLockIn;
     pthread_mutex_t                frameBufferThreadLockOut;
     pthread_cond_t                 frameBufferThreadSemIn;
     pthread_cond_t                 frameBufferThreadSemOut;
     DFBResult                      frameBufferThreadResult;
     bool                           frameBufferThreadQuit;
} bcmLayerData;

typedef struct {
    DFBDisplayLayerID   layerId;
    NEXUS_SurfaceHandle hSurface;
} bcmLayerCallSetGfxFrameBuffer;

typedef struct {
    DFBDisplayLayerID      layerId;
    NEXUS_GraphicsSettings gfxSettings;
} bcmLayerCallSetGfxSettings;

/**********************************************************************************************************************/

/* Layers */

extern DFBResult bcmSetLayerState(DFBBCMNEXUS *dfb_bcmnexus, DFBDisplayLayerID layerId, bool enable);
extern DFBResult bcmSetLayerDestRectangle(DFBBCMNEXUS *dfb_bcmnexus, DFBDisplayLayerID layerId, DFBRectangle *destRectangle);
extern DisplayLayerFuncs bcmnexusPrimaryGfxLayerFuncs;
extern DisplayLayerFuncs bcmnexusSecondaryGfxLayerFuncs;

bcmLayerData *bcmGetLayerData(DFBDisplayLayerID layerId);

#endif
