/******************************************************************************
 *    (c)2009-2011 Broadcom Corporation
 * 
 * This program is the proprietary software of Broadcom Corporation and/or its licensors,
 * and may only be used, duplicated, modified or distributed pursuant to the terms and
 * conditions of a separate, written license agreement executed between you and Broadcom
 * (an "Authorized License").  Except as set forth in an Authorized License, Broadcom grants
 * no license (express or implied), right to use, or waiver of any kind with respect to the
 * Software, and Broadcom expressly reserves all rights in and to the Software and all
 * intellectual property rights therein.  IF YOU HAVE NO AUTHORIZED LICENSE, THEN YOU
 * HAVE NO RIGHT TO USE THIS SOFTWARE IN ANY WAY, AND SHOULD IMMEDIATELY
 * NOTIFY BROADCOM AND DISCONTINUE ALL USE OF THE SOFTWARE.  
 *  
 * Except as expressly set forth in the Authorized License,
 *  
 * 1.     This program, including its structure, sequence and organization, constitutes the valuable trade
 * secrets of Broadcom, and you shall use all reasonable efforts to protect the confidentiality thereof,
 * and to use this information only in connection with your use of Broadcom integrated circuit products.
 *  
 * 2.     TO THE MAXIMUM EXTENT PERMITTED BY LAW, THE SOFTWARE IS PROVIDED "AS IS" 
 * AND WITH ALL FAULTS AND BROADCOM MAKES NO PROMISES, REPRESENTATIONS OR 
 * WARRANTIES, EITHER EXPRESS, IMPLIED, STATUTORY, OR OTHERWISE, WITH RESPECT TO 
 * THE SOFTWARE.  BROADCOM SPECIFICALLY DISCLAIMS ANY AND ALL IMPLIED WARRANTIES 
 * OF TITLE, MERCHANTABILITY, NONINFRINGEMENT, FITNESS FOR A PARTICULAR PURPOSE, 
 * LACK OF VIRUSES, ACCURACY OR COMPLETENESS, QUIET ENJOYMENT, QUIET POSSESSION 
 * OR CORRESPONDENCE TO DESCRIPTION. YOU ASSUME THE ENTIRE RISK ARISING OUT OF 
 * USE OR PERFORMANCE OF THE SOFTWARE.
 * 
 * 3.     TO THE MAXIMUM EXTENT PERMITTED BY LAW, IN NO EVENT SHALL BROADCOM OR ITS 
 * LICENSORS BE LIABLE FOR (i) CONSEQUENTIAL, INCIDENTAL, SPECIAL, INDIRECT, OR 
 * EXEMPLARY DAMAGES WHATSOEVER ARISING OUT OF OR IN ANY WAY RELATING TO YOUR 
 * USE OF OR INABILITY TO USE THE SOFTWARE EVEN IF BROADCOM HAS BEEN ADVISED OF 
 * THE POSSIBILITY OF SUCH DAMAGES; OR (ii) ANY AMOUNT IN EXCESS OF THE AMOUNT 
 * ACTUALLY PAID FOR THE SOFTWARE ITSELF OR U.S. $1, WHICHEVER IS GREATER. THESE 
 * LIMITATIONS SHALL APPLY NOTWITHSTANDING ANY FAILURE OF ESSENTIAL PURPOSE OF 
 * ANY LIMITED REMEDY.
 *
 * $brcm_Workfile: bcmnexus_pool.c $
 * $brcm_Revision: DirectFB_1_4_Port/45 $
 * $brcm_Date: 3/17/11 6:52p $
 * 
 * Module Description:
 * 
 * Revision History:
 * 
 * $brcm_Log: /AppLibs/opensource/directfb/src/broadcom_files/private/DirectFB/1.4/systems/bcmnexus/bcmnexus_pool.c $
 * 
 * DirectFB_1_4_Port/45   3/17/11 6:52p robertwm
 * SW7422-378: DirectFB: Destroying a Palettised surface in a client
 *  application causes NEXUS assertion during free.
 * 
 * DirectFB_1_4_Port/44   3/17/11 12:40p robertwm
 * SW7422-378: DirectFB: Destroying a Palettised surface in a client
 *  application causes NEXUS assertion during free.
 * SW7422-385: DirectFB: Using Image Provider results in some images being
 *  displayed incorrectly.
 * 
 * DirectFB_1_4_Port/43   3/1/11 8:49p robertwm
 * SW7420-1559: DirectFB: NEXUS surface assertion in multi-app mode when
 *  closing down a DFB client.
 * 
 * DirectFB_1_4_Port/42   3/1/11 12:01p christ
 * SW7420-1545: DirectFB: Application hang when using multi app DirectFB
 *  with SawMan window manager.
 * 
 * DirectFB_1_4_Port/41   2/23/11 6:04p christ
 * SW7425-108: DirectFB: Insignia LUT8 tests failing on 7425 packet blit
 *  build.
 * 
 * DirectFB_1_4_Port/40   2/20/11 7:34p robertwm
 * SW7420-1394: video layer support in DFB.
 * SW7422-275: DirectFB: Enabling video layer causes Segmentation fault.
 * 
 * DirectFB_1_4_Port/39   2/17/11 11:12p robertwm
 * SW7420-1510: DirectFB: Reduce number of IPC calls for gfx operations
 *  using packet-blit interface.
 * SW7422-275: DirectFB: Enabling video layer causes Segmentation fault.
 * 
 * DirectFB_1_4_Port/38   2/8/11 4:58p robertwm
 * SW7420-1470: DirectFB: Ensure width/height/pitch of PX3D texture
 *  surfaces is a power of 2.
 * SW7420-1472: DirectFB: df_brcmTest fails to display preallocated
 *  surfaces.
 * 
 * DirectFB_1_4_Port/37   2/8/11 3:18p robertwm
 * SW7420-1472: DirectFB: df_brcmTest fails to display preallocated
 *  surfaces.
 * 
 * DirectFB_1_4_Port/36   2/8/11 1:23p robertwm
 * SW7420-1468: DirectFB: Add context debug color output.
 * SW7420-1470: DirectFB: Ensure width/height/pitch of PX3D texture
 *  surfaces is a power of 2.
 * 
 * DirectFB_1_4_Port/35   2/3/11 6:02p christ
 * SW7420-1458: DirectFB: Rework internal handling of nexus heaps.
 * 
 * DirectFB_1_4_Port/34   1/19/11 12:25p scchang
 * SW7420-1394: video layer support in DFB
 * 
 * DirectFB_1_4_Port/33   12/16/10 5:54p robertwm
 * SW35230-2597: DirectFB: Cannot build bcmnexus_pool.c due to recent
 *  changes to support multiple gfx heaps.
 * 
 * DirectFB_1_4_Port/32   12/14/10 7:25p robertwm
 * SW7422-147: DirectFB: Cannot run df_dok or df_window with changes for
 *  OpenGL ES 2.0
 * 
 * DirectFB_1_4_Port/31   12/14/10 1:17p robertwm
 * SW7422-140: DirectFB: Support OpenGL ES 2.0 applications on the VC-4
 *  core.
 * 
 * DirectFB_1_4_Port/30   12/3/10 8:39p raywu
 * SW7550-629: DirectFB: Integrated ZSP code to DFB packet buffer branch
 * 
 * DirectFB_1_4_Port/29   12/1/10 3:47p christ
 * SW7420-1283: DirectFB: Add support for extra Nexus heap on dual MEMC
 *  parts
 * 
 * DirectFB_1_4_Port/28   10/19/10 5:41p robertwm
 * SW7420-1189: DirectFB: Kernel Oops when running in multi-app mode and
 *  using palletised surfaces.
 * 
 * DirectFB_1_4_Port/27   10/12/10 2:54p robertwm
 * SW7550-589: DirectFB: Add LUT4 pixel format support to DirectFB.
 * 
 * DirectFB_1_4_Port/26   9/23/10 2:48p robertwm
 * SW7405-4891: DirectFB: When running multiple instances of df_andi, a
 *  deadlock can occur.
 * 
 * DirectFB_1_4_Port/25   9/15/10 4:29p robertwm
 * SW7405-4875: DirectFB: M2MC trampling over memory when surface is
 *  reconfigured.
 * 
 * DirectFB_1_4_Port/24   8/20/10 3:40p robertwm
 * SW7405-4791: DirectFB: DirectFB-1.4.1 Phase 2.5 fails to build with
 *  Nexus Phase 5.0 reference software.
 * 
 * DirectFB_1_4_Port/23   8/17/10 11:55a robertwm
 * SW7405-4768: DirectFB: Cannot build DirectFB-1.4.1 Phase 2.5 release on
 *  7550/7540 chips.
 * 
 * DirectFB_1_4_Port/22   8/16/10 11:39p robertwm
 * SW7405-4761: DirectFB: Optimise the surface pool locking code.
 * 
 * DirectFB_1_4_Port/21   8/11/10 2:00a robertwm
 * SW7405-4250: DirectFB: Screen destination rectangle and source
 *  rectangle not saved across contexts.
 * SW7405-4743: DirectFB: Multiple instances of df_andi.c do not get
 *  displayed.
 * 
 * DirectFB_1_4_Port/20   7/26/10 5:32p jackf
 * SW35230-771: Added include for shim_nexus.h.
 * 
 * DirectFB_1_4_Port/19   7/22/10 4:23p robertwm
 * SW7405-4649: DirectFB: Optimise surface locking and wait idle syncing.
 * 
 * DirectFB_1_4_Port/18   7/2/10 12:23p robertwm
 * SW3556-1141: DirectFB: The Broadcom surface pool should support cursor
 *  surface types.
 * 
 * DirectFB_1_4_Port/17   5/27/10 9:10p robertwm
 * SW7420-786: DirectFB: the Broadcom Surface Pool should NOT allow
 *  internal/system memory allocations.
 * SW7420-785: DirectFB: 7420 cannot blit from palletised surface created
 *  in heap1.
 * 
 * DirectFB_1_4_Port/16   4/9/10 4:52p robertwm
 * SW7420-510: DirectFB: Support Vertical Scaler in Graphics Feeder.
 * SW7405-4180: DirectFB: Require independent setting of graphics
 *  resolution on multiple output paths.
 * SW7405-4184: Allow independent mode and resolution to be specified.
 * SW7405-4185: DirectFB: Cache flushing issues when changing between GPU
 *  and CPU.
 * 
 * DirectFB_1_4_Port/15   1/25/10 10:24a robertwm
 * SW7405-3827: DirectFB: system module not including
 *  bcmnexus_gfx_defines.h.
 * 
 * DirectFB_1_4_Port/14   1/21/10 1:35p robertwm
 * SW7405-3814: DirectFB: Changing surface configuration when in
 *  DLSCL_EXCLUSIVE layer mode causes seg fault.
 * 
 * DirectFB_1_4_Port/13   1/20/10 5:30p robertwm
 * SW7405-3814: DirectFB: Changing surface configuration when in
 *  DLSCL_EXCLUSIVE layer mode causes seg fault.
 * 
 * DirectFB_1_4_Port/12   1/4/10 6:01p robertwm
 * SW7405-3701: DirectFB: Different video and graphics resolutions.
 * 
 * DirectFB_1_4_Port/11   12/23/09 10:19p robertwm
 * SW7420-510: DirectFB: Support Vertical Scaler in Graphics Feeder.
 * 
 * DirectFB_1_4_Port/10   12/23/09 9:56p robertwm
 * SW7420-510: DirectFB: Support Vertical Scaler in Graphics Feeder.
 * 
 * DirectFB_1_4_Port/9   12/23/09 2:23p robertwm
 * SW7550-51: Support DirectFB on 7550.
 * 
 * DirectFB_1_4_Port/SW7550-51/1   12/22/09 3:22p mphillip
 * SW7550-51: Add 7550 build support (on branch)
 * 
 * DirectFB_1_4_Port/8   12/16/09 9:49p robertwm
 * SW7400-2643: DirectFB: Add support for setting/getting video modes.
 * 
 * DirectFB_1_4_Port/7   12/4/09 9:01p robertwm
 * SW7405-3550: DirectFB: BMEM: Back guard violation in block!
 * 
 * DirectFB_1_4_Port/6   12/4/09 7:40p robertwm
 * SW7405-3529: cacheflush issues (fixed flush when using sub-surfaces).
 * 
 * DirectFB_1_4_Port/5   12/3/09 1:37p robertwm
 * SW7405-3529: Cacheflush issues.
 * 
 * DirectFB_1_4_Port/4   11/25/09 11:08a robertwm
 * SW7405-3438: DirectFB: Palette based blits do not work correctly.
 * SW7405-3471: Nexus needs to support additional pixel formats for
 *  DirectFB.
 * 
 * DirectFB_1_4_Port/3   8/21/09 11:12a robertwm
 * PR57917: DirectFB: testman and testrun don't run anymore in 1.4.1
 *  (SaWMan)
 * 
 * DirectFB_1_4_Port/2   8/7/09 3:45p robertwm
 * PR56902: Require single graphics memory heap to avoid fragmentation.
 * 
 * DirectFB_1_4_Port/1   7/8/09 3:19p mphillip
 * PR56658: Add DirectFB 1.4.1 broadcom_files
 * 
 *****************************************************************************/

#include "shim_nexus.h" // Nexus function redirection

#include <string.h>
#include "bcmnexus.h"
#include "bcmnexus_pool.h"
#include "bcmnexus_utils.h"

#include <config.h>

#include <direct/debug.h>
#include <direct/mem.h>

#include <fusion/shmalloc.h>

#include <core/core.h>
#include <core/gfxcard.h>
#include <core/system.h>
#include <core/surface_pool.h>
#include <core/palette.h>
#include <core/layers.h>
#include <core/screens.h>

#include <directfb_util.h>

#ifdef __GNUC__
#define UNUSED_ __attribute__ ((unused))
#else
#define UNUSED_
#endif 

/* Uncomment out the line below to use Nexus memory that has been pre-allocated 
   for creating surfaces... */
/*#define PREALLOCATE_MEMORY */

#define CYAN_COLOUR "\033[36m"
#define BLUE_COLOUR "\033[34m"
#define NO_COLOUR "\033[0m"

D_DEBUG_DOMAIN( bcmnexusPool,        "bcmNexus/Pool",        "Broadcom NEXUS Surface Pool" );
D_DEBUG_DOMAIN( bcmnexusPoolX,       "bcmNexus/PoolX",       "Broadcom NEXUS Surface Pool eXtended Debug" );
D_DEBUG_DOMAIN( bcmnexusPoolPalette, "bcmNexus/PoolPalette", "Broadcom NEXUS Surface Pool Palette Debug" );

/**********************************************************************************************************************/

struct bcmPoolData {
     FusionSHMPoolShared         *pool;             /* this is where the memory is */
     DFBBCMNEXUS                 *bcmnexus;         /* shared struct to NEXUS handles e.a. */
     NEXUS_PlatformConfiguration  platformConfig;
     NEXUS_PlatformSettings       platformSettings;
};

/**********************************************************************************************************************/

static NEXUS_Error
bcmMemoryAllocate( size_t numBytes, const NEXUS_MemoryAllocationSettings *pSettings, void **ppMemory )
{
     NEXUS_Error  result;
     DFBBCMNEXUS *bcmnexus;

     if (dfb_core_is_master( dfb_bcmnexus_core ))
          result = NEXUS_Memory_Allocate(numBytes, pSettings, ppMemory);
     else
     {
          bcmPoolCallMemoryAllocate *pCallMemoryAllocateArgs;

          pCallMemoryAllocateArgs = SHMALLOC( dfb_core_shmpool(dfb_bcmnexus_core), sizeof(*pCallMemoryAllocateArgs) );

          if (!pCallMemoryAllocateArgs)
          {
              D_ERROR("%s(): Out of memory!!!\n", __FUNCTION__);
              return NEXUS_OUT_OF_SYSTEM_MEMORY;
          }

          pCallMemoryAllocateArgs->numBytes = numBytes;
          direct_memcpy( &pCallMemoryAllocateArgs->settings, pSettings, sizeof(*pSettings) );

          bcmnexus = dfb_system_data();

          fusion_call_execute( &GET_CALL_HANDLER(bcmnexus), FCEF_NONE, BCM_POOL_MEMORY_ALLOCATE, pCallMemoryAllocateArgs, (int *)&result );

          *ppMemory = pCallMemoryAllocateArgs->pMemory;

          SHFREE( dfb_core_shmpool(dfb_bcmnexus_core), pCallMemoryAllocateArgs );
     }
     return result;
}

static void
bcmMemoryFree( void *pMemory )
{
     DFBBCMNEXUS *bcmnexus = dfb_system_data();

     if (dfb_core_is_master( dfb_bcmnexus_core ))
          NEXUS_Memory_Free(pMemory);
     else
          fusion_call_execute( &GET_CALL_HANDLER(bcmnexus), FCEF_NONE, BCM_POOL_MEMORY_FREE, pMemory, NULL );
}

static NEXUS_SurfaceHandle
bcmSurfaceCreate(const NEXUS_SurfaceCreateSettings *pCreateSettings)
{
    NEXUS_SurfaceHandle surface;
    DFBBCMNEXUS        *bcmnexus;

     if (dfb_core_is_master( dfb_bcmnexus_core ))
          surface = NEXUS_Surface_Create(pCreateSettings);
     else
     {
          bcmPoolCallSurfaceCreate *pCallSurfaceCreateArgs;

          pCallSurfaceCreateArgs = SHMALLOC( dfb_core_shmpool(dfb_bcmnexus_core), sizeof(*pCallSurfaceCreateArgs) );

          if (!pCallSurfaceCreateArgs)
          {
              D_ERROR("%s(): Out of memory!!!\n", __FUNCTION__);
              return NULL;
          }

          direct_memcpy( &pCallSurfaceCreateArgs->createSettings, pCreateSettings, sizeof(*pCreateSettings) );

          bcmnexus = dfb_system_data();

          fusion_call_execute( &GET_CALL_HANDLER(bcmnexus), FCEF_NONE, BCM_POOL_SURFACE_CREATE, pCallSurfaceCreateArgs, NULL );

          surface = pCallSurfaceCreateArgs->surface;

          SHFREE( dfb_core_shmpool(dfb_bcmnexus_core), pCallSurfaceCreateArgs );
     }
     return surface;
}


static void
bcmSurfaceDestroy(NEXUS_SurfaceHandle handle)
{
    DFBBCMNEXUS *bcmnexus;

    if (dfb_core_is_master( dfb_bcmnexus_core ))
        NEXUS_Surface_Destroy(handle);
    else
    {
        bcmnexus = dfb_system_data();
        fusion_call_execute( &GET_CALL_HANDLER(bcmnexus), FCEF_NONE, BCM_POOL_SURFACE_DESTROY, handle, NULL );
    }
}

static DFBResult
createNexusSurface(         CoreSurfaceAllocation    *allocation,
                            struct bcmAllocationData *ad,
                    UNUSED_ struct bcmPoolData       *pd )
{
     CoreSurfaceBuffer             *buffer;
     CoreSurface                   *surface;
     CorePalette                   *palette;
     NEXUS_SurfaceCreateSettings    createSettings;
     NEXUS_MemoryAllocationSettings paletteMemorySettings;
     NEXUS_PixelFormat              eFormat;
     NEXUS_SurfaceMemory            memory;
#ifdef NEXUS_MAX_HEAPS
     NEXUS_MemoryStatus             memoryStatus;
#endif

     buffer  = allocation->buffer;
     surface = buffer->surface;
     palette = surface->palette;

     eFormat = bcmnexus_getNexusPixelFormat(buffer->format);

     if (eFormat == NEXUS_PixelFormat_eUnknown)
     {
        D_ERROR( "%s: unsupported format %s\n",__FUNCTION__, dfb_pixelformat_name(surface->config.format) );
        return DFB_UNSUPPORTED;
     }

     /* IMPORTANT: The GFD doesn't support 15 or 24-bit formats, so we need to make sure
        that the pixelformat for the primary layer isn't one of these formats. */
     if (surface->type & CSTF_LAYER)
     {
#ifdef BCMNEXUS_PIXELFORMAT_R8_G8_B8_SUPPORTED
         if (eFormat == NEXUS_PixelFormat_eR8_G8_B8)
         {
            D_WARN("bcmNexus/Pool: GFD cannot support DSPF_RGB24, defaulting to DSPF_RGB32 instead.\n");
            eFormat = bcmnexus_getNexusPixelFormat(DSPF_RGB32);
         }
#endif
#ifndef BCMNEXUS_GFD0_HAS_VERTICAL_SCALER
         /* Check to see if we are creating the actual framebuffer.  If so, then this needs to
            have a maximum height of all possible TV resolutions as we don't have a vertical
            scaler in the graphics feeder path.
         */
         if (surface->resource_id == DLID_PRIMARY) 
         {
             surface->config.min_size.h = MAX(surface->config.min_size.h, BCMNEXUS_MAX_HD_DISPLAY_HEIGHT);
         }
#endif
#ifndef BCMNEXUS_GFD1_HAS_VERTICAL_SCALER
         /* Check to see if we are creating the actual framebuffer.  If so, then this needs to
            have a maximum height of all possible TV resolutions as we don't have a vertical
            scaler in the graphics feeder path.
         */
         if (surface->resource_id == (DLID_PRIMARY+1)) 
         {
             surface->config.min_size.h = MAX(surface->config.min_size.h, BCMNEXUS_MAX_SD_DISPLAY_HEIGHT);
         }
#endif
     }
     dfb_gfxcard_calc_buffer_size( dfb_gfxcard_get_primary(), buffer, &ad->pitch, &ad->size );

     NEXUS_Surface_GetDefaultCreateSettings(&createSettings);

     /* Always ensure that the palette is created in heap 0 for off-screen surfaces, as otherwise the surface won't be
        blitted correctly on platforms such as the 7420 where RTS dictates that the palette and the M2MC
        packet-buffer must be on heap 0. If the surface is the actual framebuffer (GFD), then the palette must be on
        heap 1 for the 7420, because there is no bandwidth allocated on memc0 (client 29 on mem0 is disabled).
     */
     if (DFB_PIXELFORMAT_IS_INDEXED(surface->config.format)) {
          NEXUS_Memory_GetDefaultAllocationSettings(&paletteMemorySettings);
          paletteMemorySettings.alignment = 32;  /* Palette needs to be aligned on a 32-byte boundary */

#ifdef NEXUS_MAX_HEAPS
          if (surface->type & CSTF_LAYER)
              paletteMemorySettings.heap = GET_DISPLAY_HEAP_HND(pd->bcmnexus);
          else
              paletteMemorySettings.heap = GET_PALETTE_HEAP_HND(pd->bcmnexus);
#endif
          if (bcmMemoryAllocate(palette->num_entries*sizeof(NEXUS_Pixel), &paletteMemorySettings, &createSettings.pPaletteMemory) != NEXUS_SUCCESS)
          {
              D_ERROR("%s: Error creating Memory of %d bytes for palette!\n", __FUNCTION__, palette->num_entries*sizeof(NEXUS_Pixel));
              return DFB_NOSHAREDMEMORY;
          }

          /* Save address of palette for later */
          ad->palette = createSettings.pPaletteMemory;

          D_DEBUG_AT( bcmnexusPool, CYAN_COLOUR"%s() Palette memory=%p heap=%p [%d entries]\n"NO_COLOUR,
                      __FUNCTION__, (void *)createSettings.pPaletteMemory, (void *)paletteMemorySettings.heap, palette->num_entries);

     }

     /* Get requested width/height settings */
     createSettings.width  = surface->config.size.w;
     createSettings.height = surface->config.size.h;

     /* If we have created a pre-allocated surface in BMEM, then we don't have to allocate any surface data */
     if ((surface->type & CSTF_PREALLOCATED) && (buffer->policy == CSP_VIDEOONLY)) {
          int index = dfb_surface_buffer_index( buffer );

          D_ASSERT( surface->config.preallocated[index].addr != NULL );
          D_ASSERT( surface->config.preallocated[index].pitch != 0 );

          createSettings.pixelFormat = eFormat;
          createSettings.pMemory     = surface->config.preallocated[index].addr;
          createSettings.pitch       = surface->config.preallocated[index].pitch;
#ifdef NEXUS_MAX_HEAPS
          createSettings.heap        = (NEXUS_HeapHandle) surface->config.preallocated[index].handle;
#endif
     }
     else {
          createSettings.pixelFormat    = eFormat;
          createSettings.alignment      = 12; /* Ensure surface is aligned on 4096Kbyte boundary to help prevent "cache crosstalk" */
          createSettings.pitch          = ad->pitch;
          createSettings.height         = MAX(surface->config.size.h, surface->config.min_size.h);

#ifdef NEXUS_MAX_HEAPS
          /* This surface is allocated on video layer? */
          if ((surface->type & CSTF_LAYER) && (surface->resource_id == (DLID_PRIMARY+2))) { 
              createSettings.heap       = GET_VIDEO_HEAP_HND(pd->bcmnexus);
          }
#endif
     }

#ifdef NEXUS_MAX_HEAPS
     if (createSettings.heap == NULL)
     {
#ifdef PREALLOCATE_MEMORY
          NEXUS_MemoryAllocationSettings surfaceMemorySettings;
#endif

          if (surface->type & CSTF_LAYER) 
              createSettings.heap = GET_DISPLAY_HEAP_HND(pd->bcmnexus);
          else
              createSettings.heap = GET_OFFSCREEN_HEAP_HND(pd->bcmnexus);

          D_ASSERT(createSettings.heap);

          /* Check that we can actually allocate this amount of memory from our desired pool.
           * If not fall back to the secondry pool if possible */
          NEXUS_Heap_GetStatus(createSettings.heap, &memoryStatus);
           
          if (memoryStatus.largestFreeBlock < (unsigned int)ad->size )
          { 
              // If our secondary pool is full we can try the primary
              // but not the other way around
              if (surface->type != CSTF_LAYER) 
              {
                  createSettings.heap = GET_DISPLAY_HEAP_HND(pd->bcmnexus);

                  NEXUS_Heap_GetStatus(createSettings.heap, &memoryStatus);
              }

              if (memoryStatus.largestFreeBlock < (unsigned int)ad->size)
              {
                  D_ERROR( "bcmNexus/Pool: Insufficient memory to create surface (%d bytes short)\n", ad->size - memoryStatus.largestFreeBlock);
                  return DFB_LIMITEXCEEDED;
              }  
          } 
#ifdef PREALLOCATE_MEMORY
          NEXUS_Memory_GetDefaultAllocationSettings(&surfaceMemorySettings);

          surfaceMemorySettings.alignment = 4096;  /* Surface needs to be aligned on a 4K boundary */
          surfaceMemorySettings.heap = createSettings.heap;
          if (bcmMemoryAllocate((size_t)ad->size, &surfaceMemorySettings, &createSettings.pMemory) != NEXUS_SUCCESS)
          {
              D_ERROR( "bcmNexus/Pool: Insufficient memory to create surface (%d bytes)\n", ad->size);
              return DFB_NOVIDEOMEMORY;
          }
#endif
     }
#endif

     ad->bcmSurfaceHandle = bcmSurfaceCreate(&createSettings);
     if (ad->bcmSurfaceHandle==NULL) {
         D_ERROR( "bcmNexus/Pool: NEXUS_Surface_Create failed!\n" );
         return DFB_LIMITEXCEEDED;
     }

     NEXUS_Surface_GetMemory(ad->bcmSurfaceHandle, &memory);

     /* store buffer info */
     ad->allocation   = allocation;
     ad->address      = memory.buffer;
     ad->pixelformat  = createSettings.pixelFormat;

#ifdef NEXUS_MAX_HEAPS
     NEXUS_Heap_GetStatus(createSettings.heap, &memoryStatus);
     ad->heap         = createSettings.heap;
     ad->offsetInHeap = (uint8_t*)memory.buffer - (uint8_t*)memoryStatus.addr;
     ad->hwaddress    = memoryStatus.offset + ad->offsetInHeap;
#endif

     D_DEBUG_AT( bcmnexusPool, CYAN_COLOUR"%s: Nexus_Surface_Created() %s: sur handle %p vaddr=%p hwaddr=%p [heap: handle %p, vaddr=%p hwaddr=%p]"
                               ", WxH=%dx%d, pitch=%d bytes, size=%d bytes, dfb_pixelformat=%s\n"NO_COLOUR, __FUNCTION__,
                               (surface->type & CSTF_LAYER) ? "framebuffer" : "offscreen",
                               (void*)(ad->bcmSurfaceHandle), (void *)ad->address, (void *)ad->hwaddress,
                               (void *)ad->heap, (void *)memoryStatus.addr, (void *)memoryStatus.offset,
                               createSettings.width, createSettings.height, ad->pitch, ad->size, dfb_pixelformat_name(buffer->format) );

     /* If a layer, then clear memory first */
     if ((surface->type & CSTF_LAYER) && (surface->resource_id != (DLID_PRIMARY+2)))
     {
         memset(ad->address, 0, createSettings.height * createSettings.pitch);
     }

     /* Load palette if it has one */
     if (DFB_PIXELFORMAT_IS_INDEXED(surface->config.format)) {
          unsigned int i;

#ifdef NEXUS_MAX_HEAPS
          NEXUS_Heap_GetStatus(paletteMemorySettings.heap, &memoryStatus);
          ad->hwpalette = memoryStatus.offset + (uint8_t*)memory.palette - (uint8_t*)memoryStatus.addr;
#endif   

          for (i=0; i<palette->num_entries; i++)
          {
               memory.palette[i] = PIXEL_ARGB(
                    palette->entries[i].a,
                    palette->entries[i].r,
                    palette->entries[i].g,
                    palette->entries[i].b);
               D_DEBUG_AT( bcmnexusPoolPalette, CYAN_COLOUR"Palette entry %d: 0x%08x\n"NO_COLOUR, i, memory.palette[i] );
          }
          memory.numPaletteEntries = palette->num_entries;
          D_DEBUG_AT( bcmnexusPool, CYAN_COLOUR"Palette copied to %p [num entries=%d]\n"NO_COLOUR, (void *)memory.palette, memory.numPaletteEntries );
     }

     return DFB_OK;
}

/**********************************************************************************************************************/

static int bcmPoolDataSize( void )
{
     return sizeof(struct bcmPoolData);
}

static int bcmPoolLocalDataSize( void )
{
     return 0;
}

static int bcmAllocationDataSize( void )
{
     return sizeof(struct bcmAllocationData);
}

static DFBResult
bcmInitPool(         CoreDFB                    *core,
                     CoreSurfacePool            *pool,
                     void                       *pool_data,
             UNUSED_ void                       *pool_local,
             UNUSED_ void                       *system_data,
                     CoreSurfacePoolDescription *ret_desc )
{
     struct bcmPoolData *pd = pool_data;

     D_DEBUG_AT( bcmnexusPool, CYAN_COLOUR"%s()\n"NO_COLOUR, __FUNCTION__ );

     D_MAGIC_ASSERT( pool, CoreSurfacePool );
     D_ASSERT( ret_desc != NULL );

     pd->pool     = dfb_core_shmpool( core );
     pd->bcmnexus = dfb_system_data ( );

     NEXUS_Platform_GetConfiguration(&pd->platformConfig);
     NEXUS_Platform_GetSettings(&pd->platformSettings);

     ret_desc->caps = CSPCAPS_VIRTUAL | CSPCAPS_PHYSICAL;

     ret_desc->access[CSAID_CPU]    = CSAF_READ | CSAF_WRITE | CSAF_SHARED;
     ret_desc->access[CSAID_GPU]    = CSAF_READ | CSAF_WRITE | CSAF_SHARED;
     ret_desc->access[CSAID_LAYER0] = CSAF_READ;
     ret_desc->access[CSAID_LAYER1] = CSAF_READ;
     ret_desc->access[CSAID_LAYER2] = CSAF_READ;
     ret_desc->access[CSAID_LAYER3] = CSAF_READ;
     ret_desc->access[CSAID_LAYER4] = CSAF_READ;
     ret_desc->access[CSAID_LAYER5] = CSAF_READ;
     ret_desc->access[CSAID_LAYER6] = CSAF_READ;
     ret_desc->access[CSAID_LAYER7] = CSAF_READ;

     ret_desc->types    = CSTF_LAYER | CSTF_WINDOW | CSTF_CURSOR | CSTF_FONT | CSTF_SHARED | CSTF_EXTERNAL | CSTF_PREALLOCATED;
     ret_desc->priority = CSPP_PREFERED;

     snprintf( ret_desc->name, DFB_SURFACE_POOL_DESC_NAME_LENGTH, "Broadcom Surface Layers" );

     /* TODO: Add size field */

     return DFB_OK;
}

static DFBResult
bcmJoinPool( UNUSED_ CoreDFB                    *core,
                     CoreSurfacePool            *pool,
             UNUSED_ void                       *pool_data,
             UNUSED_ void                       *pool_local,
             UNUSED_ void                       *system_data )
{
     D_DEBUG_AT( bcmnexusPool, CYAN_COLOUR"%s()\n"NO_COLOUR, __FUNCTION__ );

     D_MAGIC_ASSERT( pool, CoreSurfacePool );

     return DFB_OK;
}

static DFBResult
bcmDestroyPool(         CoreSurfacePool *pool,
                UNUSED_ void            *pool_data,
                UNUSED_ void            *pool_local )
{
     D_DEBUG_AT( bcmnexusPool, CYAN_COLOUR"%s()\n"NO_COLOUR, __FUNCTION__ );

     D_MAGIC_ASSERT( pool, CoreSurfacePool );

     return DFB_OK;
}

static DFBResult
bcmLeavePool(         CoreSurfacePool *pool,
              UNUSED_ void            *pool_data,
              UNUSED_ void            *pool_local )
{
     D_DEBUG_AT( bcmnexusPool, CYAN_COLOUR"%s()\n"NO_COLOUR, __FUNCTION__ );

     D_MAGIC_ASSERT( pool, CoreSurfacePool );

     return DFB_OK;
}

static DFBResult
bcmTestConfig( UNUSED_ CoreSurfacePool         *pool,
               UNUSED_ void                    *pool_data,
               UNUSED_ void                    *pool_local,
               UNUSED_ CoreSurfaceBuffer       *buffer,
                       const CoreSurfaceConfig *config )
{
     NEXUS_PixelFormat eFormat;

     D_DEBUG_AT( bcmnexusPool, CYAN_COLOUR"%s() format=%s\n"NO_COLOUR, __FUNCTION__, dfb_pixelformat_name(config->format) );

     D_ASSERT( config != NULL );

     eFormat = bcmnexus_getNexusPixelFormat(config->format);

     if (eFormat == NEXUS_PixelFormat_eUnknown)
     {
        D_DEBUG_AT( bcmnexusPool, CYAN_COLOUR"%s: unsupported format %s\n"NO_COLOUR, __FUNCTION__, dfb_pixelformat_name(config->format) );
        return DFB_UNSUPPORTED;
     }

     if ((buffer->surface->type & CSTF_PREALLOCATED) &&
         !config->preallocated[dfb_surface_buffer_index(buffer)].handle)
     {
          D_DEBUG_AT( bcmnexusPool, "%s: no handle for preallocation\n", __FUNCTION__ );
          return DFB_UNSUPPORTED;
     }

     return DFB_OK;
}

static DFBResult
bcmAllocateBuffer( UNUSED_ CoreSurfacePool       *pool,
                           void                  *pool_data,
                   UNUSED_ void                  *pool_local,
                           CoreSurfaceBuffer     *buffer,
                           CoreSurfaceAllocation *allocation,
                           void                  *alloc_data )
{
     DFBResult                 result;
     CoreSurface              *surface;
     struct bcmAllocationData *ad = alloc_data;

     surface = buffer->surface;

     D_DEBUG_AT( bcmnexusPool,
                 CYAN_COLOUR"%s() surface=%p, buffer=%p, surface type=0x%08x, caps=0x%08x, size=%dx%d, format=%s, policy=0x%08x [master=%c]\n"NO_COLOUR,
                 __FUNCTION__, (void *)surface, (void *)buffer,  surface->type, surface->config.caps, surface->config.size.w, surface->config.size.h,
                 dfb_pixelformat_name(buffer->format), buffer->policy, dfb_core_is_master( dfb_bcmnexus_core ) ? 'y' : 'n' );

     /* allocate the NEXUS surface */
     result = createNexusSurface( allocation, ad, pool_data );
     if (result == DFB_OK) {
         allocation->size = ad->size;
         if (!((surface->type & CSTF_LAYER) && (surface->resource_id == (DLID_PRIMARY+2))))
         { 
             D_DEBUG_AT( bcmnexusPool, CYAN_COLOUR"%s() allocation=%d, flush handle=%p\n"NO_COLOUR, __FUNCTION__, 
                         allocation->size, (void *)ad->bcmSurfaceHandle);
             NEXUS_Surface_Flush(ad->bcmSurfaceHandle);
         }
     }
     return result;
}

static DFBResult
bcmDeallocateBuffer( UNUSED_ CoreSurfacePool       *pool,
                             void                  *pool_data,
                     UNUSED_ void                  *pool_local,
                             CoreSurfaceBuffer     *buffer,
                     UNUSED_ CoreSurfaceAllocation *allocation,
                             void                  *alloc_data )
{
     struct bcmAllocationData *ad = alloc_data;
     struct bcmPoolData       *pd = pool_data;
     CoreSurface              *surface;
     NEXUS_SurfaceMemory       mem;

     surface = buffer->surface;
 
     D_DEBUG_AT( bcmnexusPool,
                 CYAN_COLOUR"%s() surface=%p, buffer=%p, num_bufs=%d, handle=%p [addr: %p], type=0x%08x, size=%dx%d, format=%s [master=%c]\n"NO_COLOUR,
                 __FUNCTION__, (void *)surface, (void *)buffer, surface->num_buffers, (void *)ad->bcmSurfaceHandle, ad->address, surface->type,
                 surface->config.size.w, surface->config.size.h, dfb_pixelformat_name(buffer->format),
                 dfb_core_is_master( dfb_bcmnexus_core ) ? 'y' : 'n' );

     /* deallocate the NEXUS surface */
     if (ad->bcmSurfaceHandle) {

          /* Ensure the graphics h/w has finished before removing memory! */
#if NEXUS_HAS_GRAPHICS2D || NEXUS_HAS_ZSP_GRAPHICS
          if ((ad->syncIssued == false) || GET_GRAPHICS2D_SYNC(pd->bcmnexus))
#else
          if (true)
#endif
          {
              D_DEBUG_AT( bcmnexusPool, CYAN_COLOUR"%s() sync\n"NO_COLOUR, __FUNCTION__);
              dfb_gfxcard_sync();
          } else {
              ad->syncIssued = false;
          }

          if (!((surface->type & CSTF_LAYER) && (surface->resource_id == (DLID_PRIMARY+2))))
          { 
              D_DEBUG_AT( bcmnexusPool, CYAN_COLOUR"%s() flush handle=%p\n"NO_COLOUR, __FUNCTION__, (void *)ad->bcmSurfaceHandle);
              NEXUS_Surface_Flush(ad->bcmSurfaceHandle);
          }

          /* Free up the palette if it was created */
          if (ad->palette != NULL) {
              D_DEBUG_AT( bcmnexusPool, CYAN_COLOUR"%s() About to get surface handle %p memory\n"NO_COLOUR, __FUNCTION__, (void *)ad->bcmSurfaceHandle );
              NEXUS_Surface_GetMemory(ad->bcmSurfaceHandle, &mem); 
              D_DEBUG_AT( bcmnexusPool, CYAN_COLOUR"%s() About to free palette memory %p\n"NO_COLOUR, __FUNCTION__, (void *)ad->palette );
              bcmMemoryFree(ad->palette);
              ad->palette = NULL;
          }

          /* If not a pre-allocated surface, then we can free up the Nexus memory */
#ifdef PREALLOCATE_MEMORY
          if (!(surface->type & CSTF_PREALLOCATED))
          {
              D_DEBUG_AT( bcmnexusPool, CYAN_COLOUR"%s() About to get surface handle %p memory\n"NO_COLOUR, __FUNCTION__, (void *)ad->bcmSurfaceHandle );
              NEXUS_Surface_GetMemory(ad->bcmSurfaceHandle, &mem); 
              D_DEBUG_AT( bcmnexusPool, CYAN_COLOUR"%s() About to free buffer memory %p\n"NO_COLOUR, __FUNCTION__, (void *)mem.buffer );
              bcmMemoryFree(mem.buffer);
          }
#endif

          bcmSurfaceDestroy( ad->bcmSurfaceHandle );
          ad->bcmSurfaceHandle = NULL;
     }

     D_DEBUG_AT( bcmnexusPool, CYAN_COLOUR"%s() exiting.\n"NO_COLOUR, __FUNCTION__ );
     return DFB_OK;
}

static DFBResult
bcmLock(         CoreSurfacePool       *pool,
                 void                  *pool_data,
         UNUSED_ void                  *pool_local,
                 CoreSurfaceAllocation *allocation,
                 void                  *alloc_data,
                 CoreSurfaceBufferLock *lock )
{
     CoreSurfaceAccessorID     accessor;
     CoreSurfaceAccessFlags    access;
     CoreSurface              *surface;
     CoreSurfaceBuffer        *buffer;
     struct bcmPoolData       *pd       = pool_data;
     struct bcmAllocationData *ad       = alloc_data;
     bool                      flush_me = false;
     bool                      sync_me  = false;

     D_MAGIC_ASSERT( pool, CoreSurfacePool );
     D_MAGIC_ASSERT( allocation, CoreSurfaceAllocation );
     D_MAGIC_ASSERT( lock, CoreSurfaceBufferLock );

     buffer = allocation->buffer;
     D_MAGIC_ASSERT( buffer, CoreSurfaceBuffer );

     surface = buffer->surface;
     D_MAGIC_ASSERT( surface, CoreSurface );

     D_DEBUG_AT( bcmnexusPoolX, BLUE_COLOUR"%s() surface=%p, buffer=%p, addr=%p, pitch=%d, handle=%p accessor=%d, access=0x%08x, CPU accessed=0x%08x, "
                                "GPU accessed=0x%08x, type=0x%08x, flags=0x%08x, caps=0x%08x\n"NO_COLOUR,
                 __FUNCTION__, (void *)surface, (void *)lock->buffer, ad->address, ad->pitch, (void*)ad->bcmSurfaceHandle, lock->accessor, lock->access,
                 allocation->accessed[CSAID_CPU], allocation->accessed[CSAID_GPU], surface->type, surface->config.flags, surface->config.caps );

     /* Setup handle to surface so that graphics driver can access it */
     lock->handle = ad->bcmSurfaceHandle;

     access   = lock->access;
     accessor = lock->accessor;

     /* Check Layer lock */
     if (accessor == (CSAID_LAYER0 + allocation->surface->resource_id))
     {
          /* ...if software has written before... */
          if (allocation->accessed[CSAID_CPU] & CSAF_WRITE) {
              flush_me = true;
              /* Clear CPU W flags to avoid unwanted flushes */
              if (!lock->buffer->locked)
                   allocation->accessed[CSAID_CPU] &= ~CSAF_WRITE;
          }

          /* If hardware has written or is writing... */
          if (allocation->accessed[CSAID_GPU] & CSAF_WRITE) {
               sync_me = true;
               flush_me = true;
               /* Clear GPU R flag to avoid unwanted flushes and syncs */
               if (!lock->buffer->locked)
                   allocation->accessed[CSAID_GPU] &= ~CSAF_WRITE;
          }
     }

     /* Software read/write access... */
     if (accessor == CSAID_CPU) {
          /* If hardware has written or is writing... */
          if (allocation->accessed[CSAID_GPU] & CSAF_WRITE) {
               /* ...wait for the operation to finish. */
               sync_me = true;
               /* Software read access after hardware write requires flush of the (bus) read cache. */
               flush_me = true;

               if (!lock->buffer->locked) {
                    /* ...clear hardware write access. */
                    allocation->accessed[CSAID_GPU] &= ~CSAF_WRITE;
                    /* ...clear hardware read access (to avoid syncing twice). */
                    allocation->accessed[CSAID_GPU] &= ~CSAF_READ;
               }
          }

          /* Software write access... */
          if (access & CSAF_WRITE) {
               /* ...if hardware has (to) read... */
               if (allocation->accessed[CSAID_GPU] & CSAF_READ) {
                    /* ...wait for the operation to finish. */
                    sync_me = true;

                    /* ...clear hardware read access. */
                    if (!lock->buffer->locked)
                         allocation->accessed[CSAID_GPU] &= ~CSAF_READ;
               }
          }
     }

     /* Hardware read access... */
     if (accessor == CSAID_GPU) {
          /* ...if software has written before... */
          if (allocation->accessed[CSAID_CPU] & CSAF_WRITE) {
               /* ...flush texture cache. */
               flush_me = true;

               /* ...clear software write access. */
               if (!lock->buffer->locked)
                    allocation->accessed[CSAID_CPU] &= ~CSAF_WRITE;
          }
     }

     /* Do we need to perform a graphics sync operation? */
     if (sync_me == true)
     {
#if NEXUS_HAS_GRAPHICS2D || NEXUS_HAS_ZSP_GRAPHICS  
         if ((ad->syncIssued == false) || GET_GRAPHICS2D_SYNC(pd->bcmnexus))
#else
         if (true)
#endif
         {
             D_DEBUG_AT( bcmnexusPoolX, BLUE_COLOUR"%s() sync\n"NO_COLOUR, __FUNCTION__);
             dfb_gfxcard_sync();
         } else {
             ad->syncIssued = false;
         }
     }

     /* Do we need to flush the surface cache? */
     if ((flush_me == true) && !((surface->type & CSTF_LAYER) && (surface->resource_id == (DLID_PRIMARY+2))))
     {
         D_DEBUG_AT( bcmnexusPoolX, BLUE_COLOUR"%s() flush handle=%p\n"NO_COLOUR, __FUNCTION__, (void *)ad->bcmSurfaceHandle);
         NEXUS_Surface_Flush(ad->bcmSurfaceHandle);
     }

     lock->addr   = ad->address;
     lock->phys   = (unsigned long)ad->hwaddress;
     lock->offset = 0;         /* irrelevant */
     lock->pitch  = ad->pitch; /* ad->pitch */
     
     D_DEBUG_AT( bcmnexusPoolX, BLUE_COLOUR"%s() Exit\n"NO_COLOUR, __FUNCTION__ );
     return DFB_OK;
 
}

static DFBResult
bcmUnlock(         CoreSurfacePool       *pool,
           UNUSED_ void                  *pool_data,
           UNUSED_ void                  *pool_local,
                   CoreSurfaceAllocation *allocation,
                   void                  *alloc_data,
                   CoreSurfaceBufferLock *lock )
{
     struct bcmAllocationData *ad = alloc_data;
     CoreSurface              *surface;

     D_MAGIC_ASSERT( pool, CoreSurfacePool );
     D_MAGIC_ASSERT( allocation, CoreSurfaceAllocation );
     D_MAGIC_ASSERT( lock, CoreSurfaceBufferLock );

     surface = allocation->surface;

     D_DEBUG_AT( bcmnexusPoolX, BLUE_COLOUR"%s() surface=%p, buffer=%p, addr=%p, pitch=%d, handle=%p accessor=%d, access=0x%08x, CPU accessed=0x%08x, "
                                "GPU accessed=0x%08x, type=0x%08x, flags=0x%08x, caps=0x%08x\n"NO_COLOUR,
                 __FUNCTION__, (void *)surface, (void *)lock->buffer, ad->address, ad->pitch, (void*)ad->bcmSurfaceHandle, lock->accessor, lock->access,
                 allocation->accessed[CSAID_CPU], allocation->accessed[CSAID_GPU], surface->type, surface->config.flags, surface->config.caps );

     return DFB_OK;
}

const SurfacePoolFuncs bcmnexusPoolFunctions = {
     .PoolDataSize       = bcmPoolDataSize,
     .PoolLocalDataSize  = bcmPoolLocalDataSize,
     .AllocationDataSize = bcmAllocationDataSize,
     .InitPool           = bcmInitPool,
     .JoinPool           = bcmJoinPool,
     .DestroyPool        = bcmDestroyPool,
     .LeavePool          = bcmLeavePool,
     .TestConfig         = bcmTestConfig,
     .AllocateBuffer     = bcmAllocateBuffer,
     .DeallocateBuffer   = bcmDeallocateBuffer,
     .Lock               = bcmLock,
     .Unlock             = bcmUnlock
};

