/******************************************************************************
 *    (c)2009-2011 Broadcom Corporation
 * 
 * This program is the proprietary software of Broadcom Corporation and/or its licensors,
 * and may only be used, duplicated, modified or distributed pursuant to the terms and
 * conditions of a separate, written license agreement executed between you and Broadcom
 * (an "Authorized License").  Except as set forth in an Authorized License, Broadcom grants
 * no license (express or implied), right to use, or waiver of any kind with respect to the
 * Software, and Broadcom expressly reserves all rights in and to the Software and all
 * intellectual property rights therein.  IF YOU HAVE NO AUTHORIZED LICENSE, THEN YOU
 * HAVE NO RIGHT TO USE THIS SOFTWARE IN ANY WAY, AND SHOULD IMMEDIATELY
 * NOTIFY BROADCOM AND DISCONTINUE ALL USE OF THE SOFTWARE.  
 *  
 * Except as expressly set forth in the Authorized License,
 *  
 * 1.     This program, including its structure, sequence and organization, constitutes the valuable trade
 * secrets of Broadcom, and you shall use all reasonable efforts to protect the confidentiality thereof,
 * and to use this information only in connection with your use of Broadcom integrated circuit products.
 *  
 * 2.     TO THE MAXIMUM EXTENT PERMITTED BY LAW, THE SOFTWARE IS PROVIDED "AS IS" 
 * AND WITH ALL FAULTS AND BROADCOM MAKES NO PROMISES, REPRESENTATIONS OR 
 * WARRANTIES, EITHER EXPRESS, IMPLIED, STATUTORY, OR OTHERWISE, WITH RESPECT TO 
 * THE SOFTWARE.  BROADCOM SPECIFICALLY DISCLAIMS ANY AND ALL IMPLIED WARRANTIES 
 * OF TITLE, MERCHANTABILITY, NONINFRINGEMENT, FITNESS FOR A PARTICULAR PURPOSE, 
 * LACK OF VIRUSES, ACCURACY OR COMPLETENESS, QUIET ENJOYMENT, QUIET POSSESSION 
 * OR CORRESPONDENCE TO DESCRIPTION. YOU ASSUME THE ENTIRE RISK ARISING OUT OF 
 * USE OR PERFORMANCE OF THE SOFTWARE.
 * 
 * 3.     TO THE MAXIMUM EXTENT PERMITTED BY LAW, IN NO EVENT SHALL BROADCOM OR ITS 
 * LICENSORS BE LIABLE FOR (i) CONSEQUENTIAL, INCIDENTAL, SPECIAL, INDIRECT, OR 
 * EXEMPLARY DAMAGES WHATSOEVER ARISING OUT OF OR IN ANY WAY RELATING TO YOUR 
 * USE OF OR INABILITY TO USE THE SOFTWARE EVEN IF BROADCOM HAS BEEN ADVISED OF 
 * THE POSSIBILITY OF SUCH DAMAGES; OR (ii) ANY AMOUNT IN EXCESS OF THE AMOUNT 
 * ACTUALLY PAID FOR THE SOFTWARE ITSELF OR U.S. $1, WHICHEVER IS GREATER. THESE 
 * LIMITATIONS SHALL APPLY NOTWITHSTANDING ANY FAILURE OF ESSENTIAL PURPOSE OF 
 * ANY LIMITED REMEDY.
 *
 * $brcm_Workfile: bcmnexus_pool.h $
 * $brcm_Revision: DirectFB_1_4_Port/7 $
 * $brcm_Date: 3/17/11 12:39p $
 * 
 * Module Description:
 * 
 * Revision History:
 * 
 * $brcm_Log: /AppLibs/opensource/directfb/src/broadcom_files/private/DirectFB/1.4/systems/bcmnexus/bcmnexus_pool.h $
 * 
 * DirectFB_1_4_Port/7   3/17/11 12:39p robertwm
 * SW7422-378: DirectFB: Destroying a Palettised surface in a client
 *  application causes NEXUS assertion during free.
 * 
 * DirectFB_1_4_Port/6   3/1/11 8:48p robertwm
 * SW7420-1559: DirectFB: NEXUS surface assertion in multi-app mode when
 *  closing down a DFB client.
 * 
 * DirectFB_1_4_Port/5   2/23/11 6:04p christ
 * SW7425-108: DirectFB: Insignia LUT8 tests failing on 7425 packet blit
 *  build.
 * 
 * DirectFB_1_4_Port/4   2/17/11 11:13p robertwm
 * SW7420-1510: DirectFB: Reduce number of IPC calls for gfx operations
 *  using packet-blit interface.
 * 
 * DirectFB_1_4_Port/3   8/16/10 11:40p robertwm
 * SW7405-4761: DirectFB: Optimise the surface pool locking code.
 * 
 * DirectFB_1_4_Port/2   7/22/10 4:18p robertwm
 * SW7405-4649: DirectFB: Optimise surface locking and wait idle syncing.
 * 
 * DirectFB_1_4_Port/1   7/8/09 3:19p mphillip
 * PR56658: Add DirectFB 1.4.1 broadcom_files
 * 
 *****************************************************************************/
#ifndef BCMNEXUS_POOL_H_
#define BCMNEXUS_POOL_H_

#include <core/surface_pool.h>

typedef struct bcmAllocationData {
     CoreSurfaceAllocation  *allocation;                /* Pointer to allocation */
     void                   *address;                   /* Virtual (cached) address of this allocation */
     uint32_t                hwaddress;
     NEXUS_HeapHandle        heap;
     int                     offsetInHeap;
     int                     size;
     int                     pitch;
     NEXUS_PixelFormat       pixelformat;
     bool                    syncIssued;                /* set to true to indicate that a Checkpoint/Sync was issued */
     NEXUS_SurfaceHandle     bcmSurfaceHandle;          /* If not NULL, then holds Nexus surface handle*/
     NEXUS_Pixel            *palette;                   /* If not NULL, then points to palette */
     uint32_t                hwpalette;
} bcmAllocationData;

typedef struct {
    size_t                         numBytes;
    NEXUS_MemoryAllocationSettings settings;
    void                          *pMemory;
} bcmPoolCallMemoryAllocate;

typedef struct {
    NEXUS_SurfaceHandle         surface;
    NEXUS_SurfaceCreateSettings createSettings;
} bcmPoolCallSurfaceCreate;

extern const SurfacePoolFuncs bcmnexusPoolFunctions;

#endif

