/***************************************************************************
 *     (c)2007-2011 Broadcom Corporation
 *  
 *  This program is the proprietary software of Broadcom Corporation and/or its licensors,
 *  and may only be used, duplicated, modified or distributed pursuant to the terms and
 *  conditions of a separate, written license agreement executed between you and Broadcom
 *  (an "Authorized License").  Except as set forth in an Authorized License, Broadcom grants
 *  no license (express or implied), right to use, or waiver of any kind with respect to the
 *  Software, and Broadcom expressly reserves all rights in and to the Software and all
 *  intellectual property rights therein.  IF YOU HAVE NO AUTHORIZED LICENSE, THEN YOU
 *  HAVE NO RIGHT TO USE THIS SOFTWARE IN ANY WAY, AND SHOULD IMMEDIATELY
 *  NOTIFY BROADCOM AND DISCONTINUE ALL USE OF THE SOFTWARE.  
 *   
 *  Except as expressly set forth in the Authorized License,
 *   
 *  1.     This program, including its structure, sequence and organization, constitutes the valuable trade
 *  secrets of Broadcom, and you shall use all reasonable efforts to protect the confidentiality thereof,
 *  and to use this information only in connection with your use of Broadcom integrated circuit products.
 *   
 *  2.     TO THE MAXIMUM EXTENT PERMITTED BY LAW, THE SOFTWARE IS PROVIDED "AS IS" 
 *  AND WITH ALL FAULTS AND BROADCOM MAKES NO PROMISES, REPRESENTATIONS OR 
 *  WARRANTIES, EITHER EXPRESS, IMPLIED, STATUTORY, OR OTHERWISE, WITH RESPECT TO 
 *  THE SOFTWARE.  BROADCOM SPECIFICALLY DISCLAIMS ANY AND ALL IMPLIED WARRANTIES 
 *  OF TITLE, MERCHANTABILITY, NONINFRINGEMENT, FITNESS FOR A PARTICULAR PURPOSE, 
 *  LACK OF VIRUSES, ACCURACY OR COMPLETENESS, QUIET ENJOYMENT, QUIET POSSESSION 
 *  OR CORRESPONDENCE TO DESCRIPTION. YOU ASSUME THE ENTIRE RISK ARISING OUT OF 
 *  USE OR PERFORMANCE OF THE SOFTWARE.
 *  
 *  3.     TO THE MAXIMUM EXTENT PERMITTED BY LAW, IN NO EVENT SHALL BROADCOM OR ITS 
 *  LICENSORS BE LIABLE FOR (i) CONSEQUENTIAL, INCIDENTAL, SPECIAL, INDIRECT, OR 
 *  EXEMPLARY DAMAGES WHATSOEVER ARISING OUT OF OR IN ANY WAY RELATING TO YOUR 
 *  USE OF OR INABILITY TO USE THE SOFTWARE EVEN IF BROADCOM HAS BEEN ADVISED OF 
 *  THE POSSIBILITY OF SUCH DAMAGES; OR (ii) ANY AMOUNT IN EXCESS OF THE AMOUNT 
 *  ACTUALLY PAID FOR THE SOFTWARE ITSELF OR U.S. $1, WHICHEVER IS GREATER. THESE 
 *  LIMITATIONS SHALL APPLY NOTWITHSTANDING ANY FAILURE OF ESSENTIAL PURPOSE OF 
 *  ANY LIMITED REMEDY.
 * 
 * $brcm_Workfile: bcmnexus_video.c $
 * $brcm_Revision: DirectFB_1_4_Port/4 $
 * $brcm_Date: 3/1/11 11:30a $
 *
 * [File Description:]
 *
 * Revision History:
 *
 * $brcm_Log: /AppLibs/opensource/directfb/src/broadcom_files/private/DirectFB/1.4/systems/bcmnexus/bcmnexus_video.c $
 * 
 * DirectFB_1_4_Port/4   3/1/11 11:30a robertwm
 * SW7420-1554: DirectFB: Fusion skirmish dismiss input/output error in
 *  Video Layer Code.
 * 
 * DirectFB_1_4_Port/3   2/20/11 7:32p robertwm
 * SW7420-1394: video layer support in DFB.
 * SW7422-275: DirectFB: Enabling video layer causes Segmentation fault.
 * 
 * DirectFB_1_4_Port/2   1/24/11 4:25p christ
 * SW7420-1411: DirectFB: Merge of features required for DirectFB 1.4.5
 *  phase 1.5 release - rename conflicting structure
 * 
 * DirectFB_1_4_Port/1   1/19/11 12:32p scchang
 * SW7420-1394: video layer support in DFB
 * 
 ***************************************************************************/

#include "bcmnexus.h"
#include "bcmnexus_screen.h"
#include "bcmnexus_layer.h"
#include "bcmnexus_utils.h"
#include "nexus_video_window.h"
#include "nexus_video_image_input.h"
#include "nexus_video_input.h"
#include "bdbg.h"

#include <core/core.h>
#include <core/system.h>
#include <core/palette.h>
#include <core/layers_internal.h>
#include <fusion/shmalloc.h>
#include <misc/conf.h>

#ifdef __GNUC__
#define UNUSED_ __attribute__ ((unused))
#else
#define UNUSED_
#endif 

D_DEBUG_DOMAIN( bcmnexusVideoLayer,  "bcmNexus/VideoLayer",  "Broadcom NEXUS Video Layer" );
D_DEBUG_DOMAIN( bcmnexusVideoLayerX, "bcmNexus/VideoLayerX", "Broadcom NEXUS Video Layer eXtended debugging" );

/**********************************************************************************************************************/

typedef struct {
     CoreSurfaceBufferLock      bufferLock;     /* Remember the last surface buffer lock */
} bcmVideoRegionData;

typedef struct {
    bcmVideoRegionData         *regionData;     /* Pointer to region data */
    DFBDisplayLayerID           layerId;        /* Graphics layer ID */
    DFBScreenID                 displayId;      /* Primary or secondary display/screen */
    DFBDisplayLayerSourceID     sourceId;       /* Graphics source Id */
    bool                        layerEnabled;   /* true if this layer is enabled */

    FusionSkirmish              lock;           /* for framebuffer change synchronisation */

    NEXUS_VideoWindowHandle     videoWindow;
    NEXUS_VideoImageInputHandle imageInput;
} bcmVideoLayerData;

/**********************************************************************************************************************/

static void
frameBufferCallback( void *context, int param )
{
    bcmVideoLayerData *layerData = context;

    D_DEBUG_AT( bcmnexusVideoLayerX, "  -> screen %d video layer %d framebuffer callback handled\n", layerData->displayId, param);

    fusion_skirmish_prevail( &layerData->lock );
    fusion_skirmish_notify( &layerData->lock );
    fusion_skirmish_dismiss( &layerData->lock );
}

/**********************************************************************************************************************/

static int
bcmVideoLayerDataSize( void )
{
    return sizeof(bcmVideoLayerData);
}

static int
bcmVideoRegionDataSize( void )
{
    return sizeof(bcmVideoRegionData);
}

/* initialize the layer data
 * this function is called for the master fusionee only
 */
static DFBResult
bcmVideoInitLayer(         CoreLayer                  *layer,
                           void                       *driver_data,
                           void                       *layer_data,
                           DFBDisplayLayerDescription *description,
                           DFBDisplayLayerConfig      *config,
                   UNUSED_ DFBColorAdjustment         *adjustment )
{
    DFBBCMNEXUS       *dfb_bcmnexus = driver_data;
    bcmVideoLayerData *layerData    = layer_data;
    DFBScreenID        displayId;
    DFBDisplayLayerID  layerId;
    char               text[24];

    layerId   = layerData->layerId   = DLID_PRIMARY + 2;
    displayId = layerData->displayId = dfb_screen_id_translated(dfb_layer_screen(layer));
    layerData->sourceId  = DLSID_SURFACE;

    D_DEBUG_AT( bcmnexusVideoLayer, "%s() %s display video layer %d\n", __FUNCTION__,
                (displayId == PRIMARY_DISPLAY_ID) ? "primary" : "secondary", layerId) ;

    /* semaphore init */
    snprintf(&text[0], sizeof(text), "BRCM Video Layer %d Lock", layerId);
    fusion_skirmish_init( &layerData->lock, text, dfb_core_world(NULL) );

    /* Set type and capabilities */
    description->type = DLTF_VIDEO;
    description->caps = DLCAPS_SURFACE         | DLCAPS_OPACITY     |
                        DLCAPS_SCREEN_POSITION | DLCAPS_SCREEN_SIZE;

    /* Set the screen name. */
    snprintf( description->name, DFB_DISPLAY_LAYER_DESC_NAME_LENGTH,
              "BRCM %s disp video layer %d", (layerId == (DLID_PRIMARY + 2)) ? "primary" : "secondary", layerId );

    /* fill out the default configuration */
    config->flags       = DLCONF_WIDTH | DLCONF_HEIGHT | DLCONF_PIXELFORMAT | DLCONF_BUFFERMODE | DLCONF_OPTIONS;
    config->width       = GET_DISPLAY_WIDTH(dfb_bcmnexus, displayId);
    config->height      = GET_DISPLAY_HEIGHT(dfb_bcmnexus, displayId);
    config->pixelformat = DSPF_YUY2;
    config->buffermode  = DLBM_FRONTONLY;
    config->options     = DLOP_OPACITY;

    D_DEBUG_AT( bcmnexusVideoLayer, "%s() screen %d video layer %d: flags=0x%08x, %dx%d, format=%s, bm=%d, opt=0x%08x, surf.caps=0x%08x\n", __FUNCTION__, 
                displayId, layerId, config->flags, config->width, config->height, dfb_pixelformat_name(config->pixelformat),
                config->buffermode, config->options, config->surface_caps );

    return DFB_OK;
}

static DFBResult
bcmVideoTestRegion( UNUSED_ CoreLayer                  *layer,
                            void                       *driver_data,
                            void                       *layer_data,
                            CoreLayerRegionConfig      *config,
                            CoreLayerRegionConfigFlags *failed )
{
    DFBBCMNEXUS                *dfb_bcmnexus = driver_data;
    bcmVideoLayerData          *layerData    = layer_data;
    CoreLayerRegionConfigFlags  fail         = CLRCF_NONE;
    DFBScreenID                 displayId;
    DFBDisplayLayerID           layerId;

    D_ASSERT( dfb_bcmnexus != NULL );

    displayId  = layerData->displayId;
    layerId    = layerData->layerId;

    D_DEBUG_AT( bcmnexusVideoLayer, "%s() screen %d video layer %d (%dx%d) format=%s, caps=0x%08x, bm=%d,\n"
                               "options=0x%08x, source_id=%d, opacity=%d [%dx%d@%d,%d]=>[%dx%d@%d,%d]\n",
                               __FUNCTION__, displayId, layerId, config->width, config->height, dfb_pixelformat_name(config->format),
                               config->surface_caps, config->buffermode, config->options, config->source_id, config->opacity,
                               config->source.w, config->source.h, config->source.x, config->source.y,
                               config->dest.w, config->dest.h, config->dest.x, config->dest.y);

    /* Check width */
    if (config->width > (int) GET_DISPLAY_WIDTH(dfb_bcmnexus, displayId))
        fail |= CLRCF_WIDTH;

    /* Check height */
    if (config->height > (int) GET_DISPLAY_HEIGHT(dfb_bcmnexus, displayId))
        fail |= CLRCF_HEIGHT;

    /* Check format */
    switch (config->format) {
        case DSPF_YUY2:
            break;

        default:
            fail |= CLRCF_FORMAT;
    }

    /* Check options */
    if (!D_FLAGS_ARE_IN( config->options, DLOP_OPACITY ))
        fail |= CLRCF_OPTIONS;

    if (failed)
        *failed = fail;

    if (fail)
    {
         D_ERROR("bcmNexus/VideoLayer: %s() screen %d video layer %d: Failed (flags = 0x%08x)!\n", __FUNCTION__, displayId, layerId, fail);
         return DFB_UNSUPPORTED;
    }

    D_DEBUG_AT( bcmnexusVideoLayer, "%s() screen %d video layer %d: Exit.\n", __FUNCTION__, displayId, layerId );
    return DFB_OK; 
}

static DFBResult
bcmVideoAddRegion( UNUSED_ CoreLayer             *layer,
                           void                  *driver_data,
                           void                  *layer_data,
                           void                  *region_data,
                   UNUSED_ CoreLayerRegionConfig *config )
{
    DFBResult                      res          = DFB_OK;
    DFBBCMNEXUS                   *dfb_bcmnexus = driver_data;
    bcmVideoLayerData             *layerData    = layer_data;
    NEXUS_VideoImageInputSettings  imageInputSetting;
    NEXUS_VideoWindowSettings      windowCfg;
    DFBScreenID                    displayId;
    DFBDisplayLayerID              layerId;

    D_ASSERT( dfb_bcmnexus != NULL );

    displayId = layerData->displayId;
    layerId   = layerData->layerId;

    D_DEBUG_AT( bcmnexusVideoLayer, "%s() screen %d video layer %d: format=%s, region=%dx%d, caps=0x%08x,\n"
                                    "bm=%d, options=0x%08x, source_id=%d, opacity=%d [%dx%d@%d,%d] -> [%dx%d@%d,%d]\n",
                                     __FUNCTION__, displayId, layerId, dfb_pixelformat_name(config->format),
                                     config->width, config->height, config->surface_caps, config->buffermode,
                                     config->options, config->source_id, config->opacity,
                                     config->source.w, config->source.h, config->source.x, config->source.y,
                                     config->dest.w, config->dest.h, config->dest.x, config->dest.y);

    /* Save current region pointer in layer data */
    layerData->regionData = region_data;

    layerData->videoWindow = NEXUS_VideoWindow_Open( GET_DISPLAY_HND(dfb_bcmnexus, displayId), 0 );
    if (!layerData->videoWindow) {
        D_ERROR( "Broadcom/VideoLayer: %s() screen %d video layer %d: Could not open video window 0!\n", __FUNCTION__, displayId, layerId );
        res = DFB_FAILURE;
    } else {
        /* Should set video window settings before adding image input to video window */
        NEXUS_VideoWindow_GetSettings(layerData->videoWindow, &windowCfg);
        windowCfg.position.x = 0;
        windowCfg.position.y = 0;
        windowCfg.position.width = config->width;
        windowCfg.position.height = config->height;

        if (NEXUS_VideoWindow_SetSettings(layerData->videoWindow, &windowCfg) != NEXUS_SUCCESS) {
            D_ERROR( "Broadcom/VideoLayer: %s() screen %d video layer %d: Could not get video window 0 settings!\n", __FUNCTION__, displayId, layerId );
            res = DFB_FAILURE;
        } else {
            layerData->imageInput = NEXUS_VideoImageInput_Open( 0, NULL );

            if (!layerData->imageInput) {
                D_ERROR( "Broadcom/VideoLayer: %s() screen %d video layer %d: Could not open video image input 0!\n",  __FUNCTION__, displayId, layerId );
                res = DFB_FAILURE;
            } else {
                NEXUS_VideoImageInput_GetSettings( layerData->imageInput, &imageInputSetting );
                imageInputSetting.imageCallback.callback = frameBufferCallback;
                imageInputSetting.imageCallback.context  = layerData;
                imageInputSetting.imageCallback.param    = layerId;

                if (NEXUS_VideoImageInput_SetSettings( layerData->imageInput, &imageInputSetting ) != NEXUS_SUCCESS) {
                    D_ERROR( "Broadcom/VideoLayer: %s() screen %d video layer %d: Could not get video image input 0 settings!\n", __FUNCTION__, displayId, layerId );
                    res = DFB_FAILURE;
                } else {
                    if (NEXUS_VideoWindow_AddInput( layerData->videoWindow, NEXUS_VideoImageInput_GetConnector(layerData->imageInput) ) != NEXUS_SUCCESS) {
                        D_ERROR( "Broadcom/VideoLayer: %s() screen %d video layer %d: Could not add video image input 0 to video window 0!\n", __FUNCTION__, displayId, layerId );
                        res = DFB_FAILURE;
                    }
                }
            }
        }
    }
    D_ASSERT( res == DFB_OK );

    D_DEBUG_AT( bcmnexusVideoLayer, "%s() screen %d video layer %d: Exit [%d].\n", __FUNCTION__, displayId, layerId, res );
    return res;
}

static void
setVideoLayerBuffer( bcmVideoLayerData     *layerData,
                     bcmVideoRegionData    *regionData,
                     CoreSurface           *surface, 
                     CoreSurfaceBufferLock *lock,
                     bool                   wait )
{
    DFBScreenID         displayId;
    DFBDisplayLayerID   layerId;
    NEXUS_SurfaceHandle hSurface;

    displayId = layerData->displayId;
    layerId   = layerData->layerId;
    hSurface  = lock->handle;

    D_DEBUG_AT( bcmnexusVideoLayer, "%s() screen %d video layer %d: enabled=%d surface=%p [handle=%p, oldhandle=%p]\n",
                 __FUNCTION__, displayId, layerId, layerData->layerEnabled, (void *)surface, (void *)hSurface, (void *)regionData->bufferLock.handle );

    if (layerData->layerEnabled && (hSurface != regionData->bufferLock.handle))
    {
        fusion_skirmish_prevail( &layerData->lock );

        /* Save video framebuffer */
        regionData->bufferLock = *lock;

        NEXUS_VideoImageInput_SetSurface( layerData->imageInput, hSurface );

        if (wait) {
            D_DEBUG_AT( bcmnexusVideoLayerX, "%s() screen %d video layer %d: waiting for framebuffer %p callback...\n", __FUNCTION__,
                        displayId, layerId, (void *)surface );

            fusion_skirmish_wait( &layerData->lock, 100 );

            D_DEBUG_AT( bcmnexusVideoLayerX, "%s() screen %d video layer %d: received callback for framebuffer %p.\n", __FUNCTION__,
                        displayId, layerId, (void *)surface );
        }
        fusion_skirmish_dismiss( &layerData->lock );
    }
}

static DFBResult
bcmVideoSetRegion( UNUSED_ CoreLayer                  *layer,
                           void                       *driver_data,
                           void                       *layer_data,
                           void                       *region_data,
                           CoreLayerRegionConfig      *config,
                           CoreLayerRegionConfigFlags  updated,
                           CoreSurface                *surface,
                   UNUSED_ CorePalette                *palette,
                           CoreSurfaceBufferLock      *lock )
{
    DFBBCMNEXUS               *dfb_bcmnexus = driver_data;
    bcmVideoLayerData         *layerData    = layer_data;
    bcmVideoRegionData        *regionData   = region_data;
    NEXUS_VideoWindowSettings  windowCfg;
    DFBScreenID                displayId;
    DFBDisplayLayerID          layerId;

    D_ASSERT( dfb_bcmnexus != NULL );

    displayId    = layerData->displayId;
    layerId      = layerData->layerId;

    D_DEBUG_AT( bcmnexusVideoLayer, "%s() screen %d video layer %d: updated flags=0x%08x, surface=%p,\n"
                                    "handle=%p, format=%s, region=%dx%d, caps=0x%08x, bm=%d,\n"
                                    "options=0x%08x, source_id=%d, opacity=%d [%dx%d@%d,%d] -> [%dx%d@%d,%d]\n",
                                    __FUNCTION__, displayId, layerId, updated, (void *)surface, lock->handle, dfb_pixelformat_name(config->format),
                                    config->width, config->height, config->surface_caps, config->buffermode,
                                    config->options, config->source_id, config->opacity,
                                    config->source.w, config->source.h, config->source.x, config->source.y,
                                    config->dest.w, config->dest.h, config->dest.x, config->dest.y);

    /* Get whether the screen/display is on */
    layerData->layerEnabled = (bcmGetPowerMode(dfb_screens_at(displayId)) == DSPM_ON);

    /* Set Surface to Source */
    NEXUS_VideoWindow_GetSettings( layerData->videoWindow, &windowCfg );

    windowCfg.visible         = true;
    windowCfg.clipBase.x      = 0;
    windowCfg.clipBase.y      = 0;
    windowCfg.clipBase.width  = config->width;
    windowCfg.clipBase.height = config->height;
    windowCfg.position.x      = config->dest.x;
    windowCfg.position.y      = config->dest.y;
    /* window position has to be inside display boundary for now */
    if ((config->dest.x + config->dest.w) > config->width)
        windowCfg.position.width  = config->width*2 - (config->dest.x + config->dest.w);
    else
        windowCfg.position.width  = config->dest.w;
    /* window position has to be inside display boundary for now */
    if ((config->dest.y + config->dest.h) > config->height)
        windowCfg.position.height = config->height*2 - (config->dest.y + config->dest.h);
    else
        windowCfg.position.height = config->dest.h;

    windowCfg.clipRect.x      = config->source.x;
    windowCfg.clipRect.y      = config->source.y;
    windowCfg.clipRect.width  = config->source.w;
    windowCfg.clipRect.height = config->source.h;

    if (config->options & DLOP_OPACITY) {
        windowCfg.sourceBlendFactor = NEXUS_CompositorBlendFactor_eConstantAlpha;
        windowCfg.destBlendFactor   = NEXUS_CompositorBlendFactor_eInverseConstantAlpha;
    }
    else {
        windowCfg.sourceBlendFactor = NEXUS_CompositorBlendFactor_eOne;
        windowCfg.destBlendFactor   = NEXUS_CompositorBlendFactor_eZero;
    }

    windowCfg.alpha         =
    windowCfg.constantAlpha = config->opacity;

    windowCfg.pixelFormat = bcmnexus_getNexusPixelFormat( config->format );

    D_DEBUG_AT( bcmnexusVideoLayer, "%s() screen %d video layer %d: clip=%dx%d@%d,%d,position=%dx%d@%d,%d,sourceBlendFactor=%d,destBlendFactor=%d,"
                                    "alpha=%d,constantAlpha=%d\n", __FUNCTION__, displayId, layerId,
                                    windowCfg.clipRect.width, windowCfg.clipRect.height, windowCfg.clipRect.x, windowCfg.clipRect.y,
                                    windowCfg.position.width, windowCfg.position.height, windowCfg.position.x, windowCfg.position.y,
                                    windowCfg.sourceBlendFactor, windowCfg.destBlendFactor, windowCfg.alpha, windowCfg.constantAlpha );

    NEXUS_VideoWindow_SetSettings( layerData->videoWindow, &windowCfg );

    setVideoLayerBuffer( layerData, regionData, surface, lock, false );

    D_DEBUG_AT( bcmnexusVideoLayer, "%s() screen %d video layer %d: Exit.\n", __FUNCTION__, displayId, layerId );
    return DFB_OK;
}

static DFBResult 
bcmVideoRemoveRegion( UNUSED_ CoreLayer *layer,
                      UNUSED_ void      *driver_data,
                              void      *layer_data,
                      UNUSED_ void      *region_data )
{
    DFBBCMNEXUS       *dfb_bcmnexus = driver_data;
    bcmVideoLayerData *layerData    = layer_data;

    D_ASSERT( dfb_bcmnexus != NULL );

    D_DEBUG_AT( bcmnexusVideoLayer, "%s() screen %d video layer %d\n", __FUNCTION__, layerData->displayId, layerData->layerId );

    NEXUS_VideoImageInput_SetSurface( layerData->imageInput, NULL );
    NEXUS_VideoWindow_RemoveInput( layerData->videoWindow, NEXUS_VideoImageInput_GetConnector(layerData->imageInput) );
    NEXUS_VideoInput_Shutdown( NEXUS_VideoImageInput_GetConnector(layerData->imageInput) );
    NEXUS_VideoImageInput_Close( layerData->imageInput );
    NEXUS_VideoWindow_Close( layerData->videoWindow );

    layerData->regionData = NULL;
    return DFB_OK;
}

static DFBResult
bcmVideoFlipRegion( UNUSED_ CoreLayer             *layer,
                    UNUSED_ void                  *driver_data,
                            void                  *layer_data,
                            void                  *region_data,
                            CoreSurface           *surface,
                            DFBSurfaceFlipFlags    flags,
                            CoreSurfaceBufferLock *lock )
{
    bcmVideoLayerData *layerData    = layer_data;

    D_DEBUG_AT( bcmnexusVideoLayer, "%s() screen %d video layer %d: surface=%p, flags=0x%08x, handle=%p\n", __FUNCTION__,
                layerData->displayId, layerData->layerId, (void *)surface, flags, (void *)lock->handle );

    setVideoLayerBuffer( layerData, region_data, surface, lock, (flags & DSFLIP_WAIT) );

    /* Inform DirectFB that the buffers have changed function */
    dfb_surface_flip( surface, false );

    return DFB_OK;
}

static DFBResult
bcmVideoUpdateRegion( UNUSED_ CoreLayer             *layer,
                      UNUSED_ void                  *driver_data,
                              void                  *layer_data,
                              void                  *region_data,
                              CoreSurface           *surface,
                              const DFBRegion       *update,
                              CoreSurfaceBufferLock *lock )
{
    bcmVideoLayerData *layerData    = layer_data;

    D_DEBUG_AT( bcmnexusVideoLayer, "%s() screen %d video layer %d: surf=%p, handle=%p [%d,%d->%d,%d]\n", __FUNCTION__,
                layerData->displayId, layerData->layerId, (void *)surface, (void *)lock->handle,
                update->x1, update->y1, update->x2, update->y2 );

    setVideoLayerBuffer( layerData, region_data, surface, lock, false );

    return DFB_OK;
}

DisplayLayerFuncs bcmnexusVideoFuncs = {
    .LayerDataSize         = bcmVideoLayerDataSize,
    .RegionDataSize        = bcmVideoRegionDataSize,
    .InitLayer             = bcmVideoInitLayer,
    .TestRegion            = bcmVideoTestRegion,
    .AddRegion             = bcmVideoAddRegion,
    .SetRegion             = bcmVideoSetRegion,
    .RemoveRegion          = bcmVideoRemoveRegion,
    .FlipRegion            = bcmVideoFlipRegion,
    .UpdateRegion          = bcmVideoUpdateRegion,
};

