/******************************************************************************
 *    (c)2008-2011 Broadcom Corporation
 * 
 * This program is the proprietary software of Broadcom Corporation and/or its licensors,
 * and may only be used, duplicated, modified or distributed pursuant to the terms and
 * conditions of a separate, written license agreement executed between you and Broadcom
 * (an "Authorized License").  Except as set forth in an Authorized License, Broadcom grants
 * no license (express or implied), right to use, or waiver of any kind with respect to the
 * Software, and Broadcom expressly reserves all rights in and to the Software and all
 * intellectual property rights therein.  IF YOU HAVE NO AUTHORIZED LICENSE, THEN YOU
 * HAVE NO RIGHT TO USE THIS SOFTWARE IN ANY WAY, AND SHOULD IMMEDIATELY
 * NOTIFY BROADCOM AND DISCONTINUE ALL USE OF THE SOFTWARE.  
 *  
 * Except as expressly set forth in the Authorized License,
 *  
 * 1.     This program, including its structure, sequence and organization, constitutes the valuable trade
 * secrets of Broadcom, and you shall use all reasonable efforts to protect the confidentiality thereof,
 * and to use this information only in connection with your use of Broadcom integrated circuit products.
 *  
 * 2.     TO THE MAXIMUM EXTENT PERMITTED BY LAW, THE SOFTWARE IS PROVIDED "AS IS" 
 * AND WITH ALL FAULTS AND BROADCOM MAKES NO PROMISES, REPRESENTATIONS OR 
 * WARRANTIES, EITHER EXPRESS, IMPLIED, STATUTORY, OR OTHERWISE, WITH RESPECT TO 
 * THE SOFTWARE.  BROADCOM SPECIFICALLY DISCLAIMS ANY AND ALL IMPLIED WARRANTIES 
 * OF TITLE, MERCHANTABILITY, NONINFRINGEMENT, FITNESS FOR A PARTICULAR PURPOSE, 
 * LACK OF VIRUSES, ACCURACY OR COMPLETENESS, QUIET ENJOYMENT, QUIET POSSESSION 
 * OR CORRESPONDENCE TO DESCRIPTION. YOU ASSUME THE ENTIRE RISK ARISING OUT OF 
 * USE OR PERFORMANCE OF THE SOFTWARE.
 * 
 * 3.     TO THE MAXIMUM EXTENT PERMITTED BY LAW, IN NO EVENT SHALL BROADCOM OR ITS 
 * LICENSORS BE LIABLE FOR (i) CONSEQUENTIAL, INCIDENTAL, SPECIAL, INDIRECT, OR 
 * EXEMPLARY DAMAGES WHATSOEVER ARISING OUT OF OR IN ANY WAY RELATING TO YOUR 
 * USE OF OR INABILITY TO USE THE SOFTWARE EVEN IF BROADCOM HAS BEEN ADVISED OF 
 * THE POSSIBILITY OF SUCH DAMAGES; OR (ii) ANY AMOUNT IN EXCESS OF THE AMOUNT 
 * ACTUALLY PAID FOR THE SOFTWARE ITSELF OR U.S. $1, WHICHEVER IS GREATER. THESE 
 * LIMITATIONS SHALL APPLY NOTWITHSTANDING ANY FAILURE OF ESSENTIAL PURPOSE OF 
 * ANY LIMITED REMEDY.
 *
 * $brcm_Workfile: dfb_platform.h $
 * $brcm_Revision: DirectFB_1_4_Port/1 $
 * $brcm_Date: 2/4/11 8:37p $
 * 
 * Module Description:
 * 
 * Revision History:
 * 
 * $brcm_Log: /AppLibs/opensource/directfb/src/broadcom_files/private/DirectFB/1.4/dfb_platform.h $
 * 
 * DirectFB_1_4_Port/1   2/4/11 8:37p robertwm
 * SW7400-3000: DirectFB: Support non-DFB master application in
 *  platform_init code.
 * SW7400-3002: DirectFB: Support non-DFB master application in
 *  platform_init code.
 * 
 * 
 *****************************************************************************/
#ifndef __DFB_PLATFORM_H__
#define __DFB_PLATFORM_H__

#include "nexus_platform.h"
#include "nexus_memory.h"
#include "nexus_display.h"
#if NEXUS_HAS_GRAPHICS2D || NEXUS_HAS_ZSP_GRAPHICS
#include "nexus_graphics2d.h"
#endif
#if NEXUS_HAS_GRAPHICS3D && defined(BCMNEXUS_GFX_HAS_P3D)
#include "nexus_graphics3d.h"
#endif
#ifdef NEXUS_HAS_PICTURE_DECODER
#include "nexus_picture_decoder.h"
#endif

typedef struct {
    bool                       slave;                       /* Are we a slave NEXUS App? */
    bool                       initNexus;                   /* Init or Join NEXUS? */
    bool                       allowUnauthenticatedClients; /* Set to true if we want to allow unauthenticated clients */
    bool                       openFrontend;                /* Should we open the frontend during NEXUS platform init? */

    /* Heap related fields (these should be setup if another process/app initialises NEXUS) */
    int                        displayHeap;                 /* Framebuffer surface heap index (-1 = automatically determine) */
    int                        offscreenHeap;               /* Offscreen surface(s) heap index (-1 = automatically determine) */
    int                        paletteHeap;                 /* Palette heap index (-1 = automatically determine) */
    int                        videoHeap;                   /* Video heap index (-1 = automatically determine) */

     /* Display related fields */
    NEXUS_VideoFormat          videoFormat;                 /* Preferred startup video format (0 means use default) */
    NEXUS_DisplayHandle        display0;                    /* Primary display handle (NULL = automatically open display 0) */
#if (NEXUS_NUM_DISPLAYS > 1)
    NEXUS_DisplayHandle        display1;                    /* Secondary display handle (NULL = automatically open display 1) */
#endif

    /* Graphics related fields */
#if NEXUS_HAS_GRAPHICS2D || NEXUS_HAS_ZSP_GRAPHICS  
    NEXUS_Graphics2DHandle     graphics2d;                  /* 2D graphics handle  (NULL = automatically open graphics2d) */
#endif
#if NEXUS_HAS_GRAPHICS3D && defined(BCMNEXUS_GFX_HAS_P3D)
    NEXUS_Graphics3DHandle     graphics3d;                  /* 3D graphics handle (NULL = automatically open graphics3d) */
#endif
#ifdef NEXUS_HAS_PICTURE_DECODER
    NEXUS_PictureDecoderHandle pictureDecoder;              /* Still image decoder handle (NULL = automatically open SID) */
#endif

} DFB_PlatformSettings;


/* Functions that an external application can call to pass information to DirectFB.
   The DFB_Platform_Init() function is automatically called by the libinit.so 
   shared library. Applications can instead link with this library and call these
   functions explictly.
 */
void DFB_Platform_GetDefaultSettings(DFB_PlatformSettings *pSettings);
int  DFB_Platform_Init(DFB_PlatformSettings *pSettings);
void DFB_Platform_Uninit(void);

#endif
