/******************************************************************************
 *    (c)2008-2011 Broadcom Corporation
 *
 * This program is the proprietary software of Broadcom Corporation and/or its licensors,
 * and may only be used, duplicated, modified or distributed pursuant to the terms and
 * conditions of a separate, written license agreement executed between you and Broadcom
 * (an "Authorized License").  Except as set forth in an Authorized License, Broadcom grants
 * no license (express or implied), right to use, or waiver of any kind with respect to the
 * Software, and Broadcom expressly reserves all rights in and to the Software and all
 * intellectual property rights therein.  IF YOU HAVE NO AUTHORIZED LICENSE, THEN YOU
 * HAVE NO RIGHT TO USE THIS SOFTWARE IN ANY WAY, AND SHOULD IMMEDIATELY
 * NOTIFY BROADCOM AND DISCONTINUE ALL USE OF THE SOFTWARE.
 *
 * Except as expressly set forth in the Authorized License,
 *
 * 1.     This program, including its structure, sequence and organization, constitutes the valuable trade
 * secrets of Broadcom, and you shall use all reasonable efforts to protect the confidentiality thereof,
 * and to use this information only in connection with your use of Broadcom integrated circuit products.
 *
 * 2.     TO THE MAXIMUM EXTENT PERMITTED BY LAW, THE SOFTWARE IS PROVIDED "AS IS"
 * AND WITH ALL FAULTS AND BROADCOM MAKES NO PROMISES, REPRESENTATIONS OR
 * WARRANTIES, EITHER EXPRESS, IMPLIED, STATUTORY, OR OTHERWISE, WITH RESPECT TO
 * THE SOFTWARE.  BROADCOM SPECIFICALLY DISCLAIMS ANY AND ALL IMPLIED WARRANTIES
 * OF TITLE, MERCHANTABILITY, NONINFRINGEMENT, FITNESS FOR A PARTICULAR PURPOSE,
 * LACK OF VIRUSES, ACCURACY OR COMPLETENESS, QUIET ENJOYMENT, QUIET POSSESSION
 * OR CORRESPONDENCE TO DESCRIPTION. YOU ASSUME THE ENTIRE RISK ARISING OUT OF
 * USE OR PERFORMANCE OF THE SOFTWARE.
 *
 * 3.     TO THE MAXIMUM EXTENT PERMITTED BY LAW, IN NO EVENT SHALL BROADCOM OR ITS
 * LICENSORS BE LIABLE FOR (i) CONSEQUENTIAL, INCIDENTAL, SPECIAL, INDIRECT, OR
 * EXEMPLARY DAMAGES WHATSOEVER ARISING OUT OF OR IN ANY WAY RELATING TO YOUR
 * USE OF OR INABILITY TO USE THE SOFTWARE EVEN IF BROADCOM HAS BEEN ADVISED OF
 * THE POSSIBILITY OF SUCH DAMAGES; OR (ii) ANY AMOUNT IN EXCESS OF THE AMOUNT
 * ACTUALLY PAID FOR THE SOFTWARE ITSELF OR U.S. $1, WHICHEVER IS GREATER. THESE
 * LIMITATIONS SHALL APPLY NOTWITHSTANDING ANY FAILURE OF ESSENTIAL PURPOSE OF
 * ANY LIMITED REMEDY.
 *
 * $brcm_Workfile: platform_init.c $
 * $brcm_Revision: DirectFB_1_4_Port/48 $
 * $brcm_Date: 3/11/11 5:05p $
 *
 * Module Description:
 *
 * Revision History:
 *
 * $brcm_Log: /AppLibs/opensource/directfb/src/broadcom_files/private/DirectFB/1.4.1/libinit/platform_init.c $
 * 
 * DirectFB_1_4_Port/48   3/11/11 5:05p christ
 * SW7468-370: DirectFB: 1.4.1 release fails to compile for platforms
 *  without component output.
 *
 * DirectFB_1_4_Port/47   2/25/11 6:23p apandav
 * SW3520-44: BCM935230: Error while compiling DirectFB_1_4_Port
 *
 * DirectFB_1_4_Port/46   2/22/11 12:00p christ
 * SW7420-1527: DirectFB: Update platform code to get offscreen heap from
 *  Nexus when available
 *
 * DirectFB_1_4_Port/45   2/20/11 7:27p robertwm
 * SW7422-79: Add support for mapping video decoders to andy video source.
 * SW7420-1394: video layer support in DFB.
 * SW7422-275: DirectFB: Enabling video layer causes Segmentation fault.
 *
 * DirectFB_1_4_Port/44   2/18/11 2:38p robertwm
 * SW7468-366: DirectFB: Platforms without component output should not
 *  call component functions.
 *
 * DirectFB_1_4_Port/43   2/4/11 8:39p robertwm
 * SW7400-3000: DirectFB: Support non-DFB master application in
 *  platform_init code.
 * SW7400-3002: DirectFB: Support non-DFB master application in
 *  platform_init code.
 *
 * DirectFB_1_4_Port/42   2/4/11 4:19p robertwm
 * SW35230-2943: DirectFB: Cannot build platform_init code for 35230
 *  platform.
 *
 * DirectFB_1_4_Port/41   2/4/11 3:22p christ
 * SW7420-1458: DirectFB: Rework internal handling of nexus heaps. - fix
 *  compile issue with old nexus.
 *
 * DirectFB_1_4_Port/40   2/3/11 6:10p christ
 * SW7420-1458: DirectFB: Rework internal handling of nexus heaps.
 *
 * DirectFB_1_4_Port/39   2/3/11 6:01p christ
 * SW7420-1458: DirectFB: Rework internal handling of nexus heaps.
 *
 * DirectFB_1_4_Port/38   1/31/11 7:30p robertwm
 * SW7420-1451: DirectFB: platform_init code should get called
 *  automatically by slave applications.
 *
 * DirectFB_1_4_Port/37   1/25/11 8:06p robertwm
 * SW7420-1424: DirectFB: Refactor platform_init code to separate DTV and
 *  STB parts and to support NEXUS heap m'ment.
 *
 * DirectFB_1_4_Port/DirectFB_STEREO_3D_JIRA_SW7422_138/8   1/25/11 7:11p robertwm
 * SW7420-1422: DirectFB: Add support for NEXUS multi-app platform server
 *  in platform init code.
 *
 * DirectFB_1_4_Port/DirectFB_STEREO_3D_JIRA_SW7422_138/7   1/18/11 1:27p christ
 * Fix DTV compile problems.
 *
 * DirectFB_1_4_Port/DirectFB_STEREO_3D_JIRA_SW7422_138/5   1/17/11 5:19p christ
 * Better fix for compile issue.
 *
 * DirectFB_1_4_Port/DirectFB_STEREO_3D_JIRA_SW7422_138/4   1/17/11 4:51p christ
 * Fix build issue with libinit code.
 *
 * DirectFB_1_4_Port/DirectFB_STEREO_3D_JIRA_SW7422_138/3   1/17/11 12:37p christ
 * Reworked platform init code.
 *
 * DirectFB_1_4_Port/DirectFB_STEREO_3D_JIRA_SW7422_138/1   12/13/10 5:23p christ
 * Initial stereo 3d commit
 *
 * DirectFB_1_4_Port/35   12/3/10 8:42p raywu
 * SW7550-629: DirectFB: Integrated ZSP code to DFB packet buffer branch
 *
 * DirectFB_1_4_Port/34   12/2/10 2:43p christ
 * SW7420-1283: DirectFB: Add support for extra Nexus heap on dual MEMC
 *  parts, slight rework to avoid issues for SOCs with only 1 memory
 *  controller (eg 7550).
 *
 * DirectFB_1_4_Port/33   12/1/10 3:47p christ
 * SW7420-1283: DirectFB: Add support for extra Nexus heap on dual MEMC
 *  parts
 *
 * DirectFB_1_4_Port/32   10/20/10 5:11p robertwm
 * SW7420-1195: DirectFB: Move the platform_init shared handles definition
 *  to a header file for portability.
 *
 * DirectFB_1_4_Port/31   10/20/10 4:43p robertwm
 * SW7420-1194: DirectFB: Cannot run application more than once if it was
 *  exited abnormally.
 *
 * DirectFB_1_4_Port/30   10/19/10 1:38p robertwm
 * SW7420-1188: DirectFB: platform_init fails to initialise after
 *  launching an app more than once.
 *
 * DirectFB_1_4_Port/29   10/12/10 4:57p ismailk
 * SW7405-4884: Don't check for "dfb_no_platform_init" env variable in
 *  do_platform_init()
 *
 * DirectFB_1_4_Port/28   9/20/10 9:44p robertwm
 * SW7405-4884: DirectFB: Add shared memory handles to platform_init
 *
 * DirectFB_1_4_Port/27   9/13/10 6:23p robertwm
 * SW7405-4870: DirectFB: Add support for not running the platform_init()
 *  code.
 *
 * DirectFB_1_4_Port/26   8/20/10 3:36p robertwm
 * SW7405-4791: DirectFB: DirectFB-1.4.1 Phase 2.5 fails to build with
 *  Nexus Phase 5.0 reference software.
 *
 * DirectFB_1_4_Port/25   8/11/10 8:09p robertwm
 * SW3556-1178: DirectFB: Seg fault when trying to shut down Penguins on
 *  3556.
 *
 * DirectFB_1_4_Port/24   8/11/10 4:57p robertwm
 * SW35230-957: DirectFB: platform_init fails to build on 35230 with
 *  recent 3D changes.
 *
 * DirectFB_1_4_Port/23   8/11/10 1:57a robertwm
 * SW7405-4250: DirectFB: Screen destination rectangle and source
 *  rectangle not saved across contexts.
 * SW7405-4743: DirectFB: Multiple instances of df_andi.c do not get
 *  displayed.
 *
 * DirectFB_1_4_Port/22   8/6/10 8:04p robertwm
 * SW7405-4724: DirectFB: Support OpenGL ES 1.0 applications.
 *
 * DirectFB_1_4_Port/21   7/20/10 11:29a robertwm
 * SW7420-900: NEXUS_CORE_HEAP_ALIGNMENT has been changed to
 *  NEXUS_LARGEST_CPU_CACHE_LINE.
 *
 * DirectFB_1_4_Port/19   5/5/10 12:49p robertwm
 * SW7405-4295: DirectFB: Reference to NEXUS_NUM_HDMI_OUTPUTS should check
 *  if > 0.
 *
 * DirectFB_1_4_Port/18   4/19/10 7:29p robertwm
 * SW7420-708: DirectFB: Support DirectFB run as a slave.
 *
 * DirectFB_1_4_Port/17   4/9/10 4:54p robertwm
 * SW7405-4183: DirectFB: Add support for Texture Triangles in the
 *  graphics driver.
 *
 * DirectFB_1_4_Port/16   3/4/10 7:07p robertwm
 * SW7468-137: DirectFB: Require CVBS/RFM to be connected to display1
 *  path.
 *
 * DirectFB_1_4_Port/15   2/22/10 11:53a robertwm
 * SW7420-549: Separate heap for graphics doesn't work with 7420.
 *
 * DirectFB_1_4_Port/14   1/5/10 5:42p robertwm
 * SW7405-3711: DirectFB: Require graphics on composite and RFM outputs.
 *
 * DirectFB_1_4_Port/13   1/5/10 11:39a robertwm
 * SW7405-3711: DirectFB: Require graphics on composite and RFM outputs.
 *
 * DirectFB_1_4_Port/12   12/23/09 2:18p robertwm
 * SW7550-51: Support DirectFB on 7550.
 *
 * DirectFB_1_4_Port/SW7550-51/1   12/22/09 3:22p mphillip
 * SW7550-51: Add 7550 build support (on branch)
 *
 * DirectFB_1_4_Port/11   12/17/09 2:31p robertwm
 * SW7335-641: DFB is not working with popen.
 *
 * DirectFB_1_4_Port/10   12/3/09 1:27p robertwm
 * SW7405-3528: BERR_OUT_OF_SYSTEM_MEMORY(0x3) at bgrc_private.c:957.
 *
 * DirectFB_1_4_Port/9   11/27/09 2:05p robertwm
 * SW7405-3502: Poor graphics performance of 1.3.x and 1.4.x compared with
 *  1.0.0
 *
 * DirectFB_1_4_Port/8   10/27/09 6:12p robertwm
 * SW7405-3124: DirectFB: Allow memory architecture (UMA/non-UMA) to be
 *  specified as envar to speed up DFB init.
 * (now allows mem_non_uma=0 to be specified to indicate UMA memory
 *  architecture).
 *
 * DirectFB_1_4_Port/7   10/2/09 12:15p robertwm
 * SW7405-3124: DirectFB: Allow memory architecture (UMA/non-UMA) to be
 *  specified as envar to speed up DFB init.
 *
 * DirectFB_1_4_Port/6   8/31/09 12:41p ismailk
 * SW7405-2954: Uninitialize BDBG() module before closing the module.
 *  Also, don't NEXUS_Platform_Uninit() as the DLNA mediaplayer code lets
 *  nexus connections and handles remain open.
 *
 * DirectFB_1_4_Port/5   8/27/09 4:34p robertwm
 * SW3548-2427: DirectFB: Support 3549 panel types and fix compilation
 *  bugs.
 *
 * DirectFB_1_4_Port/4   8/20/09 12:16p robertwm
 * PR57884: DirectFB: Support 7420 B0 Platform.
 *
 * DirectFB_1_4_Port/3   8/7/09 3:41p robertwm
 * PR56902: Require single graphics memory heap to avoid fragmentation.
 * PR56721: Add HDMI output support.
 *
 * DirectFB_1_4_Port/2   7/10/09 4:22p robertwm
 * PR56721: Add HDMI output to DirectFB.
 *
 * DirectFB_1_4_Port/1   7/8/09 3:19p mphillip
 * PR56658: Add DirectFB 1.4.1 broadcom_files
 *
 * 11   2/25/09 2:33p gmohile
 * PR 25109 : Enable only panel output for DTV platforms
 *
 * 10   11/21/08 8:06p vsilyaev
 * PR 49524: Added graphics2d_sync routine that could be used by different
 *  users of Nexus_Graphics2D
 *
 * 9   11/6/08 9:59a gmohile
 * PR 45243 : Turn off pre-alloc memory for graphics
 *
 * 8   11/5/08 10:26a gmohile
 * PR 45243 : disable destructor
 *
 * 7   11/3/08 1:57p gmohile
 * PR 45243 : Default all DTV to panel output
 *
 * 6   10/24/08 12:42p gmohile
 * PR 46490 : Add hw picture decode support
 *
 * 5   10/14/08 3:53p gmohile
 * PR 45243 : Fix nexus handles
 *
 * 4   9/22/08 11:55a gmohile
 * PR 45243 : Fix 3548 display open
 *
 * 3   9/15/08 11:49a gmohile
 * PR 46734 : Add dual output support
 *
 * 2   9/9/08 1:47p gmohile
 * PR 45243 : Do not add outputs in platform init
 *
 * 1   8/29/08 2:35p gmohile
 * PR 45243 : Do nexus platform init and display open outside directfb
 *
 *****************************************************************************/
#include "bstd.h"
#include "platform_init.h"
#include "nexus_platform.h"

#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#include <sys/ipc.h>
#include <sys/shm.h>
#include <sys/mman.h>
#include <sys/stat.h>
#include <fcntl.h>

#include "bkni.h"
#include "bkni_multi.h"

// For backwards compatibility
#ifndef NEXUS_OFFSCREEN_SURFACE
#define NEXUS_OFFSCREEN_SURFACE ((unsigned)-1)
#endif

void platform_init(void) __attribute__ ((constructor));
void platform_uninit(void) __attribute__ ((destructor));

static void *sharedMem=NULL;
static int shmid_g;

static bool initSharedMemory(size_t shMemSz);
static void destroySharedMemory(void * shMem);
static void *setupSharedMemory(size_t shMemSz,bool bCreate);

#if NEXUS_HAS_GRAPHICS2D || NEXUS_HAS_ZSP_GRAPHICS
static BKNI_MutexHandle  nexus_graphics2d_sync_lock;
static BKNI_EventHandle  nexus_graphics2d_sync_event;
static void platform_nexus_graphics2d_sync(void);
void (*nexus_graphics2d_sync)(void) = platform_nexus_graphics2d_sync;
#endif

/* Get the default DFB platform settings */
void DFB_Platform_GetDefaultSettings(DFB_PlatformSettings *pSettings)
{
    memset(pSettings, 0, sizeof(*pSettings));

    pSettings->slave                        = false;

    if (getenv("dfb_slave") && !strcmp(getenv("dfb_slave"), "y"))
        pSettings->slave                    = true;

    pSettings->initNexus                    = true;
    pSettings->allowUnauthenticatedClients  = true;
    pSettings->openFrontend                 = true;
    pSettings->displayHeap                  = -1;
    pSettings->offscreenHeap                = -1;
    pSettings->paletteHeap                  = -1;
    pSettings->videoHeap                    = -1;
    pSettings->videoFormat                  = NEXUS_VideoFormat_eUnknown;
}

/* An external application can call this function directly to pass information about handles, heaps, etc. to DFB */
int DFB_Platform_Init(DFB_PlatformSettings *pSettings)
{
    int                                 err;
    NEXUS_Error                         rc;
    DFB_PlatformSettings                settings;
    NEXUS_PlatformSettings              platformSettings;
    NEXUS_PlatformConfiguration         platformConfig;
#if BCMNEXUS_HAS_IPC
    NEXUS_PlatformStartServerSettings   serverSettings;
#endif
    NEXUS_DisplaySettings               displayCfg;
    NEXUS_Shared_Handles               *sharedHandles;
#ifndef BCMNEXUS_PRECONFIGURED_HEAPS
    unsigned char                       nexusHeapIndex[2];
#endif
    int                                 nexusVideoHeapIndex;

    BDBG_Init();

    if (pSettings == NULL)
    {
        DFB_Platform_GetDefaultSettings(&settings);
        pSettings = &settings;
    }

    if (!initSharedMemory(sizeof(NEXUS_Shared_Handles)))
    {
        // Slave mode configuration
        sharedMem = setupSharedMemory(sizeof(NEXUS_Shared_Handles),false);

        if(!sharedMem)
        {
            BDBG_ERR(("Setting Up Shared Memory Failed\n"));
            return -1;
        }
        sharedHandles = sharedMem;
        BDBG_MSG(("Copying the Shmem parameters ...to global variables\n"));

        SET_NEXUS_GRAPHICS_DISPLAY_HEAP_HND(sharedHandles->display_heap);
        SET_NEXUS_GRAPHICS_OFFSCREEN_HEAP_HND(sharedHandles->offscreen_heap);
        SET_NEXUS_GRAPHICS_PALETTE_HEAP_HND(sharedHandles->palette_heap);
        SET_NEXUS_VIDEO_HEAP_HND(sharedHandles->video_heap);

        SET_NEXUS_DISPLAY_HND(0,sharedHandles->disp_handle0);
        SET_NEXUS_VIDEO_FORMAT_PREFERRED(sharedHandles->video_format_preferred);
#if NEXUS_NUM_COMPONENT_OUTPUTS
        SET_NEXUS_COMPONENT_CONNECTED(sharedHandles->is_component_connected);
#endif

#if (NUM_DISPLAYS > 1)
        SET_NEXUS_DISPLAY_HND(1,sharedHandles->disp_handle1);
#endif

#ifdef NEXUS_HAS_PICTURE_DECODER
        SET_NEXUS_PICTURE_DECODER_HND(sharedHandles->pic_decode_handle);
#endif

#if NEXUS_HAS_GRAPHICS2D || NEXUS_HAS_ZSP_GRAPHICS
        SET_NEXUS_GRAPHICS2D_HND(sharedHandles->graphice2D_Handle);
#endif

#if NEXUS_HAS_GRAPHICS3D && defined(BCMNEXUS_GFX_HAS_P3D)
        SET_NEXUS_GRAPHICS3D_HND(sharedHandles->graphice3D_Handle);
#endif

        if (pSettings->initNexus)
        {
            /* Join Nexus here rather than in our system DFB driver */
            rc = NEXUS_Platform_Join();
            if(rc!=NEXUS_SUCCESS) {rc=BERR_TRACE(rc);goto err_platform;}
            BDBG_MSG_TRACE((">>platform_join"));
        }

        sharedMem=NULL;
        return 0;
    }

    /* Bring up all modules for a platform in a default configuration for this platform */
    NEXUS_Platform_GetDefaultSettings(&platformSettings);
    platformSettings.openFrontend = pSettings->openFrontend;

    sharedMem = setupSharedMemory(sizeof(NEXUS_Shared_Handles),true);
    if(!sharedMem)
    {
        BDBG_ERR(("Setting Up Shared Memory Failed\n"));
        return -1;
    }

    sharedHandles = (NEXUS_Shared_Handles *) sharedMem;

    /* Do we need to initialise/join NEXUS? */
    if (pSettings->initNexus)
    {
        /* The slave would initialize the audio/video/window/etc nexus resources */
        if (pSettings->slave)
        {
            rc = NEXUS_Platform_Join();
            if(rc!=NEXUS_SUCCESS) {rc=BERR_TRACE(rc);goto err_platform;}
            BDBG_MSG_TRACE((">>platform_join"));
        }
        else
        {
#ifndef BCMNEXUS_PRECONFIGURED_HEAPS
            setupNexusHeaps(&platformSettings, nexusHeapIndex);
#endif
            // panel init for DTV
            setupDTVPanel(&platformSettings);

            rc = NEXUS_Platform_Init(&platformSettings);
            if(rc!=NEXUS_SUCCESS) {rc=BERR_TRACE(rc);goto err_platform;}
            BDBG_MSG_TRACE((">>%s", __FUNCTION__));

#if BCMNEXUS_HAS_IPC
            NEXUS_Platform_GetDefaultStartServerSettings(&serverSettings);
            serverSettings.allowUnauthenticatedClients = pSettings->allowUnauthenticatedClients;
            rc = NEXUS_Platform_StartServer(&serverSettings);
            BDBG_ASSERT(!rc);
#endif
        }
    }

    NEXUS_Platform_GetConfiguration(&platformConfig);

#ifdef BCMNEXUS_PRECONFIGURED_HEAPS

    // In this case the heaps have already been configured correctly
    // for the system by Nexus. All we need to do is to ask for the
    // primary display heap and set our offscreen heap

    SET_NEXUS_GRAPHICS_DISPLAY_HEAP_HND(   (pSettings->displayHeap   == -1) ? NEXUS_Platform_GetFramebufferHeap(0) : HEAP[pSettings->displayHeap] );

    if (pSettings->offscreenHeap == -1)
    {
        NEXUS_HeapHandle offscreen = NEXUS_Platform_GetFramebufferHeap(NEXUS_OFFSCREEN_SURFACE);
        if (offscreen == NULL)
        {
            BDBG_MSG(("Failed to get offscreen heap details from Nexus looking for alternative ...\n"));
            offscreen = HEAP[2]; // 40nm SOC default Application heap

            if (offscreen == NULL) // lets just use the same as the main heap
                offscreen = GET_NEXUS_GRAPHICS_DISPLAY_HEAP_HND();
        }

        SET_NEXUS_GRAPHICS_OFFSCREEN_HEAP_HND(offscreen);
    }
    else
        SET_NEXUS_GRAPHICS_OFFSCREEN_HEAP_HND( HEAP[pSettings->offscreenHeap] );
#else
    SET_NEXUS_GRAPHICS_DISPLAY_HEAP_HND(   (pSettings->displayHeap   == -1) ? HEAP[nexusHeapIndex[0]] : HEAP[pSettings->displayHeap] );
    SET_NEXUS_GRAPHICS_OFFSCREEN_HEAP_HND( (pSettings->offscreenHeap == -1) ? HEAP[nexusHeapIndex[1]] : HEAP[pSettings->offscreenHeap] );
#endif

    // Always on memc0 for reg access in nexus gfx 2d apparently.
    //SET_NEXUS_GRAPHICS_PALETTE_HEAP_HND(platformConfig.heap[0]);
    SET_NEXUS_GRAPHICS_PALETTE_HEAP_HND(   (pSettings->paletteHeap   == -1) ? HEAP[0] : HEAP[pSettings->paletteHeap]);
#ifdef BCMNEXUS_VIDEO_DECODER_MFD_MAP_HEAP
    nexusVideoHeapIndex = platformSettings.videoDecoderModuleSettings.mfdMapping[0];
#else
    nexusVideoHeapIndex = 0;
#endif
    SET_NEXUS_VIDEO_HEAP_HND(              (pSettings->videoHeap     == -1) ? HEAP[platformSettings.videoDecoderModuleSettings.avdHeapIndex[nexusVideoHeapIndex]] :
                                                                              HEAP[pSettings->videoHeap]);

    sharedHandles->display_heap   = GET_NEXUS_GRAPHICS_DISPLAY_HEAP_HND();
    sharedHandles->offscreen_heap = GET_NEXUS_GRAPHICS_OFFSCREEN_HEAP_HND();
    sharedHandles->palette_heap   = GET_NEXUS_GRAPHICS_PALETTE_HEAP_HND();
    sharedHandles->video_heap     = GET_NEXUS_VIDEO_HEAP_HND();

    displayNexusHeaps(&platformConfig);

    NEXUS_Display_GetDefaultSettings(&displayCfg);

    if (pSettings->videoFormat != NEXUS_VideoFormat_eUnknown)
        displayCfg.format = pSettings->videoFormat;

    SET_NEXUS_VIDEO_FORMAT_PREFERRED(displayCfg.format);
    sharedHandles->video_format_preferred = GET_NEXUS_VIDEO_FORMAT_PREFERRED();

    err = configurePlatformDisplay( &pSettings->display0, &pSettings->display1, displayCfg, sharedHandles, platformConfig );
    if (err == -1)
        goto err_output;
    else if (err == -2)
        goto err_display;

    /* Should we open and configure the graphics 2D module? */
#if NEXUS_HAS_GRAPHICS2D || NEXUS_HAS_ZSP_GRAPHICS
    if (pSettings->graphics2d == NULL)
    {
        NEXUS_Graphics2DOpenSettings graphicsSettings;

        NEXUS_Graphics2D_GetDefaultOpenSettings(&graphicsSettings);
#ifndef NEXUS_GRAPHICS2D_QUEUED
        graphicsSettings.preAllocPacketMemory = false;
        graphicsSettings.maxOperations = 0;
#endif
        pSettings->graphics2d = NEXUS_Graphics2D_Open(0, &graphicsSettings );
        if(!pSettings->graphics2d) {rc=BERR_TRACE(NEXUS_NOT_SUPPORTED); goto err_graphics;}
    }
    SET_NEXUS_GRAPHICS2D_HND(pSettings->graphics2d);
    sharedHandles->graphice2D_Handle = pSettings->graphics2d;
#endif

    /* Should we open and configure the graphics 3D module? */
#if NEXUS_HAS_GRAPHICS3D && defined(BCMNEXUS_GFX_HAS_P3D)
    if (pSettings->graphics3d == NULL)
    {
        pSettings->graphics3d = NEXUS_Graphics3D_Open(0, NULL);
    }
    SET_NEXUS_GRAPHICS3D_HND(pSettings->graphics3d);
    sharedHandles->graphice3D_Handle = pSettings->graphics3d;
#endif

    /* Should we open and configure the picture decoder module? */
#ifdef NEXUS_HAS_PICTURE_DECODER
    if (pSettings->pictureDecoder == NULL && !getenv("sw_picture_decode"))
    {
        NEXUS_PictureDecoderOpenSettings decoderSettings;

        NEXUS_PictureDecoder_GetDefaultOpenSettings(&decoderSettings);
        decoderSettings.multiScanBufferSize = 16*1024*1024;
        decoderSettings.bufferSize = 9*1024*1024;
        pSettings->pictureDecoder = NEXUS_PictureDecoder_Open(0, &decoderSettings);
    }

    if (!pSettings->pictureDecoder) {
        rc=BERR_TRACE(NEXUS_NOT_SUPPORTED);
        SET_NEXUS_PICTURE_DECODER_HND(NULL);
    } else {
        SET_NEXUS_PICTURE_DECODER_HND(pSettings->pictureDecoder);
    }
    sharedHandles->pic_decode_handle = pSettings->pictureDecoder;
#endif

#if NEXUS_HAS_GRAPHICS2D || NEXUS_HAS_ZSP_GRAPHICS
    rc = BKNI_CreateEvent(&nexus_graphics2d_sync_event);
    if(rc!=BERR_SUCCESS) {rc=BERR_TRACE(rc);goto err_event;}
    rc = BKNI_CreateMutex(&nexus_graphics2d_sync_lock);
    if(rc!=BERR_SUCCESS) {rc=BERR_TRACE(rc);goto err_lock;}
#endif

    BDBG_MSG_TRACE(("<<%s", __FUNCTION__));

    return 0;

#if NEXUS_HAS_GRAPHICS2D || NEXUS_HAS_ZSP_GRAPHICS
err_lock:
err_event:
err_graphics:
#endif
err_output:
err_display:
err_platform:
    return -1;
}

void DFB_Platform_Uninit(void)
{
    BDBG_MSG_TRACE((">>%s", __FUNCTION__));

    /*The SharedMem variable will be set only for the master process*/
    if(!sharedMem)
    {
        BDBG_MSG(("Shared Memory Is Not Setup. Nothing to clean\n"));
        return;
    } else {
        BDBG_MSG(("Destroying The Shared Memory\n"));
        destroySharedMemory(sharedMem);
        sharedMem=NULL;
    }

#if NEXUS_HAS_GRAPHICS3D && defined(BCMNEXUS_GFX_HAS_P3D)
    NEXUS_Graphics3D_Close(GET_NEXUS_GRAPHICS3D_HND());
#endif

#if NEXUS_HAS_GRAPHICS2D || NEXUS_HAS_ZSP_GRAPHICS
    NEXUS_Graphics2D_Close(GET_NEXUS_GRAPHICS2D_HND());

    BKNI_DestroyMutex(nexus_graphics2d_sync_lock);
    BKNI_DestroyEvent(nexus_graphics2d_sync_event);
#endif

    NEXUS_Display_RemoveAllOutputs(GET_NEXUS_DISPLAY_HND(0));
    NEXUS_Display_Close(GET_NEXUS_DISPLAY_HND(0));

#if (NUM_DISPLAYS > 1)
    NEXUS_Display_RemoveAllOutputs(GET_NEXUS_DISPLAY_HND(1));
    NEXUS_Display_Close(GET_NEXUS_DISPLAY_HND(1));
#endif

#ifdef NEXUS_HAS_PICTURE_DECODER
    if (GET_NEXUS_PICTURE_DECODER_HND() != NULL) {
        NEXUS_PictureDecoder_Close(GET_NEXUS_PICTURE_DECODER_HND());
    }
#endif
#if BCMNEXUS_HAS_IPC
    NEXUS_Platform_StopServer();
#endif

    BDBG_MSG_TRACE(("<<%s", __FUNCTION__));

    /*NEXUS_Platform_Uninit();*/
    BDBG_Uninit();
}


/* Constructor Entry Point */
void platform_init(void)
{
    if ( DFB_Platform_Init(NULL) != 0 )
        _exit(1);
}

/* Destructor Entry Point */
void platform_uninit(void)
{
    DFB_Platform_Uninit();
}

#if NEXUS_HAS_GRAPHICS2D || NEXUS_HAS_ZSP_GRAPHICS
static void platform_nexus_graphics2d_complete(void *data, int unused)
{
    BDBG_MSG_TRACE((">>platform_nexus_graphics2d"));
    BSTD_UNUSED(unused);
    BKNI_SetEvent((BKNI_EventHandle)data);
    BDBG_MSG_TRACE(("<<platform_nexus_graphics2d"));
}

static void platform_nexus_graphics2d_sync(void)
{
    NEXUS_Error rc;
    NEXUS_CallbackDesc callback;

    BDBG_MSG_TRACE((">>platform_nexus_graphics2d_sync"));
    if(nexus_graphics2d_sync_lock) {
        BKNI_AcquireMutex(nexus_graphics2d_sync_lock);
        callback.callback = platform_nexus_graphics2d_complete;
        callback.context = nexus_graphics2d_sync_event;
        rc = NEXUS_Graphics2D_Checkpoint(GET_NEXUS_GRAPHICS2D_HND(), &callback);
        if(rc==NEXUS_SUCCESS) {
            rc=BKNI_WaitForEvent(nexus_graphics2d_sync_event, 1000); /* wait one second */
            if(rc!=BERR_SUCCESS) {rc=BERR_TRACE(rc);} /* ignore an error */
        } else {
            if(rc!=BERR_SUCCESS) {rc=BERR_TRACE(rc);} /* ignore an error */
        }
        BKNI_ReleaseMutex(nexus_graphics2d_sync_lock);
    }
    BDBG_MSG_TRACE(("<<platform_nexus_graphics2d_sync"));
    return;
}
#endif

/*
* This function will create the shared memory and give you the
* void pointer to that memory
*/
static
void *
setupSharedMemory(size_t shMemSz,bool bCreate)
{
    key_t   key=BCMNEXUS_DFB_SHMEM_KEY;
    char    *shm;
    int     shmid,shmflag;

    /*
    * Create the shared memory segment.
    */
    shmflag = 0666;
    if(bCreate) {
        shmflag |= IPC_CREAT;
        BDBG_MSG(("Setting up Shared Memory in Master Mode"));
    }

    BDBG_MSG(("Creating Shared Memory\n"));
    if ((shmid = shmget(key, shMemSz, shmflag)) < 0) {
        perror("shmget");
        return NULL;
    }

    /*
    * Now we attach the segment to our process space.
    */
    BDBG_MSG(("Attaching the Shared Memory [id=%d]\n", shmid));
    if ((shm = shmat(shmid, NULL, 0)) == (char *) -1) {
        perror("shmat");
        return NULL;
    }

    BDBG_MSG(("Returning Shared Memory Address:%p\n",shm));
    shmid_g = shmid;
    return shm;
}

static
void
destroySharedMemory(void * shMem)
{
    BDBG_MSG(("destroySharedMemory shMem=%p shmid=%d", shMem, shmid_g));
    if(shmdt(shMem) == -1) {
        BDBG_ERR(("shmdt - unattach failed"));
    }
    if (shmctl(shmid_g, IPC_RMID, (struct shmid_ds *)NULL) == -1) {
        BDBG_ERR(("shmctl - destroy failed"));
    }
}

/* returns false if process is in slave mode
 *         true if process is in master mode */
static bool initSharedMemory(size_t shMemSz)
{
    key_t  key = BCMNEXUS_DFB_SHMEM_KEY;
    int    shmid, shmflag = 0666;
    struct shmid_ds shmds;

    if (getenv("dfb_no_platform_init") && !strcmp(getenv("dfb_no_platform_init"), "y")) {
        BDBG_MSG(("%s():dfb_no_platform_init=y, skip platform_init\n", __FUNCTION__));
        return false;
    }

    shmid = shmget(key, shMemSz, shmflag);
    BDBG_MSG(("%s(): shmid=%d [ret=%d]\n", __FUNCTION__, shmid, (shmid < 0)));

    if (shmid >= 0) {
        if (shmctl(shmid, IPC_STAT, &shmds) < 0)
        {
            perror("shmctl IPC_STAT");
        } else {
            BDBG_MSG(("%s(): shmds.shm_nattch=%d\n", __FUNCTION__, (unsigned)shmds.shm_nattch));
            if ((shmds.shm_nattch == 0) && (shmctl(shmid, IPC_RMID, (struct shmid_ds *)NULL) < 0)) {
                perror("shmctl destroy");
            }
        }
        return ((unsigned)shmds.shm_nattch == 0);
    }
    return true;
}

void displayNexusHeaps( NEXUS_PlatformConfiguration *platformConfig)
{

    for (int i=0;i<NEXUS_MAX_HEAPS;i++) {
        NEXUS_Error rc;
        NEXUS_MemoryStatus status;
#if NEXUS_DTV_PLATFORM
        NEXUS_HeapHandle heap = platformConfig->memc[0].heap[i];
#else
        NEXUS_HeapHandle heap = platformConfig->heap[i];
#endif
        if (!heap)
            continue;

        rc = NEXUS_Heap_GetStatus(heap, &status);
        BDBG_ASSERT(!rc);
        BDBG_MSG_TRACE(("platform_init: Heap[%d]: memcIndex=%d, size=%d, offset=0x%08x, type=%d\n",
                        i, status.memcIndex, status.size, status.offset, status.memoryType));
    }
}

unsigned long
get_gfx_heap_size(void)
{
    unsigned long heapSize = DEFAULT_GFX_HEAP_SIZE;
    char *pValue;
    char invalid = '\0';
    char *pInvalid = &invalid;

    pValue = getenv("gfx_heap_size");
    if (pValue != NULL)
    {
        heapSize = strtoul(pValue, &pInvalid, 0);
        if ((heapSize == 0) || (*pInvalid != '\0'))
        {
            BDBG_WRN(("Invalid \"gfx_heap_size\" value specified - defaulting to %ld bytes.\n", heapSize));
            heapSize = DEFAULT_GFX_HEAP_SIZE;
        }
    }

    if (heapSize % 4096)
    {
        heapSize -= (heapSize % 4096);
        BDBG_WRN(("Invalid \"gfx_heap_size\" value specified - must be 4k aligned rounding down to %ld bytes.\n", heapSize));
    }

    return heapSize;
}
