/******************************************************************************
 *    (c)2008-2011 Broadcom Corporation
 *
 * This program is the proprietary software of Broadcom Corporation and/or its licensors,
 * and may only be used, duplicated, modified or distributed pursuant to the terms and
 * conditions of a separate, written license agreement executed between you and Broadcom
 * (an "Authorized License").  Except as set forth in an Authorized License, Broadcom grants
 * no license (express or implied), right to use, or waiver of any kind with respect to the
 * Software, and Broadcom expressly reserves all rights in and to the Software and all
 * intellectual property rights therein.  IF YOU HAVE NO AUTHORIZED LICENSE, THEN YOU
 * HAVE NO RIGHT TO USE THIS SOFTWARE IN ANY WAY, AND SHOULD IMMEDIATELY
 * NOTIFY BROADCOM AND DISCONTINUE ALL USE OF THE SOFTWARE.
 *
 * Except as expressly set forth in the Authorized License,
 *
 * 1.     This program, including its structure, sequence and organization, constitutes the valuable trade
 * secrets of Broadcom, and you shall use all reasonable efforts to protect the confidentiality thereof,
 * and to use this information only in connection with your use of Broadcom integrated circuit products.
 *
 * 2.     TO THE MAXIMUM EXTENT PERMITTED BY LAW, THE SOFTWARE IS PROVIDED "AS IS"
 * AND WITH ALL FAULTS AND BROADCOM MAKES NO PROMISES, REPRESENTATIONS OR
 * WARRANTIES, EITHER EXPRESS, IMPLIED, STATUTORY, OR OTHERWISE, WITH RESPECT TO
 * THE SOFTWARE.  BROADCOM SPECIFICALLY DISCLAIMS ANY AND ALL IMPLIED WARRANTIES
 * OF TITLE, MERCHANTABILITY, NONINFRINGEMENT, FITNESS FOR A PARTICULAR PURPOSE,
 * LACK OF VIRUSES, ACCURACY OR COMPLETENESS, QUIET ENJOYMENT, QUIET POSSESSION
 * OR CORRESPONDENCE TO DESCRIPTION. YOU ASSUME THE ENTIRE RISK ARISING OUT OF
 * USE OR PERFORMANCE OF THE SOFTWARE.
 *
 * 3.     TO THE MAXIMUM EXTENT PERMITTED BY LAW, IN NO EVENT SHALL BROADCOM OR ITS
 * LICENSORS BE LIABLE FOR (i) CONSEQUENTIAL, INCIDENTAL, SPECIAL, INDIRECT, OR
 * EXEMPLARY DAMAGES WHATSOEVER ARISING OUT OF OR IN ANY WAY RELATING TO YOUR
 * USE OF OR INABILITY TO USE THE SOFTWARE EVEN IF BROADCOM HAS BEEN ADVISED OF
 * THE POSSIBILITY OF SUCH DAMAGES; OR (ii) ANY AMOUNT IN EXCESS OF THE AMOUNT
 * ACTUALLY PAID FOR THE SOFTWARE ITSELF OR U.S. $1, WHICHEVER IS GREATER. THESE
 * LIMITATIONS SHALL APPLY NOTWITHSTANDING ANY FAILURE OF ESSENTIAL PURPOSE OF
 * ANY LIMITED REMEDY.
 *
 * $brcm_Workfile: platform_init.h $
 * $brcm_Revision: DirectFB_1_4_Port/20 $
 * $brcm_Date: 3/11/11 5:05p $
 *
 * Module Description:
 *
 * Revision History:
 *
 * $brcm_Log: /AppLibs/opensource/directfb/src/broadcom_files/private/DirectFB/1.4.1/platform_init.h $
 * 
 * DirectFB_1_4_Port/20   3/11/11 5:05p christ
 * SW7468-370: DirectFB: 1.4.1 release fails to compile for platforms
 *  without component output.
 *
 * DirectFB_1_4_Port/19   2/25/11 6:23p apandav
 * SW3520-44: BCM935230: Error while compiling DirectFB_1_4_Port
 *
 * DirectFB_1_4_Port/18   2/18/11 2:35p robertwm
 * SW7468-366: DirectFB: Platforms without component output should not
 *  call component functions.
 *
 * DirectFB_1_4_Port/17   2/4/11 8:40p robertwm
 * SW7400-3002: DirectFB: Support non-DFB master application in
 *  platform_init code.
 * SW7400-3000: DirectFB: Support non-DFB master application in
 *  platform_init code.
 *
 * DirectFB_1_4_Port/16   2/3/11 6:01p christ
 * SW7420-1458: DirectFB: Rework internal handling of nexus heaps.
 *
 * DirectFB_1_4_Port/15   1/25/11 8:11p robertwm
 * SW7420-1424: DirectFB: Refactor platform_init code to separate DTV and
 *  STB parts and to support NEXUS heap m'ment.
 *
 * DirectFB_1_4_Port/DirectFB_STEREO_3D_JIRA_SW7422_138/4   1/18/11 1:26p christ
 * Fix DTV compile problems.
 *
 * DirectFB_1_4_Port/DirectFB_STEREO_3D_JIRA_SW7422_138/3   1/17/11 5:19p christ
 * Better fix for compile issue.
 *
 * DirectFB_1_4_Port/DirectFB_STEREO_3D_JIRA_SW7422_138/2   1/17/11 2:58p christ
 * libinit changes.
 *
 * DirectFB_1_4_Port/DirectFB_STEREO_3D_JIRA_SW7422_138/1   12/13/10 5:23p christ
 * Initial stereo 3d commit
 *
 * DirectFB_1_4_Port/12   10/21/10 2:06p robertwm
 * SW35230-1873: DirectFB: Platform Initialisation code fails to build on
 *  DTV platforms.
 *
 * DirectFB_1_4_Port/11   10/20/10 5:11p robertwm
 * SW7420-1195: DirectFB: Move the platform_init shared handles definition
 *  to a header file for portability.
 *
 * DirectFB_1_4_Port/10   8/20/10 3:36p robertwm
 * SW7405-4791: DirectFB: DirectFB-1.4.1 Phase 2.5 fails to build with
 *  Nexus Phase 5.0 reference software.
 *
 * DirectFB_1_4_Port/9   8/11/10 4:58p robertwm
 * SW35230-957: DirectFB: platform_init fails to build on 35230 with
 *  recent 3D changes.
 *
 * DirectFB_1_4_Port/8   8/11/10 3:24a robertwm
 * SW3556-1177: DirectFB: system_initialize segfault with DirectFB-1.4.1
 *  Phase 2.5 PreRel.
 *
 * DirectFB_1_4_Port/7   8/11/10 1:58a robertwm
 * SW7405-4250: DirectFB: Screen destination rectangle and source
 *  rectangle not saved across contexts.
 * SW7405-4743: DirectFB: Multiple instances of df_andi.c do not get
 *  displayed.
 *
 * DirectFB_1_4_Port/6   8/6/10 8:04p robertwm
 * SW7405-4724: DirectFB: Support OpenGL ES 1.0 applications.
 *
 * DirectFB_1_4_Port/5   4/9/10 4:55p robertwm
 * SW7405-4183: DirectFB: Add support for Texture Triangles in the
 *  graphics driver.
 *
 * DirectFB_1_4_Port/4   12/23/09 2:18p robertwm
 * SW7550-51: Support DirectFB on 7550.
 *
 * DirectFB_1_4_Port/SW7550-51/1   12/22/09 3:22p mphillip
 * SW7550-51: Add 7550 build support (on branch)
 *
 * DirectFB_1_4_Port/3   12/17/09 5:24p robertwm
 * SW3556-944: DirectFB: Increase size of graphics heap to 16MB for DTV
 *  platforms.
 *
 * DirectFB_1_4_Port/2   8/7/09 3:43p robertwm
 * PR56902: Require single graphics memory heap to avoid fragmentation.
 * PR56721: Add HDMI output support.
 *
 * DirectFB_1_4_Port/1   7/8/09 3:19p mphillip
 * PR56658: Add DirectFB 1.4.1 broadcom_files
 *
 * 6   11/21/08 8:06p vsilyaev
 * PR 49524: Added graphics2d_sync routine that could be used by different
 *  users of Nexus_Graphics2D
 *
 * 5   11/17/08 3:30p gmohile
 * PR 48730 : Expose surface handle to app
 *
 * 4   10/24/08 12:42p gmohile
 * PR 46490 : Add hw picture decode support
 *
 * 3   10/14/08 3:53p gmohile
 * PR 45243 : Fix nexus handles
 *
 * 2   9/15/08 11:49a gmohile
 * PR 46734 : Add dual output support
 *
 * 1   8/29/08 2:35p gmohile
 * PR 45243 : Do nexus platform init and display open outside directfb
 *
 *****************************************************************************/
#ifndef __BCMNEXUS__PLATFORM_INIT_H__
#define __BCMNEXUS__PLATFORM_INIT_H__

#include "gfxdrivers/bcmnexus/bcmnexus_gfx_defines.h"
#include "systems/bcmnexus/bcmnexus_sys_defines.h"

#include "dfb_platform.h"

typedef void (*output_callback_t)(NEXUS_DisplayHandle display, int output, bool connected);

BDBG_MODULE(platform_init);
#define BDBG_MSG_TRACE(x)   BDBG_MSG(x)

#if NEXUS_DTV_PLATFORM || (NEXUS_NUM_DISPLAYS <= 1)
#define NUM_DISPLAYS 1
#else
#define NUM_DISPLAYS 2
#endif

/* Includes graphics window and video windows */
#define NUM_GFX_LAYERS_PER_DISPLAY  1
#define NUM_VID_LAYERS_PER_DISPLAY  NEXUS_NUM_VIDEO_WINDOWS
#define NUM_LAYERS_PER_DISPLAY      (NUM_GFX_LAYERS + NUM_VID_LAYERS)
#define NUM_LAYERS                  (NUM_LAYERS_PER_DISPLAY * NUM_DISPLAYS)

#define BCMNEXUS_DFB_SHMEM_KEY      5678


/*
* Structure for the shared memory and all the handles
* that we will need to share across the processes.
*/
typedef struct _NEXUS_SHARED_HANDLES{
    NEXUS_HeapHandle            display_heap;
    NEXUS_HeapHandle            offscreen_heap;
    NEXUS_HeapHandle            palette_heap;
    NEXUS_HeapHandle            video_heap;
    NEXUS_VideoFormat           video_format_preferred;
    bool                        is_component_connected;
    NEXUS_DisplayHandle         disp_handle0;
#if (NUM_DISPLAYS > 1)
    NEXUS_DisplayHandle         disp_handle1;
#endif
#ifdef NEXUS_HAS_PICTURE_DECODER
    NEXUS_PictureDecoderHandle  pic_decode_handle;
#endif
#if NEXUS_HAS_GRAPHICS2D || NEXUS_HAS_ZSP_GRAPHICS
    NEXUS_Graphics2DHandle      graphice2D_Handle;
#endif
#if NEXUS_HAS_GRAPHICS3D && defined(BCMNEXUS_GFX_HAS_P3D)
    NEXUS_Graphics3DHandle      graphice3D_Handle;
#endif
} NEXUS_Shared_Handles;


extern NEXUS_HeapHandle  nexus_display_heap;
extern NEXUS_HeapHandle  nexus_offscreen_heap;
extern NEXUS_HeapHandle  nexus_palette_heap;
extern NEXUS_HeapHandle  nexus_video_heap;

extern void (*nexus_graphics2d_sync)(void);
extern NEXUS_DisplayHandle nexus_display_handle0;
#if (NUM_DISPLAYS > 1)
extern NEXUS_DisplayHandle nexus_display_handle1;
#endif
#if NEXUS_HAS_GRAPHICS2D || NEXUS_HAS_ZSP_GRAPHICS
extern NEXUS_Graphics2DHandle nexus_graphics2d_handle;
#endif
#if NEXUS_HAS_GRAPHICS3D && defined(BCMNEXUS_GFX_HAS_P3D)
extern NEXUS_Graphics3DHandle nexus_graphics3d_handle;
#endif
extern NEXUS_VideoFormat nexus_video_format_preferred;

#ifdef NEXUS_HAS_PICTURE_DECODER
extern NEXUS_PictureDecoderHandle nexus_picturedecoder_handle;
#endif

#ifdef NEXUS_NUM_COMPONENT_OUTPUTS
extern bool nexus_is_component_connected;
extern output_callback_t component_output_callback;
#endif

#define SET_NEXUS_DISPLAY_HND(id,val)           (nexus_display_handle##id = val)
#define GET_NEXUS_DISPLAY_HND(id)                nexus_display_handle##id

#define SET_NEXUS_GRAPHICS2D_HND(val)           (nexus_graphics2d_handle = val)
#define GET_NEXUS_GRAPHICS2D_HND()               nexus_graphics2d_handle

#define SET_NEXUS_GRAPHICS3D_HND(val)           (nexus_graphics3d_handle = val)
#define GET_NEXUS_GRAPHICS3D_HND()               nexus_graphics3d_handle

#define SET_NEXUS_HDMI_HND(val)           (nexus_hdmi_handle = val)
#define GET_NEXUS_HDMI_HND()               nexus_hdmi_handle

#define SET_NEXUS_GRAPHICS_DISPLAY_HEAP_HND(val)      (nexus_display_heap = val)
#define GET_NEXUS_GRAPHICS_DISPLAY_HEAP_HND()            nexus_display_heap

#define SET_NEXUS_GRAPHICS_OFFSCREEN_HEAP_HND(val)  (nexus_offscreen_heap = val)
#define GET_NEXUS_GRAPHICS_OFFSCREEN_HEAP_HND()        nexus_offscreen_heap

#define SET_NEXUS_GRAPHICS_PALETTE_HEAP_HND(val)  (nexus_palette_heap = val)
#define GET_NEXUS_GRAPHICS_PALETTE_HEAP_HND()        nexus_palette_heap

#define SET_NEXUS_VIDEO_HEAP_HND(val)  (nexus_video_heap = val)
#define GET_NEXUS_VIDEO_HEAP_HND()        nexus_video_heap

#define SET_NEXUS_PICTURE_DECODER_HND(val)      (nexus_picturedecoder_handle = val)
#define GET_NEXUS_PICTURE_DECODER_HND()          nexus_picturedecoder_handle

#define SET_NEXUS_VIDEO_FORMAT_PREFERRED(val)   (nexus_video_format_preferred = val)
#define GET_NEXUS_VIDEO_FORMAT_PREFERRED()       nexus_video_format_preferred

#ifdef NEXUS_NUM_COMPONENT_OUTPUTS
#define SET_NEXUS_COMPONENT_CONNECTED(val)      (nexus_is_component_connected = val)
#define GET_NEXUS_COMPONENT_CONNECTED()          nexus_is_component_connected
#endif

void setupNexusHeaps( NEXUS_PlatformSettings *platformSettings,
                      unsigned char *nexusHeapIndex);

void displayNexusHeaps( NEXUS_PlatformConfiguration *platformConfig);

void setupDTVPanel(NEXUS_PlatformSettings *platformSettings);

int configurePlatformDisplay(NEXUS_DisplayHandle *display0,
                             NEXUS_DisplayHandle *display1,
                             NEXUS_DisplaySettings displayCfg,
                             NEXUS_Shared_Handles *sharedHandles,
                             NEXUS_PlatformConfiguration platformConfig);

unsigned long get_gfx_heap_size(void);

#if NEXUS_DTV_PLATFORM
#include "dtv_init.h"
#else
#include "stb_init.h"
#endif

#endif
