/***************************************************************************
 *     (c)2007-2011 Broadcom Corporation
 *
 *  This program is the proprietary software of Broadcom Corporation and/or its licensors,
 *  and may only be used, duplicated, modified or distributed pursuant to the terms and
 *  conditions of a separate, written license agreement executed between you and Broadcom
 *  (an "Authorized License").  Except as set forth in an Authorized License, Broadcom grants
 *  no license (express or implied), right to use, or waiver of any kind with respect to the
 *  Software, and Broadcom expressly reserves all rights in and to the Software and all
 *  intellectual property rights therein.  IF YOU HAVE NO AUTHORIZED LICENSE, THEN YOU
 *  HAVE NO RIGHT TO USE THIS SOFTWARE IN ANY WAY, AND SHOULD IMMEDIATELY
 *  NOTIFY BROADCOM AND DISCONTINUE ALL USE OF THE SOFTWARE.
 *
 *  Except as expressly set forth in the Authorized License,
 *
 *  1.     This program, including its structure, sequence and organization, constitutes the valuable trade
 *  secrets of Broadcom, and you shall use all reasonable efforts to protect the confidentiality thereof,
 *  and to use this information only in connection with your use of Broadcom integrated circuit products.
 *
 *  2.     TO THE MAXIMUM EXTENT PERMITTED BY LAW, THE SOFTWARE IS PROVIDED "AS IS"
 *  AND WITH ALL FAULTS AND BROADCOM MAKES NO PROMISES, REPRESENTATIONS OR
 *  WARRANTIES, EITHER EXPRESS, IMPLIED, STATUTORY, OR OTHERWISE, WITH RESPECT TO
 *  THE SOFTWARE.  BROADCOM SPECIFICALLY DISCLAIMS ANY AND ALL IMPLIED WARRANTIES
 *  OF TITLE, MERCHANTABILITY, NONINFRINGEMENT, FITNESS FOR A PARTICULAR PURPOSE,
 *  LACK OF VIRUSES, ACCURACY OR COMPLETENESS, QUIET ENJOYMENT, QUIET POSSESSION
 *  OR CORRESPONDENCE TO DESCRIPTION. YOU ASSUME THE ENTIRE RISK ARISING OUT OF
 *  USE OR PERFORMANCE OF THE SOFTWARE.
 *
 *  3.     TO THE MAXIMUM EXTENT PERMITTED BY LAW, IN NO EVENT SHALL BROADCOM OR ITS
 *  LICENSORS BE LIABLE FOR (i) CONSEQUENTIAL, INCIDENTAL, SPECIAL, INDIRECT, OR
 *  EXEMPLARY DAMAGES WHATSOEVER ARISING OUT OF OR IN ANY WAY RELATING TO YOUR
 *  USE OF OR INABILITY TO USE THE SOFTWARE EVEN IF BROADCOM HAS BEEN ADVISED OF
 *  THE POSSIBILITY OF SUCH DAMAGES; OR (ii) ANY AMOUNT IN EXCESS OF THE AMOUNT
 *  ACTUALLY PAID FOR THE SOFTWARE ITSELF OR U.S. $1, WHICHEVER IS GREATER. THESE
 *  LIMITATIONS SHALL APPLY NOTWITHSTANDING ANY FAILURE OF ESSENTIAL PURPOSE OF
 *  ANY LIMITED REMEDY.
 *
 * $brcm_Workfile: bcmnexus.c $
 * $brcm_Revision: DirectFB_1_4_Port/25 $
 *
 * $brcm_Date: 3/17/11 12:42p $
 *
 * Module Description:
 *
 * Revision History:
 *
 * $brcm_Log: /AppLibs/opensource/directfb/src/broadcom_files/private/DirectFB/1.4/systems/bcmnexus/bcmnexus.c $
 * 
 * DirectFB_1_4_Port/25   3/17/11 12:42p robertwm
 * SW7422-378: DirectFB: Destroying a Palettised surface in a client
 *  application causes NEXUS assertion during free.
 * SW7422-383: DirectFB: Need to serialise access to SID to prevent
 *  assertions.
 * 
 * DirectFB_1_4_Port/24   3/11/11 5:05p christ
 * SW7468-370: DirectFB: 1.4.1 release fails to compile for platforms
 *  without component output.
 *
 * DirectFB_1_4_Port/23   3/1/11 8:50p robertwm
 * SW7420-1559: DirectFB: NEXUS surface assertion in multi-app mode when
 *  closing down a DFB client.
 *
 * DirectFB_1_4_Port/22   2/15/11 6:09p apandav
 * SW35230-3034: Mainline DirectFB build fails in PCL 2k11 source tree
 *
 * DirectFB_1_4_Port/21   2/8/11 12:44p robertwm
 * SW7420-1467: DirectFB: Multiple apps should NOT share NEXUS Display
 *  handle.
 *
 * DirectFB_1_4_Port/20   2/3/11 6:21p christ
 * SW7420-1458: DirectFB: Rework internal handling of nexus heaps. -
 *  remove second nexus join.
 *
 * DirectFB_1_4_Port/19   2/3/11 6:01p christ
 * SW7420-1458: DirectFB: Rework internal handling of nexus heaps.
 *
 * DirectFB_1_4_Port/18   1/19/11 12:25p scchang
 * SW7420-1394: video layer support in DFB
 *
 * DirectFB_1_4_Port/17   12/3/10 8:39p raywu
 * SW7550-629: DirectFB: Integrated ZSP code to DFB packet buffer branch
 *
 * DirectFB_1_4_Port/16   12/1/10 3:47p christ
 * SW7420-1283: DirectFB: Add support for extra Nexus heap on dual MEMC
 *  parts
 *
 * DirectFB_1_4_Port/15   8/20/10 3:38p robertwm
 * SW7405-4791: DirectFB: DirectFB-1.4.1 Phase 2.5 fails to build with
 *  Nexus Phase 5.0 reference software.
 *
 * DirectFB_1_4_Port/14   8/11/10 3:24a robertwm
 * SW3556-1177: DirectFB: system_initialize segfault with DirectFB-1.4.1
 *  Phase 2.5 PreRel.
 *
 * DirectFB_1_4_Port/13   8/11/10 2:01a robertwm
 * SW7405-4250: DirectFB: Screen destination rectangle and source
 *  rectangle not saved across contexts.
 * SW7405-4743: DirectFB: Multiple instances of df_andi.c do not get
 *  displayed.
 *
 * DirectFB_1_4_Port/12   7/26/10 5:32p jackf
 * SW35230-771: Added include for shim_nexus.h.
 *
 * DirectFB_1_4_Port/11   4/19/10 2:12p robertwm
 * SW3556-1099: DirectFB: SID should not be enabled if there is
 *  insufficient driver memory to initialise it.
 *
 * DirectFB_1_4_Port/10   4/15/10 10:36p robertwm
 * SW7405-4214: DirectFB: Multi-Process DirectFB fails to run.
 *
 * DirectFB_1_4_Port/9   4/15/10 7:52p robertwm
 * SW7405-4184: Allow independent mode and resolution to be specified.
 *
 * DirectFB_1_4_Port/8   4/9/10 4:24p robertwm
 * SW7405-4181: DirectFB: Allow the graphics and system module versions to
 *  be specified in the build system.
 * SW7405-4180: DirectFB: Require independent setting of graphics
 *  resolution on multiple output paths.
 *
 * DirectFB_1_4_Port/7   12/23/09 2:22p robertwm
 * SW7550-51: Support DirectFB on 7550.
 *
 * DirectFB_1_4_Port/SW7550-51/1   12/22/09 3:22p mphillip
 * SW7550-51: Add 7550 build support (on branch)
 *
 * DirectFB_1_4_Port/6   12/4/09 11:00a robertwm
 * SW7405-3539: DirectFB: Running df_andi (penguins) using default window
 *  manager in exclusive mode locks up.
 *
 * DirectFB_1_4_Port/5   11/16/09 1:27p robertwm
 * SW7405-3399: WaitForSync() function is unimplemented.
 * (Fixed graphics freeze issue).
 *
 * DirectFB_1_4_Port/4   11/13/09 7:10p robertwm
 * SW7405-3399: WaitForSync() function is unimplemented.
 *
 * DirectFB_1_4_Port/3   8/27/09 4:36p robertwm
 * SW3548-2427: DirectFB: Support 3549 panel types and fix compilation
 *  bugs.
 *
 * DirectFB_1_4_Port/2   8/21/09 11:12a robertwm
 * PR57917: DirectFB: testman and testrun don't run anymore in 1.4.1
 *  (SaWMan)
 *
 * DirectFB_1_4_Port/1   7/8/09 3:19p mphillip
 * PR56658: Add DirectFB 1.4.1 broadcom_files
 *
 * 13   3/12/09 11:54a gmohile
 * PR 53153 : Set memory alignment settings
 *
 * 12   2/10/09 10:25a gmohile
 * PR 45342 : Free graphics memory during shutdown
 *
 * 11   11/3/08 4:05p gmohile
 * PR 25109 : DO not shut down outputs before removing them
 *
 * 10   11/3/08 1:57p gmohile
 * PR 45243 : Default all DTV to panel output
 *
 * 9   10/24/08 12:42p gmohile
 * PR 46490 : Add hw picture decode support
 *
 * 8   10/15/08 3:22p gmohile
 * PR 45342 : No need to destroy primary surface at shutdown
 *
 *
 ***************************************************************************/

#include "shim_nexus.h" // Nexus function redirection

#include "bcmnexus.h"
#include "bcmnexus_screen.h"
#include "bcmnexus_layer.h"
#include "bcmnexus_pool.h"
#include "bcmnexus_video.h"

#include "bdbg.h"

#include <misc/conf.h>
#include <core/core.h>
#include <core/coredefs.h>
#include <core/coretypes.h>

#include <core/core_system.h>

#include <direct/mem.h>
#include <direct/memcpy.h>
#include <direct/thread.h>

#include <fusion/arena.h>
#include <fusion/shmalloc.h>

D_DEBUG_DOMAIN( bcmnexusSystem,  "bcmNexus/System",  "Broadcom NEXUS System module" );
D_DEBUG_DOMAIN( bcmnexusSystemX, "bcmNexus/SystemX", "Broadcom NEXUS System module eXtended debugging" );

DFB_CORE_SYSTEM( bcmnexus )

CoreDFB *dfb_bcmnexus_core = NULL;

#ifdef __GNUC__
#define UNUSED_ __attribute__ ((unused))
#else
#define UNUSED_
#endif

static FusionCallHandlerResult
bcmCallHandler( UNUSED_ int           caller,
                        int           call_arg,
                        void         *call_ptr,
                UNUSED_ void         *ctx,
                UNUSED_ unsigned int  serial,
                        int          *ret_val )
{
    DFBBCMNEXUS *dfb_bcmnexus = dfb_system_data();

    switch (call_arg) {
    case BCM_POOL_MEMORY_ALLOCATE:
    {
        size_t numBytes;
        const NEXUS_MemoryAllocationSettings *pSettings;
        void *pMemory;

        numBytes = ((bcmPoolCallMemoryAllocate *)call_ptr)->numBytes;
        pSettings = &((bcmPoolCallMemoryAllocate *)call_ptr)->settings;

        *ret_val = NEXUS_Memory_Allocate(numBytes, pSettings, &pMemory);
        ((bcmPoolCallMemoryAllocate *)call_ptr)->pMemory = pMemory;

        D_DEBUG_AT(bcmnexusSystemX, "%s() NEXUS_Memory_Allocate(%d) buffer=%p\n", __FUNCTION__, numBytes, pMemory);
        break;
    }

    case BCM_POOL_MEMORY_FREE:
    {
        void *pMemory = call_ptr;

        NEXUS_Memory_Free(pMemory);

        D_DEBUG_AT(bcmnexusSystemX, "%s() NEXUS_Memory_Free(%p)\n", __FUNCTION__, pMemory);
        break;
    }

    case BCM_POOL_SURFACE_CREATE:
    {
        NEXUS_SurfaceCreateSettings *pCreateSettings;
        NEXUS_SurfaceHandle surface;

        pCreateSettings = &((bcmPoolCallSurfaceCreate *)call_ptr)->createSettings;

        surface = NEXUS_Surface_Create(pCreateSettings);
        ((bcmPoolCallSurfaceCreate *)call_ptr)->surface = surface;

        D_DEBUG_AT(bcmnexusSystemX, "%s() NEXUS_Surface_Create() handle=%p\n", __FUNCTION__, (void *)surface);
        break;
    }

    case BCM_POOL_SURFACE_DESTROY:
    {
        NEXUS_SurfaceHandle surface;

        surface = (NEXUS_SurfaceHandle)call_ptr;

        D_DEBUG_AT(bcmnexusSystemX, "%s() NEXUS_Surface_Destroy(%p)\n", __FUNCTION__, (void *)surface);

        NEXUS_Surface_Destroy(surface);
        break;
    }

    case BCM_LAYER_SET_GFX_FRAMEBUFFER:
    {
        NEXUS_SurfaceHandle hSurface;
        NEXUS_DisplayHandle hDisplay;
        DFBDisplayLayerID   layerId;
        bcmLayerData       *layerData;

        layerId   = ((bcmLayerCallSetGfxFrameBuffer *)call_ptr)->layerId;
        hSurface  = ((bcmLayerCallSetGfxFrameBuffer *)call_ptr)->hSurface;
        layerData = bcmGetLayerData(layerId);
        hDisplay  = GET_DISPLAY_HND(dfb_bcmnexus, layerData->displayId);

        D_DEBUG_AT(bcmnexusSystemX, "%s() screen %d gfx layer %d: SetGraphicsFramebuffer=%p\n",
                    __FUNCTION__, layerData->displayId, layerId, (void *)hSurface);

        *ret_val = (NEXUS_Display_SetGraphicsFramebuffer(hDisplay, hSurface) == NEXUS_SUCCESS) ? DFB_OK : DFB_FAILURE;
        break;
    }

    case BCM_LAYER_SET_GFX_SETTINGS:
    {
        DFBDisplayLayerID       layerId;
        bcmLayerData           *layerData;
        NEXUS_GraphicsSettings *pGfxSettings;
        NEXUS_DisplayHandle     hDisplay;

        layerId      = ((bcmLayerCallSetGfxSettings *)call_ptr)->layerId;
        pGfxSettings = &((bcmLayerCallSetGfxSettings *)call_ptr)->gfxSettings;

        layerData = bcmGetLayerData(layerId);
        hDisplay  = GET_DISPLAY_HND(dfb_bcmnexus, layerData->displayId);

        D_DEBUG_AT(bcmnexusSystemX, "%s() screen %d gfx layer %d: SetGraphicsSettings=%p\n",
                   __FUNCTION__, layerData->displayId, layerId, (void *)pGfxSettings);

        *ret_val = (NEXUS_Display_SetGraphicsSettings(hDisplay, pGfxSettings) == NEXUS_SUCCESS) ? DFB_OK : DFB_FAILURE;
        break;
    }

    case BCM_SCREEN_SET_DISPLAY_SETTINGS:
    {
        DFBScreenID            displayId;
        NEXUS_DisplaySettings *pDisplaySettings;
        NEXUS_DisplayHandle    hDisplay;

        displayId        = ((bcmScreenCallSetDisplaySettings *)call_ptr)->displayId;
        hDisplay         = GET_DISPLAY_HND(dfb_bcmnexus, displayId);
        pDisplaySettings = &((bcmScreenCallSetDisplaySettings *)call_ptr)->displaySettings;

        D_DEBUG_AT(bcmnexusSystemX, "%s() screen %d: SetDisplaySettings=%p\n",
                   __FUNCTION__, displayId, (void *)pDisplaySettings);

        *ret_val = NEXUS_Display_SetSettings(hDisplay, pDisplaySettings);
        break;
    }

    case BCM_SCREEN_ENABLE_COMPONENT_OUTPUT:
    {
        DFBScreenID                 displayId;
        int                         output;
        bool                        enable;
        NEXUS_DisplayHandle         hDisplay;
        NEXUS_PlatformConfiguration platformConfig;

        displayId        = ((bcmScreenCallEnableComponentOutput *)call_ptr)->displayId;
        output           = ((bcmScreenCallEnableComponentOutput *)call_ptr)->output;
        enable           = ((bcmScreenCallEnableComponentOutput *)call_ptr)->enable;
        hDisplay         = GET_DISPLAY_HND(dfb_bcmnexus, displayId);

        D_DEBUG_AT(bcmnexusSystemX, "%s() screen %d: %s Component Output%d\n",
                   __FUNCTION__, displayId, enable ? "Enable" : "Disable", output);

        NEXUS_Platform_GetConfiguration(&platformConfig);

#if NEXUS_NUM_COMPONENT_OUTPUTS && !defined(NEXUS_DTV_PLATFORM)
        if (enable)
        {
            *ret_val = NEXUS_Display_AddOutput(hDisplay, NEXUS_ComponentOutput_GetConnector(platformConfig.outputs.component[output]));
        } else {
            *ret_val = NEXUS_Display_RemoveOutput(hDisplay, NEXUS_ComponentOutput_GetConnector(platformConfig.outputs.component[output]));
        }
#endif
        break;
    }

    default:
        D_BUG( "%s: unknown call!", __FUNCTION__ );
        *ret_val = DFB_BUG;
        break;
    }

    return FCHR_RETURN;
}

/** public **/
static void
system_get_info( CoreSystemInfo *info )
{
        D_DEBUG_AT( bcmnexusSystem, "%s()\n", __FUNCTION__ );

        info->version.major = BCMNEXUS_SYS_MAJOR_VERSION;
        info->version.minor = BCMNEXUS_SYS_MINOR_VERSION;
        info->type = CORE_ANY;
        info->caps = CSCAPS_ACCELERATION;

        snprintf( info->name, DFB_CORE_SYSTEM_INFO_NAME_LENGTH, "BCMNEXUS" );
        snprintf( info->vendor, DFB_CORE_SYSTEM_INFO_VENDOR_LENGTH, "Broadcom Corporation" );
        snprintf( info->url, DFB_CORE_SYSTEM_INFO_URL_LENGTH, "http://www.broadcom.com" );
        snprintf( info->license, DFB_CORE_SYSTEM_INFO_LICENSE_LENGTH, "proprietary" );
}

static void
register_screens_layers( DFBBCMNEXUS *dfb_bcmnexus )
{
        CoreScreen *primary_screen;
#if (NUM_DISPLAYS > 1)
        CoreScreen *secondary_screen;
#endif

        /* Register primary screen functions */
        primary_screen = dfb_screens_register( dfb_gfxcard_get_primary(), dfb_bcmnexus, &bcmnexusPrimaryScreenFuncs );

    if (primary_screen == NULL)
    {
        D_ERROR("Cannot register primary screen!");
        return;
    }
        /* Always register primary graphics layer functions FIRST */
        dfb_layers_register( primary_screen, dfb_bcmnexus, &bcmnexusPrimaryGfxLayerFuncs );

#if (NUM_DISPLAYS > 1)
        /* Register secondary screen functions */
        secondary_screen = dfb_screens_register( dfb_gfxcard_get_primary(), dfb_bcmnexus, &bcmnexusSecondaryScreenFuncs );

    if (secondary_screen == NULL)
    {
        D_ERROR("Cannot register secondary screen!");
        return;
    }

    /* Always register secondary graphics layer functions SECOND */
    dfb_layers_register( secondary_screen, dfb_bcmnexus, &bcmnexusSecondaryGfxLayerFuncs );
#endif

    /* register video layer functions */
    dfb_layers_register( primary_screen, dfb_bcmnexus, &bcmnexusVideoFuncs );
}

static void
unregister_screens_layers( DFBBCMNEXUS *dfb_bcmnexus )
{
        NEXUS_StopCallbacks(GET_DISPLAY_HND(dfb_bcmnexus, PRIMARY_DISPLAY_ID));
#if (NUM_DISPLAYS > 1)
        NEXUS_StopCallbacks(GET_DISPLAY_HND(dfb_bcmnexus, SECONDARY_DISPLAY_ID));
#endif
        fusion_skirmish_destroy( &GET_DISPLAY_VSYNC_MUTEX(dfb_bcmnexus, PRIMARY_DISPLAY_ID) );
    SET_DISPLAY_VSYNC_CB(dfb_bcmnexus, NULL, PRIMARY_DISPLAY_ID);
#if (NUM_DISPLAYS > 1)
        fusion_skirmish_destroy( &GET_DISPLAY_VSYNC_MUTEX(dfb_bcmnexus, SECONDARY_DISPLAY_ID));
    SET_DISPLAY_VSYNC_CB(dfb_bcmnexus, NULL, SECONDARY_DISPLAY_ID);
#endif
}

#if NEXUS_NUM_COMPONENT_OUTPUTS && !defined(NEXUS_DTV_PLATFORM)
static void
system_output_callback(UNUSED_ NEXUS_DisplayHandle display,
                               int                 output,
                               bool                connected)
{
    DFBBCMNEXUS *dfb_bcmnexus = (DFBBCMNEXUS *) dfb_system_data();
    SET_DISPLAY_COMPONENT_OUT(dfb_bcmnexus, connected, output);
}
#endif

static DFBResult
system_initialize( CoreDFB *core, void **data )
{
        DFBBCMNEXUS *dfb_bcmnexus = NULL;
        FusionSHMPoolShared *pool;

        D_DEBUG_AT( bcmnexusSystem, "%s() core=%p\n", __FUNCTION__, (void *)core );

        /* Allocate shared memory state object */
        pool = dfb_core_shmpool( core );
        dfb_bcmnexus = SHCALLOC( pool, 1, sizeof(DFBBCMNEXUS) );
        if (!dfb_bcmnexus) {
                return D_OOSHM();
        }

    /* Save the per-process core handle */
    dfb_bcmnexus_core = core;

    SET_DISPLAY_HEAP_HND(dfb_bcmnexus, GET_NEXUS_GRAPHICS_DISPLAY_HEAP_HND());
    SET_OFFSCREEN_HEAP_HND(dfb_bcmnexus, GET_NEXUS_GRAPHICS_OFFSCREEN_HEAP_HND());
    SET_PALETTE_HEAP_HND(dfb_bcmnexus, GET_NEXUS_GRAPHICS_PALETTE_HEAP_HND());
    SET_VIDEO_HEAP_HND(dfb_bcmnexus, GET_NEXUS_VIDEO_HEAP_HND());
#if NEXUS_HAS_GRAPHICS2D || NEXUS_HAS_ZSP_GRAPHICS
        SET_GRAPHICS2D_HND(dfb_bcmnexus, GET_NEXUS_GRAPHICS2D_HND());
#endif
    SET_DISPLAY_HND(dfb_bcmnexus, GET_NEXUS_DISPLAY_HND(0), PRIMARY_DISPLAY_ID);
#if (NUM_DISPLAYS > 1)
    SET_DISPLAY_HND(dfb_bcmnexus, GET_NEXUS_DISPLAY_HND(1), SECONDARY_DISPLAY_ID);
#endif
    SET_DISPLAY_HND(dfb_bcmnexus, GET_NEXUS_DISPLAY_HND(0), TERTIARY_DISPLAY_ID);

        /* register screens and layers */
        register_screens_layers( dfb_bcmnexus );

#if NEXUS_HAS_PICTURE_DECODER
    fusion_skirmish_init( &GET_PICTURE_DECODER_LOCK(dfb_bcmnexus), "BRCM Picture Decoder Lock", dfb_core_world(NULL) );
    if (GET_NEXUS_PICTURE_DECODER_HND() == NULL)
        dfb_config->sw_picture_decode = true;
    else
        SET_PICTURE_DECODER_HND(dfb_bcmnexus, GET_NEXUS_PICTURE_DECODER_HND());
#endif

        fusion_arena_add_shared_field( dfb_core_arena( core ), "bcmnexus", dfb_bcmnexus );

        /* make data accessible via dfb_system_data(); */
        /* must be set before initializing the pools! */
        *data = dfb_bcmnexus;

        dfb_surface_pool_initialize( core, &bcmnexusPoolFunctions, &GET_POOL_HND(dfb_bcmnexus) );

    /* Initialise the system layer call handler */
    fusion_call_init( &GET_CALL_HANDLER(dfb_bcmnexus), bcmCallHandler, NULL, dfb_core_world(core) );

    /* Initialise Primary Screen VSYNC callbacks */
    fusion_skirmish_init( &GET_DISPLAY_VSYNC_MUTEX(dfb_bcmnexus, PRIMARY_DISPLAY_ID), "Broadcom Primary Screen Semaphore", dfb_core_world(NULL) );
    SET_DISPLAY_VSYNC_CB(dfb_bcmnexus, NULL, PRIMARY_DISPLAY_ID);
#if NEXUS_NUM_COMPONENT_OUTPUTS && !defined(NEXUS_DTV_PLATFORM)
    SET_DISPLAY_COMPONENT_OUT(dfb_bcmnexus, NEXUS_IsComponentOutputAttached(PRIMARY_DISPLAY_ID), PRIMARY_DISPLAY_ID);
#endif

#if (NUM_DISPLAYS > 1)
    /* Initialise Secondary Screen VSYNC callbacks */
    fusion_skirmish_init( &GET_DISPLAY_VSYNC_MUTEX(dfb_bcmnexus, SECONDARY_DISPLAY_ID), "Broadcom Secondary Screen Semaphore", dfb_core_world(NULL) );
    SET_DISPLAY_VSYNC_CB(dfb_bcmnexus, NULL, SECONDARY_DISPLAY_ID);
#if NEXUS_NUM_COMPONENT_OUTPUTS && !defined(NEXUS_DTV_PLATFORM)
    SET_DISPLAY_COMPONENT_OUT(dfb_bcmnexus, NEXUS_IsComponentOutputAttached(SECONDARY_DISPLAY_ID), SECONDARY_DISPLAY_ID);
#endif
#endif

#if NEXUS_NUM_COMPONENT_OUTPUTS && !defined(NEXUS_DTV_PLATFORM)
    NEXUS_Register_ComponentOutputCallback(system_output_callback);
#endif

        return DFB_OK;
}


static DFBResult
system_join( CoreDFB *core, void **data )
{
    DFBResult         ret;
    DFBBCMNEXUS      *dfb_bcmnexus;
    void             *tmp;

    D_DEBUG_AT( bcmnexusSystem, "%s() core=%p\n", __FUNCTION__, (void *)core );

    /* Save the per-process core handle */
    dfb_bcmnexus_core = core;

    ret = fusion_arena_get_shared_field( dfb_core_arena( core ), "bcmnexus", &tmp );
    if (ret) {
        return ret;
    }

    dfb_bcmnexus = tmp;

    /* register screens and layers */
    register_screens_layers( dfb_bcmnexus );

    /* set this first before pool join */
    *data = dfb_bcmnexus;

    if (GET_POOL_HND(dfb_bcmnexus))
        dfb_surface_pool_join( core, GET_POOL_HND(dfb_bcmnexus), &bcmnexusPoolFunctions );

    return DFB_OK;
}

static DFBResult
system_shutdown( UNUSED_ bool emergency )
{
        DFBBCMNEXUS *dfb_bcmnexus = (DFBBCMNEXUS *) dfb_system_data();

        FusionSHMPoolShared *pool;

        D_DEBUG_AT( bcmnexusSystem, "%s()\n", __FUNCTION__ );

        D_ASSERT( dfb_bcmnexus != NULL );

#if NEXUS_NUM_COMPONENT_OUTPUTS && !defined(NEXUS_DTV_PLATFORM)
    NEXUS_Unregister_ComponentOutputCallback();
#endif

    fusion_call_destroy( &GET_CALL_HANDLER(dfb_bcmnexus) );

#if NEXUS_HAS_PICTURE_DECODER
    fusion_skirmish_destroy( &GET_PICTURE_DECODER_LOCK(dfb_bcmnexus) );
#endif

    /* Unregister screens */
    unregister_screens_layers(dfb_bcmnexus);

    /* Destroy the surface pool used by Nexus */
    dfb_surface_pool_destroy( GET_POOL_HND(dfb_bcmnexus) );

        pool = dfb_core_shmpool( dfb_bcmnexus_core );
        SHFREE( pool, dfb_bcmnexus );

    dfb_bcmnexus_core = NULL;
        return DFB_OK;
}

static DFBResult
system_leave( UNUSED_ bool emergency )
{
        D_DEBUG_AT( bcmnexusSystem, "%s()\n", __FUNCTION__ );
        return DFB_OK;
}

static DFBResult
system_suspend()
{
        return DFB_UNIMPLEMENTED;
}

static DFBResult
system_resume()
{
        return DFB_UNIMPLEMENTED;
}

/******************************************************************************/

static volatile void *
system_map_mmio( UNUSED_ unsigned int    offset,
                 UNUSED_ int             length )
{
        return NULL;
}

static void
system_unmap_mmio( UNUSED_ volatile void  *addr,
                   UNUSED_ int             length )
{
        return;
}

static int
system_get_accelerator()
{
        return -1;
}

static VideoMode *
system_get_modes()
{
        /* TODO: Generate and return a list of all video modes
         * possible.  The application will select amongst the mode
         * returned to set the display resolution, etc. */
        return NULL;
}

static VideoMode *
system_get_current_mode()
{
        return NULL;
}

static DFBResult
system_thread_init()
{
        return DFB_UNIMPLEMENTED;
}

static bool
system_input_filter( UNUSED_ CoreInputDevice *device,
                     UNUSED_ DFBInputEvent   *event )
{
        return false;
}

static unsigned long
system_video_memory_physical( UNUSED_ unsigned int offset )
{
        return 0;
}

static void *
system_video_memory_virtual( unsigned int offset )
{
        (void)offset;

        /* "unused" in DirectFB 1.3.x */
        return 0;
}

static unsigned int
system_videoram_length()
{
    unsigned int size = 0;
#ifdef NEXUS_MAX_HEAPS
    NEXUS_HeapHandle heap;
    NEXUS_MemoryStatus status;
    NEXUS_PlatformConfiguration platformConfig;
        DFBBCMNEXUS *dfb_bcmnexus;

    dfb_bcmnexus = (DFBBCMNEXUS *) dfb_system_data();

    NEXUS_Platform_GetConfiguration(&platformConfig);

    heap = GET_DISPLAY_HEAP_HND(dfb_bcmnexus);

    if (NEXUS_Heap_GetStatus(heap, &status) == NEXUS_SUCCESS)
    {
        size = status.size;
    }
#endif
        return size;
}

static unsigned long
system_aux_memory_physical( UNUSED_ unsigned int offset )
{
     return 0;
}

static void *
system_aux_memory_virtual( UNUSED_ unsigned int offset )
{
     return NULL;
}

static unsigned int
system_auxram_length()
{
     return 0;
}

static void
system_get_busid( int *ret_bus,
                  int *ret_dev,
                  int *ret_func )
{
        *ret_bus  = 0;
        *ret_dev  = 0;
        *ret_func = 0;
}

static void
system_get_deviceid( unsigned int *ret_vendor_id,
                     unsigned int *ret_device_id )
{
        *ret_vendor_id = 0;
        *ret_device_id = 0;
}
