/***************************************************************************
 *    (c)2005-2011 Broadcom Corporation
 * 
 * This program is the proprietary software of Broadcom Corporation and/or its licensors,
 * and may only be used, duplicated, modified or distributed pursuant to the terms and
 * conditions of a separate, written license agreement executed between you and Broadcom
 * (an "Authorized License").  Except as set forth in an Authorized License, Broadcom grants
 * no license (express or implied), right to use, or waiver of any kind with respect to the
 * Software, and Broadcom expressly reserves all rights in and to the Software and all
 * intellectual property rights therein.  IF YOU HAVE NO AUTHORIZED LICENSE, THEN YOU
 * HAVE NO RIGHT TO USE THIS SOFTWARE IN ANY WAY, AND SHOULD IMMEDIATELY
 * NOTIFY BROADCOM AND DISCONTINUE ALL USE OF THE SOFTWARE.  
 *  
 * Except as expressly set forth in the Authorized License,
 *  
 * 1.     This program, including its structure, sequence and organization, constitutes the valuable trade
 * secrets of Broadcom, and you shall use all reasonable efforts to protect the confidentiality thereof,
 * and to use this information only in connection with your use of Broadcom integrated circuit products.
 *  
 * 2.     TO THE MAXIMUM EXTENT PERMITTED BY LAW, THE SOFTWARE IS PROVIDED "AS IS" 
 * AND WITH ALL FAULTS AND BROADCOM MAKES NO PROMISES, REPRESENTATIONS OR 
 * WARRANTIES, EITHER EXPRESS, IMPLIED, STATUTORY, OR OTHERWISE, WITH RESPECT TO 
 * THE SOFTWARE.  BROADCOM SPECIFICALLY DISCLAIMS ANY AND ALL IMPLIED WARRANTIES 
 * OF TITLE, MERCHANTABILITY, NONINFRINGEMENT, FITNESS FOR A PARTICULAR PURPOSE, 
 * LACK OF VIRUSES, ACCURACY OR COMPLETENESS, QUIET ENJOYMENT, QUIET POSSESSION 
 * OR CORRESPONDENCE TO DESCRIPTION. YOU ASSUME THE ENTIRE RISK ARISING OUT OF 
 * USE OR PERFORMANCE OF THE SOFTWARE.
 * 
 * 3.     TO THE MAXIMUM EXTENT PERMITTED BY LAW, IN NO EVENT SHALL BROADCOM OR ITS 
 * LICENSORS BE LIABLE FOR (i) CONSEQUENTIAL, INCIDENTAL, SPECIAL, INDIRECT, OR 
 * EXEMPLARY DAMAGES WHATSOEVER ARISING OUT OF OR IN ANY WAY RELATING TO YOUR 
 * USE OF OR INABILITY TO USE THE SOFTWARE EVEN IF BROADCOM HAS BEEN ADVISED OF 
 * THE POSSIBILITY OF SUCH DAMAGES; OR (ii) ANY AMOUNT IN EXCESS OF THE AMOUNT 
 * ACTUALLY PAID FOR THE SOFTWARE ITSELF OR U.S. $1, WHICHEVER IS GREATER. THESE 
 * LIMITATIONS SHALL APPLY NOTWITHSTANDING ANY FAILURE OF ESSENTIAL PURPOSE OF 
 * ANY LIMITED REMEDY.
 *
 * $brcm_Workfile: bcmnexus.h $
 * $brcm_Revision: DirectFB_1_4_Port/25 $
 * $brcm_Date: 3/17/11 12:42p $
 * 
 * Module Description:
 * 
 * Revision History:
 * 
 * $brcm_Log: /AppLibs/opensource/directfb/src/broadcom_files/private/DirectFB/1.4/systems/bcmnexus/bcmnexus.h $
 * 
 * DirectFB_1_4_Port/25   3/17/11 12:42p robertwm
 * SW7422-378: DirectFB: Destroying a Palettised surface in a client
 *  application causes NEXUS assertion during free.
 * SW7422-383: DirectFB: Need to serialise access to SID to prevent
 *  assertions.
 * 
 * DirectFB_1_4_Port/24   3/1/11 8:49p robertwm
 * SW7420-1559: DirectFB: NEXUS surface assertion in multi-app mode when
 *  closing down a DFB client.
 * 
 * DirectFB_1_4_Port/23   2/8/11 12:44p robertwm
 * SW7420-1467: DirectFB: Multiple apps should NOT share NEXUS Display
 *  handle.
 * 
 * DirectFB_1_4_Port/22   2/3/11 6:04p christ
 * SW7420-1458: DirectFB: Rework internal handling of nexus heaps.
 * 
 * DirectFB_1_4_Port/20   1/19/11 12:25p scchang
 * SW7420-1394: video layer support in DFB
 * 
 * DirectFB_1_4_Port/19   12/3/10 8:39p raywu
 * SW7550-629: DirectFB: Integrated ZSP code to DFB packet buffer branch
 * 
 * DirectFB_1_4_Port/18   12/1/10 3:47p christ
 * SW7420-1283: DirectFB: Add support for extra Nexus heap on dual MEMC
 *  parts
 * 
 * DirectFB_1_4_Port/17   8/20/10 3:38p robertwm
 * SW7405-4791: DirectFB: DirectFB-1.4.1 Phase 2.5 fails to build with
 *  Nexus Phase 5.0 reference software.
 * 
 * DirectFB_1_4_Port/16   8/11/10 2:01a robertwm
 * SW7405-4250: DirectFB: Screen destination rectangle and source
 *  rectangle not saved across contexts.
 * SW7405-4743: DirectFB: Multiple instances of df_andi.c do not get
 *  displayed.
 * 
 * DirectFB_1_4_Port/15   7/22/10 4:17p robertwm
 * SW7405-4649: DirectFB: Optimise surface locking and wait idle syncing.
 * 
 * DirectFB_1_4_Port/14   4/15/10 10:37p robertwm
 * SW7405-4214: DirectFB: Multi-Process DirectFB fails to run.
 * 
 * DirectFB_1_4_Port/13   4/9/10 4:24p robertwm
 * SW7405-4180: DirectFB: Require independent setting of graphics
 *  resolution on multiple output paths.
 * 
 * DirectFB_1_4_Port/12   3/2/10 10:41a robertwm
 * SW3556-1069: DirectFB: Run-time dynamic linking between gfxdriver and
 *  system drivers not working.
 * 
 * DirectFB_1_4_Port/11   1/25/10 10:23a robertwm
 * SW7405-3827: DirectFB: system module not including
 *  bcmnexus_gfx_defines.h.
 * 
 * DirectFB_1_4_Port/10   1/18/10 8:10p robertwm
 * SW7420-551: DirectFB: Add support for Screen "SetPowerMode()" API Call.
 * 
 * DirectFB_1_4_Port/9   1/14/10 6:53p robertwm
 * SW3556-996: DirectFB: Implement ImageProvider for DirectFB-1.4.1 using
 *  SID.
 * 
 * DirectFB_1_4_Port/8   1/4/10 5:59p robertwm
 * SW7420-523: DirectFB: Require 1080p resolution support.
 * SW7405-3701: DirectFB: Different video and graphics resolutions.
 * 
 * DirectFB_1_4_Port/7   12/23/09 2:22p robertwm
 * SW7550-51: Support DirectFB on 7550.
 * 
 * DirectFB_1_4_Port/SW7550-51/1   12/22/09 3:22p mphillip
 * SW7550-51: Add 7550 build support (on branch)
 * 
 * DirectFB_1_4_Port/6   12/16/09 9:45p robertwm
 * SW7400-2643: DirectFB: Add support for setting/getting video modes.
 * 
 * DirectFB_1_4_Port/5   11/16/09 1:27p robertwm
 * SW7405-3399: WaitForSync() function is unimplemented.
 * (Fixed graphics freeze issue).
 * 
 * DirectFB_1_4_Port/4   11/13/09 7:10p robertwm
 * SW7405-3399: WaitForSync() function is unimplemented.
 * 
 * DirectFB_1_4_Port/3   8/27/09 4:36p robertwm
 * SW3548-2427: DirectFB: Support 3549 panel types and fix compilation
 *  bugs.
 * 
 * DirectFB_1_4_Port/2   8/21/09 11:12a robertwm
 * PR57917: DirectFB: testman and testrun don't run anymore in 1.4.1
 *  (SaWMan)
 * 
 * DirectFB_1_4_Port/1   7/8/09 3:19p mphillip
 * PR56658: Add DirectFB 1.4.1 broadcom_files
 * 
 * 9   11/3/08 1:57p gmohile
 * PR 45243 : Default all DTV to panel output
 * 
 * 8   10/24/08 12:42p gmohile
 * PR 46490 : Add hw picture decode support
 * 
 * 7   10/14/08 3:53p gmohile
 * PR 45243 : Fix nexus handles
 * 
 * 6   9/15/08 11:49a gmohile
 * PR 46734 : Add dual output support
 * 
 * 5   8/29/08 2:34p gmohile
 * PR 45243 : Do nexus platform init and display open outside directfb
 * 
 * 4   8/27/08 4:26p gmohile
 * PR 45561 : Remove surface LUT
 * 
 * 3   8/21/08 1:49p gmohile
 * PR 25109 : Fix compile error
 * 
 * 2   7/31/08 3:30p ttrammel
 * PR44951: Update DirectFB for 2.6.18-5.0 and 93549
 *
 * $bcmnexus_Workfile: bcmnexus.h $
 * $bcmnexus_Revision: $
 * $bcmnexus_Date: $
 *
 * [File Description:]
 *
 * Revision History:
 *
 ***************************************************************************/

#include <dfb_types.h>
#include "platform_init.h"
#include "bcmnexus_gfx_defines.h"
#include "bcmnexus_sys_defines.h"
#include "nexus_surface.h"
#ifdef NEXUS_HAS_PICTURE_DECODER
#include "nexus_picture_decoder.h"
#endif
#include "nexus_core_utils.h"
#include <core/surface_pool.h>
#include <gfx/convert.h>

#ifndef __CORE__BCMNEXUS_H__
#define __CORE__BCMNEXUS_H__

/* Macros to get/set fields in the shared DFBBCMNEXUS structure */
#define SET_DISPLAY_GFX_LAYER_ID(   BCMNEXUS_PTR, val, idx, id) (BCMNEXUS_PTR->displays[id].gfxLayerIds[idx] = val)
#define SET_DISPLAY_VID_LAYER_ID(   BCMNEXUS_PTR, val, idx, id) (BCMNEXUS_PTR->displays[id].vidLayerIds[idx] = val)
#define SET_DISPLAY_HND(            BCMNEXUS_PTR, val, id)      (BCMNEXUS_PTR->displays[id].hDisplay = val)
#define SET_DISPLAY_SIZE(           BCMNEXUS_PTR, val, id)      (BCMNEXUS_PTR->displays[id].size = val)
#define SET_DISPLAY_WIDTH(          BCMNEXUS_PTR, val, id)      (BCMNEXUS_PTR->displays[id].size.w = val)
#define SET_DISPLAY_HEIGHT(         BCMNEXUS_PTR, val, id)      (BCMNEXUS_PTR->displays[id].size.h = val)
#define SET_DISPLAY_ASPECT(         BCMNEXUS_PTR, val, id)      (BCMNEXUS_PTR->displays[id].aspectRatio = val)
#define SET_DISPLAY_INTERLACED(     BCMNEXUS_PTR, val, id)      (BCMNEXUS_PTR->displays[id].interlaced = val)
#define SET_DISPLAY_VSYNC_CB(       BCMNEXUS_PTR, val, id)      (BCMNEXUS_PTR->displays[id].vsyncCallback = val)
#define SET_DISPLAY_COMPONENT_OUT(  BCMNEXUS_PTR, val, id)      (BCMNEXUS_PTR->displays[id].componentOut = val)

#define GET_DISPLAY_GFX_LAYER_ID(   BCMNEXUS_PTR, idx, id)      (BCMNEXUS_PTR->displays[id].gfxLayerIds[idx])
#define GET_DISPLAY_VID_LAYER_ID(   BCMNEXUS_PTR, idx, id)      (BCMNEXUS_PTR->displays[id].vidLayerIds[idx])
#define GET_DISPLAY_HND(            BCMNEXUS_PTR, id)           (BCMNEXUS_PTR->displays[id].hDisplay)
#define GET_DISPLAY_SIZE(           BCMNEXUS_PTR, id)           (BCMNEXUS_PTR->displays[id].size)
#define GET_DISPLAY_WIDTH(          BCMNEXUS_PTR, id)           (BCMNEXUS_PTR->displays[id].size.w)
#define GET_DISPLAY_HEIGHT(         BCMNEXUS_PTR, id)           (BCMNEXUS_PTR->displays[id].size.h)
#define GET_DISPLAY_ASPECT(         BCMNEXUS_PTR, id)           (BCMNEXUS_PTR->displays[id].aspectRatio)
#define GET_DISPLAY_INTERLACED(     BCMNEXUS_PTR, id)           (BCMNEXUS_PTR->displays[id].interlaced)
#define GET_DISPLAY_VSYNC_MUTEX(    BCMNEXUS_PTR, id)           (BCMNEXUS_PTR->displays[id].vsyncMutex)
#define GET_DISPLAY_VSYNC_CB(       BCMNEXUS_PTR, id)           (BCMNEXUS_PTR->displays[id].vsyncCallback)
#define GET_DISPLAY_COMPONENT_OUT(  BCMNEXUS_PTR, id)           (BCMNEXUS_PTR->displays[id].componentOut)

#if NEXUS_HAS_PICTURE_DECODER
#define SET_PICTURE_DECODER_HND(    BCMNEXUS_PTR, val)          (BCMNEXUS_PTR->pictureDecoder = val)
#define GET_PICTURE_DECODER_HND(    BCMNEXUS_PTR)               (BCMNEXUS_PTR->pictureDecoder)
#define SET_PICTURE_DECODER_LOCK(   BCMNEXUS_PTR, val)          (BCMNEXUS_PTR->pictureDecoderLock = val)
#define GET_PICTURE_DECODER_LOCK(   BCMNEXUS_PTR)               (BCMNEXUS_PTR->pictureDecoderLock)
#endif

#if NEXUS_HAS_GRAPHICS2D || NEXUS_HAS_ZSP_GRAPHICS
#define SET_GRAPHICS2D_HND(         BCMNEXUS_PTR, val)          (BCMNEXUS_PTR->hGfx2D = val)
#define SET_GRAPHICS2D_SYNC(        BCMNEXUS_PTR, val)          (BCMNEXUS_PTR->gfx2DSync = val)
#define GET_GRAPHICS2D_HND(         BCMNEXUS_PTR)               (BCMNEXUS_PTR->hGfx2D)
#define GET_GRAPHICS2D_SYNC(        BCMNEXUS_PTR)               (BCMNEXUS_PTR->gfx2DSync)
#endif

#define SET_POOL_HND(               BCMNEXUS_PTR, val)          (BCMNEXUS_PTR->pool = val)
#define GET_POOL_HND(               BCMNEXUS_PTR)               (BCMNEXUS_PTR->pool)

#define SET_DISPLAY_HEAP_HND(       BCMNEXUS_PTR, val)          (BCMNEXUS_PTR->displayHeap = val)
#define GET_DISPLAY_HEAP_HND(       BCMNEXUS_PTR)               (BCMNEXUS_PTR->displayHeap)

#define SET_OFFSCREEN_HEAP_HND(     BCMNEXUS_PTR, val)          (BCMNEXUS_PTR->offscreenHeap = val)
#define GET_OFFSCREEN_HEAP_HND(     BCMNEXUS_PTR)               (BCMNEXUS_PTR->offscreenHeap)

#define SET_PALETTE_HEAP_HND(       BCMNEXUS_PTR, val)          (BCMNEXUS_PTR->paletteHeap = val)
#define GET_PALETTE_HEAP_HND(       BCMNEXUS_PTR)               (BCMNEXUS_PTR->paletteHeap)

#define SET_CALL_HANDLER(           BCMNEXUS_PTR, val)          (BCMNEXUS_PTR->call = val)
#define GET_CALL_HANDLER(           BCMNEXUS_PTR)               (BCMNEXUS_PTR->call)
#define SET_VIDEO_HEAP_HND(         BCMNEXUS_PTR, val)          (BCMNEXUS_PTR->videoHeap = val)
#define GET_VIDEO_HEAP_HND(         BCMNEXUS_PTR)               (BCMNEXUS_PTR->videoHeap)

/*** Definitions ***/
#define PRIMARY_DISPLAY_ID          (DSCID_PRIMARY)
#define SECONDARY_DISPLAY_ID        (PRIMARY_DISPLAY_ID + 1)
#define TERTIARY_DISPLAY_ID         (SECONDARY_DISPLAY_ID + 1)

#define PRIMARY_GFX_LAYER_INDEX     (0)
#define SECONDARY_GFX_LAYER_INDEX   (PRIMARY_GFX_LAYER_INDEX + 1)

#define PRIMARY_VID_LAYER_INDEX     (0)
#define SECONDARY_VID_LAYER_INDEX   (PRIMARY_VID_LAYER_INDEX + 1)

typedef enum {
     BCM_POOL_MEMORY_ALLOCATE,
     BCM_POOL_MEMORY_FREE,
     BCM_POOL_SURFACE_CREATE,
     BCM_POOL_SURFACE_DESTROY,
     BCM_LAYER_SET_GFX_FRAMEBUFFER,
     BCM_LAYER_SET_GFX_SETTINGS,
     BCM_SCREEN_SET_DISPLAY_SETTINGS,
     BCM_SCREEN_ENABLE_COMPONENT_OUTPUT,
} eBcmCall;

/* These structures are shared amongst the different processes that interface with DFB */
typedef struct {
    DFBDisplayLayerID          gfxLayerIds[NUM_GFX_LAYERS_PER_DISPLAY];
    DFBDisplayLayerID          vidLayerIds[NUM_VID_LAYERS_PER_DISPLAY];
    NEXUS_DisplayHandle        hDisplay;
    DFBDimension               size;
    int                        aspectRatio;
    bool                       interlaced;
    FusionSkirmish             vsyncMutex;
    void                     (*vsyncCallback) (void *context, int param);
    bool                       componentOut;            /* true = component output connected */
} BCMNEXUS_Display;

typedef struct  {
    BCMNEXUS_Display           displays[NUM_DISPLAYS+1];/* Handles to primary, secondary displays and video layer display */
#if NEXUS_HAS_GRAPHICS2D || NEXUS_HAS_ZSP_GRAPHICS  
    NEXUS_Graphics2DHandle     hGfx2D;                  /* Graphics 2D/blitter handle */
    bool                       gfx2DSync;               /* Flag to indicate whether we need to issue checkpoint/sync */
#endif
#ifdef NEXUS_HAS_PICTURE_DECODER
    NEXUS_PictureDecoderHandle pictureDecoder;          /* Picture decoder handle*/
    FusionSkirmish             pictureDecoderLock;      /* Picture decoder global access lock */
#endif
    FusionCall                 call;
    CoreSurfacePool           *pool;                    /* 'virtual' pool, to link NEXUS <> DFB surfaces */
    NEXUS_HeapHandle           displayHeap;             /* Index of which Nexus Heap to use for Graphics Surfaces */
    NEXUS_HeapHandle           offscreenHeap;           /* Index of secondary Nexus Heap to use for Graphics Surfaces */
    NEXUS_HeapHandle           paletteHeap;             /* Index of  Nexus Heap to use for pallettes*/
    NEXUS_HeapHandle           videoHeap;               /* Index of  Nexus Heap to use for video based graphics surfaces*/
} DFBBCMNEXUS;

extern CoreDFB *dfb_bcmnexus_core;                      /* Local for each process that joins DirectFB */

#endif
