/***************************************************************************
 *     (c)2007-2011 Broadcom Corporation
 *
 *  This program is the proprietary software of Broadcom Corporation and/or its licensors,
 *  and may only be used, duplicated, modified or distributed pursuant to the terms and
 *  conditions of a separate, written license agreement executed between you and Broadcom
 *  (an "Authorized License").  Except as set forth in an Authorized License, Broadcom grants
 *  no license (express or implied), right to use, or waiver of any kind with respect to the
 *  Software, and Broadcom expressly reserves all rights in and to the Software and all
 *  intellectual property rights therein.  IF YOU HAVE NO AUTHORIZED LICENSE, THEN YOU
 *  HAVE NO RIGHT TO USE THIS SOFTWARE IN ANY WAY, AND SHOULD IMMEDIATELY
 *  NOTIFY BROADCOM AND DISCONTINUE ALL USE OF THE SOFTWARE.
 *
 *  Except as expressly set forth in the Authorized License,
 *
 *  1.     This program, including its structure, sequence and organization, constitutes the valuable trade
 *  secrets of Broadcom, and you shall use all reasonable efforts to protect the confidentiality thereof,
 *  and to use this information only in connection with your use of Broadcom integrated circuit products.
 *
 *  2.     TO THE MAXIMUM EXTENT PERMITTED BY LAW, THE SOFTWARE IS PROVIDED "AS IS"
 *  AND WITH ALL FAULTS AND BROADCOM MAKES NO PROMISES, REPRESENTATIONS OR
 *  WARRANTIES, EITHER EXPRESS, IMPLIED, STATUTORY, OR OTHERWISE, WITH RESPECT TO
 *  THE SOFTWARE.  BROADCOM SPECIFICALLY DISCLAIMS ANY AND ALL IMPLIED WARRANTIES
 *  OF TITLE, MERCHANTABILITY, NONINFRINGEMENT, FITNESS FOR A PARTICULAR PURPOSE,
 *  LACK OF VIRUSES, ACCURACY OR COMPLETENESS, QUIET ENJOYMENT, QUIET POSSESSION
 *  OR CORRESPONDENCE TO DESCRIPTION. YOU ASSUME THE ENTIRE RISK ARISING OUT OF
 *  USE OR PERFORMANCE OF THE SOFTWARE.
 *
 *  3.     TO THE MAXIMUM EXTENT PERMITTED BY LAW, IN NO EVENT SHALL BROADCOM OR ITS
 *  LICENSORS BE LIABLE FOR (i) CONSEQUENTIAL, INCIDENTAL, SPECIAL, INDIRECT, OR
 *  EXEMPLARY DAMAGES WHATSOEVER ARISING OUT OF OR IN ANY WAY RELATING TO YOUR
 *  USE OF OR INABILITY TO USE THE SOFTWARE EVEN IF BROADCOM HAS BEEN ADVISED OF
 *  THE POSSIBILITY OF SUCH DAMAGES; OR (ii) ANY AMOUNT IN EXCESS OF THE AMOUNT
 *  ACTUALLY PAID FOR THE SOFTWARE ITSELF OR U.S. $1, WHICHEVER IS GREATER. THESE
 *  LIMITATIONS SHALL APPLY NOTWITHSTANDING ANY FAILURE OF ESSENTIAL PURPOSE OF
 *  ANY LIMITED REMEDY.
 *
 * $brcm_Workfile: bcmnexus_screen.c $
 * $brcm_Revision: DirectFB_1_4_Port/25 $
 * $brcm_Date: 3/15/11 10:58a $
 *
 * Module Description:
 *
 * Revision History:
 *
 *
 * $brcm_Log: /AppLibs/opensource/directfb/src/broadcom_files/private/DirectFB/1.4/systems/bcmnexus/bcmnexus_screen.c $
 * 
 * DirectFB_1_4_Port/25   3/15/11 10:58a robertwm
 * SW3556-1226: DirectFB: Cannot run DFB on 3556 due to VDB errors.
 * 
 * DirectFB_1_4_Port/24   3/4/11 12:17p christ
 * SW7405-5174: DirectFB: Setting 1080p50/60 modes on SOCs which don't
 *  support these modes crashes Magnum.
 *
 * DirectFB_1_4_Port/23   3/1/11 12:01p christ
 * SW7420-1545: DirectFB: Application hang when using multi app DirectFB
 *  with SawMan window manager.
 *
 * DirectFB_1_4_Port/22   2/18/11 4:30p robertwm
 * SW7468-366: DirectFB: Platforms without component output should not
 *  call component functions.
 *
 * DirectFB_1_4_Port/21   2/8/11 1:25p robertwm
 * SW7420-1467: DirectFB: Multiple apps should NOT share NEXUS Display
 *  handle.
 * SW7420-1469: DirectFB: Should check return code when trying to add an
 *  output to the display.
 *
 * DirectFB_1_4_Port/20   1/31/11 5:43p robertwm
 * SW7420-1448: DirectFB: bcmSetEncoderConfig() should not enable graphics
 *  if layer has not already been initialised.
 *
 * DirectFB_1_4_Port/19   1/28/11 5:20p robertwm
 * SW7420-1443: DirectFB: Add support for changing display/screen
 *  frequency using ScreenEncoder.
 *
 * DirectFB_1_4_Port/18   1/24/11 4:27p christ
 * SW7420-1411: DirectFB: Merge of features required for DirectFB 1.4.5
 *  phase 1.5 release - Add some encoder support
 *
 * DirectFB_1_4_Port/17   8/20/10 3:40p robertwm
 * SW7405-4791: DirectFB: DirectFB-1.4.1 Phase 2.5 fails to build with
 *  Nexus Phase 5.0 reference software.
 *
 * DirectFB_1_4_Port/16   8/11/10 1:59a robertwm
 * SW7405-4250: DirectFB: Screen destination rectangle and source
 *  rectangle not saved across contexts.
 * SW7405-4743: DirectFB: Multiple instances of df_andi.c do not get
 *  displayed.
 *
 * DirectFB_1_4_Port/15   7/26/10 5:32p jackf
 * SW35230-771: Added include for shim_nexus.h.
 *
 * DirectFB_1_4_Port/14   5/5/10 12:49p robertwm
 * SW7405-4295: DirectFB: Reference to NEXUS_NUM_HDMI_OUTPUTS should check
 *  if > 0.
 *
 * DirectFB_1_4_Port/13   4/19/10 4:47p robertwm
 * SW3556-1100: DirectFB: Remove warnings generated when NUM_DISPLAYS is
 *  1.
 *
 * DirectFB_1_4_Port/12   4/9/10 4:54p robertwm
 * SW7405-4180: DirectFB: Require independent setting of graphics
 *  resolution on multiple output paths.
 * SW7405-4184: Allow independent mode and resolution to be specified.
 *
 * DirectFB_1_4_Port/11   2/25/10 11:50a robertwm
 * SW7405-3961: DirectFB: Add support for 576p and 480p video display
 *  modes.
 *
 * DirectFB_1_4_Port/10   1/25/10 10:25a robertwm
 * SW7405-3827: DirectFB: system module not including
 *  bcmnexus_gfx_defines.h.
 *
 * DirectFB_1_4_Port/9   1/18/10 8:11p robertwm
 * SW7420-551: DirectFB: Add support for Screen "SetPowerMode()" API Call.
 *
 * DirectFB_1_4_Port/8   1/4/10 6:00p robertwm
 * SW7420-523: DirectFB: Require 1080p resolution support.
 *
 * DirectFB_1_4_Port/7   12/23/09 2:23p robertwm
 * SW7550-51: Support DirectFB on 7550.
 *
 * DirectFB_1_4_Port/SW7550-51/1   12/22/09 3:22p mphillip
 * SW7550-51: Add 7550 build support (on branch)
 *
 * DirectFB_1_4_Port/6   12/16/09 9:50p robertwm
 * SW7400-2643: DirectFB: Add support for setting/getting video modes.
 *
 * DirectFB_1_4_Port/5   11/16/09 1:27p robertwm
 * SW7405-3399: WaitForSync() function is unimplemented.
 * (Fixed graphics freeze issue).
 *
 * DirectFB_1_4_Port/4   11/13/09 7:10p robertwm
 * SW7405-3399: WaitForSync() function is unimplemented.
 *
 * DirectFB_1_4_Port/3   8/7/09 3:46p robertwm
 * PR56721: Add HDMI output support.
 *
 * DirectFB_1_4_Port/2   7/9/09 7:45p robertwm
 * PR56691: Require ability to set European display output modes (e.g.
 *  576i, 720p/50, 1080i/25).
 *
 * DirectFB_1_4_Port/1   7/8/09 3:19p mphillip
 * PR56658: Add DirectFB 1.4.1 broadcom_files
 *
 * 8   11/21/08 8:06p vsilyaev
 * PR 49524: Added graphics2d_sync routine that could be used by different
 *  users of Nexus_Graphics2D
 *
 * 7   11/3/08 1:57p gmohile
 * PR 45243 : Default all DTV to panel output
 *
 * ***************************************************************************/

#include "shim_nexus.h" // Nexus function redirection

#include <unistd.h>
#include <string.h>
#include <stdlib.h>
#include <core/layers.h>
#include <core/layer_control.h>
#include <core/screens_internal.h>
#include <fusion/shmalloc.h>
#include "bcmnexus_screen.h"
#include "bcmnexus_layer.h"
#include <directfb_strings.h>

#include "bstd.h"
#include <misc/conf.h>

#ifdef __GNUC__
#define UNUSED_ __attribute__ ((unused))
#else
#define UNUSED_
#endif

D_DEBUG_DOMAIN( bcmnexusScreen, "bcmNexus/Screen",   "Broadcom NEXUS Screen driver" );
D_DEBUG_DOMAIN( bcmnexusScreenX, "bcmNexus/ScreenX", "Broadcom NEXUS Screen Xtra debug" );

/* Strings for Screen are defined here */
const DirectFBScreenOutputConnectorsNames  ( dfb_screen_output_connectors_names    )
const DirectFBScreenOutputResolutionNames  ( dfb_screen_output_resolution_names    )
const DirectFBScreenEncoderTVStandardsNames( dfb_screen_encoder_tv_standards_names )
const DirectFBScreenEncoderScanModeNames   ( dfb_screen_encoder_scan_mode_names    )
const DirectFBScreenEncoderFrequencyNames  ( dfb_screen_encoder_frequency_names    )

typedef struct {
    DFBScreenID        id;
    DFBScreenPowerMode powerMode;
    bool               fixedResolution;
} bcmScreenData;

static int bcmScreenDataSize(void)
{
    return sizeof(bcmScreenData);
}

#ifdef BCMNEXUS_DISPLAY_HAS_VSYNC_CALLBACK
static void bcmVsyncCallback(void *context, int param)
{
    FusionSkirmish *mutex = context;

    D_DEBUG_AT( bcmnexusScreenX, "  -> VSync[%d] callback handled\n", param );

    fusion_skirmish_prevail( mutex );
    fusion_skirmish_notify( mutex );
    fusion_skirmish_dismiss( mutex );
}

static void bcmInstallVSyncCallback(DFBBCMNEXUS           *dfb_bcmnexus,
                                    DFBScreenID            screen,
                                    NEXUS_DisplaySettings *pDisplayCfg)
{
    D_DEBUG_AT( bcmnexusScreen, "%s() screen %d\n", __FUNCTION__, screen );

    if (GET_DISPLAY_VSYNC_CB(dfb_bcmnexus, screen) == NULL)
    {
        SET_DISPLAY_VSYNC_CB(dfb_bcmnexus, bcmVsyncCallback, screen);
        pDisplayCfg->vsyncCallback.callback = GET_DISPLAY_VSYNC_CB(dfb_bcmnexus, screen);
        pDisplayCfg->vsyncCallback.context  = &GET_DISPLAY_VSYNC_MUTEX(dfb_bcmnexus, screen);
        pDisplayCfg->vsyncCallback.param    = screen;
    }
}
#endif

static NEXUS_VideoFormat getNexusVideoFormat(void)
{
    NEXUS_VideoFormat format;

    format = GET_NEXUS_VIDEO_FORMAT_PREFERRED();

    if (strcmp(dfb_config->res,"1080p") == 0 || strcmp(dfb_config->res,"1080p60") == 0) {
#if BCMNEXUS_VEC_HAS_1080P_60HZ_SUPPORT
        format = NEXUS_VideoFormat_e1080p;
#else
        D_WARN("%s: Cannot support 1080p/60Hz Output.\n", __FUNCTION__);
#endif
    }
    else if (strcmp(dfb_config->res,"1080p50") == 0) {
#if BCMNEXUS_VEC_HAS_1080P_60HZ_SUPPORT
        format = NEXUS_VideoFormat_e1080p50hz;
#else
        D_WARN("%s: Cannot support 1080p/50Hz Output.\n", __FUNCTION__);
#endif
    }
    else if (strcmp(dfb_config->res,"1080p30") == 0) {
        format = NEXUS_VideoFormat_e1080p30hz;
    }
    else if (strcmp(dfb_config->res,"1080p25") == 0) {
        format = NEXUS_VideoFormat_e1080p25hz;
    }
    else if (strcmp(dfb_config->res,"1080p24") == 0) {
        format = NEXUS_VideoFormat_e1080p24hz;
    }
    else if (strcmp(dfb_config->res,"1080i") == 0 || strcmp(dfb_config->res,"1080i30") == 0) {
        format = NEXUS_VideoFormat_e1080i;
    }
    else if(strcmp(dfb_config->res,"1080i25") == 0) {
        format = NEXUS_VideoFormat_e1080i50hz;
    }
    else if (strcmp(dfb_config->res,"720p") == 0 || strcmp(dfb_config->res,"720p60") == 0) {
        format = NEXUS_VideoFormat_e720p;
    }
    else if (strcmp(dfb_config->res,"720p50") == 0) {
        format = NEXUS_VideoFormat_e720p50hz;
    }
    else if (strcmp(dfb_config->res,"720p30") == 0) {
        format = NEXUS_VideoFormat_e720p30hz;
    }
    else if (strcmp(dfb_config->res,"720p25") == 0) {
        format = NEXUS_VideoFormat_e720p25hz;
    }
    else if (strcmp(dfb_config->res,"720p24") == 0) {
        format = NEXUS_VideoFormat_e720p24hz;
    }
    else if (strcmp(dfb_config->res,"576p") == 0) {
        format = NEXUS_VideoFormat_e576p;
    }
    else if (strcmp(dfb_config->res,"576i") == 0) {
        format = NEXUS_VideoFormat_ePal;
    }
    else if (strcmp(dfb_config->res,"480p") == 0) {
        format = NEXUS_VideoFormat_e480p;
    }
    else if (strcmp(dfb_config->res,"480i") == 0) {
        format = NEXUS_VideoFormat_eNtsc;
    }
    else if (strcmp(dfb_config->res,"VGA") == 0) {
        format = NEXUS_VideoFormat_eVesa640x480p60hz;
    }
    else if (strcmp(dfb_config->res,"SVGA") == 0) {
        format = NEXUS_VideoFormat_eVesa800x600p60hz;
    }
    else if (strcmp(dfb_config->res,"XGA") == 0) {
        format = NEXUS_VideoFormat_eVesa1024x768p60hz;
    }
    return format;
}

static NEXUS_VideoFormat getNexusVideoFormatFromOutputRes(DFBScreenOutputResolution outRes, DFBScreenID screen)
{
    NEXUS_VideoFormat format;

    if (screen == PRIMARY_DISPLAY_ID)
    {
        switch (outRes)
        {
        case DSOR_640_480:
            format = NEXUS_VideoFormat_eVesa640x480p60hz;
            break;
        case DSOR_720_480:
            format = NEXUS_VideoFormat_eNtsc;
            break;
        case DSOR_720_576:
            format = NEXUS_VideoFormat_ePal;
            break;
        case DSOR_800_600:
            format = NEXUS_VideoFormat_eVesa800x600p60hz;
            break;
        case DSOR_1024_768:
            format = NEXUS_VideoFormat_eVesa1024x768p60hz;
            break;
        case DSOR_1152_864:
            format = NEXUS_VideoFormat_eVesa1152x864p75hz;
            break;
        case DSOR_1280_720:
            format = NEXUS_VideoFormat_e720p;
            break;
        case DSOR_1280_768:
            format = NEXUS_VideoFormat_eVesa1280x768p60hz;
            break;
        case DSOR_1280_960:
            format = NEXUS_VideoFormat_eVesa1280x960p60hz;
            break;
        case DSOR_1280_1024:
            format = NEXUS_VideoFormat_eVesa1280x1024p60hz;
            break;
        case DSOR_1400_1050:
            format = NEXUS_VideoFormat_eVesa1400x1050p60hz;
            break;
        case DSOR_1600_1200:
            format = NEXUS_VideoFormat_eVesa1600x1200p60hz;
            break;
        case DSOR_1920_1080:
            format = NEXUS_VideoFormat_e1080i;
            break;
        default:
            format = NEXUS_VideoFormat_eUnknown;
        }
    }
    else
    {
        switch (outRes)
        {
        case DSOR_720_480:
            format = NEXUS_VideoFormat_eNtsc;
            break;
        case DSOR_720_576:
            format = NEXUS_VideoFormat_ePal;
            break;
        default:
            format = NEXUS_VideoFormat_eUnknown;
        }
    }
    return format;
}

#if (NUM_DISPLAYS > 1)
static NEXUS_VideoFormat getVideoOutputFormat(DFBBCMNEXUS *dfb_bcmnexus, DFBScreenID screen)
{
    NEXUS_DisplaySettings displayCfg;

    NEXUS_Display_GetSettings(GET_DISPLAY_HND(dfb_bcmnexus, screen), &displayCfg);
    return displayCfg.format;
}
#endif

#if NEXUS_NUM_HDMI_OUTPUTS && NEXUS_NUM_COMPONENT_OUTPUTS && !defined(NEXUS_DTV_PLATFORM)
static NEXUS_Error bcmSetComponentOutput( DFBBCMNEXUS *dfb_bcmnexus,
                                   DFBScreenID  screen,
                                   int          output,
                                   bool         enable )
{
    NEXUS_Error res;
    NEXUS_DisplayHandle         display;
    NEXUS_PlatformConfiguration platformConfig;

    D_DEBUG_AT( bcmnexusScreen, "%s() screen %d, output=%d, enable=%d\n", __FUNCTION__, screen, output, enable );

    NEXUS_Platform_GetConfiguration(&platformConfig);

    display = GET_DISPLAY_HND(dfb_bcmnexus, screen);

    if (dfb_core_is_master( dfb_bcmnexus_core ))
    {
        if (enable)
        {
            res = NEXUS_Display_AddOutput(display, NEXUS_ComponentOutput_GetConnector(platformConfig.outputs.component[output]));
        } else {
            res = NEXUS_Display_RemoveOutput(display, NEXUS_ComponentOutput_GetConnector(platformConfig.outputs.component[output]));
        }
    }
    else
    {
        bcmScreenCallEnableComponentOutput *pCallArgs;

        pCallArgs = SHMALLOC( dfb_core_shmpool(dfb_bcmnexus_core), sizeof(*pCallArgs) );

        if (!pCallArgs)
            return D_OOSHM();

        pCallArgs->displayId = screen;
        pCallArgs->output    = output;
        pCallArgs->enable    = enable;

        fusion_call_execute( &GET_CALL_HANDLER(dfb_bcmnexus), FCEF_NONE, BCM_SCREEN_ENABLE_COMPONENT_OUTPUT, pCallArgs, (int *)&res );

        SHFREE( dfb_core_shmpool(dfb_bcmnexus_core), pCallArgs );
    }
    SET_DISPLAY_COMPONENT_OUT(dfb_bcmnexus, enable, screen);

    D_DEBUG_AT( bcmnexusScreen, "%s() screen %d, output=%d, enable=%d Exit[%d]\n", __FUNCTION__, screen, output, enable, res );

    return res;
}
#endif

static NEXUS_Error setVideoOutput(        DFBBCMNEXUS *dfb_bcmnexus,
                                          DFBScreenID screen,
                                  UNUSED_ int output,
                                          NEXUS_VideoFormat format )
{
    NEXUS_Error res = NEXUS_SUCCESS;
    NEXUS_DisplaySettings displayCfg;
    NEXUS_VideoFormatInfo info;

    D_DEBUG_AT( bcmnexusScreen, "%s() screen %d, output=%d, format=%d\n", __FUNCTION__, screen, output, format );

    NEXUS_Display_GetSettings(GET_DISPLAY_HND(dfb_bcmnexus, screen), &displayCfg);

#if (BCMNEXUS_VEC_HAS_1080P_60HZ_SUPPORT == 0)

    if ( (format == NEXUS_VideoFormat_e1080p) || (format == NEXUS_VideoFormat_e1080p60hz) )
    {
        D_ERROR("%s() Unsupported video_format %d - defaulting to 1080i60\n", __FUNCTION__, displayCfg.format);
        format = NEXUS_VideoFormat_e1080i;
    }
    else if ( format == NEXUS_VideoFormat_e1080p50hz)
    {
        D_ERROR("%s() Unsupported video_format %d - defaulting to 1080i50\n", __FUNCTION__, displayCfg.format);
        format = NEXUS_VideoFormat_e1080i50hz;
    }
#endif

    NEXUS_VideoFormat_GetInfo(format, &info);
    displayCfg.format = format;

#if NEXUS_NUM_HDMI_OUTPUTS && NEXUS_NUM_COMPONENT_OUTPUTS && !defined(NEXUS_DTV_PLATFORM)
    if (screen == PRIMARY_DISPLAY_ID)
    {
        if (GET_DISPLAY_COMPONENT_OUT(dfb_bcmnexus, screen)) {
            if (displayCfg.format >= NEXUS_VideoFormat_eVesa640x480p60hz)
            {
                res = bcmSetComponentOutput(dfb_bcmnexus, screen, output, false);
            }
        } else if (displayCfg.format < NEXUS_VideoFormat_eVesa640x480p60hz) {
            res = bcmSetComponentOutput(dfb_bcmnexus, screen, output, true);
        }
    }
#endif

    if (res == NEXUS_SUCCESS)
    {
#ifdef BCMNEXUS_DISPLAY_HAS_VSYNC_CALLBACK
        bcmInstallVSyncCallback(dfb_bcmnexus, screen, &displayCfg);
#endif

        SET_DISPLAY_WIDTH(dfb_bcmnexus,  info.width,  screen);
        SET_DISPLAY_HEIGHT(dfb_bcmnexus, info.height, screen);

        if (dfb_core_is_master( dfb_bcmnexus_core ))
        {
            res = NEXUS_Display_SetSettings(GET_DISPLAY_HND(dfb_bcmnexus, screen), &displayCfg);
        }
        else
        {
            bcmScreenCallSetDisplaySettings *pCallArgs;

            pCallArgs = SHMALLOC( dfb_core_shmpool(dfb_bcmnexus_core), sizeof(*pCallArgs) );

            if (!pCallArgs)
                return D_OOSHM();

            pCallArgs->displayId = screen;
            direct_memcpy( &pCallArgs->displaySettings, &displayCfg, sizeof(displayCfg) );

            fusion_call_execute( &GET_CALL_HANDLER(dfb_bcmnexus), FCEF_NONE, BCM_SCREEN_SET_DISPLAY_SETTINGS, pCallArgs, (int *)&res );

            SHFREE( dfb_core_shmpool(dfb_bcmnexus_core), pCallArgs );
        }
    }
    D_DEBUG_AT( bcmnexusScreen, "%s() screen %d, output=%d, format=%d Exit[%d]\n", __FUNCTION__, screen, output, format, res );
    return res;
}

static DFBResult primaryInitScreen( UNUSED_ CoreScreen         *screen,
                                    UNUSED_ CoreGraphicsDevice *device,
                                    void                       *driver_data,
                                    void                       *screen_data,
                                    DFBScreenDescription       *description )
{
    DFBResult          result;
    DFBBCMNEXUS       *dfb_bcmnexus = driver_data;
    bcmScreenData     *sd           = screen_data;
    NEXUS_VideoFormat  format;
#if NEXUS_DTV_PLATFORM
    NEXUS_DisplaySettings displayCfg;
#endif

    D_DEBUG_AT( bcmnexusScreen, "%s()\n", __FUNCTION__ );

    D_ASSERT( dfb_bcmnexus != NULL );

    /* Screen ID = 0 => Primary output ON */
    sd->id = PRIMARY_DISPLAY_ID;
    sd->powerMode = DSPM_ON;

    /* Set the screen capabilities. */
    description->caps = DSCCAPS_OUTPUTS | DSCCAPS_VSYNC | DSCCAPS_ENCODERS;
    description->outputs  = 1;
    description->encoders = 1;

    /* Set the screen name. */
    snprintf( description->name, DFB_SCREEN_DESC_NAME_LENGTH, "BCMNEXUS Primary Screen" );

    SET_DISPLAY_HND(dfb_bcmnexus, GET_NEXUS_DISPLAY_HND(0), sd->id);

#if NEXUS_DTV_PLATFORM
    if(!(getenv("output_type") && strcmp(getenv("output_type"), "component")==0))
    {
        NEXUS_Display_GetSettings(GET_DISPLAY_HND(dfb_bcmnexus, PRIMARY_DISPLAY_ID), &displayCfg);
        format = displayCfg.format;
        sd->fixedResolution = true;
    } else
#endif
    {
        format = getNexusVideoFormat();
        sd->fixedResolution = false;
    }

    result = (setVideoOutput(dfb_bcmnexus, sd->id, 0, format) == NEXUS_SUCCESS) ? DFB_OK : DFB_FAILURE;

    D_DEBUG_AT( bcmnexusScreen, "%s() screen %d, format=%d Exit[%d]\n", __FUNCTION__, sd->id, format, result );

    return result;
}

static void getDefaultEncoderCfg( UNUSED_ DFBScreenID             id,
                                          DFBScreenEncoderConfig *config)
{
    D_ASSERT(config != NULL);

    memset(config, 0, sizeof(DFBScreenEncoderConfig));
    config->flags = (DFBScreenEncoderConfigFlags)(DSECONF_TV_STANDARD | DSECONF_SCANMODE |
                                                  DSECONF_FREQUENCY | DSECONF_CONNECTORS |
                                                  DSECONF_RESOLUTION);
    /* Select default output connector based on priority
       and availability */
    config->out_connectors  = DSOC_UNKNOWN;
#if NEXUS_NUM_COMPOSITE_OUTPUTS
    config->out_connectors  = DSOC_CVBS;
#endif
#if NEXUS_NUM_SCART_INPUTS
    config->out_connectors  = DSOC_SCART;
#endif
#if NEXUS_NUM_SVIDEO_OUTPUTS
    config->out_connectors  = DSOC_YC;
#endif
#if NEXUS_NUM_COMPONENT_OUTPUTS
    config->out_connectors  = DSOC_COMPONENT;
#endif
#if NEXUS_NUM_HDMI_OUTPUTS
    config->out_connectors  = DSOC_HDMI;
#endif
    config->tv_standard     = DSETV_DIGITAL;

    if (strcmp(dfb_config->res,"480i") == 0) {
        config->tv_standard     = DSETV_NTSC;
        config->resolution      = DSOR_720_480;
        config->frequency       = DSEF_29_97HZ;
        config->scanmode        = DSESM_INTERLACED;
    }
    else if (strcmp(dfb_config->res,"480p") == 0) {
        config->resolution      = DSOR_720_480;
        config->frequency       = DSEF_59_94HZ;
        config->scanmode        = DSESM_PROGRESSIVE;
    }
    else if (strcmp(dfb_config->res,"576i") == 0) {
        config->tv_standard     = DSETV_PAL;
        config->resolution      = DSOR_720_576;
        config->frequency       = DSEF_25HZ;
        config->scanmode        = DSESM_INTERLACED;
    }
    else if (strcmp(dfb_config->res,"576p") == 0) {
        config->resolution      = DSOR_720_576;
        config->frequency       = DSEF_50HZ;
        config->scanmode        = DSESM_PROGRESSIVE;
    }
    else if (strcmp(dfb_config->res,"720p") == 0 || strcmp(dfb_config->res,"720p60") == 0) {
        config->resolution      = DSOR_1280_720;
        config->frequency       = DSEF_60HZ;
        config->scanmode        = DSESM_PROGRESSIVE;
    }
    else if (strcmp(dfb_config->res,"720ps") == 0) {
        config->resolution      = DSOR_1280_720;
        config->frequency       = DSEF_60HZ;
        config->scanmode        = DSESM_PROGRESSIVE;
        //config->framing         = DSEPF_STEREO_SEQUENTIAL;
    }
    else if (strcmp(dfb_config->res,"720p50") == 0) {
        config->resolution      = DSOR_1280_720;
        config->frequency       = DSEF_50HZ;
        config->scanmode        = DSESM_PROGRESSIVE;
    }
    else if (strcmp(dfb_config->res,"720p50s") == 0) {
        config->resolution      = DSOR_1280_720;
        config->frequency       = DSEF_50HZ;
        config->scanmode        = DSESM_PROGRESSIVE;
        //config->framing         = DSEPF_STEREO_SEQUENTIAL;
    }
    else if (strcmp(dfb_config->res,"1080i") == 0 || strcmp(dfb_config->res,"1080i30") == 0) {
        config->resolution      = DSOR_1920_1080;
        config->frequency       = DSEF_30HZ;
        config->scanmode        = DSESM_INTERLACED;
    }
    else if(strcmp(dfb_config->res,"1080i25") == 0) {
        config->resolution      = DSOR_1920_1080;
        config->frequency       = DSEF_25HZ;
        config->scanmode        = DSESM_INTERLACED;
    }
    else if (strcmp(dfb_config->res,"1080p") == 0 || strcmp(dfb_config->res,"1080p60") == 0) {
        config->resolution      = DSOR_1920_1080;
        config->frequency       = DSEF_60HZ;
        config->scanmode        = DSESM_PROGRESSIVE;
    }
    else if (strcmp(dfb_config->res,"1080p50") == 0) {
        config->resolution      = DSOR_1920_1080;
        config->frequency       = DSEF_50HZ;
        config->scanmode        = DSESM_PROGRESSIVE;
    }
    else if (strcmp(dfb_config->res,"1080p24") == 0) {
        config->resolution      = DSOR_1920_1080;
        config->frequency       = DSEF_24HZ;
        config->scanmode        = DSESM_PROGRESSIVE;
    }
    else if (strcmp(dfb_config->res,"1080p24s") == 0) {
        config->resolution      = DSOR_1920_1080;
        config->frequency       = DSEF_24HZ;
        config->scanmode        = DSESM_PROGRESSIVE;
        //config->framing         = DSEPF_STEREO_SEQUENTIAL;
    }
    else if (strcmp(dfb_config->res,"1080p25") == 0) {
        config->resolution      = DSOR_1920_1080;
        config->frequency       = DSEF_25HZ;
        config->scanmode        = DSESM_PROGRESSIVE;
    }
    else if (strcmp(dfb_config->res,"1080p30") == 0) {
        config->resolution      = DSOR_1920_1080;
        config->frequency       = DSEF_30HZ;
        config->scanmode        = DSESM_PROGRESSIVE;
    }
    else if (strcmp(dfb_config->res,"VGA") == 0) {
        config->resolution      = DSOR_640_480;
        config->frequency       = DSEF_60HZ;
        config->scanmode        = DSESM_PROGRESSIVE;
    }
    else if (strcmp(dfb_config->res,"SVGA") == 0) {
        config->resolution      = DSOR_800_600;
        config->frequency       = DSEF_60HZ;
        config->scanmode        = DSESM_PROGRESSIVE;
    }
    else if (strcmp(dfb_config->res,"XGA") == 0) {
        config->resolution      = DSOR_1024_768;
        config->frequency       = DSEF_60HZ;
        config->scanmode        = DSESM_PROGRESSIVE;
    }
    else {
        /* Default is 720p/60Hz */
        config->resolution      = DSOR_1280_720;
        config->frequency       = DSEF_60HZ;
        config->scanmode        = DSESM_PROGRESSIVE;
    }

    D_DEBUG_AT( bcmnexusScreen, "%s() screen %d defaults: o/p=%s, std=%s, res=%s, freq=%s, scan=%s\n", __FUNCTION__, id,
                dfb_screen_output_connectors_names[D_BITn32(config->out_connectors)].name,
                dfb_screen_encoder_tv_standards_names[D_BITn32(config->tv_standard)].name,
                dfb_screen_output_resolution_names[D_BITn32(config->resolution)].name,
                dfb_screen_encoder_frequency_names[D_BITn32(config->frequency)].name,
                dfb_screen_encoder_scan_mode_names[D_BITn32(config->scanmode)].name
              );
}

static void getEncoderCfgFromNexusDisplayCfg( NEXUS_DisplaySettings  *displayCfg,
                                              DFBScreenEncoderConfig *config )
{
    D_ASSERT(displayCfg != NULL);
    D_ASSERT(config != NULL);

    memset(config, 0, sizeof(DFBScreenEncoderConfig));
    config->flags = (DFBScreenEncoderConfigFlags)(DSECONF_TV_STANDARD | DSECONF_SCANMODE |
                                                  DSECONF_FREQUENCY | DSECONF_RESOLUTION );
    //config->framing = DSEPF_MONO;

    switch (displayCfg->format)
    {
    case NEXUS_VideoFormat_eNtsc:
        config->tv_standard = DSETV_NTSC;
        config->resolution  = DSOR_720_480;
        config->frequency   = DSEF_29_97HZ;
        config->scanmode    = DSESM_INTERLACED;
        break;

    case NEXUS_VideoFormat_eNtscJapan:
        config->tv_standard = DSETV_NTSC_M_JPN;
        config->resolution  = DSOR_720_480;
        config->frequency   = DSEF_29_97HZ;
        config->scanmode    = DSESM_INTERLACED;
        break;

    case NEXUS_VideoFormat_ePal:
    case NEXUS_VideoFormat_ePalH:
    case NEXUS_VideoFormat_ePalI:
    case NEXUS_VideoFormat_ePalK:
        config->tv_standard = DSETV_PAL;
        config->resolution  = DSOR_720_576;
        config->frequency   = DSEF_25HZ;
        config->scanmode    = DSESM_INTERLACED;
        break;

    case NEXUS_VideoFormat_ePalM:
        config->tv_standard = DSETV_PAL_M;
        config->resolution  = DSOR_720_480;
        config->frequency   = DSEF_30HZ;
        break;

    case NEXUS_VideoFormat_ePalN:
        config->tv_standard = DSETV_PAL_N;
        config->resolution  = DSOR_720_576;
        config->frequency   = DSEF_25HZ;
        break;

    case NEXUS_VideoFormat_ePalNc:
        config->tv_standard = DSETV_PAL_NC;
        config->resolution  = DSOR_720_576;
        config->frequency   = DSEF_25HZ;
        break;

    case NEXUS_VideoFormat_eSecam:
        config->tv_standard = DSETV_SECAM;
        config->resolution  = DSOR_720_576;
        config->frequency   = DSEF_25HZ;
        break;

    case NEXUS_VideoFormat_e480p:
        config->tv_standard = DSETV_DIGITAL;
        config->resolution  = DSOR_720_480;
        config->frequency   = DSEF_60HZ;
        config->scanmode    = DSESM_PROGRESSIVE;
        break;

    case NEXUS_VideoFormat_e576p:
        config->tv_standard = DSETV_DIGITAL;
        config->resolution  = DSOR_720_576;
        config->frequency   = DSEF_50HZ;
        config->scanmode    = DSESM_PROGRESSIVE;
        break;

    case NEXUS_VideoFormat_e720p24hz:
        config->tv_standard = DSETV_DIGITAL;
        config->resolution  = DSOR_1280_720;
        config->frequency   = DSEF_24HZ;
        config->scanmode    = DSESM_PROGRESSIVE;
        break;

    case NEXUS_VideoFormat_e720p25hz:
        config->tv_standard = DSETV_DIGITAL;
        config->resolution  = DSOR_1280_720;
        config->frequency   = DSEF_25HZ;
        config->scanmode    = DSESM_PROGRESSIVE;
        break;

    case NEXUS_VideoFormat_e720p30hz:
        config->tv_standard = DSETV_DIGITAL;
        config->resolution  = DSOR_1280_720;
        config->frequency   = DSEF_30HZ;
        config->scanmode    = DSESM_PROGRESSIVE;
        break;

    case NEXUS_VideoFormat_e720p50hz:
        config->tv_standard = DSETV_DIGITAL;
        config->resolution  = DSOR_1280_720;
        config->frequency   = DSEF_50HZ;
        config->scanmode    = DSESM_PROGRESSIVE;
        break;

/* CJT TODO
    case bdvd_video_format_720p_50hz_3d:
        config->tv_standard = DSETV_DIGITAL;
        config->resolution  = DSOR_1280_720;
        config->frequency   = DSEF_50HZ;
        config->scanmode    = DSESM_PROGRESSIVE;
        config->framing     = DSEPF_STEREO_SEQUENTIAL;
        break;
*/

    case NEXUS_VideoFormat_e720p:
        config->tv_standard = DSETV_DIGITAL;
        config->resolution  = DSOR_1280_720;
        config->frequency   = DSEF_60HZ;
        config->scanmode    = DSESM_PROGRESSIVE;
        break;

/* CJT TODO
    case bdvd_video_format_720p_3d:
        config->tv_standard = DSETV_DIGITAL;
        config->resolution  = DSOR_1280_720;
        config->frequency   = DSEF_60HZ;
        config->scanmode    = DSESM_PROGRESSIVE;
        config->framing     = DSEPF_STEREO_SEQUENTIAL;
        break;
*/

    case NEXUS_VideoFormat_e1080i50hz:
        config->tv_standard = DSETV_DIGITAL;
        config->resolution  = DSOR_1920_1080;
        config->frequency   = DSEF_50HZ;
        config->scanmode    = DSESM_INTERLACED;
        break;

    case NEXUS_VideoFormat_e1080i:
        config->tv_standard = DSETV_DIGITAL;
        config->resolution  = DSOR_1920_1080;
        config->frequency   = DSEF_60HZ;
        config->scanmode    = DSESM_INTERLACED;
        break;

    case NEXUS_VideoFormat_e1080p24hz:
        config->tv_standard = DSETV_DIGITAL;
        config->resolution  = DSOR_1920_1080;
        config->frequency   = DSEF_24HZ;
        config->scanmode    = DSESM_PROGRESSIVE;
        break;

/* CJT TODO
    case bdvd_video_format_1080p_24hz_3d:
        config->tv_standard = DSETV_DIGITAL;
        config->resolution  = DSOR_1920_1080;
        config->frequency   = DSEF_24HZ;
        config->scanmode    = DSESM_PROGRESSIVE;
        config->framing     = DSEPF_STEREO_SEQUENTIAL;
        break;
*/

    case NEXUS_VideoFormat_e1080p25hz:
        config->tv_standard = DSETV_DIGITAL;
        config->resolution  = DSOR_1920_1080;
        config->frequency   = DSEF_25HZ;
        config->scanmode    = DSESM_PROGRESSIVE;
        break;

    case NEXUS_VideoFormat_e1080p30hz:
        config->tv_standard = DSETV_DIGITAL;
        config->resolution  = DSOR_1920_1080;
        config->frequency   = DSEF_30HZ;
        config->scanmode    = DSESM_PROGRESSIVE;
        break;

    case NEXUS_VideoFormat_e1080p50hz:
        config->tv_standard = DSETV_DIGITAL;
        config->resolution  = DSOR_1920_1080;
        config->frequency   = DSEF_50HZ;
        config->scanmode    = DSESM_PROGRESSIVE;
        break;

    case NEXUS_VideoFormat_e1080p60hz:
        config->tv_standard = DSETV_DIGITAL;
        config->resolution  = DSOR_1920_1080;
        config->frequency   = DSEF_60HZ;
        config->scanmode    = DSESM_PROGRESSIVE;
        break;

    case NEXUS_VideoFormat_eVesa640x480p60hz:
        config->tv_standard = DSETV_DIGITAL;
        config->resolution  = DSOR_640_480;
        config->frequency   = DSEF_60HZ;
        config->scanmode    = DSESM_PROGRESSIVE;
        break;

    case NEXUS_VideoFormat_eVesa800x600p60hz:
        config->tv_standard = DSETV_DIGITAL;
        config->resolution  = DSOR_800_600;
        config->frequency   = DSEF_60HZ;
        config->scanmode    = DSESM_PROGRESSIVE;
        break;

    case NEXUS_VideoFormat_eVesa1024x768p60hz:
        config->tv_standard = DSETV_DIGITAL;
        config->resolution  = DSOR_1024_768;
        config->frequency   = DSEF_60HZ;
        config->scanmode    = DSESM_PROGRESSIVE;
        break;

    default:
        D_ERROR("%s() Unknown video_format enumeration, %d.\n", __FUNCTION__, displayCfg->format);
        config->tv_standard     = DSETV_UNKNOWN;
        config->resolution      = DSOR_UNKNOWN;
        config->frequency       = DSEF_UNKNOWN;
        config->scanmode        = DSESM_UNKNOWN;
        config->flags           = DSECONF_NONE;
        //config->framing         = DSEPF_UNKNOWN;
        break;
    }

/* TODO add connectors
    config->out_connectors  = DSOC_UNKNOWN;

    if (displayCfg->composite != NULL)
    {
        config->out_connectors  |= DSOC_YC;
        config->flags           |= DSECONF_CONNECTORS;
    }
    if (displayCfg->svideo != NULL)
    {
        config->out_connectors  |= DSOC_CVBS;
        config->flags           |= DSECONF_CONNECTORS;
    }
    if (displayCfg->component != NULL)
    {
        config->out_connectors  |= DSOC_COMPONENT;
        config->flags           |= DSECONF_CONNECTORS;
    }
    if (displayCfg->hdmi != NULL)
    {
        config->out_connectors  |= DSOC_HDMI;
        config->flags           |= DSECONF_CONNECTORS;
    }
*/
}

static DFBResult getNexusDisplayCfgFromEncoderCfg( DFBScreenEncoderConfig *config,
                                                   NEXUS_DisplaySettings *displayCfg )
{
    D_ASSERT(displayCfg != NULL);
    D_ASSERT(config != NULL);

    memset(displayCfg, 0, sizeof(displayCfg));
    //displayCfg->left_is_base_in_3d = true;

    if (config->flags & DSECONF_TV_STANDARD)
    {
        switch (config->tv_standard)
        {
        case DSETV_NTSC:
            displayCfg->format = NEXUS_VideoFormat_eNtsc;
            displayCfg->aspectRatio = NEXUS_DisplayAspectRatio_e4x3;
            break;

        case DSETV_NTSC_M_JPN:
            displayCfg->format = NEXUS_VideoFormat_eNtscJapan;
            displayCfg->aspectRatio = NEXUS_DisplayAspectRatio_e4x3;
            break;

        case DSETV_PAL:
            displayCfg->format = NEXUS_VideoFormat_ePal;
            displayCfg->aspectRatio = NEXUS_DisplayAspectRatio_e4x3;
            break;

        case DSETV_PAL_M:
            displayCfg->format = NEXUS_VideoFormat_ePalM;
            displayCfg->aspectRatio = NEXUS_DisplayAspectRatio_e4x3;
            break;

        case DSETV_PAL_N:
            displayCfg->format = NEXUS_VideoFormat_ePalN;
            displayCfg->aspectRatio = NEXUS_DisplayAspectRatio_e4x3;
            break;

        case DSETV_PAL_NC:
            displayCfg->format = NEXUS_VideoFormat_ePalNc;
            displayCfg->aspectRatio = NEXUS_DisplayAspectRatio_e4x3;
            break;

        case DSETV_SECAM:
            displayCfg->format = NEXUS_VideoFormat_eSecam;
            displayCfg->aspectRatio = NEXUS_DisplayAspectRatio_e4x3;
            break;

        case DSETV_DIGITAL:
            if (!(config->flags & DSECONF_RESOLUTION) ||
                !(config->flags & DSECONF_FREQUENCY) ||
                !(config->flags & DSECONF_SCANMODE))
            {
                D_ERROR("%s() Must specify resolution, frequency, and scanmode for DSETV_DIGITAL "
                        "TV standard\n", __FUNCTION__);
                config->flags = (DFBScreenEncoderConfigFlags)(DSECONF_RESOLUTION | DSECONF_FREQUENCY |
                                                              DSECONF_SCANMODE | DSECONF_TV_STANDARD);
                return DFB_INVARG;
            }

            displayCfg->format = -1;
            displayCfg->aspectRatio = NEXUS_DisplayAspectRatio_e16x9;
            switch (config->frequency)
            {
            case DSEF_23_976HZ:
            case DSEF_24HZ:
                if (config->scanmode == DSESM_PROGRESSIVE)
                {
                    if (config->resolution == DSOR_1280_720)
                        displayCfg->format = NEXUS_VideoFormat_e720p24hz;
                    else if (config->resolution == DSOR_1920_1080)
                    {
/*
                        if (config->framing == DSEPF_STEREO_SEQUENTIAL)
                        {
                            // CJT TODO
                            //displayCfg->format = bdvd_video_format_1080p_24hz_3d;
                            displayCfg->format = NEXUS_VideoFormat_e1080p24hz;
                        }
                        else
*/
                            displayCfg->format = NEXUS_VideoFormat_e1080p24hz;

                    }
                }
                break;

            case DSEF_25HZ:
                if (config->scanmode == DSESM_PROGRESSIVE)
                {
                    if (config->resolution == DSOR_1280_720)
                        displayCfg->format = NEXUS_VideoFormat_e720p25hz;
                    else if (config->resolution == DSOR_1920_1080)
                        displayCfg->format = NEXUS_VideoFormat_e1080p25hz;
                }
                else
                {
                    if (config->resolution == DSOR_720_576)
                        displayCfg->format = NEXUS_VideoFormat_ePal;
                    else if (config->resolution == DSOR_1920_1080)
                        displayCfg->format = NEXUS_VideoFormat_e1080i50hz;
                }
                break;

            case DSEF_29_97HZ:
                if (config->scanmode == DSESM_PROGRESSIVE)
                {
                    if (config->resolution == DSOR_1280_720)
                        displayCfg->format = NEXUS_VideoFormat_e720p30hz;
                    else if (config->resolution == DSOR_1920_1080)
                        displayCfg->format = NEXUS_VideoFormat_e1080p30hz;
                }
                else
                {
                    if (config->resolution == DSOR_720_480)
                        displayCfg->format = NEXUS_VideoFormat_eNtsc;
                    else if (config->resolution == DSOR_1920_1080)
                        displayCfg->format = NEXUS_VideoFormat_e1080i;
                }
                break;

            case DSEF_30HZ:
                if (config->scanmode == DSESM_PROGRESSIVE)
                {
                    if (config->resolution == DSOR_1280_720)
                        displayCfg->format = NEXUS_VideoFormat_e720p30hz;
                    else if (config->resolution == DSOR_1920_1080)
                        displayCfg->format = NEXUS_VideoFormat_e1080p30hz;
                }
                else
                {
                    if (config->resolution == DSOR_1920_1080)
                        displayCfg->format = NEXUS_VideoFormat_e1080i;
                }
                break;

            case DSEF_50HZ:
                if (config->scanmode == DSESM_PROGRESSIVE)
                {
                    if (config->resolution == DSOR_720_576)
                        displayCfg->format = NEXUS_VideoFormat_e576p;
                    else if (config->resolution == DSOR_1280_720)
/*
                        if (config->framing == DSEPF_STEREO_SEQUENTIAL)
                        {
                            // CJT TODO
                            //displayCfg->format = bdvd_video_format_720p_50hz_3d;
                            displayCfg->format = NEXUS_VideoFormat_e720p50hz;
                        }
                        else
*/
                            displayCfg->format = NEXUS_VideoFormat_e720p50hz;
                    else if (config->resolution == DSOR_1920_1080)
                        displayCfg->format = NEXUS_VideoFormat_e1080p50hz;
                }
                else
                    if (config->resolution == DSOR_1920_1080)
                        displayCfg->format = NEXUS_VideoFormat_e1080i50hz;
                break;

            case DSEF_59_94HZ:
                if (config->scanmode == DSESM_PROGRESSIVE)
                {
                    if (config->resolution == DSOR_720_480)
                        displayCfg->format = NEXUS_VideoFormat_e480p;
                    else if (config->resolution == DSOR_1280_720)
/*
                        if (config->framing == DSEPF_STEREO_SEQUENTIAL)
                        {
                            // CJT TODO
                            //displayCfg->format = bdvd_video_format_720p_3d;
                            displayCfg->format = NEXUS_VideoFormat_e720p;
                        }
                        else
*/
                            displayCfg->format = NEXUS_VideoFormat_e720p;
                    else if (config->resolution == DSOR_1920_1080)
                        displayCfg->format = NEXUS_VideoFormat_e1080p60hz;
                }
                break;

            case DSEF_60HZ:
                if (config->scanmode == DSESM_PROGRESSIVE)
                {
                    if (config->resolution == DSOR_640_480)
                        displayCfg->format = NEXUS_VideoFormat_eVesa640x480p60hz;
                    else if (config->resolution == DSOR_720_480)
                        displayCfg->format = NEXUS_VideoFormat_e480p;
                    else if (config->resolution == DSOR_800_600)
                        displayCfg->format = NEXUS_VideoFormat_eVesa800x600p60hz;
                    else if (config->resolution == DSOR_1280_720)
/*
                        if (config->framing == DSEPF_STEREO_SEQUENTIAL)
                        {
                            // CJT TODO
                            //displayCfg->format = bdvd_video_format_720p_3d;
                            displayCfg->format = NEXUS_VideoFormat_e720p;
                        }
                        else
*/
                            displayCfg->format = NEXUS_VideoFormat_e720p;
                    else if (config->resolution == DSOR_1024_768)
                        displayCfg->format = NEXUS_VideoFormat_eVesa1024x768p60hz;
                    else if (config->resolution == DSOR_1920_1080) {
                        displayCfg->format = NEXUS_VideoFormat_e1080p60hz;
/*
                        if (config->framing == DSEPF_STEREO_SEQUENTIAL)
                        {
                            // CJT TODO
                            //displayCfg->enable_1080p60_3d_mode = true;
                        }
*/
                    }
                }
                else
                    if (config->resolution == DSOR_1920_1080)
                        displayCfg->format = NEXUS_VideoFormat_e1080i;
                break;

            default:
                displayCfg->format = NEXUS_VideoFormat_eUnknown;
                break;
            }

            if (displayCfg->format == NEXUS_VideoFormat_eUnknown)
            {
                D_ERROR("%s() Unsupported resolution/frequency/scanmode specified %d/%d/%d.\n",
                    __FUNCTION__, config->resolution, config->frequency, config->scanmode);
                config->flags = (DFBScreenEncoderConfigFlags)(DSECONF_RESOLUTION | DSECONF_FREQUENCY |
                                                              DSECONF_SCANMODE | DSECONF_TV_STANDARD);
                return DFB_UNSUPPORTED;
            }
            break;

        default:
            D_ERROR("%s() Unsupported TV standard specified %d.\n", __FUNCTION__, config->tv_standard);
            config->flags = DSECONF_TV_STANDARD;
            return DFB_UNSUPPORTED;
        }
    }

    if (config->flags & DSECONF_CONNECTORS)
    {
        if (config->out_connectors & ~(DSOC_YC | DSOC_CVBS | DSOC_COMPONENT | DSOC_HDMI | DSOC_SCART))
        {
            D_ERROR("%s() Unsupported output connector.\n", __FUNCTION__);
            config->flags = DSECONF_CONNECTORS;
            return DFB_FAILURE;
        }
    }

    return DFB_OK;
}

static DFBResult bcmInitEncoder ( UNUSED_ CoreScreen                   *screen,
                                          void                         *driver_data,
                                          void                         *screen_data,
                                          int                           encoder,
                                          DFBScreenEncoderDescription  *description,
                                          DFBScreenEncoderConfig       *config )
{
    DFBBCMNEXUS             *dfb_bcmnexus = driver_data;
    bcmScreenData           *sd  = screen_data;
    NEXUS_DisplayHandle    display;

    D_DEBUG_AT( bcmnexusScreen, "%s() screen %d, encoder=%d, flags=0x%08x\n", __FUNCTION__, sd->id, encoder, config->flags );

    if (encoder > 0)
    {
        D_ERROR("%s() Unsupported encoder.\n", __FUNCTION__);
        return DFB_UNSUPPORTED;
    }

    /* Set the encoder capabilities. */
    description->caps           = (DFBScreenEncoderCapabilities)(DSECAPS_TV_STANDARDS | DSECAPS_SCANMODE   |
                                                                 DSECAPS_FREQUENCY    | DSECAPS_CONNECTORS |
                                                                 DSECAPS_RESOLUTION);
    description->type           = (DFBScreenEncoderType)(DSET_TV | DSET_DIGITAL);
    description->tv_standards   = (DFBScreenEncoderTVStandards)(DSETV_NTSC  | DSETV_NTSC_M_JPN | DSETV_PAL    |
                                                                DSETV_PAL_M | DSETV_PAL_N      | DSETV_PAL_NC |
                                                                DSETV_SECAM | DSETV_DIGITAL);
    description->all_connectors = (DFBScreenOutputConnectors)(
#if NEXUS_NUM_COMPOSITE_OUTPUTS
                                    DSOC_CVBS      |
#endif
#if NEXUS_NUM_COMPONENT_OUTPUTS
                                    DSOC_COMPONENT |
#endif
#if NEXUS_NUM_HDMI_OUTPUTS
                                    DSOC_HDMI      |
#endif
#if NEXUS_NUM_SCART_INPUTS
                                    DSOC_SCART     |
#endif
#if NEXUS_NUM_SVIDEO_OUTPUTS
                                    DSOC_YC        |
#endif
                                    DSOC_UNKNOWN);

    description->all_resolutions= (DFBScreenOutputResolution)(DSOR_640_480  | DSOR_720_480  | DSOR_720_576   | DSOR_800_600 |
                                                              DSOR_1024_768 | DSOR_1280_720 | DSOR_1920_1080);
    snprintf( description->name, DFB_SCREEN_ENCODER_DESC_NAME_LENGTH, "BCMNEXUS Pri. Encoder" );

    /* Set configuration. */
    display = GET_DISPLAY_HND(dfb_bcmnexus,PRIMARY_DISPLAY_ID);
    if (!display)
    {
        NEXUS_DisplaySettings displayCfg;

        NEXUS_Display_GetDefaultSettings(&displayCfg);

            /* Display hasn't been configured yet, so specify default configuration. */
        display = NEXUS_Display_Open(PRIMARY_DISPLAY_ID, &displayCfg);
        if (!display)
        {
            D_ERROR("%s{} Error opening NEXUS display!\n", __FUNCTION__);
            return DFB_FAILURE;
        }

        /* Get the default format (or overide if specified). */
        getDefaultEncoderCfg(sd->id, config);
    }
    else
    {
        /*
         * Display was already configured outside DirectFB, so keep
         * configuration unchanged. Since SetEncoderConfig will always be called
         * with the configuration we pass, set the flags to DSECONF_NONE to
         * prevent any calls to bdvd_display_set(). Then fill in the configuration from
         * the SetEncoderConfig function so that it is returned properly
         * when IDirectFBScreen::GetEncoderConfiguration() is called.
         */
        config->flags       = DSECONF_NONE;
        if (strcmp(dfb_config->res, " "))
        {
            getDefaultEncoderCfg(sd->id, config);
        }
    }

    /* Save display handle. */
    SET_DISPLAY_HND(dfb_bcmnexus, display, DSCID_PRIMARY);

    return DFB_OK;
}

static DFBResult bcmTestEncoderConfig( UNUSED_ CoreScreen                   *screen,
                                               void                         *driver_data,
                                               void                         *screen_data,
                                               int                           encoder,
                                               const DFBScreenEncoderConfig *config,
                                               DFBScreenEncoderConfigFlags  *failed )
{
    DFBResult               rc  = DFB_OK;
    DFBBCMNEXUS            *dfb_bcmnexus = driver_data;
    bcmScreenData          *sd  = screen_data;
    DFBScreenEncoderConfig  encoderConfig;
    NEXUS_DisplaySettings   displayCfg;

    D_DEBUG_AT( bcmnexusScreen, "%s()\n", __FUNCTION__ );

    D_ASSERT(dfb_bcmnexus != NULL);
    D_ASSERT(sd != NULL);
    D_ASSERT(encoder == 0);
    D_ASSERT(config != NULL);
    D_ASSERT(failed != NULL);

    if (encoder != 0)
    {
        D_ERROR("%s() Invalid encoder ID %d\n", __FUNCTION__, encoder);
        return DFB_UNSUPPORTED;
    }

    encoderConfig = *config;
    rc = getNexusDisplayCfgFromEncoderCfg(&encoderConfig, &displayCfg);

    if (rc != DFB_OK)
    {
        D_ERROR("%s() Couldn't determine Nexus display config from desired encoder config\n", __FUNCTION__);
        *failed = config->flags;
    }
    else
    {
        *failed = DSECONF_NONE;
    }

    return rc;
}

static DFBResult bcmSetEncoderConfig ( UNUSED_ CoreScreen                   *screen,
                                               void                         *driver_data,
                                               void                         *screen_data,
                                               int                           encoder,
                                               const DFBScreenEncoderConfig *config )
{
    DFBResult                       rc = DFB_OK;
    DFBBCMNEXUS                    *dfb_bcmnexus = driver_data;
    bcmScreenData                  *sd  = screen_data;
    NEXUS_DisplaySettings           displayCfg;
    DFBScreenEncoderConfig          encoderConfig;
    bool                            gfxDisabled;

    D_DEBUG_AT( bcmnexusScreen, "%s()\n", __FUNCTION__ );

    D_ASSERT(dfb_bcmnexus != NULL);
    D_ASSERT(sd != NULL);
    D_ASSERT(encoder == 0);
    D_ASSERT(config != NULL);

    if (encoder != 0)
    {
        D_ERROR("%s() Invalid encoder ID %d\n", __FUNCTION__, encoder);
        return DFB_UNSUPPORTED;
    }

    encoderConfig = *config;

    if (encoderConfig.flags == DSECONF_NONE)
    {
        /*
         * This is the special case during init when the display has already been
         * configured outside DirectFB, so we don't want to set anything but
         * need to set the encoder configuration so that later requests to retrieve
         * it provide the correct info.
         */

        D_DEBUG_AT( bcmnexusScreen, "%s() Using default encoder configuration\n", __FUNCTION__ );

        NEXUS_Display_GetSettings(GET_DISPLAY_HND(dfb_bcmnexus, sd->id), &displayCfg);
        getEncoderCfgFromNexusDisplayCfg(&displayCfg, &encoderConfig);
    }
    else
    {
#if NEXUS_DTV_PLATFORM
        if (!(getenv("output_type") && strcmp(getenv("output_type"), "component")==0))
        {
            NEXUS_Display_GetSettings(GET_DISPLAY_HND(dfb_bcmnexus, sd->id), &displayCfg);
            rc = DFB_OK;
        } else
#endif
        rc = getNexusDisplayCfgFromEncoderCfg(&encoderConfig, &displayCfg);

        if (rc != DFB_OK)
        {
            D_ERROR("%s() Couldn't determine NEXUS display config from desired encoder config\n", __FUNCTION__);
        }
        else
        {

            bcmLayerData *layerData = NULL;

#if 0
            if (encoderConfig.flags & DSECONF_CONNECTORS)
            {
                if (encoderConfig.out_connectors & DSOC_YC)
                {
                    if (displayCfg.composite == NULL)
                    {
                        displayCfg.composite = bdvd_output_composite_open(B_ID(0));
                        if (displayCfg.composite == NULL)
                        {
                            D_ERROR("%s() Failed to open composite output.\n", __FUNCTION__);
                            return DFB_FAILURE;
                        }
                    }
                }
                if (encoderConfig.out_connectors & DSOC_CVBS)
                {
                    if (displayCfg.svideo == NULL)
                    {
                        displayCfg.svideo = bdvd_output_svideo_open(B_ID(0));
                        if (displayCfg.svideo == NULL)
                        {
                            D_ERROR("%s() Failed to open S-Video output.\n", __FUNCTION__);
                            return DFB_FAILURE;
                        }
                    }
                }
                if (encoderConfig.out_connectors & DSOC_COMPONENT)
                {
                    if (displayCfg.component == NULL)
                    {
                        displayCfg.component = bdvd_output_component_open(B_ID(0), bdvd_output_component_type_yprpb);
                        if (displayCfg.component == NULL)
                        {
                            D_ERROR("%s() Failed to open component output.\n", __FUNCTION__);
                            return DFB_FAILURE;
                        }
                    }
                }
                if (encoderConfig.out_connectors & DSOC_HDMI)
                {
                    if (displayCfg.hdmi == NULL)
                    {
                        displayCfg.hdmi = bdvd_output_hdmi_open(B_ID(0));
                        if (displayCfg.hdmi == NULL)
                        {
                            D_ERROR("%s() Failed to open HDMI output.\n", __FUNCTION__);
                            return DFB_FAILURE;
                        }
                        bdvd_output_hdmi_get(displayCfg.hdmi, &hdmiCfg);
                        hdmiCfg.hotplug_callback_func = NULL;
                        hdmiCfg.disconnect_callback_func = NULL;
                        hdmiCfg.cable_in_callback_func = NULL;
                        hdmiCfg.cable_out_callback_func = NULL;
                        hdmiCfg.color_depth = bdvd_video_color_depth_24_bit;
                        hdmiCfg.colorimetry = bdvd_video_colorimetry_auto;
                        hdmiCfg.pixel_repetition = bdvd_pixel_repetition_none;
                        bdvd_output_hdmi_set(displayCfg.hdmi, &hdmiCfg);
                    }
                }
            }
#endif

            // Don't allow the slave app to change the layerState as this causes some lockups.
            // At the moment we don't have an API to allow the master app to decide who can control the actual display settings.
            // When we do this test will need to change.

            if (dfb_core_is_master( dfb_bcmnexus_core ) )
            {

                // We need to have a nasty little test here ...
                // If the region hasn't been realised yet there won't be any region data available inside the layer data
                // which will cause the layer calls below to barf.

                layerData = bcmGetLayerData(GET_DISPLAY_GFX_LAYER_ID(dfb_bcmnexus, PRIMARY_GFX_LAYER_INDEX, sd->id));

                if ( layerData == NULL || layerData->regionData == NULL)
                    gfxDisabled = true;
                else
                    gfxDisabled = false;

                /* Disable graphics first to help prevent unwanted artefacts */
                if (!gfxDisabled)
                    rc = bcmSetLayerState(dfb_bcmnexus, GET_DISPLAY_GFX_LAYER_ID(dfb_bcmnexus, PRIMARY_GFX_LAYER_INDEX, sd->id), false);

                if (rc == DFB_OK)
                {
                    usleep(20*1000);    /* Wait 20ms */

                    rc = (setVideoOutput(dfb_bcmnexus, sd->id, 0, displayCfg.format ) == NEXUS_SUCCESS || NEXUS_INVALID_PARAMETER ) ? DFB_OK : DFB_FAILURE;

                    if ((rc == DFB_OK) && (!gfxDisabled))
                    {
                        DFBRectangle       destRectangle;

                        destRectangle.x = 0;
                        destRectangle.y = 0;
                        destRectangle.w = GET_DISPLAY_WIDTH (dfb_bcmnexus, sd->id);
                        destRectangle.h = GET_DISPLAY_HEIGHT (dfb_bcmnexus, sd->id);

                        D_DEBUG_AT( bcmnexusScreen, "%s Setting display rect (%d,%d) %d x %d\n", __FUNCTION__,
                                    destRectangle.x,destRectangle.y,destRectangle.w,destRectangle.h );

                        /* Now set the new destination rectangle and framebuffer */
                        rc = bcmSetLayerDestRectangle(dfb_bcmnexus, GET_DISPLAY_GFX_LAYER_ID(dfb_bcmnexus, PRIMARY_GFX_LAYER_INDEX, sd->id), &destRectangle);
                    }
                }

                usleep(20*1000);    /* Wait 20ms */

                /* Now re-enable graphics */
                if (!gfxDisabled)
                    bcmSetLayerState(dfb_bcmnexus, GET_DISPLAY_GFX_LAYER_ID(dfb_bcmnexus, PRIMARY_GFX_LAYER_INDEX, sd->id), true);
            }
        }
    }
    /* Save the display settings. */
/* CJT TODO ?
    if (rc == DFB_OK)
    {
        bdvd_video_get_format_settings(displayCfg.format, &formatCfg);
        SET_DISPLAY_WIDTH(dfb_bcmnexus, formatCfg.width, sd->id);
        SET_DISPLAY_HEIGHT(dfb_bcmnexus, formatCfg.height, sd->id);
        SET_DISPLAY_INTERLACED(dfb_bcmnexus, formatCfg.interlaced, sd->id);
        SET_DISPLAY_ASPECT(dfb_bcmnexus, displayCfg.aspect_ratio, sd->id);
    }
*/

    D_DEBUG_AT( bcmnexusScreen, "%s() screen %d, format=%d Exit [%d]\n", __FUNCTION__, sd->id, displayCfg.format, rc );

    return rc;
}


#if (NUM_DISPLAYS > 1)
static DFBResult secondaryInitScreen( UNUSED_ CoreScreen         *screen,
                                      UNUSED_ CoreGraphicsDevice *device,
                                      void                       *driver_data,
                                      void                       *screen_data,
                                      DFBScreenDescription       *description )
{
    DFBResult          result;
    DFBBCMNEXUS       *dfb_bcmnexus = driver_data;
    bcmScreenData     *sd           = screen_data;
    NEXUS_VideoFormat  primaryFormat, secondaryFormat;

    D_DEBUG_AT( bcmnexusScreen, "%s()\n", __FUNCTION__ );

    D_ASSERT( dfb_bcmnexus != NULL );

    /* Screen ID = 1 => Secondary output ON */
    sd->id = SECONDARY_DISPLAY_ID;
    sd->powerMode = DSPM_ON;

    /* Set the screen capabilities. */
    description->caps = DSCCAPS_OUTPUTS | DSCCAPS_VSYNC;
    description->outputs = 1;

    /* Set the screen name. */
    snprintf( description->name, DFB_SCREEN_DESC_NAME_LENGTH, "BCMNEXUS Secondary Screen" );

    SET_DISPLAY_HND(dfb_bcmnexus, GET_NEXUS_DISPLAY_HND(1), sd->id);

    /* Read back the video format of the primary display and slave the SD output to it
       in terms of display resolution (and frequency) */
    primaryFormat = getVideoOutputFormat(dfb_bcmnexus, PRIMARY_DISPLAY_ID);

    switch (primaryFormat)
    {
    case NEXUS_VideoFormat_e1080p50hz:
    case NEXUS_VideoFormat_e1080i50hz:
    case NEXUS_VideoFormat_e720p50hz:
    case NEXUS_VideoFormat_e576p:
    case NEXUS_VideoFormat_ePal:
        secondaryFormat = NEXUS_VideoFormat_ePal;
        break;
    default:
        secondaryFormat = NEXUS_VideoFormat_eNtsc;
    }

    result = (setVideoOutput(dfb_bcmnexus, sd->id, 0, secondaryFormat) == NEXUS_SUCCESS) ? DFB_OK : DFB_FAILURE;
    D_DEBUG_AT( bcmnexusScreen, "%s() screen %d format=%d Exit [%d].\n", __FUNCTION__, sd->id, secondaryFormat, result );
    return result;
}
#endif

static DFBResult bcmInitOutput( UNUSED_ CoreScreen                 *screen,
                                UNUSED_ void                       *driver_data,
                                UNUSED_ void                       *screen_data,
                                UNUSED_ int                         output,
                                UNUSED_ DFBScreenOutputDescription *description,
                                UNUSED_ DFBScreenOutputConfig      *config )
{
    bcmScreenData *sd  = screen_data;

    D_DEBUG_AT( bcmnexusScreen, "%s() screen %d, output=%d, flags=0x%08x\n", __FUNCTION__, sd->id, output, config->flags );

    /* Set the output capabilities. */
    description->caps = DSOCONF_RESOLUTION;

    /* Set the configuration */
    config->flags |= DSOCONF_RESOLUTION;
    config->resolution = DSOR_UNKNOWN;

    return DFB_OK;
}

static DFBResult bcmTestOutputConfig( UNUSED_ CoreScreen                  *screen,
                                              void                        *driver_data,
                                              void                        *screen_data,
                                              int                          output,
                                              const DFBScreenOutputConfig *config,
                                              DFBScreenOutputConfigFlags  *failed )
{
    DFBBCMNEXUS   *dfb_bcmnexus = driver_data;
    bcmScreenData *sd           = screen_data;

    D_DEBUG_AT( bcmnexusScreen, "%s() screen %d, output=%d, flags=0x%08x\n", __FUNCTION__, sd->id, output, config->flags );

    D_ASSERT( dfb_bcmnexus != NULL );

    if (output >= NUM_DISPLAYS)
    {
        *failed |= DSOCONF_CONNECTORS;
    }

    if (config->flags & DSOCONF_RESOLUTION)
    {
        if (getNexusVideoFormatFromOutputRes(config->resolution, sd->id) == NEXUS_VideoFormat_eUnknown)
        {
            *failed |= DSOCONF_RESOLUTION;
        }
    }

    if (*failed)
    {
        D_DEBUG_AT( bcmnexusScreen, "%s() screen %d failed flags = 0x%08x\n", __FUNCTION__, sd->id, *failed );
        return DFB_UNSUPPORTED;
    }
    return DFB_OK;
}

static DFBResult bcmGetScreenSize( UNUSED_ CoreScreen *screen,
                                   void               *driver_data,
                                   void               *screen_data,
                                   int                *ret_width,
                                   int                *ret_height )
{
    DFBBCMNEXUS   *dfb_bcmnexus = driver_data;
    bcmScreenData *sd           = screen_data;

    D_ASSERT( dfb_bcmnexus != NULL );

    *ret_width  = GET_DISPLAY_WIDTH (dfb_bcmnexus, sd->id);
    *ret_height = GET_DISPLAY_HEIGHT(dfb_bcmnexus, sd->id);

    D_DEBUG_AT( bcmnexusScreen, "%s() screen %d: %dx%d\n", __FUNCTION__, sd->id, *ret_width, *ret_height );

    return DFB_OK;
}

static DFBResult bcmWaitForVSync( UNUSED_ CoreScreen *screen,
                                  void               *driver_data,
                                  void               *screen_data )
{
    DFBBCMNEXUS    *dfb_bcmnexus = driver_data;
    bcmScreenData  *sd           = screen_data;
    FusionSkirmish *mutex;

    D_ASSERT( dfb_bcmnexus != NULL );

    D_DEBUG_AT( bcmnexusScreenX, "Wait for VSync[%d]\n", sd->id );

    mutex = &GET_DISPLAY_VSYNC_MUTEX(dfb_bcmnexus, sd->id);

    fusion_skirmish_prevail( mutex );
    fusion_skirmish_wait( mutex, 0 );
    fusion_skirmish_dismiss( mutex );

    D_DEBUG_AT( bcmnexusScreenX, "Received VSync[%d]\n", sd->id );

    return DFB_OK;
}

static DFBResult bcmSetOutputConfig( UNUSED_ CoreScreen          *screen,
                                     void                        *driver_data,
                                     void                        *screen_data,
                                     int                          output,
                                     const DFBScreenOutputConfig *config )
{
    DFBResult          result       = DFB_OK;
    DFBBCMNEXUS       *dfb_bcmnexus = driver_data;
    bcmScreenData     *sd           = screen_data;
    NEXUS_VideoFormat  format;
    DFBRectangle       destRectangle;

    D_DEBUG_AT( bcmnexusScreen, "%s() screen %d, output=%d, flags=0x%08x\n", __FUNCTION__, sd->id, output, config->flags );

    D_ASSERT( dfb_bcmnexus != NULL );

    if (output >= NUM_DISPLAYS)
    {
        result = DFB_UNSUPPORTED;
    }
    else if (config->flags & DSOCONF_RESOLUTION)
    {
        format = getNexusVideoFormatFromOutputRes(config->resolution, sd->id);

        if ((format != NEXUS_VideoFormat_eUnknown) && (sd->fixedResolution == false))
        {
            D_DEBUG_AT( bcmnexusScreen, "%s() screen %d res=0x%08x, format=%d\n", __FUNCTION__, sd->id, config->resolution, format );

            /* Disable graphics first to help prevent unwanted artefacts */
            result = bcmSetLayerState(dfb_bcmnexus, GET_DISPLAY_GFX_LAYER_ID(dfb_bcmnexus, PRIMARY_GFX_LAYER_INDEX, sd->id), false);

            if (result == DFB_OK)
            {
                usleep(20*1000);    /* Wait 20ms */
                result = (setVideoOutput(dfb_bcmnexus, sd->id, 0, format) == NEXUS_SUCCESS) ? DFB_OK : DFB_FAILURE;

                if (result == DFB_OK)
                {
                    destRectangle.x = 0;
                    destRectangle.y = 0;
                    destRectangle.w = GET_DISPLAY_WIDTH(dfb_bcmnexus, sd->id);
                    destRectangle.h = GET_DISPLAY_HEIGHT(dfb_bcmnexus, sd->id);

                    /* Now set the new destination rectangle and framebuffer */
                    result = bcmSetLayerDestRectangle(dfb_bcmnexus, GET_DISPLAY_GFX_LAYER_ID(dfb_bcmnexus, PRIMARY_GFX_LAYER_INDEX, sd->id), &destRectangle);
                }
            }
            usleep(20*1000);    /* Wait 20ms */
            /* Now re-enable graphics */
            bcmSetLayerState(dfb_bcmnexus, GET_DISPLAY_GFX_LAYER_ID(dfb_bcmnexus, PRIMARY_GFX_LAYER_INDEX, sd->id), true);
        } else {
            result = DFB_UNSUPPORTED;
        }
    }
    D_DEBUG_AT( bcmnexusScreen, "%s() screen %d Exit [%d].\n", __FUNCTION__, sd->id, result );
    return result;
}

static DFBResult bcmSetPowerMode( UNUSED_ CoreScreen *screen,
                                  void               *driver_data,
                                  void               *screen_data,
                                  DFBScreenPowerMode  mode )
{
    DFBResult      res = DFB_OK;
    DFBBCMNEXUS   *dfb_bcmnexus = driver_data;
    bcmScreenData *sd           = screen_data;

    D_ASSERT( dfb_bcmnexus != NULL );

    D_DEBUG_AT( bcmnexusScreen, "%s: screen %d power mode=%d\n", __FUNCTION__, sd->id, mode );

    if (mode != sd->powerMode) {
        sd->powerMode = mode;
        res = bcmSetLayerState(dfb_bcmnexus, GET_DISPLAY_GFX_LAYER_ID(dfb_bcmnexus, PRIMARY_GFX_LAYER_INDEX, sd->id), mode==DSPM_ON);
    }
    D_DEBUG_AT( bcmnexusScreen, "%s() screen %d Exit [%d].\n", __FUNCTION__, sd->id, res );
    return res;
}

DFBScreenPowerMode bcmGetPowerMode(CoreScreen *screen)
{
    bcmScreenData *sd = screen->screen_data;
    return sd->powerMode;
}

ScreenFuncs bcmnexusPrimaryScreenFuncs = {
    .ScreenDataSize =    bcmScreenDataSize,
    .InitScreen =        primaryInitScreen,
    .InitOutput =        bcmInitOutput,
    .InitEncoder =       bcmInitEncoder,
    .TestEncoderConfig = bcmTestEncoderConfig,
    .SetEncoderConfig =  bcmSetEncoderConfig,
    .TestOutputConfig =  bcmTestOutputConfig,
    .SetOutputConfig =   bcmSetOutputConfig,
    .GetScreenSize =     bcmGetScreenSize,
    .WaitVSync =         bcmWaitForVSync,
    .SetPowerMode =      bcmSetPowerMode
};


#if (NUM_DISPLAYS > 1)
ScreenFuncs bcmnexusSecondaryScreenFuncs = {
    .ScreenDataSize =    bcmScreenDataSize,
    .InitScreen =        secondaryInitScreen,
    .InitOutput =        bcmInitOutput,
    .TestOutputConfig =  bcmTestOutputConfig,
    .SetOutputConfig =   bcmSetOutputConfig,
    .GetScreenSize =     bcmGetScreenSize,
    .WaitVSync =         bcmWaitForVSync,
    .SetPowerMode =      bcmSetPowerMode
};
#endif
