/******************************************************************************
 *    (c)2010 Broadcom Corporation
 * 
 * This program is the proprietary software of Broadcom Corporation and/or its licensors,
 * and may only be used, duplicated, modified or distributed pursuant to the terms and
 * conditions of a separate, written license agreement executed between you and Broadcom
 * (an "Authorized License").  Except as set forth in an Authorized License, Broadcom grants
 * no license (express or implied), right to use, or waiver of any kind with respect to the
 * Software, and Broadcom expressly reserves all rights in and to the Software and all
 * intellectual property rights therein.  IF YOU HAVE NO AUTHORIZED LICENSE, THEN YOU
 * HAVE NO RIGHT TO USE THIS SOFTWARE IN ANY WAY, AND SHOULD IMMEDIATELY
 * NOTIFY BROADCOM AND DISCONTINUE ALL USE OF THE SOFTWARE.  
 *  
 * Except as expressly set forth in the Authorized License,
 *  
 * 1.     This program, including its structure, sequence and organization, constitutes the valuable trade
 * secrets of Broadcom, and you shall use all reasonable efforts to protect the confidentiality thereof,
 * and to use this information only in connection with your use of Broadcom integrated circuit products.
 *  
 * 2.     TO THE MAXIMUM EXTENT PERMITTED BY LAW, THE SOFTWARE IS PROVIDED "AS IS" 
 * AND WITH ALL FAULTS AND BROADCOM MAKES NO PROMISES, REPRESENTATIONS OR 
 * WARRANTIES, EITHER EXPRESS, IMPLIED, STATUTORY, OR OTHERWISE, WITH RESPECT TO 
 * THE SOFTWARE.  BROADCOM SPECIFICALLY DISCLAIMS ANY AND ALL IMPLIED WARRANTIES 
 * OF TITLE, MERCHANTABILITY, NONINFRINGEMENT, FITNESS FOR A PARTICULAR PURPOSE, 
 * LACK OF VIRUSES, ACCURACY OR COMPLETENESS, QUIET ENJOYMENT, QUIET POSSESSION 
 * OR CORRESPONDENCE TO DESCRIPTION. YOU ASSUME THE ENTIRE RISK ARISING OUT OF 
 * USE OR PERFORMANCE OF THE SOFTWARE.
 * 
 * 3.     TO THE MAXIMUM EXTENT PERMITTED BY LAW, IN NO EVENT SHALL BROADCOM OR ITS 
 * LICENSORS BE LIABLE FOR (i) CONSEQUENTIAL, INCIDENTAL, SPECIAL, INDIRECT, OR 
 * EXEMPLARY DAMAGES WHATSOEVER ARISING OUT OF OR IN ANY WAY RELATING TO YOUR 
 * USE OF OR INABILITY TO USE THE SOFTWARE EVEN IF BROADCOM HAS BEEN ADVISED OF 
 * THE POSSIBILITY OF SUCH DAMAGES; OR (ii) ANY AMOUNT IN EXCESS OF THE AMOUNT 
 * ACTUALLY PAID FOR THE SOFTWARE ITSELF OR U.S. $1, WHICHEVER IS GREATER. THESE 
 * LIMITATIONS SHALL APPLY NOTWITHSTANDING ANY FAILURE OF ESSENTIAL PURPOSE OF 
 * ANY LIMITED REMEDY.
 *
 * $brcm_Workfile: videoAndGfxLayersTest.c $
 * $brcm_Revision: DirectFB_1_4_Port/5 $
 * $brcm_Date: 8/24/10 10:21a $
 * 
 * Module Description:
 * 
 * Revision History:
 * 
 * $brcm_Log: /AppLibs/opensource/directfb/src/broadcom_files/private/DirectFB/1.4/tools/videoAndGfxLayersTest.c $
 * 
 * DirectFB_1_4_Port/5   8/24/10 10:21a robertwm
 * SW7408-123: DirectFB: videoAndGfxLayersTest fails to compile with
 *  DirectFB-1.4.1 Phase 2.5 19th August release.
 * 
 * DirectFB_1_4_Port/4   5/5/10 1:47p robertwm
 * SW7550-412: DirectFB: videoAndGraphicsLayersTest.c should check whether
 *  Nexus features exist.
 * 
 * DirectFB_1_4_Port/3   4/19/10 2:47p robertwm
 * SW3556-1097: DirectFB: videoAndGfxLayersTest.c fails to build for DTV
 *  platforms.
 * 
 * DirectFB_1_4_Port/2   4/13/10 3:33p robertwm
 * SW7405-4199: DirectFB: Define FONT in Makefiles rather than in
 *  individual applications.
 * 
 * DirectFB_1_4_Port/1   4/13/10 1:00p robertwm
 * SW7405-4198: DirectFB: Add videoAndGfxLayersTest.c to tools dir.
 * 
 *****************************************************************************/
#include "nexus_platform.h"
#include "nexus_video_decoder.h"
#include "nexus_stc_channel.h"
#include "nexus_display.h"
#include "nexus_video_window.h"
#include "nexus_video_input.h"
#include "nexus_audio_dac.h"
#include "nexus_audio_decoder.h"
#include "nexus_audio_input.h"
#include "nexus_audio_output.h"
#if NEXUS_NUM_SPDIF_OUTPUTS
#include "nexus_spdif_output.h"
#endif
#if NEXUS_NUM_COMPONENT_OUTPUTS
#include "nexus_component_output.h"
#endif
#if NEXUS_NUM_COMPOSITE_OUTPUTS
#include "nexus_composite_output.h"
#endif
#include "nexus_parser_band.h"
#if NEXUS_HAS_FRONTEND
#include "nexus_frontend.h"
#include "nexus_frontend_ofdm.h"
#endif
#include "nexus_core_utils.h"
#include "nexus_message.h"

#if NEXUS_DTV_PLATFORM
#include "nexus_platform_boardcfg.h"
#elif NEXUS_NUM_HDMI_OUTPUTS
#include "nexus_hdmi_output_hdcp.h"
#endif

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <assert.h>
#include "bstd.h"
#include "bkni.h"

#define ATV_DEBUG(fmt...)   \
do { \
    fprintf(stdout, " ** %s:%d %s() ** ", __FILE__, __LINE__, __func__); \
    fprintf(stdout, ##fmt); \
    fprintf(stdout,"\n"); \
} while(0)

/* DirectFB By Jorge Pereira */
#include <pthread.h>
#include "platform_init.h"
#include <unistd.h>
#include <directfb.h>
#include <stdlib.h>

#include "bcmnexus_app_defines.h"

#define DFBCHECK(x...)                                         \
  {                                                            \
    DFBResult err = x;                                         \
                                                               \
    if (err != DFB_OK)                                         \
      {                                                        \
        ATV_DEBUG( "%s <%d>:\t", __FILE__, __LINE__ );         \
        DirectFBErrorFatal( #x, err );                         \
      }                                                        \
  }

/* Uncomment this line to enable DVB-T reception */
/*#define CONFIG_ENABLE_DVBT */

void print_DisplaySettings(NEXUS_DisplaySettings *displaySettings);

int exit_program = 0;
IDirectFB *dfb = NULL;
pthread_t draw_thread;
const char *text = "0x4c 0x49 0x4e 0x55 0x58";

long int ARGw, ARGh;

void *draw_thread_fun(void * arg)
{
	IDirectFBSurface *primary = NULL;
	int screen_width  = 0;
	int screen_height = 0;
	IDirectFBFont *font = NULL;
	int i, width;
	DFBFontDescription font_dsc;
	DFBSurfaceDescription dsc;
	
	(void)arg;

	i = 0;
	
	DFBCHECK (dfb->SetCooperativeLevel (dfb, DFSCL_FULLSCREEN));
	dsc.flags = DSDESC_CAPS;
    if (ARGw && ARGh)
    {
        dsc.flags += (DSDESC_WIDTH | DSDESC_HEIGHT);
        dsc.width = ARGw;
        dsc.height = ARGh;
    }
	dsc.caps  = DSCAPS_PRIMARY | DSCAPS_FLIPPING;
	printf("tttttttt        dsc.width= %d\n", dsc.width);
	printf("ttttttttt       dsc.height= %d\n", dsc.height);
	DFBCHECK (dfb->CreateSurface( dfb, &dsc, &primary ));
	DFBCHECK (primary->GetSize (primary, &screen_width, &screen_height));
	
	font_dsc.flags = DFDESC_HEIGHT;
	font_dsc.height = 64;
	
	DFBCHECK (dfb->CreateFont (dfb, FONT, &font_dsc, &font));
	DFBCHECK (primary->SetFont (primary, font));
	DFBCHECK (font->GetStringWidth (font, text, -1, &width));
	printf("\n\n\n\n ~~~~~~~~~~~~ SURFACE SIZE  %u W x %u ~~~~~~~~~~~~ H\n\n",
			screen_width, screen_height);
	
	while (! exit_program ) {
		for (i = screen_width; i > -width && !exit_program; i-=2) {
		  	DFBCHECK (primary->SetColor (primary, 0x0, 0xff, 0x0, 0x00));
		    DFBCHECK (primary->FillRectangle (primary, 0, 0, screen_width, screen_height));
		    DFBCHECK (primary->SetColor (primary, 0x80, 0x0, 0x20, 0xFF));
		    DFBCHECK (primary->DrawString (primary, text, -1, i, screen_height / 2, DSTF_LEFT));
			DFBCHECK (primary->Flip (primary, NULL, DSFLIP_WAITFORSYNC));
	  	
			usleep (500);
	    }
	}
	
	font->Release (font);
	primary->Release (primary);
	
	return NULL;
}


static void dfb_helloworld (int argc, char *argv[]) 
{
  ATV_DEBUG("Running DirectFB... main dfb_helloworld()");

  DFBCHECK (DirectFBInit (&argc, &argv));
  DFBCHECK (DirectFBCreate (&dfb));

  pthread_create(&draw_thread, NULL, draw_thread_fun,  (void*)dfb);
  
}




static NEXUS_AudioDecoderHandle pcmDecoder;
static NEXUS_AudioDecoderStartSettings audioProgram;
#if NEXUS_NUM_HDMI_OUTPUTS
static NEXUS_AudioDecoderHandle pcmDecoder, compressedDecoder;
static bool compressedHdmi = false, audioStarted = false;

static void disconnect_hdmi_audio(NEXUS_HdmiOutputHandle hdmi)
{
    NEXUS_AudioDecoderHandle decoder;

    if ( compressedHdmi )
    {
        decoder = compressedDecoder;
    }
    else
    {
        decoder = pcmDecoder;
    }

    if ( audioStarted )
        NEXUS_AudioDecoder_Stop(decoder);

    NEXUS_AudioOutput_RemoveAllInputs(NEXUS_HdmiOutput_GetAudioConnector(hdmi));

    if ( audioStarted )
        NEXUS_AudioDecoder_Start(decoder, &audioProgram);
}

static void connect_hdmi_audio(NEXUS_HdmiOutputHandle hdmi)
{
    NEXUS_AudioDecoderHandle decoder;
    NEXUS_AudioInput connector;

    if ( compressedHdmi )
    {
        decoder = compressedDecoder;
        connector = NEXUS_AudioDecoder_GetConnector(decoder, NEXUS_AudioDecoderConnectorType_eCompressed);
    }
    else
    {
        decoder = pcmDecoder;
        connector = NEXUS_AudioDecoder_GetConnector(decoder, NEXUS_AudioDecoderConnectorType_eStereo);
    }

    if ( audioStarted )
        NEXUS_AudioDecoder_Stop(decoder);

    NEXUS_AudioOutput_AddInput(NEXUS_HdmiOutput_GetAudioConnector(hdmi), connector);

    if ( audioStarted )
        NEXUS_AudioDecoder_Start(decoder, &audioProgram);
}

static void hotplug_callback(void *pParam, int iParam)
{
    static bool connected = false;
    NEXUS_HdmiOutputStatus status;
    NEXUS_HdmiOutputHandle hdmi = pParam;
    NEXUS_DisplayHandle display = (NEXUS_DisplayHandle)iParam;
    NEXUS_DisplaySettings displaySettings;

    NEXUS_HdmiOutput_GetStatus(hdmi, &status);

    if ( connected != status.connected )
    {
        if ( connected )
        {
            ATV_DEBUG("Hotplug - Disconnecting HDMI from display");
            disconnect_hdmi_audio(hdmi);
            NEXUS_Display_RemoveOutput(display, NEXUS_HdmiOutput_GetVideoConnector(hdmi));           
        }
        else
        {
            NEXUS_Display_GetSettings(display, &displaySettings);
            if ( !status.videoFormatSupported[displaySettings.format] )
            {
                ATV_DEBUG("Current format not supported by attached monitor.  Switching to preferred format %d", status.preferredVideoFormat);
                displaySettings.format = status.preferredVideoFormat;
                NEXUS_Display_SetSettings(display, &displaySettings);
            }

            ATV_DEBUG("Hotplug - connecting HDMI to display");
            NEXUS_Display_RemoveOutput(display, NEXUS_HdmiOutput_GetVideoConnector(hdmi));            
            NEXUS_Display_AddOutput(display, NEXUS_HdmiOutput_GetVideoConnector(hdmi));
            connect_hdmi_audio(hdmi);
        }
        connected = status.connected;
    }
}
#endif

#if NEXUS_HAS_FRONTEND
static void lock_callback(void *context, int param)
{
    NEXUS_FrontendHandle frontend = (NEXUS_FrontendHandle)context;
    NEXUS_FrontendOfdmStatus ofdmStatus;

    BSTD_UNUSED(param);
    NEXUS_Frontend_GetOfdmStatus(frontend, &ofdmStatus);
    ATV_DEBUG("OFDM Lock callback: receiverLock=%d, fecLock=%d", ofdmStatus.receiverLock, ofdmStatus.fecLock);
}
#endif


void print_VideoWindowSettings(NEXUS_VideoFormatInfo *videoFormatInfo)
{
	printf("  \n ~~~~~~~~~~~~NEXUS VIDEO INFO~~~~~~~~~~~~~\n");
	printf("  width = %u\n", videoFormatInfo->width);
    printf("  height= %u\n", videoFormatInfo->height);
    printf("  digitalWidth= %u\n", videoFormatInfo->digitalWidth);
    printf("  digitalHeight= %u\n", videoFormatInfo->digitalHeight);
    printf("  scanWidth= %u\n", videoFormatInfo->scanWidth);
    printf("  scanHeight= %u\n", videoFormatInfo->scanHeight);
    printf("  topActive= %u\n", videoFormatInfo->topActive);
    printf("  topMaxVbiPassThru= %u\n", videoFormatInfo->topMaxVbiPassThru);
    printf("  bottomMaxVbiPassThru= %u\n", videoFormatInfo->bottomMaxVbiPassThru);
    printf("  verticalFreq= %u\n", videoFormatInfo->verticalFreq);
    printf("  pixelFreq= %u\n", videoFormatInfo->pixelFreq);
    printf("  interlaced = %u\n", videoFormatInfo->interlaced);
	printf("  ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~\n");
}	


int main(int argc, char **argv)
{
    NEXUS_AudioDecoderHandle audioDecoder;
    NEXUS_DisplayHandle display;
	NEXUS_DisplaySettings displaySettings;

#if NEXUS_HAS_FRONTEND
    NEXUS_FrontendCapabilities capabilities;
    NEXUS_FrontendHandle frontend = NULL;
    NEXUS_FrontendOfdmSettings ofdmSettings;
    NEXUS_FrontendUserParameters userParams;    
    unsigned i = 0;
#endif

#if NEXUS_NUM_HDMI_OUTPUTS
    NEXUS_HdmiOutputSettings hdmiSettings;
	NEXUS_HdmiOutputHandle hdmiOutputHandle = NULL;
	NEXUS_HdmiOutputStatus hdmiOutputStatus;
#endif

    NEXUS_ParserBand parserBand;
    NEXUS_ParserBandSettings parserBandSettings;
    NEXUS_PlatformConfiguration platformConfig;
    NEXUS_StcChannelHandle stcChannel;
    NEXUS_StcChannelSettings stcSettings;
    NEXUS_VideoWindowHandle window;
    NEXUS_VideoDecoderHandle videoDecoder;
	NEXUS_VideoFormatInfo videoFormatInfo;
	NEXUS_VideoWindowSettings windowSettings;

    unsigned freq = 0;
    unsigned short video_pid = 0;
    unsigned short audio_pid = 0;
    unsigned short stc_pid = 0;
	int status = 0;


	/*
	 * parsing input
	 */
    if( argc > 1 ) {
        freq = atoi(argv[1]);
    }
    if (argc > 2) {
        video_pid = strtoul(argv[2], NULL, 0);
		stc_pid   = video_pid;
    }
    if (argc > 3) {
        audio_pid = strtoul(argv[3], NULL, 0);
    }
    if (argc > 4) {
        stc_pid = strtoul(argv[4], NULL, 0);
    }

    ARGw= ARGh= 0;

    if (argc > 5) {
        ARGw= strtoul(argv[5], NULL, 0); 
    }

    if (argc > 6) {
        ARGh= strtoul(argv[6], NULL, 0);
    }

    dfb_helloworld(argc, argv); 

    NEXUS_Platform_GetConfiguration(&platformConfig);

#if NEXUS_HAS_FRONTEND
	/**
	 * front end setup
	 */
    for ( i = 0; i < NEXUS_MAX_FRONTENDS; i++ ) {
        frontend = platformConfig.frontend[i];

        if (frontend) {
            NEXUS_Frontend_GetCapabilities(frontend, &capabilities);
            /* Does this frontend support ofdm? */
            if ( capabilities.ofdm ) {
                break;
            }
        }
    }

    if (frontend == NULL) {
        ATV_DEBUG("Unable to find OFDM-capable frontend");
        return 0;
    }

    NEXUS_Frontend_GetDefaultOfdmSettings(&ofdmSettings);
    ofdmSettings.frequency = freq;

#ifdef CONFIG_ENABLE_DVBT
#ifdef BCMNEXUS_FRONTEND_DVBT_SUPPORTED
	ofdmSettings.mode = NEXUS_FrontendOfdmMode_eDvbt;
#endif
	ofdmSettings.bandwidth = NEXUS_FrontendOfdmBandwidth_e8Mhz;
#else
#ifdef BCMNEXUS_FRONTEND_ISDBT_SUPPORTED
	ofdmSettings.mode = NEXUS_FrontendOfdmMode_eIsdbt;
#endif
	ofdmSettings.bandwidth = NEXUS_FrontendOfdmBandwidth_e6Mhz;
#endif

    ofdmSettings.lockCallback.callback = lock_callback;
    ofdmSettings.lockCallback.context = frontend;

    NEXUS_Frontend_GetUserParameters(frontend, &userParams);
#endif

    /* Map a parser band to the demod's input band (or streamer band if no frontend). */
    parserBand = NEXUS_ParserBand_e0;
    NEXUS_ParserBand_GetSettings(parserBand, &parserBandSettings);
    parserBandSettings.sourceType = NEXUS_ParserBandSourceType_eInputBand;
#if NEXUS_HAS_FRONTEND
    parserBandSettings.sourceTypeSettings.inputBand = userParams.param1;  /* Platform initializes this to input band */
#else
    NEXUS_Platform_GetStreamerInputBand(0, &parserBandSettings.sourceTypeSettings.inputBand);
#endif
    parserBandSettings.transportType = NEXUS_TransportType_eTs;
    NEXUS_ParserBand_SetSettings(parserBand, &parserBandSettings);

    NEXUS_StcChannel_GetDefaultSettings(0, &stcSettings);
    stcSettings.timebase = NEXUS_Timebase_e0;
    stcChannel = NEXUS_StcChannel_Open(0, &stcSettings);


    /*
	 * Bring up audio decoders and outputs
	 */
    audioDecoder = NEXUS_AudioDecoder_Open(0, NULL);
    pcmDecoder = audioDecoder;

#if NEXUS_NUM_AUDIO_DACS
    NEXUS_AudioOutput_AddInput(
        NEXUS_AudioDac_GetConnector(platformConfig.outputs.audioDacs[0]),
        NEXUS_AudioDecoder_GetConnector(audioDecoder, NEXUS_AudioDecoderConnectorType_eStereo));
#endif

#if NEXUS_NUM_SPDIF_OUTPUTS
    NEXUS_AudioOutput_AddInput(
        NEXUS_SpdifOutput_GetConnector(platformConfig.outputs.spdif[0]),
        NEXUS_AudioDecoder_GetConnector(audioDecoder, NEXUS_AudioDecoderConnectorType_eStereo));
#endif

    /*
	 * Bring up video display and outputs
	 */
    display = GET_NEXUS_DISPLAY_HND(0);
	NEXUS_Display_GetSettings(display, &displaySettings);
	displaySettings.displayType = NEXUS_DisplayType_eAuto;

#if NEXUS_NUM_HDMI_OUTPUTS
	/**
	 * verify HDMI status
	 */
	hdmiOutputHandle = platformConfig.outputs.hdmi[0];
	if(NEXUS_HdmiOutput_GetStatus(hdmiOutputHandle, &hdmiOutputStatus) != NEXUS_SUCCESS) {
		printf("error getting hdmiOutputStatus\n");
	}

	if(hdmiOutputStatus.connected == true) {
		printf(" >>>>>>> HDMI DEVICE CONNECTED <<<<<<<<< \n");
#else
    if (true) {
#endif

		if(NEXUS_DisplayModule_SetUpdateMode(NEXUS_DisplayUpdateMode_eManual) != NEXUS_SUCCESS) {
			printf("\n\n ERROR UPDATING THE DISPLAY !!!!!! \n\n");
		}

#if NEXUS_DTV_PLATFORM
		displaySettings.format = NEXUS_VideoFormat_e1080p;  
#else
		displaySettings.format = NEXUS_VideoFormat_e1080i;  
/* 		displaySettings.format = NEXUS_VideoFormat_e720p;*/
#endif
		displaySettings.aspectRatio = NEXUS_DisplayAspectRatio_e16x9;
		status = NEXUS_Display_SetSettings(display, &displaySettings);

		if( status == NEXUS_SUCCESS) {
			printf("\n\n&&&&&&&&&&& TV is %s &&&&&&&&&&&&\n\n", (displaySettings.format == NEXUS_VideoFormat_e1080p) ? "1080p" : "1080i");
		}
	
		if(NEXUS_DisplayModule_SetUpdateMode(NEXUS_DisplayUpdateMode_eNow) != NEXUS_SUCCESS) {
			printf("\n\n ERROR UPDATING THE DISPLAY !!!!!! \n\n");
		}
	
		if(NEXUS_DisplayModule_SetUpdateMode(NEXUS_DisplayUpdateMode_eAuto) != NEXUS_SUCCESS) {
			printf("\n\n ERROR UPDATING THE DISPLAY !!!!!! \n\n");
		}

#if NEXUS_NUM_HDMI_OUTPUTS
	    /* Install hotplug callback -- video only for now */
	    NEXUS_HdmiOutput_GetSettings(platformConfig.outputs.hdmi[0], &hdmiSettings);
	    hdmiSettings.hotplugCallback.callback = hotplug_callback;
	    hdmiSettings.hotplugCallback.context = platformConfig.outputs.hdmi[0];
	    hdmiSettings.hotplugCallback.param = (int)display;
	    NEXUS_HdmiOutput_SetSettings(platformConfig.outputs.hdmi[0], &hdmiSettings);
	
		/* Force a hotplug to check status */
	    hotplug_callback(platformConfig.outputs.hdmi[0], (int)display);
#endif
	}

    window = NEXUS_VideoWindow_Open(display, 0);

    /* bring up decoder and connect to display */
    videoDecoder = NEXUS_VideoDecoder_Open(0, NULL); /* take default capabilities */
    NEXUS_VideoWindow_AddInput(window, NEXUS_VideoDecoder_GetConnector(videoDecoder));

    while (1) {
        NEXUS_PidChannelHandle videoPidChannel, audioPidChannel, stcPidChannel;
        NEXUS_VideoDecoderStartSettings videoProgram;

        videoPidChannel = NEXUS_PidChannel_Open(parserBand, video_pid, NULL);
        audioPidChannel = NEXUS_PidChannel_Open(parserBand, audio_pid, NULL);

        if (video_pid == stc_pid)
            stcPidChannel = videoPidChannel;
        else
            stcPidChannel = NEXUS_PidChannel_Open(parserBand, stc_pid, NULL);
            
        NEXUS_VideoDecoder_GetDefaultStartSettings(&videoProgram);
        videoProgram.codec = NEXUS_VideoCodec_eH264;
        videoProgram.pidChannel = videoPidChannel;
        videoProgram.stcChannel = stcChannel;

        NEXUS_AudioDecoder_GetDefaultStartSettings(&audioProgram);
        audioProgram.codec = NEXUS_AudioCodec_eAacPlusLoas;
        audioProgram.pidChannel = audioPidChannel;
        audioProgram.stcChannel = NULL; /*stcChannel;*/

        NEXUS_StcChannel_GetSettings(stcChannel, &stcSettings);
        stcSettings.mode = NEXUS_StcChannelMode_ePcr; /* live */
        stcSettings.modeSettings.pcr.pidChannel = stcPidChannel;
        NEXUS_StcChannel_SetSettings(stcChannel, &stcSettings);

#if NEXUS_HAS_FRONTEND
        NEXUS_Frontend_TuneOfdm(frontend, &ofdmSettings);
#endif
        NEXUS_VideoDecoder_Start(videoDecoder, &videoProgram);
        NEXUS_AudioDecoder_Start(audioDecoder, &audioProgram);

		NEXUS_VideoWindow_GetSettings(window, &windowSettings);

		printf(" Window Settings Position x=%u y=%u w=%u h=%u\n\n",
				windowSettings.position.x,
				windowSettings.position.y,
				windowSettings.position.width,
				windowSettings.position.height);

		NEXUS_Display_GetSettings(display, &displaySettings);
		printf("video formats= %u\n\n", displaySettings.format);

		NEXUS_VideoFormat_GetInfo(displaySettings.format,
                                &videoFormatInfo);

		print_VideoWindowSettings(&videoFormatInfo);
	
        ATV_DEBUG("Press enter to tune again");
        int key = getchar();

        NEXUS_AudioDecoder_Stop(audioDecoder);
        NEXUS_VideoDecoder_Stop(videoDecoder);
        NEXUS_PidChannel_Close(videoPidChannel);
        NEXUS_PidChannel_Close(audioPidChannel);

        if(video_pid != stc_pid)
            NEXUS_PidChannel_Close(stcPidChannel);        

		if(key == 0x71 || key == 0x51) {
			printf(" q or Q was pressed....\n");
			break;
		}
    }

	printf("exiting......\n");
    exit_program = 1;
   
    NEXUS_VideoDecoder_Stop(videoDecoder);
    NEXUS_VideoInput_Shutdown(NEXUS_VideoDecoder_GetConnector(videoDecoder));
    NEXUS_VideoDecoder_Close(videoDecoder);

    NEXUS_AudioDecoder_Stop(audioDecoder);
#if NEXUS_NUM_AUDIO_DACS
    NEXUS_AudioOutput_RemoveAllInputs(NEXUS_AudioDac_GetConnector(platformConfig.outputs.audioDacs[0]));
#endif
#if NEXUS_NUM_SPDIF_OUTPUTS
    NEXUS_AudioOutput_RemoveAllInputs(NEXUS_SpdifOutput_GetConnector(platformConfig.outputs.spdif[0]));
#endif
    NEXUS_AudioInput_Shutdown(NEXUS_AudioDecoder_GetConnector(audioDecoder, NEXUS_AudioDecoderConnectorType_eStereo));
    NEXUS_AudioDecoder_Close(audioDecoder);

    NEXUS_StcChannel_Close(stcChannel);
#if NEXUS_HAS_FRONTEND
	NEXUS_Frontend_Close(frontend);
#endif

	NEXUS_Display_Close(display);

	printf("exiting successfully...\n");
    return 0;
}

