/******************************************************************************
 *    (c)2010 Broadcom Corporation
 * 
 * This program is the proprietary software of Broadcom Corporation and/or its licensors,
 * and may only be used, duplicated, modified or distributed pursuant to the terms and
 * conditions of a separate, written license agreement executed between you and Broadcom
 * (an "Authorized License").  Except as set forth in an Authorized License, Broadcom grants
 * no license (express or implied), right to use, or waiver of any kind with respect to the
 * Software, and Broadcom expressly reserves all rights in and to the Software and all
 * intellectual property rights therein.  IF YOU HAVE NO AUTHORIZED LICENSE, THEN YOU
 * HAVE NO RIGHT TO USE THIS SOFTWARE IN ANY WAY, AND SHOULD IMMEDIATELY
 * NOTIFY BROADCOM AND DISCONTINUE ALL USE OF THE SOFTWARE.  
 *  
 * Except as expressly set forth in the Authorized License,
 *  
 * 1.     This program, including its structure, sequence and organization, constitutes the valuable trade
 * secrets of Broadcom, and you shall use all reasonable efforts to protect the confidentiality thereof,
 * and to use this information only in connection with your use of Broadcom integrated circuit products.
 *  
 * 2.     TO THE MAXIMUM EXTENT PERMITTED BY LAW, THE SOFTWARE IS PROVIDED "AS IS" 
 * AND WITH ALL FAULTS AND BROADCOM MAKES NO PROMISES, REPRESENTATIONS OR 
 * WARRANTIES, EITHER EXPRESS, IMPLIED, STATUTORY, OR OTHERWISE, WITH RESPECT TO 
 * THE SOFTWARE.  BROADCOM SPECIFICALLY DISCLAIMS ANY AND ALL IMPLIED WARRANTIES 
 * OF TITLE, MERCHANTABILITY, NONINFRINGEMENT, FITNESS FOR A PARTICULAR PURPOSE, 
 * LACK OF VIRUSES, ACCURACY OR COMPLETENESS, QUIET ENJOYMENT, QUIET POSSESSION 
 * OR CORRESPONDENCE TO DESCRIPTION. YOU ASSUME THE ENTIRE RISK ARISING OUT OF 
 * USE OR PERFORMANCE OF THE SOFTWARE.
 * 
 * 3.     TO THE MAXIMUM EXTENT PERMITTED BY LAW, IN NO EVENT SHALL BROADCOM OR ITS 
 * LICENSORS BE LIABLE FOR (i) CONSEQUENTIAL, INCIDENTAL, SPECIAL, INDIRECT, OR 
 * EXEMPLARY DAMAGES WHATSOEVER ARISING OUT OF OR IN ANY WAY RELATING TO YOUR 
 * USE OF OR INABILITY TO USE THE SOFTWARE EVEN IF BROADCOM HAS BEEN ADVISED OF 
 * THE POSSIBILITY OF SUCH DAMAGES; OR (ii) ANY AMOUNT IN EXCESS OF THE AMOUNT 
 * ACTUALLY PAID FOR THE SOFTWARE ITSELF OR U.S. $1, WHICHEVER IS GREATER. THESE 
 * LIMITATIONS SHALL APPLY NOTWITHSTANDING ANY FAILURE OF ESSENTIAL PURPOSE OF 
 * ANY LIMITED REMEDY.
 *
 * $brcm_Workfile: dfbtest_alloc.c $
 * $brcm_Revision: DirectFB_1_4_Port/1 $
 * $brcm_Date: 9/6/10 5:41p $
 * 
 * Module Description:
 * 
 * Revision History:
 * 
 * $brcm_Log: /AppLibs/opensource/directfb/src/broadcom_files/public/DirectFB/1.4/tests/dfbtest_alloc.c $
 * 
 * DirectFB_1_4_Port/1   9/6/10 5:41p robertwm
 * SW7405-4843: DirectFB: Add surface allocation/deallocation DFB Test.
 * 
 *****************************************************************************/
#include <string.h>
#include <unistd.h>
#include <directfb.h>

#define ALLOC_COUNT 10000
#define WIDTH_MAX   1920
#define HEIGHT_MAX  1080

static unsigned int rand_pool = 0x12345678;
static unsigned int rand_add  = 0x87654321;

static inline unsigned int myrand()
{
     rand_pool ^= ((rand_pool << 7) | (rand_pool >> 25));
     rand_pool += rand_add;
     rand_add  += rand_pool;

     return rand_pool;
}

/* Simple test to constantly allocate and deallocate a surface */
int main( int argc, char *argv[] )
{
    IDirectFB             *dfb;
    IDirectFBDisplayLayer *primary_layer;
    IDirectFBSurface      *primary, *image, *image2;
    IDirectFBFont         *font;
    DFBFontDescription     font_desc;
    DFBResult              err;
    DFBDisplayLayerConfig  config;
    DFBSurfaceDescription  dsc, dsc2;
    int                    allocs, fontheight, pitch;
    void                  *ptr;
   
    /* initalize DirectFB and pass arguments */
    DirectFBInit( &argc, &argv );

    DirectFBSetOption ("bg-none", NULL);
    DirectFBSetOption ("no-init-layer", NULL);

    /* create the super interface */
    DirectFBCreate( &dfb );

    /* set cooperative level to DFSCL_FULLSCREEN for exclusive access to the
       primary layer */
    dfb->SetCooperativeLevel( dfb, DFSCL_FULLSCREEN );

    /* Get the primary surface, i.e. the surface of the primary layer. */
    dfb->GetDisplayLayer( dfb, DLID_PRIMARY, &primary_layer );

    primary_layer->SetCooperativeLevel( primary_layer, DLSCL_EXCLUSIVE );

    primary_layer->GetConfiguration (primary_layer , &config);
    config.flags = DLCONF_BUFFERMODE | DLCONF_SURFACE_CAPS;
    config.buffermode = DLBM_FRONTONLY;
    config.surface_caps = DSCAPS_PRIMARY;
    primary_layer->SetConfiguration (primary_layer , &config);
    primary_layer->GetSurface( primary_layer, &primary);

    dsc.flags        = DSDESC_CAPS | DSDESC_WIDTH | DSDESC_HEIGHT | DSDESC_PIXELFORMAT;
    dsc.pixelformat  = DSPF_ARGB;
    dsc.caps         = DSCAPS_VIDEOONLY | DSCAPS_SHARED;

    dsc2.flags       = DSDESC_WIDTH | DSDESC_HEIGHT | DSDESC_PIXELFORMAT;
    dsc2.pixelformat = DSPF_ARGB;

    font_desc.flags  = DFDESC_HEIGHT;
    font_desc.height = 24;

    dfb->CreateFont( dfb, FONT, &font_desc, &font );
    font->GetHeight( font, &fontheight );
    primary->SetFont( primary, font );
    
    primary->Clear( primary, 0x20, 0x20, 0x20, 0xff );
    primary->SetDrawingFlags( primary, DSDRAW_BLEND );
    primary->SetColor( primary, 0xFF, 0xFF, 0xFF, 0xFF );
    primary->DrawString( primary, "Testing Surface Allocation/Deallocation...", -1, 50, 50, DSTF_TOPLEFT );
    sleep(2);

    for (allocs = 0; allocs < ALLOC_COUNT; allocs++)
    {
        dsc.width       = myrand() & (WIDTH_MAX-1);
        dsc.height      = myrand() & (HEIGHT_MAX-1);

        dsc.width  = dsc.width  ? dsc.width  : 2;
        dsc.height = dsc.height ? dsc.height : 2;

        /* Create surface1 ... */
        err = dfb->CreateSurface( dfb, &dsc, &image );
        if (err) {
            fprintf(stderr, "ERROR: Could not create image1!\n");
            break;
        }
        /* Now try and allocate surface1 by locking it...*/
        err = image->Lock( image, DSLF_READ|DSLF_WRITE, &ptr, &pitch);

        if (err) {
            fprintf(stderr, "ERROR: Could not lock image1!\n");
            break;
        }
        dsc2.width       = myrand() & (WIDTH_MAX-1);
        dsc2.height      = myrand() & (HEIGHT_MAX-1);

        dsc2.width  = dsc2.width  ? dsc2.width  : 2;
        dsc2.height = dsc2.height ? dsc2.height : 2;

        /* Create surface2 ... */
        err = dfb->CreateSurface( dfb, &dsc2, &image2 );
        if (err) {
            fprintf(stderr, "ERROR: Could not create image2!\n");
            image->Release( image );
            break;
        }
        /* Now try and allocate surface2 by locking it...*/
        err = image2->Lock( image2, DSLF_READ|DSLF_WRITE, &ptr, &pitch);
        if (err) {
            fprintf(stderr, "ERROR: Could not lock image2!\n");
            break;
        }

        /* Now try and unlock the surfaces...*/
        err = image->Unlock( image );
        if (err) {
            fprintf(stderr, "ERROR: Could not unlock image1!\n");
            break;
        }
        err = image2->Unlock( image2 );
        if (err) {
            fprintf(stderr, "ERROR: Could not unlock image2!\n");
            break;
        }

        /* Now release the surfaces... */
        image->Release( image );
        image2->Release( image2 );
    }
    primary->Clear( primary, 0x20, 0x20, 0x20, 0xff );
    if (allocs < ALLOC_COUNT)
        primary->DrawString( primary, "Test Failed :-(", -1, 50, 50, DSTF_TOPLEFT );
    else
        primary->DrawString( primary, "Test Passed :-)", -1, 50, 50, DSTF_TOPLEFT );

    sleep(2);
    primary->Release( primary );
    dfb->Release( dfb );
    return 0;
}
