/******************************************************************************
 *    (c)2009-2011 Broadcom Corporation
 * 
 * This program is the proprietary software of Broadcom Corporation and/or its licensors,
 * and may only be used, duplicated, modified or distributed pursuant to the terms and
 * conditions of a separate, written license agreement executed between you and Broadcom
 * (an "Authorized License").  Except as set forth in an Authorized License, Broadcom grants
 * no license (express or implied), right to use, or waiver of any kind with respect to the
 * Software, and Broadcom expressly reserves all rights in and to the Software and all
 * intellectual property rights therein.  IF YOU HAVE NO AUTHORIZED LICENSE, THEN YOU
 * HAVE NO RIGHT TO USE THIS SOFTWARE IN ANY WAY, AND SHOULD IMMEDIATELY
 * NOTIFY BROADCOM AND DISCONTINUE ALL USE OF THE SOFTWARE.  
 *  
 * Except as expressly set forth in the Authorized License,
 *  
 * 1.     This program, including its structure, sequence and organization, constitutes the valuable trade
 * secrets of Broadcom, and you shall use all reasonable efforts to protect the confidentiality thereof,
 * and to use this information only in connection with your use of Broadcom integrated circuit products.
 *  
 * 2.     TO THE MAXIMUM EXTENT PERMITTED BY LAW, THE SOFTWARE IS PROVIDED "AS IS" 
 * AND WITH ALL FAULTS AND BROADCOM MAKES NO PROMISES, REPRESENTATIONS OR 
 * WARRANTIES, EITHER EXPRESS, IMPLIED, STATUTORY, OR OTHERWISE, WITH RESPECT TO 
 * THE SOFTWARE.  BROADCOM SPECIFICALLY DISCLAIMS ANY AND ALL IMPLIED WARRANTIES 
 * OF TITLE, MERCHANTABILITY, NONINFRINGEMENT, FITNESS FOR A PARTICULAR PURPOSE, 
 * LACK OF VIRUSES, ACCURACY OR COMPLETENESS, QUIET ENJOYMENT, QUIET POSSESSION 
 * OR CORRESPONDENCE TO DESCRIPTION. YOU ASSUME THE ENTIRE RISK ARISING OUT OF 
 * USE OR PERFORMANCE OF THE SOFTWARE.
 * 
 * 3.     TO THE MAXIMUM EXTENT PERMITTED BY LAW, IN NO EVENT SHALL BROADCOM OR ITS 
 * LICENSORS BE LIABLE FOR (i) CONSEQUENTIAL, INCIDENTAL, SPECIAL, INDIRECT, OR 
 * EXEMPLARY DAMAGES WHATSOEVER ARISING OUT OF OR IN ANY WAY RELATING TO YOUR 
 * USE OF OR INABILITY TO USE THE SOFTWARE EVEN IF BROADCOM HAS BEEN ADVISED OF 
 * THE POSSIBILITY OF SUCH DAMAGES; OR (ii) ANY AMOUNT IN EXCESS OF THE AMOUNT 
 * ACTUALLY PAID FOR THE SOFTWARE ITSELF OR U.S. $1, WHICHEVER IS GREATER. THESE 
 * LIMITATIONS SHALL APPLY NOTWITHSTANDING ANY FAILURE OF ESSENTIAL PURPOSE OF 
 * ANY LIMITED REMEDY.
 *
 * $brcm_Workfile: df_brcmTest.c $
 * $brcm_Revision: DirectFB_1_4_Port/7 $
 * $brcm_Date: 2/17/11 7:59p $
 * 
 * Module Description:
 * 
 * Revision History:
 * 
 * $brcm_Log: /AppLibs/opensource/directfb/src/broadcom_files/private/DirectFB/1.4.1/tools/df_brcmTest.c $
 * 
 * DirectFB_1_4_Port/7   2/17/11 7:59p raywu
 * SW7420-1509: DirectFB: Add remote key "Exit" support to df_dok, df_andi
 *  and df_brcmTest
 * 
 * DirectFB_1_4_Port/6   8/17/10 3:59p robertwm
 * SW3556-1183: DirectFB: Cannot run df_brcmTest on 3556 platform.
 * 
 * DirectFB_1_4_Port/5   8/17/10 12:06a robertwm
 * SW7405-4763: DirectFB: df_brcmTest fails to update Test Description
 *  when in "force-window" mode.
 * 
 * DirectFB_1_4_Port/4   8/13/10 4:08p robertwm
 * SW7405-4752: DirectFB: Add "Dual Output" capabilities to the "tools"
 *  unit test applications.
 * 
 * DirectFB_1_4_Port/3   7/23/10 12:05p robertwm
 * SW7405-4479: Segmentation fault happened in the example df_brcmTest.
 * 
 * DirectFB_1_4_Port/2   4/13/10 3:05p robertwm
 * SW7405-4199: DirectFB: Define FONT in Makefiles rather than in
 *  individual applications.
 * 
 * DirectFB_1_4_Port/1   12/4/09 1:10p robertwm
 * SW7405-3547: DirectFB: Create test harness for exercising all drawing
 *  and blitting combinations.
 * 
 *****************************************************************************/
#include <directfb.h>

#include <sys/time.h> /* for gettimeofday() */
#include <stdio.h>    /* for fprintf()      */
#include <stdlib.h>   /* for rand()         */
#include <unistd.h>   /* for sleep()        */
#include <string.h>   /* for strcmp()       */

#include "biglogo_A8.h"
#include "pngtest3.h"

/* Definitions */
#define VERSION "0.1"
#define BRCM_MSG "Broadcom DirectFB Test Harness - Version "

/* Uncomment the line below to enable mirroring of primary graphics on secondary display */
/*#define DUAL_OUTPUT*/

/* Specify Primary surface Pixel Format */
#define PRIMARY_PIXELFORMAT DSPF_ARGB
//#define PRIMARY_PIXELFORMAT DSPF_LUT8

/* Either use image file or internal header for image */
#define TEST_IMAGE_FILE "pngtest2.png"
#define LOGO_IMAGE_FILE "biglogo_LUT8.gif"

#define USE_IMAGE_LUT8_FILE
#define LOGO_IMAGE_LUT8_FILE "beta_LUT8.png"

/* Define colors */
#define BG_COLOR_R 0xFF
#define BG_COLOR_G 0x80
#define BG_COLOR_B 0x80
#define BG_COLOR_A 0xFF

#define BOR_COLOR_R 0xFF
#define BOR_COLOR_G 0xFF
#define BOR_COLOR_B 0x00
#define BOR_COLOR_A 0xFF

#define BLT_COLOR_R 0x20
#define BLT_COLOR_G 0x40
#define BLT_COLOR_B 0x60
#define BLT_COLOR_A 0xFF

/* macro for a safe call to DirectFB functions */
#define DFBCHECK(x...)                                                     \
    err = x;                                                    \
    if (err != DFB_OK) {                                        \
        fprintf( stderr, "%s <%d>:\n\t", __FILE__, __LINE__ ); \
        DirectFBErrorFatal( #x, err );						   \
    }


/* Main DirectFB Interface */
static IDirectFB *dfb;

static IDirectFBSurface *background;

/* Co-ords for the windows */
static int MAIN_W, MAIN_H;
static int SUB_W = 500, SUB_H = 500;
static int SUB1_X = 100;
static int SUB1_Y = 150;
static int SUB2_X = 650;
static int SUB2_Y = 150;

static int INFO_W = (650 + 500 - 100); /* SUB2_X + SUB_W - SUB1_X */
static int INFO_H = 60;
static int INFO_X = 100; /* SUB1_X */
static int INFO_Y = 40;

static DFBSurfaceBlittingFlags globalBlittingFlags = DSBLIT_NOFX;
static DFBSurfaceDrawingFlags  globalDrawingFlags  = DSDRAW_NOFX;

static IDirectFBEventBuffer *key_events;
static IDirectFBFont        *font;
static IDirectFBSurface *logo_surface, *biglogo_surface;
static IDirectFBSurface *test_surface, *pngtest2_surface;

#ifdef DUAL_OUTPUT
static IDirectFBDisplayLayer *secondary_layer;
#endif
static IDirectFBSurface *primary_hardware, *primary_software, *primary_info;
static IDirectFBSurface *primary;
static DFBSurfaceDescription dsc;


static void shutdown()
{
/* release our interfaces to shutdown DirectFB */
    font->Release( font );
    test_surface->Release( test_surface );
    logo_surface->Release( logo_surface );
    if (dsc.pixelformat == DSPF_LUT8) {
        pngtest2_surface->Release( pngtest2_surface );
#ifndef USE_IMAGE_LUT8_FILE
        biglogo_surface->Release( biglogo_surface );
#endif
    }
    background->Release( background );
    primary_info->Release( primary_info );
    primary_hardware->Release( primary_hardware );
    primary_software->Release( primary_software );
#ifdef DUAL_OUTPUT
    secondary_layer->Release( secondary_layer );
#endif
    primary->Release( primary );
    key_events->Release( key_events );
    dfb->Release( dfb );
}


static inline void get_user_input()
{
    DFBInputEvent ev;
    
    while (1)
    {
        if (key_events->GetEvent( key_events, DFB_EVENT(&ev) ) == DFB_OK) 
        {
            if (ev.type == DIET_KEYPRESS) {
                if  (ev.key_symbol == DIKS_RETURN ||
                        ev.key_symbol == DIKS_OK ||
                        ev.key_symbol == DIKS_SELECT) 
                    break;
                if  (ev.key_symbol == DIKS_ESCAPE ||
                        ev.key_symbol == DIKS_SMALL_Q ||
                        ev.key_symbol == DIKS_CAPITAL_Q ||
                        ev.key_symbol == DIKS_BACK ||
                        ev.key_symbol == DIKS_STOP ||
                        ev.key_symbol == DIKS_EXIT) 
                {
                    shutdown();
                    exit( 42 );
                    break;
                }
            }
        }
        key_events->Reset( key_events );		 
        key_events->WaitForEvent( key_events );
    }
}

static IDirectFBSurface *create_context(IDirectFBSurface *surface,
                                        int x, int y, int width, int height)
{
   DFBResult err;
   IDirectFBSurface *context;
   DFBRectangle rect = {x, y, width, height};

   DFBCHECK(surface->GetSubSurface(surface, &rect, &context));
   
   return context;
}

static void flip_context(IDirectFBSurface *info_surface, IDirectFBSurface *hardware_surface, IDirectFBSurface *software_surface)
{
    DFBResult err;

    DFBCHECK(hardware_surface->Flip(info_surface,     NULL, DSFLIP_BLIT));
    DFBCHECK(hardware_surface->Flip(hardware_surface, NULL, DSFLIP_BLIT));
    DFBCHECK(software_surface->Flip(software_surface, NULL, DSFLIP_BLIT));
}

static void display_information(IDirectFBSurface *surface, char *string)
{
    DFBResult err;
    char buf[81];

    DFBCHECK(surface->Clear(surface, BG_COLOR_R, BG_COLOR_G, BG_COLOR_B, BG_COLOR_A));
    DFBCHECK(surface->SetDrawingFlags(surface, DSDRAW_BLEND));
    DFBCHECK(surface->SetColor(surface, 0xff, 0xff, 0xff, 0xff));
    sprintf(buf, BRCM_MSG VERSION);
    DFBCHECK(surface->DrawString(surface, buf, -1, 0, 0, DSTF_TOPLEFT));
    DFBCHECK(surface->DrawString(surface, string, -1, 0, 30, DSTF_TOPLEFT));
    printf("%s\n", string);
}

static void prepare_test_surface(IDirectFBSurface *surface, bool isHwSurface)
{
    DFBResult err;
    DFBRectangle rect;

    rect.x = 0;
    rect.y = 0;
    rect.w = SUB_W;
    rect.h = SUB_H;

    DFBCHECK(surface->SetPorterDuff(surface, DSPD_NONE));
    DFBCHECK(surface->Clear( surface, BG_COLOR_R, BG_COLOR_G, BG_COLOR_B, BG_COLOR_A));
    DFBCHECK(surface->SetBlittingFlags( surface, DSBLIT_BLEND_ALPHACHANNEL ));
    DFBCHECK(surface->DisableAcceleration(surface, DFXL_NONE)); 
    DFBCHECK(surface->StretchBlit(surface, background, NULL, &rect));
    if (!isHwSurface) {
        DFBCHECK(surface->DisableAcceleration(surface, DFXL_ALL));
    }
}

static void porter_duff_fill(IDirectFBSurface *surface, DFBSurfacePorterDuffRule rule, bool isHwSurface)
{
    DFBResult err;
    DFBAccelerationMask accel_mask;

    prepare_test_surface(surface, isHwSurface);

    /* Overlapping boxes with blending enabled */
    DFBCHECK(surface->SetPorterDuff(surface, rule));
    DFBCHECK(surface->SetColor(surface, 0xff, 0x00, 0x00, 0x77));
    DFBCHECK(surface->FillRectangle(surface, 50, 150, 150, 150));
    DFBCHECK(surface->SetColor(surface, 0x00, 0xff, 0x00, 0x77));
    DFBCHECK(surface->FillRectangle(surface, 150, 200, 150, 150));
    DFBCHECK(surface->SetColor(surface, 0x00, 0x00, 0xff, 0x77));
    DFBCHECK(surface->FillRectangle(surface, 100, 250, 150, 150));

    DFBCHECK(surface->Flip(surface, NULL, DSFLIP_BLIT));
}

static void setup_blit(IDirectFBSurface *surface, DFBSurfaceBlittingFlags flags, int alpha, bool isHwSurface)
{
    DFBResult err;

    prepare_test_surface( surface, isHwSurface );
    DFBCHECK(surface->SetColor( surface, BLT_COLOR_R, BLT_COLOR_G, BLT_COLOR_B, alpha ));
    DFBCHECK(surface->SetBlittingFlags( surface, flags | globalBlittingFlags ));
}

static void setup_fill(IDirectFBSurface *surface, DFBSurfaceDrawingFlags flags, int red, int green, int blue, int alpha, bool isHwSurface)
{
    DFBResult err;

    prepare_test_surface( surface, isHwSurface );
    DFBCHECK(surface->SetColor( surface, red, green, blue, alpha ));
    DFBCHECK(surface->SetDrawingFlags( surface, flags | globalDrawingFlags ));
}

static void porter_duff_fills(IDirectFBSurface *hardware_surface, IDirectFBSurface *software_surface, DFBSurfacePorterDuffRule rule)
{
    porter_duff_fill(hardware_surface, rule, true);
    porter_duff_fill(software_surface, rule, false);
}

static void setup_blits(IDirectFBSurface *hardware_surface, IDirectFBSurface *software_surface, DFBSurfaceBlittingFlags flags, int alpha)
{
    setup_blit(hardware_surface, flags, alpha, true);
    setup_blit(software_surface, flags, alpha, false);
}

static void setup_porter_duff_blits(IDirectFBSurface *hardware_surface, IDirectFBSurface *software_surface, DFBSurfacePorterDuffRule rule,
                                    DFBSurfaceBlittingFlags flags, int alpha)
{
    DFBResult err;

    setup_blit(hardware_surface, flags, alpha, true);
    DFBCHECK(hardware_surface->SetPorterDuff(hardware_surface, rule));
    setup_blit(software_surface, flags, alpha, false);
    DFBCHECK(software_surface->SetPorterDuff(software_surface, rule));
}

static void setup_fills(IDirectFBSurface *hardware_surface, IDirectFBSurface *software_surface, DFBSurfaceDrawingFlags flags,
                       int red, int green, int blue, int alpha)
{
    setup_fill(hardware_surface, flags, red, green, blue, alpha, true);
    setup_fill(software_surface, flags, red, green, blue, alpha, false);
}

int main( int argc, char *argv[] )
{
    DFBResult err;

#ifdef DUAL_OUTPUT
    DFBDisplayLayerConfig layerCfg;
#endif
    IDirectFBImageProvider *provider;
    DFBSurfacePixelFormat pixelformat;
    DFBSurfaceDescription dsc_bg, dsc_logo, dsc_test_surface;
    DFBImageDescription image_dsc;
    int fontheight;
    char buf[128];
    int test_number = 1;
    DFBRectangle logo_rect, test_rect;
    DFBAccelerationMask accel_mask;

    /* Input interfaces: event buffer */
    int i, n;

    if (getenv("blittingflags"))
    {
        globalBlittingFlags = strtoul(getenv("blittingflags"), NULL, 0);
        fprintf(stderr, "%s: Blitting Flags = 0x%08x\n", argv[0], globalBlittingFlags);
    }

    if (getenv("drawingflags"))
    {
        globalDrawingFlags = strtoul(getenv("drawingflags"), NULL, 0);
        fprintf(stderr, "%s: Drawing Flags = 0x%08x\n", argv[0], globalDrawingFlags);
    }

    DFBCHECK(DirectFBInit( &argc, &argv ));

    DirectFBSetOption ("bg-none", NULL);

    /* create the super interface */
    DFBCHECK(DirectFBCreate( &dfb ));

    /* Set the cooperative level to DFSCL_FULLSCREEN for exclusive access to the primary layer. */
    err = dfb->SetCooperativeLevel( dfb, DFSCL_FULLSCREEN );
    if (err)
        DirectFBError( "Failed to get exclusive access", err );

    /* Get the primary surface, i.e. the surface of the primary layer. */
    dsc.flags = DSDESC_CAPS | DSDESC_PIXELFORMAT;
    dsc.caps = DSCAPS_PRIMARY;
    dsc.pixelformat = PRIMARY_PIXELFORMAT;

    /* Create the Primary Surface */
    DFBCHECK(dfb->CreateSurface( dfb, &dsc, &primary ));
    primary->GetPixelFormat( primary, &pixelformat );
    printf("Primary pixel format = 0x%08x\n", pixelformat);
    primary->GetSize( primary, &MAIN_W, &MAIN_H );

#ifdef DUAL_OUTPUT
    DFBCHECK(dfb->GetDisplayLayer(dfb, DLID_PRIMARY+1, &secondary_layer));
    DFBCHECK(secondary_layer->SetCooperativeLevel(secondary_layer, DLSCL_EXCLUSIVE));
    layerCfg.flags = DLCONF_SOURCE;
    layerCfg.source = DLSID_SURFACE;
    DFBCHECK(secondary_layer->SetConfiguration(secondary_layer, &layerCfg));
#endif

    /* create an input buffer for key events */
    DFBCHECK(dfb->CreateInputEventBuffer( dfb, DICAPS_KEYS,
                                           DFB_FALSE, &key_events ));


    /* Create Image Provider for rendering to test surface */
    if (dsc.pixelformat != DSPF_LUT8)
    {
        DFBCHECK(dfb->CreateImageProvider( dfb, DATADIR "/" TEST_IMAGE_FILE,
                                          &provider ));
        DFBCHECK(provider->GetSurfaceDescription( provider, &dsc_test_surface ));

        /* Create the test surface */
        dsc_test_surface.flags = DSDESC_WIDTH | DSDESC_HEIGHT | DSDESC_PIXELFORMAT;
        printf("%s: pixel format=0x%08x\n", TEST_IMAGE_FILE, dsc_test_surface.pixelformat);
        DFBCHECK(dfb->CreateSurface( dfb, &dsc_test_surface, &test_surface ));

        DFBCHECK(provider->RenderTo( provider, test_surface, NULL ));
        provider->Release(provider);

        /* Create Image Provider for rendering to logo surface */
        DFBCHECK(dfb->CreateImageProvider( dfb, DATADIR "/" LOGO_IMAGE_FILE,
                                          &provider ));
        DFBCHECK(provider->GetSurfaceDescription( provider, &dsc_logo ));
        dsc_logo.width  = (MAIN_H / 16) * dsc_logo.width / dsc_logo.height;
        dsc_logo.height = MAIN_H / 16;
        dsc_logo.flags = DSDESC_WIDTH | DSDESC_HEIGHT | DSDESC_PIXELFORMAT;
        printf("%s: pixel format=0x%08x\n", LOGO_IMAGE_FILE, dsc_logo.pixelformat);
        DFBCHECK(dfb->CreateSurface( dfb, &dsc_logo, &logo_surface ));
        DFBCHECK(provider->RenderTo( provider, logo_surface, NULL ));
        provider->Release(provider);
    }
    else
    {
        IDirectFBPalette *palette;

        DFBCHECK(dfb->CreateSurface( dfb, &pngtest3_png_desc, &pngtest2_surface ));
        DFBCHECK(pngtest2_surface->GetPalette( pngtest2_surface, &palette ));

        memset(&dsc_test_surface, 0, sizeof(dsc_test_surface));
        dsc_test_surface.flags = DSDESC_WIDTH | DSDESC_HEIGHT | DSDESC_PIXELFORMAT;
        dsc_test_surface.width = pngtest3_png_desc.width;
        dsc_test_surface.height = pngtest3_png_desc.height;
        dsc_test_surface.pixelformat = pngtest3_png_desc.pixelformat;
        DFBCHECK(dfb->CreateSurface( dfb, &dsc_test_surface, &test_surface ));
        DFBCHECK(test_surface->SetPalette( test_surface, palette ));
        DFBCHECK(primary->SetPalette( primary, palette ));
        test_surface->Blit( test_surface, pngtest2_surface, NULL, 0, 0 );

#ifdef USE_IMAGE_LUT8_FILE
        DFBCHECK(dfb->CreateImageProvider( dfb, DATADIR "/" LOGO_IMAGE_LUT8_FILE,
                                          &provider ));
        DFBCHECK(provider->GetSurfaceDescription( provider, &dsc_logo ));

        /* Create the test surface */
        dsc_logo.flags = DSDESC_WIDTH | DSDESC_HEIGHT | DSDESC_PIXELFORMAT;
        printf("%s: pixel format=0x%08x\n", LOGO_IMAGE_LUT8_FILE, dsc_logo.pixelformat);
        DFBCHECK(dfb->CreateSurface( dfb, &dsc_logo, &logo_surface ));

        DFBCHECK(provider->RenderTo( provider, logo_surface, NULL ));
        provider->Release(provider);
#else
        /* Create logo surface */
        DFBCHECK(dfb->CreateSurface( dfb, &biglogo_desc, &biglogo_surface ));
        memset(&dsc_logo, 0, sizeof(dsc_logo));
        dsc_logo.flags = DSDESC_WIDTH | DSDESC_HEIGHT | DSDESC_PIXELFORMAT;
        dsc_logo.width = biglogo_desc.width;
        dsc_logo.height = biglogo_desc.height;
        dsc_logo.pixelformat = biglogo_desc.pixelformat;
        DFBCHECK(dfb->CreateSurface( dfb, &dsc_logo, &logo_surface ));
        logo_surface->Blit( logo_surface, biglogo_surface, NULL, 0, 0 );
#endif
        palette->Release( palette );
    }

    /* Setup co-ords of where the logo and the test image will be placed within each sub-surface */
    logo_rect.x = (SUB_W - dsc_logo.width) / 4;
    logo_rect.y = SUB_H / 8;
    logo_rect.w = dsc_logo.width;
    logo_rect.h = dsc_logo.height;
    test_rect.x = logo_rect.x;
    test_rect.y = 20 + logo_rect.y;
    test_rect.w = dsc_test_surface.width;
    test_rect.h = dsc_test_surface.height;

    logo_surface->GetPixelFormat( logo_surface, &pixelformat );
    printf("logo_surface: pixel format=0x%08x\n", pixelformat);

    test_surface->GetPixelFormat( test_surface, &pixelformat );
    printf("test_surface: pixel format=0x%08x\n", pixelformat);

    /* load font */
    {
        DFBFontDescription desc;

        desc.flags = DFDESC_HEIGHT;
        desc.height = 24;

        DFBCHECK(dfb->CreateFont( dfb, FONT, &desc, &font ));
        DFBCHECK(font->GetHeight( font, &fontheight ));
        DFBCHECK(primary->SetFont( primary, font ));
    }

    /* Clear Primary Layer */
    DFBCHECK(primary->Clear( primary, BG_COLOR_R, BG_COLOR_G, BG_COLOR_B, BG_COLOR_A));
    DFBCHECK(primary->SetDrawingFlags (primary, DSDRAW_BLEND));
    DFBCHECK(primary->SetBlittingFlags( primary, DSBLIT_BLEND_ALPHACHANNEL ));
    DFBCHECK(primary->SetColor(primary, 0xff, 0xff, 0xff, 0xff));
    DFBCHECK(primary->DrawString(primary, "Hardware", -1,
                                 SUB1_X, SUB1_Y+SUB_H, DSTF_TOPLEFT));

    DFBCHECK(primary->SetColor(primary, 0xff, 0xff, 0xff, 0xff));
    DFBCHECK(primary->DrawString(primary, "Software", -1,
                                 SUB2_X, SUB2_Y+SUB_H, DSTF_TOPLEFT));

    DFBCHECK(primary->Flip(primary, NULL, DSFLIP_WAITFORSYNC));

     /* Create Top Test Number Surface */
    primary_info = create_context(primary, INFO_X, INFO_Y, INFO_W, INFO_H);
    DFBCHECK(primary_info->Clear( primary_info, BG_COLOR_R, BG_COLOR_G, BG_COLOR_B, BG_COLOR_A));
    DFBCHECK(primary_info->SetDrawingFlags (primary_info, DSDRAW_BLEND));
    DFBCHECK(primary_info->SetBlittingFlags( primary_info, DSBLIT_BLEND_ALPHACHANNEL ));
    DFBCHECK(primary_info->SetFont( primary_info, font ));
    DFBCHECK(primary->SetColor(primary, BOR_COLOR_R, BOR_COLOR_G, BOR_COLOR_B, BOR_COLOR_A));
    DFBCHECK(primary->DrawRectangle(primary, INFO_X-2, INFO_Y-2, INFO_W+4, INFO_H+4));

     /* Create Left Hardware Surface */
    primary_hardware = create_context(primary, SUB1_X, SUB1_Y, SUB_W, SUB_H);
    DFBCHECK(primary_hardware->Clear( primary_hardware, BG_COLOR_R, BG_COLOR_G, BG_COLOR_B, BG_COLOR_A));
    DFBCHECK(primary_hardware->SetDrawingFlags (primary_hardware, DSDRAW_BLEND));
    DFBCHECK(primary_hardware->SetBlittingFlags( primary_hardware, DSBLIT_BLEND_ALPHACHANNEL ));
    DFBCHECK(primary_hardware->DisableAcceleration(primary_hardware, DFXL_NONE)); 
    DFBCHECK(primary_hardware->SetFont( primary_hardware, font ));
    DFBCHECK(primary->SetColor(primary, BOR_COLOR_R, BOR_COLOR_G, BOR_COLOR_B, BOR_COLOR_A));
    DFBCHECK(primary->DrawRectangle(primary, SUB1_X-2, SUB1_Y-2, SUB_W+4, SUB_H+4));

     /* Create Right Software Surface */
    primary_software = create_context(primary, SUB2_X, SUB2_Y, SUB_W, SUB_H);
    DFBCHECK(primary_software->Clear( primary_software, BG_COLOR_R, BG_COLOR_G, BG_COLOR_B, BG_COLOR_A));
    DFBCHECK(primary_software->SetDrawingFlags (primary_software, DSDRAW_BLEND));
    DFBCHECK(primary_software->SetBlittingFlags( primary_software, DSBLIT_BLEND_ALPHACHANNEL ));
    DFBCHECK(primary_software->DisableAcceleration(primary_software, DFXL_ALL)); 
    DFBCHECK(primary_software->SetFont( primary_software, font ));
    DFBCHECK(primary->SetColor(primary, BOR_COLOR_R, BOR_COLOR_G, BOR_COLOR_B, BOR_COLOR_A));
    DFBCHECK(primary->DrawRectangle(primary, SUB2_X-2, SUB2_Y-2, SUB_W+4, SUB_H+4));

    /* Create Image Provider for rendering to background surface */
    DFBCHECK(dfb->CreateImageProvider( dfb, DATADIR"/fish.png",
									  &provider ));
    DFBCHECK(provider->GetSurfaceDescription( provider, &dsc_bg ));

    dsc_bg.flags = DSDESC_WIDTH | DSDESC_HEIGHT | DSDESC_PIXELFORMAT;

    /* Create background surface */
    DFBCHECK(dfb->CreateSurface( dfb, &dsc_bg, &background ));
    DFBCHECK(provider->RenderTo( provider, background, NULL ));
    provider->Release(provider);

    /* Setup Source Color Keying */
    //primary_hardware->SetSrcColorKey(primary_hardware, 0x00, 0x00, 0x00);
    //primary_software->SetSrcColorKey(primary_software, 0x00, 0x00, 0x00);

    /* Setup Source Color Keying on logo and test surfaces */
    logo_surface->SetSrcColorKey(logo_surface, 0x00, 0x00, 0x00);
    test_surface->SetSrcColorKey(test_surface, 0x00, 0x00, 0x00);
    test_surface->SetSrcColorKey(test_surface, 19, 9, 4);

    /* Setup Destination Color Keying */
    primary_hardware->SetDstColorKey(primary_hardware, 0xff, 0xff, 0xff);
    primary_software->SetDstColorKey(primary_software, 0xff, 0xff, 0xff);

    /* Test Blending with Drawing Flags = DSDRAW_BLEND */
    sprintf(buf, "Test %d: Fill Blending", test_number++);
    display_information(primary_info, buf);

    setup_fills(primary_hardware, primary_software, DSDRAW_BLEND, 0xFF, 0, 0, 77);

    primary_hardware->SetColor( primary_hardware, 0xFF, 0, 0, 77 );
    primary_hardware->FillRectangle( primary_hardware, 50, 150, 150, 150 );

    DFBCHECK(primary->Flip(primary, NULL, DSFLIP_WAITFORSYNC));

    primary_software->SetColor( primary_software, 0xFF, 0, 0, 77 );
    primary_software->FillRectangle( primary_software, 50, 150, 150, 150 );

    flip_context( primary_info, primary_hardware, primary_software );
    get_user_input();

    primary_hardware->SetBlittingFlags( primary_hardware, DSBLIT_BLEND_ALPHACHANNEL );
    primary_software->SetBlittingFlags( primary_software, DSBLIT_BLEND_ALPHACHANNEL );

    primary_hardware->SetDrawingFlags (primary_hardware, DSDRAW_BLEND);
    primary_software->SetDrawingFlags (primary_software, DSDRAW_BLEND);

    /* Test Porter-Duff Clear */
    sprintf(buf, "Test %d: Blended Fill with Porter-Duff Clear (DSPD_CLEAR)", test_number++);
    display_information(primary_info, buf);

    porter_duff_fills(primary_hardware, primary_software, DSPD_CLEAR);
    flip_context( primary_info, primary_hardware, primary_software );
    get_user_input();

    /* Test Porter-Duff Source */
    sprintf(buf, "Test %d: Blended Fill with Porter-Duff Source (DSPD_SRC)", test_number++);
    display_information(primary_info, buf);

    porter_duff_fills(primary_hardware, primary_software, DSPD_SRC);
    flip_context( primary_info, primary_hardware, primary_software );
    get_user_input();

    /* Test Porter-Duff Source Over */
    sprintf(buf, "Test %d: Blended Fill with Porter-Duff Source Over blend (DSPD_SRC_OVER)", test_number++);
    display_information(primary_info, buf);

    porter_duff_fills(primary_hardware, primary_software, DSPD_SRC_OVER);
    flip_context( primary_info, primary_hardware, primary_software );
    get_user_input();

    /* Test Porter-Duff Destination Over */
    sprintf(buf, "Test %d: Blended Fill with Porter-Duff Destination Over blend (DSPD_DST_OVER)", test_number++);
    display_information(primary_info, buf);

    porter_duff_fills(primary_hardware, primary_software, DSPD_DST_OVER);
    flip_context( primary_info, primary_hardware, primary_software );
    get_user_input();

    /* Test Porter-Duff Source In */
    sprintf(buf, "Test %d: Blended Fill with Porter-Duff Source In (DSPD_SRC_IN)", test_number++);
    display_information(primary_info, buf);

    porter_duff_fills(primary_hardware, primary_software, DSPD_SRC_IN);
    flip_context( primary_info, primary_hardware, primary_software );
    get_user_input();

    /* Test Porter-Duff Destination In */
    sprintf(buf, "Test %d: Blended Fill with Porter-Duff Destination In (DSPD_DST_IN)", test_number++);
    display_information(primary_info, buf);

    porter_duff_fills(primary_hardware, primary_software, DSPD_DST_IN);
    flip_context( primary_info, primary_hardware, primary_software );
    get_user_input();

    /* Test Porter-Duff Source Out */
    sprintf(buf, "Test %d: Blended Fill with Porter-Duff Source Out (DSPD_SRC_OUT)", test_number++);
    display_information(primary_info, buf);

    porter_duff_fills(primary_hardware, primary_software, DSPD_SRC_OUT);
    flip_context( primary_info, primary_hardware, primary_software );
    get_user_input();

    /* Test Porter-Duff Destination Out */
    sprintf(buf, "Test %d: Blended Fill with Porter-Duff Destination Out (DSPD_DST_OUT)", test_number++);
    display_information(primary_info, buf);

    porter_duff_fills(primary_hardware, primary_software, DSPD_DST_OUT);
    flip_context( primary_info, primary_hardware, primary_software );
    get_user_input();

    /* Test Porter-Duff Source ATOP */
    sprintf(buf, "Test %d: Blended Fill with Porter-Duff Source ATOP (DSPD_SRC_ATOP)", test_number++);
    display_information(primary_info, buf);

    porter_duff_fills(primary_hardware, primary_software, DSPD_SRC_ATOP);
    flip_context( primary_info, primary_hardware, primary_software );
    get_user_input();

    /* Test Porter-Duff Destination ATOP */
    sprintf(buf, "Test %d: Blended Fill with Porter-Duff Destination ATOP (DSPD_DST_ATOP)", test_number++);
    display_information(primary_info, buf);

    porter_duff_fills(primary_hardware, primary_software, DSPD_DST_ATOP);
    flip_context( primary_info, primary_hardware, primary_software );
    get_user_input();

    /* Test Porter-Duff Add */
    sprintf(buf, "Test %d: Blended Fill with Porter-Duff Add (DSPD_ADD)", test_number++);
    display_information(primary_info, buf);

    porter_duff_fills(primary_hardware, primary_software, DSPD_ADD);
    flip_context( primary_info, primary_hardware, primary_software );
    get_user_input();

    /* Test Porter-Duff Xor */
    sprintf(buf, "Test %d: Blended Fill with Porter-Duff XOR (DSPD_XOR)", test_number++);
    display_information(primary_info, buf);

    porter_duff_fills(primary_hardware, primary_software, DSPD_XOR);
    flip_context( primary_info, primary_hardware, primary_software );
    get_user_input();

    primary_hardware->SetDrawingFlags (primary_hardware, DSDRAW_BLEND|DSDRAW_SRC_PREMULTIPLY);
    primary_software->SetDrawingFlags (primary_software, DSDRAW_BLEND|DSDRAW_SRC_PREMULTIPLY);

    /* Test Porter-Duff Clear */
    sprintf(buf, "Test %d: Premultiplied Blended Fill with Porter-Duff Clear (DSPD_CLEAR)", test_number++);
    display_information(primary_info, buf);

    porter_duff_fills(primary_hardware, primary_software, DSPD_CLEAR);
    flip_context( primary_info, primary_hardware, primary_software );
    get_user_input();

    /* Test Porter-Duff Source */
    sprintf(buf, "Test %d: Premultiplied Blended Fill with Porter-Duff Source (DSPD_SRC)", test_number++);
    display_information(primary_info, buf);

    porter_duff_fills(primary_hardware, primary_software, DSPD_SRC);
    flip_context( primary_info, primary_hardware, primary_software );
    get_user_input();

    /* Test Porter-Duff Source Over */
    sprintf(buf, "Test %d: Premultiplied Blended Fill with Porter-Duff Source Over blend (DSPD_SRC_OVER)", test_number++);
    display_information(primary_info, buf);

    porter_duff_fills(primary_hardware, primary_software, DSPD_SRC_OVER);
    flip_context( primary_info, primary_hardware, primary_software );
    get_user_input();

    /* Test Porter-Duff Destination Over */
    sprintf(buf, "Test %d: Premultiplied Blended Fill with Porter-Duff Destination Over blend (DSPD_DST_OVER)", test_number++);
    display_information(primary_info, buf);

    porter_duff_fills(primary_hardware, primary_software, DSPD_DST_OVER);
    flip_context( primary_info, primary_hardware, primary_software );
    get_user_input();

    /* Test Porter-Duff Source In */
    sprintf(buf, "Test %d: Premultiplied Blended Fill with Porter-Duff Source In (DSPD_SRC_IN)", test_number++);
    display_information(primary_info, buf);

    porter_duff_fills(primary_hardware, primary_software, DSPD_SRC_IN);
    flip_context( primary_info, primary_hardware, primary_software );
    get_user_input();

    /* Test Porter-Duff Destination In */
    sprintf(buf, "Test %d: Premultiplied Blended Fill with Porter-Duff Destination In (DSPD_DST_IN)", test_number++);
    display_information(primary_info, buf);

    porter_duff_fills(primary_hardware, primary_software, DSPD_DST_IN);
    flip_context( primary_info, primary_hardware, primary_software );
    get_user_input();

    /* Test Porter-Duff Source Out */
    sprintf(buf, "Test %d: Premultiplied Blended Fill with Porter-Duff Source Out (DSPD_SRC_OUT)", test_number++);
    display_information(primary_info, buf);

    porter_duff_fills(primary_hardware, primary_software, DSPD_SRC_OUT);
    flip_context( primary_info, primary_hardware, primary_software );
    get_user_input();

    /* Test Porter-Duff Destination Out */
    sprintf(buf, "Test %d: Premultiplied Blended Fill with Porter-Duff Destination Out (DSPD_DST_OUT)", test_number++);
    display_information(primary_info, buf);

    porter_duff_fills(primary_hardware, primary_software, DSPD_DST_OUT);
    flip_context( primary_info, primary_hardware, primary_software );
    get_user_input();

    /* Test Porter-Duff Source ATOP */
    sprintf(buf, "Test %d: Premultiplied Blended Fill with Porter-Duff Source ATOP (DSPD_SRC_ATOP)", test_number++);
    display_information(primary_info, buf);

    porter_duff_fills(primary_hardware, primary_software, DSPD_SRC_ATOP);
    flip_context( primary_info, primary_hardware, primary_software );
    get_user_input();

    /* Test Porter-Duff Destination ATOP */
    sprintf(buf, "Test %d: Premultiplied Blended Fill with Porter-Duff Destination ATOP (DSPD_DST_ATOP)", test_number++);
    display_information(primary_info, buf);

    porter_duff_fills(primary_hardware, primary_software, DSPD_DST_ATOP);
    flip_context( primary_info, primary_hardware, primary_software );
    get_user_input();

    /* Test Porter-Duff Add */
    sprintf(buf, "Test %d: Premultiplied Blended Fill with Porter-Duff Add (DSPD_ADD)", test_number++);
    display_information(primary_info, buf);

    porter_duff_fills(primary_hardware, primary_software, DSPD_ADD);
    flip_context( primary_info, primary_hardware, primary_software );
    get_user_input();

    /* Test Porter-Duff Xor */
    sprintf(buf, "Test %d: Premultiplied Blended Fill with Porter-Duff XOR (DSPD_XOR)", test_number++);
    display_information(primary_info, buf);

    porter_duff_fills(primary_hardware, primary_software, DSPD_XOR);
    flip_context( primary_info, primary_hardware, primary_software );
    get_user_input();

    /* Cancel Porter-Duff */
    primary_hardware->SetPorterDuff(primary_hardware, DSPD_NONE);
    primary_software->SetPorterDuff(primary_software, DSPD_NONE);

    /* Test Normal Blit with no flags */
    sprintf(buf, "Test %d: Blit with no flags set", test_number++);
    display_information(primary_info, buf);

    setup_blits(primary_hardware, primary_software, DSBLIT_NOFX, 0x80);
    primary_hardware->Blit( primary_hardware, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_hardware->Blit( primary_hardware, test_surface, NULL, test_rect.x, test_rect.y );
    primary_software->Blit( primary_software, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_software->Blit( primary_software, test_surface, NULL, test_rect.x, test_rect.y );
    flip_context( primary_info, primary_hardware, primary_software );
    get_user_input();

     /* Test Flip Horizontally */
    sprintf(buf, "Test %d: Blit with Horizontal Flip", test_number++);
    display_information(primary_info, buf);

    setup_blits(primary_hardware, primary_software, DSBLIT_FLIP_HORIZONTAL, BLT_COLOR_A);
    primary_hardware->Blit( primary_hardware, logo_surface, NULL, logo_rect.x, logo_rect.y );
    primary_hardware->Blit( primary_hardware, test_surface, NULL,  test_rect.x, test_rect.y );
    primary_software->Blit( primary_software, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_software->Blit( primary_software, test_surface, NULL,  test_rect.x, test_rect.y );
    flip_context( primary_info, primary_hardware, primary_software );
    get_user_input();

    /* Test Flip Vertically */
    sprintf(buf, "Test %d: Blit with Vertical Flip", test_number++);
    display_information(primary_info, buf);

    setup_blits(primary_hardware, primary_software, DSBLIT_FLIP_VERTICAL, BLT_COLOR_A);
    primary_hardware->Blit( primary_hardware, logo_surface, NULL, logo_rect.x, logo_rect.y );
    primary_hardware->Blit( primary_hardware, test_surface, NULL,  test_rect.x, test_rect.y );
    primary_software->Blit( primary_software, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_software->Blit( primary_software, test_surface, NULL,  test_rect.x, test_rect.y );
    flip_context( primary_info, primary_hardware, primary_software );
    get_user_input();

     /* Test Rotate 180 */
    sprintf(buf, "Test %d: Blit with 180 Degrees Rotation", test_number++);
    display_information(primary_info, buf);

    setup_blits(primary_hardware, primary_software, DSBLIT_ROTATE180, BLT_COLOR_A);
    primary_hardware->Blit( primary_hardware, logo_surface, NULL, logo_rect.x, logo_rect.y );
    primary_hardware->Blit( primary_hardware, test_surface, NULL,  test_rect.x, test_rect.y );
    primary_software->Blit( primary_software, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_software->Blit( primary_software, test_surface, NULL,  test_rect.x, test_rect.y );
    flip_context( primary_info, primary_hardware, primary_software );
    get_user_input();

     /* Test Flip Horizontally with Alpha Channel Blending */
    sprintf(buf, "Test %d: Blit with Horizontal Flip and Alpha Channel Blending", test_number++);
    display_information(primary_info, buf);

    setup_blits(primary_hardware, primary_software, DSBLIT_FLIP_HORIZONTAL | DSBLIT_BLEND_ALPHACHANNEL, BLT_COLOR_A);
    primary_hardware->Blit( primary_hardware, logo_surface, NULL, logo_rect.x, logo_rect.y );
    primary_hardware->Blit( primary_hardware, test_surface, NULL,  test_rect.x, test_rect.y );
    primary_software->Blit( primary_software, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_software->Blit( primary_software, test_surface, NULL,  test_rect.x, test_rect.y );
    flip_context( primary_info, primary_hardware, primary_software );
    get_user_input();

     /* Test Flip Vertically with Alpha Channel Blending */
    sprintf(buf, "Test %d: Blit with Vertical Flip and Alpha Channel Blending", test_number++);
    display_information(primary_info, buf);
    
    setup_blits(primary_hardware, primary_software, DSBLIT_FLIP_VERTICAL | DSBLIT_BLEND_ALPHACHANNEL, BLT_COLOR_A);
    primary_hardware->Blit( primary_hardware, logo_surface, NULL, logo_rect.x, logo_rect.y );
    primary_hardware->Blit( primary_hardware, test_surface, NULL,  test_rect.x, test_rect.y );
    primary_software->Blit( primary_software, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_software->Blit( primary_software, test_surface, NULL,  test_rect.x, test_rect.y );
    flip_context( primary_info, primary_hardware, primary_software );
    get_user_input();

     /* Test Rotate 180 with Alpha Channel Blending */
    sprintf(buf, "Test %d: Blit with 180 Degrees Rotation and Alpha Channel Blending", test_number++);
    display_information(primary_info, buf);
    
    setup_blits(primary_hardware, primary_software, DSBLIT_ROTATE180 | DSBLIT_BLEND_ALPHACHANNEL, BLT_COLOR_A);
    primary_hardware->Blit( primary_hardware, logo_surface, NULL, logo_rect.x, logo_rect.y );
    primary_hardware->Blit( primary_hardware, test_surface, NULL,  test_rect.x, test_rect.y );
    primary_software->Blit( primary_software, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_software->Blit( primary_software, test_surface, NULL,  test_rect.x, test_rect.y );
    flip_context( primary_info, primary_hardware, primary_software );
    get_user_input();

     /* Test XOR */
    sprintf(buf, "Test %d: Blit with XOR", test_number++);
    display_information(primary_info, buf);
    
    setup_blits(primary_hardware, primary_software, DSBLIT_XOR, BLT_COLOR_A);
    primary_hardware->Blit( primary_hardware, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_hardware->Blit( primary_hardware, test_surface, NULL, test_rect.x, test_rect.y );
    primary_software->Blit( primary_software, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_software->Blit( primary_software, test_surface, NULL, test_rect.x, test_rect.y );
    flip_context( primary_info, primary_hardware, primary_software );
    get_user_input();

     /* Test XOR with Alpha Channel Blending */
    sprintf(buf, "Test %d: Blit with XOR and Alpha Channel Blending", test_number++);
    display_information(primary_info, buf);
    
    setup_blits(primary_hardware, primary_software, DSBLIT_XOR | DSBLIT_BLEND_ALPHACHANNEL, 0x80);
    primary_hardware->Blit( primary_hardware, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_hardware->Blit( primary_hardware, test_surface, NULL, test_rect.x, test_rect.y );
    primary_software->Blit( primary_software, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_software->Blit( primary_software, test_surface, NULL, test_rect.x, test_rect.y );
    flip_context( primary_info, primary_hardware, primary_software );
    get_user_input();

     /* Test XOR with Color Alpha Blending */
    sprintf(buf, "Test %d: Blit with XOR and Color Alpha Blending", test_number++);
    display_information(primary_info, buf);
    
    setup_blits(primary_hardware, primary_software, DSBLIT_XOR | DSBLIT_BLEND_COLORALPHA, 0x80);
    primary_hardware->Blit( primary_hardware, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_hardware->Blit( primary_hardware, test_surface, NULL, test_rect.x, test_rect.y );
    primary_software->Blit( primary_software, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_software->Blit( primary_software, test_surface, NULL, test_rect.x, test_rect.y );
    flip_context( primary_info, primary_hardware, primary_software );
    get_user_input();
    
     /* Test XOR with Alpha Channel and Color Alpha Blending */
    sprintf(buf, "Test %d: Blit with XOR and Alpha Channel + Color Alpha Blending", test_number++);
    display_information(primary_info, buf);
    
    setup_blits(primary_hardware, primary_software, DSBLIT_XOR | DSBLIT_BLEND_ALPHACHANNEL | DSBLIT_BLEND_COLORALPHA, 0x80);
    primary_hardware->Blit( primary_hardware, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_hardware->Blit( primary_hardware, test_surface, NULL, test_rect.x, test_rect.y );
    primary_software->Blit( primary_software, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_software->Blit( primary_software, test_surface, NULL, test_rect.x, test_rect.y );
    flip_context( primary_info, primary_hardware, primary_software );
    get_user_input();

     /* Test XOR with Premultiply and Alpha Channel and Color Alpha Blending */
    sprintf(buf, "Test %d: Blit with XOR and Premultiply + Alpha Channel + Color Alpha Blending", test_number++);
    display_information(primary_info, buf);
    
    setup_blits(primary_hardware, primary_software, DSBLIT_XOR | DSBLIT_BLEND_ALPHACHANNEL | DSBLIT_BLEND_COLORALPHA | DSBLIT_SRC_PREMULTIPLY, 0x80);
    primary_hardware->Blit( primary_hardware, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_hardware->Blit( primary_hardware, test_surface, NULL, test_rect.x, test_rect.y );
    primary_software->Blit( primary_software, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_software->Blit( primary_software, test_surface, NULL, test_rect.x, test_rect.y );
    flip_context( primary_info, primary_hardware, primary_software );
    get_user_input();

     /* Test XOR with Premultiply and Alpha Channel and Color Alpha Blending and PremultColor */
    sprintf(buf, "Test %d: Blit with XOR + Premultiply + Alpha Channel + Color Alpha + PREMULTCOLOR Blending", test_number++);
    display_information(primary_info, buf);
    
    setup_blits(primary_hardware, primary_software, DSBLIT_XOR | DSBLIT_BLEND_ALPHACHANNEL | DSBLIT_BLEND_COLORALPHA |
                                                    DSBLIT_SRC_PREMULTIPLY | DSBLIT_SRC_PREMULTCOLOR, 0x80);
    primary_hardware->Blit( primary_hardware, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_hardware->Blit( primary_hardware, test_surface, NULL, test_rect.x, test_rect.y );
    primary_software->Blit( primary_software, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_software->Blit( primary_software, test_surface, NULL, test_rect.x, test_rect.y );
    flip_context( primary_info, primary_hardware, primary_software );
    get_user_input();

     /* Test XOR with Premultiply and Alpha Channel and Color Alpha Blending and PremultColor and Colorize */
    sprintf(buf, "Test %d: Blit with XOR + Premultiply + Alpha Channel + Color Alpha + PREMULTCOLOR + Colorize Blending", test_number++);
    display_information(primary_info, buf);
    
    setup_blits(primary_hardware, primary_software, DSBLIT_XOR | DSBLIT_BLEND_ALPHACHANNEL | DSBLIT_BLEND_COLORALPHA |
                                                    DSBLIT_SRC_PREMULTIPLY | DSBLIT_SRC_PREMULTCOLOR | DSBLIT_COLORIZE, 0x80);
    primary_hardware->Blit( primary_hardware, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_hardware->Blit( primary_hardware, test_surface, NULL, test_rect.x, test_rect.y );
    primary_software->Blit( primary_software, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_software->Blit( primary_software, test_surface, NULL, test_rect.x, test_rect.y );
    flip_context( primary_info, primary_hardware, primary_software );
    get_user_input();

     /* Test Alpha Channel Blending */
    sprintf(buf, "Test %d: Blit with Alpha Channel Blending", test_number++);
    display_information(primary_info, buf);
    
    setup_blits(primary_hardware, primary_software, DSBLIT_BLEND_ALPHACHANNEL | 0//DSBLIT_SRC_COLORKEY
            , BLT_COLOR_A);
    primary_hardware->Blit( primary_hardware, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_hardware->Blit( primary_hardware, test_surface, NULL, test_rect.x, test_rect.y );
    primary_software->Blit( primary_software, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_software->Blit( primary_software, test_surface, NULL, test_rect.x, test_rect.y );
    flip_context( primary_info, primary_hardware, primary_software );
    get_user_input();

     /* Test 100% Color Alpha Blending */
    sprintf(buf, "Test %d: Blit with 100%% Color Alpha", test_number++);
    display_information(primary_info, buf);
    
    setup_blits(primary_hardware, primary_software, DSBLIT_BLEND_COLORALPHA | 0//DSBLIT_SRC_COLORKEY
            , BLT_COLOR_A);
    primary_hardware->Blit( primary_hardware, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_hardware->Blit( primary_hardware, test_surface, NULL, test_rect.x, test_rect.y );
    primary_software->Blit( primary_software, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_software->Blit( primary_software, test_surface, NULL, test_rect.x, test_rect.y );
    flip_context( primary_info, primary_hardware, primary_software );
    get_user_input();

     /* Test 50% Color Alpha Blending */
    sprintf(buf, "Test %d: Blit with 50%% Color Alpha", test_number++);
    display_information(primary_info, buf);
    
    setup_blits(primary_hardware, primary_software, DSBLIT_BLEND_COLORALPHA | 0//DSBLIT_SRC_COLORKEY
            , 0x80);
    primary_hardware->Blit( primary_hardware, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_hardware->Blit( primary_hardware, test_surface, NULL, test_rect.x, test_rect.y );
    primary_software->Blit( primary_software, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_software->Blit( primary_software, test_surface, NULL, test_rect.x, test_rect.y );
    flip_context( primary_info, primary_hardware, primary_software );
    get_user_input();

     /* Test 25% Color Alpha Blending */
    sprintf(buf, "Test %d: Blit with 25%% Color Alpha", test_number++);
    display_information(primary_info, buf);
    
    setup_blits(primary_hardware, primary_software, DSBLIT_BLEND_COLORALPHA | 0//DSBLIT_SRC_COLORKEY
            , 0x40);
    primary_hardware->Blit( primary_hardware, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_hardware->Blit( primary_hardware, test_surface, NULL, test_rect.x, test_rect.y );
    primary_software->Blit( primary_software, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_software->Blit( primary_software, test_surface, NULL, test_rect.x, test_rect.y );
    flip_context( primary_info, primary_hardware, primary_software );
    get_user_input();

    /* Test 100% Color Alpha + Alpha Channel Blending */
    sprintf(buf, "Test %d: Blit with 100%% Color Alpha + Alpha Blend", test_number++);
    display_information(primary_info, buf);
    
    setup_blits(primary_hardware, primary_software, DSBLIT_BLEND_COLORALPHA|DSBLIT_BLEND_ALPHACHANNEL|0//DSBLIT_SRC_COLORKEY
            , BLT_COLOR_A);
    primary_hardware->Blit( primary_hardware, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_hardware->Blit( primary_hardware, test_surface, NULL, test_rect.x, test_rect.y );
    primary_software->Blit( primary_software, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_software->Blit( primary_software, test_surface, NULL, test_rect.x, test_rect.y );
    flip_context( primary_info, primary_hardware, primary_software );
    get_user_input();

    /* Test 50% Color Alpha + Alpha Channel Blending */
    sprintf(buf, "Test %d: Blit with 50%% Color Alpha + Alpha Blend", test_number++);
    display_information(primary_info, buf);
    
    setup_blits(primary_hardware, primary_software, DSBLIT_BLEND_COLORALPHA|DSBLIT_BLEND_ALPHACHANNEL|0//DSBLIT_SRC_COLORKEY
            , 0x80);
    primary_hardware->Blit( primary_hardware, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_hardware->Blit( primary_hardware, test_surface, NULL, test_rect.x, test_rect.y );
    primary_software->Blit( primary_software, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_software->Blit( primary_software, test_surface, NULL, test_rect.x, test_rect.y );
    flip_context( primary_info, primary_hardware, primary_software );
    get_user_input();

    /* Test 25% Color Alpha + Alpha Channel Blending */
    sprintf(buf, "Test %d: Blit with 25%% Color Alpha + Alpha Blend", test_number++);
    display_information(primary_info, buf);
    
    setup_blits(primary_hardware, primary_software, DSBLIT_BLEND_COLORALPHA|DSBLIT_BLEND_ALPHACHANNEL|0//DSBLIT_SRC_COLORKEY
            , 0x40);
    primary_hardware->Blit( primary_hardware, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_hardware->Blit( primary_hardware, test_surface, NULL, test_rect.x, test_rect.y );
    primary_software->Blit( primary_software, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_software->Blit( primary_software, test_surface, NULL, test_rect.x, test_rect.y );
    flip_context( primary_info, primary_hardware, primary_software );
    get_user_input();

    /* Test PREMULTIPLY */
    sprintf(buf, "Test %d: Blit with Premultiply Source", test_number++);
    display_information(primary_info, buf);
    
    setup_blits(primary_hardware, primary_software, DSBLIT_SRC_PREMULTIPLY|0//DSBLIT_SRC_COLORKEY
            , 0x80);
    primary_hardware->Blit( primary_hardware, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_hardware->Blit( primary_hardware, test_surface, NULL, test_rect.x, test_rect.y );
    primary_software->Blit( primary_software, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_software->Blit( primary_software, test_surface, NULL, test_rect.x, test_rect.y );
    flip_context( primary_info, primary_hardware, primary_software );
    get_user_input();

     /* Test 100% Color Alpha Premultipled Source Blending */
    sprintf(buf, "Test %d: Blit with 100%% Color Alpha Premultiplied Source", test_number++);
    display_information(primary_info, buf);
    
    setup_blits(primary_hardware, primary_software, DSBLIT_BLEND_COLORALPHA | DSBLIT_SRC_PREMULTIPLY |0//DSBLIT_SRC_COLORKEY
            , BLT_COLOR_A);
    primary_hardware->Blit( primary_hardware, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_hardware->Blit( primary_hardware, test_surface, NULL, test_rect.x, test_rect.y );
    primary_software->Blit( primary_software, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_software->Blit( primary_software, test_surface, NULL, test_rect.x, test_rect.y );
    flip_context( primary_info, primary_hardware, primary_software );
    get_user_input();

     /* Test 50% Color Alpha Premultiplied Source Blending */
    sprintf(buf, "Test %d: Blit with 50%% Color Alpha Premultiplied Source", test_number++);
    display_information(primary_info, buf);
    
    setup_blits(primary_hardware, primary_software, DSBLIT_BLEND_COLORALPHA | DSBLIT_SRC_PREMULTIPLY | 0//DSBLIT_SRC_COLORKEY
            , 0x80);
    primary_hardware->Blit( primary_hardware, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_hardware->Blit( primary_hardware, test_surface, NULL, test_rect.x, test_rect.y );
    primary_software->Blit( primary_software, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_software->Blit( primary_software, test_surface, NULL, test_rect.x, test_rect.y );
    flip_context( primary_info, primary_hardware, primary_software );
    get_user_input();

     /* Test 25% Color Alpha Premultipled Source Blending */
    sprintf(buf, "Test %d: Blit with 25%% Color Alpha Premultipled Source ", test_number++);
    display_information(primary_info, buf);
    
    setup_blits(primary_hardware, primary_software, DSBLIT_BLEND_COLORALPHA | DSBLIT_SRC_PREMULTIPLY | 0//DSBLIT_SRC_COLORKEY
            , 0x40);
    primary_hardware->Blit( primary_hardware, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_hardware->Blit( primary_hardware, test_surface, NULL, test_rect.x, test_rect.y );
    primary_software->Blit( primary_software, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_software->Blit( primary_software, test_surface, NULL, test_rect.x, test_rect.y );
    flip_context( primary_info, primary_hardware, primary_software );
    get_user_input();

    /* Test Normal Blit with no flags */
    sprintf(buf, "Test %d: Blit with no flags set", test_number++);
    display_information(primary_info, buf);
    
    setup_blits(primary_hardware, primary_software, DSBLIT_NOFX, 0x80);
    primary_hardware->Blit( primary_hardware, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_hardware->Blit( primary_hardware, test_surface, NULL, test_rect.x, test_rect.y );
    primary_software->Blit( primary_software, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_software->Blit( primary_software, test_surface, NULL, test_rect.x, test_rect.y );
    flip_context( primary_info, primary_hardware, primary_software );
    get_user_input();

    /* Test Alpha Channel + PREMULTIPLY Blending */
    sprintf(buf, "Test %d: Blit with Premultiply Source Alpha Blending", test_number++);
    display_information(primary_info, buf);
    
    setup_blits(primary_hardware, primary_software, DSBLIT_BLEND_ALPHACHANNEL | DSBLIT_SRC_PREMULTIPLY|0//DSBLIT_SRC_COLORKEY
            , 0x80);
    primary_hardware->Blit( primary_hardware, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_hardware->Blit( primary_hardware, test_surface, NULL, test_rect.x, test_rect.y );
    primary_software->Blit( primary_software, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_software->Blit( primary_software, test_surface, NULL, test_rect.x, test_rect.y );
    flip_context( primary_info, primary_hardware, primary_software );
    get_user_input();

     /* Test 100% Alpha Color + Alpha Channel + PREMULTIPLY Blending */
    sprintf(buf, "Test %d: Blit with Premultiply Source Alpha + 100%% Alpha Color blending", test_number++);
    display_information(primary_info, buf);
    
    setup_blits(primary_hardware, primary_software, DSBLIT_BLEND_ALPHACHANNEL | DSBLIT_BLEND_COLORALPHA | DSBLIT_SRC_PREMULTIPLY | 0//DSBLIT_SRC_COLORKEY
            , BLT_COLOR_A);
    primary_hardware->Blit( primary_hardware, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_hardware->Blit( primary_hardware, test_surface, NULL, test_rect.x, test_rect.y );
    primary_software->Blit( primary_software, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_software->Blit( primary_software, test_surface, NULL, test_rect.x, test_rect.y );
    flip_context( primary_info, primary_hardware, primary_software );
    get_user_input();

     /* Test 50% Alpha Color + Alpha Channel + PREMULTIPLY Blending */
    sprintf(buf, "Test %d: Blit with Premultiply Source Alpha + 50%% Alpha Color blending", test_number++);
    display_information(primary_info, buf);
    
    setup_blits(primary_hardware, primary_software, DSBLIT_BLEND_ALPHACHANNEL | DSBLIT_BLEND_COLORALPHA | DSBLIT_SRC_PREMULTIPLY, 0x80);
    primary_hardware->Blit( primary_hardware, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_hardware->Blit( primary_hardware, test_surface, NULL, test_rect.x, test_rect.y );
    primary_software->Blit( primary_software, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_software->Blit( primary_software, test_surface, NULL, test_rect.x, test_rect.y );
    flip_context( primary_info, primary_hardware, primary_software );
    get_user_input();

     /* Test 25% Alpha Color + Alpha Channel + PREMULTIPLY Blending */
    sprintf(buf, "Test %d: Blit with Premultiply Source Alpha + 25%% Alpha Color blending", test_number++);
    display_information(primary_info, buf);
    
    setup_blits(primary_hardware, primary_software, DSBLIT_BLEND_ALPHACHANNEL | DSBLIT_BLEND_COLORALPHA | DSBLIT_SRC_PREMULTIPLY, 0x40);
    primary_hardware->Blit( primary_hardware, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_hardware->Blit( primary_hardware, test_surface, NULL, test_rect.x, test_rect.y );
    primary_software->Blit( primary_software, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_software->Blit( primary_software, test_surface, NULL, test_rect.x, test_rect.y );
    flip_context( primary_info, primary_hardware, primary_software );
    get_user_input();

    /* Test 100% PREMULTCOLOR */
    sprintf(buf, "Test %d: Blit with 100%% PREMULTCOLOR on the Source", test_number++);
    display_information(primary_info, buf);
    
    setup_blits(primary_hardware, primary_software, DSBLIT_SRC_PREMULTCOLOR|0//DSBLIT_SRC_COLORKEY
            , BLT_COLOR_A);
    primary_hardware->Blit( primary_hardware, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_hardware->Blit( primary_hardware, test_surface, NULL, test_rect.x, test_rect.y );
    primary_software->Blit( primary_software, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_software->Blit( primary_software, test_surface, NULL, test_rect.x, test_rect.y );
    flip_context( primary_info, primary_hardware, primary_software );
    get_user_input();

    /* Test 50% PREMULTCOLOR */
    sprintf(buf, "Test %d: Blit with 50%% PREMULTCOLOR on the Source", test_number++);
    display_information(primary_info, buf);
    
    setup_blits(primary_hardware, primary_software, DSBLIT_SRC_PREMULTCOLOR|0//DSBLIT_SRC_COLORKEY
            , 0x80);
    primary_hardware->Blit( primary_hardware, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_hardware->Blit( primary_hardware, test_surface, NULL, test_rect.x, test_rect.y );
    primary_software->Blit( primary_software, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_software->Blit( primary_software, test_surface, NULL, test_rect.x, test_rect.y );
    flip_context( primary_info, primary_hardware, primary_software );
    get_user_input();

    /* Test 25% PREMULTCOLOR */
    sprintf(buf, "Test %d: Blit with 25%% PREMULTCOLOR on the Source", test_number++);
    display_information(primary_info, buf);
    
    setup_blits(primary_hardware, primary_software, DSBLIT_SRC_PREMULTCOLOR, 0x40);
    primary_hardware->Blit( primary_hardware, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_hardware->Blit( primary_hardware, test_surface, NULL, test_rect.x, test_rect.y );
    primary_software->Blit( primary_software, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_software->Blit( primary_software, test_surface, NULL, test_rect.x, test_rect.y );
    flip_context( primary_info, primary_hardware, primary_software );
    get_user_input();

    /* Test 100% Alpha Color + Alpha Channel + 100% PREMULTCOLOR Blending */
    sprintf(buf, "Test %d: Blit with Source Alpha + 100%% Alpha Color + 100%% PREMULTCOLOR", test_number++);
    display_information(primary_info, buf);
    
    setup_blits(primary_hardware, primary_software, DSBLIT_BLEND_ALPHACHANNEL | DSBLIT_BLEND_COLORALPHA | DSBLIT_SRC_PREMULTCOLOR, BLT_COLOR_A);
    primary_hardware->Blit( primary_hardware, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_hardware->Blit( primary_hardware, test_surface, NULL, test_rect.x, test_rect.y );
    primary_software->Blit( primary_software, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_software->Blit( primary_software, test_surface, NULL, test_rect.x, test_rect.y );
    flip_context( primary_info, primary_hardware, primary_software );
    get_user_input();

    /* Test 50% Alpha Color + Alpha Channel + 50% PREMULTCOLOR Blending */
    sprintf(buf, "Test %d: Blit with Source Alpha + 50%% Alpha Color + 50%% PREMULTCOLOR", test_number++);
    display_information(primary_info, buf);
    
    setup_blits(primary_hardware, primary_software, DSBLIT_BLEND_ALPHACHANNEL | DSBLIT_BLEND_COLORALPHA | DSBLIT_SRC_PREMULTCOLOR, 0x80);
    primary_hardware->Blit( primary_hardware, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_hardware->Blit( primary_hardware, test_surface, NULL, test_rect.x, test_rect.y );
    primary_software->Blit( primary_software, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_software->Blit( primary_software, test_surface, NULL, test_rect.x, test_rect.y );
    flip_context( primary_info, primary_hardware, primary_software );
    get_user_input();

    /* Test 25% Alpha Color + Alpha Channel + 25% PREMULTCOLOR Blending */
    sprintf(buf, "Test %d: Blit with Source Alpha + 25%% Alpha Color + 25%% PREMULTCOLOR", test_number++);
    display_information(primary_info, buf);
    
    setup_blits(primary_hardware, primary_software, DSBLIT_BLEND_ALPHACHANNEL | DSBLIT_BLEND_COLORALPHA | DSBLIT_SRC_PREMULTCOLOR, 0x40);
    primary_hardware->Blit( primary_hardware, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_hardware->Blit( primary_hardware, test_surface, NULL, test_rect.x, test_rect.y );
    primary_software->Blit( primary_software, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_software->Blit( primary_software, test_surface, NULL, test_rect.x, test_rect.y );
    flip_context( primary_info, primary_hardware, primary_software );
    get_user_input();
    
    /* Test 100% Alpha Color + Alpha Channel + PREMULTIPLY + 100% PREMULTCOLOR Blending */
    sprintf(buf, "Test %d: Blit with Premultiply Source Alpha + 100%% Alpha Color + 100%% PREMULTCOLOR", test_number++);
    display_information(primary_info, buf);
    
    setup_blits(primary_hardware, primary_software, DSBLIT_BLEND_ALPHACHANNEL | DSBLIT_BLEND_COLORALPHA | DSBLIT_SRC_PREMULTIPLY | DSBLIT_SRC_PREMULTCOLOR,
            BLT_COLOR_A);
    primary_hardware->Blit( primary_hardware, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_hardware->Blit( primary_hardware, test_surface, NULL, test_rect.x, test_rect.y );
    primary_software->Blit( primary_software, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_software->Blit( primary_software, test_surface, NULL, test_rect.x, test_rect.y );
    flip_context( primary_info, primary_hardware, primary_software );
    get_user_input();

    /* Test 50% Alpha Color + Alpha Channel + PREMULTIPLY + 50% PREMULTCOLOR Blending */
    sprintf(buf, "Test %d: Blit with Premultiply Source Alpha + 50%% Alpha Color + 50%% PREMULTCOLOR", test_number++);
    display_information(primary_info, buf);
    
    setup_blits(primary_hardware, primary_software, DSBLIT_BLEND_ALPHACHANNEL | DSBLIT_BLEND_COLORALPHA | DSBLIT_SRC_PREMULTIPLY | DSBLIT_SRC_PREMULTCOLOR,
            0x80);
    primary_hardware->Blit( primary_hardware, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_hardware->Blit( primary_hardware, test_surface, NULL, test_rect.x, test_rect.y );
    primary_software->Blit( primary_software, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_software->Blit( primary_software, test_surface, NULL, test_rect.x, test_rect.y );
    flip_context( primary_info, primary_hardware, primary_software );
    get_user_input();

    /* Test 25% Alpha Color + Alpha Channel + PREMULTIPLY + 25% PREMULTCOLOR Blending */
    sprintf(buf, "Test %d: Blit with Premultiply Source Alpha + 25%% Alpha Color + 25%% PREMULTCOLOR", test_number++);
    display_information(primary_info, buf);
    
    setup_blits(primary_hardware, primary_software, DSBLIT_BLEND_ALPHACHANNEL | DSBLIT_BLEND_COLORALPHA | DSBLIT_SRC_PREMULTIPLY | DSBLIT_SRC_PREMULTCOLOR,
            0x40);
    primary_hardware->Blit( primary_hardware, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_hardware->Blit( primary_hardware, test_surface, NULL, test_rect.x, test_rect.y );
    primary_software->Blit( primary_software, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_software->Blit( primary_software, test_surface, NULL, test_rect.x, test_rect.y );
    flip_context( primary_info, primary_hardware, primary_software );
    get_user_input();

    /* Test Alpha Channel Blending with Colorize */
    sprintf(buf, "Test %d: Blit with Source Alpha Blending with Colorize", test_number++);
    display_information(primary_info, buf);
    
    setup_blits(primary_hardware, primary_software, DSBLIT_BLEND_ALPHACHANNEL | DSBLIT_COLORIZE, 0x80);
    primary_hardware->Blit( primary_hardware, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_hardware->Blit( primary_hardware, test_surface, NULL, test_rect.x, test_rect.y );
    primary_software->Blit( primary_software, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_software->Blit( primary_software, test_surface, NULL, test_rect.x, test_rect.y );
    flip_context( primary_info, primary_hardware, primary_software );
    get_user_input();
    
    /* Test 100% Color Alpha Blending with Colorize */
    sprintf(buf, "Test %d: Blit with 100%% Color Alpha Blending with Colorize", test_number++);
    display_information(primary_info, buf);
    
    setup_blits(primary_hardware, primary_software, DSBLIT_BLEND_COLORALPHA | DSBLIT_COLORIZE, BLT_COLOR_A);
    primary_hardware->Blit( primary_hardware, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_hardware->Blit( primary_hardware, test_surface, NULL, test_rect.x, test_rect.y );
    primary_software->Blit( primary_software, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_software->Blit( primary_software, test_surface, NULL, test_rect.x, test_rect.y );
    flip_context( primary_info, primary_hardware, primary_software );
    get_user_input();

    /* Test 50% Color Alpha Blending with Colorize */
    sprintf(buf, "Test %d: Blit with 50%% Color Alpha Blending with Colorize", test_number++);
    display_information(primary_info, buf);
    
    setup_blits(primary_hardware, primary_software, DSBLIT_BLEND_COLORALPHA | DSBLIT_COLORIZE, 0x80);
    primary_hardware->Blit( primary_hardware, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_hardware->Blit( primary_hardware, test_surface, NULL, test_rect.x, test_rect.y );
    primary_software->Blit( primary_software, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_software->Blit( primary_software, test_surface, NULL, test_rect.x, test_rect.y );
    flip_context( primary_info, primary_hardware, primary_software );
    get_user_input();

    /* Test 25% Color Alpha Blending with Colorize */
    sprintf(buf, "Test %d: Blit with 25%% Color Alpha Blending with Colorize", test_number++);
    display_information(primary_info, buf);
    
    setup_blits(primary_hardware, primary_software, DSBLIT_BLEND_COLORALPHA | DSBLIT_COLORIZE, 0x40);
    primary_hardware->Blit( primary_hardware, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_hardware->Blit( primary_hardware, test_surface, NULL, test_rect.x, test_rect.y );
    primary_software->Blit( primary_software, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_software->Blit( primary_software, test_surface, NULL, test_rect.x, test_rect.y );
    flip_context( primary_info, primary_hardware, primary_software );
    get_user_input();

    /* Test 100% Color Alpha + Alpha Channel Blending + Colorize */
    sprintf(buf, "Test %d: Blit with 100%% Color Alpha + Source Alpha Blending + Colorize", test_number++);
    display_information(primary_info, buf);
    
    setup_blits(primary_hardware, primary_software, DSBLIT_BLEND_COLORALPHA | DSBLIT_BLEND_ALPHACHANNEL | DSBLIT_COLORIZE, BLT_COLOR_A);
    primary_hardware->Blit( primary_hardware, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_hardware->Blit( primary_hardware, test_surface, NULL, test_rect.x, test_rect.y );
    primary_software->Blit( primary_software, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_software->Blit( primary_software, test_surface, NULL, test_rect.x, test_rect.y );
    flip_context( primary_info, primary_hardware, primary_software );
    get_user_input();

    /* Test 50% Color Alpha + Alpha Channel Blending + Colorize */
    sprintf(buf, "Test %d: Blit with 50%% Color Alpha + Source Alpha Blending + Colorize", test_number++);
    display_information(primary_info, buf);
    
    setup_blits(primary_hardware, primary_software, DSBLIT_BLEND_COLORALPHA | DSBLIT_BLEND_ALPHACHANNEL | DSBLIT_COLORIZE, 0x80);
    primary_hardware->Blit( primary_hardware, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_hardware->Blit( primary_hardware, test_surface, NULL, test_rect.x, test_rect.y );
    primary_software->Blit( primary_software, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_software->Blit( primary_software, test_surface, NULL, test_rect.x, test_rect.y );
    flip_context( primary_info, primary_hardware, primary_software );
    get_user_input();

    /* Test 25% Color Alpha + Alpha Channel Blending + Colorize */
    sprintf(buf, "Test %d: Blit with 25%% Color Alpha + Source Alpha Blending + Colorize", test_number++);
    display_information(primary_info, buf);
    
    setup_blits(primary_hardware, primary_software, DSBLIT_BLEND_COLORALPHA | DSBLIT_BLEND_ALPHACHANNEL | DSBLIT_COLORIZE, 0x40);
    primary_hardware->Blit( primary_hardware, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_hardware->Blit( primary_hardware, test_surface, NULL, test_rect.x, test_rect.y );
    primary_software->Blit( primary_software, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_software->Blit( primary_software, test_surface, NULL, test_rect.x, test_rect.y );
    flip_context( primary_info, primary_hardware, primary_software );
    get_user_input();

    /* Test PREMULTIPLY with Colorize */
    sprintf(buf, "Test %d: Blit with Premultiply Source with Colorize", test_number++);
    display_information(primary_info, buf);
    
    setup_blits(primary_hardware, primary_software, DSBLIT_SRC_PREMULTIPLY | DSBLIT_COLORIZE, 0x80);
    primary_hardware->Blit( primary_hardware, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_hardware->Blit( primary_hardware, test_surface, NULL, test_rect.x, test_rect.y );
    primary_software->Blit( primary_software, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_software->Blit( primary_software, test_surface, NULL, test_rect.x, test_rect.y );
    flip_context( primary_info, primary_hardware, primary_software );
    get_user_input();

    /* Test Alpha Channel + PREMULTIPLY Blending + Colorize */
    sprintf(buf, "Test %d: Blit with Premultiply Source Alpha Blending with Colorize", test_number++);
    display_information(primary_info, buf);
    
    setup_blits(primary_hardware, primary_software,  DSBLIT_BLEND_ALPHACHANNEL | DSBLIT_SRC_PREMULTIPLY | DSBLIT_COLORIZE, 0x80);
    primary_hardware->Blit( primary_hardware, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_hardware->Blit( primary_hardware, test_surface, NULL, test_rect.x, test_rect.y );
    primary_software->Blit( primary_software, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_software->Blit( primary_software, test_surface, NULL, test_rect.x, test_rect.y );
    flip_context( primary_info, primary_hardware, primary_software );
    get_user_input();

    /* Test 100% Alpha Color + Alpha Channel + PREMULTIPLY Blending + Colorize */
    sprintf(buf, "Test %d: Blit with Premultiply Source Alpha + 100%% Alpha Color blending + Colorize", test_number++);
    display_information(primary_info, buf);
    
    setup_blits(primary_hardware, primary_software,  DSBLIT_BLEND_ALPHACHANNEL | DSBLIT_BLEND_COLORALPHA | DSBLIT_SRC_PREMULTIPLY | DSBLIT_COLORIZE,
               BLT_COLOR_A);
    primary_hardware->Blit( primary_hardware, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_hardware->Blit( primary_hardware, test_surface, NULL, test_rect.x, test_rect.y );
    primary_software->Blit( primary_software, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_software->Blit( primary_software, test_surface, NULL, test_rect.x, test_rect.y );
    flip_context( primary_info, primary_hardware, primary_software );
    get_user_input();
    
    /* Test 50% Alpha Color + Alpha Channel + PREMULTIPLY Blending + Colorize */
    sprintf(buf, "Test %d: Blit with Premultiply Source Alpha + 50%% Alpha Color blending + Colorize", test_number++);
    display_information(primary_info, buf);
    
    setup_blits(primary_hardware, primary_software,  DSBLIT_BLEND_ALPHACHANNEL | DSBLIT_BLEND_COLORALPHA | DSBLIT_SRC_PREMULTIPLY | DSBLIT_COLORIZE,
               0x80);
    primary_hardware->Blit( primary_hardware, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_hardware->Blit( primary_hardware, test_surface, NULL, test_rect.x, test_rect.y );
    primary_software->Blit( primary_software, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_software->Blit( primary_software, test_surface, NULL, test_rect.x, test_rect.y );
    flip_context( primary_info, primary_hardware, primary_software );
    get_user_input();
    
    /* Test 25% Alpha Color + Alpha Channel + PREMULTIPLY Blending + Colorize */
    sprintf(buf, "Test %d: Blit with Premultiply Source Alpha + 25%% Alpha Color blending + Colorize", test_number++);
    display_information(primary_info, buf);
    
    setup_blits(primary_hardware, primary_software,  DSBLIT_BLEND_ALPHACHANNEL | DSBLIT_BLEND_COLORALPHA | DSBLIT_SRC_PREMULTIPLY | DSBLIT_COLORIZE,
               0x40);
    primary_hardware->Blit( primary_hardware, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_hardware->Blit( primary_hardware, test_surface, NULL, test_rect.x, test_rect.y );
    primary_software->Blit( primary_software, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_software->Blit( primary_software, test_surface, NULL, test_rect.x, test_rect.y );
    flip_context( primary_info, primary_hardware, primary_software );
    get_user_input();
    
    /* Test 100% PREMULTCOLOR with Colorize */
    sprintf(buf, "Test %d: Blit with 100%% PREMULTCOLOR on the Source with Colorize", test_number++);
    display_information(primary_info, buf);
    
    setup_blits(primary_hardware, primary_software,  DSBLIT_SRC_PREMULTCOLOR | DSBLIT_COLORIZE, BLT_COLOR_A);
    primary_hardware->Blit( primary_hardware, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_hardware->Blit( primary_hardware, test_surface, NULL, test_rect.x, test_rect.y );
    primary_software->Blit( primary_software, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_software->Blit( primary_software, test_surface, NULL, test_rect.x, test_rect.y );
    flip_context( primary_info, primary_hardware, primary_software );
    get_user_input();
    
    /* Test 50% PREMULTCOLOR with Colorize */
    sprintf(buf, "Test %d: Blit with 50%% PREMULTCOLOR on the Source with Colorize", test_number++);
    display_information(primary_info, buf);
    
    setup_blits(primary_hardware, primary_software,  DSBLIT_SRC_PREMULTCOLOR | DSBLIT_COLORIZE, 0x80);
    primary_hardware->Blit( primary_hardware, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_hardware->Blit( primary_hardware, test_surface, NULL, test_rect.x, test_rect.y );
    primary_software->Blit( primary_software, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_software->Blit( primary_software, test_surface, NULL, test_rect.x, test_rect.y );
    flip_context( primary_info, primary_hardware, primary_software );
    get_user_input();
    
    /* Test 25% PREMULTCOLOR with Colorize */
    sprintf(buf, "Test %d: Blit with 25%% PREMULTCOLOR on the Source with Colorize", test_number++);
    display_information(primary_info, buf);
    
    setup_blits(primary_hardware, primary_software,  DSBLIT_SRC_PREMULTCOLOR | DSBLIT_COLORIZE, 0x40);
    primary_hardware->Blit( primary_hardware, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_hardware->Blit( primary_hardware, test_surface, NULL, test_rect.x, test_rect.y );
    primary_software->Blit( primary_software, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_software->Blit( primary_software, test_surface, NULL, test_rect.x, test_rect.y );
    flip_context( primary_info, primary_hardware, primary_software );
    get_user_input();
    
    /* Test 100% Alpha Color + Alpha Channel + 100% PREMULTCOLOR Blending + Colorize */ 
    sprintf(buf, "Test %d: Blit with Source Alpha + 100%% Alpha Color + 100%% PREMULTCOLOR + Colorize", test_number++);
    display_information(primary_info, buf);
    
    setup_blits(primary_hardware, primary_software, DSBLIT_BLEND_ALPHACHANNEL | DSBLIT_BLEND_COLORALPHA | DSBLIT_SRC_PREMULTCOLOR | DSBLIT_COLORIZE,
            BLT_COLOR_A);
    primary_hardware->Blit( primary_hardware, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_hardware->Blit( primary_hardware, test_surface, NULL, test_rect.x, test_rect.y );
    primary_software->Blit( primary_software, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_software->Blit( primary_software, test_surface, NULL, test_rect.x, test_rect.y );
    flip_context( primary_info, primary_hardware, primary_software );
    get_user_input();
    
    /* Test 50% Alpha Color + Alpha Channel + 50% PREMULTCOLOR Blending + Colorize */
    sprintf(buf, "Test %d: Blit with Source Alpha + 50%% Alpha Color + 50%% PREMULTCOLOR + Colorize", test_number++);
    display_information(primary_info, buf);
    
    setup_blits(primary_hardware, primary_software, DSBLIT_BLEND_ALPHACHANNEL | DSBLIT_BLEND_COLORALPHA | DSBLIT_SRC_PREMULTCOLOR | DSBLIT_COLORIZE,
            0x80);
    primary_hardware->Blit( primary_hardware, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_hardware->Blit( primary_hardware, test_surface, NULL, test_rect.x, test_rect.y );
    primary_software->Blit( primary_software, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_software->Blit( primary_software, test_surface, NULL, test_rect.x, test_rect.y );
    flip_context( primary_info, primary_hardware, primary_software );
    get_user_input();
    
    /* Test 25% Alpha Color + Alpha Channel + 25% PREMULTCOLOR Blending + Colorize */
    sprintf(buf, "Test %d: Blit with Source Alpha + 25%% Alpha Color + 25%% PREMULTCOLOR + Colorize", test_number++);
    display_information(primary_info, buf);
    
    setup_blits(primary_hardware, primary_software, DSBLIT_BLEND_ALPHACHANNEL | DSBLIT_BLEND_COLORALPHA | DSBLIT_SRC_PREMULTCOLOR | DSBLIT_COLORIZE,
            0x40);
    primary_hardware->Blit( primary_hardware, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_hardware->Blit( primary_hardware, test_surface, NULL, test_rect.x, test_rect.y );
    primary_software->Blit( primary_software, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_software->Blit( primary_software, test_surface, NULL, test_rect.x, test_rect.y );
    flip_context( primary_info, primary_hardware, primary_software );
    get_user_input();
    
    /* Test 100% Alpha Color + Alpha Channel + PREMULTIPLY + 100% PREMULTCOLOR Blending + Colorize */
    sprintf(buf, "Test %d: Blit with Premultiply Source Alpha + 100%% Alpha Color + 100%% PREMULTCOLOR + Colorize", test_number++);
    display_information(primary_info, buf);
    
    setup_blits(primary_hardware, primary_software, DSBLIT_BLEND_ALPHACHANNEL | DSBLIT_BLEND_COLORALPHA | DSBLIT_SRC_PREMULTIPLY |
                                                   DSBLIT_SRC_PREMULTCOLOR | DSBLIT_COLORIZE, BLT_COLOR_A);
    primary_hardware->Blit( primary_hardware, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_hardware->Blit( primary_hardware, test_surface, NULL, test_rect.x, test_rect.y );
    primary_software->Blit( primary_software, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_software->Blit( primary_software, test_surface, NULL, test_rect.x, test_rect.y );
    flip_context( primary_info, primary_hardware, primary_software );
    get_user_input();
    
    /* Test 50% Alpha Color + Alpha Channel + PREMULTIPLY + 50% PREMULTCOLOR Blending + Colorize */
    sprintf(buf, "Test %d: Blit with Premultiply Source Alpha + 50%% Alpha Color + 50%% PREMULTCOLOR + Colorize", test_number++);
    display_information(primary_info, buf);
    
    setup_blits(primary_hardware, primary_software, DSBLIT_BLEND_ALPHACHANNEL | DSBLIT_BLEND_COLORALPHA | DSBLIT_SRC_PREMULTIPLY |
                                                   DSBLIT_SRC_PREMULTCOLOR | DSBLIT_COLORIZE, 0x80);
    primary_hardware->Blit( primary_hardware, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_hardware->Blit( primary_hardware, test_surface, NULL, test_rect.x, test_rect.y );
    primary_software->Blit( primary_software, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_software->Blit( primary_software, test_surface, NULL, test_rect.x, test_rect.y );
    flip_context( primary_info, primary_hardware, primary_software );
    get_user_input();
    
    /* Test 25% Alpha Color + Alpha Channel + PREMULTIPLY + 25% PREMULTCOLOR Blending + Colorize */
    sprintf(buf, "Test %d: Blit with Premultiply Source Alpha + 25%% Alpha Color + 25%% PREMULTCOLOR + Colorize", test_number++);
    display_information(primary_info, buf);
    
    setup_blits(primary_hardware, primary_software, DSBLIT_BLEND_ALPHACHANNEL | DSBLIT_BLEND_COLORALPHA | DSBLIT_SRC_PREMULTIPLY |
                                                   DSBLIT_SRC_PREMULTCOLOR | DSBLIT_COLORIZE, 0x40);
    primary_hardware->Blit( primary_hardware, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_hardware->Blit( primary_hardware, test_surface, NULL, test_rect.x, test_rect.y );
    primary_software->Blit( primary_software, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_software->Blit( primary_software, test_surface, NULL, test_rect.x, test_rect.y );
    flip_context( primary_info, primary_hardware, primary_software );
    get_user_input();
    
    /*** Test with Porter-Duff Source */
    
     /* Test Flip Horizontally */
    sprintf(buf, "Test %d: Porter-Duff Source Blit with Horizontal Flip", test_number++);
    display_information(primary_info, buf);
    
    setup_porter_duff_blits(primary_hardware, primary_software, DSPD_SRC, DSBLIT_FLIP_HORIZONTAL, BLT_COLOR_A);
    primary_hardware->Blit( primary_hardware, logo_surface, NULL, logo_rect.x, logo_rect.y );
    primary_hardware->Blit( primary_hardware, test_surface, NULL,  test_rect.x, test_rect.y );
    primary_software->Blit( primary_software, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_software->Blit( primary_software, test_surface, NULL,  test_rect.x, test_rect.y );
    flip_context( primary_info, primary_hardware, primary_software );
    get_user_input();

     /* Test Flip Vertically */
    sprintf(buf, "Test %d: Porter-Duff Source Blit with Vertical Flip", test_number++);
    display_information(primary_info, buf);
    
    setup_porter_duff_blits(primary_hardware, primary_software, DSPD_SRC, DSBLIT_FLIP_VERTICAL, BLT_COLOR_A);
    primary_hardware->Blit( primary_hardware, logo_surface, NULL, logo_rect.x, logo_rect.y );
    primary_hardware->Blit( primary_hardware, test_surface, NULL,  test_rect.x, test_rect.y );
    primary_software->Blit( primary_software, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_software->Blit( primary_software, test_surface, NULL,  test_rect.x, test_rect.y );
    flip_context( primary_info, primary_hardware, primary_software );
    get_user_input();
    
     /* Test Rotate 180 */
    sprintf(buf, "Test %d: Porter-Duff Source Blit with 180 Degrees Rotation", test_number++);
    display_information(primary_info, buf);
    
    setup_porter_duff_blits(primary_hardware, primary_software, DSPD_SRC, DSBLIT_ROTATE180, BLT_COLOR_A);
    primary_hardware->Blit( primary_hardware, logo_surface, NULL, logo_rect.x, logo_rect.y );
    primary_hardware->Blit( primary_hardware, test_surface, NULL,  test_rect.x, test_rect.y );
    primary_software->Blit( primary_software, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_software->Blit( primary_software, test_surface, NULL,  test_rect.x, test_rect.y );
    flip_context( primary_info, primary_hardware, primary_software );
    get_user_input();
    
     /* Test Flip Horizontally with Alpha Channel Blending */
    sprintf(buf, "Test %d: Porter-Duff Source Blit with Horizontal Flip and Alpha Channel Blending", test_number++);
    display_information(primary_info, buf);
    
    setup_porter_duff_blits(primary_hardware, primary_software, DSPD_SRC, DSBLIT_FLIP_HORIZONTAL | DSBLIT_BLEND_ALPHACHANNEL, BLT_COLOR_A);
    primary_hardware->Blit( primary_hardware, logo_surface, NULL, logo_rect.x, logo_rect.y );
    primary_hardware->Blit( primary_hardware, test_surface, NULL,  test_rect.x, test_rect.y );
    primary_software->Blit( primary_software, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_software->Blit( primary_software, test_surface, NULL,  test_rect.x, test_rect.y );
    flip_context( primary_info, primary_hardware, primary_software );
    get_user_input();
    
     /* Test Flip Vertically with Alpha Channel Blending */
    sprintf(buf, "Test %d: Porter-Duff Source Blit with Vertical Flip and Alpha Channel Blending", test_number++);
    display_information(primary_info, buf);
    
    setup_porter_duff_blits(primary_hardware, primary_software, DSPD_SRC, DSBLIT_FLIP_VERTICAL | DSBLIT_BLEND_ALPHACHANNEL, BLT_COLOR_A);
    primary_hardware->Blit( primary_hardware, logo_surface, NULL, logo_rect.x, logo_rect.y );
    primary_hardware->Blit( primary_hardware, test_surface, NULL,  test_rect.x, test_rect.y );
    primary_software->Blit( primary_software, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_software->Blit( primary_software, test_surface, NULL,  test_rect.x, test_rect.y );
    flip_context( primary_info, primary_hardware, primary_software );
    get_user_input();
    
     /* Test Rotate 180 with Alpha Channel Blending */
    sprintf(buf, "Test %d: Porter-Duff Source Blit with 180 Degrees Rotation and Alpha Channel Blending", test_number++);
    display_information(primary_info, buf);
    
    setup_porter_duff_blits(primary_hardware, primary_software, DSPD_SRC, DSBLIT_ROTATE180 | DSBLIT_BLEND_ALPHACHANNEL, BLT_COLOR_A);
    primary_hardware->Blit( primary_hardware, logo_surface, NULL, logo_rect.x, logo_rect.y );
    primary_hardware->Blit( primary_hardware, test_surface, NULL,  test_rect.x, test_rect.y );
    primary_software->Blit( primary_software, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_software->Blit( primary_software, test_surface, NULL,  test_rect.x, test_rect.y );
    flip_context( primary_info, primary_hardware, primary_software );
    get_user_input();
    
     /* Test XOR */
    sprintf(buf, "Test %d: Porter-Duff Source Blit with XOR", test_number++);
    display_information(primary_info, buf);
    
    setup_porter_duff_blits(primary_hardware, primary_software, DSPD_SRC, DSBLIT_XOR, BLT_COLOR_A);
    primary_hardware->Blit( primary_hardware, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_hardware->Blit( primary_hardware, test_surface, NULL, test_rect.x, test_rect.y );
    primary_software->Blit( primary_software, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_software->Blit( primary_software, test_surface, NULL, test_rect.x, test_rect.y );
    flip_context( primary_info, primary_hardware, primary_software );
    get_user_input();
    
     /* Test XOR with Alpha Channel Blending */
    sprintf(buf, "Test %d: Porter-Duff Source Blit with XOR and Alpha Channel Blending", test_number++);
    display_information(primary_info, buf);
    
    setup_porter_duff_blits(primary_hardware, primary_software, DSPD_SRC, DSBLIT_XOR | DSBLIT_BLEND_ALPHACHANNEL, 0x80);
    primary_hardware->Blit( primary_hardware, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_hardware->Blit( primary_hardware, test_surface, NULL, test_rect.x, test_rect.y );
    primary_software->Blit( primary_software, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_software->Blit( primary_software, test_surface, NULL, test_rect.x, test_rect.y );
    flip_context( primary_info, primary_hardware, primary_software );
    get_user_input();
    
     /* Test XOR with Color Alpha Blending */
    sprintf(buf, "Test %d: Porter-Duff Source Blit with XOR and Color Alpha Blending", test_number++);
    display_information(primary_info, buf);
    
    setup_porter_duff_blits(primary_hardware, primary_software, DSPD_SRC, DSBLIT_XOR | DSBLIT_BLEND_COLORALPHA, 0x80);
    primary_hardware->Blit( primary_hardware, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_hardware->Blit( primary_hardware, test_surface, NULL, test_rect.x, test_rect.y );
    primary_software->Blit( primary_software, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_software->Blit( primary_software, test_surface, NULL, test_rect.x, test_rect.y );
    flip_context( primary_info, primary_hardware, primary_software );
    get_user_input();
    
     /* Test XOR with Alpha Channel and Color Alpha Blending */
    sprintf(buf, "Test %d: Porter-Duff Source Blit with XOR and Alpha Channel + Color Alpha Blending", test_number++);
    display_information(primary_info, buf);
    
    setup_porter_duff_blits(primary_hardware, primary_software, DSPD_SRC, DSBLIT_XOR | DSBLIT_BLEND_ALPHACHANNEL | DSBLIT_BLEND_COLORALPHA, 0x80);
    primary_hardware->Blit( primary_hardware, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_hardware->Blit( primary_hardware, test_surface, NULL, test_rect.x, test_rect.y );
    primary_software->Blit( primary_software, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_software->Blit( primary_software, test_surface, NULL, test_rect.x, test_rect.y );
    flip_context( primary_info, primary_hardware, primary_software );
    get_user_input();
    
     /* Test XOR with Premultiply and Alpha Channel and Color Alpha Blending */
    sprintf(buf, "Test %d: Porter-Duff Source Blit with XOR and Premultiply + Alpha Channel + Color Alpha Blending", test_number++);
    display_information(primary_info, buf);
    
    setup_porter_duff_blits(primary_hardware, primary_software, DSPD_SRC, DSBLIT_XOR | DSBLIT_BLEND_ALPHACHANNEL | DSBLIT_BLEND_COLORALPHA | DSBLIT_SRC_PREMULTIPLY, 0x80);
    primary_hardware->Blit( primary_hardware, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_hardware->Blit( primary_hardware, test_surface, NULL, test_rect.x, test_rect.y );
    primary_software->Blit( primary_software, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_software->Blit( primary_software, test_surface, NULL, test_rect.x, test_rect.y );
    flip_context( primary_info, primary_hardware, primary_software );
    get_user_input();

     /* Test XOR with Premultiply and Alpha Channel and Color Alpha Blending and PremultColor */
    sprintf(buf, "Test %d: Porter-Duff Source Blit with XOR + Premultiply + Alpha Channel + Color Alpha + PREMULTCOLOR Blending", test_number++);
    display_information(primary_info, buf);
    
    setup_porter_duff_blits(primary_hardware, primary_software, DSPD_SRC, DSBLIT_XOR | DSBLIT_BLEND_ALPHACHANNEL | DSBLIT_BLEND_COLORALPHA |
                                                    DSBLIT_SRC_PREMULTIPLY | DSBLIT_SRC_PREMULTCOLOR, 0x80);
    primary_hardware->Blit( primary_hardware, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_hardware->Blit( primary_hardware, test_surface, NULL, test_rect.x, test_rect.y );
    primary_software->Blit( primary_software, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_software->Blit( primary_software, test_surface, NULL, test_rect.x, test_rect.y );
    flip_context( primary_info, primary_hardware, primary_software );
    get_user_input();
    
     /* Test XOR with Premultiply and Alpha Channel and Color Alpha Blending and PremultColor and Colorize */
    sprintf(buf, "Test %d: Porter-Duff Source Blit with XOR + Premultiply + Alpha Channel + Color Alpha + PREMULTCOLOR + Colorize Blending", test_number++);
    display_information(primary_info, buf);
    
    setup_porter_duff_blits(primary_hardware, primary_software, DSPD_SRC, DSBLIT_XOR | DSBLIT_BLEND_ALPHACHANNEL | DSBLIT_BLEND_COLORALPHA |
                                                    DSBLIT_SRC_PREMULTIPLY | DSBLIT_SRC_PREMULTCOLOR | DSBLIT_COLORIZE, 0x80);
    primary_hardware->Blit( primary_hardware, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_hardware->Blit( primary_hardware, test_surface, NULL, test_rect.x, test_rect.y );
    primary_software->Blit( primary_software, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_software->Blit( primary_software, test_surface, NULL, test_rect.x, test_rect.y );
    flip_context( primary_info, primary_hardware, primary_software );
    get_user_input();
    
     /* Test Alpha Channel Blending */
    sprintf(buf, "Test %d: Porter-Duff Source Blit with Alpha Channel Blending", test_number++);
    display_information(primary_info, buf);
    
    setup_porter_duff_blits(primary_hardware, primary_software, DSPD_SRC, DSBLIT_BLEND_ALPHACHANNEL, BLT_COLOR_A);
    primary_hardware->Blit( primary_hardware, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_hardware->Blit( primary_hardware, test_surface, NULL, test_rect.x, test_rect.y );
    primary_software->Blit( primary_software, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_software->Blit( primary_software, test_surface, NULL, test_rect.x, test_rect.y );
    flip_context( primary_info, primary_hardware, primary_software );
    get_user_input();
    
     /* Test 100% Color Alpha Blending */
    sprintf(buf, "Test %d: Porter-Duff Source Blit with 100%% Color Alpha Blending", test_number++);
    display_information(primary_info, buf);
    
    setup_porter_duff_blits(primary_hardware, primary_software, DSPD_SRC, DSBLIT_BLEND_COLORALPHA, BLT_COLOR_A);
    primary_hardware->Blit( primary_hardware, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_hardware->Blit( primary_hardware, test_surface, NULL, test_rect.x, test_rect.y );
    primary_software->Blit( primary_software, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_software->Blit( primary_software, test_surface, NULL, test_rect.x, test_rect.y );
    flip_context( primary_info, primary_hardware, primary_software );
    get_user_input();
    
     /* Test 50% Color Alpha Blending */
    sprintf(buf, "Test %d: Porter-Duff Source Blit with 50%% Color Alpha Blending", test_number++);
    display_information(primary_info, buf);
    
    setup_porter_duff_blits(primary_hardware, primary_software, DSPD_SRC, DSBLIT_BLEND_COLORALPHA, 0x80);
    primary_hardware->Blit( primary_hardware, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_hardware->Blit( primary_hardware, test_surface, NULL, test_rect.x, test_rect.y );
    primary_software->Blit( primary_software, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_software->Blit( primary_software, test_surface, NULL, test_rect.x, test_rect.y );
    flip_context( primary_info, primary_hardware, primary_software );
    get_user_input();
    
     /* Test 25% Color Alpha Blending */
    sprintf(buf, "Test %d: Porter-Duff Source Blit with 25%% Color Alpha Blending", test_number++);
    display_information(primary_info, buf);
    
    setup_porter_duff_blits(primary_hardware, primary_software, DSPD_SRC, DSBLIT_BLEND_COLORALPHA, 0x40);
    primary_hardware->Blit( primary_hardware, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_hardware->Blit( primary_hardware, test_surface, NULL, test_rect.x, test_rect.y );
    primary_software->Blit( primary_software, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_software->Blit( primary_software, test_surface, NULL, test_rect.x, test_rect.y );
    flip_context( primary_info, primary_hardware, primary_software );
    get_user_input();

    /* Test 100% Color Alpha + Alpha Channel Blending */
    sprintf(buf, "Test %d: Porter-Duff Source Blit with 100%% Color Alpha + Alpha Blend", test_number++);
    display_information(primary_info, buf);
    
    setup_porter_duff_blits(primary_hardware, primary_software, DSPD_SRC, DSBLIT_BLEND_COLORALPHA|DSBLIT_BLEND_ALPHACHANNEL, BLT_COLOR_A);
    primary_hardware->Blit( primary_hardware, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_hardware->Blit( primary_hardware, test_surface, NULL, test_rect.x, test_rect.y );
    primary_software->Blit( primary_software, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_software->Blit( primary_software, test_surface, NULL, test_rect.x, test_rect.y );
    flip_context( primary_info, primary_hardware, primary_software );
    get_user_input();
    
    /* Test 50% Color Alpha + Alpha Channel Blending */
    sprintf(buf, "Test %d: Porter-Duff Source Blit with 50%% Color Alpha + Alpha Blend", test_number++);
    display_information(primary_info, buf);
    
    setup_porter_duff_blits(primary_hardware, primary_software, DSPD_SRC, DSBLIT_BLEND_COLORALPHA|DSBLIT_BLEND_ALPHACHANNEL, 0x80);
    primary_hardware->Blit( primary_hardware, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_hardware->Blit( primary_hardware, test_surface, NULL, test_rect.x, test_rect.y );
    primary_software->Blit( primary_software, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_software->Blit( primary_software, test_surface, NULL, test_rect.x, test_rect.y );
    flip_context( primary_info, primary_hardware, primary_software );
    get_user_input();
    
    /* Test 25% Color Alpha + Alpha Channel Blending */
    sprintf(buf, "Test %d: Porter-Duff Source Blit with 25%% Color Alpha + Alpha Blend", test_number++);
    display_information(primary_info, buf);
    
    setup_porter_duff_blits(primary_hardware, primary_software, DSPD_SRC, DSBLIT_BLEND_COLORALPHA|DSBLIT_BLEND_ALPHACHANNEL, 0x40);
    primary_hardware->Blit( primary_hardware, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_hardware->Blit( primary_hardware, test_surface, NULL, test_rect.x, test_rect.y );
    primary_software->Blit( primary_software, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_software->Blit( primary_software, test_surface, NULL, test_rect.x, test_rect.y );
    flip_context( primary_info, primary_hardware, primary_software );
    get_user_input();
    
    /* Test PREMULTIPLY */
    sprintf(buf, "Test %d: Porter-Duff Source Blit with Premultiply Source", test_number++);
    display_information(primary_info, buf);
    
    setup_porter_duff_blits(primary_hardware, primary_software, DSPD_SRC, DSBLIT_SRC_PREMULTIPLY, 0x80);
    primary_hardware->Blit( primary_hardware, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_hardware->Blit( primary_hardware, test_surface, NULL, test_rect.x, test_rect.y );
    primary_software->Blit( primary_software, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_software->Blit( primary_software, test_surface, NULL, test_rect.x, test_rect.y );
    flip_context( primary_info, primary_hardware, primary_software );
    get_user_input();
    
    /* Test Alpha Channel + PREMULTIPLY Blending */
    sprintf(buf, "Test %d: Porter-Duff Source Blit with Premultiply Source Alpha Blending", test_number++);
    display_information(primary_info, buf);
    
    setup_porter_duff_blits(primary_hardware, primary_software, DSPD_SRC, DSBLIT_BLEND_ALPHACHANNEL | DSBLIT_SRC_PREMULTIPLY, 0x80);
    primary_hardware->Blit( primary_hardware, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_hardware->Blit( primary_hardware, test_surface, NULL, test_rect.x, test_rect.y );
    primary_software->Blit( primary_software, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_software->Blit( primary_software, test_surface, NULL, test_rect.x, test_rect.y );
    flip_context( primary_info, primary_hardware, primary_software );
    get_user_input();
    
     /* Test 100% Alpha Color + Alpha Channel + PREMULTIPLY Blending */
    sprintf(buf, "Test %d: Porter-Duff Source Blit with Premultiply Source Alpha + 100%% Alpha Color blending", test_number++);
    display_information(primary_info, buf);
    
    setup_porter_duff_blits(primary_hardware, primary_software, DSPD_SRC, DSBLIT_BLEND_ALPHACHANNEL | DSBLIT_BLEND_COLORALPHA | DSBLIT_SRC_PREMULTIPLY, BLT_COLOR_A);
    primary_hardware->Blit( primary_hardware, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_hardware->Blit( primary_hardware, test_surface, NULL, test_rect.x, test_rect.y );
    primary_software->Blit( primary_software, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_software->Blit( primary_software, test_surface, NULL, test_rect.x, test_rect.y );
    flip_context( primary_info, primary_hardware, primary_software );
    get_user_input();
    
     /* Test 50% Alpha Color + Alpha Channel + PREMULTIPLY Blending */
    sprintf(buf, "Test %d: Porter-Duff Source Blit with Premultiply Source Alpha + 50%% Alpha Color blending", test_number++);
    display_information(primary_info, buf);
    
    setup_porter_duff_blits(primary_hardware, primary_software, DSPD_SRC, DSBLIT_BLEND_ALPHACHANNEL | DSBLIT_BLEND_COLORALPHA | DSBLIT_SRC_PREMULTIPLY, 0x80);
    primary_hardware->Blit( primary_hardware, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_hardware->Blit( primary_hardware, test_surface, NULL, test_rect.x, test_rect.y );
    primary_software->Blit( primary_software, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_software->Blit( primary_software, test_surface, NULL, test_rect.x, test_rect.y );
    flip_context( primary_info, primary_hardware, primary_software );
    get_user_input();
    
     /* Test 25% Alpha Color + Alpha Channel + PREMULTIPLY Blending */
    sprintf(buf, "Test %d: Porter-Duff Source Blit with Premultiply Source Alpha + 25%% Alpha Color blending", test_number++);
    display_information(primary_info, buf);
    
    setup_porter_duff_blits(primary_hardware, primary_software, DSPD_SRC, DSBLIT_BLEND_ALPHACHANNEL | DSBLIT_BLEND_COLORALPHA | DSBLIT_SRC_PREMULTIPLY, 0x40);
    primary_hardware->Blit( primary_hardware, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_hardware->Blit( primary_hardware, test_surface, NULL, test_rect.x, test_rect.y );
    primary_software->Blit( primary_software, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_software->Blit( primary_software, test_surface, NULL, test_rect.x, test_rect.y );
    flip_context( primary_info, primary_hardware, primary_software );
    get_user_input();
    
    /* Test 100% PREMULTCOLOR */
    sprintf(buf, "Test %d: Porter-Duff Source Blit with 100%% PREMULTCOLOR on the Source", test_number++);
    display_information(primary_info, buf);
    
    setup_porter_duff_blits(primary_hardware, primary_software, DSPD_SRC, DSBLIT_SRC_PREMULTCOLOR, BLT_COLOR_A);
    primary_hardware->Blit( primary_hardware, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_hardware->Blit( primary_hardware, test_surface, NULL, test_rect.x, test_rect.y );
    primary_software->Blit( primary_software, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_software->Blit( primary_software, test_surface, NULL, test_rect.x, test_rect.y );
    flip_context( primary_info, primary_hardware, primary_software );
    get_user_input();

    /* Test 50% PREMULTCOLOR */
    sprintf(buf, "Test %d: Porter-Duff Source Blit with 50%% PREMULTCOLOR on the Source", test_number++);
    display_information(primary_info, buf);
    
    setup_porter_duff_blits(primary_hardware, primary_software, DSPD_SRC, DSBLIT_SRC_PREMULTCOLOR, 0x80);
    primary_hardware->Blit( primary_hardware, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_hardware->Blit( primary_hardware, test_surface, NULL, test_rect.x, test_rect.y );
    primary_software->Blit( primary_software, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_software->Blit( primary_software, test_surface, NULL, test_rect.x, test_rect.y );
    flip_context( primary_info, primary_hardware, primary_software );
    get_user_input();
    
    /* Test 25% PREMULTCOLOR */
    sprintf(buf, "Test %d: Porter-Duff Source Blit with 25%% PREMULTCOLOR on the Source", test_number++);
    display_information(primary_info, buf);
    
    setup_porter_duff_blits(primary_hardware, primary_software, DSPD_SRC, DSBLIT_SRC_PREMULTCOLOR, 0x40);
    primary_hardware->Blit( primary_hardware, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_hardware->Blit( primary_hardware, test_surface, NULL, test_rect.x, test_rect.y );
    primary_software->Blit( primary_software, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_software->Blit( primary_software, test_surface, NULL, test_rect.x, test_rect.y );
    flip_context( primary_info, primary_hardware, primary_software );
    get_user_input();
    
    /* Test 100% Alpha Color + Alpha Channel + 100% PREMULTCOLOR Blending */
    sprintf(buf, "Test %d: Porter-Duff Source Blit with Source Alpha + 100%% Alpha Color + 100%% PREMULTCOLOR", test_number++);
    display_information(primary_info, buf);
    
    setup_porter_duff_blits(primary_hardware, primary_software, DSPD_SRC, DSBLIT_BLEND_ALPHACHANNEL | DSBLIT_BLEND_COLORALPHA | DSBLIT_SRC_PREMULTCOLOR, BLT_COLOR_A);
    primary_hardware->Blit( primary_hardware, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_hardware->Blit( primary_hardware, test_surface, NULL, test_rect.x, test_rect.y );
    primary_software->Blit( primary_software, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_software->Blit( primary_software, test_surface, NULL, test_rect.x, test_rect.y );
    flip_context( primary_info, primary_hardware, primary_software );
    get_user_input();
    
    /* Test 50% Alpha Color + Alpha Channel + 50% PREMULTCOLOR Blending */
    sprintf(buf, "Test %d: Porter-Duff Source Blit with Source Alpha + 50%% Alpha Color + 50%% PREMULTCOLOR", test_number++);
    display_information(primary_info, buf);
    
    setup_porter_duff_blits(primary_hardware, primary_software, DSPD_SRC, DSBLIT_BLEND_ALPHACHANNEL | DSBLIT_BLEND_COLORALPHA | DSBLIT_SRC_PREMULTCOLOR, 0x80);
    primary_hardware->Blit( primary_hardware, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_hardware->Blit( primary_hardware, test_surface, NULL, test_rect.x, test_rect.y );
    primary_software->Blit( primary_software, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_software->Blit( primary_software, test_surface, NULL, test_rect.x, test_rect.y );
    flip_context( primary_info, primary_hardware, primary_software );
    get_user_input();
    
    /* Test 25% Alpha Color + Alpha Channel + 25% PREMULTCOLOR Blending */
    sprintf(buf, "Test %d: Porter-Duff Source Blit with Source Alpha + 25%% Alpha Color + 25%% PREMULTCOLOR", test_number++);
    display_information(primary_info, buf);
    
    setup_porter_duff_blits(primary_hardware, primary_software, DSPD_SRC, DSBLIT_BLEND_ALPHACHANNEL | DSBLIT_BLEND_COLORALPHA | DSBLIT_SRC_PREMULTCOLOR, 0x40);
    primary_hardware->Blit( primary_hardware, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_hardware->Blit( primary_hardware, test_surface, NULL, test_rect.x, test_rect.y );
    primary_software->Blit( primary_software, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_software->Blit( primary_software, test_surface, NULL, test_rect.x, test_rect.y );
    flip_context( primary_info, primary_hardware, primary_software );
    get_user_input();
    
    /* Test 100% Alpha Color + Alpha Channel + PREMULTIPLY + 100% PREMULTCOLOR Blending */
    sprintf(buf, "Test %d: Porter-Duff Source Blit with Premultiply Source Alpha + 100%% Alpha Color + 100%% PREMULTCOLOR", test_number++);
    display_information(primary_info, buf);
    
    setup_porter_duff_blits(primary_hardware, primary_software, DSPD_SRC, DSBLIT_BLEND_ALPHACHANNEL | DSBLIT_BLEND_COLORALPHA | DSBLIT_SRC_PREMULTIPLY | DSBLIT_SRC_PREMULTCOLOR,
            BLT_COLOR_A);
    primary_hardware->Blit( primary_hardware, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_hardware->Blit( primary_hardware, test_surface, NULL, test_rect.x, test_rect.y );
    primary_software->Blit( primary_software, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_software->Blit( primary_software, test_surface, NULL, test_rect.x, test_rect.y );
    flip_context( primary_info, primary_hardware, primary_software );
    get_user_input();
    
    /* Test 50% Alpha Color + Alpha Channel + PREMULTIPLY + 50% PREMULTCOLOR Blending */
    sprintf(buf, "Test %d: Porter-Duff Source Blit with Premultiply Source Alpha + 50%% Alpha Color + 50%% PREMULTCOLOR", test_number++);
    display_information(primary_info, buf);
    
    setup_porter_duff_blits(primary_hardware, primary_software, DSPD_SRC, DSBLIT_BLEND_ALPHACHANNEL | DSBLIT_BLEND_COLORALPHA | DSBLIT_SRC_PREMULTIPLY | DSBLIT_SRC_PREMULTCOLOR,
            0x80);
    primary_hardware->Blit( primary_hardware, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_hardware->Blit( primary_hardware, test_surface, NULL, test_rect.x, test_rect.y );
    primary_software->Blit( primary_software, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_software->Blit( primary_software, test_surface, NULL, test_rect.x, test_rect.y );
    flip_context( primary_info, primary_hardware, primary_software );
    get_user_input();

    /* Test 25% Alpha Color + Alpha Channel + PREMULTIPLY + 25% PREMULTCOLOR Blending */
    sprintf(buf, "Test %d: Porter-Duff Source Blit with Premultiply Source Alpha + 25%% Alpha Color + 25%% PREMULTCOLOR", test_number++);
    display_information(primary_info, buf);
    
    setup_porter_duff_blits(primary_hardware, primary_software, DSPD_SRC, DSBLIT_BLEND_ALPHACHANNEL | DSBLIT_BLEND_COLORALPHA | DSBLIT_SRC_PREMULTIPLY | DSBLIT_SRC_PREMULTCOLOR,
            0x40);
    primary_hardware->Blit( primary_hardware, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_hardware->Blit( primary_hardware, test_surface, NULL, test_rect.x, test_rect.y );
    primary_software->Blit( primary_software, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_software->Blit( primary_software, test_surface, NULL, test_rect.x, test_rect.y );
    flip_context( primary_info, primary_hardware, primary_software );
    get_user_input();
    
    /* Test Alpha Channel Blending with Colorize */
    sprintf(buf, "Test %d: Porter-Duff Source Blit with Source Alpha Blending with Colorize", test_number++);
    display_information(primary_info, buf);
    
    setup_porter_duff_blits(primary_hardware, primary_software, DSPD_SRC, DSBLIT_BLEND_ALPHACHANNEL | DSBLIT_COLORIZE, 0x80);
    primary_hardware->Blit( primary_hardware, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_hardware->Blit( primary_hardware, test_surface, NULL, test_rect.x, test_rect.y );
    primary_software->Blit( primary_software, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_software->Blit( primary_software, test_surface, NULL, test_rect.x, test_rect.y );
    flip_context( primary_info, primary_hardware, primary_software );
    get_user_input();
    
    /* Test 100% Color Alpha Blending with Colorize */
    sprintf(buf, "Test %d: Porter-Duff Source Blit with 100%% Color Alpha Blending with Colorize", test_number++);
    display_information(primary_info, buf);
    
    setup_porter_duff_blits(primary_hardware, primary_software, DSPD_SRC, DSBLIT_BLEND_COLORALPHA | DSBLIT_COLORIZE, BLT_COLOR_A);
    primary_hardware->Blit( primary_hardware, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_hardware->Blit( primary_hardware, test_surface, NULL, test_rect.x, test_rect.y );
    primary_software->Blit( primary_software, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_software->Blit( primary_software, test_surface, NULL, test_rect.x, test_rect.y );
    flip_context( primary_info, primary_hardware, primary_software );
    get_user_input();
    
    /* Test 50% Color Alpha Blending with Colorize */
    sprintf(buf, "Test %d: Porter-Duff Source Blit with 50%% Color Alpha Blending with Colorize", test_number++);
    display_information(primary_info, buf);
    
    setup_porter_duff_blits(primary_hardware, primary_software, DSPD_SRC, DSBLIT_BLEND_COLORALPHA | DSBLIT_COLORIZE, 0x80);
    primary_hardware->Blit( primary_hardware, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_hardware->Blit( primary_hardware, test_surface, NULL, test_rect.x, test_rect.y );
    primary_software->Blit( primary_software, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_software->Blit( primary_software, test_surface, NULL, test_rect.x, test_rect.y );
    flip_context( primary_info, primary_hardware, primary_software );
    get_user_input();
    
    /* Test 25% Color Alpha Blending with Colorize */
    sprintf(buf, "Test %d: Porter-Duff Source Blit with 25%% Color Alpha Blending with Colorize", test_number++);
    display_information(primary_info, buf);
    
    setup_porter_duff_blits(primary_hardware, primary_software, DSPD_SRC, DSBLIT_BLEND_COLORALPHA | DSBLIT_COLORIZE, 0x40);
    primary_hardware->Blit( primary_hardware, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_hardware->Blit( primary_hardware, test_surface, NULL, test_rect.x, test_rect.y );
    primary_software->Blit( primary_software, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_software->Blit( primary_software, test_surface, NULL, test_rect.x, test_rect.y );
    flip_context( primary_info, primary_hardware, primary_software );
    get_user_input();
    
    /* Test 100% Color Alpha + Alpha Channel Blending + Colorize */
    sprintf(buf, "Test %d: Porter-Duff Source Blit with 100%% Color Alpha + Source Alpha Blending + Colorize", test_number++);
    display_information(primary_info, buf);
    
    setup_porter_duff_blits(primary_hardware, primary_software, DSPD_SRC, DSBLIT_BLEND_COLORALPHA | DSBLIT_BLEND_ALPHACHANNEL | DSBLIT_COLORIZE, BLT_COLOR_A);
    primary_hardware->Blit( primary_hardware, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_hardware->Blit( primary_hardware, test_surface, NULL, test_rect.x, test_rect.y );
    primary_software->Blit( primary_software, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_software->Blit( primary_software, test_surface, NULL, test_rect.x, test_rect.y );
    flip_context( primary_info, primary_hardware, primary_software );
    get_user_input();
    
    /* Test 50% Color Alpha + Alpha Channel Blending + Colorize */
    sprintf(buf, "Test %d: Porter-Duff Source Blit with 50%% Color Alpha + Source Alpha Blending + Colorize", test_number++);
    display_information(primary_info, buf);
    
    setup_porter_duff_blits(primary_hardware, primary_software, DSPD_SRC, DSBLIT_BLEND_COLORALPHA | DSBLIT_BLEND_ALPHACHANNEL | DSBLIT_COLORIZE, 0x80);
    primary_hardware->Blit( primary_hardware, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_hardware->Blit( primary_hardware, test_surface, NULL, test_rect.x, test_rect.y );
    primary_software->Blit( primary_software, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_software->Blit( primary_software, test_surface, NULL, test_rect.x, test_rect.y );
    flip_context( primary_info, primary_hardware, primary_software );
    get_user_input();

    /* Test 25% Color Alpha + Alpha Channel Blending + Colorize */
    sprintf(buf, "Test %d: Porter-Duff Source Blit with 25%% Color Alpha + Source Alpha Blending + Colorize", test_number++);
    display_information(primary_info, buf);
    
    setup_porter_duff_blits(primary_hardware, primary_software, DSPD_SRC, DSBLIT_BLEND_COLORALPHA | DSBLIT_BLEND_ALPHACHANNEL | DSBLIT_COLORIZE, 0x40);
    primary_hardware->Blit( primary_hardware, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_hardware->Blit( primary_hardware, test_surface, NULL, test_rect.x, test_rect.y );
    primary_software->Blit( primary_software, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_software->Blit( primary_software, test_surface, NULL, test_rect.x, test_rect.y );
    flip_context( primary_info, primary_hardware, primary_software );
    get_user_input();
    
    /* Test PREMULTIPLY with Colorize */
    sprintf(buf, "Test %d: Porter-Duff Source Blit with Premultiply Source with Colorize", test_number++);
    display_information(primary_info, buf);
    
    setup_porter_duff_blits(primary_hardware, primary_software, DSPD_SRC, DSBLIT_SRC_PREMULTIPLY | DSBLIT_COLORIZE, 0x80);
    primary_hardware->Blit( primary_hardware, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_hardware->Blit( primary_hardware, test_surface, NULL, test_rect.x, test_rect.y );
    primary_software->Blit( primary_software, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_software->Blit( primary_software, test_surface, NULL, test_rect.x, test_rect.y );
    flip_context( primary_info, primary_hardware, primary_software );
    get_user_input();
    
    /* Test Alpha Channel + PREMULTIPLY Blending + Colorize */
    sprintf(buf, "Test %d: Porter-Duff Source Blit with Premultiply Source Alpha Blending with Colorize", test_number++);
    display_information(primary_info, buf);
    
    setup_porter_duff_blits(primary_hardware, primary_software, DSPD_SRC,  DSBLIT_BLEND_ALPHACHANNEL | DSBLIT_SRC_PREMULTIPLY | DSBLIT_COLORIZE, 0x80);
    primary_hardware->Blit( primary_hardware, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_hardware->Blit( primary_hardware, test_surface, NULL, test_rect.x, test_rect.y );
    primary_software->Blit( primary_software, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_software->Blit( primary_software, test_surface, NULL, test_rect.x, test_rect.y );
    flip_context( primary_info, primary_hardware, primary_software );
    get_user_input();
    
    /* Test 100% Alpha Color + Alpha Channel + PREMULTIPLY Blending + Colorize */
    sprintf(buf, "Test %d: Porter-Duff Source Blit with Premultiply Source Alpha + 100%% Alpha Color blending + Colorize", test_number++);
    display_information(primary_info, buf);
    
    setup_porter_duff_blits(primary_hardware, primary_software, DSPD_SRC,  DSBLIT_BLEND_ALPHACHANNEL | DSBLIT_BLEND_COLORALPHA | DSBLIT_SRC_PREMULTIPLY | DSBLIT_COLORIZE,
               BLT_COLOR_A);
    primary_hardware->Blit( primary_hardware, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_hardware->Blit( primary_hardware, test_surface, NULL, test_rect.x, test_rect.y );
    primary_software->Blit( primary_software, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_software->Blit( primary_software, test_surface, NULL, test_rect.x, test_rect.y );
    flip_context( primary_info, primary_hardware, primary_software );
    get_user_input();
    
    /* Test 50% Alpha Color + Alpha Channel + PREMULTIPLY Blending + Colorize */
    sprintf(buf, "Test %d: Porter-Duff Source Blit with Premultiply Source Alpha + 50%% Alpha Color blending + Colorize", test_number++);
    display_information(primary_info, buf);
    
    setup_porter_duff_blits(primary_hardware, primary_software, DSPD_SRC,  DSBLIT_BLEND_ALPHACHANNEL | DSBLIT_BLEND_COLORALPHA | DSBLIT_SRC_PREMULTIPLY | DSBLIT_COLORIZE,
               0x80);
    primary_hardware->Blit( primary_hardware, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_hardware->Blit( primary_hardware, test_surface, NULL, test_rect.x, test_rect.y );
    primary_software->Blit( primary_software, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_software->Blit( primary_software, test_surface, NULL, test_rect.x, test_rect.y );
    flip_context( primary_info, primary_hardware, primary_software );
    get_user_input();

    /* Test 25% Alpha Color + Alpha Channel + PREMULTIPLY Blending + Colorize */
    sprintf(buf, "Test %d: Porter-Duff Source Blit with Premultiply Source Alpha + 25%% Alpha Color blending + Colorize", test_number++);
    display_information(primary_info, buf);
    
    setup_porter_duff_blits(primary_hardware, primary_software, DSPD_SRC,  DSBLIT_BLEND_ALPHACHANNEL | DSBLIT_BLEND_COLORALPHA | DSBLIT_SRC_PREMULTIPLY | DSBLIT_COLORIZE,
               0x40);
    primary_hardware->Blit( primary_hardware, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_hardware->Blit( primary_hardware, test_surface, NULL, test_rect.x, test_rect.y );
    primary_software->Blit( primary_software, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_software->Blit( primary_software, test_surface, NULL, test_rect.x, test_rect.y );
    flip_context( primary_info, primary_hardware, primary_software );
    get_user_input();
    
    /* Test 100% PREMULTCOLOR with Colorize */
    sprintf(buf, "Test %d: Porter-Duff Source Blit with 100%% PREMULTCOLOR on the Source with Colorize", test_number++);
    display_information(primary_info, buf);
    
    setup_porter_duff_blits(primary_hardware, primary_software, DSPD_SRC,  DSBLIT_SRC_PREMULTCOLOR | DSBLIT_COLORIZE, BLT_COLOR_A);
    primary_hardware->Blit( primary_hardware, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_hardware->Blit( primary_hardware, test_surface, NULL, test_rect.x, test_rect.y );
    primary_software->Blit( primary_software, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_software->Blit( primary_software, test_surface, NULL, test_rect.x, test_rect.y );
    flip_context( primary_info, primary_hardware, primary_software );
    get_user_input();
    
    /* Test 50% PREMULTCOLOR with Colorize */
    sprintf(buf, "Test %d: Porter-Duff Source Blit with 50%% PREMULTCOLOR on the Source with Colorize", test_number++);
    display_information(primary_info, buf);
    
    setup_porter_duff_blits(primary_hardware, primary_software, DSPD_SRC,  DSBLIT_SRC_PREMULTCOLOR | DSBLIT_COLORIZE, 0x80);
    primary_hardware->Blit( primary_hardware, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_hardware->Blit( primary_hardware, test_surface, NULL, test_rect.x, test_rect.y );
    primary_software->Blit( primary_software, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_software->Blit( primary_software, test_surface, NULL, test_rect.x, test_rect.y );
    flip_context( primary_info, primary_hardware, primary_software );
    get_user_input();
    
    /* Test 25% PREMULTCOLOR with Colorize */
    sprintf(buf, "Test %d: Porter-Duff Source Blit with 25%% PREMULTCOLOR on the Source with Colorize", test_number++);
    display_information(primary_info, buf);
    
    setup_porter_duff_blits(primary_hardware, primary_software, DSPD_SRC,  DSBLIT_SRC_PREMULTCOLOR | DSBLIT_COLORIZE, 0x40);
    primary_hardware->Blit( primary_hardware, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_hardware->Blit( primary_hardware, test_surface, NULL, test_rect.x, test_rect.y );
    primary_software->Blit( primary_software, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_software->Blit( primary_software, test_surface, NULL, test_rect.x, test_rect.y );
    flip_context( primary_info, primary_hardware, primary_software );
    get_user_input();
    
    /* Test 100% Alpha Color + Alpha Channel + 100% PREMULTCOLOR Blending + Colorize */ 
    sprintf(buf, "Test %d: Porter-Duff Source Blit with Source Alpha + 100%% Alpha Color + 100%% PREMULTCOLOR + Colorize", test_number++);
    display_information(primary_info, buf);
    
    setup_porter_duff_blits(primary_hardware, primary_software, DSPD_SRC, DSBLIT_BLEND_ALPHACHANNEL | DSBLIT_BLEND_COLORALPHA | DSBLIT_SRC_PREMULTCOLOR | DSBLIT_COLORIZE,
            BLT_COLOR_A);
    primary_hardware->Blit( primary_hardware, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_hardware->Blit( primary_hardware, test_surface, NULL, test_rect.x, test_rect.y );
    primary_software->Blit( primary_software, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_software->Blit( primary_software, test_surface, NULL, test_rect.x, test_rect.y );
    flip_context( primary_info, primary_hardware, primary_software );
    get_user_input();

    /* Test 50% Alpha Color + Alpha Channel + 50% PREMULTCOLOR Blending + Colorize */
    sprintf(buf, "Test %d: Porter-Duff Source Blit with Source Alpha + 50%% Alpha Color + 50%% PREMULTCOLOR + Colorize", test_number++);
    display_information(primary_info, buf);
    
    setup_porter_duff_blits(primary_hardware, primary_software, DSPD_SRC, DSBLIT_BLEND_ALPHACHANNEL | DSBLIT_BLEND_COLORALPHA | DSBLIT_SRC_PREMULTCOLOR | DSBLIT_COLORIZE,
            0x80);
    primary_hardware->Blit( primary_hardware, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_hardware->Blit( primary_hardware, test_surface, NULL, test_rect.x, test_rect.y );
    primary_software->Blit( primary_software, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_software->Blit( primary_software, test_surface, NULL, test_rect.x, test_rect.y );
    flip_context( primary_info, primary_hardware, primary_software );
    get_user_input();
    
    /* Test 25% Alpha Color + Alpha Channel + 25% PREMULTCOLOR Blending + Colorize */
    sprintf(buf, "Test %d: Porter-Duff Source Blit with Source Alpha + 25%% Alpha Color + 25%% PREMULTCOLOR + Colorize", test_number++);
    display_information(primary_info, buf);
    
    setup_porter_duff_blits(primary_hardware, primary_software, DSPD_SRC, DSBLIT_BLEND_ALPHACHANNEL | DSBLIT_BLEND_COLORALPHA | DSBLIT_SRC_PREMULTCOLOR | DSBLIT_COLORIZE,
            0x40);
    primary_hardware->Blit( primary_hardware, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_hardware->Blit( primary_hardware, test_surface, NULL, test_rect.x, test_rect.y );
    primary_software->Blit( primary_software, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_software->Blit( primary_software, test_surface, NULL, test_rect.x, test_rect.y );
    flip_context( primary_info, primary_hardware, primary_software );
    get_user_input();
    
    /* Test 100% Alpha Color + Alpha Channel + PREMULTIPLY + 100% PREMULTCOLOR Blending + Colorize */
    sprintf(buf, "Test %d: Porter-Duff Source Blit with Premultiply Source Alpha + 100%% Alpha Color + 100%% PREMULTCOLOR + Colorize", test_number++);
    display_information(primary_info, buf);
    
    setup_porter_duff_blits(primary_hardware, primary_software, DSPD_SRC, DSBLIT_BLEND_ALPHACHANNEL | DSBLIT_BLEND_COLORALPHA | DSBLIT_SRC_PREMULTIPLY |
                                                   DSBLIT_SRC_PREMULTCOLOR | DSBLIT_COLORIZE, BLT_COLOR_A);
    primary_hardware->Blit( primary_hardware, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_hardware->Blit( primary_hardware, test_surface, NULL, test_rect.x, test_rect.y );
    primary_software->Blit( primary_software, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_software->Blit( primary_software, test_surface, NULL, test_rect.x, test_rect.y );
    flip_context( primary_info, primary_hardware, primary_software );
    get_user_input();
    
    /* Test 50% Alpha Color + Alpha Channel + PREMULTIPLY + 50% PREMULTCOLOR Blending + Colorize */
    sprintf(buf, "Test %d: Porter-Duff Source Blit with Premultiply Source Alpha + 50%% Alpha Color + 50%% PREMULTCOLOR + Colorize", test_number++);
    display_information(primary_info, buf);
    
    setup_porter_duff_blits(primary_hardware, primary_software, DSPD_SRC, DSBLIT_BLEND_ALPHACHANNEL | DSBLIT_BLEND_COLORALPHA | DSBLIT_SRC_PREMULTIPLY |
                                                   DSBLIT_SRC_PREMULTCOLOR | DSBLIT_COLORIZE, 0x80);
    primary_hardware->Blit( primary_hardware, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_hardware->Blit( primary_hardware, test_surface, NULL, test_rect.x, test_rect.y );
    primary_software->Blit( primary_software, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_software->Blit( primary_software, test_surface, NULL, test_rect.x, test_rect.y );
    flip_context( primary_info, primary_hardware, primary_software );
    get_user_input();
    
    /* Test 25% Alpha Color + Alpha Channel + PREMULTIPLY + 25% PREMULTCOLOR Blending + Colorize */
    sprintf(buf, "Test %d: Porter-Duff Source Blit with Premultiply Source Alpha + 25%% Alpha Color + 25%% PREMULTCOLOR + Colorize", test_number++);
    display_information(primary_info, buf);
    
    setup_porter_duff_blits(primary_hardware, primary_software, DSPD_SRC, DSBLIT_BLEND_ALPHACHANNEL | DSBLIT_BLEND_COLORALPHA | DSBLIT_SRC_PREMULTIPLY |
                                                   DSBLIT_SRC_PREMULTCOLOR | DSBLIT_COLORIZE, 0x40);
    primary_hardware->Blit( primary_hardware, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_hardware->Blit( primary_hardware, test_surface, NULL, test_rect.x, test_rect.y );
    primary_software->Blit( primary_software, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_software->Blit( primary_software, test_surface, NULL, test_rect.x, test_rect.y );
    flip_context( primary_info, primary_hardware, primary_software );
    get_user_input();

    /*** Test with Porter-Duff Source Over */
    
     /* Test Flip Horizontally */
    sprintf(buf, "Test %d: Porter-Duff Source Over Blit with Horizontal Flip", test_number++);
    display_information(primary_info, buf);
    
    setup_porter_duff_blits(primary_hardware, primary_software, DSPD_SRC_OVER, DSBLIT_FLIP_HORIZONTAL, BLT_COLOR_A);
    primary_hardware->Blit( primary_hardware, logo_surface, NULL, logo_rect.x, logo_rect.y );
    primary_hardware->Blit( primary_hardware, test_surface, NULL,  test_rect.x, test_rect.y );
    primary_software->Blit( primary_software, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_software->Blit( primary_software, test_surface, NULL,  test_rect.x, test_rect.y );
    flip_context( primary_info, primary_hardware, primary_software );
    get_user_input();
    
     /* Test Flip Vertically */
    sprintf(buf, "Test %d: Porter-Duff Source Over Blit with Vertical Flip", test_number++);
    display_information(primary_info, buf);
    
    setup_porter_duff_blits(primary_hardware, primary_software, DSPD_SRC_OVER, DSBLIT_FLIP_VERTICAL, BLT_COLOR_A);
    primary_hardware->Blit( primary_hardware, logo_surface, NULL, logo_rect.x, logo_rect.y );
    primary_hardware->Blit( primary_hardware, test_surface, NULL,  test_rect.x, test_rect.y );
    primary_software->Blit( primary_software, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_software->Blit( primary_software, test_surface, NULL,  test_rect.x, test_rect.y );
    flip_context( primary_info, primary_hardware, primary_software );
    get_user_input();
    
     /* Test Rotate 180 */
    sprintf(buf, "Test %d: Porter-Duff Source Over Blit with 180 Degrees Rotation", test_number++);
    display_information(primary_info, buf);
    
    setup_porter_duff_blits(primary_hardware, primary_software, DSPD_SRC_OVER, DSBLIT_ROTATE180, BLT_COLOR_A);
    primary_hardware->Blit( primary_hardware, logo_surface, NULL, logo_rect.x, logo_rect.y );
    primary_hardware->Blit( primary_hardware, test_surface, NULL,  test_rect.x, test_rect.y );
    primary_software->Blit( primary_software, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_software->Blit( primary_software, test_surface, NULL,  test_rect.x, test_rect.y );
    flip_context( primary_info, primary_hardware, primary_software );
    get_user_input();
    
     /* Test Flip Horizontally with Alpha Channel Blending */
    sprintf(buf, "Test %d: Porter-Duff Source Over Blit with Horizontal Flip and Alpha Channel Blending", test_number++);
    display_information(primary_info, buf);
    
    setup_porter_duff_blits(primary_hardware, primary_software, DSPD_SRC_OVER, DSBLIT_FLIP_HORIZONTAL | DSBLIT_BLEND_ALPHACHANNEL, BLT_COLOR_A);
    primary_hardware->Blit( primary_hardware, logo_surface, NULL, logo_rect.x, logo_rect.y );
    primary_hardware->Blit( primary_hardware, test_surface, NULL,  test_rect.x, test_rect.y );
    primary_software->Blit( primary_software, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_software->Blit( primary_software, test_surface, NULL,  test_rect.x, test_rect.y );
    flip_context( primary_info, primary_hardware, primary_software );
    get_user_input();

     /* Test Flip Vertically with Alpha Channel Blending */
    sprintf(buf, "Test %d: Porter-Duff Source Over Blit with Vertical Flip and Alpha Channel Blending", test_number++);
    display_information(primary_info, buf);
    
    setup_porter_duff_blits(primary_hardware, primary_software, DSPD_SRC_OVER, DSBLIT_FLIP_VERTICAL | DSBLIT_BLEND_ALPHACHANNEL, BLT_COLOR_A);
    primary_hardware->Blit( primary_hardware, logo_surface, NULL, logo_rect.x, logo_rect.y );
    primary_hardware->Blit( primary_hardware, test_surface, NULL,  test_rect.x, test_rect.y );
    primary_software->Blit( primary_software, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_software->Blit( primary_software, test_surface, NULL,  test_rect.x, test_rect.y );
    flip_context( primary_info, primary_hardware, primary_software );
    get_user_input();
    
     /* Test Rotate 180 with Alpha Channel Blending */
    sprintf(buf, "Test %d: Porter-Duff Source Over Blit with 180 Degrees Rotation and Alpha Channel Blending", test_number++);
    display_information(primary_info, buf);
    
    setup_porter_duff_blits(primary_hardware, primary_software, DSPD_SRC_OVER, DSBLIT_ROTATE180 | DSBLIT_BLEND_ALPHACHANNEL, BLT_COLOR_A);
    primary_hardware->Blit( primary_hardware, logo_surface, NULL, logo_rect.x, logo_rect.y );
    primary_hardware->Blit( primary_hardware, test_surface, NULL,  test_rect.x, test_rect.y );
    primary_software->Blit( primary_software, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_software->Blit( primary_software, test_surface, NULL,  test_rect.x, test_rect.y );
    flip_context( primary_info, primary_hardware, primary_software );
    get_user_input();
    
     /* Test XOR */
    sprintf(buf, "Test %d: Porter-Duff Source Over Blit with XOR", test_number++);
    display_information(primary_info, buf);
    
    setup_porter_duff_blits(primary_hardware, primary_software, DSPD_SRC_OVER, DSBLIT_XOR, BLT_COLOR_A);
    primary_hardware->Blit( primary_hardware, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_hardware->Blit( primary_hardware, test_surface, NULL, test_rect.x, test_rect.y );
    primary_software->Blit( primary_software, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_software->Blit( primary_software, test_surface, NULL, test_rect.x, test_rect.y );
    flip_context( primary_info, primary_hardware, primary_software );
    get_user_input();

     /* Test XOR with Alpha Channel Blending */
    sprintf(buf, "Test %d: Porter-Duff Source Over Blit with XOR and Alpha Channel Blending", test_number++);
    display_information(primary_info, buf);
    
    setup_porter_duff_blits(primary_hardware, primary_software, DSPD_SRC_OVER, DSBLIT_XOR | DSBLIT_BLEND_ALPHACHANNEL, 0x80);
    primary_hardware->Blit( primary_hardware, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_hardware->Blit( primary_hardware, test_surface, NULL, test_rect.x, test_rect.y );
    primary_software->Blit( primary_software, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_software->Blit( primary_software, test_surface, NULL, test_rect.x, test_rect.y );
    flip_context( primary_info, primary_hardware, primary_software );
    get_user_input();
    
     /* Test XOR with Color Alpha Blending */
    sprintf(buf, "Test %d: Porter-Duff Source Over Blit with XOR and Color Alpha Blending", test_number++);
    display_information(primary_info, buf);
    
    setup_porter_duff_blits(primary_hardware, primary_software, DSPD_SRC_OVER, DSBLIT_XOR | DSBLIT_BLEND_COLORALPHA, 0x80);
    primary_hardware->Blit( primary_hardware, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_hardware->Blit( primary_hardware, test_surface, NULL, test_rect.x, test_rect.y );
    primary_software->Blit( primary_software, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_software->Blit( primary_software, test_surface, NULL, test_rect.x, test_rect.y );
    flip_context( primary_info, primary_hardware, primary_software );
    get_user_input();
    
     /* Test XOR with Alpha Channel and Color Alpha Blending */
    sprintf(buf, "Test %d: Porter-Duff Source Over Blit with XOR and Alpha Channel + Color Alpha Blending", test_number++);
    display_information(primary_info, buf);
    
    setup_porter_duff_blits(primary_hardware, primary_software, DSPD_SRC_OVER, DSBLIT_XOR | DSBLIT_BLEND_ALPHACHANNEL | DSBLIT_BLEND_COLORALPHA, 0x80);
    primary_hardware->Blit( primary_hardware, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_hardware->Blit( primary_hardware, test_surface, NULL, test_rect.x, test_rect.y );
    primary_software->Blit( primary_software, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_software->Blit( primary_software, test_surface, NULL, test_rect.x, test_rect.y );
    flip_context( primary_info, primary_hardware, primary_software );
    get_user_input();
    
     /* Test XOR with Premultiply and Alpha Channel and Color Alpha Blending */
    sprintf(buf, "Test %d: Porter-Duff Source Over Blit with XOR and Premultiply + Alpha Channel + Color Alpha Blending", test_number++);
    display_information(primary_info, buf);
    
    setup_porter_duff_blits(primary_hardware, primary_software, DSPD_SRC_OVER, DSBLIT_XOR | DSBLIT_BLEND_ALPHACHANNEL | DSBLIT_BLEND_COLORALPHA | DSBLIT_SRC_PREMULTIPLY, 0x80);
    primary_hardware->Blit( primary_hardware, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_hardware->Blit( primary_hardware, test_surface, NULL, test_rect.x, test_rect.y );
    primary_software->Blit( primary_software, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_software->Blit( primary_software, test_surface, NULL, test_rect.x, test_rect.y );
    flip_context( primary_info, primary_hardware, primary_software );
    get_user_input();

     /* Test XOR with Premultiply and Alpha Channel and Color Alpha Blending and PremultColor */
    sprintf(buf, "Test %d: Porter-Duff Source Over Blit with XOR + Premultiply + Alpha Channel + Color Alpha + PREMULTCOLOR Blending", test_number++);
    display_information(primary_info, buf);
    
    setup_porter_duff_blits(primary_hardware, primary_software, DSPD_SRC_OVER, DSBLIT_XOR | DSBLIT_BLEND_ALPHACHANNEL | DSBLIT_BLEND_COLORALPHA |
                                                    DSBLIT_SRC_PREMULTIPLY | DSBLIT_SRC_PREMULTCOLOR, 0x80);
    primary_hardware->Blit( primary_hardware, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_hardware->Blit( primary_hardware, test_surface, NULL, test_rect.x, test_rect.y );
    primary_software->Blit( primary_software, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_software->Blit( primary_software, test_surface, NULL, test_rect.x, test_rect.y );
    flip_context( primary_info, primary_hardware, primary_software );
    get_user_input();
    
     /* Test XOR with Premultiply and Alpha Channel and Color Alpha Blending and PremultColor and Colorize */
    sprintf(buf, "Test %d: Porter-Duff Source Over Blit with XOR + Premultiply + Alpha Channel + Color Alpha + PREMULTCOLOR + Colorize Blending", test_number++);
    display_information(primary_info, buf);
    
    setup_porter_duff_blits(primary_hardware, primary_software, DSPD_SRC_OVER, DSBLIT_XOR | DSBLIT_BLEND_ALPHACHANNEL | DSBLIT_BLEND_COLORALPHA |
                                                    DSBLIT_SRC_PREMULTIPLY | DSBLIT_SRC_PREMULTCOLOR | DSBLIT_COLORIZE, 0x80);
    primary_hardware->Blit( primary_hardware, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_hardware->Blit( primary_hardware, test_surface, NULL, test_rect.x, test_rect.y );
    primary_software->Blit( primary_software, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_software->Blit( primary_software, test_surface, NULL, test_rect.x, test_rect.y );
    flip_context( primary_info, primary_hardware, primary_software );
    get_user_input();
    
     /* Test Alpha Channel Blending */
    sprintf(buf, "Test %d: Porter-Duff Source Over Blit with Alpha Channel Blending", test_number++);
    display_information(primary_info, buf);
    
    setup_porter_duff_blits(primary_hardware, primary_software, DSPD_SRC_OVER, DSBLIT_BLEND_ALPHACHANNEL, BLT_COLOR_A);
    
    primary_hardware->Blit( primary_hardware, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_hardware->Blit( primary_hardware, test_surface, NULL, test_rect.x, test_rect.y );
    primary_software->Blit( primary_software, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_software->Blit( primary_software, test_surface, NULL, test_rect.x, test_rect.y );
    flip_context( primary_info, primary_hardware, primary_software );
    get_user_input();

    /* Test 100% Color Alpha Blending */
    sprintf(buf, "Test %d: Porter-Duff Source Over Blit with 100%% Color Alpha Blending", test_number++);
    display_information(primary_info, buf);
    
    setup_porter_duff_blits(primary_hardware, primary_software, DSPD_SRC_OVER, DSBLIT_BLEND_COLORALPHA, BLT_COLOR_A);
    primary_hardware->Blit( primary_hardware, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_hardware->Blit( primary_hardware, test_surface, NULL, test_rect.x, test_rect.y );
    primary_software->Blit( primary_software, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_software->Blit( primary_software, test_surface, NULL, test_rect.x, test_rect.y );
    flip_context( primary_info, primary_hardware, primary_software );
    get_user_input();
    
    /* Test 50% Color Alpha Blending */
    sprintf(buf, "Test %d: Porter-Duff Source Over Blit with 50%% Color Alpha Blending", test_number++);
    display_information(primary_info, buf);
    
    setup_porter_duff_blits(primary_hardware, primary_software, DSPD_SRC_OVER, DSBLIT_BLEND_COLORALPHA, 0x80);
    primary_hardware->Blit( primary_hardware, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_hardware->Blit( primary_hardware, test_surface, NULL, test_rect.x, test_rect.y );
    primary_software->Blit( primary_software, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_software->Blit( primary_software, test_surface, NULL, test_rect.x, test_rect.y );
    flip_context( primary_info, primary_hardware, primary_software );
    get_user_input();
    
     /* Test 25% Color Alpha Blending */
    sprintf(buf, "Test %d: Porter-Duff Source Over Blit with 25%% Color Alpha Blending", test_number++);
    display_information(primary_info, buf);
    
    setup_porter_duff_blits(primary_hardware, primary_software, DSPD_SRC_OVER, DSBLIT_BLEND_COLORALPHA, 0x40);
    primary_hardware->Blit( primary_hardware, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_hardware->Blit( primary_hardware, test_surface, NULL, test_rect.x, test_rect.y );
    primary_software->Blit( primary_software, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_software->Blit( primary_software, test_surface, NULL, test_rect.x, test_rect.y );
    flip_context( primary_info, primary_hardware, primary_software );
    get_user_input();
    
    /* Test 100% Color Alpha + Alpha Channel Blending */
    sprintf(buf, "Test %d: Porter-Duff Source Over Blit with 100%% Color Alpha + Alpha Blend", test_number++);
    display_information(primary_info, buf);
    
    setup_porter_duff_blits(primary_hardware, primary_software, DSPD_SRC_OVER, DSBLIT_BLEND_COLORALPHA|DSBLIT_BLEND_ALPHACHANNEL, BLT_COLOR_A);
    primary_hardware->Blit( primary_hardware, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_hardware->Blit( primary_hardware, test_surface, NULL, test_rect.x, test_rect.y );
    primary_software->Blit( primary_software, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_software->Blit( primary_software, test_surface, NULL, test_rect.x, test_rect.y );
    flip_context( primary_info, primary_hardware, primary_software );
    get_user_input();
    
    /* Test 50% Color Alpha + Alpha Channel Blending */
    sprintf(buf, "Test %d: Porter-Duff Source Over Blit with 50%% Color Alpha + Alpha Blend", test_number++);
    display_information(primary_info, buf);
    
    setup_porter_duff_blits(primary_hardware, primary_software, DSPD_SRC_OVER, DSBLIT_BLEND_COLORALPHA|DSBLIT_BLEND_ALPHACHANNEL, 0x80);
    primary_hardware->Blit( primary_hardware, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_hardware->Blit( primary_hardware, test_surface, NULL, test_rect.x, test_rect.y );
    primary_software->Blit( primary_software, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_software->Blit( primary_software, test_surface, NULL, test_rect.x, test_rect.y );
    flip_context( primary_info, primary_hardware, primary_software );
    get_user_input();

    /* Test 25% Color Alpha + Alpha Channel Blending */
    sprintf(buf, "Test %d: Porter-Duff Source Over Blit with 25%% Color Alpha + Alpha Blend", test_number++);
    display_information(primary_info, buf);
    
    setup_porter_duff_blits(primary_hardware, primary_software, DSPD_SRC_OVER, DSBLIT_BLEND_COLORALPHA|DSBLIT_BLEND_ALPHACHANNEL, 0x40);
    primary_hardware->Blit( primary_hardware, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_hardware->Blit( primary_hardware, test_surface, NULL, test_rect.x, test_rect.y );
    primary_software->Blit( primary_software, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_software->Blit( primary_software, test_surface, NULL, test_rect.x, test_rect.y );
    flip_context( primary_info, primary_hardware, primary_software );
    get_user_input();
    
    /* Test PREMULTIPLY */
    sprintf(buf, "Test %d: Porter-Duff Source Over Blit with Premultiply Source", test_number++);
    display_information(primary_info, buf);
    
    setup_porter_duff_blits(primary_hardware, primary_software, DSPD_SRC_OVER, DSBLIT_SRC_PREMULTIPLY, 0x80);
    primary_hardware->Blit( primary_hardware, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_hardware->Blit( primary_hardware, test_surface, NULL, test_rect.x, test_rect.y );
    primary_software->Blit( primary_software, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_software->Blit( primary_software, test_surface, NULL, test_rect.x, test_rect.y );
    flip_context( primary_info, primary_hardware, primary_software );
    get_user_input();
    
    /* Test Alpha Channel + PREMULTIPLY Blending */
    sprintf(buf, "Test %d: Porter-Duff Source Over Blit with Premultiply Source Alpha Blending", test_number++);
    display_information(primary_info, buf);
    
    setup_porter_duff_blits(primary_hardware, primary_software, DSPD_SRC_OVER, DSBLIT_BLEND_ALPHACHANNEL | DSBLIT_SRC_PREMULTIPLY, 0x80);
    primary_hardware->Blit( primary_hardware, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_hardware->Blit( primary_hardware, test_surface, NULL, test_rect.x, test_rect.y );
    primary_software->Blit( primary_software, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_software->Blit( primary_software, test_surface, NULL, test_rect.x, test_rect.y );
    flip_context( primary_info, primary_hardware, primary_software );
    get_user_input();
    
     /* Test 100% Alpha Color + Alpha Channel + PREMULTIPLY Blending */
    sprintf(buf, "Test %d: Porter-Duff Source Over Blit with Premultiply Source Alpha + 100%% Alpha Color blending", test_number++);
    display_information(primary_info, buf);
    
    setup_porter_duff_blits(primary_hardware, primary_software, DSPD_SRC_OVER, DSBLIT_BLEND_ALPHACHANNEL | DSBLIT_BLEND_COLORALPHA | DSBLIT_SRC_PREMULTIPLY, BLT_COLOR_A);
    primary_hardware->Blit( primary_hardware, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_hardware->Blit( primary_hardware, test_surface, NULL, test_rect.x, test_rect.y );
    primary_software->Blit( primary_software, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_software->Blit( primary_software, test_surface, NULL, test_rect.x, test_rect.y );
    flip_context( primary_info, primary_hardware, primary_software );
    get_user_input();
    
     /* Test 50% Alpha Color + Alpha Channel + PREMULTIPLY Blending */
    sprintf(buf, "Test %d: Porter-Duff Source Over Blit with Premultiply Source Alpha + 50%% Alpha Color blending", test_number++);
    display_information(primary_info, buf);
    
    setup_porter_duff_blits(primary_hardware, primary_software, DSPD_SRC_OVER, DSBLIT_BLEND_ALPHACHANNEL | DSBLIT_BLEND_COLORALPHA | DSBLIT_SRC_PREMULTIPLY, 0x80);
    primary_hardware->Blit( primary_hardware, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_hardware->Blit( primary_hardware, test_surface, NULL, test_rect.x, test_rect.y );
    primary_software->Blit( primary_software, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_software->Blit( primary_software, test_surface, NULL, test_rect.x, test_rect.y );
    flip_context( primary_info, primary_hardware, primary_software );
    get_user_input();

     /* Test 25% Alpha Color + Alpha Channel + PREMULTIPLY Blending */
    sprintf(buf, "Test %d: Porter-Duff Source Over Blit with Premultiply Source Alpha + 25%% Alpha Color blending", test_number++);
    display_information(primary_info, buf);
    
    setup_porter_duff_blits(primary_hardware, primary_software, DSPD_SRC_OVER, DSBLIT_BLEND_ALPHACHANNEL | DSBLIT_BLEND_COLORALPHA | DSBLIT_SRC_PREMULTIPLY, 0x40);
    primary_hardware->Blit( primary_hardware, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_hardware->Blit( primary_hardware, test_surface, NULL, test_rect.x, test_rect.y );
    primary_software->Blit( primary_software, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_software->Blit( primary_software, test_surface, NULL, test_rect.x, test_rect.y );
    flip_context( primary_info, primary_hardware, primary_software );
    get_user_input();
    
    /* Test 100% PREMULTCOLOR */
    sprintf(buf, "Test %d: Porter-Duff Source Over Blit with 100%% PREMULTCOLOR on the Source", test_number++);
    display_information(primary_info, buf);
    
    setup_porter_duff_blits(primary_hardware, primary_software, DSPD_SRC_OVER, DSBLIT_SRC_PREMULTCOLOR, BLT_COLOR_A);
    primary_hardware->Blit( primary_hardware, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_hardware->Blit( primary_hardware, test_surface, NULL, test_rect.x, test_rect.y );
    primary_software->Blit( primary_software, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_software->Blit( primary_software, test_surface, NULL, test_rect.x, test_rect.y );
    flip_context( primary_info, primary_hardware, primary_software );
    get_user_input();
    
    /* Test 50% PREMULTCOLOR */
    sprintf(buf, "Test %d: Porter-Duff Source Over Blit with 50%% PREMULTCOLOR on the Source", test_number++);
    display_information(primary_info, buf);
    
    setup_porter_duff_blits(primary_hardware, primary_software, DSPD_SRC_OVER, DSBLIT_SRC_PREMULTCOLOR, 0x80);
    primary_hardware->Blit( primary_hardware, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_hardware->Blit( primary_hardware, test_surface, NULL, test_rect.x, test_rect.y );
    primary_software->Blit( primary_software, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_software->Blit( primary_software, test_surface, NULL, test_rect.x, test_rect.y );
    flip_context( primary_info, primary_hardware, primary_software );
    get_user_input();
    
    /* Test 25% PREMULTCOLOR */
    sprintf(buf, "Test %d: Porter-Duff Source Over Blit with 25%% PREMULTCOLOR on the Source", test_number++);
    display_information(primary_info, buf);
    
    setup_porter_duff_blits(primary_hardware, primary_software, DSPD_SRC_OVER, DSBLIT_SRC_PREMULTCOLOR, 0x40);
    primary_hardware->Blit( primary_hardware, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_hardware->Blit( primary_hardware, test_surface, NULL, test_rect.x, test_rect.y );
    primary_software->Blit( primary_software, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_software->Blit( primary_software, test_surface, NULL, test_rect.x, test_rect.y );
    flip_context( primary_info, primary_hardware, primary_software );
    get_user_input();

    /* Test 100% Alpha Color + Alpha Channel + 100% PREMULTCOLOR Blending */
    sprintf(buf, "Test %d: Porter-Duff Source Over Blit with Source Alpha + 100%% Alpha Color + 100%% PREMULTCOLOR", test_number++);
    display_information(primary_info, buf);
    
    setup_porter_duff_blits(primary_hardware, primary_software, DSPD_SRC_OVER, DSBLIT_BLEND_ALPHACHANNEL | DSBLIT_BLEND_COLORALPHA | DSBLIT_SRC_PREMULTCOLOR, BLT_COLOR_A);
    primary_hardware->Blit( primary_hardware, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_hardware->Blit( primary_hardware, test_surface, NULL, test_rect.x, test_rect.y );
    primary_software->Blit( primary_software, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_software->Blit( primary_software, test_surface, NULL, test_rect.x, test_rect.y );
    flip_context( primary_info, primary_hardware, primary_software );
    get_user_input();
    
    /* Test 50% Alpha Color + Alpha Channel + 50% PREMULTCOLOR Blending */
    sprintf(buf, "Test %d: Porter-Duff Source Over Blit with Source Alpha + 50%% Alpha Color + 50%% PREMULTCOLOR", test_number++);
    display_information(primary_info, buf);
    
    setup_porter_duff_blits(primary_hardware, primary_software, DSPD_SRC_OVER, DSBLIT_BLEND_ALPHACHANNEL | DSBLIT_BLEND_COLORALPHA | DSBLIT_SRC_PREMULTCOLOR, 0x80);
    primary_hardware->Blit( primary_hardware, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_hardware->Blit( primary_hardware, test_surface, NULL, test_rect.x, test_rect.y );
    primary_software->Blit( primary_software, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_software->Blit( primary_software, test_surface, NULL, test_rect.x, test_rect.y );
    flip_context( primary_info, primary_hardware, primary_software );
    get_user_input();
    
    /* Test 25% Alpha Color + Alpha Channel + 25% PREMULTCOLOR Blending */
    sprintf(buf, "Test %d: Porter-Duff Source Over Blit with Source Alpha + 25%% Alpha Color + 25%% PREMULTCOLOR", test_number++);
    display_information(primary_info, buf);
    
    setup_porter_duff_blits(primary_hardware, primary_software, DSPD_SRC_OVER, DSBLIT_BLEND_ALPHACHANNEL | DSBLIT_BLEND_COLORALPHA | DSBLIT_SRC_PREMULTCOLOR, 0x40);
    primary_hardware->Blit( primary_hardware, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_hardware->Blit( primary_hardware, test_surface, NULL, test_rect.x, test_rect.y );
    primary_software->Blit( primary_software, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_software->Blit( primary_software, test_surface, NULL, test_rect.x, test_rect.y );
    flip_context( primary_info, primary_hardware, primary_software );
    get_user_input();

    /* Test 100% Alpha Color + Alpha Channel + PREMULTIPLY + 100% PREMULTCOLOR Blending */
    sprintf(buf, "Test %d: Porter-Duff Source Over Blit with Premultiply Source Alpha + 100%% Alpha Color + 100%% PREMULTCOLOR", test_number++);
    display_information(primary_info, buf);
    
    setup_porter_duff_blits(primary_hardware, primary_software, DSPD_SRC_OVER, DSBLIT_BLEND_ALPHACHANNEL | DSBLIT_BLEND_COLORALPHA | DSBLIT_SRC_PREMULTIPLY | DSBLIT_SRC_PREMULTCOLOR,
            BLT_COLOR_A);
    primary_hardware->Blit( primary_hardware, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_hardware->Blit( primary_hardware, test_surface, NULL, test_rect.x, test_rect.y );
    primary_software->Blit( primary_software, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_software->Blit( primary_software, test_surface, NULL, test_rect.x, test_rect.y );
    flip_context( primary_info, primary_hardware, primary_software );
    get_user_input();
    
    /* Test 50% Alpha Color + Alpha Channel + PREMULTIPLY + 50% PREMULTCOLOR Blending */
    sprintf(buf, "Test %d: Porter-Duff Source Over Blit with Premultiply Source Alpha + 50%% Alpha Color + 50%% PREMULTCOLOR", test_number++);
    display_information(primary_info, buf);
    
    setup_porter_duff_blits(primary_hardware, primary_software, DSPD_SRC_OVER, DSBLIT_BLEND_ALPHACHANNEL | DSBLIT_BLEND_COLORALPHA | DSBLIT_SRC_PREMULTIPLY | DSBLIT_SRC_PREMULTCOLOR,
            0x80);
    primary_hardware->Blit( primary_hardware, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_hardware->Blit( primary_hardware, test_surface, NULL, test_rect.x, test_rect.y );
    primary_software->Blit( primary_software, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_software->Blit( primary_software, test_surface, NULL, test_rect.x, test_rect.y );
    flip_context( primary_info, primary_hardware, primary_software );
    get_user_input();
    
    /* Test 25% Alpha Color + Alpha Channel + PREMULTIPLY + 25% PREMULTCOLOR Blending */
    sprintf(buf, "Test %d: Porter-Duff Source Over Blit with Premultiply Source Alpha + 25%% Alpha Color + 25%% PREMULTCOLOR", test_number++);
    display_information(primary_info, buf);
    
    setup_porter_duff_blits(primary_hardware, primary_software, DSPD_SRC_OVER, DSBLIT_BLEND_ALPHACHANNEL | DSBLIT_BLEND_COLORALPHA | DSBLIT_SRC_PREMULTIPLY | DSBLIT_SRC_PREMULTCOLOR,
            0x40);
    primary_hardware->Blit( primary_hardware, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_hardware->Blit( primary_hardware, test_surface, NULL, test_rect.x, test_rect.y );
    primary_software->Blit( primary_software, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_software->Blit( primary_software, test_surface, NULL, test_rect.x, test_rect.y );
    flip_context( primary_info, primary_hardware, primary_software );
    get_user_input();
    
    /* Test Alpha Channel Blending with Colorize */
    sprintf(buf, "Test %d: Porter-Duff Source Over Blit with Source Alpha Blending with Colorize", test_number++);
    display_information(primary_info, buf);
    
    setup_porter_duff_blits(primary_hardware, primary_software, DSPD_SRC_OVER, DSBLIT_BLEND_ALPHACHANNEL | DSBLIT_COLORIZE, 0x80);
    primary_hardware->Blit( primary_hardware, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_hardware->Blit( primary_hardware, test_surface, NULL, test_rect.x, test_rect.y );
    primary_software->Blit( primary_software, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_software->Blit( primary_software, test_surface, NULL, test_rect.x, test_rect.y );
    flip_context( primary_info, primary_hardware, primary_software );
    get_user_input();
    
    /* Test 100% Color Alpha Blending with Colorize */
    sprintf(buf, "Test %d: Porter-Duff Source Over Blit with 100%% Color Alpha Blending with Colorize", test_number++);
    display_information(primary_info, buf);
    
    setup_porter_duff_blits(primary_hardware, primary_software, DSPD_SRC_OVER, DSBLIT_BLEND_COLORALPHA | DSBLIT_COLORIZE, BLT_COLOR_A);
    primary_hardware->Blit( primary_hardware, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_hardware->Blit( primary_hardware, test_surface, NULL, test_rect.x, test_rect.y );
    primary_software->Blit( primary_software, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_software->Blit( primary_software, test_surface, NULL, test_rect.x, test_rect.y );
    flip_context( primary_info, primary_hardware, primary_software );
    get_user_input();
    
    /* Test 50% Color Alpha Blending with Colorize */
    sprintf(buf, "Test %d: Porter-Duff Source Over Blit with 50%% Color Alpha Blending with Colorize", test_number++);
    display_information(primary_info, buf);
    
    setup_porter_duff_blits(primary_hardware, primary_software, DSPD_SRC_OVER, DSBLIT_BLEND_COLORALPHA | DSBLIT_COLORIZE, 0x80);
    primary_hardware->Blit( primary_hardware, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_hardware->Blit( primary_hardware, test_surface, NULL, test_rect.x, test_rect.y );
    primary_software->Blit( primary_software, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_software->Blit( primary_software, test_surface, NULL, test_rect.x, test_rect.y );
    flip_context( primary_info, primary_hardware, primary_software );
    get_user_input();

    /* Test 25% Color Alpha Blending with Colorize */
    sprintf(buf, "Test %d: Porter-Duff Source Over Blit with 25%% Color Alpha Blending with Colorize", test_number++);
    display_information(primary_info, buf);
    
    setup_porter_duff_blits(primary_hardware, primary_software, DSPD_SRC_OVER, DSBLIT_BLEND_COLORALPHA | DSBLIT_COLORIZE, 0x40);
    primary_hardware->Blit( primary_hardware, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_hardware->Blit( primary_hardware, test_surface, NULL, test_rect.x, test_rect.y );
    primary_software->Blit( primary_software, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_software->Blit( primary_software, test_surface, NULL, test_rect.x, test_rect.y );
    flip_context( primary_info, primary_hardware, primary_software );
    get_user_input();
    
    /* Test 100% Color Alpha + Alpha Channel Blending + Colorize */
    sprintf(buf, "Test %d: Porter-Duff Source Over Blit with 100%% Color Alpha + Source Alpha Blending + Colorize", test_number++);
    display_information(primary_info, buf);
    
    setup_porter_duff_blits(primary_hardware, primary_software, DSPD_SRC_OVER, DSBLIT_BLEND_COLORALPHA | DSBLIT_BLEND_ALPHACHANNEL | DSBLIT_COLORIZE, BLT_COLOR_A);
    primary_hardware->Blit( primary_hardware, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_hardware->Blit( primary_hardware, test_surface, NULL, test_rect.x, test_rect.y );
    primary_software->Blit( primary_software, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_software->Blit( primary_software, test_surface, NULL, test_rect.x, test_rect.y );
    flip_context( primary_info, primary_hardware, primary_software );
    get_user_input();
    
    /* Test 50% Color Alpha + Alpha Channel Blending + Colorize */
    sprintf(buf, "Test %d: Porter-Duff Source Over Blit with 50%% Color Alpha + Source Alpha Blending + Colorize", test_number++);
    display_information(primary_info, buf);
    
    setup_porter_duff_blits(primary_hardware, primary_software, DSPD_SRC_OVER, DSBLIT_BLEND_COLORALPHA | DSBLIT_BLEND_ALPHACHANNEL | DSBLIT_COLORIZE, 0x80);
    primary_hardware->Blit( primary_hardware, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_hardware->Blit( primary_hardware, test_surface, NULL, test_rect.x, test_rect.y );
    primary_software->Blit( primary_software, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_software->Blit( primary_software, test_surface, NULL, test_rect.x, test_rect.y );
    flip_context( primary_info, primary_hardware, primary_software );
    get_user_input();
    
    /* Test 25% Color Alpha + Alpha Channel Blending + Colorize */
    sprintf(buf, "Test %d: Porter-Duff Source Over Blit with 25%% Color Alpha + Source Alpha Blending + Colorize", test_number++);
    display_information(primary_info, buf);
    
    setup_porter_duff_blits(primary_hardware, primary_software, DSPD_SRC_OVER, DSBLIT_BLEND_COLORALPHA | DSBLIT_BLEND_ALPHACHANNEL | DSBLIT_COLORIZE, 0x40);
    primary_hardware->Blit( primary_hardware, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_hardware->Blit( primary_hardware, test_surface, NULL, test_rect.x, test_rect.y );
    primary_software->Blit( primary_software, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_software->Blit( primary_software, test_surface, NULL, test_rect.x, test_rect.y );
    flip_context( primary_info, primary_hardware, primary_software );
    get_user_input();
    
    /* Test PREMULTIPLY with Colorize */
    sprintf(buf, "Test %d: Porter-Duff Source Over Blit with Premultiply Source with Colorize", test_number++);
    display_information(primary_info, buf);
    
    setup_porter_duff_blits(primary_hardware, primary_software, DSPD_SRC_OVER, DSBLIT_SRC_PREMULTIPLY | DSBLIT_COLORIZE, 0x80);
    primary_hardware->Blit( primary_hardware, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_hardware->Blit( primary_hardware, test_surface, NULL, test_rect.x, test_rect.y );
    primary_software->Blit( primary_software, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_software->Blit( primary_software, test_surface, NULL, test_rect.x, test_rect.y );
    flip_context( primary_info, primary_hardware, primary_software );
    get_user_input();
    
    /* Test Alpha Channel + PREMULTIPLY Blending + Colorize */
    sprintf(buf, "Test %d: Porter-Duff Source Over Blit with Premultiply Source Alpha Blending with Colorize", test_number++);
    display_information(primary_info, buf);
    
    setup_porter_duff_blits(primary_hardware, primary_software, DSPD_SRC_OVER,  DSBLIT_BLEND_ALPHACHANNEL | DSBLIT_SRC_PREMULTIPLY | DSBLIT_COLORIZE, 0x80);
    primary_hardware->Blit( primary_hardware, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_hardware->Blit( primary_hardware, test_surface, NULL, test_rect.x, test_rect.y );
    primary_software->Blit( primary_software, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_software->Blit( primary_software, test_surface, NULL, test_rect.x, test_rect.y );
    flip_context( primary_info, primary_hardware, primary_software );
    get_user_input();

    /* Test 100% Alpha Color + Alpha Channel + PREMULTIPLY Blending + Colorize */
    sprintf(buf, "Test %d: Porter-Duff Source Over Blit with Premultiply Source Alpha + 100%% Alpha Color blending + Colorize", test_number++);
    display_information(primary_info, buf);
    
    setup_porter_duff_blits(primary_hardware, primary_software, DSPD_SRC_OVER,  DSBLIT_BLEND_ALPHACHANNEL | DSBLIT_BLEND_COLORALPHA | DSBLIT_SRC_PREMULTIPLY | DSBLIT_COLORIZE,
               BLT_COLOR_A);
    primary_hardware->Blit( primary_hardware, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_hardware->Blit( primary_hardware, test_surface, NULL, test_rect.x, test_rect.y );
    primary_software->Blit( primary_software, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_software->Blit( primary_software, test_surface, NULL, test_rect.x, test_rect.y );
    flip_context( primary_info, primary_hardware, primary_software );
    get_user_input();
    
    /* Test 50% Alpha Color + Alpha Channel + PREMULTIPLY Blending + Colorize */
    sprintf(buf, "Test %d: Porter-Duff Source Over Blit with Premultiply Source Alpha + 50%% Alpha Color blending + Colorize", test_number++);
    display_information(primary_info, buf);
    
    setup_porter_duff_blits(primary_hardware, primary_software, DSPD_SRC_OVER,  DSBLIT_BLEND_ALPHACHANNEL | DSBLIT_BLEND_COLORALPHA | DSBLIT_SRC_PREMULTIPLY | DSBLIT_COLORIZE,
               0x80);
    primary_hardware->Blit( primary_hardware, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_hardware->Blit( primary_hardware, test_surface, NULL, test_rect.x, test_rect.y );
    primary_software->Blit( primary_software, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_software->Blit( primary_software, test_surface, NULL, test_rect.x, test_rect.y );
    flip_context( primary_info, primary_hardware, primary_software );
    get_user_input();
    
    /* Test 25% Alpha Color + Alpha Channel + PREMULTIPLY Blending + Colorize */
    sprintf(buf, "Test %d: Porter-Duff Source Over Blit with Premultiply Source Alpha + 25%% Alpha Color blending + Colorize", test_number++);
    display_information(primary_info, buf);
    
    setup_porter_duff_blits(primary_hardware, primary_software, DSPD_SRC_OVER,  DSBLIT_BLEND_ALPHACHANNEL | DSBLIT_BLEND_COLORALPHA | DSBLIT_SRC_PREMULTIPLY | DSBLIT_COLORIZE,
               0x40);
    primary_hardware->Blit( primary_hardware, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_hardware->Blit( primary_hardware, test_surface, NULL, test_rect.x, test_rect.y );
    primary_software->Blit( primary_software, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_software->Blit( primary_software, test_surface, NULL, test_rect.x, test_rect.y );
    flip_context( primary_info, primary_hardware, primary_software );
    get_user_input();
    
    /* Test 100% PREMULTCOLOR with Colorize */
    sprintf(buf, "Test %d: Porter-Duff Source Over Blit with 100%% PREMULTCOLOR on the Source with Colorize", test_number++);
    display_information(primary_info, buf);
    
    setup_porter_duff_blits(primary_hardware, primary_software, DSPD_SRC_OVER,  DSBLIT_SRC_PREMULTCOLOR | DSBLIT_COLORIZE, BLT_COLOR_A);
    primary_hardware->Blit( primary_hardware, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_hardware->Blit( primary_hardware, test_surface, NULL, test_rect.x, test_rect.y );
    primary_software->Blit( primary_software, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_software->Blit( primary_software, test_surface, NULL, test_rect.x, test_rect.y );
    flip_context( primary_info, primary_hardware, primary_software );
    get_user_input();
    
    /* Test 50% PREMULTCOLOR with Colorize */
    sprintf(buf, "Test %d: Porter-Duff Source Over Blit with 50%% PREMULTCOLOR on the Source with Colorize", test_number++);
    display_information(primary_info, buf);
    
    setup_porter_duff_blits(primary_hardware, primary_software, DSPD_SRC_OVER,  DSBLIT_SRC_PREMULTCOLOR | DSBLIT_COLORIZE, 0x80);
    primary_hardware->Blit( primary_hardware, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_hardware->Blit( primary_hardware, test_surface, NULL, test_rect.x, test_rect.y );
    primary_software->Blit( primary_software, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_software->Blit( primary_software, test_surface, NULL, test_rect.x, test_rect.y );
    flip_context( primary_info, primary_hardware, primary_software );
    get_user_input();
    
    /* Test 25% PREMULTCOLOR with Colorize */
    sprintf(buf, "Test %d: Porter-Duff Source Over Blit with 25%% PREMULTCOLOR on the Source with Colorize", test_number++);
    display_information(primary_info, buf);
    
    setup_porter_duff_blits(primary_hardware, primary_software, DSPD_SRC_OVER,  DSBLIT_SRC_PREMULTCOLOR | DSBLIT_COLORIZE, 0x40);
    primary_hardware->Blit( primary_hardware, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_hardware->Blit( primary_hardware, test_surface, NULL, test_rect.x, test_rect.y );
    primary_software->Blit( primary_software, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_software->Blit( primary_software, test_surface, NULL, test_rect.x, test_rect.y );
    flip_context( primary_info, primary_hardware, primary_software );
    get_user_input();
    
    /* Test 100% Alpha Color + Alpha Channel + 100% PREMULTCOLOR Blending + Colorize */ 
    sprintf(buf, "Test %d: Porter-Duff Source Over Blit with Source Alpha + 100%% Alpha Color + 100%% PREMULTCOLOR + Colorize", test_number++);
    display_information(primary_info, buf);
    
    setup_porter_duff_blits(primary_hardware, primary_software, DSPD_SRC_OVER, DSBLIT_BLEND_ALPHACHANNEL | DSBLIT_BLEND_COLORALPHA | DSBLIT_SRC_PREMULTCOLOR | DSBLIT_COLORIZE,
            BLT_COLOR_A);
    primary_hardware->Blit( primary_hardware, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_hardware->Blit( primary_hardware, test_surface, NULL, test_rect.x, test_rect.y );
    primary_software->Blit( primary_software, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_software->Blit( primary_software, test_surface, NULL, test_rect.x, test_rect.y );
    flip_context( primary_info, primary_hardware, primary_software );
    get_user_input();
    
    /* Test 50% Alpha Color + Alpha Channel + 50% PREMULTCOLOR Blending + Colorize */
    sprintf(buf, "Test %d: Porter-Duff Source Over Blit with Source Alpha + 50%% Alpha Color + 50%% PREMULTCOLOR + Colorize", test_number++);
    display_information(primary_info, buf);
    
    setup_porter_duff_blits(primary_hardware, primary_software, DSPD_SRC_OVER, DSBLIT_BLEND_ALPHACHANNEL | DSBLIT_BLEND_COLORALPHA | DSBLIT_SRC_PREMULTCOLOR | DSBLIT_COLORIZE,
            0x80);
    primary_hardware->Blit( primary_hardware, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_hardware->Blit( primary_hardware, test_surface, NULL, test_rect.x, test_rect.y );
    primary_software->Blit( primary_software, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_software->Blit( primary_software, test_surface, NULL, test_rect.x, test_rect.y );
    flip_context( primary_info, primary_hardware, primary_software );
    get_user_input();
    
    /* Test 25% Alpha Color + Alpha Channel + 25% PREMULTCOLOR Blending + Colorize */
    sprintf(buf, "Test %d: Porter-Duff Source Over Blit with Source Alpha + 25%% Alpha Color + 25%% PREMULTCOLOR + Colorize", test_number++);
    display_information(primary_info, buf);
    
    setup_porter_duff_blits(primary_hardware, primary_software, DSPD_SRC_OVER, DSBLIT_BLEND_ALPHACHANNEL | DSBLIT_BLEND_COLORALPHA | DSBLIT_SRC_PREMULTCOLOR | DSBLIT_COLORIZE,
            0x40);
    primary_hardware->Blit( primary_hardware, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_hardware->Blit( primary_hardware, test_surface, NULL, test_rect.x, test_rect.y );
    primary_software->Blit( primary_software, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_software->Blit( primary_software, test_surface, NULL, test_rect.x, test_rect.y );
    flip_context( primary_info, primary_hardware, primary_software );
    get_user_input();
    
    /* Test 100% Alpha Color + Alpha Channel + PREMULTIPLY + 100% PREMULTCOLOR Blending + Colorize */
    sprintf(buf, "Test %d: Porter-Duff Source Over Blit with Premultiply Source Alpha + 100%% Alpha Color + 100%% PREMULTCOLOR + Colorize", test_number++);
    display_information(primary_info, buf);
    
    setup_porter_duff_blits(primary_hardware, primary_software, DSPD_SRC_OVER, DSBLIT_BLEND_ALPHACHANNEL | DSBLIT_BLEND_COLORALPHA | DSBLIT_SRC_PREMULTIPLY |
                                                   DSBLIT_SRC_PREMULTCOLOR | DSBLIT_COLORIZE, BLT_COLOR_A);
    primary_hardware->Blit( primary_hardware, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_hardware->Blit( primary_hardware, test_surface, NULL, test_rect.x, test_rect.y );
    primary_software->Blit( primary_software, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_software->Blit( primary_software, test_surface, NULL, test_rect.x, test_rect.y );
    flip_context( primary_info, primary_hardware, primary_software );
    get_user_input();

    /* Test 50% Alpha Color + Alpha Channel + PREMULTIPLY + 50% PREMULTCOLOR Blending + Colorize */
    sprintf(buf, "Test %d: Porter-Duff Source Over Blit with Premultiply Source Alpha + 50%% Alpha Color + 50%% PREMULTCOLOR + Colorize", test_number++);
    display_information(primary_info, buf);
    
    setup_porter_duff_blits(primary_hardware, primary_software, DSPD_SRC_OVER, DSBLIT_BLEND_ALPHACHANNEL | DSBLIT_BLEND_COLORALPHA | DSBLIT_SRC_PREMULTIPLY |
                                                   DSBLIT_SRC_PREMULTCOLOR | DSBLIT_COLORIZE, 0x80);
    primary_hardware->Blit( primary_hardware, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_hardware->Blit( primary_hardware, test_surface, NULL, test_rect.x, test_rect.y );
    primary_software->Blit( primary_software, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_software->Blit( primary_software, test_surface, NULL, test_rect.x, test_rect.y );
    flip_context( primary_info, primary_hardware, primary_software );
    get_user_input();
    
    /* Test 25% Alpha Color + Alpha Channel + PREMULTIPLY + 25% PREMULTCOLOR Blending + Colorize */
    sprintf(buf, "Test %d: Porter-Duff Source Over Blit with Premultiply Source Alpha + 25%% Alpha Color + 25%% PREMULTCOLOR + Colorize", test_number++);
    display_information(primary_info, buf);
    
    setup_porter_duff_blits(primary_hardware, primary_software, DSPD_SRC_OVER, DSBLIT_BLEND_ALPHACHANNEL | DSBLIT_BLEND_COLORALPHA | DSBLIT_SRC_PREMULTIPLY |
                                                   DSBLIT_SRC_PREMULTCOLOR | DSBLIT_COLORIZE, 0x40);
    primary_hardware->Blit( primary_hardware, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_hardware->Blit( primary_hardware, test_surface, NULL, test_rect.x, test_rect.y );
    primary_software->Blit( primary_software, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_software->Blit( primary_software, test_surface, NULL, test_rect.x, test_rect.y );
    flip_context( primary_info, primary_hardware, primary_software );
    get_user_input();
    
    /*** Test with Porter-Duff Destination Over */
    
     /* Test Flip Horizontally */
    sprintf(buf, "Test %d: Porter-Duff Dest. Over Blit with Horizontal Flip", test_number++);
    display_information(primary_info, buf);
    
    setup_porter_duff_blits(primary_hardware, primary_software, DSPD_DST_OVER, DSBLIT_FLIP_HORIZONTAL, BLT_COLOR_A);
    primary_hardware->Blit( primary_hardware, logo_surface, NULL, logo_rect.x, logo_rect.y );
    primary_hardware->Blit( primary_hardware, test_surface, NULL,  test_rect.x, test_rect.y );
    primary_software->Blit( primary_software, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_software->Blit( primary_software, test_surface, NULL,  test_rect.x, test_rect.y );
    flip_context( primary_info, primary_hardware, primary_software );
    get_user_input();
    
     /* Test Flip Vertically */
    sprintf(buf, "Test %d: Porter-Duff Dest. Over Blit with Vertical Flip", test_number++);
    display_information(primary_info, buf);
    
    setup_porter_duff_blits(primary_hardware, primary_software, DSPD_DST_OVER, DSBLIT_FLIP_VERTICAL, BLT_COLOR_A);
    primary_hardware->Blit( primary_hardware, logo_surface, NULL, logo_rect.x, logo_rect.y );
    primary_hardware->Blit( primary_hardware, test_surface, NULL,  test_rect.x, test_rect.y );
    primary_software->Blit( primary_software, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_software->Blit( primary_software, test_surface, NULL,  test_rect.x, test_rect.y );
    flip_context( primary_info, primary_hardware, primary_software );
    get_user_input();
    
     /* Test Rotate 180 */
    sprintf(buf, "Test %d: Porter-Duff Dest. Over Blit with 180 Degrees Rotation", test_number++);
    display_information(primary_info, buf);
    
    setup_porter_duff_blits(primary_hardware, primary_software, DSPD_DST_OVER, DSBLIT_ROTATE180, BLT_COLOR_A);
    primary_hardware->Blit( primary_hardware, logo_surface, NULL, logo_rect.x, logo_rect.y );
    primary_hardware->Blit( primary_hardware, test_surface, NULL,  test_rect.x, test_rect.y );
    primary_software->Blit( primary_software, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_software->Blit( primary_software, test_surface, NULL,  test_rect.x, test_rect.y );
    flip_context( primary_info, primary_hardware, primary_software );
    get_user_input();
    
     /* Test Flip Horizontally with Alpha Channel Blending */
    sprintf(buf, "Test %d: Porter-Duff Dest. Over Blit with Horizontal Flip and Alpha Channel Blending", test_number++);
    display_information(primary_info, buf);
    
    setup_porter_duff_blits(primary_hardware, primary_software, DSPD_DST_OVER, DSBLIT_FLIP_HORIZONTAL | DSBLIT_BLEND_ALPHACHANNEL, BLT_COLOR_A);
    primary_hardware->Blit( primary_hardware, logo_surface, NULL, logo_rect.x, logo_rect.y );
    primary_hardware->Blit( primary_hardware, test_surface, NULL,  test_rect.x, test_rect.y );
    primary_software->Blit( primary_software, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_software->Blit( primary_software, test_surface, NULL,  test_rect.x, test_rect.y );
    flip_context( primary_info, primary_hardware, primary_software );
    get_user_input();
    
     /* Test Flip Vertically with Alpha Channel Blending */
    sprintf(buf, "Test %d: Porter-Duff Dest. Over Blit with Vertical Flip and Alpha Channel Blending", test_number++);
    display_information(primary_info, buf);
    
    setup_porter_duff_blits(primary_hardware, primary_software, DSPD_DST_OVER, DSBLIT_FLIP_VERTICAL | DSBLIT_BLEND_ALPHACHANNEL, BLT_COLOR_A);
    primary_hardware->Blit( primary_hardware, logo_surface, NULL, logo_rect.x, logo_rect.y );
    primary_hardware->Blit( primary_hardware, test_surface, NULL,  test_rect.x, test_rect.y );
    primary_software->Blit( primary_software, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_software->Blit( primary_software, test_surface, NULL,  test_rect.x, test_rect.y );
    flip_context( primary_info, primary_hardware, primary_software );
    get_user_input();
    
     /* Test Rotate 180 with Alpha Channel Blending */
    sprintf(buf, "Test %d: Porter-Duff Dest. Over Blit with 180 Degrees Rotation and Alpha Channel Blending", test_number++);
    display_information(primary_info, buf);
    
    setup_porter_duff_blits(primary_hardware, primary_software, DSPD_DST_OVER, DSBLIT_ROTATE180 | DSBLIT_BLEND_ALPHACHANNEL, BLT_COLOR_A);
    primary_hardware->Blit( primary_hardware, logo_surface, NULL, logo_rect.x, logo_rect.y );
    primary_hardware->Blit( primary_hardware, test_surface, NULL,  test_rect.x, test_rect.y );
    primary_software->Blit( primary_software, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_software->Blit( primary_software, test_surface, NULL,  test_rect.x, test_rect.y );
    flip_context( primary_info, primary_hardware, primary_software );
    get_user_input();
    
     /* Test XOR */
    sprintf(buf, "Test %d: Porter-Duff Dest. Over Blit with XOR", test_number++);
    display_information(primary_info, buf);
    
    setup_porter_duff_blits(primary_hardware, primary_software, DSPD_DST_OVER, DSBLIT_XOR, BLT_COLOR_A);
    primary_hardware->Blit( primary_hardware, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_hardware->Blit( primary_hardware, test_surface, NULL, test_rect.x, test_rect.y );
    primary_software->Blit( primary_software, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_software->Blit( primary_software, test_surface, NULL, test_rect.x, test_rect.y );
    flip_context( primary_info, primary_hardware, primary_software );
    get_user_input();
    
     /* Test XOR with Alpha Channel Blending */
    sprintf(buf, "Test %d: Porter-Duff Dest. Over Blit with XOR and Alpha Channel Blending", test_number++);
    display_information(primary_info, buf);
    
    setup_porter_duff_blits(primary_hardware, primary_software, DSPD_DST_OVER, DSBLIT_XOR | DSBLIT_BLEND_ALPHACHANNEL, 0x80);
    primary_hardware->Blit( primary_hardware, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_hardware->Blit( primary_hardware, test_surface, NULL, test_rect.x, test_rect.y );
    primary_software->Blit( primary_software, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_software->Blit( primary_software, test_surface, NULL, test_rect.x, test_rect.y );
    flip_context( primary_info, primary_hardware, primary_software );
    get_user_input();
    
     /* Test XOR with Color Alpha Blending */
    sprintf(buf, "Test %d: Porter-Duff Dest. Over Blit with XOR and Color Alpha Blending", test_number++);
    display_information(primary_info, buf);
    
    setup_porter_duff_blits(primary_hardware, primary_software, DSPD_DST_OVER, DSBLIT_XOR | DSBLIT_BLEND_COLORALPHA, 0x80);
    primary_hardware->Blit( primary_hardware, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_hardware->Blit( primary_hardware, test_surface, NULL, test_rect.x, test_rect.y );
    primary_software->Blit( primary_software, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_software->Blit( primary_software, test_surface, NULL, test_rect.x, test_rect.y );
    flip_context( primary_info, primary_hardware, primary_software );
    get_user_input();
    
     /* Test XOR with Alpha Channel and Color Alpha Blending */
    sprintf(buf, "Test %d: Porter-Duff Dest. Over Blit with XOR and Alpha Channel + Color Alpha Blending", test_number++);
    display_information(primary_info, buf);
    
    setup_porter_duff_blits(primary_hardware, primary_software, DSPD_DST_OVER, DSBLIT_XOR | DSBLIT_BLEND_ALPHACHANNEL | DSBLIT_BLEND_COLORALPHA, 0x80);
    primary_hardware->Blit( primary_hardware, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_hardware->Blit( primary_hardware, test_surface, NULL, test_rect.x, test_rect.y );
    primary_software->Blit( primary_software, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_software->Blit( primary_software, test_surface, NULL, test_rect.x, test_rect.y );
    flip_context( primary_info, primary_hardware, primary_software );
    get_user_input();
    
     /* Test XOR with Premultiply and Alpha Channel and Color Alpha Blending */
    sprintf(buf, "Test %d: Porter-Duff Dest. Over Blit with XOR and Premultiply + Alpha Channel + Color Alpha Blending", test_number++);
    display_information(primary_info, buf);
    
    setup_porter_duff_blits(primary_hardware, primary_software, DSPD_DST_OVER, DSBLIT_XOR | DSBLIT_BLEND_ALPHACHANNEL | DSBLIT_BLEND_COLORALPHA | DSBLIT_SRC_PREMULTIPLY, 0x80);
    primary_hardware->Blit( primary_hardware, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_hardware->Blit( primary_hardware, test_surface, NULL, test_rect.x, test_rect.y );
    primary_software->Blit( primary_software, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_software->Blit( primary_software, test_surface, NULL, test_rect.x, test_rect.y );
    flip_context( primary_info, primary_hardware, primary_software );
    get_user_input();
    
     /* Test XOR with Premultiply and Alpha Channel and Color Alpha Blending and PremultColor */
    sprintf(buf, "Test %d: Porter-Duff Dest. Over Blit with XOR + Premultiply + Alpha Channel + Color Alpha + PREMULTCOLOR Blending", test_number++);
    display_information(primary_info, buf);
    
    setup_porter_duff_blits(primary_hardware, primary_software, DSPD_DST_OVER, DSBLIT_XOR | DSBLIT_BLEND_ALPHACHANNEL | DSBLIT_BLEND_COLORALPHA |
                                                    DSBLIT_SRC_PREMULTIPLY | DSBLIT_SRC_PREMULTCOLOR, 0x80);
    primary_hardware->Blit( primary_hardware, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_hardware->Blit( primary_hardware, test_surface, NULL, test_rect.x, test_rect.y );
    primary_software->Blit( primary_software, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_software->Blit( primary_software, test_surface, NULL, test_rect.x, test_rect.y );
    flip_context( primary_info, primary_hardware, primary_software );
    get_user_input();
    
     /* Test XOR with Premultiply and Alpha Channel and Color Alpha Blending and PremultColor and Colorize */
    sprintf(buf, "Test %d: Porter-Duff Dest. Over Blit with XOR + Premultiply + Alpha Channel + Color Alpha + PREMULTCOLOR + Colorize Blending", test_number++);
    display_information(primary_info, buf);
    
    setup_porter_duff_blits(primary_hardware, primary_software, DSPD_DST_OVER, DSBLIT_XOR | DSBLIT_BLEND_ALPHACHANNEL | DSBLIT_BLEND_COLORALPHA |
                                                    DSBLIT_SRC_PREMULTIPLY | DSBLIT_SRC_PREMULTCOLOR | DSBLIT_COLORIZE, 0x80);
    primary_hardware->Blit( primary_hardware, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_hardware->Blit( primary_hardware, test_surface, NULL, test_rect.x, test_rect.y );
    primary_software->Blit( primary_software, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_software->Blit( primary_software, test_surface, NULL, test_rect.x, test_rect.y );
    flip_context( primary_info, primary_hardware, primary_software );
    get_user_input();
    
     /* Test Alpha Channel Blending */
    sprintf(buf, "Test %d: Porter-Duff Dest. Over Blit with Alpha Channel Blending", test_number++);
    display_information(primary_info, buf);
    
    setup_porter_duff_blits(primary_hardware, primary_software, DSPD_DST_OVER, DSBLIT_BLEND_ALPHACHANNEL, BLT_COLOR_A);
    primary_hardware->Blit( primary_hardware, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_hardware->Blit( primary_hardware, test_surface, NULL, test_rect.x, test_rect.y );
    primary_software->Blit( primary_software, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_software->Blit( primary_software, test_surface, NULL, test_rect.x, test_rect.y );
    flip_context( primary_info, primary_hardware, primary_software );
    get_user_input();
    
     /* Test 100% Color Alpha Blending */
    sprintf(buf, "Test %d: Porter-Duff Dest. Over Blit with 100%% Color Alpha Blending", test_number++);
    display_information(primary_info, buf);
    
    setup_porter_duff_blits(primary_hardware, primary_software, DSPD_DST_OVER, DSBLIT_BLEND_COLORALPHA, BLT_COLOR_A);
    primary_hardware->Blit( primary_hardware, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_hardware->Blit( primary_hardware, test_surface, NULL, test_rect.x, test_rect.y );
    primary_software->Blit( primary_software, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_software->Blit( primary_software, test_surface, NULL, test_rect.x, test_rect.y );
    flip_context( primary_info, primary_hardware, primary_software );
    get_user_input();
    
     /* Test 50% Color Alpha Blending */
    sprintf(buf, "Test %d: Porter-Duff Dest. Over Blit with 50%% Color Alpha Blending", test_number++);
    display_information(primary_info, buf);
    
    setup_porter_duff_blits(primary_hardware, primary_software, DSPD_DST_OVER, DSBLIT_BLEND_COLORALPHA, 0x80);
    primary_hardware->Blit( primary_hardware, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_hardware->Blit( primary_hardware, test_surface, NULL, test_rect.x, test_rect.y );
    primary_software->Blit( primary_software, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_software->Blit( primary_software, test_surface, NULL, test_rect.x, test_rect.y );
    flip_context( primary_info, primary_hardware, primary_software );
    get_user_input();
    
     /* Test 25% Color Alpha Blending */
    sprintf(buf, "Test %d: Porter-Duff Dest. Over Blit with 25%% Color Alpha Blending", test_number++);
    display_information(primary_info, buf);
    
    setup_porter_duff_blits(primary_hardware, primary_software, DSPD_DST_OVER, DSBLIT_BLEND_COLORALPHA, 0x40);
    primary_hardware->Blit( primary_hardware, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_hardware->Blit( primary_hardware, test_surface, NULL, test_rect.x, test_rect.y );
    primary_software->Blit( primary_software, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_software->Blit( primary_software, test_surface, NULL, test_rect.x, test_rect.y );
    flip_context( primary_info, primary_hardware, primary_software );
    get_user_input();
    
    /* Test 100% Color Alpha + Alpha Channel Blending */
    sprintf(buf, "Test %d: Porter-Duff Dest. Over Blit with 100%% Color Alpha + Alpha Blend", test_number++);
    display_information(primary_info, buf);
    
    setup_porter_duff_blits(primary_hardware, primary_software, DSPD_DST_OVER, DSBLIT_BLEND_COLORALPHA|DSBLIT_BLEND_ALPHACHANNEL, BLT_COLOR_A);
    primary_hardware->Blit( primary_hardware, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_hardware->Blit( primary_hardware, test_surface, NULL, test_rect.x, test_rect.y );
    primary_software->Blit( primary_software, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_software->Blit( primary_software, test_surface, NULL, test_rect.x, test_rect.y );
    flip_context( primary_info, primary_hardware, primary_software );
    get_user_input();
    
    /* Test 50% Color Alpha + Alpha Channel Blending */
    sprintf(buf, "Test %d: Porter-Duff Dest. Over Blit with 50%% Color Alpha + Alpha Blend", test_number++);
    display_information(primary_info, buf);
    
    setup_porter_duff_blits(primary_hardware, primary_software, DSPD_DST_OVER, DSBLIT_BLEND_COLORALPHA|DSBLIT_BLEND_ALPHACHANNEL, 0x80);
    primary_hardware->Blit( primary_hardware, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_hardware->Blit( primary_hardware, test_surface, NULL, test_rect.x, test_rect.y );
    primary_software->Blit( primary_software, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_software->Blit( primary_software, test_surface, NULL, test_rect.x, test_rect.y );
    flip_context( primary_info, primary_hardware, primary_software );
    get_user_input();
    
    /* Test 25% Color Alpha + Alpha Channel Blending */
    sprintf(buf, "Test %d: Porter-Duff Dest. Over Blit with 25%% Color Alpha + Alpha Blend", test_number++);
    display_information(primary_info, buf);
    
    setup_porter_duff_blits(primary_hardware, primary_software, DSPD_DST_OVER, DSBLIT_BLEND_COLORALPHA|DSBLIT_BLEND_ALPHACHANNEL, 0x40);
    primary_hardware->Blit( primary_hardware, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_hardware->Blit( primary_hardware, test_surface, NULL, test_rect.x, test_rect.y );
    primary_software->Blit( primary_software, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_software->Blit( primary_software, test_surface, NULL, test_rect.x, test_rect.y );
    flip_context( primary_info, primary_hardware, primary_software );
    get_user_input();
    
    /* Test PREMULTIPLY */
    sprintf(buf, "Test %d: Porter-Duff Dest. Over Blit with Premultiply Source", test_number++);
    display_information(primary_info, buf);
    
    setup_porter_duff_blits(primary_hardware, primary_software, DSPD_DST_OVER, DSBLIT_SRC_PREMULTIPLY, 0x80);
    primary_hardware->Blit( primary_hardware, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_hardware->Blit( primary_hardware, test_surface, NULL, test_rect.x, test_rect.y );
    primary_software->Blit( primary_software, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_software->Blit( primary_software, test_surface, NULL, test_rect.x, test_rect.y );
    flip_context( primary_info, primary_hardware, primary_software );
    get_user_input();
    
    /* Test Alpha Channel + PREMULTIPLY Blending */
    sprintf(buf, "Test %d: Porter-Duff Dest. Over Blit with Premultiply Source Alpha Blending", test_number++);
    display_information(primary_info, buf);
    
    setup_porter_duff_blits(primary_hardware, primary_software, DSPD_DST_OVER, DSBLIT_BLEND_ALPHACHANNEL | DSBLIT_SRC_PREMULTIPLY, 0x80);
    primary_hardware->Blit( primary_hardware, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_hardware->Blit( primary_hardware, test_surface, NULL, test_rect.x, test_rect.y );
    primary_software->Blit( primary_software, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_software->Blit( primary_software, test_surface, NULL, test_rect.x, test_rect.y );
    flip_context( primary_info, primary_hardware, primary_software );
    get_user_input();
    
     /* Test 100% Alpha Color + Alpha Channel + PREMULTIPLY Blending */
    sprintf(buf, "Test %d: Porter-Duff Dest. Over Blit with Premultiply Source Alpha + 100%% Alpha Color blending", test_number++);
    display_information(primary_info, buf);
    
    setup_porter_duff_blits(primary_hardware, primary_software, DSPD_DST_OVER, DSBLIT_BLEND_ALPHACHANNEL | DSBLIT_BLEND_COLORALPHA | DSBLIT_SRC_PREMULTIPLY, BLT_COLOR_A);
    primary_hardware->Blit( primary_hardware, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_hardware->Blit( primary_hardware, test_surface, NULL, test_rect.x, test_rect.y );
    primary_software->Blit( primary_software, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_software->Blit( primary_software, test_surface, NULL, test_rect.x, test_rect.y );
    flip_context( primary_info, primary_hardware, primary_software );
    get_user_input();
    
     /* Test 50% Alpha Color + Alpha Channel + PREMULTIPLY Blending */
    sprintf(buf, "Test %d: Porter-Duff Dest. Over Blit with Premultiply Source Alpha + 50%% Alpha Color blending", test_number++);
    display_information(primary_info, buf);
    
    setup_porter_duff_blits(primary_hardware, primary_software, DSPD_DST_OVER, DSBLIT_BLEND_ALPHACHANNEL | DSBLIT_BLEND_COLORALPHA | DSBLIT_SRC_PREMULTIPLY, 0x80);
    primary_hardware->Blit( primary_hardware, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_hardware->Blit( primary_hardware, test_surface, NULL, test_rect.x, test_rect.y );
    primary_software->Blit( primary_software, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_software->Blit( primary_software, test_surface, NULL, test_rect.x, test_rect.y );
    flip_context( primary_info, primary_hardware, primary_software );
    get_user_input();
    
     /* Test 25% Alpha Color + Alpha Channel + PREMULTIPLY Blending */
    sprintf(buf, "Test %d: Porter-Duff Dest. Over Blit with Premultiply Source Alpha + 25%% Alpha Color blending", test_number++);
    display_information(primary_info, buf);
    
    setup_porter_duff_blits(primary_hardware, primary_software, DSPD_DST_OVER, DSBLIT_BLEND_ALPHACHANNEL | DSBLIT_BLEND_COLORALPHA | DSBLIT_SRC_PREMULTIPLY, 0x40);
    primary_hardware->Blit( primary_hardware, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_hardware->Blit( primary_hardware, test_surface, NULL, test_rect.x, test_rect.y );
    primary_software->Blit( primary_software, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_software->Blit( primary_software, test_surface, NULL, test_rect.x, test_rect.y );
    flip_context( primary_info, primary_hardware, primary_software );
    get_user_input();
    
    /* Test 100% PREMULTCOLOR */
    sprintf(buf, "Test %d: Porter-Duff Dest. Over Blit with 100%% PREMULTCOLOR on the Source", test_number++);
    display_information(primary_info, buf);
    
    setup_porter_duff_blits(primary_hardware, primary_software, DSPD_DST_OVER, DSBLIT_SRC_PREMULTCOLOR, BLT_COLOR_A);
    primary_hardware->Blit( primary_hardware, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_hardware->Blit( primary_hardware, test_surface, NULL, test_rect.x, test_rect.y );
    primary_software->Blit( primary_software, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_software->Blit( primary_software, test_surface, NULL, test_rect.x, test_rect.y );
    flip_context( primary_info, primary_hardware, primary_software );
    get_user_input();
    
    /* Test 50% PREMULTCOLOR */
    sprintf(buf, "Test %d: Porter-Duff Dest. Over Blit with 50%% PREMULTCOLOR on the Source", test_number++);
    display_information(primary_info, buf);
    
    setup_porter_duff_blits(primary_hardware, primary_software, DSPD_DST_OVER, DSBLIT_SRC_PREMULTCOLOR, 0x80);
    primary_hardware->Blit( primary_hardware, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_hardware->Blit( primary_hardware, test_surface, NULL, test_rect.x, test_rect.y );
    primary_software->Blit( primary_software, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_software->Blit( primary_software, test_surface, NULL, test_rect.x, test_rect.y );
    flip_context( primary_info, primary_hardware, primary_software );
    get_user_input();
    
    /* Test 25% PREMULTCOLOR */
    sprintf(buf, "Test %d: Porter-Duff Dest. Over Blit with 25%% PREMULTCOLOR on the Source", test_number++);
    display_information(primary_info, buf);
    
    setup_porter_duff_blits(primary_hardware, primary_software, DSPD_DST_OVER, DSBLIT_SRC_PREMULTCOLOR, 0x40);
    primary_hardware->Blit( primary_hardware, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_hardware->Blit( primary_hardware, test_surface, NULL, test_rect.x, test_rect.y );
    primary_software->Blit( primary_software, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_software->Blit( primary_software, test_surface, NULL, test_rect.x, test_rect.y );
    flip_context( primary_info, primary_hardware, primary_software );
    get_user_input();
    
    /* Test 100% Alpha Color + Alpha Channel + 100% PREMULTCOLOR Blending */
    sprintf(buf, "Test %d: Porter-Duff Dest. Over Blit with Source Alpha + 100%% Alpha Color + 100%% PREMULTCOLOR", test_number++);
    display_information(primary_info, buf);
    
    setup_porter_duff_blits(primary_hardware, primary_software, DSPD_DST_OVER, DSBLIT_BLEND_ALPHACHANNEL | DSBLIT_BLEND_COLORALPHA | DSBLIT_SRC_PREMULTCOLOR, BLT_COLOR_A);
    primary_hardware->Blit( primary_hardware, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_hardware->Blit( primary_hardware, test_surface, NULL, test_rect.x, test_rect.y );
    primary_software->Blit( primary_software, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_software->Blit( primary_software, test_surface, NULL, test_rect.x, test_rect.y );
    flip_context( primary_info, primary_hardware, primary_software );
    get_user_input();
    
    /* Test 50% Alpha Color + Alpha Channel + 50% PREMULTCOLOR Blending */
    sprintf(buf, "Test %d: Porter-Duff Dest. Over Blit with Source Alpha + 50%% Alpha Color + 50%% PREMULTCOLOR", test_number++);
    display_information(primary_info, buf);
    
    setup_porter_duff_blits(primary_hardware, primary_software, DSPD_DST_OVER, DSBLIT_BLEND_ALPHACHANNEL | DSBLIT_BLEND_COLORALPHA | DSBLIT_SRC_PREMULTCOLOR, 0x80);
    primary_hardware->Blit( primary_hardware, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_hardware->Blit( primary_hardware, test_surface, NULL, test_rect.x, test_rect.y );
    primary_software->Blit( primary_software, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_software->Blit( primary_software, test_surface, NULL, test_rect.x, test_rect.y );
    flip_context( primary_info, primary_hardware, primary_software );
    get_user_input();
    
    /* Test 25% Alpha Color + Alpha Channel + 25% PREMULTCOLOR Blending */
    sprintf(buf, "Test %d: Porter-Duff Dest. Over Blit with Source Alpha + 25%% Alpha Color + 25%% PREMULTCOLOR", test_number++);
    display_information(primary_info, buf);
    
    setup_porter_duff_blits(primary_hardware, primary_software, DSPD_DST_OVER, DSBLIT_BLEND_ALPHACHANNEL | DSBLIT_BLEND_COLORALPHA | DSBLIT_SRC_PREMULTCOLOR, 0x40);
    primary_hardware->Blit( primary_hardware, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_hardware->Blit( primary_hardware, test_surface, NULL, test_rect.x, test_rect.y );
    primary_software->Blit( primary_software, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_software->Blit( primary_software, test_surface, NULL, test_rect.x, test_rect.y );
    flip_context( primary_info, primary_hardware, primary_software );
    get_user_input();
    
    /* Test 100% Alpha Color + Alpha Channel + PREMULTIPLY + 100% PREMULTCOLOR Blending */
    sprintf(buf, "Test %d: Porter-Duff Dest. Over Blit with Premultiply Source Alpha + 100%% Alpha Color + 100%% PREMULTCOLOR", test_number++);
    display_information(primary_info, buf);
    
    setup_porter_duff_blits(primary_hardware, primary_software, DSPD_DST_OVER, DSBLIT_BLEND_ALPHACHANNEL | DSBLIT_BLEND_COLORALPHA | DSBLIT_SRC_PREMULTIPLY | DSBLIT_SRC_PREMULTCOLOR,
            BLT_COLOR_A);
    primary_hardware->Blit( primary_hardware, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_hardware->Blit( primary_hardware, test_surface, NULL, test_rect.x, test_rect.y );
    primary_software->Blit( primary_software, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_software->Blit( primary_software, test_surface, NULL, test_rect.x, test_rect.y );
    flip_context( primary_info, primary_hardware, primary_software );
    get_user_input();
    
    /* Test 50% Alpha Color + Alpha Channel + PREMULTIPLY + 50% PREMULTCOLOR Blending */
    sprintf(buf, "Test %d: Porter-Duff Dest. Over Blit with Premultiply Source Alpha + 50%% Alpha Color + 50%% PREMULTCOLOR", test_number++);
    display_information(primary_info, buf);
    
    setup_porter_duff_blits(primary_hardware, primary_software, DSPD_DST_OVER, DSBLIT_BLEND_ALPHACHANNEL | DSBLIT_BLEND_COLORALPHA | DSBLIT_SRC_PREMULTIPLY | DSBLIT_SRC_PREMULTCOLOR,
            0x80);
    primary_hardware->Blit( primary_hardware, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_hardware->Blit( primary_hardware, test_surface, NULL, test_rect.x, test_rect.y );
    primary_software->Blit( primary_software, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_software->Blit( primary_software, test_surface, NULL, test_rect.x, test_rect.y );
    flip_context( primary_info, primary_hardware, primary_software );
    get_user_input();
    
    /* Test 25% Alpha Color + Alpha Channel + PREMULTIPLY + 25% PREMULTCOLOR Blending */
    sprintf(buf, "Test %d: Porter-Duff Dest. Over Blit with Premultiply Source Alpha + 25%% Alpha Color + 25%% PREMULTCOLOR", test_number++);
    display_information(primary_info, buf);
    
    setup_porter_duff_blits(primary_hardware, primary_software, DSPD_DST_OVER, DSBLIT_BLEND_ALPHACHANNEL | DSBLIT_BLEND_COLORALPHA | DSBLIT_SRC_PREMULTIPLY | DSBLIT_SRC_PREMULTCOLOR,
            0x40);
    primary_hardware->Blit( primary_hardware, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_hardware->Blit( primary_hardware, test_surface, NULL, test_rect.x, test_rect.y );
    primary_software->Blit( primary_software, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_software->Blit( primary_software, test_surface, NULL, test_rect.x, test_rect.y );
    flip_context( primary_info, primary_hardware, primary_software );
    get_user_input();
    
    /* Test Alpha Channel Blending with Colorize */
    sprintf(buf, "Test %d: Porter-Duff Dest. Over Blit with Source Alpha Blending with Colorize", test_number++);
    display_information(primary_info, buf);
    
    setup_porter_duff_blits(primary_hardware, primary_software, DSPD_DST_OVER, DSBLIT_BLEND_ALPHACHANNEL | DSBLIT_COLORIZE, 0x80);
    primary_hardware->Blit( primary_hardware, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_hardware->Blit( primary_hardware, test_surface, NULL, test_rect.x, test_rect.y );
    primary_software->Blit( primary_software, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_software->Blit( primary_software, test_surface, NULL, test_rect.x, test_rect.y );
    flip_context( primary_info, primary_hardware, primary_software );
    get_user_input();

    /* Test 100% Color Alpha Blending with Colorize */
    sprintf(buf, "Test %d: Porter-Duff Dest. Over Blit with 100%% Color Alpha Blending with Colorize", test_number++);
    display_information(primary_info, buf);
    
    setup_porter_duff_blits(primary_hardware, primary_software, DSPD_DST_OVER, DSBLIT_BLEND_COLORALPHA | DSBLIT_COLORIZE, BLT_COLOR_A);
    primary_hardware->Blit( primary_hardware, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_hardware->Blit( primary_hardware, test_surface, NULL, test_rect.x, test_rect.y );
    primary_software->Blit( primary_software, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_software->Blit( primary_software, test_surface, NULL, test_rect.x, test_rect.y );
    flip_context( primary_info, primary_hardware, primary_software );
    get_user_input();
    
    /* Test 50% Color Alpha Blending with Colorize */
    sprintf(buf, "Test %d: Porter-Duff Dest. Over Blit with 50%% Color Alpha Blending with Colorize", test_number++);
    display_information(primary_info, buf);
    
    setup_porter_duff_blits(primary_hardware, primary_software, DSPD_DST_OVER, DSBLIT_BLEND_COLORALPHA | DSBLIT_COLORIZE, 0x80);
    primary_hardware->Blit( primary_hardware, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_hardware->Blit( primary_hardware, test_surface, NULL, test_rect.x, test_rect.y );
    primary_software->Blit( primary_software, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_software->Blit( primary_software, test_surface, NULL, test_rect.x, test_rect.y );
    flip_context( primary_info, primary_hardware, primary_software );
    get_user_input();
    
    /* Test 25% Color Alpha Blending with Colorize */
    sprintf(buf, "Test %d: Porter-Duff Dest. Over Blit with 25%% Color Alpha Blending with Colorize", test_number++);
    display_information(primary_info, buf);
    
    setup_porter_duff_blits(primary_hardware, primary_software, DSPD_DST_OVER, DSBLIT_BLEND_COLORALPHA | DSBLIT_COLORIZE, 0x40);
    primary_hardware->Blit( primary_hardware, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_hardware->Blit( primary_hardware, test_surface, NULL, test_rect.x, test_rect.y );
    primary_software->Blit( primary_software, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_software->Blit( primary_software, test_surface, NULL, test_rect.x, test_rect.y );
    flip_context( primary_info, primary_hardware, primary_software );
    get_user_input();
    
    /* Test 100% Color Alpha + Alpha Channel Blending + Colorize */
    sprintf(buf, "Test %d: Porter-Duff Dest. Over Blit with 100%% Color Alpha + Source Alpha Blending + Colorize", test_number++);
    display_information(primary_info, buf);
    
    setup_porter_duff_blits(primary_hardware, primary_software, DSPD_DST_OVER, DSBLIT_BLEND_COLORALPHA | DSBLIT_BLEND_ALPHACHANNEL | DSBLIT_COLORIZE, BLT_COLOR_A);
    primary_hardware->Blit( primary_hardware, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_hardware->Blit( primary_hardware, test_surface, NULL, test_rect.x, test_rect.y );
    primary_software->Blit( primary_software, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_software->Blit( primary_software, test_surface, NULL, test_rect.x, test_rect.y );
    flip_context( primary_info, primary_hardware, primary_software );
    get_user_input();
    
    /* Test 50% Color Alpha + Alpha Channel Blending + Colorize */
    sprintf(buf, "Test %d: Porter-Duff Dest. Over Blit with 50%% Color Alpha + Source Alpha Blending + Colorize", test_number++);
    display_information(primary_info, buf);
    
    setup_porter_duff_blits(primary_hardware, primary_software, DSPD_DST_OVER, DSBLIT_BLEND_COLORALPHA | DSBLIT_BLEND_ALPHACHANNEL | DSBLIT_COLORIZE, 0x80);
    primary_hardware->Blit( primary_hardware, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_hardware->Blit( primary_hardware, test_surface, NULL, test_rect.x, test_rect.y );
    primary_software->Blit( primary_software, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_software->Blit( primary_software, test_surface, NULL, test_rect.x, test_rect.y );
    flip_context( primary_info, primary_hardware, primary_software );
    get_user_input();
    
    /* Test 25% Color Alpha + Alpha Channel Blending + Colorize */
    sprintf(buf, "Test %d: Porter-Duff Dest. Over Blit with 25%% Color Alpha + Source Alpha Blending + Colorize", test_number++);
    display_information(primary_info, buf);
    
    setup_porter_duff_blits(primary_hardware, primary_software, DSPD_DST_OVER, DSBLIT_BLEND_COLORALPHA | DSBLIT_BLEND_ALPHACHANNEL | DSBLIT_COLORIZE, 0x40);
    primary_hardware->Blit( primary_hardware, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_hardware->Blit( primary_hardware, test_surface, NULL, test_rect.x, test_rect.y );
    primary_software->Blit( primary_software, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_software->Blit( primary_software, test_surface, NULL, test_rect.x, test_rect.y );
    flip_context( primary_info, primary_hardware, primary_software );
    get_user_input();
    
    /* Test PREMULTIPLY with Colorize */
    sprintf(buf, "Test %d: Porter-Duff Dest. Over Blit with Premultiply Source with Colorize", test_number++);
    display_information(primary_info, buf);
    
    setup_porter_duff_blits(primary_hardware, primary_software, DSPD_DST_OVER, DSBLIT_SRC_PREMULTIPLY | DSBLIT_COLORIZE, 0x80);
    primary_hardware->Blit( primary_hardware, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_hardware->Blit( primary_hardware, test_surface, NULL, test_rect.x, test_rect.y );
    primary_software->Blit( primary_software, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_software->Blit( primary_software, test_surface, NULL, test_rect.x, test_rect.y );
    flip_context( primary_info, primary_hardware, primary_software );
    get_user_input();
    
    /* Test Alpha Channel + PREMULTIPLY Blending + Colorize */
    sprintf(buf, "Test %d: Porter-Duff Dest. Over Blit with Premultiply Source Alpha Blending with Colorize", test_number++);
    display_information(primary_info, buf);
    
    setup_porter_duff_blits(primary_hardware, primary_software, DSPD_DST_OVER,  DSBLIT_BLEND_ALPHACHANNEL | DSBLIT_SRC_PREMULTIPLY | DSBLIT_COLORIZE, 0x80);
    primary_hardware->Blit( primary_hardware, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_hardware->Blit( primary_hardware, test_surface, NULL, test_rect.x, test_rect.y );
    primary_software->Blit( primary_software, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_software->Blit( primary_software, test_surface, NULL, test_rect.x, test_rect.y );
    flip_context( primary_info, primary_hardware, primary_software );
    get_user_input();
    
    /* Test 100% Alpha Color + Alpha Channel + PREMULTIPLY Blending + Colorize */
    sprintf(buf, "Test %d: Porter-Duff Dest. Over Blit with Premultiply Source Alpha + 100%% Alpha Color blending + Colorize", test_number++);
    display_information(primary_info, buf);
    
    setup_porter_duff_blits(primary_hardware, primary_software, DSPD_DST_OVER,  DSBLIT_BLEND_ALPHACHANNEL | DSBLIT_BLEND_COLORALPHA | DSBLIT_SRC_PREMULTIPLY | DSBLIT_COLORIZE,
               BLT_COLOR_A);
    primary_hardware->Blit( primary_hardware, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_hardware->Blit( primary_hardware, test_surface, NULL, test_rect.x, test_rect.y );
    primary_software->Blit( primary_software, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_software->Blit( primary_software, test_surface, NULL, test_rect.x, test_rect.y );
    flip_context( primary_info, primary_hardware, primary_software );
    get_user_input();
    
    /* Test 50% Alpha Color + Alpha Channel + PREMULTIPLY Blending + Colorize */
    sprintf(buf, "Test %d: Porter-Duff Dest. Over Blit with Premultiply Source Alpha + 50%% Alpha Color blending + Colorize", test_number++);
    display_information(primary_info, buf);
    
    setup_porter_duff_blits(primary_hardware, primary_software, DSPD_DST_OVER,  DSBLIT_BLEND_ALPHACHANNEL | DSBLIT_BLEND_COLORALPHA | DSBLIT_SRC_PREMULTIPLY | DSBLIT_COLORIZE,
               0x80);
    primary_hardware->Blit( primary_hardware, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_hardware->Blit( primary_hardware, test_surface, NULL, test_rect.x, test_rect.y );
    primary_software->Blit( primary_software, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_software->Blit( primary_software, test_surface, NULL, test_rect.x, test_rect.y );
    flip_context( primary_info, primary_hardware, primary_software );
    get_user_input();
    
    /* Test 25% Alpha Color + Alpha Channel + PREMULTIPLY Blending + Colorize */
    sprintf(buf, "Test %d: Porter-Duff Dest. Over Blit with Premultiply Source Alpha + 25%% Alpha Color blending + Colorize", test_number++);
    display_information(primary_info, buf);
    
    setup_porter_duff_blits(primary_hardware, primary_software, DSPD_DST_OVER,  DSBLIT_BLEND_ALPHACHANNEL | DSBLIT_BLEND_COLORALPHA | DSBLIT_SRC_PREMULTIPLY | DSBLIT_COLORIZE,
               0x40);
    primary_hardware->Blit( primary_hardware, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_hardware->Blit( primary_hardware, test_surface, NULL, test_rect.x, test_rect.y );
    primary_software->Blit( primary_software, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_software->Blit( primary_software, test_surface, NULL, test_rect.x, test_rect.y );
    flip_context( primary_info, primary_hardware, primary_software );
    get_user_input();
    
    /* Test 100% PREMULTCOLOR with Colorize */
    sprintf(buf, "Test %d: Porter-Duff Dest. Over Blit with 100%% PREMULTCOLOR on the Source with Colorize", test_number++);
    display_information(primary_info, buf);
    
    setup_porter_duff_blits(primary_hardware, primary_software, DSPD_DST_OVER,  DSBLIT_SRC_PREMULTCOLOR | DSBLIT_COLORIZE, BLT_COLOR_A);
    primary_hardware->Blit( primary_hardware, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_hardware->Blit( primary_hardware, test_surface, NULL, test_rect.x, test_rect.y );
    primary_software->Blit( primary_software, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_software->Blit( primary_software, test_surface, NULL, test_rect.x, test_rect.y );
    flip_context( primary_info, primary_hardware, primary_software );
    get_user_input();
    
    /* Test 50% PREMULTCOLOR with Colorize */
    sprintf(buf, "Test %d: Porter-Duff Dest. Over Blit with 50%% PREMULTCOLOR on the Source with Colorize", test_number++);
    display_information(primary_info, buf);
    
    setup_porter_duff_blits(primary_hardware, primary_software, DSPD_DST_OVER,  DSBLIT_SRC_PREMULTCOLOR | DSBLIT_COLORIZE, 0x80);
    primary_hardware->Blit( primary_hardware, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_hardware->Blit( primary_hardware, test_surface, NULL, test_rect.x, test_rect.y );
    primary_software->Blit( primary_software, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_software->Blit( primary_software, test_surface, NULL, test_rect.x, test_rect.y );
    flip_context( primary_info, primary_hardware, primary_software );
    get_user_input();
    
    /* Test 25% PREMULTCOLOR with Colorize */
    sprintf(buf, "Test %d: Porter-Duff Dest. Over Blit with 25%% PREMULTCOLOR on the Source with Colorize", test_number++);
    display_information(primary_info, buf);
    
    setup_porter_duff_blits(primary_hardware, primary_software, DSPD_DST_OVER,  DSBLIT_SRC_PREMULTCOLOR | DSBLIT_COLORIZE, 0x40);
    primary_hardware->Blit( primary_hardware, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_hardware->Blit( primary_hardware, test_surface, NULL, test_rect.x, test_rect.y );
    primary_software->Blit( primary_software, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_software->Blit( primary_software, test_surface, NULL, test_rect.x, test_rect.y );
    flip_context( primary_info, primary_hardware, primary_software );
    get_user_input();
    
    /* Test 100% Alpha Color + Alpha Channel + 100% PREMULTCOLOR Blending + Colorize */ 
    sprintf(buf, "Test %d: Porter-Duff Dest. Over Blit with Source Alpha + 100%% Alpha Color + 100%% PREMULTCOLOR + Colorize", test_number++);
    display_information(primary_info, buf);
    
    setup_porter_duff_blits(primary_hardware, primary_software, DSPD_DST_OVER, DSBLIT_BLEND_ALPHACHANNEL | DSBLIT_BLEND_COLORALPHA | DSBLIT_SRC_PREMULTCOLOR | DSBLIT_COLORIZE,
            BLT_COLOR_A);
    primary_hardware->Blit( primary_hardware, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_hardware->Blit( primary_hardware, test_surface, NULL, test_rect.x, test_rect.y );
    primary_software->Blit( primary_software, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_software->Blit( primary_software, test_surface, NULL, test_rect.x, test_rect.y );
    flip_context( primary_info, primary_hardware, primary_software );
    get_user_input();
    
    /* Test 50% Alpha Color + Alpha Channel + 50% PREMULTCOLOR Blending + Colorize */
    sprintf(buf, "Test %d: Porter-Duff Dest. Over Blit with Source Alpha + 50%% Alpha Color + 50%% PREMULTCOLOR + Colorize", test_number++);
    display_information(primary_info, buf);
    
    setup_porter_duff_blits(primary_hardware, primary_software, DSPD_DST_OVER, DSBLIT_BLEND_ALPHACHANNEL | DSBLIT_BLEND_COLORALPHA | DSBLIT_SRC_PREMULTCOLOR | DSBLIT_COLORIZE,
            0x80);
    primary_hardware->Blit( primary_hardware, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_hardware->Blit( primary_hardware, test_surface, NULL, test_rect.x, test_rect.y );
    primary_software->Blit( primary_software, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_software->Blit( primary_software, test_surface, NULL, test_rect.x, test_rect.y );
    flip_context( primary_info, primary_hardware, primary_software );
    get_user_input();
    
    /* Test 25% Alpha Color + Alpha Channel + 25% PREMULTCOLOR Blending + Colorize */
    sprintf(buf, "Test %d: Porter-Duff Dest. Over Blit with Source Alpha + 25%% Alpha Color + 25%% PREMULTCOLOR + Colorize", test_number++);
    display_information(primary_info, buf);
    
    setup_porter_duff_blits(primary_hardware, primary_software, DSPD_DST_OVER, DSBLIT_BLEND_ALPHACHANNEL | DSBLIT_BLEND_COLORALPHA | DSBLIT_SRC_PREMULTCOLOR | DSBLIT_COLORIZE,
            0x40);
    primary_hardware->Blit( primary_hardware, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_hardware->Blit( primary_hardware, test_surface, NULL, test_rect.x, test_rect.y );
    primary_software->Blit( primary_software, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_software->Blit( primary_software, test_surface, NULL, test_rect.x, test_rect.y );
    flip_context( primary_info, primary_hardware, primary_software );
    get_user_input();
    
    /* Test 100% Alpha Color + Alpha Channel + PREMULTIPLY + 100% PREMULTCOLOR Blending + Colorize */
    sprintf(buf, "Test %d: Porter-Duff Dest. Over Blit with Premultiply Source Alpha + 100%% Alpha Color + 100%% PREMULTCOLOR + Colorize", test_number++);
    display_information(primary_info, buf);
    
    setup_porter_duff_blits(primary_hardware, primary_software, DSPD_DST_OVER, DSBLIT_BLEND_ALPHACHANNEL | DSBLIT_BLEND_COLORALPHA | DSBLIT_SRC_PREMULTIPLY |
                                                   DSBLIT_SRC_PREMULTCOLOR | DSBLIT_COLORIZE, BLT_COLOR_A);
    primary_hardware->Blit( primary_hardware, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_hardware->Blit( primary_hardware, test_surface, NULL, test_rect.x, test_rect.y );
    primary_software->Blit( primary_software, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_software->Blit( primary_software, test_surface, NULL, test_rect.x, test_rect.y );
    flip_context( primary_info, primary_hardware, primary_software );
    get_user_input();
    
    /* Test 50% Alpha Color + Alpha Channel + PREMULTIPLY + 50% PREMULTCOLOR Blending + Colorize */
    sprintf(buf, "Test %d: Porter-Duff Dest. Over Blit with Premultiply Source Alpha + 50%% Alpha Color + 50%% PREMULTCOLOR + Colorize", test_number++);
    display_information(primary_info, buf);

    setup_porter_duff_blits(primary_hardware, primary_software, DSPD_DST_OVER, DSBLIT_BLEND_ALPHACHANNEL | DSBLIT_BLEND_COLORALPHA | DSBLIT_SRC_PREMULTIPLY |
                                                   DSBLIT_SRC_PREMULTCOLOR | DSBLIT_COLORIZE, 0x80);
    primary_hardware->Blit( primary_hardware, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_hardware->Blit( primary_hardware, test_surface, NULL, test_rect.x, test_rect.y );
    primary_software->Blit( primary_software, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_software->Blit( primary_software, test_surface, NULL, test_rect.x, test_rect.y );
    flip_context( primary_info, primary_hardware, primary_software );
    get_user_input();
    
    /* Test 25% Alpha Color + Alpha Channel + PREMULTIPLY + 25% PREMULTCOLOR Blending + Colorize */
    sprintf(buf, "Test %d: Porter-Duff Dest. Over Blit with Premultiply Source Alpha + 25%% Alpha Color + 25%% PREMULTCOLOR + Colorize", test_number++);
    display_information(primary_info, buf);
    
    setup_porter_duff_blits(primary_hardware, primary_software, DSPD_DST_OVER, DSBLIT_BLEND_ALPHACHANNEL | DSBLIT_BLEND_COLORALPHA | DSBLIT_SRC_PREMULTIPLY |
                                                   DSBLIT_SRC_PREMULTCOLOR | DSBLIT_COLORIZE, 0x40);
    primary_hardware->Blit( primary_hardware, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_hardware->Blit( primary_hardware, test_surface, NULL, test_rect.x, test_rect.y );
    primary_software->Blit( primary_software, logo_surface, NULL,  logo_rect.x, logo_rect.y );
    primary_software->Blit( primary_software, test_surface, NULL, test_rect.x, test_rect.y );
    flip_context( primary_info, primary_hardware, primary_software );
    get_user_input();
    
    /* Cancel Porter-Duff */
    primary_hardware->SetPorterDuff(primary_hardware, DSPD_NONE);
    primary_software->SetPorterDuff(primary_software, DSPD_NONE);
    
    /*** Test using Stretch Blit ***/
    printf("##### TESTING STRETCH BLIT (x1.5) #####\n\n");
    
    logo_rect.x = (SUB_W - biglogo_desc.width) / 8;
    logo_rect.y = SUB_H / 8;
    logo_rect.w = (biglogo_desc.width * 3) >> 1;
    logo_rect.h = (biglogo_desc.height * 3) >> 1;
    
    test_rect.x = logo_rect.x;
    test_rect.y = 20 + logo_rect.y;
    test_rect.w = (dsc_test_surface.width  * 3) >> 1;
    test_rect.h = (dsc_test_surface.height * 3) >> 1;
    
    /* Test Flip Horizontally */
    sprintf(buf, "Test %d: Stretch Blit with Horizontal Flip", test_number++);
    display_information(primary_info, buf);
    
    setup_blits(primary_hardware, primary_software, DSBLIT_FLIP_HORIZONTAL|0//DSBLIT_SRC_COLORKEY
            , BLT_COLOR_A);
    fprintf(stderr, "### Stretch Blit x,y,w,h = %d,%d,%d,%d\n", test_rect.x, test_rect.y, test_rect.w, test_rect.h);
    
    primary_hardware->StretchBlit( primary_hardware, logo_surface, NULL, &logo_rect );
    primary_hardware->StretchBlit( primary_hardware, test_surface, NULL,  &test_rect );
    primary_software->StretchBlit( primary_software, logo_surface, NULL, &logo_rect );
    primary_software->StretchBlit( primary_software, test_surface, NULL,  &test_rect );
    flip_context( primary_info, primary_hardware, primary_software );
    get_user_input();
    
    /* Test Flip Vertically */
    sprintf(buf, "Test %d: Stretch Blit with Vertical Flip", test_number++);
    display_information(primary_info, buf);
    
    setup_blits(primary_hardware, primary_software, DSBLIT_FLIP_VERTICAL|0//DSBLIT_SRC_COLORKEY
            , BLT_COLOR_A);
    primary_hardware->StretchBlit( primary_hardware, logo_surface, NULL, &logo_rect );
    primary_hardware->StretchBlit( primary_hardware, test_surface, NULL,  &test_rect );
    primary_software->StretchBlit( primary_software, logo_surface, NULL, &logo_rect );
    primary_software->StretchBlit( primary_software, test_surface, NULL,  &test_rect );
    flip_context( primary_info, primary_hardware, primary_software );
    get_user_input();
    
    /* Test Rotate 180 */
    sprintf(buf, "Test %d: Stretch Blit with 180 Degrees Rotation", test_number++);
    display_information(primary_info, buf);
    
    setup_blits(primary_hardware, primary_software, DSBLIT_ROTATE180|0//DSBLIT_SRC_COLORKEY
            , BLT_COLOR_A);
    primary_hardware->StretchBlit( primary_hardware, logo_surface, NULL, &logo_rect );
    primary_hardware->StretchBlit( primary_hardware, test_surface, NULL,  &test_rect );
    primary_software->StretchBlit( primary_software, logo_surface, NULL, &logo_rect );
    primary_software->StretchBlit( primary_software, test_surface, NULL,  &test_rect );
    flip_context( primary_info, primary_hardware, primary_software );
    get_user_input();
    
    /* Test XOR */
    sprintf(buf, "Test %d: Stretch Blit with XOR", test_number++);
    display_information(primary_info, buf);
    
    setup_blits(primary_hardware, primary_software, DSBLIT_XOR|0//DSBLIT_SRC_COLORKEY
            , BLT_COLOR_A);
    primary_hardware->StretchBlit( primary_hardware, logo_surface, NULL, &logo_rect );
    primary_hardware->StretchBlit( primary_hardware, test_surface, NULL, &test_rect );
    primary_software->StretchBlit( primary_software, logo_surface, NULL, &logo_rect );
    primary_software->StretchBlit( primary_software, test_surface, NULL, &test_rect );
    flip_context( primary_info, primary_hardware, primary_software );
    get_user_input();
    
    /* Test Alpha Channel Blending */
    sprintf(buf, "Test %d: Stretch Blit with Alpha Channel Blending", test_number++);
    display_information(primary_info, buf);
    
    setup_blits(primary_hardware, primary_software, DSBLIT_BLEND_ALPHACHANNEL|0//DSBLIT_SRC_COLORKEY
            , BLT_COLOR_A);
    primary_hardware->StretchBlit( primary_hardware, logo_surface, NULL, &logo_rect );
    primary_hardware->StretchBlit( primary_hardware, test_surface, NULL, &test_rect );
    primary_software->StretchBlit( primary_software, logo_surface, NULL, &logo_rect );
    primary_software->StretchBlit( primary_software, test_surface, NULL, &test_rect );
    flip_context( primary_info, primary_hardware, primary_software );
    get_user_input();
    
    /* Test 100% Color Alpha Blending */
    sprintf(buf, "Test %d: Stretch 100%% Color Alpha Blending", test_number++);
    display_information(primary_info, buf);
    
    setup_blits(primary_hardware, primary_software, DSBLIT_BLEND_COLORALPHA|0//DSBLIT_SRC_COLORKEY
            , BLT_COLOR_A);
    primary_hardware->StretchBlit( primary_hardware, logo_surface, NULL, &logo_rect );
    primary_hardware->StretchBlit( primary_hardware, test_surface, NULL, &test_rect );
    primary_software->StretchBlit( primary_software, logo_surface, NULL, &logo_rect );
    primary_software->StretchBlit( primary_software, test_surface, NULL, &test_rect );
    flip_context( primary_info, primary_hardware, primary_software );
    get_user_input();
    
    /* Test 50% Color Alpha Blending */
    sprintf(buf, "Test %d: Stretch 50%% Color Alpha Blending", test_number++);
    display_information(primary_info, buf);
    
    setup_blits(primary_hardware, primary_software, DSBLIT_BLEND_COLORALPHA, 0x80);
    primary_hardware->StretchBlit( primary_hardware, logo_surface, NULL, &logo_rect );
    primary_hardware->StretchBlit( primary_hardware, test_surface, NULL, &test_rect );
    primary_software->StretchBlit( primary_software, logo_surface, NULL, &logo_rect );
    primary_software->StretchBlit( primary_software, test_surface, NULL, &test_rect );
    flip_context( primary_info, primary_hardware, primary_software );
    get_user_input();
    
    /* Test 25% Color Alpha Blending */
    sprintf(buf, "Test %d: Stretch 25%% Color Alpha Blending", test_number++);
    display_information(primary_info, buf);
    
    setup_blits(primary_hardware, primary_software, DSBLIT_BLEND_COLORALPHA, 0x40);
    primary_hardware->StretchBlit( primary_hardware, logo_surface, NULL, &logo_rect );
    primary_hardware->StretchBlit( primary_hardware, test_surface, NULL, &test_rect );
    primary_software->StretchBlit( primary_software, logo_surface, NULL, &logo_rect );
    primary_software->StretchBlit( primary_software, test_surface, NULL, &test_rect );
    flip_context( primary_info, primary_hardware, primary_software );
    get_user_input();
    
    /* Test 100% Color Alpha + Alpha Channel Blending */
    sprintf(buf, "Test %d: Stretch 100%% Color Alpha + Alpha Blend", test_number++);
    display_information(primary_info, buf);
    
    setup_blits(primary_hardware, primary_software, DSBLIT_BLEND_COLORALPHA|DSBLIT_BLEND_ALPHACHANNEL|0//DSBLIT_SRC_COLORKEY
            , BLT_COLOR_A);
    primary_hardware->StretchBlit( primary_hardware, logo_surface, NULL, &logo_rect );
    primary_hardware->StretchBlit( primary_hardware, test_surface, NULL, &test_rect );
    primary_software->StretchBlit( primary_software, logo_surface, NULL, &logo_rect );
    primary_software->StretchBlit( primary_software, test_surface, NULL, &test_rect );
    flip_context( primary_info, primary_hardware, primary_software );
    get_user_input();
    
    /* Test 50% Color Alpha + Alpha Channel Blending */
    sprintf(buf, "Test %d: Stretch 50%% Color Alpha + Alpha Blend", test_number++);
    display_information(primary_info, buf);
    
    setup_blits(primary_hardware, primary_software, DSBLIT_BLEND_COLORALPHA|DSBLIT_BLEND_ALPHACHANNEL, 0x80);
    primary_hardware->StretchBlit( primary_hardware, logo_surface, NULL, &logo_rect );
    primary_hardware->StretchBlit( primary_hardware, test_surface, NULL, &test_rect );
    primary_software->StretchBlit( primary_software, logo_surface, NULL, &logo_rect );
    primary_software->StretchBlit( primary_software, test_surface, NULL, &test_rect );
    flip_context( primary_info, primary_hardware, primary_software );
    get_user_input();
    
    /* Test 25% Color Alpha + Alpha Channel Blending */
    sprintf(buf, "Test %d: Stretch 25%% Color Alpha + Alpha Blend", test_number++);
    display_information(primary_info, buf);
    
    setup_blits(primary_hardware, primary_software, DSBLIT_BLEND_COLORALPHA|DSBLIT_BLEND_ALPHACHANNEL, 0x40);
    primary_hardware->StretchBlit( primary_hardware, logo_surface, NULL, &logo_rect );
    primary_hardware->StretchBlit( primary_hardware, test_surface, NULL, &test_rect );
    primary_software->StretchBlit( primary_software, logo_surface, NULL, &logo_rect );
    primary_software->StretchBlit( primary_software, test_surface, NULL, &test_rect );
    flip_context( primary_info, primary_hardware, primary_software );
    get_user_input();
    
    /* Test PREMULTIPLY */
    sprintf(buf, "Test %d: Stretch Premultiply Source", test_number++);
    display_information(primary_info, buf);
    
    setup_blits(primary_hardware, primary_software, DSBLIT_SRC_PREMULTIPLY|0//DSBLIT_SRC_COLORKEY
            , 0x80);
    primary_hardware->StretchBlit( primary_hardware, logo_surface, NULL, &logo_rect );
    primary_hardware->StretchBlit( primary_hardware, test_surface, NULL, &test_rect );
    primary_software->StretchBlit( primary_software, logo_surface, NULL, &logo_rect );
    primary_software->StretchBlit( primary_software, test_surface, NULL, &test_rect );
    flip_context( primary_info, primary_hardware, primary_software );
    get_user_input();
    
    /* Test Alpha Channel + PREMULTIPLY Blending */
    sprintf(buf, "Test %d: Stretch Premultiply Source Alpha Blending", test_number++);
    display_information(primary_info, buf);
    
    setup_blits(primary_hardware, primary_software, DSBLIT_BLEND_ALPHACHANNEL | DSBLIT_SRC_PREMULTIPLY | 0//DSBLIT_SRC_COLORKEY
            , 0x80);
    primary_hardware->StretchBlit( primary_hardware, logo_surface, NULL, &logo_rect );
    primary_hardware->StretchBlit( primary_hardware, test_surface, NULL, &test_rect );
    primary_software->StretchBlit( primary_software, logo_surface, NULL, &logo_rect );
    primary_software->StretchBlit( primary_software, test_surface, NULL, &test_rect );
    flip_context( primary_info, primary_hardware, primary_software );
    get_user_input();
    
    /* Test 100% Alpha Color + Alpha Channel + PREMULTIPLY Blending */
    sprintf(buf, "Test %d: Stretch Premultiply Source Alpha + 100%% Alpha Color blending", test_number++);
    display_information(primary_info, buf);
    
    setup_blits(primary_hardware, primary_software, DSBLIT_BLEND_ALPHACHANNEL | DSBLIT_BLEND_COLORALPHA | DSBLIT_SRC_PREMULTIPLY | 0//DSBLIT_SRC_COLORKEY
            , BLT_COLOR_A);
    primary_hardware->StretchBlit( primary_hardware, logo_surface, NULL, &logo_rect );
    primary_hardware->StretchBlit( primary_hardware, test_surface, NULL, &test_rect );
    primary_software->StretchBlit( primary_software, logo_surface, NULL, &logo_rect );
    primary_software->StretchBlit( primary_software, test_surface, NULL, &test_rect );
    flip_context( primary_info, primary_hardware, primary_software );
    get_user_input();
    
    /* Test 50% Alpha Color + Alpha Channel + PREMULTIPLY Blending */
    sprintf(buf, "Test %d: Stretch Premultiply Source Alpha + 50%% Alpha Color blending", test_number++);
    display_information(primary_info, buf);
    
    setup_blits(primary_hardware, primary_software, DSBLIT_BLEND_ALPHACHANNEL | DSBLIT_BLEND_COLORALPHA | DSBLIT_SRC_PREMULTIPLY, 0x80);
    primary_hardware->StretchBlit( primary_hardware, logo_surface, NULL, &logo_rect );
    primary_hardware->StretchBlit( primary_hardware, test_surface, NULL, &test_rect );
    primary_software->StretchBlit( primary_software, logo_surface, NULL, &logo_rect );
    primary_software->StretchBlit( primary_software, test_surface, NULL, &test_rect );
    flip_context( primary_info, primary_hardware, primary_software );
    get_user_input();
    
    /* Test 25% Alpha Color + Alpha Channel + PREMULTIPLY Blending */
    sprintf(buf, "Test %d: Stretch Premultiply Source Alpha + 25%% Alpha Color blending", test_number++);
    display_information(primary_info, buf);
    
    setup_blits(primary_hardware, primary_software, DSBLIT_BLEND_ALPHACHANNEL | DSBLIT_BLEND_COLORALPHA | DSBLIT_SRC_PREMULTIPLY, 0x40);
    primary_hardware->StretchBlit( primary_hardware, logo_surface, NULL, &logo_rect );
    primary_hardware->StretchBlit( primary_hardware, test_surface, NULL, &test_rect );
    primary_software->StretchBlit( primary_software, logo_surface, NULL, &logo_rect );
    primary_software->StretchBlit( primary_software, test_surface, NULL, &test_rect );
    flip_context( primary_info, primary_hardware, primary_software );
    get_user_input();
    
    /* Test 100% PREMULTCOLOR */
    sprintf(buf, "Test %d: Stretch 100%% PREMULTCOLOR on the Source", test_number++);
    display_information(primary_info, buf);
    
    setup_blits(primary_hardware, primary_software, DSBLIT_SRC_PREMULTCOLOR, BLT_COLOR_A);
    primary_hardware->StretchBlit( primary_hardware, logo_surface, NULL, &logo_rect );
    primary_hardware->StretchBlit( primary_hardware, test_surface, NULL, &test_rect );
    primary_software->StretchBlit( primary_software, logo_surface, NULL, &logo_rect );
    primary_software->StretchBlit( primary_software, test_surface, NULL, &test_rect );
    flip_context( primary_info, primary_hardware, primary_software );
    get_user_input();

    /* Test 50% PREMULTCOLOR */
    sprintf(buf, "Test %d: Stretch 50%% PREMULTCOLOR on the Source", test_number++);
    display_information(primary_info, buf);
    
    setup_blits(primary_hardware, primary_software, DSBLIT_SRC_PREMULTCOLOR, 0x80);
    primary_hardware->StretchBlit( primary_hardware, logo_surface, NULL, &logo_rect );
    primary_hardware->StretchBlit( primary_hardware, test_surface, NULL, &test_rect );
    primary_software->StretchBlit( primary_software, logo_surface, NULL, &logo_rect );
    primary_software->StretchBlit( primary_software, test_surface, NULL, &test_rect );
    flip_context( primary_info, primary_hardware, primary_software );
    get_user_input();
    
    /* Test 25% PREMULTCOLOR */
    sprintf(buf, "Test %d: Stretch 25%% PREMULTCOLOR on the Source", test_number++);
    display_information(primary_info, buf);
    
    setup_blits(primary_hardware, primary_software, DSBLIT_SRC_PREMULTCOLOR, 0x40);
    primary_hardware->StretchBlit( primary_hardware, logo_surface, NULL, &logo_rect );
    primary_hardware->StretchBlit( primary_hardware, test_surface, NULL, &test_rect );
    primary_software->StretchBlit( primary_software, logo_surface, NULL, &logo_rect );
    primary_software->StretchBlit( primary_software, test_surface, NULL, &test_rect );
    flip_context( primary_info, primary_hardware, primary_software );
    get_user_input();
    
    /* Test 100% Alpha Color + Alpha Channel + 100% PREMULTCOLOR Blending */
    sprintf(buf, "Test %d: Stretch Source Alpha + 100%% Alpha Color + 100%% PREMULTCOLOR", test_number++);
    display_information(primary_info, buf);
    
    setup_blits(primary_hardware, primary_software, DSBLIT_BLEND_ALPHACHANNEL | DSBLIT_BLEND_COLORALPHA | DSBLIT_SRC_PREMULTCOLOR, BLT_COLOR_A);
    primary_hardware->StretchBlit( primary_hardware, logo_surface, NULL, &logo_rect );
    primary_hardware->StretchBlit( primary_hardware, test_surface, NULL, &test_rect );
    primary_software->StretchBlit( primary_software, logo_surface, NULL, &logo_rect );
    primary_software->StretchBlit( primary_software, test_surface, NULL, &test_rect );
    flip_context( primary_info, primary_hardware, primary_software );
    get_user_input();
    
    /* Test 50% Alpha Color + Alpha Channel + 50% PREMULTCOLOR Blending */
    sprintf(buf, "Test %d: Stretch Source Alpha + 50%% Alpha Color + 50%% PREMULTCOLOR", test_number++);
    display_information(primary_info, buf);
    
    setup_blits(primary_hardware, primary_software, DSBLIT_BLEND_ALPHACHANNEL | DSBLIT_BLEND_COLORALPHA | DSBLIT_SRC_PREMULTCOLOR, 0x80);
    primary_hardware->StretchBlit( primary_hardware, logo_surface, NULL, &logo_rect );
    primary_hardware->StretchBlit( primary_hardware, test_surface, NULL, &test_rect );
    primary_software->StretchBlit( primary_software, logo_surface, NULL, &logo_rect );
    primary_software->StretchBlit( primary_software, test_surface, NULL, &test_rect );
    flip_context( primary_info, primary_hardware, primary_software );
    get_user_input();
    
    /* Test 25% Alpha Color + Alpha Channel + 25% PREMULTCOLOR Blending */
    sprintf(buf, "Test %d: Stretch Source Alpha + 25%% Alpha Color + 25%% PREMULTCOLOR", test_number++);
    display_information(primary_info, buf);
    
    setup_blits(primary_hardware, primary_software, DSBLIT_BLEND_ALPHACHANNEL | DSBLIT_BLEND_COLORALPHA | DSBLIT_SRC_PREMULTCOLOR, 0x40);
    primary_hardware->StretchBlit( primary_hardware, logo_surface, NULL, &logo_rect );
    primary_hardware->StretchBlit( primary_hardware, test_surface, NULL, &test_rect );
    primary_software->StretchBlit( primary_software, logo_surface, NULL, &logo_rect );
    primary_software->StretchBlit( primary_software, test_surface, NULL, &test_rect );
    flip_context( primary_info, primary_hardware, primary_software );
    get_user_input();
    
    /* Test 100% Alpha Color + Alpha Channel + PREMULTIPLY + 100% PREMULTCOLOR Blending */
    sprintf(buf, "Test %d: Stretch Premultiply Source Alpha + 100%% Alpha Color + 100%% PREMULTCOLOR", test_number++);
    display_information(primary_info, buf);
    
    setup_blits(primary_hardware, primary_software, DSBLIT_BLEND_ALPHACHANNEL | DSBLIT_BLEND_COLORALPHA | DSBLIT_SRC_PREMULTIPLY | DSBLIT_SRC_PREMULTCOLOR,
            BLT_COLOR_A);
    primary_hardware->StretchBlit( primary_hardware, logo_surface, NULL, &logo_rect );
    primary_hardware->StretchBlit( primary_hardware, test_surface, NULL, &test_rect );
    primary_software->StretchBlit( primary_software, logo_surface, NULL, &logo_rect );
    primary_software->StretchBlit( primary_software, test_surface, NULL, &test_rect );
    flip_context( primary_info, primary_hardware, primary_software );
    get_user_input();
    
    /* Test 50% Alpha Color + Alpha Channel + PREMULTIPLY + 50% PREMULTCOLOR Blending */
    sprintf(buf, "Test %d: Stretch Premultiply Source Alpha + 50%% Alpha Color + 50%% PREMULTCOLOR", test_number++);
    display_information(primary_info, buf);
    
    setup_blits(primary_hardware, primary_software, DSBLIT_BLEND_ALPHACHANNEL | DSBLIT_BLEND_COLORALPHA | DSBLIT_SRC_PREMULTIPLY | DSBLIT_SRC_PREMULTCOLOR,
            0x80);
    primary_hardware->StretchBlit( primary_hardware, logo_surface, NULL, &logo_rect );
    primary_hardware->StretchBlit( primary_hardware, test_surface, NULL, &test_rect );
    primary_software->StretchBlit( primary_software, logo_surface, NULL, &logo_rect );
    primary_software->StretchBlit( primary_software, test_surface, NULL, &test_rect );
    flip_context( primary_info, primary_hardware, primary_software );
    get_user_input();
    
    /* Test 25% Alpha Color + Alpha Channel + PREMULTIPLY + 25% PREMULTCOLOR Blending */
    sprintf(buf, "Test %d: Stretch Premultiply Source Alpha + 25%% Alpha Color + 25%% PREMULTCOLOR", test_number++);
    display_information(primary_info, buf);
    
    setup_blits(primary_hardware, primary_software, DSBLIT_BLEND_ALPHACHANNEL | DSBLIT_BLEND_COLORALPHA | DSBLIT_SRC_PREMULTIPLY | DSBLIT_SRC_PREMULTCOLOR,
            0x40);
    primary_hardware->StretchBlit( primary_hardware, logo_surface, NULL, &logo_rect );
    primary_hardware->StretchBlit( primary_hardware, test_surface, NULL, &test_rect );
    primary_software->StretchBlit( primary_software, logo_surface, NULL, &logo_rect );
    primary_software->StretchBlit( primary_software, test_surface, NULL, &test_rect );
    flip_context( primary_info, primary_hardware, primary_software );
    get_user_input();
    
    /* Test Alpha Channel Blending with Colorize */
    sprintf(buf, "Test %d: Stretch Source Alpha Blending with Colorize", test_number++);
    display_information(primary_info, buf);
    
    setup_blits(primary_hardware, primary_software, DSBLIT_BLEND_ALPHACHANNEL | DSBLIT_COLORIZE, 0x80);
    primary_hardware->StretchBlit( primary_hardware, logo_surface, NULL, &logo_rect );
    primary_hardware->StretchBlit( primary_hardware, test_surface, NULL, &test_rect );
    primary_software->StretchBlit( primary_software, logo_surface, NULL, &logo_rect );
    primary_software->StretchBlit( primary_software, test_surface, NULL, &test_rect );
    flip_context( primary_info, primary_hardware, primary_software );
    get_user_input();
    
    /* Test 100% Color Alpha Blending with Colorize */
    sprintf(buf, "Test %d: Stretch 100%% Color Alpha Blending with Colorize", test_number++);
    display_information(primary_info, buf);
    
    setup_blits(primary_hardware, primary_software, DSBLIT_BLEND_COLORALPHA | DSBLIT_COLORIZE, BLT_COLOR_A);
    primary_hardware->StretchBlit( primary_hardware, logo_surface, NULL, &logo_rect );
    primary_hardware->StretchBlit( primary_hardware, test_surface, NULL, &test_rect );
    primary_software->StretchBlit( primary_software, logo_surface, NULL, &logo_rect );
    primary_software->StretchBlit( primary_software, test_surface, NULL, &test_rect );
    flip_context( primary_info, primary_hardware, primary_software );
    get_user_input();
    
    /* Test 50% Color Alpha Blending with Colorize */
    sprintf(buf, "Test %d: Stretch 50%% Color Alpha Blending with Colorize", test_number++);
    display_information(primary_info, buf);
    
    setup_blits(primary_hardware, primary_software, DSBLIT_BLEND_COLORALPHA | DSBLIT_COLORIZE, 0x80);
    primary_hardware->StretchBlit( primary_hardware, logo_surface, NULL, &logo_rect );
    primary_hardware->StretchBlit( primary_hardware, test_surface, NULL, &test_rect );
    primary_software->StretchBlit( primary_software, logo_surface, NULL, &logo_rect );
    primary_software->StretchBlit( primary_software, test_surface, NULL, &test_rect );
    flip_context( primary_info, primary_hardware, primary_software );
    get_user_input();
    
    /* Test 25% Color Alpha Blending with Colorize */
    sprintf(buf, "Test %d: Stretch 25%% Color Alpha Blending with Colorize", test_number++);
    display_information(primary_info, buf);
    
    setup_blits(primary_hardware, primary_software, DSBLIT_BLEND_COLORALPHA | DSBLIT_COLORIZE, 0x40);
    primary_hardware->StretchBlit( primary_hardware, logo_surface, NULL, &logo_rect );
    primary_hardware->StretchBlit( primary_hardware, test_surface, NULL, &test_rect );
    primary_software->StretchBlit( primary_software, logo_surface, NULL, &logo_rect );
    primary_software->StretchBlit( primary_software, test_surface, NULL, &test_rect );
    flip_context( primary_info, primary_hardware, primary_software );
    get_user_input();
    
    /* Test 100% Color Alpha + Alpha Channel Blending + Colorize */
    sprintf(buf, "Test %d: Stretch 100%% Color Alpha + Source Alpha Blending + Colorize", test_number++);
    display_information(primary_info, buf);
    
    setup_blits(primary_hardware, primary_software, DSBLIT_BLEND_COLORALPHA | DSBLIT_BLEND_ALPHACHANNEL | DSBLIT_COLORIZE, BLT_COLOR_A);
    primary_hardware->StretchBlit( primary_hardware, logo_surface, NULL, &logo_rect );
    primary_hardware->StretchBlit( primary_hardware, test_surface, NULL, &test_rect );
    primary_software->StretchBlit( primary_software, logo_surface, NULL, &logo_rect );
    primary_software->StretchBlit( primary_software, test_surface, NULL, &test_rect );
    flip_context( primary_info, primary_hardware, primary_software );
    get_user_input();
    
    /* Test 50% Color Alpha + Alpha Channel Blending + Colorize */
    sprintf(buf, "Test %d: Stretch 50%% Color Alpha + Source Alpha Blending + Colorize", test_number++);
    display_information(primary_info, buf);
    
    setup_blits(primary_hardware, primary_software, DSBLIT_BLEND_COLORALPHA | DSBLIT_BLEND_ALPHACHANNEL | DSBLIT_COLORIZE, 0x80);
    primary_hardware->StretchBlit( primary_hardware, logo_surface, NULL, &logo_rect );
    primary_hardware->StretchBlit( primary_hardware, test_surface, NULL, &test_rect );
    primary_software->StretchBlit( primary_software, logo_surface, NULL, &logo_rect );
    primary_software->StretchBlit( primary_software, test_surface, NULL, &test_rect );
    flip_context( primary_info, primary_hardware, primary_software );
    get_user_input();
    
    /* Test 25% Color Alpha + Alpha Channel Blending + Colorize */
    sprintf(buf, "Test %d: Stretch 25%% Color Alpha + Source Alpha Blending + Colorize", test_number++);
    display_information(primary_info, buf);
    
    setup_blits(primary_hardware, primary_software, DSBLIT_BLEND_COLORALPHA | DSBLIT_BLEND_ALPHACHANNEL | DSBLIT_COLORIZE, 0x40);
    primary_hardware->StretchBlit( primary_hardware, logo_surface, NULL, &logo_rect );
    primary_hardware->StretchBlit( primary_hardware, test_surface, NULL, &test_rect );
    primary_software->StretchBlit( primary_software, logo_surface, NULL, &logo_rect );
    primary_software->StretchBlit( primary_software, test_surface, NULL, &test_rect );
    flip_context( primary_info, primary_hardware, primary_software );
    get_user_input();
    
    /* Test PREMULTIPLY with Colorize */
    sprintf(buf, "Test %d: Stretch Premultiply Source with Colorize", test_number++);
    display_information(primary_info, buf);
    
    setup_blits(primary_hardware, primary_software, DSBLIT_SRC_PREMULTIPLY | DSBLIT_COLORIZE, 0x80);
    primary_hardware->StretchBlit( primary_hardware, logo_surface, NULL, &logo_rect );
    primary_hardware->StretchBlit( primary_hardware, test_surface, NULL, &test_rect );
    primary_software->StretchBlit( primary_software, logo_surface, NULL, &logo_rect );
    primary_software->StretchBlit( primary_software, test_surface, NULL, &test_rect );
    flip_context( primary_info, primary_hardware, primary_software );
    get_user_input();
    
    /* Test Alpha Channel + PREMULTIPLY Blending + Colorize */
    sprintf(buf, "Test %d: Stretch Premultiply Source Alpha Blending with Colorize", test_number++);
    display_information(primary_info, buf);
    
    setup_blits(primary_hardware, primary_software,  DSBLIT_BLEND_ALPHACHANNEL | DSBLIT_SRC_PREMULTIPLY | DSBLIT_COLORIZE, 0x80);
    primary_hardware->StretchBlit( primary_hardware, logo_surface, NULL, &logo_rect );
    primary_hardware->StretchBlit( primary_hardware, test_surface, NULL, &test_rect );
    primary_software->StretchBlit( primary_software, logo_surface, NULL, &logo_rect );
    primary_software->StretchBlit( primary_software, test_surface, NULL, &test_rect );
    flip_context( primary_info, primary_hardware, primary_software );
    get_user_input();
    
    /* Test 100% Alpha Color + Alpha Channel + PREMULTIPLY Blending + Colorize */
    sprintf(buf, "Test %d: Stretch Premultiply Source Alpha + 100%% Alpha Color blending + Colorize", test_number++);
    display_information(primary_info, buf);
    
    setup_blits(primary_hardware, primary_software,  DSBLIT_BLEND_ALPHACHANNEL | DSBLIT_BLEND_COLORALPHA | DSBLIT_SRC_PREMULTIPLY | DSBLIT_COLORIZE,
               BLT_COLOR_A);
    primary_hardware->StretchBlit( primary_hardware, logo_surface, NULL, &logo_rect );
    primary_hardware->StretchBlit( primary_hardware, test_surface, NULL, &test_rect );
    primary_software->StretchBlit( primary_software, logo_surface, NULL, &logo_rect );
    primary_software->StretchBlit( primary_software, test_surface, NULL, &test_rect );
    flip_context( primary_info, primary_hardware, primary_software );
    get_user_input();
    
    /* Test 50% Alpha Color + Alpha Channel + PREMULTIPLY Blending + Colorize */
    sprintf(buf, "Test %d: Stretch Premultiply Source Alpha + 50%% Alpha Color blending + Colorize", test_number++);
    display_information(primary_info, buf);
    
    setup_blits(primary_hardware, primary_software,  DSBLIT_BLEND_ALPHACHANNEL | DSBLIT_BLEND_COLORALPHA | DSBLIT_SRC_PREMULTIPLY | DSBLIT_COLORIZE,
               0x80);
    primary_hardware->StretchBlit( primary_hardware, logo_surface, NULL, &logo_rect );
    primary_hardware->StretchBlit( primary_hardware, test_surface, NULL, &test_rect );
    primary_software->StretchBlit( primary_software, logo_surface, NULL, &logo_rect );
    primary_software->StretchBlit( primary_software, test_surface, NULL, &test_rect );
    flip_context( primary_info, primary_hardware, primary_software );
    get_user_input();
    
    /* Test 25% Alpha Color + Alpha Channel + PREMULTIPLY Blending + Colorize */
    sprintf(buf, "Test %d: Stretch Premultiply Source Alpha + 25%% Alpha Color blending + Colorize", test_number++);
    display_information(primary_info, buf);
    
    setup_blits(primary_hardware, primary_software,  DSBLIT_BLEND_ALPHACHANNEL | DSBLIT_BLEND_COLORALPHA | DSBLIT_SRC_PREMULTIPLY | DSBLIT_COLORIZE,
               0x40);
    primary_hardware->StretchBlit( primary_hardware, logo_surface, NULL, &logo_rect );
    primary_hardware->StretchBlit( primary_hardware, test_surface, NULL, &test_rect );
    primary_software->StretchBlit( primary_software, logo_surface, NULL, &logo_rect );
    primary_software->StretchBlit( primary_software, test_surface, NULL, &test_rect );
    flip_context( primary_info, primary_hardware, primary_software );
    get_user_input();
    
    /* Test 100% PREMULTCOLOR with Colorize */
    sprintf(buf, "Test %d: Stretch 100%% PREMULTCOLOR on the Source with Colorize", test_number++);
    display_information(primary_info, buf);
    
    setup_blits(primary_hardware, primary_software,  DSBLIT_SRC_PREMULTCOLOR | DSBLIT_COLORIZE, BLT_COLOR_A);
    primary_hardware->StretchBlit( primary_hardware, logo_surface, NULL, &logo_rect );
    primary_hardware->StretchBlit( primary_hardware, test_surface, NULL, &test_rect );
    primary_software->StretchBlit( primary_software, logo_surface, NULL, &logo_rect );
    primary_software->StretchBlit( primary_software, test_surface, NULL, &test_rect );
    flip_context( primary_info, primary_hardware, primary_software );
    get_user_input();

    /* Test 50% PREMULTCOLOR with Colorize */
    sprintf(buf, "Test %d: Stretch 50%% PREMULTCOLOR on the Source with Colorize", test_number++);
    display_information(primary_info, buf);
    
    setup_blits(primary_hardware, primary_software,  DSBLIT_SRC_PREMULTCOLOR | DSBLIT_COLORIZE, 0x80);
    primary_hardware->StretchBlit( primary_hardware, logo_surface, NULL, &logo_rect );
    primary_hardware->StretchBlit( primary_hardware, test_surface, NULL, &test_rect );
    primary_software->StretchBlit( primary_software, logo_surface, NULL, &logo_rect );
    primary_software->StretchBlit( primary_software, test_surface, NULL, &test_rect );
    flip_context( primary_info, primary_hardware, primary_software );
    get_user_input();
    
    /* Test 25% PREMULTCOLOR with Colorize */
    sprintf(buf, "Test %d: Stretch 25%% PREMULTCOLOR on the Source with Colorize", test_number++);
    display_information(primary_info, buf);
    
    setup_blits(primary_hardware, primary_software,  DSBLIT_SRC_PREMULTCOLOR | DSBLIT_COLORIZE, 0x40);
    primary_hardware->StretchBlit( primary_hardware, logo_surface, NULL, &logo_rect );
    primary_hardware->StretchBlit( primary_hardware, test_surface, NULL, &test_rect );
    primary_software->StretchBlit( primary_software, logo_surface, NULL, &logo_rect );
    primary_software->StretchBlit( primary_software, test_surface, NULL, &test_rect );
    flip_context( primary_info, primary_hardware, primary_software );
    get_user_input();
    
    /* Test 100% Alpha Color + Alpha Channel + 100% PREMULTCOLOR Blending + Colorize */ 
    sprintf(buf, "Test %d: Stretch Source Alpha + 100%% Alpha Color + 100%% PREMULTCOLOR + Colorize", test_number++);
    display_information(primary_info, buf);
    
    setup_blits(primary_hardware, primary_software, DSBLIT_BLEND_ALPHACHANNEL | DSBLIT_BLEND_COLORALPHA | DSBLIT_SRC_PREMULTCOLOR | DSBLIT_COLORIZE,
            BLT_COLOR_A);
    primary_hardware->StretchBlit( primary_hardware, logo_surface, NULL, &logo_rect );
    primary_hardware->StretchBlit( primary_hardware, test_surface, NULL, &test_rect );
    primary_software->StretchBlit( primary_software, logo_surface, NULL, &logo_rect );
    primary_software->StretchBlit( primary_software, test_surface, NULL, &test_rect );
    flip_context( primary_info, primary_hardware, primary_software );
    get_user_input();
    
    /* Test 50% Alpha Color + Alpha Channel + 50% PREMULTCOLOR Blending + Colorize */
    sprintf(buf, "Test %d: Stretch Source Alpha + 50%% Alpha Color + 50%% PREMULTCOLOR + Colorize", test_number++);
    display_information(primary_info, buf);
    
    setup_blits(primary_hardware, primary_software, DSBLIT_BLEND_ALPHACHANNEL | DSBLIT_BLEND_COLORALPHA | DSBLIT_SRC_PREMULTCOLOR | DSBLIT_COLORIZE,
            0x80);
    primary_hardware->StretchBlit( primary_hardware, logo_surface, NULL, &logo_rect );
    primary_hardware->StretchBlit( primary_hardware, test_surface, NULL, &test_rect );
    primary_software->StretchBlit( primary_software, logo_surface, NULL, &logo_rect );
    primary_software->StretchBlit( primary_software, test_surface, NULL, &test_rect );
    flip_context( primary_info, primary_hardware, primary_software );
    get_user_input();
    
    /* Test 25% Alpha Color + Alpha Channel + 25% PREMULTCOLOR Blending + Colorize */
    sprintf(buf, "Test %d: Stretch Source Alpha + 25%% Alpha Color + 25%% PREMULTCOLOR + Colorize", test_number++);
    display_information(primary_info, buf);
    
    setup_blits(primary_hardware, primary_software, DSBLIT_BLEND_ALPHACHANNEL | DSBLIT_BLEND_COLORALPHA | DSBLIT_SRC_PREMULTCOLOR | DSBLIT_COLORIZE,
            0x40);
    primary_hardware->StretchBlit( primary_hardware, logo_surface, NULL, &logo_rect );
    primary_hardware->StretchBlit( primary_hardware, test_surface, NULL, &test_rect );
    primary_software->StretchBlit( primary_software, logo_surface, NULL, &logo_rect );
    primary_software->StretchBlit( primary_software, test_surface, NULL, &test_rect );
    flip_context( primary_info, primary_hardware, primary_software );
    get_user_input();
    
    /* Test 100% Alpha Color + Alpha Channel + PREMULTIPLY + 100% PREMULTCOLOR Blending + Colorize */
    sprintf(buf, "Test %d: Stretch Premultiply Source Alpha + 100%% Alpha Color + 100%% PREMULTCOLOR + Colorize", test_number++);
    display_information(primary_info, buf);
    
    setup_blits(primary_hardware, primary_software, DSBLIT_BLEND_ALPHACHANNEL | DSBLIT_BLEND_COLORALPHA | DSBLIT_SRC_PREMULTIPLY |
                                                   DSBLIT_SRC_PREMULTCOLOR | DSBLIT_COLORIZE, BLT_COLOR_A);
    primary_hardware->StretchBlit( primary_hardware, logo_surface, NULL, &logo_rect );
    primary_hardware->StretchBlit( primary_hardware, test_surface, NULL, &test_rect );
    primary_software->StretchBlit( primary_software, logo_surface, NULL, &logo_rect );
    primary_software->StretchBlit( primary_software, test_surface, NULL, &test_rect );
    flip_context( primary_info, primary_hardware, primary_software );
    get_user_input();
    
    /* Test 50% Alpha Color + Alpha Channel + PREMULTIPLY + 50% PREMULTCOLOR Blending + Colorize */
    sprintf(buf, "Test %d: Stretch Premultiply Source Alpha + 50%% Alpha Color + 50%% PREMULTCOLOR + Colorize", test_number++);
    display_information(primary_info, buf);
    
    setup_blits(primary_hardware, primary_software, DSBLIT_BLEND_ALPHACHANNEL | DSBLIT_BLEND_COLORALPHA | DSBLIT_SRC_PREMULTIPLY |
                                                   DSBLIT_SRC_PREMULTCOLOR | DSBLIT_COLORIZE, 0x80);
    primary_hardware->StretchBlit( primary_hardware, logo_surface, NULL, &logo_rect );
    primary_hardware->StretchBlit( primary_hardware, test_surface, NULL, &test_rect );
    primary_software->StretchBlit( primary_software, logo_surface, NULL, &logo_rect );
    primary_software->StretchBlit( primary_software, test_surface, NULL, &test_rect );
    flip_context( primary_info, primary_hardware, primary_software );
    get_user_input();
    
    /* Test 25% Alpha Color + Alpha Channel + PREMULTIPLY + 25% PREMULTCOLOR Blending + Colorize */
    sprintf(buf, "Test %d: Stretch Premultiply Source Alpha + 25%% Alpha Color + 25%% PREMULTCOLOR + Colorize", test_number++);
    display_information(primary_info, buf);
    
    setup_blits(primary_hardware, primary_software, DSBLIT_BLEND_ALPHACHANNEL | DSBLIT_BLEND_COLORALPHA | DSBLIT_SRC_PREMULTIPLY |
                                                   DSBLIT_SRC_PREMULTCOLOR | DSBLIT_COLORIZE, 0x40);
    primary_hardware->StretchBlit( primary_hardware, logo_surface, NULL, &logo_rect );
    primary_hardware->StretchBlit( primary_hardware, test_surface, NULL, &test_rect );
    primary_software->StretchBlit( primary_software, logo_surface, NULL, &logo_rect );
    primary_software->StretchBlit( primary_software, test_surface, NULL, &test_rect );
    flip_context( primary_info, primary_hardware, primary_software );
    get_user_input();
    
    /*** Test with DSDRAW_NOFX flags set ***/
    
    printf("##### TESTING WITH DSDRAW_NOFX FLAG SET #####\n");
    
    sprintf(buf, "Test %d: Drawing Text with various levels of Alpha blending", test_number++);
    display_information(primary_info, buf);

    snprintf( buf, sizeof(buf), "B" );
    primary_hardware->SetBlittingFlags( primary_hardware, DSBLIT_BLEND_ALPHACHANNEL );
    primary_software->SetBlittingFlags( primary_software, DSBLIT_BLEND_ALPHACHANNEL );

    /* Row 1 */
    setup_fills(primary_hardware, primary_software, DSDRAW_NOFX, 0, 0, 60, BLT_COLOR_A);
    
    primary_hardware->FillRectangle( primary_hardware, 20, 20, 440, fontheight+5 );
    
    primary_hardware->SetColor( primary_hardware, 180, 200, 255, 0xFF );
    primary_hardware->DrawString( primary_hardware, buf, -1, 100, 20, DSTF_LEFT | DSTF_TOP );
    
    primary_hardware->SetColor( primary_hardware, 180, 200, 255, 0x80 );
    primary_hardware->DrawString( primary_hardware, buf, -1, 115, 20, DSTF_LEFT | DSTF_TOP );
    
    primary_hardware->SetColor( primary_hardware, 255, 0, 0, 0xFF );
    primary_hardware->DrawString( primary_hardware, buf, -1, 130, 20, DSTF_LEFT | DSTF_TOP );
    
    primary_hardware->SetColor( primary_hardware, 255, 0, 0, 0x80 );
    primary_hardware->DrawString( primary_hardware, buf, -1, 145, 20, DSTF_LEFT | DSTF_TOP );
    
    primary_software->FillRectangle( primary_software, 20, 20, 440, fontheight+5 );
    
    primary_software->SetColor( primary_software, 180, 200, 255, 0xFF );
    primary_software->DrawString( primary_software, buf, -1, 100, 20, DSTF_LEFT | DSTF_TOP );
    
    primary_software->SetColor( primary_software, 180, 200, 255, 0x80 );
    primary_software->DrawString( primary_software, buf, -1, 115, 20, DSTF_LEFT | DSTF_TOP );
    
    primary_software->SetColor( primary_software, 255, 0, 0, 0xFF );
    primary_software->DrawString( primary_software, buf, -1, 130, 20, DSTF_LEFT | DSTF_TOP );
    
    primary_software->SetColor( primary_software, 255, 0, 0, 0x80 );
    primary_software->DrawString( primary_software, buf, -1, 145, 20, DSTF_LEFT | DSTF_TOP );
    
    /* Row 2 */
    primary_hardware->SetColor( primary_hardware, 0, 0, 60, 0x80 );
    primary_hardware->FillRectangle( primary_hardware, 20, 40 + fontheight+5, 440, fontheight+5 );
    
    primary_hardware->SetColor( primary_hardware, 180, 200, 255, 0x80 );
    primary_hardware->DrawString( primary_hardware, buf, -1, 100, (40 + fontheight+5), DSTF_LEFT | DSTF_TOP );
    
    primary_hardware->SetColor( primary_hardware, 180, 200, 255, 0x40 );
    primary_hardware->DrawString( primary_hardware, buf, -1, 115, (40 + fontheight+5), DSTF_LEFT | DSTF_TOP );
    
    primary_hardware->SetColor( primary_hardware, 255, 0, 0, 0x80 );
    primary_hardware->DrawString( primary_hardware, buf, -1, 130, (40 + fontheight+5), DSTF_LEFT | DSTF_TOP );
    
    primary_hardware->SetColor( primary_hardware, 255, 0, 0, 0x40 );
    primary_hardware->DrawString( primary_hardware, buf, -1, 145, (40 + fontheight+5), DSTF_LEFT | DSTF_TOP );
    
    primary_software->SetColor( primary_software, 0, 0, 60, 0x80 );
    primary_software->FillRectangle( primary_software, 20, 40 + fontheight+5, 440, fontheight+5 );
    
    primary_software->SetColor( primary_software, 180, 200, 255, 0x80 );
    primary_software->DrawString( primary_software, buf, -1, 100, (40 + fontheight+5), DSTF_LEFT | DSTF_TOP );
    
    primary_software->SetColor( primary_software, 180, 200, 255, 0x40 );
    primary_software->DrawString( primary_software, buf, -1, 115, (40 + fontheight+5), DSTF_LEFT | DSTF_TOP );
    
    primary_software->SetColor( primary_software, 255, 0, 0, 0x80 );
    primary_software->DrawString( primary_software, buf, -1, 130, (40 + fontheight+5), DSTF_LEFT | DSTF_TOP );
    
    primary_software->SetColor( primary_software, 255, 0, 0, 0x40 );
    primary_software->DrawString( primary_software, buf, -1, 145, (40 + fontheight+5), DSTF_LEFT | DSTF_TOP );
    
    /* Row 3 */
    primary_hardware->SetColor( primary_hardware, 0, 0, 60, 0x40 );
    primary_hardware->FillRectangle( primary_hardware, 20, 60 + 2*(fontheight+5), 440, fontheight+5 );
    
    primary_hardware->SetColor( primary_hardware, 180, 200, 255, 0x40 );
    primary_hardware->DrawString( primary_hardware, buf, -1, 100, 60 + 2*(fontheight+5), DSTF_LEFT | DSTF_TOP );
    
    primary_hardware->SetColor( primary_hardware, 180, 200, 255, 0x0 );
    primary_hardware->DrawString( primary_hardware, buf, -1, 115, 60 + 2*(fontheight+5), DSTF_LEFT | DSTF_TOP );
    
    primary_hardware->SetColor( primary_hardware, 255, 0, 0, 0x40 );
    primary_hardware->DrawString( primary_hardware, buf, -1, 130, 60 + 2*(fontheight+5), DSTF_LEFT | DSTF_TOP );
    
    primary_hardware->SetColor( primary_hardware, 255, 0, 0, 0x0 );
    primary_hardware->DrawString( primary_hardware, buf, -1, 145, 60 + 2*(fontheight+5), DSTF_LEFT | DSTF_TOP );
    
    primary_software->SetColor( primary_software, 0, 0, 60, 0x40 );
    primary_software->FillRectangle( primary_software, 20, 60 + 2*(fontheight+5), 440, fontheight+5 );
    
    primary_software->SetColor( primary_software, 180, 200, 255, 0x40 );
    primary_software->DrawString( primary_software, buf, -1, 100, 60 + 2*(fontheight+5), DSTF_LEFT | DSTF_TOP );
    
    primary_software->SetColor( primary_software, 180, 200, 255, 0x0 );
    primary_software->DrawString( primary_software, buf, -1, 115, 60 + 2*(fontheight+5), DSTF_LEFT | DSTF_TOP );
    
    primary_software->SetColor( primary_software, 255, 0, 0, 0x40 );
    primary_software->DrawString( primary_software, buf, -1, 130, 60 + 2*(fontheight+5), DSTF_LEFT | DSTF_TOP );
    
    primary_software->SetColor( primary_software, 255, 0, 0, 0x0 );
    primary_software->DrawString( primary_software, buf, -1, 145, 60 + 2*(fontheight+5), DSTF_LEFT | DSTF_TOP );
    
    test_surface->SetDrawingFlags (test_surface, DSDRAW_NOFX);
    test_surface->SetColor(test_surface, 0, 0xff, 0, 0xff);
    test_surface->FillRectangle(test_surface, 0, 0, dsc_test_surface.width, dsc_test_surface.height);
    
    primary_hardware->SetColor(primary_hardware, 50, 100, 200, 0x80);
    primary_hardware->FillRectangle(primary_hardware, 200, 200, 100, 100);
    primary_hardware->SetBlittingFlags(primary_hardware, DSBLIT_BLEND_ALPHACHANNEL);
    primary_hardware->Blit( primary_hardware, test_surface, NULL, 250, 250);
    
    primary_software->SetColor(primary_software, 50, 100, 200, 0x80);
    primary_software->FillRectangle(primary_software, 200, 200, 100, 100);
    primary_software->SetBlittingFlags(primary_software, DSBLIT_BLEND_ALPHACHANNEL);
    primary_software->Blit( primary_software, test_surface, NULL, 250, 250);

    flip_context( primary_info, primary_hardware, primary_software );
    get_user_input();
    
    /*** Test with DSDRAW_BLEND set ***/
    
    printf("##### TESTING WITH DSDRAW_BLEND FLAG SET #####\n");
    
    sprintf(buf, "Test %d: Drawing Text with blending and various levels of Alpha blending", test_number++);
    display_information(primary_info, buf);
    
    snprintf( buf, sizeof(buf), "B" );
    
    /* Row 1 */
    setup_fills(primary_hardware, primary_software, DSDRAW_BLEND, 0, 0, 60, BLT_COLOR_A);
    primary_hardware->FillRectangle( primary_hardware, 20, 20, 440, fontheight+5 );
    
    primary_hardware->SetColor( primary_hardware, 180, 200, 255, 0xFF );
    primary_hardware->DrawString( primary_hardware, buf, -1, 100, 20, DSTF_LEFT | DSTF_TOP );
    
    primary_hardware->SetColor( primary_hardware, 180, 200, 255, 0x80 );
    primary_hardware->DrawString( primary_hardware, buf, -1, 115, 20, DSTF_LEFT | DSTF_TOP );
    
    primary_hardware->SetColor( primary_hardware, 255, 0, 0, 0xFF );
    primary_hardware->DrawString( primary_hardware, buf, -1, 130, 20, DSTF_LEFT | DSTF_TOP );
    
    primary_hardware->SetColor( primary_hardware, 255, 0, 0, 0x80 );
    primary_hardware->DrawString( primary_hardware, buf, -1, 145, 20, DSTF_LEFT | DSTF_TOP );
    
    primary_software->FillRectangle( primary_software, 20, 20, 440, fontheight+5 );
    
    primary_software->SetColor( primary_software, 180, 200, 255, 0xFF );
    primary_software->DrawString( primary_software, buf, -1, 100, 20, DSTF_LEFT | DSTF_TOP );
    
    primary_software->SetColor( primary_software, 180, 200, 255, 0x80 );
    primary_software->DrawString( primary_software, buf, -1, 115, 20, DSTF_LEFT | DSTF_TOP );
    
    primary_software->SetColor( primary_software, 255, 0, 0, 0xFF );
    primary_software->DrawString( primary_software, buf, -1, 130, 20, DSTF_LEFT | DSTF_TOP );
    
    primary_software->SetColor( primary_software, 255, 0, 0, 0x80 );
    primary_software->DrawString( primary_software, buf, -1, 145, 20, DSTF_LEFT | DSTF_TOP );
    
    
    /* Row 2 */
    primary_hardware->SetColor( primary_hardware, 0, 0, 60, 0x80 );
    primary_hardware->FillRectangle( primary_hardware, 20, 40 + fontheight+5, 440, fontheight+5 );
    
    primary_hardware->SetColor( primary_hardware, 180, 200, 255, 0x80 );
    primary_hardware->DrawString( primary_hardware, buf, -1, 100, (40 + fontheight+5), DSTF_LEFT | DSTF_TOP );
    
    primary_hardware->SetColor( primary_hardware, 180, 200, 255, 0x40 );
    primary_hardware->DrawString( primary_hardware, buf, -1, 115, (40 + fontheight+5), DSTF_LEFT | DSTF_TOP );
    
    primary_hardware->SetColor( primary_hardware, 255, 0, 0, 0x80 );
    primary_hardware->DrawString( primary_hardware, buf, -1, 130, (40 + fontheight+5), DSTF_LEFT | DSTF_TOP );
    
    primary_hardware->SetColor( primary_hardware, 255, 0, 0, 0x40 );
    primary_hardware->DrawString( primary_hardware, buf, -1, 145, (40 + fontheight+5), DSTF_LEFT | DSTF_TOP );
    
    primary_software->SetColor( primary_software, 0, 0, 60, 0x80 );
    primary_software->FillRectangle( primary_software, 20, (40 + fontheight+5), 440, fontheight+5 );
    
    primary_software->SetColor( primary_software, 180, 200, 255, 0x80 );
    primary_software->DrawString( primary_software, buf, -1, 100, (40 + fontheight+5), DSTF_LEFT | DSTF_TOP );
    
    primary_software->SetColor( primary_software, 180, 200, 255, 0x40 );
    primary_software->DrawString( primary_software, buf, -1, 115, (40 + fontheight+5), DSTF_LEFT | DSTF_TOP );
    
    primary_software->SetColor( primary_software, 255, 0, 0, 0x80 );
    primary_software->DrawString( primary_software, buf, -1, 130, (40 + fontheight+5), DSTF_LEFT | DSTF_TOP );
    
    primary_software->SetColor( primary_software, 255, 0, 0, 0x40 );
    primary_software->DrawString( primary_software, buf, -1, 145, (40 + fontheight+5), DSTF_LEFT | DSTF_TOP );
    
    
    /* Row 3 */
    primary_hardware->SetColor( primary_hardware, 0, 0, 60, 0x40 );
    primary_hardware->FillRectangle( primary_hardware, 20, 60 + 2*(fontheight+5), 440, fontheight+5 );
    
    primary_hardware->SetColor( primary_hardware, 180, 200, 255, 0x40 );
    primary_hardware->DrawString( primary_hardware, buf, -1, 100, 60 + 2*(fontheight+5), DSTF_LEFT | DSTF_TOP );
    
    primary_hardware->SetColor( primary_hardware, 180, 200, 255, 0x0 );
    primary_hardware->DrawString( primary_hardware, buf, -1, 115, 60 + 2*(fontheight+5), DSTF_LEFT | DSTF_TOP );
    
    primary_hardware->SetColor( primary_hardware, 255, 0, 0, 0x40 );
    primary_hardware->DrawString( primary_hardware, buf, -1, 130, 60 + 2*(fontheight+5), DSTF_LEFT | DSTF_TOP );
    
    primary_hardware->SetColor( primary_hardware, 255, 0, 0, 0x0 );
    primary_hardware->DrawString( primary_hardware, buf, -1, 145, 60 + 2*(fontheight+5), DSTF_LEFT | DSTF_TOP );
    
    primary_software->SetColor( primary_software, 0, 0, 60, 0x40 );
    primary_software->FillRectangle( primary_software, 20, 60 + 2*(fontheight+5), 440, fontheight+5 );
    
    primary_software->SetColor( primary_software, 180, 200, 255, 0x40 );
    primary_software->DrawString( primary_software, buf, -1, 100, 60 + 2*(fontheight+5), DSTF_LEFT | DSTF_TOP );
    
    primary_software->SetColor( primary_software, 180, 200, 255, 0x0 );
    primary_software->DrawString( primary_software, buf, -1, 115, 60 + 2*(fontheight+5), DSTF_LEFT | DSTF_TOP );
    
    primary_software->SetColor( primary_software, 255, 0, 0, 0x40 );
    primary_software->DrawString( primary_software, buf, -1, 130, 60 + 2*(fontheight+5), DSTF_LEFT | DSTF_TOP );
    
    primary_software->SetColor( primary_software, 255, 0, 0, 0x0 );
    primary_software->DrawString( primary_software, buf, -1, 145, 60 + 2*(fontheight+5), DSTF_LEFT | DSTF_TOP );
    
    test_surface->SetDrawingFlags (test_surface, DSDRAW_BLEND);
    test_surface->SetColor(test_surface, 0, 0xff, 0, 0xff);
    test_surface->FillRectangle(test_surface, 0, 0, dsc_test_surface.width, dsc_test_surface.height);
    
    primary_hardware->SetColor(primary_hardware, 50, 100, 200, 0x80);
    primary_hardware->FillRectangle(primary_hardware, 200, 200, 100, 100);
    primary_hardware->SetBlittingFlags(primary_hardware, DSBLIT_BLEND_ALPHACHANNEL);
    primary_hardware->Blit( primary_hardware, test_surface, NULL, 250, 250);
    
    primary_software->SetColor(primary_software, 50, 100, 200, 0x80);
    primary_software->FillRectangle(primary_software, 200, 200, 100, 100);
    primary_software->SetBlittingFlags(primary_software, DSBLIT_BLEND_ALPHACHANNEL);
    primary_software->Blit( primary_software, test_surface, NULL, 250, 250);

    flip_context( primary_info, primary_hardware, primary_software );
    get_user_input();
    
    sprintf(buf, "End of All Tests - Press <ENTER> to end.", test_number++);
    display_information(primary_info, buf);
    flip_context( primary_info, primary_hardware, primary_software );
    get_user_input();

    shutdown();
    return 0;
}
